#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "mfhdf.h"
#include "xxx_Types.h"
#include "xxx_LogStatus.h"
#include "xxx_const.h"
#include "axx_alimodel.h"

struct ELEMENT_ATTRIBS
{
  char name[STRLEN]; /* Name of the field                               */
  int  dfnt;         /* DFNT type                                       */
  int  size;         /* Size (number of bytes) of an individual element */
  int  order;        /* Order/number of the elements                    */
};
struct ELEMENT_ATTRIBS alimodel1_elements[] =
{
  {"satnum", DFNT_INT32, sizeof (int32), 1},
  {"dce_year", DFNT_DOUBLE, sizeof (double), 1},
  {"dce_day", DFNT_DOUBLE, sizeof (double), 1},
  {"dce_time", DFNT_DOUBLE, sizeof (double), 1},
  {"ms_frate", DFNT_DOUBLE, sizeof (double), 1},
  {"pan_frate", DFNT_DOUBLE, sizeof (double), 1},
  {"ms_itime", DFNT_DOUBLE, sizeof (double), 1},
  {"pan_itime", DFNT_DOUBLE, sizeof (double), 1},
  {"acs_year", DFNT_DOUBLE, sizeof (double), 1},
  {"acs_day", DFNT_DOUBLE, sizeof (double), 1},
  {"acs_time", DFNT_DOUBLE, sizeof (double), 1},
  {"nephem", DFNT_INT32, sizeof (int32), 1},
  {"ephem_type", DFNT_INT32, sizeof (int32), 1},
  {"ephem_year", DFNT_DOUBLE, sizeof (double), 1},
  {"ephem_day", DFNT_DOUBLE, sizeof (double), 1},
  {"ephem_time", DFNT_DOUBLE, sizeof (double), 1},
  {"nsca", DFNT_INT32, sizeof (int32), 1},
  {"nrpy", DFNT_INT32, sizeof (int32), 1},
  {"sc_roll", DFNT_DOUBLE, sizeof (double), 1},
  {"sc_pitch", DFNT_DOUBLE, sizeof (double), 1},
  {"sc_yaw", DFNT_DOUBLE, sizeof (double), 1},
  {"correction_type", DFNT_INT32, sizeof (int32), 1},
  {"p_roll", DFNT_DOUBLE, sizeof (double), 1},
  {"p_rrate", DFNT_DOUBLE, sizeof (double), 1},
  {"p_pitch", DFNT_DOUBLE, sizeof (double), 1},
  {"p_prate", DFNT_DOUBLE, sizeof (double), 1},
  {"p_yaw", DFNT_DOUBLE, sizeof (double), 1},
  {"p_yrate", DFNT_DOUBLE, sizeof (double), 1},
  {"p_x", DFNT_DOUBLE, sizeof (double), 1},
  {"p_vx", DFNT_DOUBLE, sizeof (double), 1},
  {"p_y", DFNT_DOUBLE, sizeof (double), 1},
  {"p_vy", DFNT_DOUBLE, sizeof (double), 1},
  {"p_z", DFNT_DOUBLE, sizeof (double), 1},
  {"p_vz", DFNT_DOUBLE, sizeof (double), 1},
  {"p_ref_time", DFNT_DOUBLE, sizeof (double), 1},
  {"l0r_major_version", DFNT_INT32, sizeof (int32), 1},
  {"l0r_minor_version", DFNT_INT32, sizeof (int32), 1},
  {"l0r_sub_version", DFNT_INT32, sizeof (int32), 1}
};
#define NUM_INTS1 9 
#define NUM_CHARS1 0
#define NUM_DBLS1 29

struct ELEMENT_ATTRIBS alimodel2_elements[] =
{
  {"ecr_satposX", DFNT_DOUBLE, sizeof (double), 1},
  {"ecr_satposY", DFNT_DOUBLE, sizeof (double), 1},
  {"ecr_satposZ", DFNT_DOUBLE, sizeof (double), 1},
  {"ecr_satvelX", DFNT_DOUBLE, sizeof (double), 1},
  {"ecr_satvelY", DFNT_DOUBLE, sizeof (double), 1},
  {"ecr_satvelZ", DFNT_DOUBLE, sizeof (double), 1},
  {"eci_satposX", DFNT_DOUBLE, sizeof (double), 1},
  {"eci_satposY", DFNT_DOUBLE, sizeof (double), 1},
  {"eci_satposZ", DFNT_DOUBLE, sizeof (double), 1},
  {"eci_satvelX", DFNT_DOUBLE, sizeof (double), 1},
  {"eci_satvelY", DFNT_DOUBLE, sizeof (double), 1},
  {"eci_satvelZ", DFNT_DOUBLE, sizeof (double), 1}
};
#define NUM_DBLS2 12

#define NUM_COEFF_PER_BAND	3
struct ELEMENT_ATTRIBS alimodel3a_elements[] =
{
  /* Legendres are defined as double sca_coef_x[NSCA][NBAND][3] */
  {"sca_coef_x", DFNT_DOUBLE, sizeof (double), NSCA*NBAND*NUM_COEFF_PER_BAND}
};
struct ELEMENT_ATTRIBS alimodel3b_elements[] =
{
  /* Legendres are defined as double sca_coef_x[NSCA][NBAND][3] */
  {"sca_coef_y", DFNT_DOUBLE, sizeof (double), NSCA*NBAND*NUM_COEFF_PER_BAND}
};
struct ELEMENT_ATTRIBS alimodel3c_elements[] =
{
  {"sca_ndet", DFNT_INT32, sizeof (int32), NBAND}
};
struct ELEMENT_ATTRIBS alimodel3d_elements[] =
{
  {"rpy", DFNT_DOUBLE, sizeof (double), NPCD*NAXIS}
};

#define NUM_VDATAS 2
#define NUM_SDS 4
#define NELEMENTS1 (sizeof(alimodel1_elements) / sizeof(struct ELEMENT_ATTRIBS))
#define NELEMENTS2 (sizeof(alimodel2_elements) / sizeof(struct ELEMENT_ATTRIBS))

/******************************************************************************
NAME:           axx_put_alimodel

PURPOSE:
Write the ALI model to an HDF file.

DEVELOPMENT HISTORY:
$Header: /llite/cvseo1/eo1/src/global/axxLib/axxALIModel/axx_alimodel_io.c,v 1.4 2001/03/28 22:41:01 storey Exp $
AUTHOR        Change-ID    DATE     Description of Change
------        ---------  --------   ---------------------
Jim Storey       1.0     11/17/00   Original development.

RETURN VALUE:
Type = int
Value    Description
-----    -----------
SUCCESS  Successful completion
ERROR    Operation failed

NOTES:
Any changes made to the ALI model structure need to be reflected in this
routine.  Any changes made to this routine need to be reflected in the
axx_get_alimodel routine.

ALGORITHM REFERENCES:
See axx_alimodel.h for a description of the alimodel structure.

******************************************************************************
                        Property of the U.S. Government
                             USGS EROS Data Center
******************************************************************************/

/* The following is how the alimodel structure will be stored in the Vgroup:
 * 1. Vdata --> only 1 record
 *    satnum, dce_year, dce_day, dce_time, ms_frate, pan_frate, ms_itime, pan_itime,
 *    acs_year, acs_day, acs_time, nephem, ephem_type, ephem_year, ephem_day,
 *    ephem_time, nsca, nrpy, sc_roll, sc_pitch, sc_yaw, correction_type, p_roll,
 *    p_rrate, p_pitch, p_prate, p_yaw, p_yrate, p_x, p_vx, p_y, p_vy, p_z,
 *    p_vz, p_ref_time
 * 2. Vdata --> NPCD records
 *    ecr_satpos, ecr_satvel, eci_satpos, eci_satvel
 * 3. A. SDS --> NSCA*NBAND records
 *       sca_coef_x
 *    B. SDS --> NSCA*NBAND records
 *       sca_coef_y
 *    C. SDS --> NBAND records
 *       sca_ndet
 *    D. SDS --> NPCD*NAXIS records
 *       rpy
 */

int axx_put_alimodel (
  const char alimodel_filename[STRLEN],	/* I: HDF output file name               */
  const ALIMODEL *alimodel)    		/* I: ALI model structure to be output  */
{
  int  i, j,                 /* Loop variables                      */
       currsize,             /* Current size of the data element    */
       num_ints,             /* Number of ints to malloc for        */
       num_doubles,          /* Number of doubles to malloc for     */
       num_chars;            /* Number of characters to malloc for  */
  intn istatn;               /* Return values            DMW        */
  int32 file_id,             /* DDR HDF file pointer     DMW        */
        sd_id,               /* DDR HDF file pointer for sds        */
        vdata_id1,           /* Vdata pointer                       */
        vdata_id2,           /* Vdata pointer                       */
        sds_id3a,            /* SDS id pointer (mprofiles)          */
        sds_id3b,            /* SDS id pointer (scantime)           */
        sds_id3c,            /* SDS id pointer (mscd)               */
        sds_id3d,            /* SDS id pointer (rpy)                */
        rank,                /* Rank of the dimsizes array.         */
        dimsizes[3],         /* Array of the dimension sizes.       */
        start[] = {0, 0, 0}, /* Array of 0 starting positions.      */
        istat32,             /* Return values                       */
        i32,                 /* Holder for integers                 */
        vgroup_id;           /* Vgroup pointer (master)  DMW        */
  uint8 *databuf,            /* Data pointer                        */
        *pntr;               /* Current position in data pointer    */
  char fields[NELEMENTS1 * XXX_ODL_FIELD_SIZE],
                             /* Packed string of field names of the
                                model.  The alimodel1_elements structure
                                is the largest so give enough space
                                for it.                             */
       ErrMsg[STRLEN];       /* Error message                       */
  double d;                  /* Holder for doubles                  */

  /***********************************************************
   * Open the HDF output file.
   */
  file_id = Hopen (alimodel_filename, DFACC_CREATE, 0);
  if (file_id == FAIL)
  {
    sprintf (ErrMsg, "Error opening new HDF alimodel file %s in axx_put_alimodel.",
             alimodel_filename);
    xxx_LogStatus ("AXXLIB", __FILE__, __LINE__, ErrMsg);
    return (ERROR);
  }
  Vstart (file_id);

  /***********************************************************
   * Create the master Vgroup with write-only access.
   */
  vgroup_id = Vattach (file_id, -1, "w");
  if (vgroup_id == FAIL)
  {
    strcpy (ErrMsg, "Error creating new Vgroup in axx_put_alimodel.");
    xxx_LogStatus ("AXXLIB", __FILE__, __LINE__, ErrMsg);
    Vend (file_id);
    Hclose (file_id);
    return (ERROR);
  }
  Vsetname (vgroup_id, "Master Vgroup");  /* assign Vgroup name              */

  /***********************************************************
   * Assign a vdata id (Vdata1) for write-only access.
   */
  vdata_id1 = VSattach (file_id, -1, "w");
  if (vdata_id1 == FAIL)
  {
    strcpy (ErrMsg,"Error creating new Vdata in axx_put_alimodel.");
    xxx_LogStatus ("AXXLIB", __FILE__, __LINE__, ErrMsg);
    Vdetach (vgroup_id);
    Vend (file_id);
    Hclose (file_id);
    return (ERROR);
  }
  VSsetname (vdata_id1, "Vdata1");         /* assign Vdata name              */
  istatn = VSsetinterlace (vdata_id1, FULL_INTERLACE);
  if (istatn == FAIL)
  {
    strcpy (ErrMsg, "Error setting the interface in axx_put_alimodel.");
    xxx_LogStatus ("AXXLIB", __FILE__, __LINE__, ErrMsg);
    VSdetach (vdata_id1);
    Vdetach (vgroup_id);
    Vend (file_id);
    Hclose (file_id);
    return (ERROR);
  }

  /* Define the fields for the alimodel (Vdata1).
   */
  VSfdefine (vdata_id1, alimodel1_elements[0].name,
            (int32)alimodel1_elements[0].dfnt,
            (int32)alimodel1_elements[0].order);
  strcpy (fields, alimodel1_elements[0].name); /*Initialize the fields. DMW */
  for (i = 1; i < NELEMENTS1; i++)
  {
    VSfdefine (vdata_id1, alimodel1_elements[i].name, 
              (int32)alimodel1_elements[i].dfnt,
              (int32)alimodel1_elements[i].order);  /* DMW  */
    strcat (fields, ",");
    strcat (fields, alimodel1_elements[i].name);
  }
  istatn = VSsetfields(vdata_id1, fields);
  if (istatn == FAIL)
  {
    strcpy (ErrMsg, "Error setting the field names 1 in axx_put_alimodel.");
    xxx_LogStatus ("AXXLIB", __FILE__, __LINE__, ErrMsg);
    VSdetach (vdata_id1);
    Vdetach (vgroup_id);
    Vend (file_id);
    Hclose (file_id);
    return (ERROR);
  }

  /* Set up the data buffer and its pointer for Vdata1.
   */
  num_ints = NUM_INTS1;
  num_chars = NUM_CHARS1;
  num_doubles = NUM_DBLS1;
  databuf = (uint8 *) malloc (num_ints*sizeof(int32) +
                              num_doubles*sizeof(double) +
                              num_chars*sizeof(char));
  if (databuf == NULL)
  {
    strcpy (ErrMsg, "Error allocating data buffer memory in axx_put_alimodel.");
    xxx_LogStatus ("AXXLIB", __FILE__, __LINE__, ErrMsg);
    VSdetach (vdata_id1);
    Vdetach (vgroup_id);
    Vend (file_id);
    Hclose (file_id);
    return (ERROR);
  }
  pntr = databuf;
  j = 0;

  /* Write Vdata1 data to buffers.
   */
  currsize = alimodel1_elements[j].order * alimodel1_elements[j].size;
  i32 = alimodel->satnum;
  memcpy (pntr, (void *)&i32, currsize);
  j++;
  pntr += currsize;

  currsize = alimodel1_elements[j].order * alimodel1_elements[j].size;
  d = alimodel->dce_time[0];
  memcpy (pntr, (void *)&d, currsize);
  j++;
  pntr += currsize;

  currsize = alimodel1_elements[j].order * alimodel1_elements[j].size;
  d = alimodel->dce_time[1];
  memcpy (pntr, (void *)&d, currsize);
  j++;
  pntr += currsize;

  currsize = alimodel1_elements[j].order * alimodel1_elements[j].size;
  d = alimodel->dce_time[2];
  memcpy (pntr, (void *)&d, currsize);
  j++;
  pntr += currsize;

  currsize = alimodel1_elements[j].order * alimodel1_elements[j].size;
  d = alimodel->ms_frame_rate;
  memcpy (pntr, (void *)&d, currsize);
  j++;
  pntr += currsize;

  currsize = alimodel1_elements[j].order * alimodel1_elements[j].size;
  d = alimodel->pan_frame_rate;
  memcpy (pntr, (void *)&d, currsize);
  j++;
  pntr += currsize;

  currsize = alimodel1_elements[j].order * alimodel1_elements[j].size;
  d = alimodel->ms_int_time;
  memcpy (pntr, (void *)&d, currsize);
  j++;
  pntr += currsize;

  currsize = alimodel1_elements[j].order * alimodel1_elements[j].size;
  d = alimodel->pan_int_time;
  memcpy (pntr, (void *)&d, currsize);
  j++;
  pntr += currsize;

  currsize = alimodel1_elements[j].order * alimodel1_elements[j].size;
  d = alimodel->acs_time[0];
  memcpy (pntr, (void *)&d, currsize);
  j++;
  pntr += currsize;

  currsize = alimodel1_elements[j].order * alimodel1_elements[j].size;
  d = alimodel->acs_time[1];
  memcpy (pntr, (void *)&d, currsize);
  j++;
  pntr += currsize;

  currsize = alimodel1_elements[j].order * alimodel1_elements[j].size;
  d = alimodel->acs_time[2];
  memcpy (pntr, (void *)&d, currsize);
  j++;
  pntr += currsize;

  currsize = alimodel1_elements[j].order * alimodel1_elements[j].size;
  i32 = alimodel->nephem;
  memcpy (pntr, (void *)&i32, currsize);
  j++;
  pntr += currsize;

  currsize = alimodel1_elements[j].order * alimodel1_elements[j].size;
  i32 = (int32)alimodel->ephem_type;
  memcpy (pntr, (void *)&i32, currsize);
  j++;
  pntr += currsize;

  currsize = alimodel1_elements[j].order * alimodel1_elements[j].size;
  d = alimodel->ephem_time[0];
  memcpy (pntr, (void *)&d, currsize);
  j++;
  pntr += currsize;

  currsize = alimodel1_elements[j].order * alimodel1_elements[j].size;
  d = alimodel->ephem_time[1];
  memcpy (pntr, (void *)&d, currsize);
  j++;
  pntr += currsize;

  currsize = alimodel1_elements[j].order * alimodel1_elements[j].size;
  d = alimodel->ephem_time[2];
  memcpy (pntr, (void *)&d, currsize);
  j++;
  pntr += currsize;

  currsize = alimodel1_elements[j].order * alimodel1_elements[j].size;
  i32 = alimodel->nsca;
  memcpy (pntr, (void *)&i32, currsize);
  j++;
  pntr += currsize;

  currsize = alimodel1_elements[j].order * alimodel1_elements[j].size;
  i32 = alimodel->nrpy;
  memcpy (pntr, (void *)&i32, currsize);
  j++;
  pntr += currsize;

  currsize = alimodel1_elements[j].order * alimodel1_elements[j].size;
  d = alimodel->sc_roll;
  memcpy (pntr, (void *)&d, currsize);
  j++;
  pntr += currsize;

  currsize = alimodel1_elements[j].order * alimodel1_elements[j].size;
  d = alimodel->sc_pitch;
  memcpy (pntr, (void *)&d, currsize);
  j++;
  pntr += currsize;

  currsize = alimodel1_elements[j].order * alimodel1_elements[j].size;
  d = alimodel->sc_yaw;
  memcpy (pntr, (void *)&d, currsize);
  j++;
  pntr += currsize;

  currsize = alimodel1_elements[j].order * alimodel1_elements[j].size;
  i32 = (int32)alimodel->correction_type;
  memcpy (pntr, (void *)&i32, currsize);
  j++;
  pntr += currsize;

  currsize = alimodel1_elements[j].order * alimodel1_elements[j].size;
  d = alimodel->p_roll;
  memcpy (pntr, (void *)&d, currsize);
  j++;
  pntr += currsize;

  currsize = alimodel1_elements[j].order * alimodel1_elements[j].size;
  d = alimodel->p_rrate;
  memcpy (pntr, (void *)&d, currsize);
  j++;
  pntr += currsize;

  currsize = alimodel1_elements[j].order * alimodel1_elements[j].size;
  d = alimodel->p_pitch;
  memcpy (pntr, (void *)&d, currsize);
  j++;
  pntr += currsize;

  currsize = alimodel1_elements[j].order * alimodel1_elements[j].size;
  d = alimodel->p_prate;
  memcpy (pntr, (void *)&d, currsize);
  j++;
  pntr += currsize;

  currsize = alimodel1_elements[j].order * alimodel1_elements[j].size;
  d = alimodel->p_yaw;
  memcpy (pntr, (void *)&d, currsize);
  j++;
  pntr += currsize;

  currsize = alimodel1_elements[j].order * alimodel1_elements[j].size;
  d = alimodel->p_yrate;
  memcpy (pntr, (void *)&d, currsize);
  j++;
  pntr += currsize;

  currsize = alimodel1_elements[j].order * alimodel1_elements[j].size;
  d = alimodel->p_x;
  memcpy (pntr, (void *)&d, currsize);
  j++;
  pntr += currsize;

  currsize = alimodel1_elements[j].order * alimodel1_elements[j].size;
  d = alimodel->p_vx;
  memcpy (pntr, (void *)&d, currsize);
  j++;
  pntr += currsize;

  currsize = alimodel1_elements[j].order * alimodel1_elements[j].size;
  d = alimodel->p_y;
  memcpy (pntr, (void *)&d, currsize);
  j++;
  pntr += currsize;

  currsize = alimodel1_elements[j].order * alimodel1_elements[j].size;
  d = alimodel->p_vy;
  memcpy (pntr, (void *)&d, currsize);
  j++;
  pntr += currsize;

  currsize = alimodel1_elements[j].order * alimodel1_elements[j].size;
  d = alimodel->p_z;
  memcpy (pntr, (void *)&d, currsize);
  j++;
  pntr += currsize;

  currsize = alimodel1_elements[j].order * alimodel1_elements[j].size;
  d = alimodel->p_vz;
  memcpy (pntr, (void *)&d, currsize);
  j++;
  pntr += currsize;

  currsize = alimodel1_elements[j].order * alimodel1_elements[j].size;
  d = alimodel->p_ref_time;
  memcpy (pntr, (void *)&d, currsize);
  j++;
  pntr += currsize;

  currsize = alimodel1_elements[j].order * alimodel1_elements[j].size;
  i32 = alimodel->l0r_sw_version[0];
  memcpy (pntr, (void *)&i32, currsize);
  j++;
  pntr += currsize;

  currsize = alimodel1_elements[j].order * alimodel1_elements[j].size;
  i32 = alimodel->l0r_sw_version[1];
  memcpy (pntr, (void *)&i32, currsize);
  j++;
  pntr += currsize;

  currsize = alimodel1_elements[j].order * alimodel1_elements[j].size;
  i32 = alimodel->l0r_sw_version[2];
  memcpy (pntr, (void *)&i32, currsize);
  j++;
  pntr += currsize;

  /* Write buffer to Vdata1.  Note there is only one record for this Vdata.
   */
  istat32 = VSwrite (vdata_id1, databuf, 1, FULL_INTERLACE);
  if (istat32 == FAIL)
  {
    strcpy (ErrMsg,
     "Error writing alimodel structure to the HDF file in axx_put_alimodel.");
    xxx_LogStatus ("AXXLIB", __FILE__, __LINE__, ErrMsg);
    free (databuf);
    VSdetach (vdata_id1);
    Vdetach (vgroup_id);
    Vend (file_id);
    Hclose (file_id);
    return (ERROR);
  }

  /* Insert the Vdata into the Vgroup and detach the Vdata.
   */
  Vinsert (vgroup_id, vdata_id1);
  VSdetach (vdata_id1);
  free (databuf);

  /***********************************************************
   * Assign a vdata id (Vdata2) for write-only access.
   */
  vdata_id2 = VSattach (file_id, -1, "w");
  if (vdata_id2 == FAIL)
  {
    strcpy (ErrMsg,"Error creating new Vdata in axx_put_alimodel.");
    xxx_LogStatus ("AXXLIB", __FILE__, __LINE__, ErrMsg);
    Vdetach (vgroup_id);
    Vend (file_id);
    Hclose (file_id);
    return (ERROR);
  }
  VSsetname (vdata_id2, "Vdata2");         /* assign Vdata name              */
  istatn = VSsetinterlace (vdata_id2, FULL_INTERLACE);
  if (istatn == FAIL)
  {
    strcpy (ErrMsg, "Error setting the interface in axx_put_alimodel.");
    xxx_LogStatus ("AXXLIB", __FILE__, __LINE__, ErrMsg);
    VSdetach (vdata_id2);
    Vdetach (vgroup_id);
    Vend (file_id);
    Hclose (file_id);
    return (ERROR);
  }

  /* Define the fields for the alimodel (Vdata2).
   */
  VSfdefine (vdata_id2, alimodel2_elements[0].name, 
            (int32)alimodel2_elements[0].dfnt,
            (int32)alimodel2_elements[0].order);
  strcpy (fields, alimodel2_elements[0].name); /*Initialize the fields. DMW */
  for (i = 1; i < NELEMENTS2; i++)
  {
    VSfdefine (vdata_id2, alimodel2_elements[i].name, 
              (int32)alimodel2_elements[i].dfnt,
              (int32)alimodel2_elements[i].order);  /*  DMW  */
    strcat (fields, ",");
    strcat (fields, alimodel2_elements[i].name);
  }
  istatn = VSsetfields(vdata_id2, fields);
  if (istatn == FAIL)
  {
    strcpy (ErrMsg, "Error setting the field names 2 in axx_put_alimodel.");
    xxx_LogStatus ("AXXLIB", __FILE__, __LINE__, ErrMsg);
    VSdetach (vdata_id2);
    Vdetach (vgroup_id);
    Vend (file_id);
    Hclose (file_id);
    return (ERROR);
  }

  /* Set up the data buffer and its pointer for Vdata2.
   */
  num_doubles = NUM_DBLS2;
  databuf = (uint8 *) malloc (num_doubles*sizeof(double) * NPCD);
  if (databuf == NULL)
  {
    strcpy (ErrMsg, "Error allocating data buffer memory in axx_put_alimodel.");
    xxx_LogStatus ("AXXLIB", __FILE__, __LINE__, ErrMsg);
    VSdetach (vdata_id2);
    Vdetach (vgroup_id);
    Vend (file_id);
    Hclose (file_id);
    return (ERROR);
  }
  pntr = databuf;

  /* Write the data to buffers.
   */
  for (i = 0, j = 0; i < NPCD; i++, j = 0)
  {
    currsize = alimodel2_elements[j].order * alimodel2_elements[j].size;
    d = alimodel->ecr_satpos[0][i];
    memcpy (pntr, (void *)&d, currsize);
    j++;
    pntr += currsize;
  
    currsize = alimodel2_elements[j].order * alimodel2_elements[j].size;
    d = alimodel->ecr_satpos[1][i];
    memcpy (pntr, (void *)&d, currsize);
    j++;
    pntr += currsize;
  
    currsize = alimodel2_elements[j].order * alimodel2_elements[j].size;
    d = alimodel->ecr_satpos[2][i];
    memcpy (pntr, (void *)&d, currsize);
    j++;
    pntr += currsize;
  
    currsize = alimodel2_elements[j].order * alimodel2_elements[j].size;
    d = alimodel->ecr_satvel[0][i];
    memcpy (pntr, (void *)&d, currsize);
    j++;
    pntr += currsize;
  
    currsize = alimodel2_elements[j].order * alimodel2_elements[j].size;
    d = alimodel->ecr_satvel[1][i];
    memcpy (pntr, (void *)&d, currsize);
    j++;
    pntr += currsize;
  
    currsize = alimodel2_elements[j].order * alimodel2_elements[j].size;
    d = alimodel->ecr_satvel[2][i];
    memcpy (pntr, (void *)&d, currsize);
    j++;
    pntr += currsize;
    currsize = alimodel2_elements[j].order * alimodel2_elements[j].size;
    d = alimodel->eci_satpos[0][i];
    memcpy (pntr, (void *)&d, currsize);
    j++;
    pntr += currsize;
  
    currsize = alimodel2_elements[j].order * alimodel2_elements[j].size;
    d = alimodel->eci_satpos[1][i];
    memcpy (pntr, (void *)&d, currsize);
    j++;
    pntr += currsize;
  
    currsize = alimodel2_elements[j].order * alimodel2_elements[j].size;
    d = alimodel->eci_satpos[2][i];
    memcpy (pntr, (void *)&d, currsize);
    j++;
    pntr += currsize;
  
    currsize = alimodel2_elements[j].order * alimodel2_elements[j].size;
    d = alimodel->eci_satvel[0][i];
    memcpy (pntr, (void *)&d, currsize);
    j++;
    pntr += currsize;
  
    currsize = alimodel2_elements[j].order * alimodel2_elements[j].size;
    d = alimodel->eci_satvel[1][i];
    memcpy (pntr, (void *)&d, currsize);
    j++;
    pntr += currsize;
  
    currsize = alimodel2_elements[j].order * alimodel2_elements[j].size;
    d = alimodel->eci_satvel[2][i];
    memcpy (pntr, (void *)&d, currsize);
    j++;
    pntr += currsize;
  }

  /* Write buffer to Vdata2.  Note there are NPCD records in this Vdata.
   */
  istat32 = VSwrite (vdata_id2, databuf, NPCD, FULL_INTERLACE);
  if (istat32 == FAIL)
  {
    strcpy (ErrMsg,
     "Error writing alimodel structure to the HDF file in axx_put_alimodel.");
    xxx_LogStatus ("AXXLIB", __FILE__, __LINE__, ErrMsg);
    free (databuf);
    VSdetach (vdata_id2);
    Vdetach (vgroup_id);
    Vend (file_id);
    Hclose (file_id);
    return (ERROR);
  }

  /* Insert the Vdata into the Vgroup and detach the Vdata.
   */
  Vinsert (vgroup_id, vdata_id2);
  VSdetach (vdata_id2);
  free (databuf);

  /* Close the Vgroup and the file.
   */
  Vdetach (vgroup_id);
  Vend (file_id);
  Hclose (file_id);

  /***********************************************************
   * Open the file for the SDS interface.
   */
  sd_id = SDstart (alimodel_filename, DFACC_RDWR);
  if (sd_id == FAIL)
  {
    sprintf (ErrMsg, "Error opening alimodel file %s for SDS in axx_put_alimodel.",
      alimodel_filename);
    xxx_LogStatus ("AXXLIB", __FILE__, __LINE__, ErrMsg);
    return (ERROR);
  }

  /***********************************************************
   * Assign an sds (sca_coef_x) for write-only access.
   */
  /* Create the SDS array.  The array will have 3 dimensions.  One will be
   * for the number of SCAs (NSCA), one for the number of bands (NBAND), and
   * one will be for the NUM_COEFF_PER_BAND Legendre coefficient dimension
   * (size = NUM_COEFF_PER_BAND).
   */
  rank = 3;
  dimsizes[0] = NSCA;
  dimsizes[1] = NBAND;
  dimsizes[2] = NUM_COEFF_PER_BAND;
  sds_id3a = SDcreate (sd_id, "sca_coef_x", (int32)alimodel3a_elements[0].dfnt,
                       rank, dimsizes);    /*  DMW  */
  if (sds_id3a == FAIL)
  {
    strcpy (ErrMsg, "Error creating sds_id3a in axx_put_alimodel.");
    xxx_LogStatus ("AXXLIB", __FILE__, __LINE__, ErrMsg);
    SDendaccess (sds_id3a);
    SDend (sd_id);
    return (ERROR);
  }

  /* Write data to sca_coef_x SDS.
   */
  istatn = SDwritedata (sds_id3a, start, NULL, dimsizes,
           (VOIDP)alimodel->sca_coef_x);
  if (istatn == FAIL)
  {
    strcpy (ErrMsg,
     "Error writing the alimodel structure to the HDF file in axx_put_alimodel");
    xxx_LogStatus ("AXXLIB", __FILE__, __LINE__, ErrMsg);
    SDendaccess (sds_id3a);
    SDend (sd_id);
    return (ERROR);
  }

  /* Terminate access to the SDS.
   */
  SDendaccess (sds_id3a);

  /***********************************************************
   * Assign an sds (sca_coef_y) for write-only access.
   */
  /* Create the SDS array.  The array will have 3 dimensions.  One will be
   * for the number of SCAs (NSCA), one for the number of bands (NBAND), and
   * one will be for the NUM_COEFF_PER_BAND Legendre coefficient dimension
   * (size = NUM_COEFF_PER_BAND).
   */
  rank = 3;
  dimsizes[0] = NSCA;
  dimsizes[1] = NBAND;
  dimsizes[2] = NUM_COEFF_PER_BAND;
  sds_id3b = SDcreate (sd_id, "sca_coef_y", (int32)alimodel3b_elements[0].dfnt,
                       rank, dimsizes);    /*  DMW  */
  if (sds_id3b == FAIL)
  {
    strcpy (ErrMsg, "Error creating sds_id3b in axx_put_alimodel.");
    xxx_LogStatus ("AXXLIB", __FILE__, __LINE__, ErrMsg);
    SDendaccess (sds_id3b);
    SDend (sd_id);
    return (ERROR);
  }

  /* Write data to sca_coef_y SDS.
   */
  istatn = SDwritedata (sds_id3b, start, NULL, dimsizes,
           (VOIDP)alimodel->sca_coef_y);
  if (istatn == FAIL)
  {
    strcpy (ErrMsg,
     "Error writing alimodel structure to the HDF file in axx_put_alimodel.");
    xxx_LogStatus ("AXXLIB", __FILE__, __LINE__, ErrMsg);
    SDendaccess (sds_id3b);
    SDend (sd_id);
    return (ERROR);
  }

  /* Terminate access to the SDS.
   */
  SDendaccess (sds_id3b);

  /***********************************************************
   * Assign an sds (sca_ndet) for write-only access.
   */
  /* Create the SDS array.  The array will have 1 dimension (NBANDS).
   */
  rank = 1;
  dimsizes[0] = NBAND;
  sds_id3c = SDcreate (sd_id, "sca_ndet", DFNT_INT32, rank, dimsizes);  
  if (sds_id3c == FAIL)
  {
    strcpy (ErrMsg, "Error creating sds_id3c in axx_put_alimodel.");
    xxx_LogStatus ("AXXLIB", __FILE__, __LINE__, ErrMsg);
    SDendaccess (sds_id3c);
    SDend (sd_id);
    return (ERROR);
  }

  /* Write buffer to sca_ndet SDS.
   */
  istatn = SDwritedata (sds_id3c, start, NULL, dimsizes,
           (VOIDP)alimodel->sca_num_det);
  if (istatn == FAIL)
  {
    strcpy (ErrMsg,
      "Error writing alimodel structure to the HDF file in axx_put_alimodel.");
    xxx_LogStatus ("AXXLIB", __FILE__, __LINE__, ErrMsg);
    SDendaccess (sds_id3c);
    SDend (sd_id);
    return (ERROR);
  }

  /* Terminate access to the SDS.
   */
  SDendaccess (sds_id3c);

  /***********************************************************
   * Assign an sds (rpy) for write-only access.
   */
  /* Create the SDS array.  The array will have 2 dimensions.  One will be
   * for the number of ADS data values (size = NPCD).  One will be for the
   * number of axis. (size = NAXIS).
   */
  rank = 2;
  dimsizes[0] = NPCD;
  dimsizes[1] = NAXIS;
  sds_id3d = SDcreate (sd_id, "RPY", (int32)alimodel3d_elements[0].dfnt,
                       rank, dimsizes);      /*  DMW  */
  if (sds_id3d == FAIL)
  {
    strcpy (ErrMsg, "Error creating sds_id3d in axx_put_alimodel.");
    xxx_LogStatus ("AXXLIB", __FILE__, __LINE__, ErrMsg);
    SDendaccess (sds_id3d);
    SDend (sd_id);
    return (ERROR);
  }

  /* Write data to rpy SDS.
   */
  istatn = SDwritedata (sds_id3d, start, NULL, dimsizes,
           (VOIDP)alimodel->rpy);
  if (istatn == FAIL)
  {
    strcpy (ErrMsg,
      "Error writing alimodel structure to the HDF file in axx_put_alimodel.");
    xxx_LogStatus ("AXXLIB", __FILE__, __LINE__, ErrMsg);
    SDendaccess (sds_id3d);
    SDend (sd_id);
    return (ERROR);
  }

  /* Terminate access to the SDS.
   */
  SDendaccess (sds_id3d);

  /***********************************************************
   * Close the file.
   */
  SDend (sd_id);

  return (SUCCESS);
}

/******************************************************************************
NAME:           axx_get_alimodel

PURPOSE:
Read the ALI model from an HDF file.

DEVELOPMENT HISTORY:
AUTHOR        Change-ID    DATE     Description of Change
------        ---------  --------   ---------------------
Jim Storey       1.0     11/17/00   Original development.

RETURN VALUE:
Type = int
Value    Description
-----    -----------
SUCCESS  Successful completion
ERROR    Operation failed

NOTES:
Any changes made to the ALI model structure need to be reflected in this
routine.  Any changes made to this routine need to be reflected in the
axx_put_alimodel routine.

ALGORITHM REFERENCES:
See axx_alimodel.h for a description of the alimodel structure.

******************************************************************************
                        Property of the U.S. Government
                            USGS EROS Data Center
******************************************************************************/

int axx_get_alimodel (
  const char alimodel_filename[STRLEN],	/* I: HDF input file name              */
  ALIMODEL *alimodel)             	/* O: ALI model structure to be read  */
{
  int  i, j,                 /* Loop variables                      */
       currsize,             /* Current size of the data element    */
       nvdatas,              /* Number of vdatas in the vgroup      */
       num_ints,             /* Number of ints to malloc for        */
       num_doubles,          /* Number of doubles to malloc for     */
       num_chars;            /* Number of characters to malloc for  */
  intn istatn;               /* Return values             DMW       */
  int32 file_id,              /* DDR HDF file pointer     DMW       */
        num_sds,              /* Number of sds in the file          */
        sd_id,                /* DDR HDF file pointer for sds       */
        sds_index,            /* SDS index pointer                  */
        num_file_attribs,     /* Number of attribs in the file       */
        num_type,             /* Data type for the data in the array */
        attrib,               /* Number of attributes in the data set*/
        vdata_id1,            /* Vdata pointer (constants)          */
        vdata_id2,            /* Vdata pointer (band-dependent)     */
        sds_id3a,             /* SDS id pointer (in_lines/samps)    */
        sds_id3b,             /* SDS id pointer (out_lines/samps)   */
        sds_id3c,             /* SDS id pointer (poly_lines/samps)  */
        sds_id3d,             /* SDS id pointer (sattoproj)         */
        rank,                 /* Rank of the dimsizes array.        */
        dim_sizes[3],         /* Array of the dimension sizes.      */
        start[] = {0, 0, 0},  /* Array of 0 starting positions.     */
        vdata_tag[NUM_VDATAS],/* Array of vdata tags in the vgroup  */
        vdata_ref[NUM_VDATAS],/* Array of vdata refs in the vgroup  */
        istat32,              /* Return values            DMW       */
        vgroup_id,            /* Vgroup pointer (master)  DMW       */
        vgroup_ref,           /* Vgroup reference number(master)DMW */
        i32,                  /* Holder for integers                */
	num_fields;           /* Number of fields in VData          */
  uint8 *databuf,             /* Data pointer                       */
        *pntr;                /* Current position in data pointer   */
  char sds_name[STRLEN],      /* Name of the sds                    */
       vdata_name[STRLEN],    /* Name of the vdata                  */
       fields[NELEMENTS1 * XXX_ODL_FIELD_SIZE],
                             /* Packed string of field names of the
                                model.  The alimodel1_elements structure
                                is the largest so give enough space
                                for it.                             */
       ErrMsg[STRLEN];       /* Error message                       */
  double d;                  /* Holder for doubles                  */

  /***********************************************************
   * Open the HDF output file.
   */
  file_id = Hopen (alimodel_filename, DFACC_RDONLY, 0);
  if (file_id == FAIL)
  {
    sprintf (ErrMsg, "Error opening HDF alimodel file %s axx_get_alimodel.",
             alimodel_filename);
    xxx_LogStatus ("AXXLIB", __FILE__, __LINE__, ErrMsg);
    return (ERROR);
  }
  Vstart (file_id);

  /* Get the reference number of the first vgroup in the file and attach
   * to it for read only.
   */
  vgroup_ref = Vgetid (file_id, -1);
  vgroup_id = Vattach (file_id, vgroup_ref, "r");
  if (vgroup_id == FAIL)
  {
    strcpy (ErrMsg,"Error reading first Vgroup in axx_get_alimodel.");
    xxx_LogStatus ("AXXLIB", __FILE__, __LINE__, ErrMsg);
    Vend (file_id);
    Hclose (file_id);
    return (ERROR);
  }

  /* Verify the Vgroup contains all of our Vdatas.  The Vgroup should
   * have NUM_VDATAS Vdatas, in the order of Vdata names "Vdata1" and
   * "Vdata2".
   */
  nvdatas = Vntagrefs (vgroup_id);
  if (nvdatas != NUM_VDATAS)
  {
    sprintf (ErrMsg, "Only %ld vdatas in the HDF file in axx_get_alimodel.",
             nvdatas);
    xxx_LogStatus ("AXXLIB", __FILE__, __LINE__, ErrMsg);
    Vend (file_id);
    Hclose (file_id);
    return (ERROR);
  }
  istat32 = Vgettagrefs (vgroup_id, vdata_tag, vdata_ref, nvdatas);
  if (istat32 == FAIL)
  {
    strcpy (ErrMsg,
      "Error getting the Vdata tags and refs in axx_get_alimodel.");
    xxx_LogStatus ("AXXLIB", __FILE__, __LINE__, ErrMsg);
    Vend (file_id);
    Hclose (file_id);
    return (ERROR);
  }

  /****************************************************
   * Find Vdata1.
   */
  /* Assign a vdata id for read-only access.
   */
  vdata_id1 = VSattach (file_id, vdata_ref[0], "r");
  if (vdata_id1 == FAIL)
  {
    strcpy (ErrMsg, "Error reading constants Vdata in axx_get_alimodel.");
    xxx_LogStatus ("AXXLIB", __FILE__, __LINE__, ErrMsg);
    Vdetach (vgroup_id);
    Vend (file_id);
    Hclose (file_id);
    return (ERROR);
  }

  /* Verify the name of the Vdata is "Vdata1".
   */
  istatn = VSQueryname (vdata_id1, vdata_name);
  if (istatn == FAIL)
  {
    strcpy (ErrMsg, "Error inquiring about the vdata in axx_get_alimodel.");
    xxx_LogStatus ("AXXLIB", __FILE__, __LINE__, ErrMsg);
    Vdetach (vgroup_id);
    VSdetach (vdata_id1);
    Vend (file_id);
    Hclose (file_id);
    return (ERROR);
  }
  if (strcmp (vdata_name, "Vdata1"))
  {
    strcpy (ErrMsg, "Error in the name of the vdata in axx_get_alimodel.");
    xxx_LogStatus ("AXXLIB", __FILE__, __LINE__, ErrMsg);
    Vdetach (vgroup_id);
    VSdetach (vdata_id1);
    Vend (file_id);
    Hclose (file_id);
    return (ERROR);
  }

  /* Set the order of the fields for Vdata1.  This is the same order we
   * wrote them.
   */
  num_fields = VSQueryfields (vdata_id1, fields);
  if (num_fields == FAIL)
  {
    strcpy (ErrMsg, "Error inquiring about the vdata in axx_get_alimodel.");
    xxx_LogStatus ("AXXLIB", __FILE__, __LINE__, ErrMsg);
    Vdetach (vgroup_id);
    VSdetach (vdata_id1);
    Vend (file_id);
    Hclose (file_id);
    return (ERROR);
  }
  istatn = VSsetfields(vdata_id1, fields);
  if (istatn == FAIL)
  {
    strcpy (ErrMsg, "Error setting the field names axx_get_alimodel.");
    xxx_LogStatus ("AXXLIB", __FILE__, __LINE__, ErrMsg);
    Vdetach (vgroup_id);
    VSdetach (vdata_id1);
    Vend (file_id);
    Hclose (file_id);
    return (ERROR);
  }

  /* Set up the data buffer and its pointer.  (Count the number of ints,
   * doubles, and characters for the malloc.)
   */
  num_ints = NUM_INTS1;
  num_chars = NUM_CHARS1;
  num_doubles = NUM_DBLS1;
  databuf = (uint8 *) malloc (num_ints*sizeof(int32) +
                              num_doubles*sizeof(double) +
                              num_chars*sizeof(char));
  if (databuf == NULL)
  {
    strcpy (ErrMsg, "Error allocating data buffer memory in axx_get_alimodel.");
    xxx_LogStatus ("AXXLIB", __FILE__, __LINE__, ErrMsg);
    VSdetach (vdata_id1);
    Vdetach (vgroup_id);
    Vend (file_id);
    Hclose (file_id);
    return (ERROR);
  }
  pntr = databuf;

  /* Read the data.  There is only one record in this Vdata.
   */
  istat32 = VSread (vdata_id1, databuf, 1, FULL_INTERLACE);
  if (istat32 == FAIL)
  {
    strcpy (ErrMsg, "Error reading the vdata in axx_get_alimodel.");
    xxx_LogStatus ("AXXLIB", __FILE__, __LINE__, ErrMsg);
    free (databuf);
    Vdetach (vgroup_id);
    VSdetach (vdata_id1);
    Vend (file_id);
    Hclose (file_id);
    return (ERROR);
  }
  j = 0;

  /* Unpack the data elements from the buffer into the model structure.
   */
  currsize = alimodel1_elements[j].order * alimodel1_elements[j].size;
  memcpy (&i32, pntr, currsize);
  alimodel->satnum = i32;
  j++;
  pntr += currsize;

  currsize = alimodel1_elements[j].order * alimodel1_elements[j].size;
  memcpy (&d, pntr, currsize);
  alimodel->dce_time[0] = d;
  j++;
  pntr += currsize;

  currsize = alimodel1_elements[j].order * alimodel1_elements[j].size;
  memcpy (&d, pntr, currsize);
  alimodel->dce_time[1] = d;
  j++;
  pntr += currsize;

  currsize = alimodel1_elements[j].order * alimodel1_elements[j].size;
  memcpy (&d, pntr, currsize);
  alimodel->dce_time[2] = d;
  j++;
  pntr += currsize;

  currsize = alimodel1_elements[j].order * alimodel1_elements[j].size;
  memcpy (&d, pntr, currsize);
  alimodel->ms_frame_rate = d;
  j++;
  pntr += currsize;

  currsize = alimodel1_elements[j].order * alimodel1_elements[j].size;
  memcpy (&d, pntr, currsize);
  alimodel->pan_frame_rate = d;
  j++;
  pntr += currsize;

  currsize = alimodel1_elements[j].order * alimodel1_elements[j].size;
  memcpy (&d, pntr, currsize);
  alimodel->ms_int_time = d;
  j++;
  pntr += currsize;

  currsize = alimodel1_elements[j].order * alimodel1_elements[j].size;
  memcpy (&d, pntr, currsize);
  alimodel->pan_int_time = d;
  j++;
  pntr += currsize;

  currsize = alimodel1_elements[j].order * alimodel1_elements[j].size;
  memcpy (&d, pntr, currsize);
  alimodel->acs_time[0] = d;
  j++;
  pntr += currsize;

  currsize = alimodel1_elements[j].order * alimodel1_elements[j].size;
  memcpy (&d, pntr, currsize);
  alimodel->acs_time[1] = d;
  j++;
  pntr += currsize;

  currsize = alimodel1_elements[j].order * alimodel1_elements[j].size;
  memcpy (&d, pntr, currsize);
  alimodel->acs_time[2] = d;
  j++;
  pntr += currsize;

  currsize = alimodel1_elements[j].order * alimodel1_elements[j].size;
  memcpy (&i32, pntr, currsize);
  alimodel->nephem = i32;
  j++;
  pntr += currsize;

  currsize = alimodel1_elements[j].order * alimodel1_elements[j].size;
  memcpy (&i32, pntr, currsize);
  alimodel->ephem_type = (EphemType)i32;
  j++;
  pntr += currsize;

  currsize = alimodel1_elements[j].order * alimodel1_elements[j].size;
  memcpy (&d, pntr, currsize);
  alimodel->ephem_time[0] = d;
  j++;
  pntr += currsize;

  currsize = alimodel1_elements[j].order * alimodel1_elements[j].size;
  memcpy (&d, pntr, currsize);
  alimodel->ephem_time[1] = d;
  j++;
  pntr += currsize;

  currsize = alimodel1_elements[j].order * alimodel1_elements[j].size;
  memcpy (&d, pntr, currsize);
  alimodel->ephem_time[2] = d;
  j++;
  pntr += currsize;

  currsize = alimodel1_elements[j].order * alimodel1_elements[j].size;
  memcpy (&i32, pntr, currsize);
  alimodel->nsca = i32;
  j++;
  pntr += currsize;

  currsize = alimodel1_elements[j].order * alimodel1_elements[j].size;
  memcpy (&i32, pntr, currsize);
  alimodel->nrpy = i32;
  j++;
  pntr += currsize;

  currsize = alimodel1_elements[j].order * alimodel1_elements[j].size;
  memcpy (&d, pntr, currsize);
  alimodel->sc_roll = d;
  j++;
  pntr += currsize;

  currsize = alimodel1_elements[j].order * alimodel1_elements[j].size;
  memcpy (&d, pntr, currsize);
  alimodel->sc_pitch = d;
  j++;
  pntr += currsize;

  currsize = alimodel1_elements[j].order * alimodel1_elements[j].size;
  memcpy (&d, pntr, currsize);
  alimodel->sc_yaw = d;
  j++;
  pntr += currsize;

  currsize = alimodel1_elements[j].order * alimodel1_elements[j].size;
  memcpy (&i32, pntr, currsize);
  alimodel->correction_type = (CorrectionType)i32;
  j++;
  pntr += currsize;

  currsize = alimodel1_elements[j].order * alimodel1_elements[j].size;
  memcpy (&d, pntr, currsize);
  alimodel->p_roll = d;
  j++;
  pntr += currsize;

  currsize = alimodel1_elements[j].order * alimodel1_elements[j].size;
  memcpy (&d, pntr, currsize);
  alimodel->p_rrate = d;
  j++;
  pntr += currsize;

  currsize = alimodel1_elements[j].order * alimodel1_elements[j].size;
  memcpy (&d, pntr, currsize);
  alimodel->p_pitch = d;
  j++;
  pntr += currsize;

  currsize = alimodel1_elements[j].order * alimodel1_elements[j].size;
  memcpy (&d, pntr, currsize);
  alimodel->p_prate = d;
  j++;
  pntr += currsize;

  currsize = alimodel1_elements[j].order * alimodel1_elements[j].size;
  memcpy (&d, pntr, currsize);
  alimodel->p_yaw = d;
  j++;
  pntr += currsize;

  currsize = alimodel1_elements[j].order * alimodel1_elements[j].size;
  memcpy (&d, pntr, currsize);
  alimodel->p_yrate = d;
  j++;
  pntr += currsize;

  currsize = alimodel1_elements[j].order * alimodel1_elements[j].size;
  memcpy (&d, pntr, currsize);
  alimodel->p_x = d;
  j++;
  pntr += currsize;

  currsize = alimodel1_elements[j].order * alimodel1_elements[j].size;
  memcpy (&d, pntr, currsize);
  alimodel->p_vx = d;
  j++;
  pntr += currsize;

  currsize = alimodel1_elements[j].order * alimodel1_elements[j].size;
  memcpy (&d, pntr, currsize);
  alimodel->p_y = d;
  j++;
  pntr += currsize;

  currsize = alimodel1_elements[j].order * alimodel1_elements[j].size;
  memcpy (&d, pntr, currsize);
  alimodel->p_vy = d;
  j++;
  pntr += currsize;

  currsize = alimodel1_elements[j].order * alimodel1_elements[j].size;
  memcpy (&d, pntr, currsize);
  alimodel->p_z = d;
  j++;
  pntr += currsize;

  currsize = alimodel1_elements[j].order * alimodel1_elements[j].size;
  memcpy (&d, pntr, currsize);
  alimodel->p_vz = d;
  j++;
  pntr += currsize;

  currsize = alimodel1_elements[j].order * alimodel1_elements[j].size;
  memcpy (&d, pntr, currsize);
  alimodel->p_ref_time = d;
  j++;
  pntr += currsize;

  if ( strstr( fields, alimodel1_elements[j].name ) != NULL )
  {
    currsize = alimodel1_elements[j].order * alimodel1_elements[j].size;
    memcpy (&i32, pntr, currsize);
    alimodel->l0r_sw_version[0] = i32;
    j++;
    pntr += currsize;

    currsize = alimodel1_elements[j].order * alimodel1_elements[j].size;
    memcpy (&i32, pntr, currsize);
    alimodel->l0r_sw_version[1] = i32;
    j++;
    pntr += currsize;

    currsize = alimodel1_elements[j].order * alimodel1_elements[j].size;
    memcpy (&i32, pntr, currsize);
    alimodel->l0r_sw_version[2] = i32;
    j++;
    pntr += currsize;
  }
  else
  {
    alimodel->l0r_sw_version[0] = 1;
    alimodel->l0r_sw_version[1] = 1;
    alimodel->l0r_sw_version[2] = 1;
  }

  /* Close the Vdata.
   */
  VSdetach (vdata_id1);
  free (databuf);

  /****************************************************
   * Find Vdata2.
   */
  /* Assign a vdata id for read-only access.
   */
  vdata_id2 = VSattach (file_id, vdata_ref[1], "r");
  if (vdata_id2 == FAIL)
  {
    strcpy (ErrMsg, "Error reading Vdata2 in axx_get_alimodel.");
    xxx_LogStatus ("AXXLIB", __FILE__, __LINE__, ErrMsg);
    Vdetach (vgroup_id);
    Vend (file_id);
    Hclose (file_id);
    return (ERROR);
  }

  /* Verify the name of the Vdata is "Vdata2".
   */
  istatn = VSQueryname (vdata_id2, vdata_name);
  if (istatn == FAIL)
  {
    strcpy (ErrMsg, "Error inquiring about the vdata in axx_get_alimodel.");
    xxx_LogStatus ("AXXLIB", __FILE__, __LINE__, ErrMsg);
    Vdetach (vgroup_id);
    VSdetach (vdata_id2);
    Vend (file_id);
    Hclose (file_id);
    return (ERROR);
  }
  if (strcmp (vdata_name, "Vdata2"))
  {
    strcpy (ErrMsg, "Error in the name of the vdata in axx_get_alimodel.");
    xxx_LogStatus ("AXXLIB", __FILE__, __LINE__, ErrMsg);
    Vdetach (vgroup_id);
    VSdetach (vdata_id2);
    Vend (file_id);
    Hclose (file_id);
    return (ERROR);
  }

  /* Set the order of the fields for Vdata2.  This is the same order we
   * wrote them.
   */
  istatn = VSQueryfields (vdata_id2, fields);
  if (istatn == FAIL)
  {
    strcpy (ErrMsg, "Error inquiring about the vdata in axx_get_alimodel.");
    xxx_LogStatus ("AXXLIB", __FILE__, __LINE__, ErrMsg);
    Vdetach (vgroup_id);
    VSdetach (vdata_id2);
    Vend (file_id);
    Hclose (file_id);
    return (ERROR);
  }
  istatn = VSsetfields(vdata_id2, fields);
  if (istatn == FAIL)
  {
    strcpy (ErrMsg, "Error setting the field names axx_get_alimodel.");
    xxx_LogStatus ("AXXLIB", __FILE__, __LINE__, ErrMsg);
    Vdetach (vgroup_id);
    VSdetach (vdata_id2);
    Vend (file_id);
    Hclose (file_id);
    return (ERROR);
  }

  /* Set up the data buffer and its pointer for Vdata2.
   */
  num_doubles = NUM_DBLS2;
  databuf = (uint8 *) malloc (num_doubles*sizeof(double) * NPCD);
  if (databuf == NULL)
  {
    strcpy (ErrMsg, "Error allocating data buffer memory in axx_get_alimodel.");
    xxx_LogStatus ("AXXLIB", __FILE__, __LINE__, ErrMsg);
    VSdetach (vdata_id2);
    Vdetach (vgroup_id);
    Vend (file_id);
    Hclose (file_id);
    return (ERROR);
  }
  pntr = databuf;

  /* Read the data.  There are NPCD records in this Vdata.
   */
  istat32 = VSread (vdata_id2, databuf, NPCD, FULL_INTERLACE);
  if (istat32 == FAIL)
  {
    strcpy (ErrMsg, "Error reading the vdata in axx_get_alimodel.");
    xxx_LogStatus ("AXXLIB", __FILE__, __LINE__, ErrMsg);
    free (databuf);
    Vdetach (vgroup_id);
    VSdetach (vdata_id2);
    Vend (file_id);
    Hclose (file_id);
    return (ERROR);
  }

  /* Unpack the data elements from the buffer into the etmodel structure.
   */
  for (i = 0, j = 0; i < NPCD; i++, j = 0)
  {
    currsize = alimodel2_elements[j].order * alimodel2_elements[j].size;
    memcpy (&d, pntr, currsize);
    alimodel->ecr_satpos[0][i] = d;
    j++;
    pntr += currsize;
 
    currsize = alimodel2_elements[j].order * alimodel2_elements[j].size;
    memcpy (&d, pntr, currsize);
    alimodel->ecr_satpos[1][i] = d;
    j++;
    pntr += currsize;
 
    currsize = alimodel2_elements[j].order * alimodel2_elements[j].size;
    memcpy (&d, pntr, currsize);
    alimodel->ecr_satpos[2][i] = d;
    j++;
    pntr += currsize;

    currsize = alimodel2_elements[j].order * alimodel2_elements[j].size;
    memcpy (&d, pntr, currsize);
    alimodel->ecr_satvel[0][i] = d;
    j++;
    pntr += currsize;

    currsize = alimodel2_elements[j].order * alimodel2_elements[j].size;
    memcpy (&d, pntr, currsize);
    alimodel->ecr_satvel[1][i] = d;
    j++;
    pntr += currsize;

    currsize = alimodel2_elements[j].order * alimodel2_elements[j].size;
    memcpy (&d, pntr, currsize);
    alimodel->ecr_satvel[2][i] = d;
    j++;
    pntr += currsize;

    currsize = alimodel2_elements[j].order * alimodel2_elements[j].size;
    memcpy (&d, pntr, currsize);
    alimodel->eci_satpos[0][i] = d;
    j++;
    pntr += currsize;

    currsize = alimodel2_elements[j].order * alimodel2_elements[j].size;
    memcpy (&d, pntr, currsize);
    alimodel->eci_satpos[1][i] = d;
    j++;
    pntr += currsize;

    currsize = alimodel2_elements[j].order * alimodel2_elements[j].size;
    memcpy (&d, pntr, currsize);
    alimodel->eci_satpos[2][i] = d;
    j++;
    pntr += currsize;

    currsize = alimodel2_elements[j].order * alimodel2_elements[j].size;
    memcpy (&d, pntr, currsize);
    alimodel->eci_satvel[0][i] = d;
    j++;
    pntr += currsize;

    currsize = alimodel2_elements[j].order * alimodel2_elements[j].size;
    memcpy (&d, pntr, currsize);
    alimodel->eci_satvel[1][i] = d;
    j++;
    pntr += currsize;

    currsize = alimodel2_elements[j].order * alimodel2_elements[j].size;
    memcpy (&d, pntr, currsize);
    alimodel->eci_satvel[2][i] = d;
    j++;
    pntr += currsize;
  }

  /* Close the Vdata.
   */
  VSdetach (vdata_id2);
  free (databuf);

  /* Close the Vgroup and the file.
   */
  Vdetach (vgroup_id);
  Vend (file_id);
  Hclose (file_id);

  /***********************************************************
   * Open the file for the SDS interface.
   */
  sd_id = SDstart (alimodel_filename, DFACC_RDONLY);
  if (sd_id == FAIL)
  {
    sprintf (ErrMsg, "Error opening alimodel file %s for SDS in axx_get_alimodel.",
             alimodel_filename);
    xxx_LogStatus ("AXXLIB", __FILE__, __LINE__, ErrMsg);
    return (ERROR);
  }

  /* Verify the file contains all of our SDSs.  The file should have
   * NUM_SDS SDSs, in the order of SDS names "sca_coef_x", "sca_coef_y",
   * "sca_ndet", and "RPY". 
   */
  SDfileinfo (sd_id, &num_sds, &num_file_attribs);
  if (num_sds != NUM_SDS)
  {
    sprintf (ErrMsg,"Only %ld SDSs in the HDF file in axx_get_alimodel.",
             num_sds);
    xxx_LogStatus ("AXXLIB", __FILE__, __LINE__, ErrMsg);
    SDend (sd_id);
    return (ERROR);
  }

  /****************************************************
   * Find the sca_coef_x SDS.
   */
  sds_index = SDnametoindex (sd_id, "sca_coef_x");
  if (sds_index == FAIL)
  {
    strcpy (ErrMsg, "Error finding the sca_coef_x SDS in axx_get_alimodel.");
    xxx_LogStatus ("AXXLIB", __FILE__, __LINE__, ErrMsg);
    SDend (sd_id);
    return (ERROR);
  }
  sds_id3a = SDselect (sd_id, sds_index);

  /* Get the rank and dimension sizes.
   */
  istatn = SDgetinfo (sds_id3a, sds_name, &rank, dim_sizes, &num_type, &attrib);
  if (istatn == FAIL)
  {
    strcpy (ErrMsg, "Error getting SDS info in axx_get_alimodel.");
    xxx_LogStatus ("AXXLIB", __FILE__, __LINE__, ErrMsg);
    SDendaccess (sds_id3a);
    SDend (sd_id);
    return (ERROR);
  }

  /* Read the sca_coef_x data.  This is the same order we wrote them.
   */
  istatn = SDreaddata (sds_id3a, start, NULL, dim_sizes,
          (VOIDP)alimodel->sca_coef_x);
  if (istatn == FAIL)
  {
    strcpy (ErrMsg, "Error reading the sca_coef_x SDS in axx_get_alimodel.");
    xxx_LogStatus ("AXXLIB", __FILE__, __LINE__, ErrMsg);
    SDendaccess (sds_id3a);
    SDend (sd_id);
    return (ERROR);
  }

  /* Close the SDS.
   */
  SDendaccess (sds_id3a);

  /****************************************************
   * Find the sca_coef_y SDS.
   */
  sds_index = SDnametoindex (sd_id, "sca_coef_y");
  if (sds_index == FAIL)
  {
    strcpy (ErrMsg, "Error finding the sca_coef_y SDS in axx_get_alimodel.");
    xxx_LogStatus ("AXXLIB", __FILE__, __LINE__, ErrMsg);
    SDend (sd_id);
    return (ERROR);
  }
  sds_id3b = SDselect (sd_id, sds_index);

  /* Get the rank and dimension sizes.
   */
  istatn = SDgetinfo (sds_id3b, sds_name, &rank, dim_sizes, &num_type, &attrib);
  if (istatn == FAIL)
  {
    strcpy (ErrMsg, "Error getting SDS info in axx_get_alimodel.");
    xxx_LogStatus ("AXXLIB", __FILE__, __LINE__, ErrMsg);
    SDendaccess (sds_id3b);
    SDend (sd_id);
    return (ERROR);
  }

  /* Read the sca_coef_y data.  This is the same order we wrote them.
   */
  istatn = SDreaddata (sds_id3b, start, NULL, dim_sizes,
          (VOIDP)alimodel->sca_coef_y);
  if (istatn == FAIL)
  {
    strcpy (ErrMsg, "Error reading the sca_coef_y SDS in axx_get_alimodel.");
    xxx_LogStatus ("AXXLIB", __FILE__, __LINE__, ErrMsg);
    SDendaccess (sds_id3b);
    SDend (sd_id);
    return (ERROR);
  }

  /* Close the SDS.
   */
  SDendaccess (sds_id3b);

  /****************************************************
   * Find the sca_ndet SDS.
   */
  sds_index = SDnametoindex (sd_id, "sca_ndet");
  if (sds_index == FAIL)
  {
    strcpy (ErrMsg, "Error finding the sca_ndet SDS in axx_get_alimodel.");
    xxx_LogStatus ("AXXLIB", __FILE__, __LINE__, ErrMsg);
    SDend (sd_id);
    return (ERROR);
  }
  sds_id3c = SDselect (sd_id, sds_index);

  /* Get the rank and dimension sizes.
   */
  istatn = SDgetinfo (sds_id3c, sds_name, &rank, dim_sizes, &num_type, &attrib);
  if (istatn == FAIL)
  {
    strcpy (ErrMsg, "Error getting SDS info in axx_get_alimodel.");
    xxx_LogStatus ("AXXLIB", __FILE__, __LINE__, ErrMsg);
    SDendaccess (sds_id3c);
    SDend (sd_id);
    return (ERROR);
  }

  /* Read the sca_ndet data.  This is the same order we wrote them.
   */
  istatn = SDreaddata (sds_id3c, start, NULL, dim_sizes,
                       (VOIDP)alimodel->sca_num_det);
  if (istatn == FAIL)
  {
    strcpy (ErrMsg, "Error reading the sca_ndet SDS in axx_get_alimodel.");
    xxx_LogStatus ("AXXLIB", __FILE__, __LINE__, ErrMsg);
    SDendaccess (sds_id3c);
    SDend (sd_id);
    return (ERROR);
  }

  /* Close the Vdata.
   */
  SDendaccess (sds_id3c);

  /***********************************************************
   * Find the RPY SDS.
   */
  sds_index = SDnametoindex (sd_id, "RPY");
  if (sds_index == FAIL)
  {
    strcpy (ErrMsg, "Error finding the RPY SDS in axx_get_alimodel.");
    xxx_LogStatus ("AXXLIB", __FILE__, __LINE__, ErrMsg);
    SDend (sd_id);
    return (ERROR);
  }
  sds_id3d = SDselect (sd_id, sds_index);

  /* Get the rank and dimension sizes.
   */
  istatn = SDgetinfo (sds_id3d, sds_name, &rank, dim_sizes, &num_type, &attrib);
  if (istatn == FAIL)
  {
    strcpy (ErrMsg, "Error getting SDS info in axx_get_alimodel.");
    xxx_LogStatus ("AXXLIB", __FILE__, __LINE__, ErrMsg);
    SDendaccess (sds_id3d);
    SDend (sd_id);
    return (ERROR);
  }

  /* Read the RPY.  This is the same order we wrote them.
   */
  istatn = SDreaddata (sds_id3d, start, NULL, dim_sizes,
           (VOIDP)alimodel->rpy);
  if (istatn == FAIL)
  {
    strcpy (ErrMsg, "Error reading the RPY SDS in axx_get_alimodel.");
    xxx_LogStatus ("AXXLIB", __FILE__, __LINE__, ErrMsg);
    SDendaccess (sds_id3d);
    SDend (sd_id);
    return (ERROR);
  }

  /* Close the SDS.
   */
  SDendaccess (sds_id3d);

  /***********************************************************
   * Close the file.
   */
  SDend (sd_id);

  return (SUCCESS);
}
