/* ********************************************************************
NAME: axx_los_coeffs

PURPOSE:
Extract the ALI LOS coefficients from the CPF and load them into
the ALI model structure.

DEVELOPMENT HISTORY:
$Log: axx_los_coeffs.c,v $
Revision 1.2  2000/11/28 16:30:18  storey
Fixed bug which was improperly loading the across track (Y) SCA Legendre
coefficients into the model from the CPF.

Revision 1.1  2000/11/27 13:29:28  storey
Original version


RETURN VALUE:
Type = int
Value    Description
-----    -----------
SUCCESS  Successful completion 
ERROR    Operation failed

NOTES:

ALGORITHM REFERENCES:
******************************************************************************
		        Property of the U.S. Government
                             USGS EROS Data Center
******************************************************************************/
#include "xxx_Types.h"  	/* defines the ERROR variables */
#include "exx_cpf.h"  		/* cpf structure definition */
#include "axx_alimodel.h"	/* model structure definition */


int axx_los_coeffs(
    ALIMODEL *amodel, 		/* IO: The ALI model information */
    const CPF *cpf )		/* I:  The EO1 CPF structure */

{
int 	isca, iband;     	/* Loop variables                            */

/* ***************************
  Set the number of SCAs
  **************************** */
amodel->nsca = cpf->Focal_Plane_Parameters.Num_SCA;

/********************************************************************
  Load the number of detectors array from the first SCA
******************************************************************** */
for ( iband=0; iband<NUM_ALI_BANDS; iband++ )
  amodel->sca_num_det[iband] =
    cpf->Focal_Plane_Parameters.SCA_Parameters[0].Number_Detectors[iband];

/*******************************************************************
  Loop through the SCAs and load the Legendre coefficients
********************************************************************/
for ( isca=0; isca<amodel->nsca; isca++ )
  for ( iband=0; iband<NUM_ALI_BANDS; iband++ )
  {
    amodel->sca_coef_x[isca][iband][0] =
      cpf->Focal_Plane_Parameters.SCA_Parameters[isca].Along_LOS_Legendre[iband][0];
    amodel->sca_coef_x[isca][iband][1] =
      cpf->Focal_Plane_Parameters.SCA_Parameters[isca].Along_LOS_Legendre[iband][1];
    amodel->sca_coef_x[isca][iband][2] =
      cpf->Focal_Plane_Parameters.SCA_Parameters[isca].Along_LOS_Legendre[iband][2];
    amodel->sca_coef_y[isca][iband][0] =
      cpf->Focal_Plane_Parameters.SCA_Parameters[isca].Across_LOS_Legendre[iband][0];
    amodel->sca_coef_y[isca][iband][1] =
      cpf->Focal_Plane_Parameters.SCA_Parameters[isca].Across_LOS_Legendre[iband][1];
    amodel->sca_coef_y[isca][iband][2] =
      cpf->Focal_Plane_Parameters.SCA_Parameters[isca].Across_LOS_Legendre[iband][2];
  }

return(SUCCESS);
}
