/******************************************************************************
FUNCTION:	axx_zero_amodel

PURPOSE:	initialize the ALI model to all 0's

DEVELOPMENT HISTORY:
$Log: axx_zero_amodel.c,v $
Revision 1.2  2001/03/28 22:42:00  storey
Added new L0R software version number fields.

Revision 1.1  2000/11/27 13:29:33  storey
Original version


RETURN VALUE:
No return Value.

ALGORITHM REFERENCES:
  See alimodel.h for description of the amodel structure.
*******************************************************************************/
#include <stdio.h>
#include <sys/types.h>
#include "axx_alimodel.h"

void axx_zero_amodel
(
  ALIMODEL *amodel 	 /* O: amodel to init to 0 */
)
{
long 	i,j;		  /* loopers */


/*----------------------------------------------------------------
 * Initialize the model structure to zeroes
 ----------------------------------------------------------------*/

amodel->satnum = 0;
amodel->l0r_sw_version[0] = 1;
amodel->l0r_sw_version[1] = 1;
amodel->l0r_sw_version[2] = 1;
amodel->dce_time[0] = 0.0;
amodel->dce_time[1] = 0.0;
amodel->dce_time[2] = 0.0;
amodel->ms_frame_rate = 0.0;
amodel->pan_frame_rate = 0.0;
amodel->ms_int_time = 0.0;
amodel->pan_int_time = 0.0;
amodel->acs_time[0] = 0.0;
amodel->acs_time[1] = 0.0;
amodel->acs_time[2] = 0.0;
amodel->nephem = 0;
amodel->ephem_type = ACSPTS;
amodel->ephem_time[0] = 0.0;
amodel->ephem_time[1] = 0.0;
amodel->ephem_time[2] = 0.0;

/*----------------------------------------------------------------
 * Initialize precision and r-p-y values
 ----------------------------------------------------------------*/
amodel->sc_roll = 0.0;
amodel->sc_pitch = 0.0;
amodel->sc_yaw = 0.0;
amodel->correction_type = SYSTEMATIC;
amodel->p_roll = 0.0;
amodel->p_rrate = 0.0;
amodel->p_pitch = 0.0;
amodel->p_prate = 0.0;
amodel->p_yaw = 0.0;
amodel->p_yrate = 0.0;
amodel->p_x = 0.0;
amodel->p_vx = 0.0;
amodel->p_y = 0.0;
amodel->p_vy = 0.0;
amodel->p_z = 0.0;
amodel->p_vz = 0.0;
amodel->p_ref_time = 0.0;

/*----------------------------------------------------------------
 * Initialize arrays of NPCD values
 ----------------------------------------------------------------*/

for (i=0; i<NPCD; i++)
  {
  amodel->ecr_satpos[0][i] = 0.0;
  amodel->ecr_satpos[1][i] = 0.0;
  amodel->ecr_satpos[2][i] = 0.0;
  amodel->ecr_satvel[0][i] = 0.0;
  amodel->ecr_satvel[1][i] = 0.0;
  amodel->ecr_satvel[2][i] = 0.0;
  amodel->eci_satpos[0][i] = 0.0;
  amodel->eci_satpos[1][i] = 0.0;
  amodel->eci_satpos[2][i] = 0.0;
  amodel->eci_satvel[0][i] = 0.0;
  amodel->eci_satvel[1][i] = 0.0;
  amodel->eci_satvel[2][i] = 0.0;
  }

/*----------------------------------------------------------------
 * Initialize arrays of NSCA values
 ----------------------------------------------------------------*/

for (j=0; j<NBAND; j++)
  {
  for (i=0; i<NSCA; i++)
    {
    amodel->sca_coef_x[i][j][0] = 0.0;
    amodel->sca_coef_x[i][j][1] = 0.0;
    amodel->sca_coef_x[i][j][2] = 0.0;
    }
  amodel->sca_num_det[j] = 0;
  }

/*----------------------------------------------------------------
 * Initialize arrays of NPCD values
 ----------------------------------------------------------------*/
for (i=0; i<NPCD; i++)
  {
  amodel->rpy[i][0] = 0.0;
  amodel->rpy[i][1] = 0.0;
  amodel->rpy[i][2] = 0.0;
  }
}
