/* ******************************** */
/* if we haven't used this file yet */
/* ******************************** */
#ifndef _AXX_ALIMODEL_H
#define _AXX_ALIMODEL_H 1


/* ************* */
/* include files */
/* ************* */
#include "xxx_structures.h"
#include "xxx_const.h"
#include "exx_cpf.h"

/* ********* */
/* constants */
/* ********* */
#define NBAND		10	/* number of ALI bands */

/* ********** */
/* structures */
/* ********** */

typedef struct ALIMODEL
    {
    int  satnum;		 	/* Satellite number			    */
    int  l0r_sw_version[3];		/* Level 0R generation software version #   */
    double dce_time[3];		 	/* DCE image start time (year,day,sod)      */
    double ms_frame_rate;        	/* Multispectral band frame rate (hertz)    */
    double pan_frame_rate;       	/* Panchromatic band frame rate (hertz)     */
    double ms_int_time;          	/* Multispectral integration time (msec)    */
    double pan_int_time;         	/* Panchromatic integration time (msec)     */
    double acs_time[3];          	/* ACS reference time (year,day,sod)        */
    int  nephem;		 	/* Number of ephemeris points		    */
    EphemType ephem_type;		/* Ephemeris type:  ACS or GPS              */
    double ephem_time[3];        	/* Ephemeris reference time (year,day,sod)  */
    double ecr_satpos[3][NPCD]; 	/* Satellite position in ECR                */
    double ecr_satvel[3][NPCD];  	/* Satellite velocity in ECR                */
    double eci_satpos[3][NPCD];  	/* Satellite position in ECI                */
    double eci_satvel[3][NPCD];  	/* Satellite velocity in ECI                */
    int  nsca;                   	/* Number of ALI SCAs                       */
    double sca_coef_x[NSCA][NBAND][3];	/* Along-track SCA LOS Legendre coeffs      */
    double sca_coef_y[NSCA][NBAND][3];	/* Across-track SCA LOS Legendre coeffs     */
    int sca_num_det[NBAND];		/* Number of detectors per SCA              */
    int  nrpy;                   	/* Number of attitude points                */
    double rpy[NPCD][NAXIS];     	/* Roll, Pitch, and Yaw correction values   */
    double sc_roll;              	/* Space craft roll at acs_time             */
    double sc_pitch;             	/* Space craft pitch at acs_time            */
    double sc_yaw;               	/* Space craft yaw at acs_time              */
    CorrectionType correction_type; 	/* This flag specifies if the model is a
                                 	   systematic or precision model.	    */
    double p_roll;               	/* Precision roll value                     */
    double p_rrate;              	/* Precision roll rate value                */
    double p_pitch;              	/* Precision pitch value                    */
    double p_prate;              	/* Precision pitch rate value               */
    double p_yaw;                	/* Precision yaw value                      */
    double p_yrate;              	/* Precision yaw rate value                 */
    double p_x;                  	/* Precision along-track position           */
    double p_vx;                 	/* Precision along-track velocity           */
    double p_y;                  	/* Precision across-track position          */
    double p_vy;                 	/* Precision across-track velocity          */
    double p_z;                  	/* Precision altitude position              */
    double p_vz;                 	/* Precision altitude velocity              */
    double p_ref_time;           	/* Precision reference time                 */
    }ALIMODEL;

/********************************************/
/* Prototypes for the ALI model I/O routines. */
/********************************************/
int  axx_get_alimodel (
  const char alimodel_filename[STRLEN],	/* I: HDF input file name             */
  ALIMODEL *alimodel);          	/* O: ALI model structure to be read  */

int  axx_put_alimodel (
  const char alimodel_filename[STRLEN],	/* I: HDF output file name              */
  const ALIMODEL *alimodel);    	/* I: ALI model structure to be output  */

/*********************************************/
/* include files (more at beginning of file) */
/*********************************************/
#include "axx_init_model.h"
#include "exx_init_model.h"
#include "axx_forward_model.h"

#endif
