/******************************************************************************
  ALI forward model include file
   this file includes:
     function prototypes for the axxALIModel library (forward model functions)
*******************************************************************************/
#ifndef _AXX_FORWARD_MODEL_H_
#define _AXX_FORWARD_MODEL_H_

#include <stdio.h>
#include <math.h>
#include <values.h>
#include "exx_cpf.h"
#include "xxx_structures.h"

#define ABER_CORRECTION -25.2267e-6     /* velocity aberration correction for
					   across_angle calculation     */
#define COS35 		0.819152044	/* adjustment for 
					   across_angle calculation     */

/* ALI FORWARD MODEL */
int axx_attitude(const VECTOR *satpos, const VECTOR *satvel, 
	const VECTOR *sensor_los, double roll, double pitch, 
	double yaw, const CPF *cpf, VECTOR *plos, VECTOR *newlos);

int axx_calc_position( double line, double sample, BandNumber band,
	DetType dettype, const CPF *cpf, ALIMODEL *alimodel, 
        VECTOR *satpos, VECTOR *satvel, double *Time, int *year, int *day);

int axx_findlos(BandNumber band, int sca, double sample, DetType dettype,
	ALIMODEL *amodel, const CPF *cpf, VECTOR *losv );

int axx_findatt(ALIMODEL *alimodel, double time, 
        double *roll, double *pitch, double *yaw);

int axx_findtarpos(const VECTOR *satvec, const VECTOR *losv, double tarelev, 
	const CPF *cpf, double *tarlat, double *tarlong, double *tarrad);

int axx_findtime(double line, double sample, BandNumber band, ALIMODEL *amodel, 
        const CPF *cpf, DetType dettype, double *Time);

int axx_forward_ali(double line, double sample, BandNumber band, int sca,
        double tarelev, const CPF *cpf, ALIMODEL *alimodel, DetType dettype,
	double *tarlatd, double *tarlong);

int axx_getlos(double line, double sample, BandNumber band, int sca,
	double tarelev, const CPF *cpf, ALIMODEL *alimodel, DetType dettype,
        double *tarlatc, double *tarlong, double *tarrc, 
        VECTOR *satpos, VECTOR *pertlos);

#endif
