/******************************************************************************
  GPS include file
     this file includes:
       HDF include file
       GPS HDF IO constants and structures
       function prototypes for the IAS HDF IO library
*******************************************************************************/
#ifndef _AXX_HDF_IO_H_
#define _AXX_HDF_IO_H_

#include <string.h>
#include "mfhdf.h"
#include "exx_dem.h"
#include "xxx_const.h"
#include "xxx_structures.h"

#define IMAGE_CLASS "Image"

#define SDS_PAN		"PAN"
#define SDS_MS1P	"B1P"
#define SDS_MS1		"B10"
#define SDS_MS2		"B20"
#define SDS_MS3		"B30"
#define SDS_MS4		"B40"
#define SDS_MS4P	"B4P"
#define SDS_MS5P	"B5P"
#define SDS_MS5		"B50"
#define SDS_MS7		"B70"

#define OUTSIDE 99                /* code for a point outside the image   */
#define LINES_TO_BUFFER 2
/* defines the number of lines to buffer at a time for exx_gpyramid. */

#ifndef _XXX_HDF_IO_H_
typedef struct HDFIO {
    int32 file_id;              /* HDF file id                  */
    int32 group_id;             /* HDF group id                 */
    int32 sd_id;                /* HDF SDS interface id         */
    int32 sds_id[NSCA*NBANDS];       /* HDF SDS id per band          */
    char  sds_name[NSCA*NBANDS][STRLEN];/* HDF SDS name per band        */
    char  hdf_dir_name[STRLEN]; /* name of HDF directory file   */
    char  extdir[STRLEN];       /* external file directory      */
    } HDFIO;
#endif

/* General */
BandNumber axx_parse_user_band(int user_band_number);
int axx_get_user_band(BandNumber band_number);

/* 1G I/O */
HDFIO *axx_open_1g_image(const char *image_filename, AccessMode access_mode, 
	int *nbands, int nsca, BandNumber bandno[NBANDS], DataType dtype[NBANDS], 
	int nl[NSCA][NBANDS], int ns[NSCA][NBANDS]);
int axx_close_1g_image(HDFIO *hdfid);
int axx_read_1g_image (const HDFIO *hdfid, int nband, int sca, BandNumber band, int sl, 
	int ss, int nl, int ns, int subsamp, void *data);
int axx_write_1g_image(const HDFIO *hdfid, int sca, int nband, BandNumber band, int sl, 
	int ss, int nl, int ns, int subsamp, void *data);
int32 axx_open_1g_band(HDFIO *hdfid, int sca_number, int nband, BandNumber bandno);
int axx_create_1g_band(HDFIO *hdfid, int sca_number, int nband, BandNumber bandno, DataType dtype, 
	int nl, int ns);
void axx_create_band_name(const char *HDF_name, const char *type, 
	int sca_number, BandNumber bandno, char *band_name);

int axx_extract_chip(const char *chip_name, const int chip_size[2], void *chip_buf);

int exx_get_dem
(
    char *DEM_name,              /* INPUT: name of DEM to read */
    struct CORNERS *image_corners_ptr, /* INPUT: corners of image */
    int lines_in_image,          /* INPUT: lines in image */
    int samples_per_image_line,  /* INPUT: samples in an image line */
    double image_pixsize,        /* INPUT: input image pixel size */
    DEM_DATA *dem_ptr            /* OUTPUT: data read from the DEM file */
);

/* IMAGE PROCESSING FUNCTIONS */
int axx_extract_window(
    int  image_nl,          /* I-# of lines in the 1G image         */
    int  image_ns,          /* I-# of samples in the 1G image       */
    int  nband,
    int  sca,
    BandNumber band,        /* I-band number to read from           */
    DataType dtype,         /* I-data type of the band              */
    double pred_line,       /* I-center line of window to extract   */
    double pred_samp,       /* I-center sample of window to extract */
    int  win_size[2],       /* I-size of window to extract          */
    HDFIO *img_ptr,         /* I-image pointer                      */
    float *img_buf);        /* O-Buffer read from 1G image          */

#endif
