/* ******************************** */
/* if we haven't used this file yet */
/* ******************************** */
#ifndef _AXX_INIT_MODEL_H
#define _AXX_INIT_MODEL_H 1


/* ******************* */
/* other include files */
/* ******************* */
#include "xxx_ALIPCD.h"
#include "xxx_math.h"
#include "xxx_structures.h"
#include "exx_cpf.h"
#include "xxx_EDCOdl.h"
#include "xxx_ODL.h"
#include "lablib3.h"

/***********************/
/* attitude prototypes */
/***********************/
int axx_align_attitude(
    ALIPCD *alipcd,
    ALIMODEL *alimodel,
    CPF *cpf
    );

/* ************************************** */
/* the ephemeris prototypes for functions */
/* ************************************** */
int axx_convert_ephem(
    struct EPHEM ephem[],/* IO:  Ephemeris information			*/
    double  xp,		/* I: True pole postion in the mean pole coords system,
		           x-axis                            			*/
    double yp, 		/* I: True pole position in the mean pole coords
                           system, y-axis                            		*/
    double ut1_utc, 	/* I: Earth rotation angle difference ut1 - utc 	*/
    ALIMODEL *alimodel  /* IO: ali model					*/
);

/* ************************************* */
/* the forward model prototype functions */
/* ************************************* */
int axx_process_ali_model(
    struct EPHEM *ephem,
    double xp,
    double yp,
    double ut1_utc,
    CPF *cpf,
    ALIPCD *pcd,
    ALIMODEL *alimodel);

int axx_out_of_pcd(
    ALIPCD *pcd,
    ALIMODEL *alimodel	
    );

void axx_zero_amodel(
    ALIMODEL *amodel);

int axx_update_amodel
(
   char *amodel_name,
   char *prec_amodel_name,
   char *hk_dir,	/* I: Housekeeping data directory      */
   char *hk_root,	/* I: Housekeeping data file root name */
   char *cpf_name,
   double *reftime,
   VECTOR satpos,
   VECTOR satvel,
   VECTOR rpyangle,
   VECTOR rpyrate,
   double *refatt
);

int axx_run_alimodel
(
   char *hk_dir,	/* I: Housekeeping data directory      */
   char *hk_root,	/* I: Housekeeping data file root name */
   char *cpf_name,	/* I: CPF file name                    */
   ALIMODEL *alimodel,  /* I/O:  ALI model structure           */
   CPF **cpf		/* O: Loaded CPF structure             */
);

int axx_los_coeffs
(
   ALIMODEL *alimodel,	/* I/O: ALI model structure            */
   const CPF *cpf	/* I: CPF structure                    */
);

#endif
