/******************************************************************************
NAME: axx_close_l1r_sca_band

PURPOSE: Closes a Level 1R ALI SCA file that was opened with 
         axx_open_l1r_sca_band and frees any memory allocated to the tracking
         structure.

RETURNS:
    SUCCESS when the close succeeds
    ERROR when the close fails
*******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include "xxx_Types.h"
#include "xxx_LogStatus.h"
#include "xxx_ali_level1r.h"
#include "axx_l1r_image.h"

int axx_close_l1r_sca_band
(
    L1R_SCA_BAND* image     /* I: image to close */
)
{
    int error = 0;

    if (SDendaccess(image->sds_id) == FAIL)
        error = 1;

    if (SDend(image->sd_id) == FAIL)
        error = 1;

    if (error)
    {
        char msg[STRLEN];
        sprintf(msg,"Error closing L1R image %s", image->filename);
        xxx_LogStatus("READ_L1R_IMAGE",__FILE__,__LINE__,msg);
        free(image);
        return ERROR;
    }

    free(image);

    return SUCCESS;
}
