/******************************************************************************
NAME:           axx_create_band_name

PURPOSE:
axx_create_band_name forms a band file name or band SDS name given the 
HDF file name.
The "_HDF" at the end of the root HDF file name is replaced with the appropriate
band tag (e.g., "_B10").  If no "_HDF" is at the end of the file name, the band
tag is appended to the end of the root HDF file name.

******************************************************************************/
#include <stdlib.h>        /* exit prototype */
#include <stdio.h>         /* sprintf prototype */
#include "xxx_Types.h"     /* SUCCESS/ERROR definitions */
#include "xxx_LogStatus.h" /* xxx_LogStatus prototype */
#include "xxx_const.h"     /* ALI constants */
#include "axx_hdf_io.h"	 /* HDF i/o function prototypes & SDS band tags */
#include "xxx_utility.h" /* createname and splitname prototypes */

void axx_create_band_name (
    const char *HDF_name,/* I: HDF name to form band file name from 	*/
    const char *type,	/* I: SDS or FILE name ("SDS" or "FILE")	*/
    int sca_number,
    BandNumber bandno,	/* I: enumerated band number			*/
    char *band_name)  	/* O: constructed band file name 		*/
{
char directory[STRLEN]; /* directory name */
char root[STRLEN];      /* root file name */
char extension[STRLEN]; /* file extension */
char *temp_char_ptr;    /* temporary character pointer */
static char band_tag[NBANDS][4] = {SDS_PAN, SDS_MS1P, SDS_MS1, SDS_MS2, 
    SDS_MS3, SDS_MS4, SDS_MS4P, SDS_MS5P, SDS_MS5, SDS_MS7};
char sca_ext[8];
    
/* form the band file file name from the HDF name */
xxx_splitname(HDF_name, directory, root, extension);

/* find the "_HDF" part of the file name if it exits */
temp_char_ptr = strrchr(root,'_');
if (temp_char_ptr == NULL)
    {
    /* no "_HDF" so just append the appropriate band tag */
    /*  for SDS  name it is root.tag */
    /*  for file name it is root_tag.ext */
    if (strcmp(type,"SDS") == 0)
        {
        sprintf(sca_ext,"_SCA%d_",sca_number);
        strcat(root, sca_ext);
        strcpy(extension, band_tag[(int)bandno]);
        }
    else
	{
        strcat(root, "_");
        sprintf(sca_ext,"SCA%d_",sca_number);
        strcat(root, sca_ext);
        strcat(root, band_tag[(int)bandno]);
	}
    }
else
    {
    if (strcmp(temp_char_ptr, "_HDF") == 0)
        {
        /* replace the "_HDF" with the band substring */
        /*  for SDS  name it is root.tag so set a NULL where the _HDF is 
	    and set the extension to the tag */
        /*  for file name it is root_tag.ext so replace the HDF with the tag */
        if (strcmp(type,"SDS") == 0)
	    {
            *temp_char_ptr = '\0';
            sprintf(sca_ext,"SCA%d_",sca_number);
            strcpy(extension, sca_ext);
            strcat(extension, band_tag[(int)bandno]);
	    }
        else
            {
            sprintf(sca_ext,"SCA%d_",sca_number);
            strcpy(temp_char_ptr+1, sca_ext);
            strcpy(temp_char_ptr+6, band_tag[(int)bandno]);
            }
        }
    else
        {
        /* doesn't end with "_HDF" so just append the band tag */
        /*  for SDS  name it is root.tag */
        /*  for file name it is root_tag.ext */
        if (strcmp(type,"SDS") == 0)
            {
            sprintf(sca_ext,"_SCA%d_",sca_number);
            strcpy(extension, sca_ext);
            strcat(extension, band_tag[(int)bandno]);
            }
        else
	    {
            sprintf(sca_ext,"_SCA%d_",sca_number);
            strcat(root, sca_ext);
            strcat(root, band_tag[(int)bandno]);
	    }
        }
    }

/* create band file name */
xxx_createname(directory, root, extension, band_name);

return;
}
