/******************************************************************************
NAME:		axx_open_1g_band

PURPOSE:	Open a band of the 1G image.

RETURN VALUE: 
Type = int32
Value           Description
-----           -----------
band id		HDF SDS id used to access the band
FAIL     	HDF failure code--Band not opened (not found)

NOTES:		THIS ROUTINE SHOULD ONLY BE CALLED BY OTHER GPS 1G I/O ROUTINES
		
		The 1G image is a VGroup containing a SDS for each band.  The 
		data for each SDS are stored in an external element.

		The band number specified in bandno is assumed to be in
		the range b1..b8.

		The HDFIO structure will fill in the sds_name field for the 
		requested band [bandno] and return the sds_id.  

		***This routine assumes the band SDS is already created.
		The band is simply selected (opened) so that other routines
		may access it.

		***This routine does not issue any error messages, because
		a non-present band may not necessarily be an error (ie
		if the caller is checking to see which of the bands are
		present).

******************************************************************************/
#include <stdio.h>
#include <unistd.h>
#include "axx_hdf_io.h"
#include "xxx_Types.h"
#include "xxx_LogStatus.h"

int32 axx_open_1g_band (
	HDFIO *hdfid,		/* I-HDF info structure for the image	*/
	int sca_number,
        int nband,
	BandNumber bandno)	/* I-enumerated band number		*/

{
int32	sds_index,		/* HDF SDS index number		*/
	sds_id;			/* HDF SDS id			*/
char	msg[STRLEN];		/* error message		*/
int	max_bands = NBANDS;

/*
 * Initialize the sds id
 */
sds_id = -1;

/*
 * Construct the name of the SDS for this band
 */
axx_create_band_name(hdfid->hdf_dir_name, "SDS", sca_number, bandno, 
                     hdfid->sds_name[sca_number*max_bands+(int)bandno]);

/*
 * Select the user-specified band
 */
sds_index = SDnametoindex(hdfid->sd_id, hdfid->sds_name[sca_number*max_bands+(int)bandno]);
if (sds_index == FAIL)
    {
    /* xxx_LogStatus("AXXLIB", __FILE__, __LINE__, "Band not found"); */
    return(FAIL); /* band not found */
    }

sds_id = SDselect(hdfid->sd_id, sds_index);
if (sds_id == FAIL)
    {
    sprintf("Band %s not found",hdfid->sds_name[sca_number*max_bands+(int)bandno]);
    xxx_LogStatus("AXXLIB", __FILE__, __LINE__, msg);
    return(FAIL);
    }

return(sds_id);
}
