#include <stdio.h>

#include "xxx_Types.h"
#include "xxx_LogStatus.h"
#include "xxx_ali_level1r.h"
#include "xxx_ali_level0r.h"
#include "axx_rps.h"


int axx_calibrate(
LEVEL0R  *level0r,       /* I: Structure containing level 0R data */
CAL_INFO *cal_info,      /* I: Structure containing gain & bias information */
int32    band,           /* I: 0-referenced index to band number */
int16    skip_length,    /* I: Number of frames to skip to get to next data
			    in sequence */
int32    scale,          /* I: Scale Factor to apply to Level 1R data */
LEVEL1R  *level1r)       /* O: Structure to contain level 1R data */
{
  char       msg[STRLEN]; /* Message buffer */
  float64    step1;       /* Temporary computation space */
  int32	     det;         /* Current detector */
  int32	     frame;       /* Current frame */
  int32	     l0r_index;   /* Index into level0r/level1r image data */
  int32      xx;          /* Index into gain/offset data */
  int32      num_dets;    /* Total number of detectors */
  int32      num_frames;  /* Total number of frames */
  int16      skip;        /* Current counter for data skipping */
 
  /* Generate the Level1R  (recalibrated) image
   -----------------------------------------------*/
  num_dets = cal_info->ndets;
  num_frames = level0r->image[band].dim_sizes[N_FRAMES_IDX];

  /* Insert the scale_factor into the level 1R structure
   --------------------------------------------------------*/
  level1r->response[band].attributes.scale_factor = scale;

  for(skip = 0; skip < skip_length; skip++)
    {
      for(det = 0; det < num_dets; det++)
	{
	  xx = det + skip*num_dets;
	  for(frame = skip; frame < num_frames; frame += skip_length)
	    {
	      l0r_index = det + frame*num_dets;

	      /* Right now, only fix the non-fill data.
	       -----------------------------------------------*/
	      if((level0r->image[band].data[l0r_index] & FILL_DATA) == 0)
		{
		  /* Remove the (dark) bias level from the image data. */
		  step1 = (float64)(level0r->image[band].data[l0r_index]) -
		    cal_info->dark.mean[xx];

		  step1 *= scale*cal_info->gain.p1[det];

		  level1r->image[band].data[l0r_index] = (int16)(step1+0.5);
		}
	      else
		/* change the fill data to zeros
		   ------------------------------------------*/
		level1r->image[band].data[l0r_index] = 0;



	      /* Create the Level1R response image and offset images.
	       -----------------------------------------------------------*/
	      level1r->response[band].data[l0r_index] = cal_info->gain.p1[det];
	      level1r->offset[band].data[l0r_index] = cal_info->dark.mean[xx];
	      
	    }
	}
    }
  

  return SUCCESS ;

}
