/*  axx_find_minmax():         routine to determine minimum and maximum
                               DN values for each detector */
#include <stdio.h>

#include "xxx_Types.h"
#include "xxx_LogStatus.h"
#include "axx_rps.h"

int axx_find_minmax(
int32 det,             /* I: Current detector */
int32 num_dets,        /* I: Number of detectors */
int32 num_frames,      /* I: Number of frames */
int16 *pData,          /* I: Pointer to image data array */
int16 *pMax,           /* O: Maximum DN value */
int16 *pMin)           /* O: Minimum DN value */
{
  int32   frame;         /* Current frame */
  int32   loc;           /* Current pixel location in image data array */
  char    msg[STRLEN];   /* Message string */

  /* Do some basic sanity checking of pointer parameters first, just to
     make sure... */
  if((pData == NULL) || (pMax == NULL) || (pMin == NULL))
    {
      sprintf(msg, "Invalid parameter passed in, quitting axx_find_minmax().");
      xxx_LogStatus("rps", __FILE__, __LINE__, msg);
      return ERROR;
    }

  /* Find the minimum and maximum for each detector. */
  *pMax = 0;
  *pMin = 10000;
  
  for(frame = 0; frame < num_frames; frame++)
    {
      loc = det + frame*num_dets;
      if((pData[loc] & FILL_DATA) == 0)
	{
	  if(pData[loc] > *pMax)
	    *pMax = pData[loc];
	  else if(pData[loc] < *pMin)
	    *pMin = pData[loc];
	}
    }
 

  return SUCCESS;
}

