/* axx_free_cal_info():    routine to free up allocations for each SCA's
                           CAL_INFO structure.  Sets the pointers to NULL
			   after each free() to force a crash if they're
			   inadvertantly used later on before they're
			   reallocated (good for diagnostic purposes).  */

#include <stdio.h>
#include <stdlib.h>

#include "xxx_Types.h"
#include "axx_rps.h"

int axx_free_cal_info(CAL_INFO *cal_info)  /* I/O:  Array member to deallocate */
{
  int32 ii;                     /* loop counter */
  int32 num_elements;           /* number of detectors */

  num_elements = cal_info->ndets;

  /* Free up the histogram buffers */

  free(cal_info->hist.maxDN);
  cal_info->hist.maxDN = NULL;
  free(cal_info->hist.minDN);
  cal_info->hist.minDN = NULL;
  free(cal_info->hist.valid_frames);
  cal_info->hist.valid_frames = NULL;
  free(cal_info->hist.meanDN);
  cal_info->hist.meanDN = NULL;
  free(cal_info->hist.stddev);
  cal_info->hist.stddev = NULL;
  


  /* Free up the lamp state buffers */

  free(cal_info->ic_lamp.det_mean);
  cal_info->ic_lamp.det_mean = NULL;
  free(cal_info->ic_lamp.det_stddev);
  cal_info->ic_lamp.det_stddev = NULL;
  free(cal_info->ic_lamp.valid_frames);
  cal_info->ic_lamp.valid_frames = NULL;
  free(cal_info->ic_lamp.band_average);
  cal_info->ic_lamp.band_average = NULL;



  /* Free up the gain buffers */

  free(cal_info->gain.p1);
  cal_info->gain.p1 = NULL;
  free(cal_info->gain.p1_stderr);
  cal_info->gain.p1_stderr = NULL;
  free(cal_info->gain.p2);
  cal_info->gain.p2 = NULL;
  free(cal_info->gain.p2_stderr);
  cal_info->gain.p2_stderr = NULL;
  free(cal_info->gain.cal_method);
  cal_info->gain.cal_method = NULL;



  /* Free up the offset buffers */

  free(cal_info->precal.mean);
  cal_info->precal.mean = NULL;
  free(cal_info->precal.stddev);
  cal_info->precal.stddev = NULL;
  free(cal_info->precal.valid_frames);
  cal_info->precal.valid_frames = NULL;
  free(cal_info->dark.mean);
  cal_info->dark.mean = NULL;
  free(cal_info->dark.stddev);
  cal_info->dark.stddev = NULL;
  free(cal_info->dark.valid_frames);
  cal_info->dark.valid_frames = NULL;

  return SUCCESS;
}
