#include "xxx_Types.h"
#include "xxx_ali_level1r.h"
#include "xxx_ali_level0r.h"
#include "axx_rps.h"

int axx_generate_1r(
LEVEL0R level0r,                /* I: Structure containing level 0R data */
LEVEL1R *level1r)               /* O: Structure to contian level 1R data */
{

     int32	band;           /* Current band index */

  /* Copy Level 0R header to Level 1R header
   * and populate header info that doesn't exist in Level 0R
   */
  axx_create_l1r_header(level0r.header, &level1r->header);

  /****************************************/
  /* Generate level1r image               */

  /* Copy all L0R_IMAGE to L1R_IMAGE for MS bands */
  memcpy(&level1r->image[0], &level0r.image[0], sizeof(L0R_IMAGE));

  /* Allocate enough memory for MS bands  */
  level1r->image[0].data=(int16 *)malloc(
                            sizeof(int16)
                             *level0r.image[0].dim_sizes[N_BANDS_IDX]
                             *level0r.image[0].dim_sizes[N_DETS_IDX]
                             *level0r.image[0].dim_sizes[N_FRAMES_IDX]);

   /* Set name, dataset_type and ali_sensor */
   sprintf(level1r->image[0].name, "Level 1R");
   sprintf(level1r->image[0].attributes.dataset_type, "Level 1");
   memcpy(level1r->image[0].attributes.ali_sensor,
          &level0r.image[0].attributes.ali_sensor,
          sizeof(level1r->image[0].attributes.ali_sensor));

   /* Populate other band structures.
   --------------------------------*/
   for( band=1; band<PAN_BAND; band++ )
   {
      memcpy( &level1r->image[band], &level1r->image[0], sizeof(L1R_IMAGE) );
      level1r->image[band].data += band * level1r->image[0].dim_sizes[N_DETS_IDX]
                                        * level1r->image[0].dim_sizes[N_FRAMES_IDX];
   }

   /* Copy all L0R_IMAGE to L1R_IMAGE for pan band */
   memcpy(&level1r->image[PAN_BAND], &level0r.image[PAN_BAND], sizeof(level0r.image[PAN_BAND]));

  /* Allocate enough memory for pan band */
  level1r->image[PAN_BAND].data=(int16 *)malloc(
                            sizeof(int16)
                            *level0r.image[PAN_BAND].dim_sizes[N_DETS_IDX]
                            *level0r.image[PAN_BAND].dim_sizes[N_FRAMES_IDX]);

   /* Set name, dataset_type and ali_sensor */
   sprintf(level1r->image[PAN_BAND].name, "Level 1R");
   sprintf(level1r->image[PAN_BAND].attributes.dataset_type, "Level 1");
   memcpy(level1r->image[PAN_BAND].attributes.ali_sensor,
          &level0r.image[PAN_BAND].attributes.ali_sensor,
          sizeof(level1r->image[PAN_BAND].attributes.ali_sensor));

  /****************************************/
  /* Generate level1r response            */
  /* Allocate enough memory for MS bands  */
  level1r->response[0].data=(float32 *)malloc(
                                         sizeof(float32)
                                         *level0r.image[0].dim_sizes[N_BANDS_IDX]
                                         *level0r.image[0].dim_sizes[N_DETS_IDX]
                                         *level0r.image[0].dim_sizes[N_FRAMES_IDX]);

  /* Create level1r's MS response attributes and "header"
   * ----------------------------------------------------
   */
   sprintf(level1r->response[0].name, "Response");
   level1r->response[0].number_of_attributes = 6;
   sprintf(level1r->response[0].attributes.dataset_type, "Response 1");
   level1r->response[0].attributes.scale_factor = 4000;
   level1r->response[0].HDF_type = DFNT_FLOAT32;

   level1r->response[0].number_of_dims = level0r.image[0].number_of_dims,
   memcpy(&level1r->response[0].dim_sizes,
          &level0r.image[0].dim_sizes,
	  sizeof(level1r->response[0].dim_sizes));

   sprintf(level1r->response[0].attributes.ali_sensor, "%s",
                                  level0r.image[0].attributes.ali_sensor);
   sprintf(level1r->response[0].attributes.data_gate_open_time, "%s",
                                  level0r.image[0].attributes.data_gate_open_time);
   sprintf(level1r->response[0].attributes.data_gate_close_time, "%s",
                                  level0r.image[0].attributes.data_gate_close_time);
   level1r->response[0].attributes.number_cross_track_pixels
                                   = level0r.image[0].attributes.number_cross_track_pixels;
   level1r->response[0].attributes.number_along_track_pixels
                                   = level0r.image[0].attributes.number_along_track_pixels;
   level1r->response[0].attributes.number_of_bands
                                   = level0r.image[0].attributes.number_of_bands;
   memcpy(&level1r->response[0].attributes.valid_map,
          level0r.image[0].attributes.valid_map,
	  sizeof(level1r->response[0].attributes.valid_map));

   /* Populate other band structures.
   --------------------------------*/
   for( band=1; band<PAN_BAND; band++ )
   {
      memcpy( &level1r->response[band], &level1r->response[0], sizeof(L1R_CAL) );
      level1r->response[band].data += band * level1r->response[0].dim_sizes[N_DETS_IDX]
                                        * level1r->response[0].dim_sizes[N_FRAMES_IDX];
   }

  /* Allocate enough memory for pan band */
  level1r->response[PAN_BAND].data=(float32 *)malloc(
                                         sizeof(float32)
                                         *level0r.image[PAN_BAND].dim_sizes[N_DETS_IDX]
                                         *level0r.image[PAN_BAND].dim_sizes[N_FRAMES_IDX]);

  /* Create level1r's pan response attributes and "header"
   * -----------------------------------------------
   */
   sprintf(level1r->response[PAN_BAND].name, "Response");
   level1r->response[PAN_BAND].number_of_attributes = 6;
   sprintf(level1r->response[PAN_BAND].attributes.dataset_type, "Response 1");
   level1r->response[PAN_BAND].attributes.scale_factor = 4000;
   level1r->response[PAN_BAND].HDF_type = DFNT_FLOAT32;

   level1r->response[PAN_BAND].number_of_dims = level0r.image[PAN_BAND].number_of_dims,
   memcpy(&level1r->response[PAN_BAND].dim_sizes,
          &level0r.image[PAN_BAND].dim_sizes,
	  sizeof(level1r->response[PAN_BAND].dim_sizes));

   sprintf(level1r->response[PAN_BAND].attributes.ali_sensor, "%s",
                                  level0r.image[PAN_BAND].attributes.ali_sensor);
   sprintf(level1r->response[PAN_BAND].attributes.data_gate_open_time, "%s",
                                  level0r.image[PAN_BAND].attributes.data_gate_open_time);
   sprintf(level1r->response[PAN_BAND].attributes.data_gate_close_time, "%s",
                                  level0r.image[PAN_BAND].attributes.data_gate_close_time);
   level1r->response[PAN_BAND].attributes.number_cross_track_pixels
                                   = level0r.image[PAN_BAND].attributes.number_cross_track_pixels;
   level1r->response[PAN_BAND].attributes.number_along_track_pixels
                                   = level0r.image[PAN_BAND].attributes.number_along_track_pixels;
   level1r->response[PAN_BAND].attributes.number_of_bands
                                   = level0r.image[PAN_BAND].attributes.number_of_bands;
   memcpy(&level1r->response[PAN_BAND].attributes.valid_map,
          level0r.image[PAN_BAND].attributes.valid_map,
	  sizeof(level1r->response[PAN_BAND].attributes.valid_map));

  /***************************************/
  /* Generate level1r offset             */
  /* Allocate enough memory for MS bands */
  level1r->offset[0].data=(float32 *)malloc(
                                         sizeof(float32)
                                         *level0r.image[0].dim_sizes[N_BANDS_IDX]
                                         *level0r.image[0].dim_sizes[N_DETS_IDX]
                                         *level0r.image[0].dim_sizes[N_FRAMES_IDX]);

  /* Create level1r's MS offset attributes and "header"
   * -----------------------------------------------
   */
   sprintf(level1r->offset[0].name, "Offset");
   level1r->offset[0].number_of_attributes = 6;
   sprintf(level1r->offset[0].attributes.dataset_type, "Offset  ");
   level1r->offset[0].attributes.scale_factor = 4000;
   level1r->offset[0].HDF_type = DFNT_FLOAT32;

   level1r->offset[0].number_of_dims = level0r.image[0].number_of_dims,
   memcpy(&level1r->offset[0].dim_sizes,
          &level0r.image[0].dim_sizes,
	  sizeof(level1r->offset[0].dim_sizes));

   sprintf(level1r->offset[0].attributes.ali_sensor, "%s",
                                  level0r.image[0].attributes.ali_sensor);
   sprintf(level1r->offset[0].attributes.data_gate_open_time, "%s",
                                  level0r.image[0].attributes.data_gate_open_time);
   sprintf(level1r->offset[0].attributes.data_gate_close_time, "%s",
                                  level0r.image[0].attributes.data_gate_close_time);
   level1r->offset[0].attributes.number_cross_track_pixels
                                   = level0r.image[0].attributes.number_cross_track_pixels;
   level1r->offset[0].attributes.number_along_track_pixels
                                   = level0r.image[0].attributes.number_along_track_pixels;
   level1r->offset[0].attributes.number_of_bands
                                   = level0r.image[0].attributes.number_of_bands;
   memcpy(&level1r->offset[0].attributes.valid_map,
          level0r.image[0].attributes.valid_map,
	  sizeof(level1r->offset[0].attributes.valid_map));

   /* Populate other band structures.
   --------------------------------*/
   for( band=1; band<PAN_BAND; band++ )
   {
      memcpy( &level1r->offset[band], &level1r->offset[0], sizeof(L1R_CAL) );
      level1r->offset[band].data += band * level1r->offset[0].dim_sizes[N_DETS_IDX]
                                        * level1r->offset[0].dim_sizes[N_FRAMES_IDX];
   }

  /* Allocate enough memory for pan band */
  level1r->offset[PAN_BAND].data=(float32 *)malloc(
                                         sizeof(float32)
                                         *level0r.image[PAN_BAND].dim_sizes[N_DETS_IDX]
                                         *level0r.image[PAN_BAND].dim_sizes[N_FRAMES_IDX]);

  /* Create level1r's pan offset attributes and "header"
   * -----------------------------------------------
   */
   sprintf(level1r->offset[PAN_BAND].name, "Offset");
   level1r->offset[PAN_BAND].number_of_attributes = 6;
   sprintf(level1r->offset[PAN_BAND].attributes.dataset_type, "Offset  ");
   level1r->offset[PAN_BAND].attributes.scale_factor = 4000;
   level1r->offset[PAN_BAND].HDF_type = DFNT_FLOAT32;

   level1r->offset[PAN_BAND].number_of_dims = level0r.image[PAN_BAND].number_of_dims,
   memcpy(&level1r->offset[PAN_BAND].dim_sizes,
          &level0r.image[PAN_BAND].dim_sizes,
	  sizeof(level1r->offset[PAN_BAND].dim_sizes));

   sprintf(level1r->offset[PAN_BAND].attributes.ali_sensor, "%s",
                                  level0r.image[PAN_BAND].attributes.ali_sensor);
   sprintf(level1r->offset[PAN_BAND].attributes.data_gate_open_time, "%s",
                                  level0r.image[PAN_BAND].attributes.data_gate_open_time);
   sprintf(level1r->offset[PAN_BAND].attributes.data_gate_close_time, "%s",
                                  level0r.image[PAN_BAND].attributes.data_gate_close_time);
   level1r->offset[PAN_BAND].attributes.number_cross_track_pixels
                                   = level0r.image[PAN_BAND].attributes.number_cross_track_pixels;
   level1r->offset[PAN_BAND].attributes.number_along_track_pixels
                                   = level0r.image[PAN_BAND].attributes.number_along_track_pixels;
   level1r->offset[PAN_BAND].attributes.number_of_bands
                                   = level0r.image[PAN_BAND].attributes.number_of_bands;
   memcpy(&level1r->offset[PAN_BAND].attributes.valid_map,
          level0r.image[PAN_BAND].attributes.valid_map,
	  sizeof(level1r->offset[PAN_BAND].attributes.valid_map));

  return(SUCCESS);

}
