/*
 * axx_vert_moment(p_array, xdim, ydim, p_mean, p_stddev, p_num_valid_points)
 * 
 *      This function finds the mean and standard deviation of a two
 *      diminsional array (p_array) in the vertical direction.
 */
#include <stdio.h>
#include <stdlib.h>

#include "hdf.h"
#include "mfhdf.h"
#include "xxx_Types.h"

int axx_vert_moment_pan(
int16 *p_array,             /* I: 2D array to determine detector mean & stddev */
int32 xdim,                 /* I: number of detectors */
int32 ydim,                 /* I: number of frames */
float64 *p_mean,            /* O: 2D array of means */
float64 *p_stddev,          /* O: 2D array of standard deviations */
int32 *p_num_valid_points)  /* O: 2D array of number of valid points in mean & stddev */
{
     int32 skip;
     int32 yy;
     int32 xx;
     int32 loc;

for(skip=0;skip<3;skip++)
{
for(yy=skip;yy<ydim;yy+=3)
 {
   for(xx=0;xx<xdim;xx++)
    {
      loc = xx+yy*xdim;
      /* Check for fill data
       * with info in upper 4 bits
       */
      if((p_array[loc] & 0xf000) == 0)
       {
         p_mean[xx+skip*xdim] += p_array[loc];                  /* sums for now */
         p_stddev[xx+skip*xdim] += p_array[loc]*p_array[loc];   /* sum of squares for now */
         p_num_valid_points[xx+skip*xdim] ++;
       }
    }
 }
for(xx=0;xx<xdim;xx++)
 {
   p_mean[xx+skip*xdim] /= p_num_valid_points[xx+skip*xdim];                 /* now mean is mean */
   p_stddev[xx+skip*xdim] /= p_num_valid_points[xx+skip*xdim];               /* now stddev is mean of squares */
   p_stddev[xx+skip*xdim] -= p_mean[xx]*p_mean[xx+skip*xdim];                       /* now stddev is std */
 }
}
    return(SUCCESS);
}
