/******************************************************************************
FUNCTION:	exx_check_date

PURPOSE:
See if the date of the data is in the valid times of the CPF

DEVELOPMENT HISTORY:
$Log: exx_check_date.c,v $
Revision 1.1  2000/11/22 21:52:44  storey
Original version - derived from gxx_check_date.c



RETURN VALUE:
Type = int
Value    Description
-----    -----------
ERROR    Error processing the model data
SUCCESS  Model data was updated  

NOTES:

ALGORITHM REFERENCES:
*******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "xxx_Types.h"
#include "xxx_LogStatus.h"
#include "xxx_math.h"
#include "exx_init_model.h"
#include "axx_alimodel.h"

int exx_check_date
(
  double *time, 	/* I: YYYY,DOY,SOD time array */
  const CPF *cpf	/* I: CPF structure */
)

{
int year;		/* the year */
int month;		/* the month */
int day;		/* the day */
int byear;		/* the year for the begin date*/
int bmonth;		/* the month for the begin date*/
int bday;		/* the day for the begin date*/
int eyear;		/* the year for the end date*/
int emonth;		/* the month for the end date*/
int eday;		/* the day for the end date*/
double jdate_data;	/* the julian date for the data */
double jdate_end;	/* the julian date for end of the effective time of CPF */
double jdate_begin;	/* the julian date for beginning of the effective time 
			   of the CPF */
int leap[12] = {31,29,31,30,31,30,31,31,30,31,30,31}; /* number of days in months
							 for a leap year */
int notleap[12] = {31,28,31,30,31,30,31,31,30,31,30,31}; /* number of days in
							    months, non leap yr */
int *days_in_month_ptr;	/* ptr to to number of days in the month */
int ii;			/* loop variable */
char errmsg[STRLEN];	/* error message */
int sum;		/* sum of the days in the year until greater than day */
int sum2;		/* sum of the days in the year until just less then day */
int mday;		/* the day of the month */
char tmp[STRLEN];	/* temporary variable */

/* ********************************************************
   getting the full julian date for the data's date
******************************************************** */
year = (int)(time[0]+0.5);
day = (int)(time[1]+0.5);

/* 
  see if the year is a leap year and find the month 
*/
ii = 0;
sum = 0;
month = 0;

if (xxx_isleap(year) == FALSE)
    days_in_month_ptr = notleap;
else
    days_in_month_ptr = leap;

while (month == 0 && ii < 12)
    {
    sum2 = sum;
    sum += days_in_month_ptr[ii];

	/* chenged from > to >= to account for 1-based logic (day of 
	year ranges from 1-365 or 1-366 not 0-364
	or 0-365 */
    if (sum >= day)
	month = ii + 1; 
    else
        ii++;
    }

/* if we didn't set the month, then the day in the year is bad */
if (month == 0)
    {
    sprintf(errmsg, "Error: Day %ld in year is incorrect", day);
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__, errmsg);
    return(ERROR);
    }

/* find the day of the month */
mday = day - sum2;

/* 
  find the julian date for the data's date
*/
jdate_data = xxx_fulljd ( year, month, mday, 0);

/* ********************************************************
   getting the full julian date for the effective dates
******************************************************** */

/* pull the year, month and day out of the effective begin and end 
   dates from the CPF */
strncpy(tmp,(const char *)&cpf->File_Attributes.Effective_Date_Begin,4);
tmp[4] = '\0';
byear = atoi(tmp);
if (byear < MIN_YEAR || byear > MAX_YEAR)
    {
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__, "Invalid year in cpf date");
    return(ERROR);
    }

strncpy(tmp,(const char *)&cpf->File_Attributes.Effective_Date_Begin[5],2);
tmp[2] = '\0';
bmonth = atoi(tmp);
if (bmonth < MIN_MONTH || bmonth > MAX_MONTH)
    {
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__, "Invalid month in cpf date");
    return(ERROR);
    }

strncpy(tmp,(const char *)&cpf->File_Attributes.Effective_Date_Begin[8],2);
tmp[2] = '\0';
bday = atoi(tmp);
if (bday < MIN_DAY || bday > MAX_DAY)
    {
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__, "Invalid day in cpf date");
    return(ERROR);
    }

strncpy(tmp,(const char *)&cpf->File_Attributes.Effective_Date_End,4);
tmp[4] = '\0';
eyear = atoi(tmp);
if (eyear < MIN_YEAR || eyear > MAX_YEAR)
    {
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__, "Invalid year in cpf date");
    return(ERROR);
    }

strncpy(tmp,(const char *)&cpf->File_Attributes.Effective_Date_End[5],2);
tmp[2] = '\0';
emonth = atoi(tmp);
if (emonth < MIN_MONTH || emonth > MAX_MONTH)
    {
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__, "Invalid month in cpf date");
    return(ERROR);
    }

strncpy(tmp,(const char *)&cpf->File_Attributes.Effective_Date_End[8],2);
tmp[2] = '\0';
eday = atoi(tmp);
if (eday < MIN_DAY || eday > MAX_DAY)
    {
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__, "Invalid day in cpf date");
    return(ERROR);
    }

/* the fulljd for the effective dates */
jdate_begin = xxx_fulljd ( byear, bmonth, bday, 0);
jdate_end = xxx_fulljd ( eyear, emonth, eday, 0);

/* making sure that the end date is greater than or equal
   to the begin date */
if (jdate_begin > jdate_end)
    {
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__,
	"Error: Begin and End Effiective dates of CPF are corrupt.");
    return(ERROR);
    }

/* ********************************************************
   seeing if the cpf is valid for the data
******************************************************** */
if (jdate_end < jdate_data || jdate_begin > jdate_data)
    {
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__,
	"Error: CPF effective dates don't contain data date");
    return(ERROR);
    }

return(SUCCESS); 
}
