/******************************************************************************
NAME: exx_correct_ephem_time

PURPOSE:
Correct jitter in ephemeris time codes.

DEVELOPMENT HISTORY:

RETURN VALUE:
Type = int
Value    Description
-----    -----------
SUCCESS  Successful completion
ERROR    Operation failed

******************************************************************************
		        Property of the U.S. Government
                            USGS EROS Data Center
******************************************************************************/
#include <stdio.h>
#include <math.h>
#include <string.h>
#include "xxx_Types.h"
#include "xxx_LogStatus.h"
#include "xxx_const.h"
#include "xxx_structures.h"

int exx_correct_ephem_time(
    double rawpos[3][NPCD],         /* ephemeris position telemetry data */
    double rawvel[3][NPCD],         /* ephemeris velocity telemetry data */
    int nephem,			    /* Number of ephemeris points	 */
    double etime[NPCD])             /* ephemeris time telemetry 	 */
{
int i;				/* Loop counter */
double dt;			/* Time between GPS samples		*/
double sx,sy,sz;		/* Sum of position difference (i+1 - i) */
double svx,svy,svz;		/* Sum of veloctiy difference (i+1 - i) */
double s;			/* Total sum of position differences */
double sv;			/* Total sum of velocity differences */
double sum;			/* Sum of corrected delta time */
double dtc[NPCD];		/* Corrected delta time */
double avg;			/* Average time correction */

for( i=0, sum=0.0; i<nephem; i++)
   {
      if( i==0 )
         dtc[i] = 0.0;
      else
         {
         sx = (rawpos[0][i] - rawpos[0][i-1]);
         sy = (rawpos[1][i] - rawpos[1][i-1]);
         sz = (rawpos[2][i] - rawpos[2][i-1]);
         svx = rawvel[0][i-1] * 1000.0;
         svy = rawvel[1][i-1] * 1000.0;
         svz = rawvel[2][i-1] * 1000.0;
         s = sqrt( sx * sx + sy * sy + sz * sz );
         sv = sqrt( svx * svx + svy * svy + svz * svz );
         dtc[i] = s / sv - ( etime[i] - etime[i-1] );
         }
      sum+=dtc[i];
   }
avg = sum / (double)nephem;
for( i=0; i<nephem; i++ )
   {
   etime[i] = etime[i] + dtc[i] - avg;
   }

return(SUCCESS);
}
