/******************************************************************************
NAME: exx_gpstime

PURPOSE:
Package of routines to convert between GPS epoch time (in seconds) and
year, day of year, and seconds of day date/time notation.

DEVELOPMENT HISTORY:
$Log: exx_gpstime.c,v $
Revision 1.2  2000/11/28 13:00:42  storey
Fixed bug in use of ISLEAP macro that was causing year to increment
too fast.

Revision 1.1  2000/11/22 21:52:45  storey
Original version - GPS time conversion routines


RETURN VALUE:
NONE

NOTES:

ALGORITHM REFERENCES:

******************************************************************************
		        Property of the U.S. Government
                             USGS EROS Data Center
******************************************************************************/
#include <math.h>
#include "exx_init_model.h"

#define	ISLEAP(Y)	(((Y%4 != 0) || ((Y%100 == 0) && (Y%400 != 0))) ? 0 : 1)
#define	GPSYEAR	1980
#define	GPSDOY	5
#define	GPSSOD	86400.0
#define	SECPERDAY	86400
#define	DAYPERYEAR	365

double exx_gpstime(
    double *time	/* I: Year, DOY, SOD array to convert to GPS epoch time */
    )
{
  int year;
  int cyear;
  int days;
  double gpssec;

  cyear = (int)(time[0] + 0.5);
  gpssec = (time[1]-1.0)*SECPERDAY + time[2]
         - ((double)(GPSDOY - 1)*SECPERDAY + GPSSOD);

  days = 0;
  year = GPSYEAR;
  while ( year < cyear )
  {
    days += (DAYPERYEAR + ISLEAP(year));
    year++;
  }
  while ( year > cyear )
  {
    days -= (DAYPERYEAR + ISLEAP(cyear));
    cyear++;
  }

  gpssec += (double)(days * SECPERDAY);

  return( gpssec );
}

void exx_gpstoyds(
    double gpstime,	/* I: GPS epoch time to convert to Year, DOY, SOD */
    double *time	/* O: Year, DOY, SOD output array */
    )
{
  int year;
  int days;
  int cdays;
  double gpssec;

  year = GPSYEAR;
  gpssec = gpstime + ((double)(GPSDOY - 1)*SECPERDAY + GPSSOD);
  days = (int)floor(gpssec/SECPERDAY);
  time[2] = gpssec - (double)(days*SECPERDAY);
  cdays = DAYPERYEAR + ISLEAP(year);

  while ( days >= cdays )
  {
    days -= cdays;
    year++;
    cdays = DAYPERYEAR + ISLEAP(year);
  }
  while ( days < 0 )
  {
    year--;
    cdays = DAYPERYEAR + ISLEAP(year);
    days += cdays;
  }

  time[0] = (double)year;
  time[1] = (double)(days+1);

}

