/* ******************************** */
/* if we haven't used this file yet */
/* ******************************** */
#ifndef _EXX_INIT_MODEL_H
#define _EXX_INIT_MODEL_H 1


/* ******************* */
/* other include files */
/* ******************* */
#include "xxx_math.h"
#include "xxx_structures.h"
#include "xxx_const.h"
#include "exx_cpf.h"
#include "xxx_EDCOdl.h"
#include "xxx_ODL.h"
#include "xxx_ALIPCD.h"
#include "lablib3.h"

/***********************/
/* attitude prototypes */
/***********************/
int exx_sc_attitude(
    double *acs_time,   /* I: Attitude control system ref time */
    double *ephem_time, /* I: Ephemeris ref time */
    int nephem,         /* I: Number of ephemeris points */
    double *satpos_x,   /* I: Satellite position X */
    double *satpos_y,   /* I: Satellite position Y */
    double *satpos_z,   /* I: Satellite position Z */
    double *satvel_x,   /* I: Satellite velocity X */
    double *satvel_y,   /* I: Satellite velocity Y */
    double *satvel_z,   /* I: Satellite velocity Z */
    double epatime,	/* I: Time at which to construct attitude */
    double *quat,	/* I: Input quaternion to reduce to r,p,y */
    double *epa_roll,	 /* O: roll calculated for epa point */
    double *epa_pitch,	 /* IO: pitch calculated for epa point */
    double *epa_yaw 	 /* O: yaw calculated for epa point */
    );

int exx_iru2acs(
    int niru,		/* I: number of IRU points */
    VECTOR *iru,	/* I/O: IRU data */
    const CPF *cpf	/* I: CPF structure */
    );

int exx_init_iru(
    double *acs_time,   /* I: Attitude control system ref time */
    double *ephem_time, /* I: Ephemeris ref time */
    int nephem,         /* I: Number of ephemeris points */
    double *satpos_x,   /* I: Satellite position X */
    double *satpos_y,   /* I: Satellite position Y */
    double *satpos_z,   /* I: Satellite position Z */
    double *satvel_x,   /* I: Satellite velocity X */
    double *satvel_y,   /* I: Satellite velocity Y */
    double *satvel_z,   /* I: Satellite velocity Z */
    int niru,		/* I: number of IRU point */
    double *irutime,	/* I/O: IRU data point times */
    VECTOR *iru		/* I/O: IRU data */
    );

int exx_check_date
(
  double *time, 	/* I: data time as year, doy, sod */
  const CPF *cpf	/* I: CPF structure */
);

/* ************************************** */
/* the ephemeris prototypes for functions */
/* ************************************** */
int exx_valid_ephem(
    const ALIPCD *pcd,          /* I: pcd structure */
    EphemType ephem_type,	/* I: ephem type to use GPSPTS or ACSPTS */
    const CPF *cpf,		/* I: CPF structure */
    int  *num_valid,		/* O: the number of valid ephemeris points */
    struct EPHEM *ephem);	/* O: ephemeris struct with valid ephem data */

void exx_movesat(
    int nephem,         /*I: Number of ephemeris points */
    double *satpos_x,   /*I: Satellite position X */
    double *satpos_y,   /*I: Satellite position Y */
    double *satpos_z,   /*I: Satellite position Z */
    double *satvel_x,   /*I: Satellite velocity X */
    double *satvel_y,   /*I: Satellite velocity Y */
    double *satvel_z,   /*I: Satellite velocity Z */
    double dtime,	/* I: Time from ephem_time */
    VECTOR *satpos,	/* O: Output position vector */
    VECTOR *satvel	/* O: Output velocity vector */
    );

/* **************************** */
/* the kalman filter prototypes */
/* **************************** */
int exx_kalman_smooth_iru(
    int  *natt,			/* I: Number of atttitude measurements */
    double *atttime,		/* I: Attitude measurement times */
    VECTOR *att,		/* I: Satellite epa attitude data */
    int  *niru,			/* I/O: Number of gyro measurements */
    double *irutime,		/* I/O: IRU measurement times */
    VECTOR *iru			/* I/O: IRU measurements */
);

int exx_kalman_smooth_gps(
    double rawpos[3][NPCD],         /* ephemeris position telemetry data */
    double rawvel[3][NPCD],         /* ephemeris velocity telemetry data */
    int nephem,                     /* Number of ephemeris points        */
    double etime[NPCD],             /* ephemeris time telemetry          */
    const CPF *cpf,                 /* CPF structure */
    double smpos[3][NPCD],          /* Smoothed GPS position             */
    double smvel[3][NPCD]);	    /* Smoothed GPS velocity             */

/* -----------------------------------------------------------------------
   GPS epoch time to Year, Day of Year, Seconds of Day Conversions
-------------------------------------------------------------------------- */
double exx_gpstime(
    double *time		/* I: Year, DOY, SOD */
);

void exx_gpstoyds(
    double gpstime,		/* I: Input GPS epoch time (seconds) */
    double *time                /* O: Output Year, DOY, SOD */
);

/* -----------------------------------------------------------------------
   Spacecraft Time Code to GPS Epoch Time Conversions
-------------------------------------------------------------------------- */
void exx_init_timecode(
    double gpstime,		/* I: Reference GPS epoch time */
    double timecode,		/* I: Reference spacecraft time code */
    double timecode_rate	/* I: GPS seconds per S/C clock second */
);

double exx_timecodetogps(
    double timecode		/* I: Spacecraft time code */
);

double exx_gpstotimecode(
    double gpstime		/* I: GPS epoch time */
);

int exx_correct_ephem_time(
    double rawpos[3][NPCD],         /* ephemeris position telemetry data */
    double rawvel[3][NPCD],         /* ephemeris velocity telemetry data */
    int nephem,                     /* Number of ephemeris points        */
    double etime[NPCD]);            /* ephemeris time telemetry          */
#endif
