/******************************************************************************
NAME: exx_timecode

PURPOSE:
Package of routines to convert between GPS epoch time (in seconds) and
EO-1 spacecraft clock time (time code) in nominal seconds.

DEVELOPMENT HISTORY:
$Log: exx_timecode.c,v $
Revision 1.1  2000/11/22 21:52:51  storey
Original version - EO1 time code processing routines


RETURN VALUE:
NONE

NOTES:

ALGORITHM REFERENCES:

******************************************************************************
		        Property of the U.S. Government
                             USGS EROS Data Center
******************************************************************************/
#include <math.h>
#include "xxx_LogStatus.h"	/* prototype for xxx_LogStatus.c */

static double gpstime0;		/* Reference GPS Time */
static double timecode0;	/* Reference Time Code */
static double tc_rate;		/* GPS seconds per Clock second */

void exx_init_timecode(
    double gpstime,	/* I: GPS epoch time to initialize */
    double timecode, 	/* I: Clock time code to initialize */
    double rate  	/* I: Clock rate to initialize */
    )
{

/* All we're doing here is setting the static variables for future use. */
  gpstime0 = gpstime;
  timecode0 = timecode;
  tc_rate = rate;
}

double exx_gpstotimecode(
    double gpstime	/* I: GPS epoch time to convert to time code */
    )
{
/* Always check for valid initialization first */
if ( fabs( tc_rate - 1.0 ) > 0.5 )
  {
    xxx_LogStatus( "EXXLIB", __FILE__, __LINE__,
          "EO-1 time code conversion not correctly initialized" );
    return( gpstime );
  }

return( timecode0 + (gpstime - gpstime0)/tc_rate );
}

double exx_timecodetogps(
    double timecode	/* I: Spacecraft time code to convert to GPS */
    )
{
/* Always check for valid initialization first */
if ( fabs( tc_rate - 1.0 ) > 0.5 )
  {
    xxx_LogStatus( "EXXLIB", __FILE__, __LINE__,
          "EO-1 time code conversion not correctly initialized" );
    return( timecode );
  }

return( gpstime0 + (timecode - timecode0)*tc_rate );
}

