/******************************************************************************
NAME:                           exx_ariesgha.c

PURPOSE: Find the Greenwich Hour Angle of Aries

DEVELOPMENT HISTORY:
$Log: exx_ariesgha.c,v $
Revision 1.1  2000/11/22 22:20:41  storey
Original version - derived from gxx_ariesgha.c


RETURN VALUE: none

REFERENCE: "Methods Of Orbit Determination" by Escobal

      Code is from Navel Observatory Vector Astronomy Subroutines (NOVAS)
      C Version 1.0
      June 1996

      U. S. Navel Observatory
      Astronomical Applications Dept.
      3450 Massachusetts Ave., NW
      Washington, DC  20392-5420

****************************************************************************
                        Property of the U.S. Government
                           USGS EROS Data Center
*****************************************************************************/
#include <math.h>
#include "exx_geometric.h"
void exx_ariesgha 
(
   double tjd, 		/* I: full julian day */
   double seconds, 	/* I: seconds of the day */
   double *gha 		/* O: Greenwich hour angle */
)
{
   double tjd2,
	  tu,
          gst0,
	  gst,
          mid,
          mid2;

   tjd2 = tjd - seconds / SEC_PER_DAY;
   tu = (tjd2 - 2415020.0) / 36525;
   mid = 36000.7689 * tu;
   mid2 = (mid / 360.0 - (int)(mid / 360.0)) * 360.0;
   gst0 = 99.69098 + mid2 + 0.00038708 * tu * tu;
   gst = (gst0 + seconds / 60.0 * 0.25068447);
   gst = (gst / 360.0 - (int)(gst / 360.0)) * 360.0;
   *gha = gst * D2R;

}

