/*******************************************************************************
NAME:		exx_cart2sph

PURPOSE:	
Convert catesian coordinates to spherical coordinates

DEVELOPMENT HISTORY:
$LOG$

RETURN VALUE:
Type = int (SUCCESS or ERROR)
********************************************************************************
               Landsat 7 Image Assessment System (IAS) Software
                       Property of the U.S. Government
                               NASA/GSFC/EDC
*******************************************************************************/
#include "exx_geometric.h"

int exx_cart2sph(VECTOR *vec, /* I Vector containing Cartesian coords   */
	         double *lat,	  /* O Latitude in spherical coordinates    */
	         double *longs,	  /* O Longitude in spherical coordinates   */
	         double *radius)  /* O Distance from origin to the point    */
{
/*
  Calculate the radius (length of the vector from the origin)
*/
*radius = sqrt(vec->x * vec->x + vec->y * vec->y + vec->z * vec->z);

/*
  Calculate the latitude spherical coordinate
*/
*lat = asin(vec->z / *radius);

/*
  Calculate the longitude spherical coordinate.  The longitude depends on
  the quadrant that the vector lies in.
*/
*longs = atan2(vec->y, vec->x);

return SUCCESS;
}
