/******************************************************************************
FUNCTION:	exx_check_start_end_date

PURPOSE:
See if the start or end date is valid

DEVELOPMENT HISTORY:
$Log: exx_check_start_end_date.c,v $
Revision 1.1  2000/12/18 19:45:51  storey
Original version.


RETURN VALUE:
Type = int
Value    Description
-----    -----------
ERROR    Error processing the model data
SUCCESS  Model data was updated  

NOTES:

ALGORITHM REFERENCES:
*******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "xxx_Types.h"
#include "xxx_LogStatus.h"
#include "xxx_math.h"
#include "axx_alimodel.h"

int exx_check_start_end_date
(
  int isdate, 	/* I: start date (YYYYMMDD) */
  int iedate 	/* I: end date (YYYYMMDD) */
)

{
char sdate[STRLEN];
char edate[STRLEN];
int byear;		/* the year of the scan time for the begin date*/
int bmonth;		/* the month of the scan time for the begin date*/
int bday;		/* the day of the scan time for the begin date*/
int eyear;		/* the year of the scan time for the end date*/
int emonth;		/* the month of the scan time for the end date*/
int eday;		/* the day of the scan time for the end date*/
double jdate_end;	/* the julian date for end of the effective time of CPF */
double jdate_begin;	/* the julian date for beginning of the effective time 
			   of the CPF */
int leap[12] = {31,29,31,30,31,30,31,31,30,31,30,31}; /* number of days in months
							 for a leap year */
int notleap[12] = {31,28,31,30,31,30,31,31,30,31,30,31}; /* number of days in
							    months, non leap yr */
int min_day;	/* smallest day # in month */
int max_day;	/* largest day # in month */
char tmp[STRLEN];	/* temporary variable */


/* convert the integers to strings */
if (sprintf(sdate, "%d\0", isdate) <= 0 )
{
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__, "Error in sprintf");
    return(ERROR);
}

if (sprintf(edate, "%d\0", iedate) <= 0 )
{
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__, "Error in sprintf");
    return(ERROR);
}

/* ********************************************************
   getting the full julian date for the effective dates
******************************************************** */

/* pull the year, month and day out of the effective begin and end 
   dates from the CPF */
strncpy(tmp,&sdate[0],4);
tmp[4] = '\0';
byear = atoi(tmp);
if (byear < MIN_YEAR || byear > MAX_YEAR)
{
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__, "Invalid year in start date");
    return(ERROR);
}

strncpy(tmp,&sdate[4],2);
tmp[2] = '\0';
bmonth = atoi(tmp);
if (bmonth < MIN_MONTH || bmonth > MAX_MONTH)
{
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__, "Invalid month in start date");
    return(ERROR);
}

if (xxx_isleap(byear) == TRUE)
	max_day = leap[bmonth - 1];
else
	max_day = notleap[bmonth - 1];
min_day = 1.0;
strncpy(tmp,&sdate[6],2);
tmp[2] = '\0';
bday = atoi(tmp);
if (bday < min_day || bday > max_day)
{
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__, "Invalid day in start date");
    return(ERROR);
}

strncpy(tmp,&edate[0],4);
tmp[4] = '\0';
eyear = atoi(tmp);
if (eyear < MIN_YEAR || eyear > MAX_YEAR)
{
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__, "Invalid year in end date");
    return(ERROR);
}

strncpy(tmp,&edate[4],2);
tmp[2] = '\0';
emonth = atoi(tmp);
if (emonth < MIN_MONTH || emonth > MAX_MONTH)
{
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__, "Invalid month in end date");
    return(ERROR);
}

if (xxx_isleap(eyear) == TRUE)
	max_day = leap[emonth - 1];
else
	max_day = notleap[emonth - 1];
min_day = 1.0;
strncpy(tmp,&edate[6],2);
tmp[2] = '\0';
eday = atoi(tmp);
if (eday < min_day || eday > max_day)
{
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__, "Invalid day in end date");
    return(ERROR);
}

/* the fulljd for the effective dates */
jdate_begin = xxx_fulljd ( byear, bmonth, bday, 0);
jdate_end = xxx_fulljd ( eyear, emonth, eday, 0);

/* making sure that the end date is greater than or equal
   to the begin date */
if (jdate_begin > jdate_end)
{
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__,
	"Error: Begin date greater then End date");
    return(ERROR);
}

return(SUCCESS); 
}
