/*******************************************************************************
NAME:		exx_earthradius

PURPOSE:
Finds the radius of the earth at a specified latitude

DEVELOPMENT HISTORY:
$Log: exx_earthradius.c,v $
Revision 1.2  2001/01/03 19:23:39  storey
Added #include xxx_LogStatus.h to declare xxx_LogStatus().

Revision 1.1  2000/11/22 22:20:48  storey
Original version - derived from gxx_earthradius.c


RETURN VALUE:
Type = int (SUCCESS or ERROR)
Value		Description
-----		-----------
SUCCESS		Successful calculation of earth radius
ERROR		Error calculating earth radius (error in CPF eccentricity)

********************************************************************************
                       Property of the U.S. Government
                           USGS EROS Data Center
*******************************************************************************/
#include "exx_geometric.h"
#include "xxx_LogStatus.h"

int  exx_earthradius(double latc,    	/* I: Lat to find the radius at (rad) */
		     const CPF *cpf,	/* I: Cal parm file constants 	      */
	             double *radius) 	/* O: Radius of earth at given lat (m)*/
{
double	tmp1,		/* temporary for checking calculation	*/
	tmp2;		/* temporary for checking calculation	*/

tmp1 = 1.0 - cpf->Earth_Constants.Eccentricity;
tmp2 = 1.0 - cpf->Earth_Constants.Eccentricity * cos(latc) * cos(latc);
if ((tmp1 < 0.0) || (tmp2 <= 0.0))
{
    xxx_LogStatus("EXXLIB", __FILE__,__LINE__,"Error calculating earth radius");
    return(ERROR);
}

*radius = (cpf->Earth_Constants.Semi_Major_Axis * sqrt(tmp1)) / sqrt(tmp2);


return(SUCCESS);
}
