/****************************************************************************
NAME:                           exx_find_deg

PURPOSE:  Extracts deg portions of an angle.

DEVELOPMENT HISTORY:
AUTHOR        Change-ID    DATE     Description of Change
------        ---------  --------   ---------------------
Gail Schmidt     1.0     05/22/97   Brought over from LAS.

RETURN VALUE:   Type = int
Value    Description
-----    -----------
SUCCESS  Successful completion
ERROR    Operation failed

ALGORITHM DESCRIPTION:
        Extract degree portion of angle
        Return

ALGORITHM REFERENCES:

******************************************************************************
                        Property of the U.S. Government
                            USGS EROS Data Center
*****************************************************************************/
#include "exx_geometric.h"

int exx_find_deg (
  double angle,			/* I: Angle in total degrees      */
  int  *degree)                 /* O: Degree portion of the angle */
{
  int  sign;			/* Sign of angle            */
  int  minute;			/* Minutes portion of angle */
  double sec;			/* Seconds portion of angle */

  *degree = (int) angle;
  sign = 1;
  if (*degree < 0) sign = -1;
  *degree = (int) fabs (angle);
  minute =  (int) ((fabs (angle) - *degree) * 60.0);
  sec =  (((fabs (angle) - *degree) * 60.0) - minute) * 60.0;
  if (sec >= 60.0) minute++;
  if (minute >= 60) (*degree)++;
  *degree *= sign;

  return (SUCCESS);
}
