#include "exx_geometric.h"

/****************************************************************************
NAME:                           exx_find_min

PURPOSE:  Extracts min portions of an angle.

DEVELOPMENT HISTORY:
AUTHOR        Change-ID    DATE     Description of Change
------        ---------  --------   ---------------------
Gail Schmidt     1.0     05/22/97   Brought over from LAS.

RETURN VALUE:   Type = int
Value    Description
-----    -----------
SUCCESS  Successful completion
ERROR    Operation failed

ALGORITHM DESCRIPTION:
        Extract minute portion of angle
        Return

ALGORITHM REFERENCES:

******************************************************************************
                        Property of the U.S. Government
                            USGS EROS Data Center
*****************************************************************************/

int exx_find_min (
  double angle,			  /* I: Angle in total degrees      */
  int  *minute)                   /* O: Minute portion of the angle */
{
  double sec;                     /* Seconds portion of angle */

  angle = fabs (angle);
  angle -= (int)angle;
  *minute = (int) (angle * 60.0);
  sec = ((angle * 60.0) - *minute) * 60.0;
  if (sec > 60.0) (*minute)++;
  if (*minute >= 60) *minute -= 60;

  return (SUCCESS);
}
