#include "exx_geometric.h"

/****************************************************************************
NAME:                           exx_find_sec

PURPOSE:  Extracts sec portions of an angle.

DEVELOPMENT HISTORY:
AUTHOR        Change-ID    DATE     Description of Change
------        ---------  --------   ---------------------
Gail Schmidt     1.0     05/22/97   Brought over from LAS.

RETURN VALUE:   Type = int
Value    Description
-----    -----------
SUCCESS  Successful completion
ERROR    Operation failed

ALGORITHM DESCRIPTION:
        Extract seconds portion of angle
        Return

ALGORITHM REFERENCES:

******************************************************************************
                        Property of the U.S. Government
                           USGS EROS Data Center
*****************************************************************************/

int exx_find_sec (
  double angle,                 /* I: Angle in total degrees      */
  double *second)               /* O: Second portion of the angle */
{
  int temp_sec;

  *second = fabs (angle);
  *second -= (int)(*second);
  *second *= 60.0;
  *second -= (int)(*second);
  *second *= 60.0;
  if (*second > 60.0) *second -= 60.0;
  temp_sec = (int) (*second * 1000);      /* Truncate to 0.001 sec */
  *second = temp_sec / 1000.0;

  return (SUCCESS);
}
