#include <stdio.h>
#include <math.h>
#include "xxx_const.h"
#include "xxx_LogStatus.h"
#include "exx_geometric.h"

/*
	Function to compute the Geoid Height separation at a given
	Latitude and Longitude.

	(double) exx_geoidheight ((double)Latitude, (double)Longitude);

	Input:	Latitude  (+- 90 decimal degrees) (double)
				Longitude (+-180 decimal degrees) (double)

	Output:	Geoid Separation in meters. (double)
*/


double exx_geoidheight (double lat, double lng)
{
	int	i,j;
	double	x, y, Height;
	double	GH0, GH1, GH2, GH3;

	/* Compute initial (SW) file position */
	
	i = (int) ceil((90.0 - lat) * 2.0);     /* DJD 3/15/95 */

	if ( lng < 0.0 ) lng += 360.0;
	j = (int) floor(lng * 2.0);
	
	if (j < 0)
		j += 360;

	/* Get surrounding points */

	GH0 = exx_getpoint(   i, j );    /* SW Point */
	GH1 = exx_getpoint(   i, j+1 );  /* SE Point */
	GH2 = exx_getpoint( i-1, j+1 );  /* NE Point */
	GH3 = exx_getpoint( i-1, j );    /* NW Point */


	/* Bi-Linear Interpolation of the four points */

	/*
	     Data is half-degree; doubling normalizes the fractional
		  step to range from 0 to 1    DJD 3/15/95)
        */

	x = 2.0*lng - (floor)(lng*2.);       /* DJD 3/16/95 */
	y = 2.0*lat - (floor)(lat*2);        /* DJD 3/16/95 */

	Height =	(1.0 - x) * (1.0 - y) * GH0
			 + (1.0 - x) * y * GH3
			 + x * (1.0 - y) * GH1
			 + x * y * GH2;

	/* Return Geoid Height separation value */

	return (Height);
}


double exx_getpoint(int i, int j)
{
  FILE		*GHFile;
  char		GHFileName[STRLEN];
  static unsigned char *GHBuffer = NULL;
  short int	GeoidSep;
  long int	FilePos;

  /* Load the buffer the first time through */
  if ( GHBuffer == NULL )
  {
     strcpy(GHFileName, getenv("EO1SYS"));
     strcat(GHFileName, "/ghgrid.dat");
     GHBuffer = (unsigned char *)malloc(720*361*sizeof(unsigned char));
     if ( (GHBuffer == NULL) ||
          ((GHFile = fopen(GHFileName, "rb")) == NULL) )
     {
	  xxx_LogStatus("EXXLIB", __FILE__, __LINE__,
                        "Cannot load Geoid Height table.");
          free(GHBuffer);
          GHBuffer = NULL;
	  return(0.0);
     }
     fread( GHBuffer, 1, 720*361, GHFile );
     fclose  (GHFile);
  }

  FilePos = ((long)i * 720) + ((long)j % 720);   /* DJD 3/15/95 */
  if ( (FilePos >= 0) && (FilePos < 720*361) )
     GeoidSep = GHBuffer[FilePos];
  else
     GeoidSep = 0;
  if ( GeoidSep > 127 )
    GeoidSep |= 0xFF00;
  return ((double)GeoidSep);
}


