/* ******************************** */
/* if we haven't used this file yet */
/* ******************************** */
#ifndef _EXX_GEOMETRIC_H
#define _EXX_GEOMETRIC_H 1


/* ******************* */
/* other include files */
/* ******************* */
#include "xxx_DDR.h"
#include "exx_gcplib.h"
#include "xxx_math.h"
#include "xxx_structures.h"
#include "xxx_const.h"
#include "exx_cpf.h"
#include "proj.h"

/*****************************************/
/* coordinate system conversion routines */
/*****************************************/

void exx_calc_nutang (
double tdbtime,  	/*TDB (Barycentric Dynamic Time) time in Julian 
			centuries since J2000.0 */
double *longnutation, 	/* Nutation in longitude in seconds of arc. */
double *obliqnutation 	/*  Nutation in obliquity in seconds of arc. */
);

int exx_findmjdcoords (
    double mjd,        /* I: mjd value to search for              */
    double *x,       /* O: x coord for mjd value                */
    double *y,       /* O: y coord for mjd value                */
    double *ut1_utc, /* O: UT1-UTC coord for mjd value          */
    const CPF *cpf);/* I: info from Calibraion Parameter File  */

int exx_getmjdcoords(
    const double ephem_time[3],/* I: year, day, & second of the ephemeris time*/
    double *mjd,        /* O: modified Julian date                      */
    double *x,          /* O: x shift pole wander in arc sec.           */
    double *y,          /* O: y shift pole wander in arc sec.           */
    double *ut1_utc,    /* O: UT1 - UTC time difference in seconds      */
    const CPF *cpf);   /* I: the information from the CPF              */


/* ************************************** */
/* the ephemeris prototypes for functions */
/* ************************************** */
void exx_calcgha(
    const double reftime[],   /* I: Reference time                                    */
    double ut1_utc,     /* I: UT1-UTC, in seconds, due to variation             */
                        /* of Earth's spin rate                                 */
    double *zi,         /* O: Precession angles (radians)                       */
    double *theta,      /* O: Precession angles (radians)                       */
    double *zeta,       /* O: Precession angles (radians)                       */
    double *e,          /* O: true obliquity of the ecliptic                    */
    double *ebar,       /* O: Mean ecliptic                                     */
    double *psisum,     /* O: Nutation in logitude (radians)                    */
    double *gha);       /* O: Greenwich Apparent Sidereal Time (radians)        */

int exx_ic2efc(
    double xp,                  /* I: Earth's true pole offset from mean pole */
                                /* in arc second */
    double yp,                  /* I: Earth's true pole offset from mean pole */
                                /* in arc second */
    double ut1_utc,             /* I: Irregular Earth rotation, in seconds */
    const VECTOR *craft_pos,  	/* I: Satellite position in ECI, input */
    const VECTOR *craft_vel,  	/* I: Satellite velocity in ECI, input */
    const double ephem_time[3],       /* I: Time of the ephemeris point, */
                                /* in year, day and second, input */
    VECTOR *fe_satpos,  	/* O: Satellite position in ECR, output */
    VECTOR *fe_satvel); 	/* O: Satellite velocity in ECR, output */

int exx_efc2ic(
    double xp,                  /* I: Earth's true pole offset from mean pole */
                                /* in arc second */
    double yp,                  /* I: Earth's true pole offset from mean pole */
                                /* in arc second */
    double ut1_utc,             /* I: Irregular Earth rotation, in seconds */
    const VECTOR *craft_pos,  	/* I: Satellite position in ECR, input */
    const VECTOR *craft_vel,  	/* I: Satellite velocity in ECR, input */
    const double ephem_time[3], /* I: Time of the ephemeris point, */
                                /* in year, day and second, input */
    VECTOR *ic_satpos,  	/* O: Satellite position in ECI, output */
    VECTOR *ic_satvel); 	/* O: Satellite velocity in ECI, output */

void exx_ariesgha(double tjd, double seconds, double *gha );
int exx_earthradius(double latitude, const CPF *cpf, double *earthradius); 
void exx_geod2cart(double latitude, double longitude, double height,
        double semimajor, double flattening, VECTOR *cart);
int exx_cart2sph(VECTOR *vec, double *lat, double *longs, double *radius);
int exx_sph2cart(double *, double *, double *, VECTOR *);
int exx_centh2det(double latc, double radius, const CPF *cpf,
	double *latd, double *height);
int exx_det2centh(double latd, double height, const CPF *cpf, 
	double *latc, double *radius);
int exx_transmat(const VECTOR *, const VECTOR *, double[3][3]);
void exx_nutation(VECTOR, double, double, double, VECTOR *);
int exx_polar_motion(VECTOR, double, double, VECTOR *);
void exx_precession(VECTOR, double, double, double, VECTOR *);
int exx_gmt2gst(int, int, double, double *);
int exx_check_start_end_date(int isdate, int iedate );

int exx_decdeg ( double *angle, char *coform, char *type);
int exx_degdms ( double *deg, double *dms, char *code, char *check);
int exx_find_deg ( double angle, int *degree);
int exx_find_min ( double angle, int *minute);
int exx_find_sec ( double angle, double *sec);
int exx_get_units ( char *unit_name, int *unit_num );
int exx_mapedg ( double *seast, double *eeast, double *snorth, double *enorth, 
	int *inunits, int *inspheroid, int *proj_code, int *zone, 
	double *parout, int *units, int *spheroid, double *pxmin, 
	double *pxmax, double *pymin, double *pymax);
int exx_proj_err ( int err);
int exx_projtran (  int *inproj, int *inunit, int *inzone, double *inparm,
  int *inspheroid, int *outproj, int *outunit, int *outzone,
  double *outparm, int *outspheroid, double *inx, double *iny, double *outx,
  double *outy);

int exx_mapedg (
  double *seast,        /* I/O: Upper left longitude coordinate        */
  double *eeast,        /* I/O: Lower right longitude coordinate       */
  double *snorth,       /* I/O: Upper left latitude coordinate         */
  double *enorth,       /* I/O: Lower right latitude coordinate        */
  int  *inunits,        /* I/O: Input space units                      */
  int  *inspheroid,     /* I: Input space spheroid code                */
  int  *proj_code,      /* I: Projection code; output space            */
  int  *zone,           /* I: Projection zone code; output space       */
  double parout[15],    /* I: Projection parameters; output space      */
  int  *units,          /* I: Projection units code; output space      */
  int  *spheroid,       /* I: Projection spheroid code; output space   */
  double *pxmin,        /* O: Projection minimum in X                  */
  double *pxmax,        /* O: Projection maximum in X                  */
  double *pymin,        /* O: Projection minimum in Y                  */
  double *pymax);       /* O: Projection maximum in Y                  */

/* ************************************** */
/* the GCP prototypes for functions */
/* ************************************** */
int exx_gcp_ls (
  const xxx_DDR_TYPE *ddr,    /* I: DDR for the desired band resolution   */
  int  numgcps,               /* I: Number of GCPs in the GCPLIB          */
  const GCPLIB *in_gcps,      /* I: Input array of GCPLib tiepoints in row-major format */
  GCPLIB *gcps);               /* O: Output array of GCPLib tiepoints in row-major format */

/*
 * Geoid Separation Table Access Routines
 */
double exx_geoidheight(double Lat, double Lon);
double exx_getpoint(int row, int col);

#endif
