/******************************************************************************
NAME: exx_getmjdcoords

PURPOSE:
calculates the modified julian date and then sees if the date is in the cpf

DEVELOPMENT HISTORY:
$Log: exx_getmjdcoords.c,v $
Revision 1.1  2000/11/22 22:20:55  storey
Original version - derived from gxx_getmjdcoords.c


RETURN VALUE:
Type = int
Value    Description
-----    -----------
SUCCESS  Successful completion 
ERROR    Operation failed

NOTES:

ALGORITHM REFERENCES:
*****************************************************************************
                        Property of the U.S. Government
                            USGS EROS Data Center
******************************************************************************/
#include <stdio.h>
#include <math.h>
#include "xxx_const.h"
#include "xxx_Types.h"
#include "xxx_LogStatus.h"
#include "xxx_math.h"
#include "exx_geometric.h"
#include "exx_cpf.h"

int exx_getmjdcoords(
    const double ephem_time[3],/* I: year, day, & second of the ephemeris time*/
    double *mjd,	/* O: modified Julian date			*/
    double *x,		/* O: x shift pole wander in arc sec.		*/
    double *y,		/* O: y shift pole wander in arc sec.           */
    double *ut1_utc,		/* O: UT1 - UTC time difference in seconds	*/
    const CPF *cpf)	/* I: the information from the CPF		*/

{
  int  year = 1,			/* Year					*/
        mo = 1,			/* First Month				*/
	day = 1,		/* First Day				*/
	status;

/*
** Calculate the full Julian Date					
*/
  year = (int)ephem_time[0];
  *mjd = xxx_fulljd(year, mo, day, 0);
  *mjd += (ephem_time[1] - 1.0);
  *mjd += (ephem_time[2] / SEC_PER_DAY);

/*
** Calculate the Modified Julian Date					

   To do this 
   Modified Julian Date = Full Julian Date - MJD
*/
  *mjd -= MJD;

/*
  Get the coordinates that correspond to the mjd from the CPF
*/
status = exx_findmjdcoords(*mjd,x,y,ut1_utc,cpf);
if (status != SUCCESS) 
    {
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__,
	"Error returned from exx_findmjdcoords");
    return(ERROR);
    }

return(SUCCESS);
}
