/*******************************************************************************
NAME: 		exx_polar_motion(r_old, xp, yp, r_new)

PURPOSE:	
Transformation from Earth's true spin axis (old system) to the mean
pole (CIO) (new system).

DEVELOPMENT HISTORY:
$Log: exx_polar_motion.c,v $
Revision 1.1  2000/11/22 22:20:58  storey
Original version - derived from gxx_polar_motion.c


RETURN VALUE:
Type = int (SUCCESS)

********************************************************************************
                       Property of the U.S. Government
                          USGS EROS Data Center
*******************************************************************************/
#include "exx_geometric.h"

int  exx_polar_motion(VECTOR r_old, /* I: coordinates (x, y, z) in the
                                               old system                    */
                   double xp, /* I: true pole position in the mean pole coords
                                    system, x-axis pointing along Greenwich
                                    meridian; in arc seconds                 */
                   double yp, /* I: true pole position in the mean pole coords
                                    system, y-axis pointing along west 90
                                    degree meridian; in arc seconds          */
                   VECTOR *r_new) /* O: coordinates in the new system*/
{
  double s2r;			/* arc second to radian		*/
  
  /* Multiply the x and y pole positions by PI / (180*3600).
   */
  s2r = PIE / (648000.0);
  xp *= s2r;
  yp *= s2r;

  /* Calculate the mean pole coords using the following:
   * new x coord = old x coord + x pole position * old z coord
   * new y coord = old y coord - y pole position * old z coord
   * new z coord = old z coord - x pole position * old x coord + 
   *               y pole position * old y coord
   */
  r_new->x = r_old.x + xp * r_old.z;
  r_new->y = r_old.y - yp * r_old.z;
  r_new->z = r_old.z - xp * r_old.x + yp * r_old.y;

  return SUCCESS;
}

