/*******************************************************************************
NAME:		exx_transmat

PURPOSE:
Compose the coordinate transformation matrix from ECI system to Orbit system

PROGRAM HISTORY:
$Log: exx_transmat.c,v $
Revision 1.2  2001/01/03 19:25:16  storey
Added #include xxx_LogStatus.h to declare xxx_LogStatus().

Revision 1.1  2000/11/22 22:21:02  storey
Original version - derived from gxx_transmat.c


RETURN VALUE:
Type = int (SUCCESS or ERROR)

ALGORITHM REFERENCES:
********************************************************************************
                       Property of the U.S. Government
                          USGS EROS Data Center
*******************************************************************************/
#include "exx_geometric.h"
#include "xxx_math.h"
#include "xxx_LogStatus.h"

int  exx_transmat(const VECTOR *satpos, /* I: pointer to sat. position vec */
                  const VECTOR *satvel, /* I: pointer to sat. velocity vec */
                  double eci2ob[3][3])    /* O: transformation matrix from ECI
                                             to orbit system                 */
{
VECTOR scy, scx;	/* Y and X vector in Orbit System	     */
double mag;		/* magnitude of vector			     */

/*
**  Unit Z vector as the last row in the transformation matrix
*/
mag = xxx_norm(satpos);
if(mag == 0.0)
{
   xxx_LogStatus("EXXLIB", __FILE__, __LINE__, 
                 "Magnitude of vector is zero...will result in divide by zero");
    return(ERROR);
}    

eci2ob[2][0] = -satpos->x / mag;
eci2ob[2][1] = -satpos->y / mag;
eci2ob[2][2] = -satpos->z / mag;

/*
**  Unit Y vector as the second row in the transformation matrix
*/
xxx_cross(satpos, satvel, &scy);
mag = xxx_norm(&scy);
if(mag == 0.0)
{
   xxx_LogStatus("EXXLIB", __FILE__, __LINE__, 
                 "Magnitude of vector is zero...will result in divide by zero");
    return(ERROR);
}
eci2ob[1][0] = -scy.x / mag;
eci2ob[1][1] = -scy.y / mag;
eci2ob[1][2] = -scy.z / mag;

/*
**  Unit X vector as the first row in the transformation matrix
*/
xxx_cross(&scy, satpos, &scx);
mag = xxx_norm(&scx);
if(mag == 0.0)
{
   xxx_LogStatus("EXXLIB", __FILE__, __LINE__, 
                 "Magnitude of vector is zero...will result in divide by zero");
    return(ERROR);
}
eci2ob[0][0] =  scx.x / mag;
eci2ob[0][1] =  scx.y / mag;
eci2ob[0][2] =  scx.z / mag;

return SUCCESS;
}
