/****************************************************************************
NAME:    gxx_reverse_to_input

PURPOSE: 
gxx_reverse_to_input takes an output pixel and maps it back into input space
using the projtosat coefficients for the indicated grid cell.  This is used 
as part of the process of finding the scan gap.

DEVELOPMENT HISTORY:
$Log: gxx_reverse_to_input.c,v $
Revision 1.2  2000/11/22 22:14:22  storey
Replaced by exx_transmat.c

Revision 1.1.1.1  1999/12/01 19:01:20  mbs
Import of IAS 3.22

 * 
 *    Rev 1.1   Thu Mar 05 13:28:58 1998   tbeckman
 * Changed longs to ints (except for GCTP calls) to remove compiler warnings in -64 mode.
 * 
 *    Rev 1.0   Fri Aug 08 11:30:44 1997   tbeckman
 * Original version

NOTES:
none

ALGORITHM REFERENCES:
none

*****************************************************************************/

#include "gxx_grid.h"       /* gxx_GRID_BAND_TYPE definition */
#include "gxx_geometric.h"  /* prototype for this routine */
#include "gxx_debug.h"      /* GXX_ASSERT definition */


void gxx_reverse_to_input 
(
    const gxx_GRID_BAND_TYPE *grid_band_ptr, /* I: pointer to grid band */
    int  row,          /* I: row in grid */ 
    int  col,          /* I: column in grid */
    double in_samp,    /* I: input space sample */
    double in_line,    /* I: input space line */
    double *out_samp,  /* O: output space sample */
    double *out_line   /* O: output space line */
)          
{
    double lms;  /* cached value of in_line * in_samp */
    struct COEFFICIENTS *coeff_ptr; /* pointer to projtosat coefficients for
                                       the current grid cell */

    /* verify row and col are legal */
    GXX_ASSERT(row < (grid_band_ptr->ngrid_lines));
    GXX_ASSERT(col < grid_band_ptr->ngrid_samps);

    /* get the pointer to the coefficients */
    coeff_ptr = &grid_band_ptr->projtosat[row/2 * 
                      (grid_band_ptr->ngrid_samps - 1) + col];

   lms = in_line * in_samp;
  
   /* calculate the pixel position in input space using the projtosat 
      (inverse) coefficients */
   *out_samp = coeff_ptr->a[0] + coeff_ptr->a[1] * in_samp + 
               coeff_ptr->a[2] * in_line + coeff_ptr->a[3] * lms;
   *out_line = coeff_ptr->b[0] + coeff_ptr->b[1] * in_samp + 
               coeff_ptr->b[2] * in_line + coeff_ptr->b[3] * lms;
}


