/* the structures to go into the CPF structure */
#ifndef _EXX_CPF_H
#define _EXX_CPF_H 1

#include "xxx_Types.h"
#include "lablib3.h"
#include "xxx_ODL.h"

#define	NUM_ALI_BANDS	10
#define	NUM_HYP_BANDS	242
#define	MAX_ALI_SCA	20

struct FILE_ATTRIBUTES
   {
   char Effective_Date_Begin[11];
   char Effective_Date_End[11];
   char CPF_File_Name[27];
   };

struct EARTH_CONSTANTS
   {
   char Ellipsoid_Name[6];
   double Semi_Major_Axis;
   double Semi_Minor_Axis;
   double Ellipticity;
   double Eccentricity;
   double Earth_Spin_Rate;
   double Gravity_Constant;
   double J2_Earth_Model_Term;
   };

struct ORBIT_PARAMETERS
   {
   int WRS_Cycle_Days;
   int WRS_Cycle_Orbits;
   int Scenes_Per_Orbit;
   double Orbital_Period;
   double Angular_Momentum;
   double Orbit_Radius;
   double Orbit_Semimajor_Axis;
   double Orbit_Semiminor_Axis;
   double Orbit_Eccentricity;
   double Inclination_Angle;
   float Argument_Of_Perigee;
   int Descending_Node_Row;
   float Long_Path1_Row60;
   char Descending_Node_Time_Min[6];
   char Descending_Node_Time_Max[6];
   double Nodal_Regression_Rate;
   };

struct ALI_PARAMETERS
   {
   float IFOV_MS_along;
   float IFOV_MS_across;
   float IFOV_PAN_along;
   float IFOV_PAN_across;
   };

struct ALI_SCA_PARAMETERS
   {
   int Number_Detectors[NUM_ALI_BANDS];
   float Along_LOS_Legendre[NUM_ALI_BANDS][3];   /* Along track viewing vector X Legendre coefficients */
   float Across_LOS_Legendre[NUM_ALI_BANDS][3];  /* Across track viewing vector Y Legendre coefficients */
                                                 /* The viewing vector Z coordinate is 1 by definition */
   float Along_Detector_Offset_MS[NUM_ALI_BANDS-1][320];
   float Across_Detector_Offset_MS[NUM_ALI_BANDS-1][320];
   float Along_Detector_Offset_PAN[960];
   float Across_Detector_Offset_PAN[960];
   };

struct ALI_ODD_EVEN_OFFSETS
   {
   float Nom_Detector_Shift[NUM_ALI_BANDS];  /* Line # of nominal detector at image start */
   float Even_Detector_Shift[NUM_ALI_BANDS]; /* IFOV offset from Nominal detector */
   float Odd_Detector_Shift[NUM_ALI_BANDS];  /* IFOV offset from Nominal detector */
   };

struct ALI_FOCAL_PLANE_PARAMETERS
   {
   int Num_SCA;
   struct ALI_SCA_PARAMETERS SCA_Parameters[MAX_ALI_SCA];
   struct ALI_ODD_EVEN_OFFSETS Odd_Even_Offsets;
   };

struct HYP_SCAN_PARAMETERS
   {
   float IFOV_VNIR_Along;
   float IFOV_VNIR_Across;
   float IFOV_SWIR_Along;
   float IFOV_SWIR_Across;
   float Integration_Time;
   float Frame_Rate;
   };

struct HYP_FOCAL_PLANE_PARAMETERS
   {
   int Number_Detectors;
   float Along_LOS_Legendre[NUM_HYP_BANDS][3];     /* Along track viewing vector X Legendre coefficients */
   float Across_LOS_Legendre[NUM_HYP_BANDS][3];    /* Across track viewing vector Y Legendre coefficients */
                                    /* The viewing vector Z coordinate is 1 by definition */
   float Along_Detector_Offset[NUM_HYP_BANDS][256];
   float Across_Detector_Offset[NUM_HYP_BANDS][256];
   };

struct ATTITUDE_PARAMETERS
   {
   double IRU_To_Attitude_Matrix[3][3];
   double Attitude_To_ALI_Matrix[3][3];
   double Attitude_To_HYP_Matrix[3][3];
   double IRU_Drift_Bias_X;
   double IRU_Drift_Bias_Y;
   double IRU_Drift_Bias_Z;
   };

struct UT1_TIME_PARAMETERS
   {
   int UT1_Year[180];
   char *UT1_Month[180];
   int UT1_Day[180];
   int UT1_Modified_Julian[180];
   float UT1_X[180];
   float UT1_Y[180];
   float UT1_UTC[180];
   };

struct ALI_SCALING_PARAMETERS
    {
    float PAN_Lmin_Lmax[2];
    float MS1P_Lmin_Lmax[2];
    float MS1_Lmin_Lmax[2];
    float MS2_Lmin_Lmax[2];
    float MS3_Lmin_Lmax[2];
    float MS4_Lmin_Lmax[2];
    float MS4P_Lmin_Lmax[2];
    float MS5P_Lmin_Lmax[2];
    float MS5_Lmin_Lmax[2];
    float MS7_Lmin_Lmax[2];
    };

struct HYP_SCALING_PARAMETERS
    {
    float Lmin_Lmax[2];
    };

struct ALI_MTF_COMPENSATION
    {
    double PAN_weights_along[5];
    double PAN_weights_across[5];
    double MS1P_weights_along[5];
    double MS1P_weights_across[5];
    double MS1_weights_along[5];
    double MS1_weights_across[5];
    double MS2_weights_along[5];
    double MS2_weights_across[5];
    double MS3_weights_along[5];
    double MS3_weights_across[5];
    double MS4_weights_along[5];
    double MS4_weights_across[5];
    double MS4P_weights_along[5];
    double MS4P_weights_across[5];
    double MS5P_weights_along[5];
    double MS5P_weights_across[5];
    double MS5_weights_along[5];
    double MS5_weights_across[5];
    double MS7_weights_along[5];
    double MS7_weights_across[5];
    };

struct HYP_MTF_COMPENSATION
    {
    double weights_along[5];
    double weights_across[5];
    };

/* the cpf stucture */
typedef struct CPF
   {
   struct FILE_ATTRIBUTES File_Attributes;
   struct EARTH_CONSTANTS Earth_Constants;
   struct ORBIT_PARAMETERS Orbit_Parameters;
   struct ALI_PARAMETERS ALI_Parameters;
   struct ALI_FOCAL_PLANE_PARAMETERS Focal_Plane_Parameters;
   struct HYP_SCAN_PARAMETERS Hyp_Scan_Parameters;
   struct HYP_FOCAL_PLANE_PARAMETERS Hyp_Focal_Plane_Parameters;
   struct ATTITUDE_PARAMETERS Attitude_Parameters;
   struct UT1_TIME_PARAMETERS UT1_Time_Parameters;
   struct ALI_SCALING_PARAMETERS Scaling_Parameters;
   struct ALI_MTF_COMPENSATION MTF_Compensation;
   struct HYP_SCALING_PARAMETERS Hyp_Scaling_Parameters;
   struct HYP_MTF_COMPENSATION Hyp_MTF_Compensation;
   }CPF;


/* prototype functions for reading the cpf */

int exx_read_cpf_param(
    void *ptr,  
    OBJDESC *cpf_parms, 
    xxx_ValType_TYPE xxx_type,
    int sz,
    char group[],
    char param_name[]);

int exx_get_earth_constants(
    struct CPF *cpf,    /* O: the structure of CPF info                 */
    OBJDESC *cpf_parms); /* I: params read from CPF, and to be
                              put in the cpf structure (lablib3.h)      */

int exx_get_file_attributes(
    struct CPF *cpf,    /* O: the structure of CPF info                 */
    OBJDESC *cpf_parms);  /* I: params read from CPF, and to be
                              put in the cpf structure (lablib3.h)      */

int exx_get_orbit_parameters(
    struct CPF *cpf,    /* O: the structure of CPF info                 */
    OBJDESC *cpf_parms);  /* I: params read from CPF, and to be
                              put in the cpf structure (lablib3.h)      */

int exx_get_ali_parameters(
    struct CPF *cpf,    /* O: the structure of CPF info                 */
    OBJDESC *cpf_parms);  /* I: params read from CPF, and to be
                              put in the cpf structure (lablib3.h)      */

int exx_get_ali_focal_plane_parameters(
    struct CPF *cpf,
    OBJDESC *cpf_parms);

int exx_get_attitude_parameters(
    struct CPF *cpf,
    OBJDESC *cpf_parms);

int exx_get_UT1_time_parameters(
    struct CPF *cpf,
    OBJDESC *cpf_parms);

int exx_get_ali_scaling_parameters(
    struct CPF *cpf,
    OBJDESC *cpf_parms);

int exx_get_ali_mtf_compensation(
    struct CPF *cpf,
    OBJDESC *cpf_parms);

CPF *exx_read_cpf(
    const char* cpf_pathname);

void exx_free_cpf(
    CPF *cpf);  /* I: pointer to CPF structure to free */

#endif

