#ifndef _EXX_GCP_RESIDUAL_H_
#define _EXX_GCP_RESIDUAL_H_

#include "xxx_const.h"
#include "xxx_structures.h"

/* GCP_RESIDUAL is GCP residual information for one point read from the 
   precision residual file */
typedef struct GCP_RESIDUAL {
    char   pt_id[PTIDLEN];	/* Ground Control Point ID 		*/
    int bandno;			/* Band number  of the systematic image */
    int    outlier_flag;	/* Outlier flag where 0=ok,1=outlier 	*/
    double pred_line;		/* Predicted line */
    double pred_samp;		/* predicted sample */
    double seconds;		/* Seconds of the day of the sensor time*/
    double lat;			/* Latitude of the GCP 			*/
    double lon;			/* Longitude of the GCP 		*/
    double height;		/* Height in meters of the GCP 		*/
    double delta;		/* Along-scan angle of the GCP look vector*/
    double va;			/* Residual along-scan error in meters	*/
    double vb;			/* Residual across-scan error in meters	*/
    double residual_x;		/* x projection residual in meters */
    double residual_y;		/* y projection residual in meters */
    } GCP_RESIDUAL;

typedef struct GCP_SOLUTION
    {
    VECTOR p_satpos;
    VECTOR p_satvel;
    VECTOR p_rpyangle;
    VECTOR p_rpyrate;
    double p_time;
    double p_year;
    double p_day;
    double p_second;
    double p_refroll;
    double p_refpitch;
    double p_refyaw;
    double p_cov_along[6][6];
    double p_cov_across[6][6];
    double p_rpy_sterr[6];
    double p_pos_sterr[6];
    } GCP_SOLUTION;

typedef struct NORMAL_STRUCT
    {
    double obs_mx[6][6];
    double obs_rgt[6];
    double apr_corr[6];
    double apr_wgt_par[6];
    double obs_num;
    double obs_wgt;
    double apr_wgt_fact;
    double obs_vwv;
    } NORMAL_STRUCT;

typedef struct GCP_RES_STATS	/* type is used for GCP residual stats */
{
    int  num_points;		/* number of points used to calculate the stats*/
    double mean_lat;		/* mean of the latitudes */
    double mean_lon;		/* mean of the longitudes */
    double mean_along_res;	/* mean of the along-scan residuals */
    double mean_across_res;	/* mean of the across_scan residuals */
    double rmse_along_res;	/* root-mean-square error of along-scan res. */
    double rmse_across_res;	/* root-mean-square error of across-scan res. */
    double sd_along_res;	/* standard deviation of along-scan res. */
    double sd_across_res;	/* standard deviation of across-scan res. */
    double corr_coeff;		/* correlation coeff. (along- vs across-scan) */
}GCP_RES_STATS;

#endif
