/******************************************************************************
NAME: exx_free_cpf

PURPOSE:
Free the memory allocated to the cpf structure.

DEVELOPMENT HISTORY:
$Log: exx_free_cpf.c,v $
Revision 1.1  2000/11/27 20:33:42  storey
Original version


RETURN VALUE:
Type = void

NOTES:
The only field in the cpf structure that needs to be free'ed is the
contents of the char pointer array UT1_Month since the xxx_ReadODL routine
allocates memory when an array of strings is read.

ALGORITHM REFERENCES:
none

******************************************************************************
                        Property of the U.S. Government
                            USGS EROS Data Center
******************************************************************************/

#include <stdlib.h>     /* free prototype */
#include "exx_cpf.h"    /* prototypes for the CPF routines */

void exx_free_cpf(
    CPF *cpf)  /* I: pointer to CPF structure to free */
{
int count;  /* loop counter */
int size;   /* size of array */

/* just return on NULL */
if (cpf == NULL) return;

/* get the number of char pointers in UT1_Month */
size = sizeof(cpf->UT1_Time_Parameters.UT1_Month)/sizeof(char *);

/* free the memory pointed to by the UT1_Month member of the CPF */
for (count = 0; count < size; count++)
    if (cpf->UT1_Time_Parameters.UT1_Month[count] != NULL)
        free(cpf->UT1_Time_Parameters.UT1_Month[count]);

/* free the CPF structure itself */
free(cpf);

}
