/******************************************************************************
NAME: exx_get_UT1_time_parameters

PURPOSE:
Read the cpf's UT1_time_parameters params one by one using GSFC's 
xxx_GetODLField, then stuff them into a structure.

DEVELOPMENT HISTORY:
$Log: exx_get_UT1_time_parameters.c,v $
Revision 1.1  2000/11/27 20:33:43  storey
Original version


RETURN VALUE:
Type = int
Value    Description
-----    -----------
  0      Successful completion (SUCCESS)
  1      Operation failed (ERROR)

NOTES:
This function assumes that the CPF has been opened already.

ALGORITHM REFERENCES:

******************************************************************************
                        Property of the U.S. Government
                           USGS EROS Data Center
******************************************************************************/
#include <stdio.h>
#include "xxx_Types.h"  /* include defines ERROR vairaibles (xxx_EDCOdl_type)*/
#include "xxx_LogStatus.h"
#include "exx_cpf.h"



int exx_get_UT1_time_parameters(
    CPF *cpf,	/* O: the structure of CPF info			*/
    OBJDESC *cpf_parms	/* I: params read from CPF, and to be
			      put in the cpf structure (lablib3.h)	*/
    )

{
/* VARS */
int status;			/* status of return from function	*/
int sz;				/* size of a variable			*/

/* GSFC CSC LIB VARS */
xxx_ValType_TYPE parm_type;	/* the type of param to read from CPF 
				     (xxx_ODL.h)			*/


/* **** UT1 TIME PARAMETERS **** */

/* retrieve the UT1_year */
parm_type = XXX_Long;
sz = sizeof(cpf->UT1_Time_Parameters.UT1_Year);
status = exx_read_cpf_param((cpf->UT1_Time_Parameters.UT1_Year), 
	cpf_parms, parm_type, sz, "UT1_TIME_PARAMETERS","UT1_Year");
if (status != SUCCESS) 
    {
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__, 
	"Error returned from exx_read_cpf_param.");
    return(ERROR);
    }

/* retrieve the UT1_Month */
parm_type = XXX_ArrayOfString;
sz = sizeof(cpf->UT1_Time_Parameters.UT1_Month);
status = exx_read_cpf_param(cpf->UT1_Time_Parameters.UT1_Month, 
	cpf_parms, parm_type, sz, "UT1_TIME_PARAMETERS","UT1_Month");
if (status != SUCCESS) 
    {
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__, 
	"Error returned from exx_read_cpf_param.");
    return(ERROR);
    }

/* retrieve the UT1_Day */
parm_type = XXX_Int;
sz = sizeof(cpf->UT1_Time_Parameters.UT1_Day);
status = exx_read_cpf_param((cpf->UT1_Time_Parameters.UT1_Day), 
	cpf_parms, parm_type, sz, "UT1_TIME_PARAMETERS","UT1_Day");
if (status != SUCCESS) 
    {
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__, 
	"Error returned from exx_read_cpf_param.");
    return(ERROR);
    }

/* retrieve the UT1_Modified_Julian */
parm_type = XXX_Long;
sz = sizeof(cpf->UT1_Time_Parameters.UT1_Modified_Julian);
status = exx_read_cpf_param((cpf->UT1_Time_Parameters.UT1_Modified_Julian), 
	cpf_parms, parm_type, sz, "UT1_TIME_PARAMETERS","UT1_Modified_Julian");
if (status != SUCCESS) 
    {
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__, 
	"Error returned from exx_read_cpf_param.");
    return(ERROR);
    }

/* retrieve the UT1_X */
parm_type = XXX_Float;
sz = sizeof(cpf->UT1_Time_Parameters.UT1_X);
status = exx_read_cpf_param((cpf->UT1_Time_Parameters.UT1_X), 
	cpf_parms, parm_type, sz, "UT1_TIME_PARAMETERS","UT1_X");
if (status != SUCCESS) 
    {
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__, 
	"Error returned from exx_read_cpf_param.");
    return(ERROR);
    }

/* retrieve the UT1_Y */
parm_type = XXX_Float;
sz = sizeof(cpf->UT1_Time_Parameters.UT1_Y);
status = exx_read_cpf_param((cpf->UT1_Time_Parameters.UT1_Y), 
	cpf_parms, parm_type, sz, "UT1_TIME_PARAMETERS","UT1_Y");
if (status != SUCCESS) 
    {
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__, 
	"Error returned from exx_read_cpf_param.");
    return(ERROR);
    }

/* retrieve the UT1_UTC */
parm_type = XXX_Float;
sz = sizeof(cpf->UT1_Time_Parameters.UT1_UTC);
status = exx_read_cpf_param((cpf->UT1_Time_Parameters.UT1_UTC), 
	cpf_parms, parm_type, sz, "UT1_TIME_PARAMETERS","UT1_UTC");
if (status != SUCCESS) 
    {
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__, 
	"Error returned from exx_read_cpf_param.");
    return(ERROR);
    }


return(SUCCESS);
}
