/******************************************************************************
NAME: exx_get_ali_mtf_compensation

PURPOSE:
Read the cpf's mtf_compensation params one by one using GSFC's 
xxx_GetODLField, then stuff them into a structure.

DEVELOPMENT HISTORY:
$Log: exx_get_ali_mtf_compensation.c,v $
Revision 1.1  2000/11/27 20:33:45  storey
Original version


RETURN VALUE:
Type = int
Value    Description
-----    -----------
  0      Successful completion (SUCCESS)
  1      Operation failed (ERROR)

NOTES:
This function assumes that the CPF has been opened already.

ALGORITHM REFERENCES:

******************************************************************************
                        Property of the U.S. Government
                            USGS EROS Data Center
******************************************************************************/
#include <stdio.h>
#include "xxx_Types.h"  /* include defines ERROR vairaibles (xxx_EDCOdl_type)*/
#include "xxx_LogStatus.h"
#include "exx_cpf.h"



int exx_get_ali_mtf_compensation(
    CPF *cpf,	/* O: the structure of CPF info			*/
    OBJDESC *cpf_parms	/* I: params read from CPF, and to be
			      put in the cpf structure (lablib3.h)	*/
    )

{
/* VARS */
int status;			/* status of return from function	*/
int sz;				/* size of a variable			*/

/* GSFC CSC LIB VARS */
xxx_ValType_TYPE parm_type;	/* the type of param to read from CPF 
				     (xxx_ODL.h)			*/


/* **** MTF COMPENSATION PARAMETERS **** */


/* retrieve the MTF compensation for MS1p along */
parm_type = XXX_Double;
sz = sizeof(cpf->MTF_Compensation.MS1P_weights_along);
status = exx_read_cpf_param(
	cpf->MTF_Compensation.MS1P_weights_along,
	cpf_parms, parm_type, sz, "ALI_MTF_COMPENSATION","MS1P_weights_along");
if (status != SUCCESS) 
    {
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__,
	"Error returned from exx_read_cpf_param");
    return(ERROR);
    }

/* retrieve the MTF compensation for MS1p across */
parm_type = XXX_Double;
sz = sizeof(cpf->MTF_Compensation.MS1P_weights_across);
status = exx_read_cpf_param(
	cpf->MTF_Compensation.MS1P_weights_across,
	cpf_parms, parm_type, sz, "ALI_MTF_COMPENSATION","MS1P_weights_across");
if (status != SUCCESS) 
    {
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__,
	"Error returned from exx_read_cpf_param");
    return(ERROR);
    }

/* retrieve the MTF compensation for MS1 along */
parm_type = XXX_Double;
sz = sizeof(cpf->MTF_Compensation.MS1_weights_along);
status = exx_read_cpf_param(
	cpf->MTF_Compensation.MS1_weights_along,
	cpf_parms, parm_type, sz, "ALI_MTF_COMPENSATION","MS1_weights_along");
if (status != SUCCESS) 
    {
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__,
	"Error returned from exx_read_cpf_param");
    return(ERROR);
    }

/* retrieve the MTF compensation for MS1 across */
parm_type = XXX_Double;
sz = sizeof(cpf->MTF_Compensation.MS1_weights_across);
status = exx_read_cpf_param(
	cpf->MTF_Compensation.MS1_weights_across,
	cpf_parms, parm_type, sz, "ALI_MTF_COMPENSATION","MS1_weights_across");
if (status != SUCCESS) 
    {
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__,
	"Error returned from exx_read_cpf_param");
    return(ERROR);
    }

/* retrieve the MTF compensation for MS2 along */
parm_type = XXX_Double;
sz = sizeof(cpf->MTF_Compensation.MS2_weights_along);
status = exx_read_cpf_param(
	cpf->MTF_Compensation.MS2_weights_along,
	cpf_parms, parm_type, sz, "ALI_MTF_COMPENSATION","MS2_weights_along");
if (status != SUCCESS) 
    {
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__,
	"Error returned from exx_read_cpf_param");
    return(ERROR);
    }

/* retrieve the MTF compensation for MS2 across */
parm_type = XXX_Double;
sz = sizeof(cpf->MTF_Compensation.MS2_weights_across);
status = exx_read_cpf_param(
	cpf->MTF_Compensation.MS2_weights_across,
	cpf_parms, parm_type, sz, "ALI_MTF_COMPENSATION","MS2_weights_across");
if (status != SUCCESS) 
    {
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__,
	"Error returned from exx_read_cpf_param");
    return(ERROR);
    }

/* retrieve the MTF compensation for MS3 along */
parm_type = XXX_Double;
sz = sizeof(cpf->MTF_Compensation.MS3_weights_along);
status = exx_read_cpf_param(
	cpf->MTF_Compensation.MS3_weights_along,
	cpf_parms, parm_type, sz, "ALI_MTF_COMPENSATION","MS3_weights_along");
if (status != SUCCESS) 
    {
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__,
	"Error returned from exx_read_cpf_param");
    return(ERROR);
    }

/* retrieve the MTF compensation for MS3 across */
parm_type = XXX_Double;
sz = sizeof(cpf->MTF_Compensation.MS3_weights_across);
status = exx_read_cpf_param(
	cpf->MTF_Compensation.MS3_weights_across,
	cpf_parms, parm_type, sz, "ALI_MTF_COMPENSATION","MS3_weights_across");
if (status != SUCCESS) 
    {
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__,
	"Error returned from exx_read_cpf_param");
    return(ERROR);
    }

/* retrieve the MTF compensation for MS4 along */
parm_type = XXX_Double;
sz = sizeof(cpf->MTF_Compensation.MS4_weights_along);
status = exx_read_cpf_param(
	cpf->MTF_Compensation.MS4_weights_along,
	cpf_parms, parm_type, sz, "ALI_MTF_COMPENSATION","MS4_weights_along");
if (status != SUCCESS) 
    {
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__,
	"Error returned from exx_read_cpf_param");
    return(ERROR);
    }

/* retrieve the MTF compensation for MS4 across */
parm_type = XXX_Double;
sz = sizeof(cpf->MTF_Compensation.MS4_weights_across);
status = exx_read_cpf_param(
	cpf->MTF_Compensation.MS4_weights_across,
	cpf_parms, parm_type, sz, "ALI_MTF_COMPENSATION","MS4_weights_across");
if (status != SUCCESS) 
    {
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__,
	"Error returned from exx_read_cpf_param");
    return(ERROR);
    }

/* retrieve the MTF compensation for MS4P along */
parm_type = XXX_Double;
sz = sizeof(cpf->MTF_Compensation.MS4P_weights_along);
status = exx_read_cpf_param(
	cpf->MTF_Compensation.MS4P_weights_along,
	cpf_parms, parm_type, sz, "ALI_MTF_COMPENSATION","MS4P_weights_along");
if (status != SUCCESS) 
    {
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__,
	"Error returned from exx_read_cpf_param");
    return(ERROR);
    }

/* retrieve the MTF compensation for MS4P across */
parm_type = XXX_Double;
sz = sizeof(cpf->MTF_Compensation.MS4P_weights_across);
status = exx_read_cpf_param(
	cpf->MTF_Compensation.MS4P_weights_across,
	cpf_parms, parm_type, sz, "ALI_MTF_COMPENSATION","MS4P_weights_across");
if (status != SUCCESS) 
    {
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__,
	"Error returned from exx_read_cpf_param");
    return(ERROR);
    }

/* retrieve the MTF compensation for MS5P along */
parm_type = XXX_Double;
sz = sizeof(cpf->MTF_Compensation.MS5P_weights_along);
status = exx_read_cpf_param(
	cpf->MTF_Compensation.MS5P_weights_along,
	cpf_parms, parm_type, sz, "ALI_MTF_COMPENSATION","MS5P_weights_along");
if (status != SUCCESS) 
    {
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__,
	"Error returned from exx_read_cpf_param");
    return(ERROR);
    }

/* retrieve the MTF compensation for MS5P across */
parm_type = XXX_Double;
sz = sizeof(cpf->MTF_Compensation.MS5P_weights_across);
status = exx_read_cpf_param(
	cpf->MTF_Compensation.MS5P_weights_across,
	cpf_parms, parm_type, sz, "ALI_MTF_COMPENSATION","MS5P_weights_across");
if (status != SUCCESS) 
    {
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__,
	"Error returned from exx_read_cpf_param");
    return(ERROR);
    }

/* retrieve the MTF compensation for MS5 along */
parm_type = XXX_Double;
sz = sizeof(cpf->MTF_Compensation.MS5_weights_along);
status = exx_read_cpf_param(
	cpf->MTF_Compensation.MS5_weights_along,
	cpf_parms, parm_type, sz, "ALI_MTF_COMPENSATION","MS5_weights_along");
if (status != SUCCESS) 
    {
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__,
	"Error returned from exx_read_cpf_param");
    return(ERROR);
    }

/* retrieve the MTF compensation for MS5 across */
parm_type = XXX_Double;
sz = sizeof(cpf->MTF_Compensation.MS5_weights_across);
status = exx_read_cpf_param(
	cpf->MTF_Compensation.MS5_weights_across,
	cpf_parms, parm_type, sz, "ALI_MTF_COMPENSATION","MS5_weights_across");
if (status != SUCCESS) 
    {
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__,
	"Error returned from exx_read_cpf_param");
    return(ERROR);
    }

/* retrieve the MTF compensation for MS7 along */
parm_type = XXX_Double;
sz = sizeof(cpf->MTF_Compensation.MS7_weights_along);
status = exx_read_cpf_param(
	cpf->MTF_Compensation.MS7_weights_along,
	cpf_parms, parm_type, sz, "ALI_MTF_COMPENSATION","MS7_weights_along");
if (status != SUCCESS) 
    {
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__,
	"Error returned from exx_read_cpf_param");
    return(ERROR);
    }

/* retrieve the MTF compensation for MS7 across */
parm_type = XXX_Double;
sz = sizeof(cpf->MTF_Compensation.MS7_weights_across);
status = exx_read_cpf_param(
	cpf->MTF_Compensation.MS7_weights_across,
	cpf_parms, parm_type, sz, "ALI_MTF_COMPENSATION","MS7_weights_across");
if (status != SUCCESS) 
    {
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__,
	"Error returned from exx_read_cpf_param");
    return(ERROR);
    }

/* retrieve the MTF compensation for PAN along */
parm_type = XXX_Double;
sz = sizeof(cpf->MTF_Compensation.PAN_weights_along);
status = exx_read_cpf_param(
	cpf->MTF_Compensation.PAN_weights_along,
	cpf_parms, parm_type, sz, "ALI_MTF_COMPENSATION","PAN_weights_along");
if (status != SUCCESS) 
    {
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__,
	"Error returned from exx_read_cpf_param");
    return(ERROR);
    }

/* retrieve the MTF compensation for PAN across */
parm_type = XXX_Double;
sz = sizeof(cpf->MTF_Compensation.PAN_weights_across);
status = exx_read_cpf_param(
	cpf->MTF_Compensation.PAN_weights_across,
	cpf_parms, parm_type, sz, "ALI_MTF_COMPENSATION","PAN_weights_across");
if (status != SUCCESS) 
    {
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__,
	"Error returned from exx_read_cpf_param");
    return(ERROR);
    }

return(SUCCESS);
}
