/*****************************************************************************
NAME: exx_get_ali_parameters

PURPOSE:
Read the cpf's scanner params one by one using GSFC's 
xxx_GetODLField, then stuff them into a structure.

DEVELOPMENT HISTORY:
$Log: exx_get_ali_parameters.c,v $
Revision 1.1  2000/11/27 20:33:46  storey
Original version


RETURN VALUE:
Type = int
Value    Description
-----    -----------
  0      Successful completion (SUCCESS)
  1      Operation failed (ERROR)

NOTES:
This function assumes that the CPF has been opened already.

ALGORITHM REFERENCES:

******************************************************************************
                        Property of the U.S. Government
                            USGS EROS Data Center
******************************************************************************/
#include <stdio.h>
#include "xxx_Types.h"  /* include defines ERROR vairaibles (xxx_EDCOdl_type)*/
#include "exx_cpf.h"
#include "xxx_LogStatus.h"

int exx_get_ali_parameters(
    CPF *cpf,	/* O: the structure of CPF info			*/
    OBJDESC *cpf_parms)	/* I: params read from CPF, and to be
			      put in the cpf structure (lablib3.h)	*/
{
/* VARS */
int status;			/* status of return from function	*/
int sz;				/* size of a variable			*/

/* GSFC CSC LIB VARS */
xxx_ValType_TYPE parm_type;	/* the type of param to read from CPF 
				     (xxx_ODL.h)			*/


/* **** ALI PARAMETERS **** */

/* retrieve the along-track IFOV for the multispectral bands */
parm_type = XXX_Float;
sz = sizeof(cpf->ALI_Parameters.IFOV_MS_along);
status = exx_read_cpf_param(&(cpf->ALI_Parameters.IFOV_MS_along),
        cpf_parms, parm_type, sz, "ALI_PARAMETERS","IFOV_MS_along");
if (status != SUCCESS) 
{
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__,
	"Error returned from exx_read_cpf_param");
    return(ERROR);
}

/* retrieve the across-track IFOV for the multispectral bands */
parm_type = XXX_Float;
sz = sizeof(cpf->ALI_Parameters.IFOV_MS_across);
status = exx_read_cpf_param(&(cpf->ALI_Parameters.IFOV_MS_across),
        cpf_parms, parm_type, sz, "ALI_PARAMETERS","IFOV_MS_across");
if (status != SUCCESS) 
{
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__,
	"Error returned from exx_read_cpf_param");
    return(ERROR);
}

/* retrieve the along-track IFOV for the panchromatic band */
parm_type = XXX_Float;
sz = sizeof(cpf->ALI_Parameters.IFOV_PAN_along);
status = exx_read_cpf_param(&(cpf->ALI_Parameters.IFOV_PAN_along),
        cpf_parms, parm_type, sz, "ALI_PARAMETERS","IFOV_PAN_along");
if (status != SUCCESS) 
{
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__,
	"Error returned from exx_read_cpf_param");
    return(ERROR);
}

/* retrieve the across-track IFOV for the panchromatic band */
parm_type = XXX_Float;
sz = sizeof(cpf->ALI_Parameters.IFOV_PAN_across);
status = exx_read_cpf_param(&(cpf->ALI_Parameters.IFOV_PAN_across),
        cpf_parms, parm_type, sz, "ALI_PARAMETERS","IFOV_PAN_across");
if (status != SUCCESS) 
{
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__,
	"Error returned from exx_read_cpf_param");
    return(ERROR);
}

return(SUCCESS);
}
