/******************************************************************************
NAME: exx_get_ali_scaling_parameters

PURPOSE:
Read the cpf's scaling_paramters params one by one using GSFC's 
xxx_GetODLField, then stuff them into a structure.

DEVELOPMENT HISTORY:
$Log: exx_get_ali_scaling_parameters.c,v $
Revision 1.2  2000/11/27 20:41:34  storey
Fixed module name (exx_get_ali_scaling_parameters).

Revision 1.1  2000/11/27 20:33:47  storey
Original version


RETURN VALUE:
Type = int
Value    Description
-----    -----------
  0      Successful completion (SUCCESS)
  1      Operation failed (ERROR)

NOTES:
This function assumes that the CPF has been opened already.

ALGORITHM REFERENCES:

******************************************************************************
                        Property of the U.S. Government
                            USGS EROS Data Center
******************************************************************************/
#include <stdio.h>
#include "xxx_Types.h"  /* include defines ERROR vairaibles (xxx_EDCOdl_type)*/
#include "xxx_LogStatus.h"
#include "exx_cpf.h"



int exx_get_ali_scaling_parameters(
    CPF *cpf,	/* O: the structure of CPF info			*/
    OBJDESC *cpf_parms	/* I: params read from CPF, and to be
			      put in the cpf structure (lablib3.h)	*/
    )

{
/* VARS */
int status;			/* status of return from function	*/
int sz;				/* size of a variable			*/

/* GSFC CSC LIB VARS */
xxx_ValType_TYPE parm_type;	/* the type of param to read from CPF 
				     (xxx_ODL.h)			*/


/* **** SCALING PARAMETERS **** */

/* retrieve the MS1P scaling params */
parm_type = XXX_Float;
sz = sizeof(cpf->Scaling_Parameters.MS1P_Lmin_Lmax);
status = exx_read_cpf_param(
	cpf->Scaling_Parameters.MS1P_Lmin_Lmax,
	cpf_parms, parm_type, sz, "ALI_SCALING_PARAMETERS","MS1P_Lmin_Lmax");
if (status != SUCCESS) 
{
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__,
	"Error returned from exx_read_cpf_param");
    return(ERROR);
}

/* retrieve the MS1 scaling params */
parm_type = XXX_Float;
sz = sizeof(cpf->Scaling_Parameters.MS1_Lmin_Lmax);
status = exx_read_cpf_param(
	cpf->Scaling_Parameters.MS1_Lmin_Lmax,
	cpf_parms, parm_type, sz, "ALI_SCALING_PARAMETERS","MS1_Lmin_Lmax");
if (status != SUCCESS) 
{
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__,
	"Error returned from exx_read_cpf_param");
    return(ERROR);
}

/* retrieve the MS2 scaling params */
parm_type = XXX_Float;
sz = sizeof(cpf->Scaling_Parameters.MS2_Lmin_Lmax);
status = exx_read_cpf_param(
	cpf->Scaling_Parameters.MS2_Lmin_Lmax,
	cpf_parms, parm_type, sz, "ALI_SCALING_PARAMETERS","MS2_Lmin_Lmax");
if (status != SUCCESS) 
{
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__,
	"Error returned from exx_read_cpf_param");
    return(ERROR);
}

/* retrieve the MS3 scaling params */
parm_type = XXX_Float;
sz = sizeof(cpf->Scaling_Parameters.MS3_Lmin_Lmax);
status = exx_read_cpf_param(
	cpf->Scaling_Parameters.MS3_Lmin_Lmax,
	cpf_parms, parm_type, sz, "ALI_SCALING_PARAMETERS","MS3_Lmin_Lmax");
if (status != SUCCESS) 
{
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__,
	"Error returned from exx_read_cpf_param");
    return(ERROR);
}

/* retrieve the MS4 scaling params */
parm_type = XXX_Float;
sz = sizeof(cpf->Scaling_Parameters.MS4_Lmin_Lmax);
status = exx_read_cpf_param(
	cpf->Scaling_Parameters.MS4_Lmin_Lmax,
	cpf_parms, parm_type, sz, "ALI_SCALING_PARAMETERS","MS4_Lmin_Lmax");
if (status != SUCCESS) 
{
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__,
	"Error returned from exx_read_cpf_param");
    return(ERROR);
}

/* retrieve the MS4P scaling params */
parm_type = XXX_Float;
sz = sizeof(cpf->Scaling_Parameters.MS4P_Lmin_Lmax);
status = exx_read_cpf_param(
	cpf->Scaling_Parameters.MS4P_Lmin_Lmax,
	cpf_parms, parm_type, sz, "ALI_SCALING_PARAMETERS","MS4P_Lmin_Lmax");
if (status != SUCCESS) 
{
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__,
	"Error returned from exx_read_cpf_param");
    return(ERROR);
}

/* retrieve the MS5P scaling params */
parm_type = XXX_Float;
sz = sizeof(cpf->Scaling_Parameters.MS5P_Lmin_Lmax);
status = exx_read_cpf_param(
	cpf->Scaling_Parameters.MS5P_Lmin_Lmax,
	cpf_parms, parm_type, sz, "ALI_SCALING_PARAMETERS","MS5P_Lmin_Lmax");
if (status != SUCCESS) 
{
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__,
	"Error returned from exx_read_cpf_param");
    return(ERROR);
}

/* retrieve the MS5 scaling params */
parm_type = XXX_Float;
sz = sizeof(cpf->Scaling_Parameters.MS5_Lmin_Lmax);
status = exx_read_cpf_param(
	cpf->Scaling_Parameters.MS5_Lmin_Lmax,
	cpf_parms, parm_type, sz, "ALI_SCALING_PARAMETERS","MS5_Lmin_Lmax");
if (status != SUCCESS) 
{
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__,
	"Error returned from exx_read_cpf_param");
    return(ERROR);
}

/* retrieve the MS7 scaling params */
parm_type = XXX_Float;
sz = sizeof(cpf->Scaling_Parameters.MS7_Lmin_Lmax);
status = exx_read_cpf_param(
	cpf->Scaling_Parameters.MS7_Lmin_Lmax,
	cpf_parms, parm_type, sz, "ALI_SCALING_PARAMETERS","MS7_Lmin_Lmax");
if (status != SUCCESS) 
{
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__,
	"Error returned from exx_read_cpf_param");
    return(ERROR);
}

/* retrieve the PAN scaling params */
parm_type = XXX_Float;
sz = sizeof(cpf->Scaling_Parameters.PAN_Lmin_Lmax);
status = exx_read_cpf_param(
	cpf->Scaling_Parameters.PAN_Lmin_Lmax,
	cpf_parms, parm_type, sz, "ALI_SCALING_PARAMETERS","PAN_Lmin_Lmax");
if (status != SUCCESS) 
{
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__,
	"Error returned from exx_read_cpf_param");
    return(ERROR);
}

return(SUCCESS);
}
