/******************************************************************************
NAME: exx_get_attitude_parameters

PURPOSE:
Read the cpf's attitude_paramters params one by one using GSFC's 
xxx_GetODLField, then stuff them into a structure.

DEVELOPMENT HISTORY:
$Log: exx_get_attitude_parameters.c,v $
Revision 1.2  2001/07/09 12:44:46  choate
Attitude Hyperion matrix

Revision 1.1  2000/11/27 20:33:48  storey
Original version


RETURN VALUE:
Type = int
Value    Description
-----    -----------
  0      Successful completion (SUCCESS)
  1      Operation failed (ERROR)

NOTES:
This function assumes that the CPF has been opened already.

ALGORITHM REFERENCES:

******************************************************************************
                        Property of the U.S. Government
                           USGS EROS Data Center
******************************************************************************/
#include <stdio.h>
#include "xxx_Types.h"  /* include defines ERROR vairaibles (xxx_EDCOdl_type)*/
#include "xxx_LogStatus.h"
#include "exx_cpf.h"



int exx_get_attitude_parameters(
    CPF *cpf,	/* O: the structure of CPF info			*/
    OBJDESC *cpf_parms	/* I: params read from CPF, and to be
			      put in the cpf structure (lablib3.h)	*/
    )

{
/* VARS */
int status;			/* status of return from function	*/
int sz;				/* size of a variable			*/

/* GSFC CSC LIB VARS */
xxx_ValType_TYPE parm_type;	/* the type of param to read from CPF 
				     (xxx_ODL.h)			*/


/* **** ATTITUDE PARAMETERS **** */

/* retrieve the IRU to attitude matrix */
parm_type = XXX_Sci_Not;
sz = sizeof(cpf->Attitude_Parameters.IRU_To_Attitude_Matrix);
status = exx_read_cpf_param((cpf->Attitude_Parameters.IRU_To_Attitude_Matrix), 
	cpf_parms, parm_type, sz, "ATTITUDE_PARAMETERS","IRU_To_Attitude_Matrix");
if (status != SUCCESS)
{
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__,
	"Error returned from exx_read_cpf_param.");
    return(ERROR);
}

/* retrieve the Attitude to ALI matrix */
parm_type = XXX_Sci_Not;
sz = sizeof(cpf->Attitude_Parameters.Attitude_To_ALI_Matrix);
status = exx_read_cpf_param((cpf->Attitude_Parameters.Attitude_To_ALI_Matrix), 
	cpf_parms, parm_type, sz, "ATTITUDE_PARAMETERS","Attitude_To_ALI_Matrix");
if (status != SUCCESS)
{
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__,
	"Error returned from exx_read_cpf_param.");
    return(ERROR);
}

/* retrieve the Attitude to ALI matrix */
parm_type = XXX_Sci_Not;
sz = sizeof(cpf->Attitude_Parameters.Attitude_To_HYP_Matrix);
status = exx_read_cpf_param((cpf->Attitude_Parameters.Attitude_To_HYP_Matrix),
        cpf_parms, parm_type, sz, "ATTITUDE_PARAMETERS","Attitude_To_HYP_Matrix");
if (status != SUCCESS)
{
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__,
        "Error returned from exx_read_cpf_param.");
    return(ERROR);
}

/* retrieve the IRU X Drift Bias */
parm_type = XXX_Float;
sz = sizeof(cpf->Attitude_Parameters.IRU_Drift_Bias_X);
status = exx_read_cpf_param(&(cpf->Attitude_Parameters.IRU_Drift_Bias_X), 
	cpf_parms, parm_type, sz, "ATTITUDE_PARAMETERS","IRU_Drift_Bias_X");
if (status != SUCCESS)
{
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__,
	"Error returned from exx_read_cpf_param.");
    return(ERROR);
}

/* retrieve the IRU Y Drift Bias */
parm_type = XXX_Float;
sz = sizeof(cpf->Attitude_Parameters.IRU_Drift_Bias_Y);
status = exx_read_cpf_param(&(cpf->Attitude_Parameters.IRU_Drift_Bias_Y), 
	cpf_parms, parm_type, sz, "ATTITUDE_PARAMETERS","IRU_Drift_Bias_Y");
if (status != SUCCESS)
{
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__,
	"Error returned from exx_read_cpf_param.");
    return(ERROR);
}

/* retrieve the IRU Z Drift Bias */
parm_type = XXX_Float;
sz = sizeof(cpf->Attitude_Parameters.IRU_Drift_Bias_Z);
status = exx_read_cpf_param(&(cpf->Attitude_Parameters.IRU_Drift_Bias_Z), 
	cpf_parms, parm_type, sz, "ATTITUDE_PARAMETERS","IRU_Drift_Bias_Z");
if (status != SUCCESS)
{
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__,
	"Error returned from exx_read_cpf_param.");
    return(ERROR);
}

return(SUCCESS);
}
