/*****************************************************************************
NAME: exx_get_earth_constants

PURPOSE:
Read the cpf's earth_constants params one by one using GSFC's 
xxx_GetODLField, then stuff them into a structure.

DEVELOPMENT HISTORY:
$Log: exx_get_earth_constants.c,v $
Revision 1.1  2000/11/27 20:33:49  storey
Original version


RETURN VALUE:
Type = int
Value    Description
-----    -----------
  0      Successful completion (SUCCESS)
  1      Operation failed (ERROR)

NOTES:
This function assumes that the CPF has been opened already.

ALGORITHM REFERENCES:

******************************************************************************
                        Property of the U.S. Government
                            USGS EROS Data Center
******************************************************************************/
#include <stdio.h>
#include "xxx_Types.h"  /* include defines ERROR vairaibles (xxx_EDCOdl_type)*/
#include "exx_cpf.h"

/* these don't exist yet...*/
#include "xxx_LogStatus.h"



int exx_get_earth_constants(
    CPF *cpf,	/* O: the structure of CPF info			*/
    OBJDESC *cpf_parms)	/* I: params read from CPF, and to be
			      put in the cpf structure (lablib3.h)	*/
{
/* VARS */
int status;			/* status of return from function	*/
int sz;				/* size of a variable			*/

/* GSFC CSC LIB VARS */
xxx_ValType_TYPE parm_type;	/* the type of param to read from CPF 
				     (xxx_ODL.h)			*/



/* **** EARTH CONSTANTS **** */

/* retrieve the name of the ellipsoid */
parm_type = XXX_String;
sz = sizeof(cpf->Earth_Constants.Ellipsoid_Name);
status = exx_read_cpf_param(&(cpf->Earth_Constants.Ellipsoid_Name),
        cpf_parms, parm_type, sz, "EARTH_CONSTANTS","Ellipsoid_Name");
if (status != SUCCESS) 
    {
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__,
	"Error returned from exx_read_cpf_param");
    return(ERROR);
    }

/* retrieve the Semi_Major_Axis */
parm_type = XXX_Double;
sz = sizeof(cpf->Earth_Constants.Semi_Major_Axis);
status = exx_read_cpf_param(&(cpf->Earth_Constants.Semi_Major_Axis),
        cpf_parms, parm_type, sz, "EARTH_CONSTANTS","Semi_Major_Axis");
if (status != SUCCESS) 
    {
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__,
	"Error returned from exx_read_cpf_param");
    return(ERROR);
    }

/* retrieve the Semi_Minor_Axis */
parm_type = XXX_Double;
sz = sizeof(cpf->Earth_Constants.Semi_Minor_Axis);
status = exx_read_cpf_param(&(cpf->Earth_Constants.Semi_Minor_Axis),
        cpf_parms, parm_type, sz, "EARTH_CONSTANTS","Semi_Minor_Axis");
if (status != SUCCESS) 
    {
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__,
	"Error returned from exx_read_cpf_param");
    return(ERROR);
    }

/* retrieve the Ellipticity */
parm_type = XXX_Double;
sz = sizeof(cpf->Earth_Constants.Ellipticity);
status = exx_read_cpf_param(&(cpf->Earth_Constants.Ellipticity),
        cpf_parms, parm_type, sz, "EARTH_CONSTANTS","Ellipticity");
if (status != SUCCESS) 
    {
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__,
	"Error returned from exx_read_cpf_param");
    return(ERROR);
    }

/* retrieve the Eccentricity */
parm_type = XXX_Double;
sz = sizeof(cpf->Earth_Constants.Eccentricity);
status = exx_read_cpf_param(&(cpf->Earth_Constants.Eccentricity),
        cpf_parms, parm_type, sz, "EARTH_CONSTANTS","Eccentricity");
if (status != SUCCESS) 
    {
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__,
	"Error returned from exx_read_cpf_param");
    return(ERROR);
    }

/* retrieve the Earth_Spin_Rate */
parm_type = XXX_Double;
sz = sizeof(cpf->Earth_Constants.Earth_Spin_Rate);
status = exx_read_cpf_param(&(cpf->Earth_Constants.Earth_Spin_Rate),
        cpf_parms, parm_type, sz, "EARTH_CONSTANTS","Earth_Spin_Rate");
if (status != SUCCESS) 
    {
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__,
	"Error returned from exx_read_cpf_param");
    return(ERROR);
    }

/* retrieve the Gravity_Constant */
parm_type = XXX_Double;
sz = sizeof(cpf->Earth_Constants.Gravity_Constant);
status = exx_read_cpf_param(&(cpf->Earth_Constants.Gravity_Constant),
        cpf_parms, parm_type, sz, "EARTH_CONSTANTS","Gravity_Constant");
if (status != SUCCESS) 
    {
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__,
	"Error returned from exx_read_cpf_param");
    return(ERROR);
    }

/* retrieve the J2_Earth_Model_Term */
parm_type = XXX_Double;
sz = sizeof(cpf->Earth_Constants.J2_Earth_Model_Term);
status = exx_read_cpf_param(&(cpf->Earth_Constants.J2_Earth_Model_Term),
        cpf_parms, parm_type, sz, "EARTH_CONSTANTS","J2_Earth_Model_Term");
if (status != SUCCESS) 
    {
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__,
	"Error returned from exx_read_cpf_param");
    return(ERROR);
    }

return(SUCCESS);
}
