/******************************************************************************
NAME: exx_get_file_attributes

PURPOSE:
Read the cpf's file_attributes params one by one using GSFC's 
xxx_GetODLField, then stuff them into a structure.

DEVELOPMENT HISTORY:
$Log: exx_get_file_attributes.c,v $
Revision 1.2  2000/11/28 13:06:10  storey
Changed name of field from IAS_CPF_File_Name to CPF_File_Name.

Revision 1.1  2000/11/27 20:33:50  storey
Original version


RETURN VALUE:
Type = int
Value    Description
-----    -----------
  0      Successful completion (SUCCESS)
  1      Operation failed (ERROR)

NOTES:
This function assumes that the CPF has been opened already.

ALGORITHM REFERENCES:

******************************************************************************
                        Property of the U.S. Government
                           USGS EROS Data Center
******************************************************************************/
#include <stdio.h>
#include "xxx_Types.h"  /* include defines ERROR vairaibles (xxx_EDCOdl_type)*/
#include "xxx_LogStatus.h"
#include "exx_cpf.h"



int exx_get_file_attributes(
    CPF *cpf,	/* O: the structure of CPF info			*/
    OBJDESC *cpf_parms)	/* I: params read from CPF, and to be
 			      put in the cpf structure (lablib3.h)	*/
{
/* VARS */
int status;			/* status of return from function	*/
int sz;				/* size of a variable			*/

/* GSFC CSC LIB VARS */
xxx_ValType_TYPE parm_type;	/* the type of param to read from CPF 
				     (xxx_ODL.h)			*/


/* **** FILE ATTRIBUTES **** */


/* retrieve the date that this cpf becomes effective */
parm_type = XXX_String;
sz = sizeof(cpf->File_Attributes.Effective_Date_Begin);
status = exx_read_cpf_param(cpf->File_Attributes.Effective_Date_Begin, 
	cpf_parms, parm_type, sz, "FILE_ATTRIBUTES","Effective_Date_Begin");
if (status != SUCCESS) 
    {
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__,
	"Error returned from exx_read_cpf_param");
    return(ERROR);
    }

/* retrieve the date that this cpf is not effective anymore */
parm_type = XXX_String;
sz = sizeof(cpf->File_Attributes.Effective_Date_End);
status = exx_read_cpf_param(cpf->File_Attributes.Effective_Date_End, 
	cpf_parms, parm_type, sz, "FILE_ATTRIBUTES","Effective_Date_End");
if (status != SUCCESS) 
    {
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__,
	"Error returned from exx_read_cpf_param");
    return(ERROR);
    }

/* retrieve the filename of the CPF */
parm_type = XXX_String;
sz = sizeof(cpf->File_Attributes.CPF_File_Name);
status = exx_read_cpf_param(cpf->File_Attributes.CPF_File_Name,
	cpf_parms, parm_type, sz, "FILE_ATTRIBUTES","CPF_File_Name");
if (status != SUCCESS) 
    {
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__,
	"Error returned from exx_read_cpf_param");
    return(ERROR);
    }


return(SUCCESS);
}
