/******************************************************************************
NAME: exx_get_hyp_mtf_compensation

PURPOSE:
Read the cpf's mtf_compensation params one by one using GSFC's 
xxx_GetODLField, then stuff them into a structure.

DEVELOPMENT HISTORY:
Original version


RETURN VALUE:
Type = int
Value    Description
-----    -----------
  0      Successful completion (SUCCESS)
  1      Operation failed (ERROR)

NOTES:
This function assumes that the CPF has been opened already.

ALGORITHM REFERENCES:

******************************************************************************
                        Property of the U.S. Government
                            USGS EROS Data Center
******************************************************************************/
#include <stdio.h>
#include "xxx_Types.h"  /* include defines ERROR vairaibles (xxx_EDCOdl_type)*/
#include "xxx_LogStatus.h"
#include "exx_cpf.h"

int exx_get_hyp_mtf_compensation(
    CPF *cpf,	/* O: the structure of CPF info			*/
    OBJDESC *cpf_parms	/* I: params read from CPF, and to be
			      put in the cpf structure (lablib3.h)	*/
    )

{
/* VARS */
int status;			/* status of return from function	*/
int sz;				/* size of a variable			*/

/* GSFC CSC LIB VARS */
xxx_ValType_TYPE parm_type;	/* the type of param to read from CPF 
				     (xxx_ODL.h)			*/


/* **** MTF COMPENSATION PARAMETERS **** */


/* retrieve the MTF compensation for along */
parm_type = XXX_Double;
sz = sizeof(cpf->Hyp_MTF_Compensation.weights_along);
status = exx_read_cpf_param(
	cpf->Hyp_MTF_Compensation.weights_along,
	cpf_parms, parm_type, sz, "HYP_MTF_COMPENSATION","weights_along");
if (status != SUCCESS) 
    {
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__,
	"Error returned from exx_read_cpf_param");
    return(ERROR);
    }

/* retrieve the MTF compensation for across */
parm_type = XXX_Double;
sz = sizeof(cpf->Hyp_MTF_Compensation.weights_across);
status = exx_read_cpf_param(
	cpf->Hyp_MTF_Compensation.weights_across,
	cpf_parms, parm_type, sz, "HYP_MTF_COMPENSATION","weights_across");
if (status != SUCCESS) 
    {
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__,
	"Error returned from exx_read_cpf_param");
    return(ERROR);
    }

return(SUCCESS);
}
