/*****************************************************************************
NAME: exx_get_hyp_scan_parameters

PURPOSE:
Read the cpf's scanner params one by one using GSFC's 
xxx_GetODLField, then stuff them into a structure.

DEVELOPMENT HISTORY:

RETURN VALUE:
Type = int
Value    Description
-----    -----------
  0      Successful completion (SUCCESS)
  1      Operation failed (ERROR)

NOTES:
This function assumes that the CPF has been opened already.

ALGORITHM REFERENCES:

******************************************************************************
                        Property of the U.S. Government
                            USGS EROS Data Center
******************************************************************************/
#include <stdio.h>
#include "xxx_Types.h"  /* include defines ERROR vairaibles (xxx_EDCOdl_type)*/
#include "exx_cpf.h"
#include "xxx_LogStatus.h"

int exx_get_hyp_scan_parameters(
    CPF *cpf,	/* O: the structure of CPF info			*/
    OBJDESC *cpf_parms)	/* I: params read from CPF, and to be
			      put in the cpf structure (lablib3.h)	*/
{
/* VARS */
int status;			/* status of return from function	*/
int sz;				/* size of a variable			*/

/* GSFC CSC LIB VARS */
xxx_ValType_TYPE parm_type;	/* the type of param to read from CPF 
				     (xxx_ODL.h)			*/


/* **** Hyperion scan PARAMETERS **** */

/* retrieve the along-track IFOV for the VNIR bands */
parm_type = XXX_Float;
sz = sizeof(cpf->Hyp_Scan_Parameters.IFOV_VNIR_Along);
status = exx_read_cpf_param(&(cpf->Hyp_Scan_Parameters.IFOV_VNIR_Along),
        cpf_parms, parm_type, sz, "HYP_PARAMETERS","IFOV_VNIR_Along");
if (status != SUCCESS) 
{
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__,
	"Error returned from exx_read_cpf_param");
    return(ERROR);
}

/* retrieve the across-track IFOV for the VNIR bands */
parm_type = XXX_Float;
sz = sizeof(cpf->Hyp_Scan_Parameters.IFOV_VNIR_Across);
status = exx_read_cpf_param(&(cpf->Hyp_Scan_Parameters.IFOV_VNIR_Across),
        cpf_parms, parm_type, sz, "HYP_PARAMETERS","IFOV_VNIR_Across");
if (status != SUCCESS)
{
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__,
        "Error returned from exx_read_cpf_param");
    return(ERROR);
}

/* retrieve the along-track IFOV for the SWIR bands */
parm_type = XXX_Float;
sz = sizeof(cpf->Hyp_Scan_Parameters.IFOV_SWIR_Along);
status = exx_read_cpf_param(&(cpf->Hyp_Scan_Parameters.IFOV_SWIR_Along),
        cpf_parms, parm_type, sz, "HYP_PARAMETERS","IFOV_SWIR_Along");
if (status != SUCCESS)
{
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__,
        "Error returned from exx_read_cpf_param");
    return(ERROR);
}

/* retrieve the across-track IFOV for the SWIR bands */
parm_type = XXX_Float;
sz = sizeof(cpf->Hyp_Scan_Parameters.IFOV_SWIR_Across);
status = exx_read_cpf_param(&(cpf->Hyp_Scan_Parameters.IFOV_SWIR_Across),
        cpf_parms, parm_type, sz, "HYP_PARAMETERS","IFOV_SWIR_Across");
if (status != SUCCESS)
{
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__,
        "Error returned from exx_read_cpf_param");
    return(ERROR);
}

/* retrieve the integration time */
parm_type = XXX_Float;
sz = sizeof(cpf->Hyp_Scan_Parameters.Integration_Time);
status = exx_read_cpf_param(&(cpf->Hyp_Scan_Parameters.Integration_Time),
        cpf_parms, parm_type, sz, "HYP_PARAMETERS","Integration_Time");
if (status != SUCCESS)
{
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__,
        "Error returned from exx_read_cpf_param");
    return(ERROR);
}

/* retrieve the frame rate */
parm_type = XXX_Float;
sz = sizeof(cpf->Hyp_Scan_Parameters.Frame_Rate);
status = exx_read_cpf_param(&(cpf->Hyp_Scan_Parameters.Frame_Rate),
        cpf_parms, parm_type, sz, "HYP_PARAMETERS","Frame_Rate");
if (status != SUCCESS)
{
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__,
        "Error returned from exx_read_cpf_param");
    return(ERROR);
}

return(SUCCESS);
}
