/******************************************************************************
NAME: exx_get_orbit_parameters

PURPOSE:
Read the cpf's orbit_parameters params one by one using GSFC's 
xxx_GetODLField, then stuff them into a structure.

DEVELOPMENT HISTORY:
$Log: exx_get_orbit_parameters.c,v $
Revision 1.1  2000/11/27 20:33:51  storey
Original version


RETURN VALUE:
Type = int
Value    Description
-----    -----------
  0      Successful completion (SUCCESS)
  1      Operation failed (ERROR)

NOTES:
This function assumes that the CPF has been opened already.

ALGORITHM REFERENCES:

******************************************************************************
                        Property of the U.S. Government
                           USGS EROS Data Center
******************************************************************************/
#include <stdio.h>
#include "xxx_Types.h"
#include "xxx_LogStatus.h"
#include "exx_cpf.h"



int exx_get_orbit_parameters(
    CPF *cpf,	/* O: the structure of CPF info			*/
    OBJDESC *cpf_parms)	/* I: params read from CPF, and to be
			      put in the cpf structure (lablib3.h)	*/
{
/* VARS */
int status;			/* status of return from function	*/
int sz;			/* size of a variable			*/

/* GSFC CSC LIB VARS */
xxx_ValType_TYPE parm_type;	/* the type of param to read from CPF 
				     (xxx_ODL.h)			*/



/* **** ORBIT PARAMETERS **** */

/* retrieve the time for sat to view earth once */
parm_type = XXX_Int;
sz = sizeof(cpf->Orbit_Parameters.WRS_Cycle_Days);
status = exx_read_cpf_param(&(cpf->Orbit_Parameters.WRS_Cycle_Days),
        cpf_parms, parm_type, sz, "ORBIT_PARAMETERS","WRS_Cycle_Days");
if (status != SUCCESS) 
    {
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__,
	"Error returned from exx_read_cpf_param");
    return(ERROR);
    }

/* retrieve the number of orbits/paths in a complete WRS cycle */
parm_type = XXX_Int;
sz = sizeof(cpf->Orbit_Parameters.WRS_Cycle_Orbits);
status = exx_read_cpf_param(&(cpf->Orbit_Parameters.WRS_Cycle_Orbits),
        cpf_parms, parm_type, sz, "ORBIT_PARAMETERS","WRS_Cycle_Orbits");
if (status != SUCCESS) 
    {
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__,
	"Error returned from exx_read_cpf_param");
    return(ERROR);
    }

/* retrieve the number of scenes/rows per orbit */
parm_type = XXX_Int;
sz = sizeof(cpf->Orbit_Parameters.Scenes_Per_Orbit);
status = exx_read_cpf_param(&(cpf->Orbit_Parameters.Scenes_Per_Orbit),
        cpf_parms, parm_type, sz, "ORBIT_PARAMETERS","Scenes_Per_Orbit");

/* retrieve the time in seconds to complete one orbit */
parm_type = XXX_Double;
sz = sizeof(cpf->Orbit_Parameters.Orbital_Period);
status = exx_read_cpf_param(&(cpf->Orbit_Parameters.Orbital_Period),
        cpf_parms, parm_type, sz, "ORBIT_PARAMETERS","Orbital_Period");
if (status != SUCCESS) 
    {
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__,
	"Error returned from exx_read_cpf_param");
    return(ERROR);
    }

/* retrieve the angular momentum in the orbit (meters squared/sec) */
parm_type = XXX_Double;
sz = sizeof(cpf->Orbit_Parameters.Angular_Momentum);
status = exx_read_cpf_param(&(cpf->Orbit_Parameters.Angular_Momentum),
        cpf_parms, parm_type, sz, "ORBIT_PARAMETERS","Angular_Momentum");
if (status != SUCCESS) 
    {
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__,
	"Error returned from exx_read_cpf_param");
    return(ERROR);
    }

/* retrieve the nominal dist in km from center of earth to spaceraft */
parm_type = XXX_Double;
sz = sizeof(cpf->Orbit_Parameters.Orbit_Radius);
status = exx_read_cpf_param(&(cpf->Orbit_Parameters.Orbit_Radius),
        cpf_parms, parm_type, sz, "ORBIT_PARAMETERS","Orbit_Radius");
if (status != SUCCESS) 
    {
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__,
	"Error returned from exx_read_cpf_param");
    return(ERROR);
    }

/* retrieve the nominal semi major axis of sat. orbit */
parm_type = XXX_Double;
sz = sizeof(cpf->Orbit_Parameters.Orbit_Semimajor_Axis);
status = exx_read_cpf_param(&(cpf->Orbit_Parameters.Orbit_Semimajor_Axis),
        cpf_parms, parm_type, sz, "ORBIT_PARAMETERS","Orbit_Semimajor_Axis");
if (status != SUCCESS) 
    {
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__,
	"Error returned from exx_read_cpf_param");
    return(ERROR);
    }

/* retrieve the nominal semi minor axis of sat. orbit */
parm_type = XXX_Double;
sz = sizeof(cpf->Orbit_Parameters.Orbit_Semiminor_Axis);
status = exx_read_cpf_param(&(cpf->Orbit_Parameters.Orbit_Semiminor_Axis),
        cpf_parms, parm_type, sz, "ORBIT_PARAMETERS","Orbit_Semiminor_Axis");
if (status != SUCCESS) 
    {
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__,
	"Error returned from exx_read_cpf_param");
    return(ERROR);
    }

/* retrieve the nominal eccentricity of the satellite's orbit */
parm_type = XXX_Double;
sz = sizeof(cpf->Orbit_Parameters.Orbit_Eccentricity);
status = exx_read_cpf_param(&(cpf->Orbit_Parameters.Orbit_Eccentricity),
        cpf_parms, parm_type, sz, "ORBIT_PARAMETERS","Orbit_Eccentricity");
if (status != SUCCESS) 
    {
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__,
	"Error returned from exx_read_cpf_param");
    return(ERROR);
    }

/* retrieve the inclination angle of the satellite */
parm_type = XXX_Double;
sz = sizeof(cpf->Orbit_Parameters.Inclination_Angle);
status = exx_read_cpf_param(&(cpf->Orbit_Parameters.Inclination_Angle),
        cpf_parms, parm_type, sz, "ORBIT_PARAMETERS","Inclination_Angle");
if (status != SUCCESS) 
    {
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__,
	"Error returned from exx_read_cpf_param");
    return(ERROR);
    }

/* retrieve the Arguement of perigee */
parm_type = XXX_Float;
sz = sizeof(cpf->Orbit_Parameters.Argument_Of_Perigee);
status = exx_read_cpf_param(&(cpf->Orbit_Parameters.Argument_Of_Perigee),
        cpf_parms, parm_type, sz, "ORBIT_PARAMETERS","Argument_Of_Perigee");
if (status != SUCCESS) 
    {
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__,
	"Error returned from exx_read_cpf_param");
    return(ERROR);
    }

/* retrieve the row corresponding to the earth's equator (descending) */
parm_type = XXX_Int;
sz = sizeof(cpf->Orbit_Parameters.Descending_Node_Row);
status = exx_read_cpf_param(&(cpf->Orbit_Parameters.Descending_Node_Row),
        cpf_parms, parm_type, sz, "ORBIT_PARAMETERS","Descending_Node_Row");
if (status != SUCCESS) 
    {
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__,
	"Error returned from exx_read_cpf_param");
    return(ERROR);
    }

/* retrieve the longitude, in degrees, West of the point at which path 1
   crossed the equator							*/
parm_type = XXX_Float;
sz = sizeof(cpf->Orbit_Parameters.Long_Path1_Row60);
status = exx_read_cpf_param(&(cpf->Orbit_Parameters.Long_Path1_Row60),
        cpf_parms, parm_type, sz, "ORBIT_PARAMETERS","Long_Path1_Row60");
if (status != SUCCESS) 
    {
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__,
	"Error returned from exx_read_cpf_param");
    return(ERROR);
    }

/* retrieve the minimum local solar time for descending node */
parm_type = XXX_String;
sz = sizeof(cpf->Orbit_Parameters.Descending_Node_Time_Min);
status = exx_read_cpf_param(cpf->Orbit_Parameters.Descending_Node_Time_Min,
        cpf_parms, parm_type, sz, "ORBIT_PARAMETERS","Descending_Node_Time_Min");
if (status != SUCCESS) 
    {
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__,
	"Error returned from exx_read_cpf_param");
    return(ERROR);
    }

/* retrieve the maximum local solar time for descending node */
parm_type = XXX_String;
sz = sizeof(cpf->Orbit_Parameters.Descending_Node_Time_Max);
status = exx_read_cpf_param(cpf->Orbit_Parameters.Descending_Node_Time_Max,
        cpf_parms, parm_type, sz, "ORBIT_PARAMETERS","Descending_Node_Time_Max");
if (status != SUCCESS) 
    {
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__,
	"Error returned from exx_read_cpf_param");
    return(ERROR);
    }

/* retrieve the rate in degrees/day that the orbital plane rotates w/ resp. to earth */
sz = sizeof(cpf->Orbit_Parameters.Nodal_Regression_Rate);
parm_type = XXX_Double;
status = exx_read_cpf_param(&(cpf->Orbit_Parameters.Nodal_Regression_Rate),
        cpf_parms, parm_type, sz, "ORBIT_PARAMETERS","Nodal_Regression_Rate");
if (status != SUCCESS) 
    {
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__,
	"Error returned from exx_read_cpf_param");
    return(ERROR);
    }

return(SUCCESS);
}
