/******************************************************************************
NAME: exx_read_cpf_param

PURPOSE:
Read the one param from CPF using GSFC's 
xxx_GetODLField, then stuff them into a ptr.

DEVELOPMENT HISTORY:
$Log: exx_read_cpf_param.c,v $
Revision 1.1  2000/11/27 20:33:53  storey
Original version


RETURN VALUE:
Type = int
Value    Description
-----    -----------
SUCCESS  Successful completion
ERROR	 Error reading CPF parameter

NOTES:
This function assumes that the CPF has been opened already.

ALGORITHM REFERENCES:

******************************************************************************
                        Property of the U.S. Government
                            USGS EROS Data Center
******************************************************************************/
#include <stdio.h>
#include "xxx_Types.h"
#include "xxx_LogStatus.h"
#include "xxx_const.h"
#include "exx_cpf.h"

int exx_read_cpf_param(
    void *ptr,			/* I: pointer to location to place fields */
    OBJDESC *cpf_parms,		/* I: ODL tree */
    xxx_ValType_TYPE XXX_TYPE,	/* I: type of values to retrieve */
    int sz,			/* I: size in bytes to be placed at ptr */
    char group[],		/* I: group/object name */
    char param_name[])		/* I: field to retrieve */
{
/* VARS */
int  status;			/* status of return from function	*/
int  array_count;		/* number of elements in an array	*/
char error_msg[STRLEN];		/* the error message returned from 
				   xxx_GetODLField 			*/
char errmsg[STRLEN];		/* the error message */

/* retrieve the date that this cpf becomes effective */
status = xxx_GetODLField(ptr, sz, XXX_TYPE, cpf_parms, group, 
	 param_name, error_msg, &array_count);

	 if (status != SUCCESS)
		{
		xxx_LogStatus("EXXLIB",__FILE__,__LINE__, error_msg);
		sprintf(errmsg,"Error Reading: %s.%s from CPF\n",group,param_name);
		xxx_LogStatus("EXXLIB",__FILE__,__LINE__, errmsg);
		return(ERROR);
		}

return(SUCCESS);
}
