/******************************************************************************
NAME:           hxx_findlos

PURPOSE:
Find the line of sight vector in sensor coordinates

DEVELOPMENT HISTORY:

RETURN VALUE:
    ERROR
    SUCCESS

NOTES:

ALGORITHM REFERENCES:
******************************************************************************
                        Property of the U.S. Government
                            USGS EROS Data Center
******************************************************************************/
#include "xxx_Types.h"
#include "hxx_hypmodel.h"

int hxx_findlos(
    HBandNumber band,	/*I: Input point band number		*/
    double detector,	/*I: Input point detector number	*/
    HYPMODEL *hmodel,	/*I: ALI model structure 		*/
    const CPF *cpf,	/*I: CPF data structure			*/
    VECTOR *losv) 	/*O: Line of sight vector               */
{
double norm_det;	/* Legendre normalized detector number  */
double x,		/* Look vector X coordinate		*/
       y,		/* Look vector Y coordinate		*/
       maglos;		/* LOS vector magnitude			*/
double ifov,		/* ALI IFOV (band dependent)		*/
       x_off,		/* Along-track detector offset		*/
       y_off;		/* Across-track detector offset		*/
VECTOR losvec;		/* Local LOS vector			*/
int ndet,		/* Nearest integer detector		*/
    even_odd;		/* Even/odd detector flag		*/

/* Do we need to check version for hyperion? */
norm_det = 2.0 * (hmodel->num_det-1-detector) / ((double)hmodel->num_det-1) - 1.0;
/**********************************************************************
norm_det = 2.0 * detector / ((double)hmodel->num_det-1) - 1.0;
**********************************************************************/

/* Evaluate the Legendre polynomials from the Hyperion model */

x = hmodel->coef_x[band][0] + hmodel->coef_x[band][1] * norm_det
  + hmodel->coef_x[band][2] * (1.5*norm_det*norm_det - 0.5);
y = hmodel->coef_y[band][0] + hmodel->coef_y[band][1] * norm_det
  + hmodel->coef_y[band][2] * (1.5*norm_det*norm_det - 0.5);

/* Construct the LOS vector */
losvec.x = x;
losvec.y = y;
losvec.z = 1.0;


/* Normalize for output */
maglos = xxx_norm( &losvec );
losv->x = losvec.x/maglos;
losv->y = losvec.y/maglos;
losv->z = losvec.z/maglos;

return(SUCCESS);
}
