/******************************************************************************
NAME:           hxx_findtime

PURPOSE:
Find the time into the scene given the line, sample, and band

RETURN VALUE:
   ERROR
   SUCCESS

NOTES:
   The input sample number is 0-relative and relative to the SCA

   The accounting for the odd/even detector offsets is based on the value
   of the dettype variable

ALGORITHM REFERENCES:
******************************************************************************
                        Property of the U.S. Government
                           USGS EROS Data Center
******************************************************************************/
#include "xxx_LogStatus.h"
#include "xxx_const.h"
#include "hxx_hypmodel.h"

int hxx_findtime (
	double line,		/*I: Line from start of the image (0-rel)*/
	double sample,		/*I: Sample from start of the line (0-rel)*/
	HBandNumber band,	/*I: Band in the image			*/
	HYPMODEL *hmodel,	/*I: ALI model structure		*/
	const CPF *cpf,		/*I: CPF constants			*/
	double *Time)		/*O: Time from start of image (seconds)	*/
{
int	detector,		/* Detector number			*/
	even_odd,		/* Flag for even/odd detector 
				     0 = even, 1 = odd			*/
	nom_shift,		/* CPF line shift for nominal detector	*/
	even_odd_shift[2];	/* Even/Odd detector shifts from CPF	*/

/* compute the time from the line number and frame rate */
*Time = line / hmodel->frame_rate;

return(SUCCESS);
}
