#include <stdio.h>
#include <math.h>
#include "hdf.h"
#include "mfhdf.h"
#include "hxx_hypmodel.h"

#define		TIME_CODE_SCALE		65536.0
#define		SYNC_TIME_SCALE		32e-6

/**********************************************************************************
NAME: get_time_code

PURPOSE: 
Using the Time Code image attribute find the times in seconds, subseconds,
offsets due to frame associated with Time Code and offset due to Sync Time.
From these values find the total start time of imagery.

RETURN VALUE:
Type = double

 *********************************************************************************/

double hxx_get_time_code(
int *time_code, 	/* I: Integer array representing time (coarse resolution) */
int *sync_time)		/* I: Integer array representing time (fine resolution)   */
{
   double sub_seconds;			/* Sub-seconds of time */
   double seconds;			/* Seconds of time */
   double time;				/* Total time in seconds */
   double delta_frame;			/* Offset due to current begin frame of Time Code */
   double delta_sync;			/* Offset due to current begin frame of Time Sync */

   double b1,b2,b3,b4;

   /*
    * Convert first "bits" to seconds  
    */
   
   seconds = time_code[4] 
           + (256.0*time_code[5]) 
           + (256.0*256.0*time_code[6]) 
           + (256.0*256.0*256.0*time_code[7]);

   sub_seconds = time_code[2]
               + (256.0*time_code[3]); 
   sub_seconds = sub_seconds / TIME_CODE_SCALE;

   /*
    * Find total time
    */
   delta_frame = time_code[0] * HYP_FRAME_RATE; /* Offset due to begin frame */
   delta_sync  = sync_time[2] * SYNC_TIME_SCALE; /* Offset due to Sync Time */
   time = seconds + sub_seconds - delta_frame - delta_sync; /* Total time */

   return(time);
}
