/* ******************************** */
/* if we haven't used this file yet */
/* ******************************** */
#ifndef _HXX_HYPMODEL_H
#define _HXX_HYPMODEL_H 1


/* ************* */
/* include files */
/* ************* */
#include "xxx_structures.h"
#include "xxx_const.h"
#include "exx_cpf.h"

/* ********* */
/* constants */
/* ********* */
#define HYP_NBAND		242	/* number of Hyperion bands */

/* ********** */
/* structures */
/* ********** */

typedef struct HYPMODEL
    {
    int  satnum;		 	/* Satellite number			    */
    int  l0r_sw_version[3];		/* Level 0R generation software version #   */
    double dce_time[3];		 	/* DCE image start time (year,day,sod)      */
    double acs_time[3];          	/* ACS reference time (year,day,sod)        */
    double integration_time;		/* Integration time			    */
    double frame_rate;			/* Frame rate			    	    */
    int  nephem;		 	/* Number of ephemeris points		    */
    EphemType ephem_type;		/* Ephemeris type:  ACS or GPS              */
    double ephem_time[3];        	/* Ephemeris reference time (year,day,sod)  */
    double ecr_satpos[3][NPCD]; 	/* Satellite position in ECR                */
    double ecr_satvel[3][NPCD];  	/* Satellite velocity in ECR                */
    double eci_satpos[3][NPCD];  	/* Satellite position in ECI                */
    double eci_satvel[3][NPCD];  	/* Satellite velocity in ECI                */
    double coef_x[HYP_NBAND][3];  	/* Along-track LOS Legendre coeffs     	    */
    double coef_y[HYP_NBAND][3];  	/* Across-track LOS Legendre coeffs         */
    int num_det;             		/* Number of detectors 			    */
    int  nrpy;                  	/* Number of attitude points                */
    double vnir_time_code;		/* VNIR Time code (MET)			    */
    double swir_time_code;		/* SWIR Time code (MET)			    */
    double rpy[NPCD][NAXIS];     	/* Roll, Pitch, and Yaw correction values   */
    double sc_roll;              	/* Space craft roll at acs_time             */
    double sc_pitch;             	/* Space craft pitch at acs_time            */
    double sc_yaw;               	/* Space craft yaw at acs_time              */
    CorrectionType correction_type; 	/* This flag specifies if the model is a
                                 	   systematic or precision model.	    */
    double p_roll;               	/* Precision roll value                     */
    double p_rrate;              	/* Precision roll rate value                */
    double p_pitch;              	/* Precision pitch value                    */
    double p_prate;              	/* Precision pitch rate value               */
    double p_yaw;                	/* Precision yaw value                      */
    double p_yrate;              	/* Precision yaw rate value                 */
    double p_x;                  	/* Precision along-track position           */
    double p_vx;                 	/* Precision along-track velocity           */
    double p_y;                  	/* Precision across-track position          */
    double p_vy;                 	/* Precision across-track velocity          */
    double p_z;                  	/* Precision altitude position              */
    double p_vz;                 	/* Precision altitude velocity              */
    double p_ref_time;           	/* Precision reference time                 */
    }HYPMODEL;

/********************************************/
/* Prototypes for the Hyperion model I/O routines. */
/********************************************/
int  axx_get_hypmodel (
  const char hypmodel_filename[STRLEN],	/* I: HDF input file name             */
  HYPMODEL *hypmodel);          	/* O: Hyperion model structure to be read  */

int  axx_put_hypmodel (
  const char hypmodel_filename[STRLEN],	/* I: HDF output file name              */
  const HYPMODEL *hypmodel);    	/* I: Hyperion model structure to be output  */

/*********************************************/
/* include files (more at beginning of file) */
/*********************************************/
#include "hxx_init_model.h"
#include "exx_init_model.h"
#include "hxx_forward_model.h"

#endif
