/* ******************************** */
/* if we haven't used this file yet */
/* ******************************** */
#ifndef _HXX_INIT_MODEL_H
#define _HXX_INIT_MODEL_H 1


/* ******************* */
/* other include files */
/* ******************* */
#include "xxx_HYPPCD.h"
#include "xxx_math.h"
#include "xxx_structures.h"
#include "exx_cpf.h"
#include "xxx_EDCOdl.h"
#include "xxx_ODL.h"
#include "lablib3.h"

/***********************/
/* attitude prototypes */
/***********************/
int hxx_align_attitude(
    HYPPCD *alipcd,
    HYPMODEL *hypmodel,
    CPF *cpf
    );

/* ************************************** */
/* the ephemeris prototypes for functions */
/* ************************************** */
int hxx_convert_ephem(
    struct EPHEM ephem[],/* IO:  Ephemeris information			*/
    double  xp,		 /* I: True pole postion in the mean pole coords system,
		            x-axis                            			*/
    double yp, 		 /* I: True pole position in the mean pole coords
                            system, y-axis                            		*/
    double ut1_utc, 	 /* I: Earth rotation angle difference ut1 - utc 	*/
    HYPMODEL *hypmodel     /* IO: ali model					*/
);

/* ************************************* */
/* the forward model prototype functions */
/* ************************************* */
int hxx_process_hyp_model(
    struct EPHEM *ephem,
    double xp,
    double yp,
    double ut1_utc,
    CPF *cpf,
    HYPPCD *pcd,
    HYPMODEL *hypmodel);

int hxx_out_of_pcd(
    int nlines,
    HYPPCD *pcd,
    HYPMODEL *hypmodel	
    );

void hxx_zero_amodel(
    HYPMODEL *amodel);

int hxx_update_amodel
(
   char *hmodel_name,
   char *prec_amodel_name,
   char *hk_dir,	/* I: Housekeeping data directory      */
   char *hk_root,	/* I: Housekeeping data file root name */
   char *cpf_name,
   double *reftime,
   VECTOR satpos,
   VECTOR satvel,
   VECTOR rpyangle,
   VECTOR rpyrate
);

int hxx_run_hypmodel
(
   char *hk_dir,	/* I: Housekeeping data directory      */
   char *hk_root,	/* I: Housekeeping data file root name */
   char *cpf_name,	/* I: CPF file name                    */
   int nlines,
   HYPMODEL *hypmodel,    /* I/O:  Hyperion model structure      */
   CPF **cpf		/* O: Loaded CPF structure             */
);

int hxx_init_iru(
    double *acs_time,
    double *ephem_time,
    int nephem,
    double *satpos_x,
    double *satpos_y,
    double *satpos_z,
    double *satvel_x,
    double *satvel_y,
    double *satvel_z,
    int niru,           /* I: Number of IRU points                       */
    double *irutime,    /* I: IRU data point times (from acs_time)       */
    VECTOR *iru         /* I/O: IRU data                                 */
    );

int exx_iru2acs(
    int niru,           /* I: number of IRU points */
    VECTOR *iru,        /* I/O: IRU data */
    const CPF *cpf      /* I: CPF structure */
    );

int hxx_sc_attitude(
    double *acs_time,
    double *ephem_time,
    int nephem,
    double *satpos_x,
    double *satpos_y,
    double *satpos_z,
    double *satvel_x,
    double *satvel_y,
    double *satvel_z,
    double epatime,     /* I: quaternion time relative to acs_time */
    double *quat,       /* I: quaternion vector */
    double *epa_roll,   /* O: roll calculated for epa point */
    double *epa_pitch,  /* IO: pitch calculated for epa point */
    double *epa_yaw     /* O: yaw calculated for epa point */
    );

void exx_movesat(
    int nephem,
    double *satpos_x,
    double *satpos_y,
    double *satpos_z,
    double *satvel_x,
    double *satvel_y,
    double *satvel_z,
    double dtime,       /*I: Delta time from the reference time */
    VECTOR *satpos,     /*O: New satellite position at "dtime" */
    VECTOR *satvel );   /*O: New satellite velocity at "dtime" */


/* **************************** */
/* the kalman filter prototypes */
/* **************************** */
int exx_kalman_smooth_iru(
    int  *natt,                 /* I: Number of atttitude measurements */
    double *atttime,            /* I: Attitude measurement times */
    VECTOR *att,                /* I: Satellite epa attitude data */
    int  *niru,                 /* I/O: Number of gyro measurements */
    double *irutime,            /* I/O: IRU measurement times */
    VECTOR *iru                 /* I/O: IRU measurements */
);

int exx_kalman_smooth_gps(
    double rawpos[3][NPCD],         /* ephemeris position telemetry data */
    double rawvel[3][NPCD],         /* ephemeris velocity telemetry data */
    int nephem,                     /* Number of ephemeris points        */
    double etime[NPCD],             /* ephemeris time telemetry          */
    const CPF *cpf,                 /* CPF structure */
    double smpos[3][NPCD],          /* Smoothed GPS position             */
    double smvel[3][NPCD]);         /* Smoothed GPS velocity             */

/* -----------------------------------------------------------------------
   GPS epoch time to Year, Day of Year, Seconds of Day Conversions
-------------------------------------------------------------------------- */
double exx_gpstime(
    double *time                /* I: Year, DOY, SOD */
);

void exx_gpstoyds(
    double gpstime,             /* I: Input GPS epoch time (seconds) */
    double *time                /* O: Output Year, DOY, SOD */
);

/* -----------------------------------------------------------------------
   Spacecraft Time Code to GPS Epoch Time Conversions
-------------------------------------------------------------------------- */
void exx_init_timecode(
    double gpstime,             /* I: Reference GPS epoch time */
    double timecode,            /* I: Reference spacecraft time code */
    double timecode_rate        /* I: GPS seconds per S/C clock second */
);

double exx_timecodetogps(
    double timecode             /* I: Spacecraft time code */
);

double exx_gpstotimecode(
    double gpstime              /* I: GPS epoch time */
);

#endif
