/*********************************************************************
NAME: hxx_los_coeffs

PURPOSE:
Extract the ALI LOS coefficients from the CPF and load them into
the ALI model structure.

DEVELOPMENT HISTORY:

RETURN VALUE:
Type = int
Value    Description
-----    -----------
SUCCESS  Successful completion 
ERROR    Operation failed

NOTES:

ALGORITHM REFERENCES:
******************************************************************************
		        Property of the U.S. Government
                             USGS EROS Data Center
******************************************************************************/
#include "xxx_Types.h"  	/* defines the ERROR variables */
#include "exx_cpf.h"  		/* cpf structure definition */
#include "hxx_hypmodel.h"	/* model structure definition */


int hxx_los_coeffs(
    HYPMODEL *hmodel, 		/* IO: The ALI model information */
    const CPF *cpf )		/* I:  The EO1 CPF structure */
{
int iband;

/********************************************************************
  Load the number of detectors 
********************************************************************/
hmodel->num_det = cpf->Hyp_Focal_Plane_Parameters.Number_Detectors;

for ( iband=0; iband<HYP_NBAND; iband++ )
{
  hmodel->coef_x[iband][0] = cpf->Hyp_Focal_Plane_Parameters.Along_LOS_Legendre[iband][0];
  hmodel->coef_x[iband][1] = cpf->Hyp_Focal_Plane_Parameters.Along_LOS_Legendre[iband][1];
  hmodel->coef_x[iband][2] = cpf->Hyp_Focal_Plane_Parameters.Along_LOS_Legendre[iband][2];
  hmodel->coef_y[iband][0] = cpf->Hyp_Focal_Plane_Parameters.Across_LOS_Legendre[iband][0];
  hmodel->coef_y[iband][1] = cpf->Hyp_Focal_Plane_Parameters.Across_LOS_Legendre[iband][1];
  hmodel->coef_y[iband][2] = cpf->Hyp_Focal_Plane_Parameters.Across_LOS_Legendre[iband][2];
}

return(SUCCESS);
}
