/******************************************************************************
  Hyperion forward model include file
   this file includes:
     function prototypes for the axxHYPModel library (forward model functions)
*******************************************************************************/
#ifndef _HXX_FORWARD_MODEL_H_
#define _HXX_FORWARD_MODEL_H_

#include <stdio.h>
#include <math.h>
#include <values.h>
#include "exx_cpf.h"
#include "xxx_structures.h"

#define ABER_CORRECTION -25.2267e-6     /* velocity aberration correction for
					   across_angle calculation     */

/* Hyperion FORWARD MODEL */
int hxx_attitude(const VECTOR *satpos, const VECTOR *satvel, 
	const VECTOR *sensor_los, double roll, double pitch, 
	double yaw, const CPF *cpf, VECTOR *plos, VECTOR *newlos);

int hxx_calc_position( double line, double sample, HBandNumber band,
	const CPF *cpf, HYPMODEL *alimodel, 
        VECTOR *satpos, VECTOR *satvel, double *Time, int *year, int *day);

int hxx_findlos(HBandNumber band, double sample, HYPMODEL *hmodel, 
        const CPF *cpf, VECTOR *losv );

int hxx_findatt(HYPMODEL *hmodel, double time, double *roll, double *pitch, 
        double *yaw);

int hxx_findtarpos(const VECTOR *satvec, const VECTOR *losv, double tarelev, 
	const CPF *cpf, double *tarlat, double *tarlong, double *tarrad);

int hxx_findtime(double line, double sample, HBandNumber band, HYPMODEL *hmodel, 
        const CPF *cpf, double *Time);

int hxx_forward_hyp(double line, double sample, HBandNumber band,
        double tarelev, const CPF *cpf, HYPMODEL *hmodel, double *tarlatd, 
        double *tarlong);

int hxx_getlos(double line, double sample, HBandNumber band, double tarelev, 
        const CPF *cpf, HYPMODEL *hmodel, double *tarlatc, double *tarlong, 
        double *tarrc, VECTOR *satpos, VECTOR *pertlos);

int hxx_run_hypmodel( char *hk_dir, char *hk_root, char *cpf_name, int nlines,
        HYPMODEL *hmodel, CPF **cpf);

#endif
