/******************************************************************************
  GPS include file
     this file includes:
       HDF include file
       GPS HDF IO constants and structures
       function prototypes for the IAS HDF IO library
*******************************************************************************/
#ifndef _HXX_HDF_IO_H_
#define _HXX_HDF_IO_H_

#include <string.h>
#include "mfhdf.h"
#include "xxx_const.h"

typedef struct HYPHDFIO {
    int32 file_id;              /* HDF file id                  */
    int32 group_id;             /* HDF group id                 */
    int32 sd_id;                /* HDF SDS interface id         */
    int32 sds_id;  		/* HDF SDS id per band          */
    char  sds_name[STRLEN];	/* HDF SDS name per band        */
    char  hdf_dir_name[STRLEN]; /* name of HDF directory file   */
    char  extdir[STRLEN];       /* external file directory      */
    } HYPHDFIO;

HYPHDFIO *hxx_open_1g_image(
  	int *nb,
  	int *nl,
  	int *ns,
  	DataType *dtype,
  	const char *image_filename,
  	AccessMode access_mode);

int hxx_close_1g_image (HYPHDFIO *hdfid);

int hxx_read_1g_image (
        const HYPHDFIO *hdfid,     /* I-HDF IO structure                   */
        int  sb,		/* I-Band to start reading (0-rel)	*/
        int  sl,                /* I-Line to start reading (0-rel)      */
        int  ss,                /* I-Sample to start reading (0-rel)    */
	int  nb,		/* I-Number of bands to read		*/
        int  nl,                /* I-Number of lines to read            */
        int  ns,                /* I-Number of samples to read          */
        int  subb,              /* I-Subsampling factor                 */
        int  subl,              /* I-Subsampling factor                 */
        int  subs,              /* I-Subsampling factor                 */
        void *data);            /* O-Data buffer                        */

int hxx_write_1g_image (
        const HYPHDFIO *hdfid,     /* I-HDF IO structure                   */
        int  sb,		/* I-Band to start reading (0-rel)	*/
        int  sl,                /* I-Line to start writing (0-rel)      */
        int  ss,                /* I-Sample to start writing (0-rel)    */
	int  nb,		/* I-Number of bands to write		*/
        int  nl,                /* I-Number of lines to write           */
        int  ns,                /* I-Number of samples to write         */
        int  subb,              /* I-Subsampling factor                 */
        int  subl,              /* I-Subsampling factor                 */
        int  subs,              /* I-Subsampling factor                 */
        void *data);            /* O-Data buffer                        */

int hxx_extract_window(
        int  image_nl,          /* I-# of lines in the 1G image         */
        int  image_ns,          /* I-# of samples in the 1G image       */
        HBandNumber band,       /* I-band number to read from           */
        DataType dtype,         /* I-data type of the band              */
        double pred_line,       /* I-center line of window to extract   */
        double pred_samp,       /* I-center sample of window to extract */
        int  win_size[2],       /* I-size of window to extract          */
        HYPHDFIO *img_ptr,      /* I-image pointer                      */
        float *img_buf);        /* O-Buffer read from 1G image          */

#endif
