/******************************************************************************
NAME:           hxx_create_band_name

PURPOSE:
hxx_create_band_name forms a band file name or band SDS name given the 
HDF file name.
******************************************************************************/
#include "hxx_hdf_io.h"	   /* HDF i/o function prototypes & SDS band tags */
#include "xxx_utility.h"   /* createname and splitname prototypes */

void hxx_create_band_name (
    const char *HDF_name,/* I: HDF name to form band file name from 	*/
    const char *type,	/* I: SDS or FILE name ("SDS" or "FILE")	*/
    char *band_name)  	/* O: constructed band file name 		*/
{
char directory[STRLEN]; /* directory name */
char root[STRLEN];      /* root file name */
char extension[STRLEN]; /* file extension */
char *temp_char_ptr;    /* temporary character pointer */
    
/* form the band file file name from the HDF name */
xxx_splitname(HDF_name, directory, root, extension);

/* find the "_HDF" part of the file name if it exits */
temp_char_ptr = strrchr(root,'_');
if (temp_char_ptr == NULL)
    {
    /* no "_HDF" so just append the appropriate band tag */
    /*  for SDS  name it is root.tag */
    /*  for file name it is root_tag.ext */
    if (strcmp(type,"SDS") == 0)
        {
        strcpy(extension, ".img");
        }
    else
	{
        strcat(root, ".img");
	}
    }
else
    {
    if (strcmp(temp_char_ptr, "_HDF") == 0)
        {
        /* replace the "_HDF" with the band substring */
        /*  for SDS  name it is root.tag so set a NULL where the _HDF is 
	    and set the extension to the tag */
        /*  for file name it is root_tag.ext so replace the HDF with the tag */
        if (strcmp(type,"SDS") == 0)
	    {
            *temp_char_ptr = '\0';
            strcat(extension, ".img");
	    }
        else
            {
            strcpy(temp_char_ptr+1, "img");
            }
        }
    else
        {
        /* doesn't end with "_HDF" so just append the band tag */
        /*  for SDS  name it is root.tag */
        /*  for file name it is root_tag.ext */
        if (strcmp(type,"SDS") == 0)
            {
            strcat(extension, ".img");
            }
        else
	    {
            strcat(root, ".img");
	    }
        }
    }

/* create band file name */
xxx_createname(directory, root, extension, band_name);

return;
}
