#include <stdio.h>
#include <unistd.h>
#include <sys/types.h>
#include <stddef.h>
#include <stdlib.h>
#include "xxx_EDCOdl.h"
#include "proj.h"
#include "hxx_grid.h"

struct ELEMENT_ATTRIBS
{
  char name[STRLEN]; /* Name of the field                               */
  int  dfnt;         /* DFNT type                                       */
  int  size;         /* Size (number of bytes) of an individual element */
  int  order;        /* Order/number of the elements                    */
};

/* Structure for the hxx_GRID_TYPE data.
 */
static struct ELEMENT_ATTRIBS grid_type_elements[] =
{
  {"gridname", DFNT_CHAR, sizeof (char), GRIDNAME_SIZE},
  {"nbands", DFNT_INT32, sizeof (int32), 1},
  {"bands", DFNT_INT32, sizeof (int32), HYP_NBANDS},
  {"dtype", DFNT_INT32, sizeof (int32), 1},
  {"correction_type", DFNT_INT32, sizeof (int32), 1},
  {"satnum", DFNT_INT32, sizeof (int32), 1},
  {"frame_type", DFNT_INT32, sizeof (int32), 1},
  {"units", DFNT_CHAR, sizeof (char), UNITS_SIZE},
  {"code", DFNT_INT32, sizeof (int32), 1},
  {"datum", DFNT_CHAR, sizeof (char), DATUM_SIZE},
  {"spheroid", DFNT_INT32, sizeof (int32), 1},
  {"zone", DFNT_INT32, sizeof (int32), 1},
  {"projprms", DFNT_DOUBLE, sizeof (double), PROJPRMS_SIZE},
  {"cornerX", DFNT_DOUBLE, sizeof (double), CORNERS_SIZE},
  {"cornerY", DFNT_DOUBLE, sizeof (double), CORNERS_SIZE},
  {"center_pt", DFNT_DOUBLE, sizeof (double), XY_SIZE},
  {"proj_center_pt", DFNT_DOUBLE, sizeof (double), XY_SIZE},
  {"rotate_angle", DFNT_DOUBLE, sizeof (double), 1},
  {"orient_angle", DFNT_DOUBLE, sizeof (double), 1}
};

/* Total number of ints, chars, and doubles in the grid_type_elements
 * structure.
 */
#define NUM_GRID_TYPE_INTS   (8 + HYP_NBANDS)
#define NUM_GRID_TYPE_CHARS  (GRIDNAME_SIZE + UNITS_SIZE + DATUM_SIZE)
#define NUM_GRID_TYPE_DBLS   (PROJPRMS_SIZE + 2*CORNERS_SIZE + 2*XY_SIZE + 2)

/* Structure for constants in the hxx_GRID_BAND_TYPE data.  This structure
 * will store all bands together.
 */
static struct ELEMENT_ATTRIBS constant_elements[] =
{
  {"bandno", DFNT_INT32, sizeof (int32), 1},
  {"grid_cell_sizeX", DFNT_INT32, sizeof (int32), 1},
  {"grid_cell_sizeY", DFNT_INT32, sizeof (int32), 1},
  {"pixsize", DFNT_DOUBLE, sizeof (double), 1},
  {"zspacing", DFNT_DOUBLE, sizeof (double), 1},
  {"lines", DFNT_INT32, sizeof (int32), 1},
  {"samps", DFNT_INT32, sizeof (int32), 1},
  {"ngrid_lines", DFNT_INT32, sizeof (int32), 1},
  {"ngrid_samps", DFNT_INT32, sizeof (int32), 1},
  {"nzplanes", DFNT_INT32, sizeof (int32), 1},
  {"zeroplane", DFNT_INT32, sizeof (int32), 1},
  {"degree", DFNT_INT32, sizeof (int32), 1}
};

/* Total number of ints, chars, and doubles in the constant_elements structure.
 */
#define NUM_CONSTANT_INTS 11
#define NUM_CONSTANT_DBLS 2

/* Structure for the in_lines data elements.
 */
static struct ELEMENT_ATTRIBS in_lines_elements[] =
{
  {"in_lines", DFNT_INT32, sizeof (int32), 0}
};

/* Structure for the in_samps data elements.
 */
static struct ELEMENT_ATTRIBS in_samps_elements[] =
{
  {"in_samps", DFNT_INT32, sizeof (int32), 0}
};

/* Structure for the out_lines and out_samps data elements.
 */
static struct ELEMENT_ATTRIBS out_ls_elements[] =
{
  {"out_lines", DFNT_DOUBLE, sizeof (double), 0},
  {"out_samps", DFNT_DOUBLE, sizeof (double), 0}
};

/* Structure for the poly_lines and poly_samps data elements.
 */
static struct ELEMENT_ATTRIBS poly_ls_elements[] =
{
  {"poly_lines", DFNT_DOUBLE, sizeof (double), 0},
  {"poly_samps", DFNT_DOUBLE, sizeof (double), 0}
};

/* Structure for the sattoproj and projtosat data elements.
 */
static struct ELEMENT_ATTRIBS sat_proj_elements[] =
{
  {"sattoproj", DFNT_DOUBLE, sizeof (double), 0},
  {"projtosat", DFNT_DOUBLE, sizeof (double), 0}
};

#define NGRIDTYPE (sizeof(grid_type_elements) / sizeof(struct ELEMENT_ATTRIBS))
#define NCONSTANT (sizeof(constant_elements) / sizeof(struct ELEMENT_ATTRIBS))

/* Define the number of SDSs, one for in_lines, in_samps, out_ls, deltas, poly_ls
 * and sat_proj.
 */
#define NUM_SDS 5

/******************************************************************************
NAME:           hxx_put_grid

PURPOSE:
Write the grid to an HDF file.

RETURN VALUE:   Type = int
Value    Description
-----    -----------
SUCCESS  Successful completion
ERROR    Operation failed

NOTES:   This function is a bridge routine to the actual hxx_write_grid
         support.  This function only flips SOM corner coorindates
         before they are sent to be written.  This allows a consistant
         user interface, yet allows the algorithms using SOM coodinates
         not to change.

ALGORITHM REFERENCES:
See hxx_grid.h for a description of the grid structure.

******************************************************************************/

int hxx_put_grid (
  char grid_filename[STRLEN],/* I: HDF output file name                     */
  hxx_GRID_TYPE *grid)       /* I: Grid structure to be output              */
{
double tmp;		     /* Temporary variable for flipping corner points*/

/*
  Flip the corner points for the SOM projection ONLY
*/
if (grid->code == SOM)
    {
    tmp = -grid->corners.upleft.y;
    grid->corners.upleft.y = grid->corners.upleft.x;
    grid->corners.upleft.x = tmp;

    tmp = -grid->corners.loleft.y;
    grid->corners.loleft.y = grid->corners.loleft.x;
    grid->corners.loleft.x = tmp;

    tmp = -grid->corners.upright.y;
    grid->corners.upright.y = grid->corners.upright.x;
    grid->corners.upright.x = tmp;

    tmp = -grid->corners.loright.y;
    grid->corners.loright.y = grid->corners.loright.x;
    grid->corners.loright.x = tmp;
    }

/*
  Write the grid structure to the HDF file
*/
return(hxx_write_grid(grid_filename,grid));
}

/******************************************************************************
NAME:           hxx_write_grid

PURPOSE:
Write the grid to an HDF file.

RETURN VALUE:   Type = int
Value    Description
-----    -----------
SUCCESS  Successful completion
ERROR    Operation failed

NOTES:
When writing the grid, the entire grid structure will be written.  The main
hxx_GRID_TYPE structure will be written first, to a vdata named hxx_GRID_TYPE.
These data elements are all of known size, except for the bands array which
may contain up to HYP_NBANDS data elements.  Technically this array contains
HYP_NBANDS elements, although the unused data values are =berror.  Thus
HYP_NBANDS data elements will always be written to this vdata for the bands
data element.  The name of this vdata will be hxx_GRID_TYPE.

The constant values for the hxx_GRID_BAND_TYPE structures for each band will
all be packed into another vdata named Constants.  This vdata will be for the
constant, non-pointer type values (e.g., bandno, grid_cell_size, pixsize,
lines, samps, ngrid_lines, ngrid_samps, and degree.)

To write out the pointers in the hxx_GRID_BAND_TYPE structures for each band,
the routine will loop through all the grid->nbands valid bands in the bands
data element of hxx_GRID_TYPE.  Each hxx_GRID_BAND_TYPE structure pointer will
be written to its own set of SDSs.  Each hxx_GRID_BAND_TYPE will require 6
SDSs to store its data.  The pointer-type values are dependent upon one or
more of the constant values (which have already been written).  The first
SDS will be for the in_lines, which contains ngrid_lines data values.  The
second SDS will be for the in_samps, which contains ngrid_samps data values.
The third SDS will be for the out_lines and out_samps pointers, which both
contain ngrid_lines*ngrid_samps data elements.  The fourth SDS will be for the 
detector deltas. The fifth will be for the poly_lines and poly_samps pointers, 
which both contain (degree+1)^2 data elements. The sixth SDS will be for the 
sattoproj and projtosat pointers, which both contain ngrid_lines * (ngrid_samps-1) 
data elements.  

Each of these SDSs will have their name be in_lines, in_samps, out_ls,
delta_ls_oe, poly_ls, and sat_proj concatinated with the band number.  Thus 
the in_lines pointer data for band b7 will be stored in an SDS with the 
name of in_lines7.

This routine will not free the grid or any of the pointers within the grid
after it is written.  The user is responsible for freeing the grid by calling
free_grid after returning from this routine.

ALGORITHM REFERENCES:
See hxx_grid.h for a description of the grid structure.

******************************************************************************/

int hxx_write_grid (
  char grid_filename[STRLEN],/* I: HDF output file name                     */
  hxx_GRID_TYPE *grid)       /* I: Grid structure to be output              */
{
  int  i, j, l,              /* Loop variables                      */
       currsize,             /* Current size of the data element    */
       currband,             /* Current band in the bands array     */
       ref,                  /* Current reference band value        */
       status,				 /* status of return from function		*/
       order,                /* Order of the current array          */
       num_ints,             /* Number of ints to malloc for  */
       num_doubles,          /* Number of doubles to malloc for     */
       num_chars;            /* Number of characters to malloc for  */
  int32 rank,                /* Rank of the dimsizes array.         */
        dimsizes[4],         /* Array of the dimension sizes.       */
        file_id,             /* DDR HDF file pointer                */
        sd_id,               /* DDR HDF file pointer for sds        */
        grid_type_vdata,     /* Vdata pointer (grid_type)           */
        constant_vdata,      /* Vdata pointer (constants)           */
        in_lines_sds,        /* SDS id pointer (in_lines)           */
        in_samps_sds,        /* SDS id pointer (in_samps)           */
        out_ls_sds,          /* SDS id pointer (out_lines/samps)    */
        delta_sds,           /* SDS id pointer (deltas)             */
        poly_ls_sds,         /* SDS id pointer (poly_lines/samps)   */
        sat_proj_sds,        /* SDS id pointer (sattoproj/projtosat)*/
        start[] = {0, 0, 0, 0}, /* Array of 0 starting positions.   */
        istat32,             /* Return values     DMW               */
        i32,                 /* Holder for integers                 */
        *iarr;               /* Holder for integer arrays           */
  intn  istatn;               /* Return values     DMW              */
  uint8 *databuf,            /* Data pointer                        */
        *pntr;               /* Current position in data pointer    */
  char str[STRLEN],          /* Holder for character strings        */
       name[STRLEN],         /* Name for the SDS                    */
       fields[NGRIDTYPE * XXX_ODL_FIELD_SIZE],
                             /* Packed string of field names of the
                                model.  The hxx_GRID_TYPE structure
                                is the largest so give enough space
                                for it.                             */
       ErrMsg[STRLEN];       /* Error message                       */
  double d,                  /* Holder for doubles                  */
         darr[PROJPRMS_SIZE], /* Holder for double arrays (largest size) */
         *darr2;             /* Holder for double arrays of unknown size */

  /***********************************************************
   * Open the HDF output file.
   */
  file_id = Hopen (grid_filename, DFACC_CREATE, 0);
  if (file_id == FAIL)
  {
    sprintf (ErrMsg, "Error opening new HDF grid file %s.", grid_filename);
    xxx_LogStatus ("HXXLIB", __FILE__, __LINE__, ErrMsg);
    return (ERROR);
  }
  Vstart (file_id);

  /***********************************************************
   * Assign a vdata id (grid_type) for write-only access.
   */
  grid_type_vdata = VSattach (file_id, -1, "w");
  if (grid_type_vdata == FAIL)
  {
    strcpy (ErrMsg,"Error creating grid_type Vdata.");
    xxx_LogStatus ("HXXLIB", __FILE__, __LINE__, ErrMsg);
    Vend (file_id);
    Hclose (file_id);
    return (ERROR);
  }
  VSsetname (grid_type_vdata, "hxx_GRID_TYPE");
  istatn = VSsetinterlace (grid_type_vdata, FULL_INTERLACE);
  if (istatn == FAIL)
  {
    strcpy (ErrMsg, "Error setting the interface.");
    xxx_LogStatus ("HXXLIB", __FILE__, __LINE__, ErrMsg);
    VSdetach (grid_type_vdata);
    Vend (file_id);
    Hclose (file_id);
    return (ERROR);
  }

  /* Define the fields for the grid (grid_type).
   */
  VSfdefine (grid_type_vdata, grid_type_elements[0].name,
            (int32)grid_type_elements[0].dfnt, 
            (int32)grid_type_elements[0].order);
  strcpy (fields, grid_type_elements[0].name);
  for (i = 1; i < NGRIDTYPE; i++)
  {
    VSfdefine (grid_type_vdata, grid_type_elements[i].name,
              (int32)grid_type_elements[i].dfnt, 
              (int32)grid_type_elements[i].order); 
    strcat (fields, ",");
    strcat (fields, grid_type_elements[i].name);
  }
  istatn = VSsetfields(grid_type_vdata, fields);
  if (istatn == FAIL)
  {
    strcpy (ErrMsg, "Error setting the grid_type field names.");
    xxx_LogStatus ("HXXLIB", __FILE__, __LINE__, ErrMsg);
    VSdetach (grid_type_vdata);
    Vend (file_id);
    Hclose (file_id);
    return (ERROR);
  }

  /* Malloc space for the integer array for the bands.
   */
  iarr = malloc (HYP_NBANDS * sizeof(int32));
  if (iarr == NULL)
  {
    strcpy (ErrMsg, "Error allocating memory.");
    xxx_LogStatus ("HXXLIB", __FILE__, __LINE__, ErrMsg);
    VSdetach (grid_type_vdata);
    Vend (file_id);
    Hclose (file_id);
    return (ERROR);
  }

  /* Set up the data buffer and its pointer for the grid_types.
   */
  num_ints = NUM_GRID_TYPE_INTS;
  num_chars = NUM_GRID_TYPE_CHARS;
  num_doubles = NUM_GRID_TYPE_DBLS;
  databuf = malloc (num_ints*sizeof(int32) + num_doubles*sizeof(double) +
                    num_chars*sizeof(char));
  if (databuf == NULL)
  {
    strcpy (ErrMsg, "Error allocating data buffer memory.");
    xxx_LogStatus ("HXXLIB", __FILE__, __LINE__, ErrMsg);
    VSdetach (grid_type_vdata);
    Vend (file_id);
    Hclose (file_id);
    free (iarr);
    return (ERROR);
  }
  pntr = databuf;
  j = 0;

  /* Write grid_type data to buffers.
   */
  currsize = grid_type_elements[j].order * grid_type_elements[j].size;
  strcpy (str, grid->gridname);
  memcpy (pntr, (void *)str, currsize);
  j++;
  pntr += currsize;

  currsize = grid_type_elements[j].order * grid_type_elements[j].size;
  i32 = grid->nbands;
  memcpy (pntr, (void *)&i32, currsize);
  j++;
  pntr += currsize;
  
  currsize = grid_type_elements[j].order * grid_type_elements[j].size;
  for (i = 0; i < HYP_NBANDS; i++)
    iarr[i] = grid->bands[i];
  memcpy (pntr, (void *)iarr, currsize);
  j++;
  pntr += currsize;

  currsize = grid_type_elements[j].order * grid_type_elements[j].size;
  i32 = grid->dtype;
  memcpy (pntr, (void *)&i32, currsize);
  j++;
  pntr += currsize;
  
  currsize = grid_type_elements[j].order * grid_type_elements[j].size;
  i32 = grid->correction_type;
  memcpy (pntr, (void *)&i32, currsize);
  j++;
  pntr += currsize;
  
  currsize = grid_type_elements[j].order * grid_type_elements[j].size;
  i32 = grid->satnum;
  memcpy (pntr, (void *)&i32, currsize);
  j++;
  pntr += currsize;
  
  currsize = grid_type_elements[j].order * grid_type_elements[j].size;
  i32 = grid->frame_type;
  memcpy (pntr, (void *)&i32, currsize);
  j++;
  pntr += currsize;
  
  currsize = grid_type_elements[j].order * grid_type_elements[j].size;
  strcpy (str, grid->units);
  memcpy (pntr, (void *)str, currsize);
  j++;
  pntr += currsize;

  currsize = grid_type_elements[j].order * grid_type_elements[j].size;
  i32 = grid->code;
  memcpy (pntr, (void *)&i32, currsize);
  j++;
  pntr += currsize;
  
  currsize = grid_type_elements[j].order * grid_type_elements[j].size;
  strcpy (str, grid->datum);
  memcpy (pntr, (void *)str, currsize);
  j++;
  pntr += currsize;

  currsize = grid_type_elements[j].order * grid_type_elements[j].size;
  i32 = grid->spheroid;
  memcpy (pntr, (void *)&i32, currsize);
  j++;
  pntr += currsize;
  
  currsize = grid_type_elements[j].order * grid_type_elements[j].size;
  i32 = grid->zone;
  memcpy (pntr, (void *)&i32, currsize);
  j++;
  pntr += currsize;
  
  currsize = grid_type_elements[j].order * grid_type_elements[j].size;
  for (i = 0; i < PROJPRMS_SIZE; i++)
    darr[i] = grid->projprms[i];
  memcpy (pntr, (void *)darr, currsize);
  j++;
  pntr += currsize;

  currsize = grid_type_elements[j].order * grid_type_elements[j].size;
  darr[0] = grid->corners.upleft.x;
  darr[1] = grid->corners.upright.x;
  darr[2] = grid->corners.loleft.x;
  darr[3] = grid->corners.loright.x;
  memcpy (pntr, (void *)darr, currsize);
  j++;
  pntr += currsize;

  currsize = grid_type_elements[j].order * grid_type_elements[j].size;
  darr[0] = grid->corners.upleft.y;
  darr[1] = grid->corners.upright.y;
  darr[2] = grid->corners.loleft.y;
  darr[3] = grid->corners.loright.y;
  memcpy (pntr, (void *)darr, currsize);
  j++;
  pntr += currsize;

  currsize = grid_type_elements[j].order * grid_type_elements[j].size;
  darr[0] = grid->center_pt.lat;
  darr[1] = grid->center_pt.lng;
  memcpy (pntr, (void *)darr, currsize);
  j++;
  pntr += currsize;

  currsize = grid_type_elements[j].order * grid_type_elements[j].size;
  darr[0] = grid->proj_center_pt.x;
  darr[1] = grid->proj_center_pt.y;
  memcpy (pntr, (void *)darr, currsize);
  j++;
  pntr += currsize;

  currsize = grid_type_elements[j].order * grid_type_elements[j].size;
  d = grid->rotate_angle;
  memcpy (pntr, (void *)&d, currsize);
  j++;
  pntr += currsize;

  currsize = grid_type_elements[j].order * grid_type_elements[j].size;
  d = grid->orient_angle;
  memcpy (pntr, (void *)&d, currsize);
  j++;
  pntr += currsize;

  /* Write buffer to grid_type Vdata.  Note there is only one record for
   * this Vdata.
   */
  istat32 = VSwrite (grid_type_vdata, databuf, 1, FULL_INTERLACE);
  if (istat32 == FAIL)
  {
    strcpy (ErrMsg, "Error writing the grid structure to the HDF file.");
    xxx_LogStatus ("HXXLIB", __FILE__, __LINE__, ErrMsg);
    free (databuf);
    free (iarr);
    VSdetach (grid_type_vdata);
    Vend (file_id);
    Hclose (file_id);
    return (ERROR);
  }

  /* Detach the Vdata.
   */
  VSdetach (grid_type_vdata);
  free (databuf);
  free (iarr);

  /******************************************************
   * Assign a vdata id (constants) for write-only access.
   */
  constant_vdata = VSattach (file_id, -1, "w");
  if (constant_vdata == FAIL)
  {
    strcpy (ErrMsg,"Error creating constants Vdata.");
    xxx_LogStatus ("HXXLIB", __FILE__, __LINE__, ErrMsg);
    Vend (file_id);
    Hclose (file_id);
    return (ERROR);
  }
  VSsetname (constant_vdata, "Constants");
  istatn = VSsetinterlace (constant_vdata, FULL_INTERLACE);
  if (istatn == FAIL)
  {
    strcpy (ErrMsg, "Error setting the interface.");
    xxx_LogStatus ("HXXLIB", __FILE__, __LINE__, ErrMsg);
    VSdetach (constant_vdata);
    Vend (file_id);
    Hclose (file_id);
    return (ERROR);
  }

  /* Define the fields for the grid (constants).
   */
  VSfdefine (constant_vdata, constant_elements[0].name,
            (int32)constant_elements[0].dfnt, 
            (int32)constant_elements[0].order);
  strcpy (fields, constant_elements[0].name);
  for (i = 1; i < NCONSTANT; i++)
  {
    VSfdefine (constant_vdata, constant_elements[i].name,
              (int32)constant_elements[i].dfnt, 
              (int32)constant_elements[i].order);
    strcat (fields, ",");
    strcat (fields, constant_elements[i].name);
  }
  istatn = VSsetfields(constant_vdata, fields);
  if (istatn == FAIL)
  {
    strcpy (ErrMsg, "Error setting the constants field names.");
    xxx_LogStatus ("HXXLIB", __FILE__, __LINE__, ErrMsg);
    VSdetach (constant_vdata);
    Vend (file_id);
    Hclose (file_id);
    return (ERROR);
  }

  /* Set up the data buffer and its pointer for the constants.
   */
  num_ints = NUM_CONSTANT_INTS;
  num_doubles = NUM_CONSTANT_DBLS;
  databuf = malloc ((num_ints*sizeof(int32) + num_doubles*sizeof(double)) * grid->nbands);
  if (databuf == NULL)
  {
    strcpy (ErrMsg, "Error allocating data buffer memory.");
    xxx_LogStatus ("HXXLIB", __FILE__, __LINE__, ErrMsg);
    VSdetach (constant_vdata);
    Vend (file_id);
    Hclose (file_id);
    return (ERROR);
  }
  pntr = databuf;

  for (i = 0, j = 0; i < grid->nbands; i++, j = 0)
  {
    /* Get the reference value of the current band.
     */
    ref = grid->bands[i];

    /* Write constants data to buffers.
     */
    currsize = constant_elements[j].order * constant_elements[j].size;
    i32 = grid->gridbands[ref].bandno;
    memcpy (pntr, (void *)&i32, currsize);
    j++;
    pntr += currsize;
     
    currsize = constant_elements[j].order * constant_elements[j].size;
    i32 = grid->gridbands[ref].grid_cell_size.line;
    memcpy (pntr, (void *)&i32, currsize);
    j++;
    pntr += currsize;
  
    currsize = constant_elements[j].order * constant_elements[j].size;
    i32 = grid->gridbands[ref].grid_cell_size.samp;
    memcpy (pntr, (void *)&i32, currsize);
    j++;
    pntr += currsize;
  
    currsize = constant_elements[j].order * constant_elements[j].size;
    d = grid->gridbands[ref].pixsize;
    memcpy (pntr, (void *)&d, currsize);
    j++;
    pntr += currsize;
    
    currsize = constant_elements[j].order * constant_elements[j].size;
    d = grid->gridbands[ref].zspacing;
    memcpy (pntr, (void *)&d, currsize);
    j++;
    pntr += currsize;
    
    currsize = constant_elements[j].order * constant_elements[j].size;
    i32 = grid->gridbands[ref].lines;
    memcpy (pntr, (void *)&i32, currsize);
    j++;
    pntr += currsize;
  
    currsize = constant_elements[j].order * constant_elements[j].size;
    i32 = grid->gridbands[ref].samps;
    memcpy (pntr, (void *)&i32, currsize);
    j++;
    pntr += currsize;
  
    currsize = constant_elements[j].order * constant_elements[j].size;
    i32 = grid->gridbands[ref].ngrid_lines;
    memcpy (pntr, (void *)&i32, currsize);
    j++;
    pntr += currsize;
  
    currsize = constant_elements[j].order * constant_elements[j].size;
    i32 = grid->gridbands[ref].ngrid_samps;
    memcpy (pntr, (void *)&i32, currsize);
    j++;
    pntr += currsize;

    currsize = constant_elements[j].order * constant_elements[j].size;
    i32 = grid->gridbands[ref].nzplanes;
    memcpy (pntr, (void *)&i32, currsize);
    j++;
    pntr += currsize;

    currsize = constant_elements[j].order * constant_elements[j].size;
    i32 = grid->gridbands[ref].zeroplane;
    memcpy (pntr, (void *)&i32, currsize);
    j++;
    pntr += currsize;

    currsize = constant_elements[j].order * constant_elements[j].size;
    i32 = grid->gridbands[ref].degree;
    memcpy (pntr, (void *)&i32, currsize);
    j++;
    pntr += currsize;
  }

  /* Write buffer to constants Vdata.  There are nbands of these records.
   */
  istat32 = VSwrite (constant_vdata, databuf, (int32)(grid->nbands), FULL_INTERLACE);
  if (istat32 == FAIL)
  {
    strcpy (ErrMsg, "Error writing the grid structure to the HDF file.");
    xxx_LogStatus ("HXXLIB", __FILE__, __LINE__, ErrMsg);
    free (databuf);
    VSdetach (constant_vdata);
    Vend (file_id);
    Hclose (file_id);
    return (ERROR);
  }

  /* Detach the Vdata.
   */
  VSdetach (constant_vdata);
  free (databuf);

  /* Close the file.
   */
  Vend (file_id);
  Hclose (file_id);

  /***********************************************************
   * Open the file for the SDS interface.
   */
  sd_id = SDstart (grid_filename, DFACC_RDWR);
  if (sd_id == FAIL)
  {
    sprintf (ErrMsg, "Error opening grid file %s for SDS.", grid_filename);
    xxx_LogStatus ("HXXLIB", __FILE__, __LINE__, ErrMsg);
    return (ERROR);
  }

  /**********************************************************************
   * Loop through the bands in the grid and place their pointer values in
   * separate SDSs for each band.
   */
  for (currband = 0; currband < grid->nbands; currband++)
  {
    /* Find the reference position.
     */
    ref = grid->bands[currband];

    /***********************************************************
     * Assign an sds (in_lines) for write-only access.
     */
    /* Create the SDS array.  The array will have 1 dimension.  It will be
     * for the size of in_lines (size = ngrid_lines).
     */
    rank = 1;
    dimsizes[0] = (int32)grid->gridbands[ref].ngrid_lines;
    status = sprintf (name, "In_lines_b%d", hxx_get_user_band (grid->bands[currband]));
    if (status < 1)
    {
      xxx_LogStatus ("HXXLIB", __FILE__, __LINE__, "Error in sprintf");
      return (ERROR);
    }
    in_lines_sds = SDcreate (sd_id, name, (int32)in_lines_elements[0].dfnt, rank, dimsizes);
    if (in_lines_sds == FAIL)
    {
      strcpy (ErrMsg, "Error creating in_lines_sds.");
      xxx_LogStatus ("HXXLIB", __FILE__, __LINE__, ErrMsg);
      SDendaccess (in_lines_sds);
      SDend (sd_id);
      return (ERROR);
    }
  
    /* Set up the data buffer and its pointer for the in_lines.
     */
    databuf = calloc (grid->gridbands[ref].ngrid_lines, sizeof(int32));
    if (databuf == NULL)
    {
      strcpy (ErrMsg, "Error allocating data buffer memory.");
      xxx_LogStatus ("HXXLIB", __FILE__, __LINE__, ErrMsg);
      SDendaccess (in_lines_sds);
      SDend (sd_id);
      return (ERROR);
    }
    pntr = databuf;
  
    /* Write the in_lines data to buffers.
     */
    iarr = calloc (grid->gridbands[ref].ngrid_lines, sizeof(int32));
    if (iarr == NULL)
    {
      strcpy (ErrMsg, "Error allocating int array memory.");
      xxx_LogStatus ("HXXLIB", __FILE__, __LINE__, ErrMsg);
      free (databuf);
      SDendaccess (in_lines_sds);
      SDend (sd_id);
      return (ERROR);
    }
  
    for (j = 0; j < grid->gridbands[ref].ngrid_lines; j++)
      iarr[j] = grid->gridbands[ref].in_lines[j];
    memcpy (pntr, (void *)iarr, grid->gridbands[ref].ngrid_lines * in_lines_elements[0].size);
    free (iarr);

    /* Write buffer to in_lines SDS.
     */
    istatn = SDwritedata (in_lines_sds, start, NULL, dimsizes, (VOIDP)databuf);
    if (istatn == FAIL)
    {
      strcpy (ErrMsg, "Error writing the in_lines to the HDF file.");
      xxx_LogStatus ("HXXLIB", __FILE__, __LINE__, ErrMsg);
      free (databuf);
      SDendaccess (in_lines_sds);
      SDend (sd_id);
      return (ERROR);
    }

    /* Terminate access to the SDS.
     */
    SDendaccess (in_lines_sds);
    free (databuf);

    /***********************************************************
     * Assign an sds (in_samps) for write-only access.
     */
    /* Create the SDS array.  The array will have 1 dimension.  It will be
     * for the size of in_samps (size = ngrid_samps).
     */
    rank = 1;
    dimsizes[0] = (int32)grid->gridbands[ref].ngrid_samps;
    status = sprintf (name, "In_samps_b%d", hxx_get_user_band (grid->bands[currband]));
    if (status < 1)
    {
       xxx_LogStatus ("HXXLIB", __FILE__, __LINE__, "Error in sprintf");
       return (ERROR);
    }
    in_samps_sds = SDcreate (sd_id, name, in_samps_elements[0].dfnt, rank, dimsizes);
    if (in_samps_sds == FAIL)
    {
      strcpy (ErrMsg, "Error creating in_samps_sds.");
      xxx_LogStatus ("HXXLIB", __FILE__, __LINE__, ErrMsg);
      SDendaccess (in_samps_sds);
      SDend (sd_id);
      return (ERROR);
    }

    /* Set up the data buffer and its pointer for the in_samps.
     */
    databuf = calloc (grid->gridbands[ref].ngrid_samps, sizeof(int32));
    if (databuf == NULL)
    {
      strcpy (ErrMsg, "Error allocating data buffer memory.");
      xxx_LogStatus ("HXXLIB", __FILE__, __LINE__, ErrMsg);
      SDendaccess (in_samps_sds);
      SDend (sd_id);
      return (ERROR);
    }
    pntr = databuf;

    /* Write the in_samps data to buffers.
     */
    iarr = calloc (grid->gridbands[ref].ngrid_samps, sizeof(int32));
    if (iarr == NULL)
    {
      strcpy (ErrMsg, "Error allocating int array memory.");
      xxx_LogStatus ("HXXLIB", __FILE__, __LINE__, ErrMsg);
      free (databuf);
      SDendaccess (in_samps_sds);
      SDend (sd_id);
      return (ERROR);
    }

    for (j = 0; j < grid->gridbands[ref].ngrid_samps; j++)
      iarr[j] = grid->gridbands[ref].in_samps[j];
    memcpy (pntr, (void *)iarr, grid->gridbands[ref].ngrid_samps * in_samps_elements[0].size);
    free (iarr);

    /* Write buffer to in_samps SDS.
     */
    istatn = SDwritedata (in_samps_sds, start, NULL, dimsizes, (VOIDP)databuf);
    if (istatn == FAIL)
    {
      strcpy (ErrMsg, "Error writing the in_samps to the HDF file.");
      xxx_LogStatus ("HXXLIB", __FILE__, __LINE__, ErrMsg);
      free (databuf);
      SDendaccess (in_samps_sds);
      SDend (sd_id);
      return (ERROR);
    }

    /* Terminate access to the SDS.
     */
    SDendaccess (in_samps_sds);
    free (databuf);

    /***********************************************************
     * Assign an sds (out_lines/samps) for write-only access.
     */
    /* Create the SDS array.  The array will have 2 dimensions.  One will be
     * for the out_lines and out_samps themselves and the other will be
     * the size of out_lines/out_samps (size = ngrid_lines * ngrid_samps).
     */
    rank = 2;
    dimsizes[0] = 2;
    dimsizes[1] = (int32)grid->gridbands[ref].nzplanes *
                  (int32)grid->gridbands[ref].ngrid_lines * 
                  (int32)grid->gridbands[ref].ngrid_samps;
    status = sprintf (name, "Out_ls_b%d", hxx_get_user_band (grid->bands[currband]));
    if (status < 1)
    {
       xxx_LogStatus ("HXXLIB", __FILE__, __LINE__, "Error in sprintf");
       return (ERROR);
    }
    out_ls_sds = SDcreate (sd_id, name, out_ls_elements[0].dfnt, rank, dimsizes);
    if (out_ls_sds == FAIL)
    {
      strcpy (ErrMsg, "Error creating out_ls_sds.");
      xxx_LogStatus ("HXXLIB", __FILE__, __LINE__, ErrMsg);
      SDendaccess (out_ls_sds);
      SDend (sd_id);
      return (ERROR);
    }

    /* Set up the data buffer and its pointer for the out_lines/samps.
     */
    databuf = calloc (2 * grid->gridbands[ref].nzplanes *
                      grid->gridbands[ref].ngrid_lines *
                      grid->gridbands[ref].ngrid_samps, sizeof(double));
    if (databuf == NULL)
    {
      strcpy (ErrMsg, "Error allocating data buffer memory.");
      xxx_LogStatus ("HXXLIB", __FILE__, __LINE__, ErrMsg);
      SDendaccess (out_ls_sds);
      SDend (sd_id);
      return (ERROR);
    }
    pntr = databuf;

    /* Write the out_lines/samps data to buffers.
     */
    darr2 = calloc (grid->gridbands[ref].nzplanes *
                    grid->gridbands[ref].ngrid_lines *
                    grid->gridbands[ref].ngrid_samps, sizeof(double));
    if (darr2 == NULL)
    {
      strcpy (ErrMsg, "Error allocating double array memory.");
      xxx_LogStatus ("HXXLIB", __FILE__, __LINE__, ErrMsg);
      free (databuf);
      SDendaccess (out_ls_sds);
      SDend (sd_id);
      return (ERROR);
    }

    order = grid->gridbands[ref].nzplanes * grid->gridbands[ref].ngrid_lines * 
            grid->gridbands[ref].ngrid_samps;
    for (j = 0; j < order; j++)
       darr2[j] = grid->gridbands[ref].out_lines[j];
    memcpy (pntr, (void *)darr2, order*out_ls_elements[0].size);
    pntr += order*out_ls_elements[0].size;

    for (j = 0; j < order; j++)
       darr2[j] = grid->gridbands[ref].out_samps[j];
    memcpy (pntr, (void *)darr2, order*out_ls_elements[1].size);
    pntr += order*out_ls_elements[1].size;
    free (darr2);

    /* Write buffer to out_lines/samps SDS.
     */
    istatn = SDwritedata (out_ls_sds, start, NULL, dimsizes, (VOIDP)databuf);
    if (istatn == FAIL)
    {
      strcpy (ErrMsg, "Error writing the grid structure to the HDF file.");
      xxx_LogStatus ("HXXLIB", __FILE__, __LINE__, ErrMsg);
      free (databuf);
      SDendaccess (out_ls_sds);
      SDend (sd_id);
      return (ERROR);
    }

    /* Terminate access to the SDS.
     */
    SDendaccess (out_ls_sds);
    free (databuf);
  
    /***********************************************************
     * Assign an sds (poly_lines/samps) for write-only access.
     */
    /* Create the SDS array.  The array will have 2 dimensions.  One will be
     * for the poly_lines and poly_samps themselves and the other will be
     * the size of poly_lines/poly_samps (size = (degree+1) * (degree+1)).
     */
    rank = 2;
    dimsizes[0] = 2;
    dimsizes[1] = (int32)grid->gridbands[ref].nzplanes *
                  (int32)(grid->gridbands[ref].degree+1) * 
                  (int32)(grid->gridbands[ref].degree+1);
    status = sprintf (name, "Poly_ls_b%d", hxx_get_user_band (grid->bands[currband]));
    if (status < 1)
    {
       xxx_LogStatus ("HXXLIB", __FILE__, __LINE__, "Error in sprintf");
       return (ERROR);
    }
    poly_ls_sds = SDcreate (sd_id, name, poly_ls_elements[0].dfnt, rank, dimsizes);
    if (poly_ls_sds == FAIL)
    {
      strcpy (ErrMsg, "Error creating poly_ls_sds.");
      xxx_LogStatus ("HXXLIB", __FILE__, __LINE__, ErrMsg);
      SDendaccess (poly_ls_sds);
      SDend (sd_id);
      return (ERROR);
    }

    /* Set up the data buffer and its pointer for the poly_lines/samps.
     */
    databuf = calloc (2 * grid->gridbands[ref].nzplanes * 
                     (grid->gridbands[ref].degree+1) *
                     (grid->gridbands[ref].degree+1), sizeof(double));
    if (databuf == NULL)
    {
      strcpy (ErrMsg, "Error allocating data buffer memory.");
      xxx_LogStatus ("HXXLIB", __FILE__, __LINE__, ErrMsg);
      SDendaccess (poly_ls_sds);
      SDend (sd_id);
      return (ERROR);
    }
    pntr = databuf;

    /* Write the poly_lines/samps data to buffers.
     */
    darr2 = calloc(grid->gridbands[ref].nzplanes * (grid->gridbands[ref].degree+1) *
                   (grid->gridbands[ref].degree+1), sizeof(double));
    if (darr2 == NULL)
    {
      strcpy (ErrMsg, "Error mallocing space.");
      xxx_LogStatus ("HXXLIB", __FILE__, __LINE__, ErrMsg);
      free (databuf);
      SDendaccess (poly_ls_sds);
      SDend (sd_id);
      return (ERROR);
    }

    order = grid->gridbands[ref].nzplanes * (grid->gridbands[ref].degree + 1) * 
            (grid->gridbands[ref].degree + 1);
    for (j = 0; j < order; j++)
        darr2[j] = grid->gridbands[ref].poly_lines[j];
    memcpy (pntr, (void *)darr2, order*poly_ls_elements[0].size);
    pntr += order*poly_ls_elements[0].size;

    for (j = 0; j < order; j++)
      darr2[j] = grid->gridbands[ref].poly_samps[j];
    memcpy (pntr, (void *)darr2, order*poly_ls_elements[1].size);
    pntr += order*poly_ls_elements[1].size;
    free (darr2);

    /* Write buffer to poly_lines/samps SDS.
     */
    istatn = SDwritedata (poly_ls_sds, start, NULL, dimsizes, (VOIDP)databuf);
    if (istatn == FAIL)
    {
      strcpy (ErrMsg, "Error writing the grid structure to the HDF file.");
      xxx_LogStatus ("HXXLIB", __FILE__, __LINE__, ErrMsg);
      free (databuf);
      SDendaccess (poly_ls_sds);
      SDend (sd_id);
      return (ERROR);
    }

    /* Terminate access to the SDS.
     */
    SDendaccess (poly_ls_sds);
    free (databuf);

    /***********************************************************
     * Assign an sds (sattoproj/projtosat) for write-only access.
     */
    /* Create the SDS array.  The array will have 4 dimensions.  One will be
     * for the sattoproj/projtosat themselves (size = 2).  One will be for the
     * COEFS_SIZE.  One will be for the XY_SIZE.  One will be for the size
     * of sattoproj/projtosat (size = grid->gridbands[currref].ngrid_lines/2 *
     * (grid->gridbands[currref].ngrid_samps - 1)).
     */
    rank = 4;
    dimsizes[0] = 2;
    dimsizes[1] = COEFS_SIZE;
    dimsizes[2] = XY_SIZE;
    dimsizes[3] = (int32)grid->gridbands[ref].nzplanes *
                  (int32)(grid->gridbands[ref].ngrid_lines - 1) *
                  (int32)(grid->gridbands[ref].ngrid_samps - 1);
    status = sprintf (name, "Sat_proj_b%d", hxx_get_user_band (grid->bands[currband]));
    if (status < 1)
    {
      xxx_LogStatus ("HXXLIB", __FILE__, __LINE__, "Error in sprintf");
      return (ERROR);
    }
    sat_proj_sds = SDcreate (sd_id, name, sat_proj_elements[0].dfnt, rank, dimsizes);
    if (sat_proj_sds == FAIL)
    {
      strcpy (ErrMsg, "Error creating the sat_proj_sds.");
      xxx_LogStatus ("HXXLIB", __FILE__, __LINE__, ErrMsg);
      SDendaccess (sat_proj_sds);
      SDend (sd_id);
      return (ERROR);
    }

    /* Set up the data buffer and its pointer for the sattoproj/projtosat.
     */
    databuf = calloc (grid->gridbands[ref].nzplanes * 
                     (grid->gridbands[ref].ngrid_lines - 1) *
                     (grid->gridbands[ref].ngrid_samps - 1) *
                     COEFS_SIZE*XY_SIZE*2, sizeof(double));
    if (databuf == NULL)
    {
      strcpy (ErrMsg, "Error allocating data buffer memory.");
      xxx_LogStatus ("HXXLIB", __FILE__, __LINE__, ErrMsg);
      SDendaccess (sat_proj_sds);
      SDend (sd_id);
      return (ERROR);
    }
    pntr = databuf;

    /* Write the sattoproj/projtosat data to buffers.
     */
    darr2 = calloc (grid->gridbands[ref].nzplanes * 
                   (grid->gridbands[ref].ngrid_lines - 1) *
                   (grid->gridbands[ref].ngrid_samps - 1), sizeof(double));
    if (darr2 == NULL)
    {
      strcpy (ErrMsg, "Error mallocing space.");
      xxx_LogStatus ("HXXLIB", __FILE__, __LINE__, ErrMsg);
      free (databuf);
      SDendaccess (sat_proj_sds);
      SDend (sd_id);
      return (ERROR);
    }

    order = grid->gridbands[ref].nzplanes * (grid->gridbands[ref].ngrid_lines - 1) * 
            (grid->gridbands[ref].ngrid_samps - 1);
    for (j = 0; j < COEFS_SIZE; j++)
    {
        /* Sattoproj a coefs
         */
       for (l = 0; l < order; l++)
          darr2[l] = grid->gridbands[ref].sattoproj[l].a[j];
       memcpy (pntr, (void *)darr2, order*sat_proj_elements[0].size);
       pntr += order*sat_proj_elements[0].size;

       /* Sattoproj b coefs
        */
       for (l = 0; l < order; l++)
          darr2[l] = grid->gridbands[ref].sattoproj[l].b[j];
       memcpy (pntr, (void *)darr2, order*sat_proj_elements[0].size);
       pntr += order*sat_proj_elements[0].size;

       /* Projtosat a coefs
        */
       for (l = 0; l < order; l++)
          darr2[l] = grid->gridbands[ref].projtosat[l].a[j];
       memcpy (pntr, (void *)darr2, order*sat_proj_elements[1].size);
       pntr += order*sat_proj_elements[1].size;

       /* Projtosat b coefs
        */
       for (l = 0; l < order; l++)
          darr2[l] = grid->gridbands[ref].projtosat[l].b[j];
       memcpy (pntr, (void *)darr2, order*sat_proj_elements[1].size);
       pntr += order*sat_proj_elements[1].size;
    }
    free (darr2);

    /* Write buffer to sattoproj/projtosat SDS.
     */
    istatn = SDwritedata (sat_proj_sds, start, NULL, dimsizes, (VOIDP)databuf);
    if (istatn == FAIL)
    {
       strcpy (ErrMsg, "Error writing the grid structure to the HDF file.");
       xxx_LogStatus ("HXXLIB", __FILE__, __LINE__, ErrMsg);
       free (databuf);
       SDendaccess (sat_proj_sds);
       SDend (sd_id);
       return (ERROR);
    }

    /* Terminate access to the SDS.
     */
    SDendaccess (sat_proj_sds);
    free (databuf);
  } /* Loop on the bands */

  /***********************************************************
   * Close the file.
   */
  SDend (sd_id);

  return (SUCCESS);
}
/******************************************************************************
NAME:           hxx_get_grid

PURPOSE:
Read the grid from an HDF file.


RETURN VALUE:   Type = int
Value    Description
-----    -----------
SUCCESS  Successful completion
ERROR    Operation failed

NOTES:   This function is a bridge routine to the actual hxx_read_grid
         support.  This function only flips SOM corner coorindates
         after they are read.  This allows a consistant user interface,
         yet allows the algorithms using SOM coodinates not to change.

ALGORITHM REFERENCES:
See hxx_grid.h for a description of the grid structure.

******************************************************************************/
int hxx_get_grid (
  char grid_filename[STRLEN],/* I: HDF input file name                    */
  HBandNumber *bands,        /* I/O: Array of bands to free from the grid */
  int *nbands,               /* I/O: Number of bands in the bands array   */
  hxx_GRID_TYPE *grid)       /* O: Grid structure to be output            */
{
double tmp;		     /* Temporary variable for flipping corner points*/

if (hxx_read_grid (grid_filename, bands, nbands, grid) != SUCCESS)
    return(ERROR);

/* flip the coords if the projection is SOM */
hxx_SOM_grid_calc(grid);

/*
  Write the grid structure to the HDF file
*/
return(SUCCESS);
}

/******************************************************************************
NAME:           hxx_read_grid

PURPOSE:
Read the grid from an HDF file.

RETURN VALUE:   Type = int
Value    Description
-----    -----------
SUCCESS  Successful completion
ERROR    Operation failed

NOTES:
This routine will read the grid from the HDF file for the specified band
numbers.  The hxx_GRID_TYPE elements are read first, so that we know the
number of bands in the grid (stored in nbands.)  Next the constant
hxx_GRID_BAND_TYPE data elements are read for the entire grid.  These data
elements specify the number of elements in the hxx_GRID_BAND_TYPE pointer
data elements.  Next the values in lines, samps, ngrid_lines, ngrid_samps, and
degree are used to malloc the appropriate amount of space for the pointers (of
the specified bands) in the grid.  After the space is malloced for each
pointer, their data is read from the file.

The user may specify one band, several bands, or no bands.  If no bands are
specified, then all the bands are read from the HDF file.  In this case, the
bands array will be filled in with the bands in the grid and nbands will be
set to the correct value.  This routine will check the grid to verify that
the specified bands are in the HDF file.  If a band does not exist, then this
routine will exit with an error.

The hxx_GRID_TYPE data is stored in a vdata named hxx_GRID_TYPE.  The constant
data in the hxx_GRID_BAND_TYPE structure is stored in a vdata named Constants.
This vdata contains the constant data for all of the specified bands.  The
pointer data is stored separately for each band, in its own SDS.  The name of
the SDS will be in_lines, in_samps, out_ls, poly_ls, and sat_proj concatinated
with the band number.  Thus the in_lines pointer data for band b7 will be
stored in the SDS with a name of in_lines7.

If a grid structure contains either bands 6L and/or 6H, then the data will be
read from an SDS with the name of the pointer followed by "6" instead of "61"
or "62".  The reason for this is that this data is the same for both bands
6L and 6H.

It is important to note that this routine will malloc the space needed for the
pointers (for the specified bands) within the grid.  The grid structure itself
must already be available, but the space for the pointers within the structure
is malloced in this routine.

ALGORITHM REFERENCES:
See hxx_grid.h for a description of the grid structure.

******************************************************************************/

int hxx_read_grid (
  char grid_filename[STRLEN],/* I: HDF input file name                    */
  HBandNumber *bands,        /* I/O: Array of bands to free from the grid */
  int *nbands,               /* I/O: Number of bands in the bands array   */
  hxx_GRID_TYPE *grid)       /* O: Grid structure to be output            */
{
  char ErrMsg[STRLEN];       /* Error message                       */

  /* Get the hxx_GRID_TYPE data and the constant data for the
   * hxx_GRID_BAND_TYPEs.
   */ 
  if (hxx_get_grid_headers (grid_filename, grid) == ERROR)
  {
    strcpy (ErrMsg,"Error reading the grid headers.");
    xxx_LogStatus ("HXXLIB", __FILE__, __LINE__, ErrMsg);
    return (ERROR);
  }

  /* Get the pointer data for the specified bands.
   */
  if (hxx_get_grid_pointers (grid_filename, bands, nbands, grid) == ERROR)
  {
    strcpy (ErrMsg,"Error reading the grid pointers.");
    xxx_LogStatus ("HXXLIB", __FILE__, __LINE__, ErrMsg);
    return (ERROR);
  }

  return (SUCCESS);
}

/******************************************************************************
NAME:           hxx_get_grid_headers

PURPOSE:
Read the hxx_GRID_TYPE and hxx_GRID_BAND_TYPE constants from an HDF file.

RETURN VALUE:   Type = int
Value    Description
-----    -----------
SUCCESS  Successful completion
ERROR    Operation failed

NOTES:
This routine will read the hxx_GRID_TYPE and hxx_GRID_BAND_TYPE constants for
the entire grid.  The hxx_GRID_TYPE elements are read first, so that we know
the number of bands in the grid (stored in nbands.)  Next the constant
hxx_GRID_BAND_TYPE data elements are read for the entire grid.  These data
elements specify the number of elements in the hxx_GRID_BAND_TYPE pointer
data elements.

The hxx_GRID_TYPE data is stored in a vdata named hxx_GRID_TYPE.  The constant
data in the hxx_GRID_BAND_TYPE structure is stored in a vdata named Constants.
This vdata contains the constant data for all of the specified bands.

ALGORITHM REFERENCES:
See hxx_grid.h for a description of the grid structure.

******************************************************************************/

int hxx_get_grid_headers (
  char grid_filename[STRLEN],/* I: HDF input file name                    */
  hxx_GRID_TYPE *grid)       /* O: Grid structure to be output            */
{
  int  i, j,                 /* Loop variables                      */
       currsize,             /* Current size of the data element    */
       ref,                  /* Current reference band value        */
       num_ints,             /* Number of ints to malloc for        */
       num_doubles,          /* Number of doubles to malloc for     */
       num_chars;            /* Number of characters to malloc for  */
  int32 file_id,             /* DDR HDF file pointer                */
        ref_num,             /* Reference number of the vdata       */
        grid_type_vdata,     /* Vdata pointer (grid_type)           */
        constant_vdata,      /* Vdata pointer (constants)           */
        istat32,             /* Return values                       */
        i32,                 /* Holder for integers                 */
        *iarr;               /* Holder for integer arrays           */
  intn istatn;               /* Return values                       */
  uint8 *databuf,            /* Data pointer                        */
        *pntr;               /* Current position in data pointer    */
  char str[STRLEN],          /* Holder for character strings        */
       fields[NGRIDTYPE * XXX_ODL_FIELD_SIZE],
                             /* Packed string of field names of the
                                model.  The hxx_GRID_TYPE structure
                                is the largest so give enough space
                                for it.                             */
       ErrMsg[STRLEN];       /* Error message                       */
  double d,                  /* Holder for doubles                  */
         darr[PROJPRMS_SIZE]; /* Holder for double arrays (largest size) */

  /***********************************************************
   * Open the HDF output file.
   */
  file_id = Hopen (grid_filename, DFACC_RDONLY, 0);
  if (file_id == FAIL)
  {
    sprintf (ErrMsg, "Error opening HDF grid file %s.", grid_filename);
    xxx_LogStatus ("HXXLIB", __FILE__, __LINE__, ErrMsg);
    return (ERROR);
  }
  Vstart (file_id);

  /****************************************************
   * Find the hxx_GRID_TYPE Vdata.
   */
  ref_num = VSfind (file_id, "hxx_GRID_TYPE");
  if (ref_num == 0)
  {
    strcpy (ErrMsg,"Error finding hxx_GRID_TYPE vdata.");
    xxx_LogStatus ("HXXLIB", __FILE__, __LINE__, ErrMsg);
    Vend (file_id);
    Hclose (file_id);
    return (ERROR);
  }

  /* Assign a vdata id for read-only access.
   */
  grid_type_vdata = VSattach (file_id, ref_num, "r");
  if (grid_type_vdata == FAIL)
  {
    strcpy (ErrMsg,"Error reading hxx_GRID_TYPE vdata.");
    xxx_LogStatus ("HXXLIB", __FILE__, __LINE__, ErrMsg);
    Vend (file_id);
    Hclose (file_id);
    return (ERROR);
  }

  /* Set the order of the fields for the grid_type.  This is the same order we
   * wrote them.
   */
  istatn = VSQueryfields (grid_type_vdata, fields);
  if (istatn == FAIL)
  {
    strcpy (ErrMsg, "Error inquiring about the hxx_GRID_TYPE vdata.");
    xxx_LogStatus ("HXXLIB", __FILE__, __LINE__, ErrMsg);
    VSdetach (grid_type_vdata);
    Vend (file_id);
    Hclose (file_id);
    return (ERROR);
  }
  istatn = VSsetfields(grid_type_vdata, fields);
  if (istatn == FAIL)
  {
    strcpy (ErrMsg, "Error setting the field names.");
    xxx_LogStatus ("HXXLIB", __FILE__, __LINE__, ErrMsg);
    VSdetach (grid_type_vdata);
    Vend (file_id);
    Hclose (file_id);
    return (ERROR);
  }

  /* Set up the data buffer and its pointer.
   */
  num_ints = NUM_GRID_TYPE_INTS;
  num_chars = NUM_GRID_TYPE_CHARS;
  num_doubles = NUM_GRID_TYPE_DBLS;
  databuf = malloc (num_ints*sizeof(int32) + num_doubles*sizeof(double) + num_chars*sizeof(char));
  if (databuf == NULL)
  {
    strcpy (ErrMsg, "Error allocating data buffer memory.");
    xxx_LogStatus ("HXXLIB", __FILE__, __LINE__, ErrMsg);
    VSdetach (grid_type_vdata);
    Vend (file_id);
    Hclose (file_id);
    return (ERROR);
  }
  pntr = databuf;

  /* Malloc space for the integer array for the bands.
   */
  iarr = malloc (HYP_NBANDS * sizeof(int32));
  if (iarr == NULL)
  {
    strcpy (ErrMsg, "Error allocating memory.");
    xxx_LogStatus ("HXXLIB", __FILE__, __LINE__, ErrMsg);
    VSdetach (grid_type_vdata);
    Vend (file_id);
    Hclose (file_id);
    return (ERROR);
  }

  /* Read the data.  There is only one record in this Vdata.
   */
  istat32 = VSread (grid_type_vdata, databuf, 1, FULL_INTERLACE);
  if (istat32 == FAIL)
  {
    strcpy (ErrMsg, "Error reading the hxx_GRID_BAND vdata.");
    xxx_LogStatus ("HXXLIB", __FILE__, __LINE__, ErrMsg);
    free (databuf);
    free (iarr);
    VSdetach (grid_type_vdata);
    Vend (file_id);
    Hclose (file_id);
    return (ERROR);
  }
  j = 0;

  /* Unpack the data elements from the buffer into the grid structure.
   */
  currsize = grid_type_elements[j].order * grid_type_elements[j].size;
  memcpy (str, pntr, currsize);
  strcpy (grid->gridname, str);
  j++;
  pntr += currsize;

  currsize = grid_type_elements[j].order * grid_type_elements[j].size;
  memcpy (&i32, pntr, currsize);
  grid->nbands = i32;
  j++;
  pntr += currsize;

  currsize = grid_type_elements[j].order * grid_type_elements[j].size;
  memcpy (iarr, pntr, currsize);
  for (i = 0; i < HYP_NBANDS; i++)
    grid->bands[i] = (HBandNumber) iarr[i];
  j++;
  pntr += currsize;

  currsize = grid_type_elements[j].order * grid_type_elements[j].size;
  memcpy (&i32, pntr, currsize);
  grid->dtype = i32;
  j++;
  pntr += currsize;

  currsize = grid_type_elements[j].order * grid_type_elements[j].size;
  memcpy (&i32, pntr, currsize);
  grid->correction_type = (CorrectionType) i32;
  j++;
  pntr += currsize;

  currsize = grid_type_elements[j].order * grid_type_elements[j].size;
  memcpy (&i32, pntr, currsize);
  grid->satnum = i32;
  j++;
  pntr += currsize;

  currsize = grid_type_elements[j].order * grid_type_elements[j].size;
  memcpy (&i32, pntr, currsize);
  grid->frame_type = (FrameType) i32;
  j++;
  pntr += currsize;

  currsize = grid_type_elements[j].order * grid_type_elements[j].size;
  memcpy (str, pntr, currsize);
  strcpy (grid->units, str);
  j++;
  pntr += currsize;

  currsize = grid_type_elements[j].order * grid_type_elements[j].size;
  memcpy (&i32, pntr, currsize);
  grid->code = i32;
  j++;
  pntr += currsize;

  currsize = grid_type_elements[j].order * grid_type_elements[j].size;
  memcpy (str, pntr, currsize);
  strcpy (grid->datum, str);
  j++;
  pntr += currsize;

  currsize = grid_type_elements[j].order * grid_type_elements[j].size;
  memcpy (&i32, pntr, currsize);
  grid->spheroid = i32;
  j++;
  pntr += currsize;

  currsize = grid_type_elements[j].order * grid_type_elements[j].size;
  memcpy (&i32, pntr, currsize);
  grid->zone = i32;
  j++;
  pntr += currsize;

  currsize = grid_type_elements[j].order * grid_type_elements[j].size;
  memcpy (darr, pntr, currsize);
  for (i = 0; i < PROJPRMS_SIZE; i++)
    grid->projprms[i] = darr[i];
  j++;
  pntr += currsize;

  currsize = grid_type_elements[j].order * grid_type_elements[j].size;
  memcpy (darr, pntr, currsize);
  grid->corners.upleft.x = darr[0];
  grid->corners.upright.x = darr[1];
  grid->corners.loleft.x = darr[2];
  grid->corners.loright.x = darr[3];
  j++;
  pntr += currsize;

  currsize = grid_type_elements[j].order * grid_type_elements[j].size;
  memcpy (darr, pntr, currsize);
  grid->corners.upleft.y = darr[0];
  grid->corners.upright.y = darr[1];
  grid->corners.loleft.y = darr[2];
  grid->corners.loright.y = darr[3];
  j++;
  pntr += currsize;

  currsize = grid_type_elements[j].order * grid_type_elements[j].size;
  memcpy (darr, pntr, currsize);
  grid->center_pt.lat = darr[0];
  grid->center_pt.lng = darr[1];
  j++;
  pntr += currsize;

  currsize = grid_type_elements[j].order * grid_type_elements[j].size;
  memcpy (darr, pntr, currsize);
  grid->proj_center_pt.x = darr[0];
  grid->proj_center_pt.y = darr[1];
  j++;
  pntr += currsize;

  currsize = grid_type_elements[j].order * grid_type_elements[j].size;
  memcpy (&d, pntr, currsize);
  grid->rotate_angle = d;
  j++;
  pntr += currsize;

  currsize = grid_type_elements[j].order * grid_type_elements[j].size;
  memcpy (&d, pntr, currsize);
  grid->orient_angle = d;
  j++;
  pntr += currsize;

  /* Close the Vdata.
   */
  VSdetach (grid_type_vdata);
  free (iarr);
  free (databuf);

  /****************************************************
   * Find the hxx_GRID_BAND_TYPE constants Vdata.
   */
  ref_num = VSfind (file_id, "Constants");
  if (ref_num == 0)
  {
    strcpy (ErrMsg,"Error finding hxx_GRID_BAND_TYPE constants vdata.");
    xxx_LogStatus ("HXXLIB", __FILE__, __LINE__, ErrMsg);
    Vend (file_id);
    Hclose (file_id);
    return (ERROR);
  }

  /* Assign a vdata id for read-only access.
   */
  constant_vdata = VSattach (file_id, ref_num, "r");
  if (constant_vdata == FAIL)
  {
    strcpy (ErrMsg,"Error reading hxx_GRID_BAND_TYPE constants vdata.");
    xxx_LogStatus ("HXXLIB", __FILE__, __LINE__, ErrMsg);
    Vend (file_id);
    Hclose (file_id);
    return (ERROR);
  }

  /* Set the order of the fields for the Constants.  This is the same
   * order we wrote them.
   */
  istatn = VSQueryfields (constant_vdata, fields);
  if (istatn == FAIL)
  {
    strcpy (ErrMsg, "Error inquiring about the constants vdata.");
    xxx_LogStatus ("HXXLIB", __FILE__, __LINE__, ErrMsg);
    VSdetach (constant_vdata);
    Vend (file_id);
    Hclose (file_id);
    return (ERROR);
  }
  istatn = VSsetfields(constant_vdata, fields);
  if (istatn == FAIL)
  {
    strcpy (ErrMsg, "Error setting the field names.");
    xxx_LogStatus ("HXXLIB", __FILE__, __LINE__, ErrMsg);
    VSdetach (constant_vdata);
    Vend (file_id);
    Hclose (file_id);
    return (ERROR);
  }

  /* Set up the data buffer and its pointer for the constants.
   */
  num_ints = NUM_CONSTANT_INTS;
  num_doubles = NUM_CONSTANT_DBLS;
  databuf = malloc ((num_ints*sizeof(int32) + num_doubles*sizeof(double)) * grid->nbands);
  if (databuf == NULL)
  {
    strcpy (ErrMsg, "Error allocating data buffer memory.");
    xxx_LogStatus ("HXXLIB", __FILE__, __LINE__, ErrMsg);
    VSdetach (constant_vdata);
    Vend (file_id);
    Hclose (file_id);
    return (ERROR);
  }
  pntr = databuf;

  /* Read the data.  There are grid->nbands records in this Vdata.
   */
  istat32 = VSread (constant_vdata, databuf, grid->nbands, FULL_INTERLACE);
  if (istat32 == FAIL)
  {
    strcpy (ErrMsg, "Error reading the constant vdata.");
    xxx_LogStatus ("HXXLIB", __FILE__, __LINE__, ErrMsg);
    free (databuf);
    VSdetach (constant_vdata);
    Vend (file_id);
    Hclose (file_id);
    return (ERROR);
  }

  /* Unpack the data elements from the buffer into the grid structure.
   */
  for (i = 0, j = 0; i < grid->nbands; i++, j = 0)
  {
    /* Get the reference value of the current band.
     */
    ref = grid->bands[i];

    currsize = constant_elements[j].order * constant_elements[j].size;
    memcpy (&i32, pntr, currsize);
    grid->gridbands[ref].bandno = (HBandNumber) i32;
    j++;
    pntr += currsize;

    currsize = constant_elements[j].order * constant_elements[j].size;
    memcpy (&i32, pntr, currsize);
    grid->gridbands[ref].grid_cell_size.line = i32;
    j++;
    pntr += currsize;

    currsize = constant_elements[j].order * constant_elements[j].size;
    memcpy (&i32, pntr, currsize);
    grid->gridbands[ref].grid_cell_size.samp = i32;
    j++;
    pntr += currsize;

    currsize = constant_elements[j].order * constant_elements[j].size;
    memcpy (&d, pntr, currsize);
    grid->gridbands[ref].pixsize = d;
    j++;
    pntr += currsize;

    currsize = constant_elements[j].order * constant_elements[j].size;
    memcpy (&d, pntr, currsize);
    grid->gridbands[ref].zspacing = d;
    j++;
    pntr += currsize;

    currsize = constant_elements[j].order * constant_elements[j].size;
    memcpy (&i32, pntr, currsize);
    grid->gridbands[ref].lines = i32;
    j++;
    pntr += currsize;

    currsize = constant_elements[j].order * constant_elements[j].size;
    memcpy (&i32, pntr, currsize);
    grid->gridbands[ref].samps = i32;
    j++;
    pntr += currsize;

    currsize = constant_elements[j].order * constant_elements[j].size;
    memcpy (&i32, pntr, currsize);
    grid->gridbands[ref].ngrid_lines = i32;
    j++;
    pntr += currsize;

    currsize = constant_elements[j].order * constant_elements[j].size;
    memcpy (&i32, pntr, currsize);
    grid->gridbands[ref].ngrid_samps = i32;
    j++;
    pntr += currsize;

    currsize = constant_elements[j].order * constant_elements[j].size;
    memcpy (&i32, pntr, currsize);
    grid->gridbands[ref].nzplanes = i32;
    j++;
    pntr += currsize;

    currsize = constant_elements[j].order * constant_elements[j].size;
    memcpy (&i32, pntr, currsize);
    grid->gridbands[ref].zeroplane = i32;
    j++;
    pntr += currsize;

    currsize = constant_elements[j].order * constant_elements[j].size;
    memcpy (&i32, pntr, currsize);
    grid->gridbands[ref].degree = i32;
    j++;
    pntr += currsize;

  } /* Loop on bands */

  /* Close the Vdata.
   */
  VSdetach (constant_vdata);
  free (databuf);

  /* Close the file.
   */
  Vend (file_id);
  Hclose (file_id);

  /* Set the pointers for all bands in the grid to NULL.
   */
  for (i = 0; i < HYP_NBANDS; i++)
  {
    /* Set the pointers to NULL.
     */
    grid->gridbands[i].in_lines = NULL;
    grid->gridbands[i].in_samps = NULL;
    grid->gridbands[i].out_lines = NULL;
    grid->gridbands[i].out_samps = NULL;
    grid->gridbands[i].poly_lines = NULL;
    grid->gridbands[i].poly_samps = NULL;
    grid->gridbands[i].sattoproj = NULL;
    grid->gridbands[i].projtosat = NULL;
  }

  return (SUCCESS);
}


/******************************************************************************
NAME:           hxx_get_grid_pointers

PURPOSE:
Read the grid pointers from an HDF file for the specified bands.


RETURN VALUE:   Type = int
Value    Description
-----    -----------
SUCCESS  Successful completion
ERROR    Operation failed

NOTES:
This routine will read the grid pointers from the HDF file for the specified
band numbers.  hxx_get_grid_headers should already have been called so that
the appropriate header and constant information is available for the size of
the grid pointers.  Now the values in lines, samps, ngrid_lines, ngrid_samps,
and degree are used to malloc the appropriate amount of space for the pointers
(of the specified bands) in the grid.  After the space is malloced for each
pointer, their data is read from the file.

The user may specify one band, several bands, or no bands.  If no bands are
specified, then all the bands are read from the HDF file.  In this case, the
bands array will be filled in with the bands in the grid and nbands will be
set to the correct value.  This routine will check the grid to verify that
the specified bands are in the HDF file.  If a band does not exist, then this
routine will exit with an error.

The pointer data is stored separately for each band, in its own SDS.  The name
of the SDS will be in_lines, in_samps, out_ls, poly_ls, and sat_proj
concatinated with the band number.  Thus the in_lines pointer data for band b7
will be stored in the SDS with a name of in_lines7.

It is important to note that this routine will malloc the space needed for the
pointers (for the specified bands) within the grid.  The grid structure itself
must already be available, but the space for the pointers within the structure
is malloced in this routine.

ALGORITHM REFERENCES:
See hxx_grid.h for a description of the grid structure.

******************************************************************************/

int hxx_get_grid_pointers (
  char grid_filename[STRLEN],/* I: HDF input file name                    */
  HBandNumber *bands,        /* I/O: Array of bands to free from the grid */
  int *nbands,               /* I/O: Number of bands in the bands array   */
  hxx_GRID_TYPE *grid)       /* O: Grid structure to be output            */
{
  int  i, j, l,              /* Loop variables                           */
       currband,             /* Current band in the bands array          */
       found,                /* Boolean to specify if the band was found */
       ref,                  /* Current reference band value             */
       status,		     /* status of return from function           */
       order,                /* Order of the current array               */
       istatnt;              /* Return values                            */
  intn istatn;               /* Return values                            */
  int32 num_sds,             /* Number of sds in the file                */
        sd_id,               /* DDR HDF file pointer for sds             */
        sds_index,           /* SDS index pointer                        */
        in_lines_sds,        /* SDS id pointer (in_lines)                */
        in_samps_sds,        /* SDS id pointer (in_samps)                */
        out_ls_sds,          /* SDS id pointer (out_lines/samps)         */
        poly_ls_sds,         /* SDS id pointer (poly_lines/samps)        */
        sat_proj_sds,        /* SDS id pointer (sattoproj/projtosat)     */
        rank,                /* Rank of the dimsizes array.              */
        dim_sizes[4],        /* Array of the dimension sizes.            */
        start[] = {0, 0, 0, 0, 0}, /* Array of 0 starting positions.     */
        num_type,            /* Data type for the data in the array      */
        num_file_attribs,    /* Number of attribs in the file            */
        attrib,              /* Number of attributes in the data set     */
        istat32,             /* Return values                            */
        *iarr;               /* Holder for integer arrays                */
  uint8 *databuf,            /* Data pointer                             */
        *pntr;               /* Current position in data pointer         */
  char sds_name[STRLEN],     /* Name of the sds                          */
       name[STRLEN],         /* SDS name                                 */
       ErrMsg[STRLEN];       /* Error message                            */
  double darr[PROJPRMS_SIZE], /* Holder for double arrays (largest size) */
         *darr2;             /* Holder for double arrays of unknown size */

  /* If no bands are specified then read all the bands in the grid.
   */
  if (*nbands == 0)
  {
    *nbands = grid->nbands;
    memcpy (bands, grid->bands, *nbands * sizeof (HBandNumber));
  }
  else
  /* Verify the specified bands exist in the grid.
   */
  {
    for (i = 0; i < *nbands; i++)
    {
      for (j = 0, found = FALSE; j < grid->nbands; j++)
        if (bands[i] == grid->bands[j])
        {
          found = TRUE;
          break;
        }
        
        /* If the band was not found then exit with an error.
         */
      if (!found)
      {
        sprintf (ErrMsg, "Band %d is not available in the HDF file.", hxx_get_user_band (bands[i]));
        xxx_LogStatus ("HXXLIB", __FILE__, __LINE__, ErrMsg);
        return (ERROR);
      }
    }
  }

  /***********************************************************
   * Malloc space for the grid.
   */
  istatnt = hxx_malloc_grid (bands, nbands, grid);
  if (istatnt == ERROR)
  {
    strcpy (ErrMsg,"Error mallocing space for the grid pointers.");
    xxx_LogStatus ("HXXLIB", __FILE__, __LINE__, ErrMsg);
    return (ERROR);
  }

  /***********************************************************
   * Open the file for the SDS interface.
   */
  sd_id = SDstart (grid_filename, DFACC_RDONLY);
  if (sd_id == FAIL)
  {
    sprintf (ErrMsg, "Error opening grid file %s for SDS.", grid_filename);
    xxx_LogStatus ("HXXLIB", __FILE__, __LINE__, ErrMsg);
    return (ERROR);
  }

  /* Verify the file contains all of our SDSs.  The file should have
   * (grid->nbands * NUM_SDS) SDSs.
   */
  SDfileinfo (sd_id, &num_sds, &num_file_attribs);
  if (num_sds != (grid->nbands * NUM_SDS))
  {
    sprintf (ErrMsg,"Only %ld SDSs in the HDF file.  Should be %ld SDSs.",
             num_sds, (grid->nbands * NUM_SDS));
    xxx_LogStatus ("HXXLIB", __FILE__, __LINE__, ErrMsg);
    SDend (sd_id);
    return (ERROR);
  }

  /**********************************************************************
   * Loop through the bands and read the pointer values in the SDSs for
   * each band.
   */
  for (currband = 0; currband < *nbands; currband++)
  {
    /* Find the reference position.
     */
    ref = bands[currband];

    /****************************************************
     * Find the in_lines SDS.
     */
    status = sprintf (name, "In_lines_b%d", hxx_get_user_band (bands[currband]));
    if (status < 1)
    {
       xxx_LogStatus ("HXXLIB", __FILE__, __LINE__, "Error in sprintf");
       return (ERROR);
    }
    sds_index = SDnametoindex (sd_id, name);
    if (sds_index == FAIL)
    {
      sprintf (ErrMsg, "Error finding the in_lines SDS for band %d.",
        hxx_get_user_band (bands[currband]));
      xxx_LogStatus ("HXXLIB", __FILE__, __LINE__, ErrMsg);
      SDend (sd_id);
      return (ERROR);
    }
    in_lines_sds = SDselect (sd_id, sds_index);

    /* Get the rank and dimension sizes.
     */
    istatn = SDgetinfo (in_lines_sds, sds_name, &rank, dim_sizes, &num_type, &attrib);
    if (istatn == FAIL)
    {
      strcpy (ErrMsg, "Error getting SDS info.");
      xxx_LogStatus ("HXXLIB", __FILE__, __LINE__, ErrMsg);
      SDendaccess (in_lines_sds);
      SDend (sd_id);
      return (ERROR);
    }

    /* Set up the data buffer and its pointer for the in_lines.
     */
    databuf = calloc (grid->gridbands[ref].ngrid_lines, sizeof(int32));
    if (databuf == NULL)
    {
      strcpy (ErrMsg, "Error allocating data buffer memory.");
      xxx_LogStatus ("HXXLIB", __FILE__, __LINE__, ErrMsg);
      SDendaccess (in_lines_sds);
      SDend (sd_id);
      return (ERROR);
    }
    pntr = databuf;

    /* Read the in_lines.  This is the same order we wrote them.
     */
    istatn = SDreaddata (in_lines_sds, start, NULL, dim_sizes, (VOIDP)databuf);
    if (istatn == FAIL)
    {
      strcpy (ErrMsg, "Error reading the vdata.");
      xxx_LogStatus ("HXXLIB", __FILE__, __LINE__, ErrMsg);
      free (databuf);
      SDendaccess (in_lines_sds);
      SDend (sd_id);
      return (ERROR);
    }

    /* Unpack the data elements from the buffer into the grid structure.
     */
    iarr = calloc (grid->gridbands[ref].ngrid_lines, sizeof(int32));
    if (iarr == NULL)
    {
      strcpy (ErrMsg, "Error allocating int array memory.");
      xxx_LogStatus ("HXXLIB", __FILE__, __LINE__, ErrMsg);
      free (databuf);
      SDendaccess (in_lines_sds);
      SDend (sd_id);
      return (ERROR);
    }

    memcpy (iarr, pntr, grid->gridbands[ref].ngrid_lines * in_lines_elements[0].size);
    for (j = 0; j < grid->gridbands[ref].ngrid_lines; j++)
        grid->gridbands[ref].in_lines[j] = iarr[j];

    /* Close the SDS.
     */
    SDendaccess (in_lines_sds);
    free (iarr);
    free (databuf);

    /****************************************************
     * Find the in_samps SDS.
     */
    status = sprintf (name, "In_samps_b%d", hxx_get_user_band (bands[currband]));
    if (status < 1)
    {
       xxx_LogStatus ("HXXLIB", __FILE__, __LINE__, "Error in sprintf");
       return (ERROR);
    }
    sds_index = SDnametoindex (sd_id, name);
    if (sds_index == FAIL)
    {
      sprintf (ErrMsg, "Error finding the in_samps SDS for band %d.",
        hxx_get_user_band (bands[currband]));
      xxx_LogStatus ("HXXLIB", __FILE__, __LINE__, ErrMsg);
      SDend (sd_id);
      return (ERROR);
    }
    in_samps_sds = SDselect (sd_id, sds_index);

    /* Get the rank and dimension sizes.
     */
    istatn = SDgetinfo (in_samps_sds, sds_name, &rank, dim_sizes, &num_type, &attrib);
    if (istatn == FAIL)
    {
      strcpy (ErrMsg, "Error getting SDS info.");
      xxx_LogStatus ("HXXLIB", __FILE__, __LINE__, ErrMsg);
      SDendaccess (in_samps_sds);
      SDend (sd_id);
      return (ERROR);
    }

    /* Set up the data buffer and its pointer for the in_samps.
     */
    databuf = calloc (grid->gridbands[ref].ngrid_samps, sizeof(int32));
    if (databuf == NULL)
    {
      strcpy (ErrMsg, "Error allocating data buffer memory.");
      xxx_LogStatus ("HXXLIB", __FILE__, __LINE__, ErrMsg);
      SDendaccess (in_samps_sds);
      SDend (sd_id);
      return (ERROR);
    }
    pntr = databuf;

    /* Read the in_samps.  This is the same order we wrote them.
     */
    istatn = SDreaddata (in_samps_sds, start, NULL, dim_sizes, (VOIDP)databuf);
    if (istatn == FAIL)
    {
      strcpy (ErrMsg, "Error reading the vdata.");
      xxx_LogStatus ("HXXLIB", __FILE__, __LINE__, ErrMsg);
      free (databuf);
      SDendaccess (in_samps_sds);
      SDend (sd_id);
      return (ERROR);
    }

    /* Unpack the data elements from the buffer into the grid structure.
     */
    iarr = calloc (grid->gridbands[ref].ngrid_samps, sizeof(int32));
    if (iarr == NULL)
    {
      strcpy (ErrMsg, "Error allocating int array memory.");
      xxx_LogStatus ("HXXLIB", __FILE__, __LINE__, ErrMsg);
      free (databuf);
      SDendaccess (in_samps_sds);
      SDend (sd_id);
      return (ERROR);
    }

    memcpy (iarr, pntr, grid->gridbands[ref].ngrid_samps *
            in_samps_elements[0].size);
    for (j = 0; j < grid->gridbands[ref].ngrid_samps; j++)
      grid->gridbands[ref].in_samps[j] = iarr[j];

    /* Close the SDS.
     */
    SDendaccess (in_samps_sds);
    free (iarr);
    free (databuf);

    /****************************************************
     * Find the out_lines/samps SDS.
     */
    status = sprintf (name, "Out_ls_b%d", hxx_get_user_band (bands[currband]));
    if (status < 1)
    {
       xxx_LogStatus ("HXXLIB", __FILE__, __LINE__, "Error in sprintf");
       return (ERROR);
    }
    sds_index = SDnametoindex (sd_id, name);
    if (sds_index == FAIL)
    {
      sprintf (ErrMsg, "Error finding the out_lines/samps SDS for band %d.",
        hxx_get_user_band (bands[currband]));
      xxx_LogStatus ("HXXLIB", __FILE__, __LINE__, ErrMsg);
      SDend (sd_id);
      return (ERROR);
    }
    out_ls_sds = SDselect (sd_id, sds_index);

    /* Get the rank and dimension sizes.
     */
    istatn = SDgetinfo (out_ls_sds, sds_name, &rank, dim_sizes, &num_type, &attrib);
    if (istatn == FAIL)
    {
      strcpy (ErrMsg, "Error getting out_ls SDS info.");
      xxx_LogStatus ("HXXLIB", __FILE__, __LINE__, ErrMsg);
      SDendaccess (out_ls_sds);
      SDend (sd_id);
      return (ERROR);
    }

    /* Set up the data buffer and its pointer for the out_lines/samps.
     */
    databuf = calloc (2 * grid->gridbands[ref].nzplanes * 
                      grid->gridbands[ref].ngrid_lines *
                      grid->gridbands[ref].ngrid_samps, sizeof(double));
    if (databuf == NULL)
    {
      strcpy (ErrMsg, "Error allocating data buffer memory.");
      xxx_LogStatus ("HXXLIB", __FILE__, __LINE__, ErrMsg);
      SDendaccess (out_ls_sds);
      SDend (sd_id);
      return (ERROR);
    }
    pntr = databuf;

    /* Read the out_lines/samps.  This is the same order we wrote them.
     */
    istatn = SDreaddata (out_ls_sds, start, NULL, dim_sizes, (VOIDP)databuf);
    if (istatn == FAIL)
    {
      strcpy (ErrMsg, "Error reading the out_ls vdata.");
      xxx_LogStatus ("HXXLIB", __FILE__, __LINE__, ErrMsg);
      free (databuf);
      SDendaccess (out_ls_sds);
      SDend (sd_id);
      return (ERROR);
    }

    /* Unpack the data elements from the buffer into the grid structure.
     */
    darr2 = calloc (grid->gridbands[ref].nzplanes * grid->gridbands[ref].ngrid_lines *
                    grid->gridbands[ref].ngrid_samps, sizeof(double));
    if (darr2 == NULL)
    {
      strcpy (ErrMsg, "Error allocating double array memory.");
      xxx_LogStatus ("HXXLIB", __FILE__, __LINE__, ErrMsg);
      free (databuf);
      SDendaccess (out_ls_sds);
      SDend (sd_id);
      return (ERROR);
    }

    order = grid->gridbands[ref].nzplanes * grid->gridbands[ref].ngrid_lines *
            grid->gridbands[ref].ngrid_samps;
    memcpy (darr2, pntr, order*out_ls_elements[0].size);
    for (j = 0; j < order; j++)
      grid->gridbands[ref].out_lines[j] = darr2[j];
    pntr += order*out_ls_elements[0].size;

    memcpy (darr2, pntr, order*out_ls_elements[1].size);
    for (j = 0; j < order; j++)
      grid->gridbands[ref].out_samps[j] = darr2[j];
    pntr += order*out_ls_elements[1].size;

    /* Close the SDS.
     */
    SDendaccess (out_ls_sds);
    free (darr2);
    free (databuf);

    /****************************************************
     * Find the poly_lines/samps SDS.
     */
    status = sprintf (name, "Poly_ls_b%d", hxx_get_user_band (bands[currband]));
    if (status < 1)
    {
        xxx_LogStatus ("HXXLIB", __FILE__, __LINE__, "Error in sprintf");
       return (ERROR);
    }
    sds_index = SDnametoindex (sd_id, name);
    if (sds_index == FAIL)
    {
      sprintf (ErrMsg, "Error finding the poly_lines/samps SDS for band %d.",
        hxx_get_user_band (bands[currband]));
      xxx_LogStatus ("HXXLIB", __FILE__, __LINE__, ErrMsg);
      SDend (sd_id);
      return (ERROR);
    }
    poly_ls_sds = SDselect (sd_id, sds_index);

    /* Get the rank and dimension sizes.
     */
    istatn = SDgetinfo (poly_ls_sds, sds_name, &rank, dim_sizes, &num_type, &attrib);
    if (istatn == FAIL)
    {
      strcpy (ErrMsg, "Error getting poly_ls SDS info.");
      xxx_LogStatus ("HXXLIB", __FILE__, __LINE__, ErrMsg);
      SDendaccess (poly_ls_sds);
      SDend (sd_id);
      return (ERROR);
    }

    /* Set up the data buffer and its pointer for the poly_lines/samps.
     */
    databuf = calloc (2 * grid->gridbands[ref].nzplanes * 
                     (grid->gridbands[ref].degree+1) *
                     (grid->gridbands[ref].degree+1), sizeof(double));
    if (databuf == NULL)
    {
      strcpy (ErrMsg, "Error allocating data buffer memory.");
      xxx_LogStatus ("HXXLIB", __FILE__, __LINE__, ErrMsg);
      SDendaccess (poly_ls_sds);
      SDend (sd_id);
      return (ERROR);
    }
    pntr = databuf;

    /* Read the poly_lines/samps.  This is the same order we wrote them.
     */
    istatn = SDreaddata (poly_ls_sds, start, NULL, dim_sizes, (VOIDP)databuf);
    if (istatn == FAIL)
    {
      strcpy (ErrMsg, "Error reading the poly_ls vdata.");
      xxx_LogStatus ("HXXLIB", __FILE__, __LINE__, ErrMsg);
      free (databuf);
      SDendaccess (poly_ls_sds);
      SDend (sd_id);
      return (ERROR);
    }

    /* Unpack the data elements from the buffer into the grid structure.
     */
    darr2 = calloc (grid->gridbands[ref].nzplanes * 
                    (grid->gridbands[ref].degree+1) *
                    (grid->gridbands[ref].degree+1), sizeof(double));
    if (darr2 == NULL)
    {
      strcpy (ErrMsg, "Error mallocing space.");
      xxx_LogStatus ("HXXLIB", __FILE__, __LINE__, ErrMsg);
      free (databuf);
      SDendaccess (poly_ls_sds);
      SDend (sd_id);
      return (ERROR);
    }

    order = grid->gridbands[ref].nzplanes * (grid->gridbands[ref].degree + 1) *
            (grid->gridbands[ref].degree + 1);
    memcpy (darr2, pntr, order*poly_ls_elements[0].size);
    for (j = 0; j < order; j++)
        grid->gridbands[ref].poly_lines[j] = darr2[j];
    pntr += order*poly_ls_elements[0].size;

    memcpy (darr2, pntr, order*poly_ls_elements[1].size);
    for (j = 0; j < order; j++)
        grid->gridbands[ref].poly_samps[j] = darr2[j];
    pntr += order*poly_ls_elements[1].size;

    /* Close the Vdata.
     */
    SDendaccess (poly_ls_sds);
    free (darr2);
    free (databuf);

    /***********************************************************
     * Find the sattoproj/projtosat SDS.
     */
    status = sprintf (name, "Sat_proj_b%d", hxx_get_user_band (bands[currband]));
    if (status < 1)
    {
       xxx_LogStatus ("HXXLIB", __FILE__, __LINE__, "Error in sprintf");
       return (ERROR);
    }
    sds_index = SDnametoindex (sd_id, name);
    if (sds_index == FAIL)
    {
      sprintf (ErrMsg, "Error finding the sat_proj SDS for band %d.",
        hxx_get_user_band (bands[currband]));
      xxx_LogStatus ("HXXLIB", __FILE__, __LINE__, ErrMsg);
      SDend (sd_id);
      return (ERROR);
    }
    sat_proj_sds = SDselect (sd_id, sds_index);

    /* Get the rank and dimension sizes.
     */
    istatn = SDgetinfo (sat_proj_sds, sds_name, &rank, dim_sizes, &num_type, &attrib);
    if (istatn == FAIL)
    {
      strcpy (ErrMsg, "Error getting sat_proj SDS info.");
      xxx_LogStatus ("HXXLIB", __FILE__, __LINE__, ErrMsg);
      SDendaccess (sat_proj_sds);
      SDend (sd_id);
      return (ERROR);
    }

    /* Set up the data buffer and its pointer for the sattoproj/projtosat.
     */
    databuf = calloc (grid->gridbands[ref].nzplanes * 
                     (grid->gridbands[ref].ngrid_lines - 1) *
                     (grid->gridbands[ref].ngrid_samps - 1) *
                     COEFS_SIZE*XY_SIZE*2, sizeof(double));
    if (databuf == NULL)
    {
      strcpy (ErrMsg, "Error allocating data buffer memory.");
      xxx_LogStatus ("HXXLIB", __FILE__, __LINE__, ErrMsg);
      SDendaccess (sat_proj_sds);
      SDend (sd_id);
      return (ERROR);
    }
    pntr = databuf;

    /* Read the sattoproj/projtosat.  This is the same order we wrote them.
     */
    istatn = SDreaddata (sat_proj_sds, start, NULL, dim_sizes, (VOIDP)databuf);
    if (istatn == FAIL)
    {
      strcpy (ErrMsg, "Error reading the sat_proj vdata.");
      xxx_LogStatus ("HXXLIB", __FILE__, __LINE__, ErrMsg);
      free (databuf);
      SDendaccess (sat_proj_sds);
      SDend (sd_id);
      return (ERROR);
    }

    /* Unpack the data elements from the buffer into the grid structure.
     */
    darr2 = calloc (grid->gridbands[ref].nzplanes * 
                   (grid->gridbands[ref].ngrid_lines - 1) *
                   (grid->gridbands[ref].ngrid_samps - 1), sizeof(double));
    if (darr2 == NULL)
    {
      strcpy (ErrMsg, "Error mallocing space.");
      xxx_LogStatus ("HXXLIB", __FILE__, __LINE__, ErrMsg);
      free (databuf);
      SDendaccess (sat_proj_sds);
      SDend (sd_id);
      return (ERROR);
    }

    for (j = 0; j < COEFS_SIZE; j++)
    {
      /* Sattoproj a coefs
       */
      order = grid->gridbands[ref].nzplanes *
              (grid->gridbands[ref].ngrid_lines - 1) *
              (grid->gridbands[ref].ngrid_samps - 1);
      memcpy (darr2, pntr, order*sat_proj_elements[0].size);
      for (l = 0; l < order; l++)
        grid->gridbands[ref].sattoproj[l].a[j] = darr2[l];
      pntr += order*sat_proj_elements[0].size;

      /* Sattoproj b coefs
       */
      memcpy (darr2, pntr, order*sat_proj_elements[0].size);
      for (l = 0; l < order; l++)
        grid->gridbands[ref].sattoproj[l].b[j] = darr2[l];
      pntr += order*sat_proj_elements[0].size;

      /* Projtosat a coefs
       */
      memcpy (darr2, pntr, order*sat_proj_elements[1].size);
      for (l = 0; l < order; l++)
        grid->gridbands[ref].projtosat[l].a[j] = darr2[l];
      pntr += order*sat_proj_elements[1].size;

      /* Projtosat b coefs
       */
      memcpy (darr2, pntr, order*sat_proj_elements[1].size);
      for (l = 0; l < order; l++)
        grid->gridbands[ref].projtosat[l].b[j] = darr2[l];
      pntr += order*sat_proj_elements[1].size;
    }

    /* Close the SDS.
     */
    SDendaccess (sat_proj_sds);
    free (darr2);
    free (databuf);

  } /* Loop over the bands. */

  /***********************************************************
   * Close the file.
   */
  SDend (sd_id);

  return (SUCCESS);
}

/******************************************************************************
NAME:           hxx_SOM_grid_calc

PURPOSE:
flip the grid coords if the data is SOM, so we can perfom calculations with
the coordinates

RETURN VALUE:   Type = int
Value    Description
-----    -----------
SUCCESS  Successful completion
ERROR    Operation failed

NOTES:   This function was written so that we can have the SOM logic in one place
		 and hxx_get_grid and tmresample can call it.  This function only flips 
		 SOM corner coorindates after they are read.  This allows a consistant i
		 user interface, yet allows the algorithms using SOM coodinates not to 
		 change.

ALGORITHM REFERENCES:
See hxx_grid.h for a description of the grid structure.

******************************************************************************/
void hxx_SOM_grid_calc (
  hxx_GRID_TYPE *grid)       /* I/O: Grid structure to be output            */
{
double tmp;		     /* Temporary variable for flipping corner points*/

/*
  Flip the corner points for the SOM projection ONLY
*/
if (grid->code == SOM)
    {
    tmp = -grid->corners.upleft.x;
    grid->corners.upleft.x = grid->corners.upleft.y;
    grid->corners.upleft.y = tmp;

    tmp = -grid->corners.loleft.x;
    grid->corners.loleft.x = grid->corners.loleft.y;
    grid->corners.loleft.y = tmp;

    tmp = -grid->corners.upright.x;
    grid->corners.upright.x = grid->corners.upright.y;
    grid->corners.upright.y = tmp;

    tmp = -grid->corners.loright.x;
    grid->corners.loright.x = grid->corners.loright.y;
    grid->corners.loright.y = tmp;
    }
}
