/******************************************************************************
NAME:		hxx_open_1g_band

PURPOSE:	Open a band of the 1G image.

RETURN VALUE: 
Type = int32
Value           Description
-----           -----------
band id		HDF SDS id used to access the band
FAIL     	HDF failure code--Band not opened (not found)

NOTES:		THIS ROUTINE SHOULD ONLY BE CALLED BY OTHER GPS 1G I/O ROUTINES
******************************************************************************/
#include <stdio.h>
#include <unistd.h>
#include "hxx_hdf_io.h"
#include "xxx_Types.h"
#include "xxx_LogStatus.h"

int32 hxx_open_1g_band (
	HYPHDFIO *hdfid)		/* I-HDF info structure for the image	*/
{
int32	sds_index,		/* HDF SDS index number		*/
	sds_id;			/* HDF SDS id			*/
char	msg[STRLEN];		/* error message		*/

/*
 * Initialize the sds id
 */
sds_id = -1;

/*
 * Construct the name of the SDS for this band
 */
hxx_create_band_name(hdfid->hdf_dir_name, "SDS", hdfid->sds_name);

/*
 * Select the user-specified band
 */
sds_index = SDnametoindex(hdfid->sd_id, hdfid->sds_name);
if (sds_index == FAIL)
    {
    /* xxx_LogStatus("HXXLIB", __FILE__, __LINE__, "Imagery not found"); */
    return(FAIL); /* band not found */
    }

sds_id = SDselect(hdfid->sd_id, sds_index);
if (sds_id == FAIL)
    {
    sprintf("SDS %s not found",hdfid->sds_name);
    xxx_LogStatus("HXXLIB", __FILE__, __LINE__, msg);
    return(FAIL);
    }

return(sds_id);
}
