/******************************************************************************
NAME:		hxx_read_1g_image

PURPOSE:	Read the specified 1G image.

RETURN VALUE: 
Type = int
Value           Description
-----           -----------
SUCCESS     	Successfully read image
ERROR     	Error occurred; image not read


******************************************************************************/
#include <stdio.h>
#include "xxx_Types.h"
#include "xxx_LogStatus.h"
#include "hxx_hdf_io.h"

int hxx_read_1g_image (
	const HYPHDFIO *hdfid,	/* I-HDF IO structure			*/
	int sb,
	int sl,
	int ss,
	int nb,
	int nl,
	int ns,
	int subb,
	int subl,
	int subs,
	void *data)		/* O-Data buffer			*/
{
intn	hdf_status;		/* HDF function return status	*/
int32	start[3],		/* starting data location	*/
	edge[3],		/* size of data buffer		*/
	stride[3];		/* subsampling factor		*/
char	msg[STRLEN];		/* Error message string		*/

start[0] = sb; /* start is 0-relative */
start[1] = sl;
start[2] = ss;
edge[0] = nb;
edge[1] = nl;
edge[2] = ns;
stride[0] = subb;
stride[1] = subl;
stride[2] = subs;

/*
 * Check the structure
 */
if (hdfid == NULL)
    {
    sprintf(msg,"Error reading from 1G image");
    xxx_LogStatus("HXXLIB", __FILE__, __LINE__, msg);
    return(ERROR);
    }

/*
 * Set the search path 
 */
HXsetdir(hdfid->extdir);

hdf_status = SDreaddata(hdfid->sds_id, start, stride, edge, (VOIDP)data);
if (hdf_status == FAIL)
    {
    sprintf(msg,"Error reading from SDS %s", hdfid->sds_name);
    xxx_LogStatus("HXXLIB", __FILE__, __LINE__, msg);
    return(ERROR);
    }

return(SUCCESS);
}
