/******************************************************************************
NAME:		hxx_write_1g_image

PURPOSE:	Write to the specified 1G image.

RETURN VALUE: 
Type = int
Value           Description
-----           -----------
SUCCESS     	Successfully wrote image
ERROR     	Error occurred; image not written


ALGORITHM REFERENCES:
******************************************************************************/
#include <stdio.h>
#include "xxx_Types.h"
#include "xxx_LogStatus.h"
#include "hxx_hdf_io.h"

int hxx_write_1g_image (
	const HYPHDFIO *hdfid,	/* I-HDF IO structure			*/
	int  sb,		/* I-Band to start writing (0-rel)	*/
        int  sl,                /* I-Line to start writing (0-rel)	*/
        int  ss,                /* I-Sample to start writing (0-rel)	*/
	int  nb,		/* I-Number of bands to write		*/
        int  nl,                /* I-Number of lines to write   	*/
        int  ns,                /* I-Number of samples to write 	*/
        int  subb,              /* I-Subsampling factor         	*/
        int  subl,              /* I-Subsampling factor         	*/
        int  subs,              /* I-Subsampling factor         	*/
        void *data)             /* O-Data buffer			*/
{
intn	hdf_status;		/* HDF function return status	*/
int32	start[3],		/* starting data location	*/
	stride[3],		/* subsampling factor		*/
	edge[3];		/* size of data buffer		*/
char	msg[STRLEN];		/* Error message string		*/

/*
 * Set up the structure
 */
if (hdfid == NULL)
    {
    sprintf(msg,"Error writing to 1G image");
    xxx_LogStatus("HXXLIB", __FILE__, __LINE__, msg);
    return(ERROR);
    }

/*
 * Set up the specification for the block to write
 */
start[0] = sb; /* start is 0-relative */
start[1] = sl; 
start[2] = ss;
edge[0] = nb;
edge[1] = nl;
edge[2] = ns;
stride[0] = subb;
stride[1] = subl;
stride[2] = subs;

hdf_status = SDwritedata(hdfid->sds_id, start, stride, edge, (VOIDP)data);

if (hdf_status == FAIL)
   {
   sprintf(msg, "Error writing to SDS %s",hdfid->sds_name);
   xxx_LogStatus("HXXLIB", __FILE__, __LINE__, msg);
   return(ERROR);
   }

return(SUCCESS);
}
