#ifndef XXX_ALIPCD
#define XXX_ALIPCD

#include "mfhdf.h"	      /* HDF definitions */

#include "xxx_ACSHK.h"          /* Describes ACS telemetry structure */
#include "xxx_ALIHK.h"          /* Describes ALI telemetry structure */
#include "xxx_GPSHK.h"          /* Describes GPS telemetry structure */

#define	MAX_DCE	256           /* Max number of DCEs in a housekeeping file (arbitrary) */

/* Define DCE descriptor data structure */
typedef struct DCE_Desc
{
  double  start_time;         /* DCE start time in seconds from GPS epoch */
  double  stop_time;          /* DCE stop time in seconds from GPS epoch */
  double  acs_start_tc;       /* Time code of first ACS telemetry record in DCE */
  double  acs_stop_tc;        /* Time code of last ACS telemetry record in DCE */
  int     ali_start_index;    /* Index of first ALI telemetry record in DCE */
  int     ali_stop_index;     /* Index of last ALI telemetry record in DCE */
  int     gps_start_index;    /* Index of first GPS telemetry record in DCE */
  int     gps_stop_index;     /* Index of last GPS telemetry record in DCE */
  int     utcf_index;         /* Index of GPS UTCF time offset record for DCE */
} DCE_Desc;

/* Define ALI PCD data structure */
typedef struct ALIPCD
{
  int		active_DCE;           /* Index of currently selected DCE */
  int		num_DCE;              /* Number of DCEs in this housekeeping interval */
  DCE_Desc	DCE_List[MAX_DCE];    /* Array of DCE descriptors */
  ACS		*ACS_Group;           /* Pointer to ACS housekeeping data */
  ALI		*ALI_Group;           /* Pointer to ALI housekeeping data */
  GPS		*GPS_Group;           /* Pointer to GPS housekeeping data */
} ALIPCD;

/* Routine to load ALI PCD data structure */
int xxx_read_ALIPCD( ALIPCD *pcd, char *ddir, char *froot );

/* Utility to construct path name (fname) from directory, file root, and extension */
void xxx_hkfname( char *fname, char *ddir, char *froot, const char *ext );

/* Routine to locate DCEs in housekeeping list */
int xxx_find_DCEs( ALIPCD *pcd );

/* Routine to check for housekeeping data consistency */
int xxx_check_hk( ALIPCD *pcd );

/* Routine to find the index of a telemetry field based on time code */
int xxx_find_index( double time_code, telem_field field );

/* Routine to free the dynamically allocated memory in the ALIPCD structure */
void xxx_free_ALIPCD( ALIPCD *pcd );

#endif

