/******************************************************************************
                Landsat 7 Image Assessment System (IAS) Software
                        Property of the U.S. Government
                            NASA / GSFC / Code 510
*******************************************************************************

UNIT NAME:   xxx_FindDirectory.c

PURPOSE:  Find the directory path for the give file path name.

INVOCATION METHOD:
	Status = xxx_FindDirectory(p_Directory, p_PathName)

RETURN VALUE:
Type = void

Value         	Description
--------------	----------------------------------------------------------------
none

ARGUMENT LIST:
Name            Type              	Use	Description
--------------- ------------------	---	---------------------------------------
p_Directory	char *			I	Path name to directory
p_PathName	char *			I	Path name to some file

EXTERNAL FILE/RECORD/DBTABLE REFERENCES:
Name                      Use     Description
------------------------- ---     -------------------------------------------------
xxx_Imagery.h             I       Defines imagery structures, constants, and prototype

EXTERNAL VARIABLES:
Source          Name                  Type  Use Description
--------------- -------------         ----- --- -----------------------
none

EXTERNAL UNIT/MACRO/PROCEDURE/FUNCTION REFERENCES:
Name           		Description
--------------- 	-------------------------------------------------
none

ABNORMAL TERMINATION CONDITIONS, ERROR MESSAGES:
none

ASSUMPTIONS, CONSTRAINTS, RESTRICTIONS:
none

DEVELOPMENT HISTORY:
Author            Change-ID   Release Date       Description of Change
----------------- ----------- ------- ---------- ------------------------------
J. Kerich			R1.0  02/29/97   Original Implementation

NOTES:
Pass in "/usr/fred/data.hdf" you get "/usr/fred" as the directory path returned.
Pass in "data.hdf" you get "." as the directory path returned.

Unit xxx_FindDirectory PDL:

        Copy the p_PathName to p_Directory
1	IF the p_Directory has a file name at the end THEN
                Chop off the file name from p_Directory
2       ELSE
		Set p_Directory to "."
	ENDIF

End Unit xxx_FindDirectory
******************************************************************************/

/* System headers */
#include <stdio.h>
#include <string.h>

/* Local headers */
#include <xxx_Imagery.h>

void xxx_FindDirectory(char *p_Directory, char *p_PathName)
{
	char    *p_Slash, *p_LastSlash;

	(void)strcpy(p_Directory, p_PathName);

	for (p_LastSlash = p_Slash = p_Directory; (p_Slash = strstr(p_Slash, "/")); )
	{
		p_LastSlash = p_Slash;
		p_Slash++;
	}

	if (p_LastSlash[0] == '/')
	{
		p_LastSlash[0] = '\0';
	}
	else
	{
	    (void)strcpy(p_Directory, ".");
	}
}
