/******************************************************************************
*                              PACOR II SOFTWARE
*                      Property of the U.S. Government
*                              NASA/GSFC/Code 560
*******************************************************************************
*
* UNIT NAME:
*     xxx_GetTime.c
*
* PURPOSE:
*     To create a time and/or date stamp.
*
* INVOCATION METHOD:
*     int xxx_GetTime(p_stamp, stampsize, p_format)
*
* RETURN VALUES:
*
* VALUE               DESCRIPTION
* ---------------------------------------------------
*  SUCCESS            Successfully created time stamp.
*
*  ERROR              Error creating time stamp.
*
* ARGUMENT LIST:
*
* NAME            TYPE          USE        DESCRIPTION
* -------------   -------       ---        -------------------------
* p_stamp         char *        I/O        Pointer to string to place time/date.
* stampsize       int           I          Size of p_stamp.
* p_format        const char *  I          Pointer to format string.
* 
* EXTERNAL VARIABLES:
*
* SOURCE          NAME          TYPE  USE DESCRIPTION
* ---------------------------------------------------
* xxx_Types.h     SUCCESS       int    O  Stamp successfully created.
* xxx_Types.h     ERROR         int    O  Error creating time stamp.
*
* EXTERNAL FILE/RECORD REFERENCES:
*
* NAME           USE    DESCRIPTION
* -------------  ---    -----------------
* xxx_GetTime.h  I      Defines return values
* xxx_Types.h    I      Defines return values
* time.h         I      Defines system call error codes
* 
* EXTERNAL UNIT/MACRO/PROCEDURE/FUNCTION REFERENCES:
*
* NAME             DESCRIPTION
* ----------------------------
* NONE
*
* ABNORMAL TERMINATION CONDITIONS, ERROR MESSAGES:
*  NONE
*
* ASSUMPTIONS, CONSTRAINTS, RESTRICTIONS:
*  1) The input p_stamp must be large enough to hold the result of the
*     format specification.
*
* DEVELOPMENT HISTORY:
*
* AUTHOR        CHANGE-ID RELEASE  DATE     DESCRIPTION OF CHANGE
* ------------- --------- -------- -------- ----------------------
* W. Zepp/CSC     N/A     R0 B1    07/21/93 Original Implementation
* J. kerich                R1.0    03/24/97 Ported to IAS
*
* NOTES:
*
*  1) The input p_format will contain the format specification of the 
*     time stamp.  Format specifications can be found in the UNIX manual
*     page of strftime(3).
*        Examples:
*              "%H:%M:%S"     14:55:20
*              "%X on %x"     4:25 PM on 7/13/93
*              "%c"           Sun Apr 19 16:10:20 1993
*
*  2) The input p_stamp must be large enough to hold the result of the
*     format specification.
*
******************************************************************************/
/******************************************************************************
UNIT xxx_GetTime PDL
-------------------------------------------------------------------------------

         Get the system time
         Format into p_stamp using p_format and stampsize
1        IF error formating time THEN
            RETURN ERROR
2        ENDIF
 
         RETURN SUCCESS
   
-------------------------------------------------------------------------------
END UNIT xxx_GetTime PDL
******************************************************************************/

/* System headers */
#include <time.h>
#include <string.h>

/* Local headers */
#include <xxx_Types.h>
#include <xxx_GetTime.h>

int xxx_GetTime(char *p_stamp, int  stampsize, const char *p_format)
{
      time_t   ptime;               /*  Time in seconds  */

      ptime = time((time_t *) 0);

      /* added clear string, mbs dec 98 */
      memset(p_stamp,0,stampsize);

      if (strftime(p_stamp, stampsize, p_format, gmtime(&ptime) ) <= 0) {
           return ERROR;
      }

      return SUCCESS;
}
