/******************************************************************************
                Landsat 7 Image Assessment System (IAS) Software
                        Property of the U.S. Government
                            NASA / GSFC / Code 510
*******************************************************************************

UNIT NAME: xxx_LogStatus.c

PURPOSE: Echo an application's status message to standard out.  This function
is intended to be called by Radiometry (RPS) and Geometry(GPS) functions only and NOT the Process Control (PCS) and Data Management (DMS) subsystems.
         
INVOCATION METHOD:
         xxx_LogStatus (p_program, p_function, line_number, p_msg)

RETURN VALUE:
Type = void

ARGUMENT LIST:
Name             Type               Use Description
---------------- ------------------ --- ---------------------------------------
p_program        char *             I   Program name
p_function       char *             I   Filename of calling unit (see C macro __FILE__) 
line_number      int                I   Line number of calling unit (see C macro "__LINE__) 
p_msg            char *             I   Error message to write to standard out

EXTERNAL FILE/RECORD/DBTABLE REFERENCES:
Name                      Use Description
------------------------- --- -------------------------------------------------
none

EXTERNAL VARIABLES:
Source          Name                Type            Use Description
--------------- ------------------- --------------- --- -----------------------
none

EXTERNAL UNIT/MACRO/PROCEDURE/FUNCTION REFERENCES:
Name                          Description
----------------------------- -------------------------------------------------
xxx_GetTime		      Retrieves current time

ABNORMAL TERMINATION CONDITIONS, ERROR MESSAGES:
none

ASSUMPTIONS, CONSTRAINTS, RESTRICTIONS:
none

DEVELOPMENT HISTORY:
Author            Change-ID   Release Date       Description of Change
----------------- ----------- ------- ---------- ------------------------------
A Williard        n/a         R1.0    1/30/97    Original PDL

NOTES:

Unit "xxx_LogStatus" PDL:
    Call xxx_GetTimeStamp() to retrieve current time stamp 
    Write the following information to standard out:
      p_program, p_function, line_number, process id, date/time, and p_msg
 

End Unit "xxx_LogStatus"
******************************************************************************/

/* System headers */
#include <errno.h>
#include <fcntl.h>
#include <netdb.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <sys/param.h>
#include <sys/stat.h>

/* External functions */
#include <xxx_GetTime.h>
#include <xxx_Types.h>
#include <xxx_LogStatus.h>

/* Main */

void xxx_LogStatus(char *p_program, char *p_function, int LineNumber, char *p_msg)
{
	char TimeStamp[18];

	(void) xxx_GetTime(TimeStamp, sizeof(TimeStamp),"%Y %j %H:%M:%S");
	(void)printf("%17s  %3s   %-7ld   %-20s    %6d    %s\n",
      	  TimeStamp, p_program, getpid(), p_function, LineNumber, p_msg);
	return;
}


