#ifndef _XXX_ALI_LEVEL0R_H
#define _XXX_ALI_LEVEL0R_H

#include "hdf.h"
#include "mfhdf.h"

#include "xxx_const.h"                /* STRLEN and definitions */

/* -----------------------------------
   Level 0 R Constants
   ----------------------------------- */
#define         MAX_SDS_DIM     3
#define		VALID		1
#define		INVALID		0
#define		MAX_SCAS	4   /* Maximum number of SCAs */
#define		N_FILE_ATTRS	13  /* For L0R there are 13 file attributes */
#define		N_OLD_FILE_ATTRS	11  /* For L0R there were 11 file attributes */
#define		N_DATASETS	8   /* For L0R there are 8 data sets */
#define		N_BANDS_IDX	0   /* Index of dim_sizes for number of bands */
#define		N_FRAMES_IDX	1   /* Index of dim_sizes for number of frames */
#define		N_DETS_IDX	2   /* Index of dim_sizes for number of detectors */
#define		PAN_BAND	9   /* 0-referenced index to pan band */
#define		NUM_BAND	10  /* Number of bands */


/* -----------------------------------
   Level 0 R Structures
   ----------------------------------- */
typedef struct L0R_CAL_ATTRIBUTES
{
  char8 ali_sensor[5];
  char8 dataset_type[13];
  float32 integration_time;
  float32 frame_rate;
  char8 data_gate_open_time[21];
  char8 data_gate_close_time[21];
  int16 number_cross_track_pixels;
  int32 number_along_track_pixels;
  int16 number_of_bands;
  int16 percent_missing_data;
  uint8 valid_map[10];
} L0R_CAL_ATTRIBUTES;

typedef struct L0R_IMAGE_ATTRIBUTES
{
  char8 ali_sensor[5];
  char8 dataset_type[13];
  char8 data_start_time[15];
  float32 data_duration;
  float32 integration_time;
  float32 frame_rate;
  char8 data_gate_open_time[21];
  char8 data_gate_close_time[21];
  int32 bytes_prior_sync;
  int16 number_cross_track_pixels;
  int32 number_along_track_pixels;
  int16 number_of_bands;
  int16 percent_missing_data;
  uint8 valid_map[13];
} L0R_IMAGE_ATTRIBUTES;

typedef struct L0R_HEADER
{
  char8 filename[STRLEN];
  int16 data_product_level;
  int16 software_version[3];
  int16 big_little_endian;
  char8 time_of_generation[15];
  int16 number_of_related_files;
  char8 related_file1[STRLEN]; 
  char8 related_file2[STRLEN]; 
  char8 related_file3[STRLEN]; 
  char8 related_file4[STRLEN]; 
  char8 processing_notes[STRLEN];
  char8 scene_id[STRLEN];
  char8 target_description[STRLEN];
} L0R_HEADER;

typedef struct L0R_CAL
{
  int32 number_of_dims;
  int32 dim_sizes[MAX_SDS_DIM];
  char8 name[9];
  int32 HDF_type;
  int32 number_of_attributes;
  L0R_CAL_ATTRIBUTES attributes;
  int16 *data;
} L0R_CAL;

typedef struct L0R_IMAGE
{
  int32 number_of_dims;
  int32 dim_sizes[MAX_SDS_DIM];
  char8 name[9];
  int32 HDF_type;
  int32 number_of_attributes;
  L0R_IMAGE_ATTRIBUTES attributes;
  int16 *data;
} L0R_IMAGE;

typedef struct LEVEL0R
{
  L0R_HEADER header;
  L0R_CAL precal[NUM_BAND];
  L0R_IMAGE image[NUM_BAND];
  L0R_CAL dark[NUM_BAND];
  L0R_CAL lamp[NUM_BAND];
} LEVEL0R;

/* Access Functions */
int xxx_read_ali_level0r(
char *l0r_name,         	/* I: Name of Level 0R file             */
LEVEL0R *level0r);      	/* I: Structure containing data         */

int xxx_read_ali_l0r_header(
int32 sd_id,            	/* I: SD file ID	 		*/
L0R_HEADER *p_header);  	/* O: Structure containing L0R header 	*/

int xxx_read_ali_l0r_cal(
int32 sd_id,                    /* I: SD identifier             	*/
int32 sds_index,                /* I: SDS index         		*/
L0R_CAL *cal);                  /* O: Structure containing L1R data     */

int xxx_read_ali_l0r_image(
int32 sd_id,                    /* I: Name of Level 0R file             */
int32 sds_index,                /* Index of current SDS                 */
L0R_IMAGE *image);              /* O: Structure containing L0R data     */

int xxx_write_ali_level0r(
char *l0r_name,         	/* I: Name of Level 0R file             */
LEVEL0R *level0r);       	/* I: Structure containing data         */

int xxx_write_ali_l0r_cal(
int32 sd_id,                    /* I: SD identifier                     */
L0R_CAL cal);                   /* I: Structure containing L0R data     */

int xxx_write_ali_l0r_image(
int32 sd_id,                    /* I: SD identifier                     */
L0R_IMAGE image);               /* I: Structure containing L0R data     */

int xxx_close_ali_level0r(
LEVEL0R *level0r);      	/* I: Structure containing data         */

#endif
