#ifndef	XXX_ALI_LEVEL1R_H
#define	XXX_ALI_LEVEL1R_H

#include "hdf.h"
#include "mfhdf.h"
#include "xxx_const.h"

#define		MAX_SDS_DIM	3
#define		VALID	1
#define		INVALID	0
#define		PAN_BAND	9
#define		NUM_BAND	10

typedef struct L1R_CAL_ATTRIBUTES
{
  char8 dataset_type[13];
  char8 ali_sensor[5];
  char8 data_gate_open_time[21];
  char8 data_gate_close_time[21];
  int16 number_cross_track_pixels;
  int32 number_along_track_pixels;
  int16 number_of_bands;
  int16 scale_factor;
  uint8 valid_map[8];
} L1R_CAL_ATTRIBUTES;

typedef struct L1R_IMAGE_ATTRIBUTES
{
  char8 dataset_type[13];
  char8 ali_sensor[5];
  char8 data_start_time[15];
  float32 data_duration;
  float32 integration_time;
  float32 frame_rate;
  char8 data_gate_open_time[21];
  char8 data_gate_close_time[21];
  int32 bytes_prior_sync;
  int16 number_cross_track_pixels;
  int32 number_along_track_pixels;
  int16 number_of_bands;
  int16 percent_missing_data;
  int32 frame_shift[10];
  uint8 valid_map[23];
} L1R_IMAGE_ATTRIBUTES;

typedef struct L1R_HEADER
{
  char8 filename[33];
  int16 data_product_level;
  int16 software_version[3];
  int16 big_little_endian;
  char8 time_of_generation[15];
  char8 type_of_DCE[5];
  int16 number_of_related_files;
  char8 related_file1[33]; 
  char8 related_file2[33]; 
  char8 related_file3[33]; 
  char8 related_file4[33]; 
  char8 processing_notes[22];
  int16 cal_pipe_software_version[3];
  char8 cal_pipe_processing_notes[73];
  char8 scene_id[STRLEN];
  char8 target_description[STRLEN];
} L1R_HEADER;

typedef struct L1R_IMAGE
{
  int32 number_of_dims;
  int32 dim_sizes[3];
  char8 name[9];
  int32 HDF_type;
  int32 number_of_attributes;
  L1R_IMAGE_ATTRIBUTES attributes;
  int16 *data;
} L1R_IMAGE;

typedef struct L1R_CAL
{
  int32 number_of_dims;
  int32 dim_sizes[3];
  char8 name[9];
  int32 HDF_type;
  int32 number_of_attributes;
  L1R_CAL_ATTRIBUTES attributes;
  float32 *data;
} L1R_CAL;

typedef struct LEVEL1R
{
  L1R_HEADER header;
  L1R_IMAGE image[NUM_BAND];
  L1R_CAL offset[NUM_BAND];
  L1R_CAL response[NUM_BAND];
} LEVEL1R;

/* Function Declarations */
/* Close a Level 1R data set */
int xxx_close_ali_level1r(
	LEVEL1R *level1r );	/* I: Structure containing L1R data	*/

/* Read/load a Level 1R data set */
int xxx_read_ali_level1r(
	char *l1r_name,		/* I: Name of Level 1R file		*/
	LEVEL1R *level1r);	/* O: Structure containing L1R data	*/

/* Read/load a Level 1R header */
int xxx_read_ali_l1r_header(
        int32 sd_id,		/* I: Input SDS ID			*/
        L1R_HEADER *p_header );	/* O: Structure containing header info	*/

/* Read/load a Level 1R cal SDS */
int xxx_read_ali_l1r_cal(
        int32 sd_id,		/* I: Input SD ID			*/
        int32 sds_index,	/* I: Input SDS index			*/
        L1R_CAL *cal );		/* O: Structure containing cal data	*/

/* Read/load a Level 1R image SDS */
int xxx_read_ali_l1r_image(
        int32 sd_id,		/* I: Input SD ID			*/
        int32 sds_index,	/* I: Input SDS index			*/
        L1R_IMAGE *image );	/* O: Structure containing image data	*/

/* Write a Level 1R data set */
int xxx_write_ali_level1r(
	char *l1r_name,		/* I: Name of Level 1R file		*/
	LEVEL1R level1r);	/* I: Structure containing data		*/

/* Write a Level 1R cal SDS */
int xxx_write_ali_l1r_cal(
        int32 sd_id,		/* I: SD identifier			*/
        L1R_CAL cal );		/* I: Structure containing cal data	*/

/* Write a Level 1R image SDS */
int xxx_write_ali_l1r_image(
        int32 sd_id,		/* I: SD identifier			*/
        L1R_IMAGE image );	/* I: Structure containing image data	*/

#endif
