#include <stdio.h>
#include <math.h>

#include "xxx_ALIPCD.h"

#define	EO1_TIME(a,b,c)	(a*256*256 + b + c/256/256)
#define	MIN_DCE_DURATION	4.0
#define ALI_ON	1
#define	ALI_OFF	0
#define	EPS	0.001	/* Maximum allowable time delta (1 msec) */

int xxx_find_index( double time_code, telem_field field );

int xxx_find_DCEs( ALIPCD *pcd ) 
{
  int		num_timerec;	/* Number of time records in telemetry */
  int		curr_index;	/* Current ALI telemetry record index */
  int		ali_state;	/* Flag indicating whether ALI is on or off */
  int		back_count;	/* Counts number of telemetry points added */
  double	curr_start;	/* Current data start time */
  double	curr_stop;	/* Current data stop time */
  double	last_start;	/* Previous data start time */
  double	last_stop;	/* Previous data stop time */

  /* Check the housekeeping data */
  if ( xxx_check_hk( pcd ) == FAIL )
  {
    printf("\nHousekeeping data validation failure!\n\n");
    return(FAIL);
  }

  /* Get the number of records from the start time most-significant-word telemetry count */
  num_timerec = pcd->ALI_Group->starttime_msw.count;

  /* Initialize start/stop times (note these are MET times here,
     they'll be converted to GPS times later) */
  curr_index = 0;
  last_start = EO1_TIME( pcd->ALI_Group->starttime_msw.value.d[curr_index],
                         pcd->ALI_Group->starttime_lsw.value.d[curr_index],
                         pcd->ALI_Group->starttime_sub.value.d[curr_index] );
  last_stop = EO1_TIME( pcd->ALI_Group->stoptime_msw.value.d[curr_index],
                        pcd->ALI_Group->stoptime_lsw.value.d[curr_index],
                        pcd->ALI_Group->stoptime_sub.value.d[curr_index] );
  ali_state = ALI_OFF;

  /* Loop through the ALI time records */
  while ( curr_index < num_timerec )
  {
    curr_start = EO1_TIME( pcd->ALI_Group->starttime_msw.value.d[curr_index],
                           pcd->ALI_Group->starttime_lsw.value.d[curr_index],
                           pcd->ALI_Group->starttime_sub.value.d[curr_index] );
    curr_stop = EO1_TIME( pcd->ALI_Group->stoptime_msw.value.d[curr_index],
                          pcd->ALI_Group->stoptime_lsw.value.d[curr_index],
                          pcd->ALI_Group->stoptime_sub.value.d[curr_index] );

    /* See if we are starting a new DCE */
    if ( ali_state == ALI_OFF &&
         curr_start > last_start &&
         curr_stop <= last_stop )
    {
      /* Start new DCE */
      pcd->num_DCE++;
      pcd->active_DCE = pcd->num_DCE - 1;
      pcd->DCE_List[pcd->active_DCE].start_time = curr_start;
      pcd->DCE_List[pcd->active_DCE].ali_start_index = curr_index;
      ali_state = ALI_ON;
    }

    /* See if we are ending an ongoing DCE */
    if ( ali_state == ALI_ON &&
         curr_stop > last_stop &&
         curr_stop > last_start )
    {
      /* Close out DCE */
      pcd->DCE_List[pcd->active_DCE].stop_time = curr_stop;
      pcd->DCE_List[pcd->active_DCE].ali_stop_index = curr_index;
      ali_state = ALI_OFF;
      /* Check for a DCE which is long enough */
      if ( pcd->DCE_List[pcd->active_DCE].stop_time
         - pcd->DCE_List[pcd->active_DCE].start_time < MIN_DCE_DURATION )
        pcd->num_DCE--;
    }

    /* On to the next record */
    curr_index++;
    last_start = curr_start;
    last_stop = curr_stop;
  }

  /* Complete the DCE information */
  for ( pcd->active_DCE=0; pcd->active_DCE < pcd->num_DCE; pcd->active_DCE++ )
  {
    /* Get the DCE start time code */
    curr_start = pcd->ALI_Group->starttime_msw.time[pcd->DCE_List[pcd->active_DCE].ali_start_index];
 
    /* Find the GPS UTCF record that immediately follows this time code */
    curr_index = 0;
    while ( pcd->GPS_Group->utcftime.time[curr_index] < curr_start &&
            curr_index < pcd->GPS_Group->utcftime.count ) curr_index++;
    pcd->DCE_List[pcd->active_DCE].utcf_index = curr_index;

    /* Convert DCE start/stop times from MET to GPS epoch */
    pcd->DCE_List[pcd->active_DCE].start_time += pcd->GPS_Group->utcftime.value.d[curr_index];
    pcd->DCE_List[pcd->active_DCE].stop_time += pcd->GPS_Group->utcftime.value.d[curr_index];

    /* Reset curr_start to current DCE GPS start time */
    curr_start = pcd->DCE_List[pcd->active_DCE].start_time;

    /* Now find the corresponding GPS telemetry using the GPS epoch times */
    curr_index = 0;
    back_count = 0;
    while ( pcd->GPS_Group->acsutctime.value.d[curr_index] < curr_start &&
            curr_index < pcd->GPS_Group->acsutctime.count ) curr_index++;
    /* Now back up until there's a time break */
    while ( curr_index > 0 &&
            back_count++ < 60 &&
            fabs(pcd->GPS_Group->acsutctime.value.d[curr_index]
               - pcd->GPS_Group->acsutctime.value.d[curr_index-1] - 1.0) < EPS ) curr_index--;
    pcd->DCE_List[pcd->active_DCE].gps_start_index = curr_index;
    /* Now go forward until there's a time break */
    back_count = 0;
    while ( curr_index+1 < pcd->GPS_Group->acsutctime.count &&
            back_count++ < 150 &&
            fabs(pcd->GPS_Group->acsutctime.value.d[curr_index+1]
               - pcd->GPS_Group->acsutctime.value.d[curr_index] - 1.0) < EPS ) curr_index++;
    pcd->DCE_List[pcd->active_DCE].gps_stop_index = curr_index;

    /* Now find the corresponding ACS telemetry using the time codes */
    pcd->DCE_List[pcd->active_DCE].acs_start_tc =
         pcd->GPS_Group->acsutctime.time[pcd->DCE_List[pcd->active_DCE].gps_start_index];
    pcd->DCE_List[pcd->active_DCE].acs_stop_tc =
         pcd->GPS_Group->acsutctime.time[pcd->DCE_List[pcd->active_DCE].gps_stop_index];
  }

  return(SUCCEED);
}

int xxx_check_hk( ALIPCD *pcd )
{
  /* Check time record counts */
  if ( pcd->ALI_Group->starttime_msw.count != pcd->ALI_Group->starttime_lsw.count ||
       pcd->ALI_Group->starttime_msw.count != pcd->ALI_Group->starttime_sub.count ||
       pcd->ALI_Group->starttime_msw.count != pcd->ALI_Group->stoptime_msw.count ||
       pcd->ALI_Group->starttime_msw.count != pcd->ALI_Group->stoptime_lsw.count ||
       pcd->ALI_Group->starttime_msw.count != pcd->ALI_Group->stoptime_sub.count )
  {
    printf("\nALI Start/Stop Time field sizes are inconsistent!\n\n");
    return(FAIL);
  }

  /* Check time record time codes */
  if ( fabs(pcd->ALI_Group->starttime_msw.time[0] - pcd->ALI_Group->starttime_lsw.time[0]) > EPS ||
       fabs(pcd->ALI_Group->starttime_msw.time[0] - pcd->ALI_Group->starttime_sub.time[0]) > EPS ||
       fabs(pcd->ALI_Group->starttime_msw.time[0] - pcd->ALI_Group->stoptime_msw.time[0]) > EPS ||
       fabs(pcd->ALI_Group->starttime_msw.time[0] - pcd->ALI_Group->stoptime_lsw.time[0]) > EPS ||
       fabs(pcd->ALI_Group->starttime_msw.time[0] - pcd->ALI_Group->stoptime_sub.time[0]) > EPS )
  {
    printf("\nALI Start/Stop Time field time origins are inconsistent!\n\n");
    return(FAIL);
  }

  /* More stuff could be checked here */

  return(SUCCEED);
}
