#include <stdio.h>
#include <math.h>
#include "xxx_Types.h"
#include "xxx_LogStatus.h"
#include "xxx_const.h"

#include "xxx_HYPPCD.h"

#define EPS     		0.001   /* Maximum allowable time delta (1 msec) */
#define	MIN_DCE_DURATION	4.0
#define HYP_ON			1
#define	HYP_OFF			0

int xxx_find_index( double time_code, telem_field field );

int xxx_find_DCEs_hyp( HYPPCD *pcd ) 
{
  int		num_timerec;	/* Number of time records in telemetry */
  int		curr_index;	/* Current Hyperion telemetry record index */
  int		hyp_state;	/* Flag indicating whether Hyperion is on or off */
  int		back_count;	/* Counts number of telemetry points added */
  int		start_index;	/* Start of Hyperion on */
  int		index;		/* Counter */
  double	curr_start;	/* Current data start time */
  double	curr_stop;	/* Current data stop time */
  double	last_start;	/* Previous data start time */
  double	last_stop;	/* Previous data stop time */
  char		emsg[STRLEN];	/* Error message string */

  /* Initialize loop variables */
  pcd->num_DCE = 0;
  pcd->active_DCE = 0;
  start_index = 0;
  hyp_state = HYP_OFF;

  if( pcd->HYP_Group->yimage_mode.count != 0 )
  {
      sprintf(emsg,"Using IMAGEMODEL mnuemonic for DCE");
      xxx_LogStatus("XXXLIB",__FILE__,__LINE__,emsg);

     /* Go through HDF file checking to see when image_model is greater
        than 1.  This will indicate that the Hyperion instrument is
        turned on. */
     for ( index=0; index<pcd->HYP_Group->yimage_mode.count; index++ )
     {
       if( pcd->HYP_Group->yimage_mode.value.us[index] > 1 ) 
       {
         if( hyp_state == HYP_OFF )
         {
           hyp_state = HYP_ON;
           start_index = index;
         }
       }
       else
       {
         if( hyp_state == HYP_ON )
         {
           /* Check to see if start and stop indexes are far enough apart
              to indicate imagery. */
           if( (index-start_index) > 12 )
           {
             printf("Hyperion imagery from %d %d (%d)\n",start_index,index,(index-start_index));
             /* pcd->DCE_List[pcd->num_DCE].gps_start_index = start_index; */
             /* pcd->DCE_List[pcd->num_DCE].gps_stop_index = index-1;      */
             pcd->DCE_List[pcd->num_DCE].hyp_start_index = start_index;
             pcd->DCE_List[pcd->num_DCE].hyp_stop_index = index-1;
             start_index = index;
             pcd->num_DCE++;
           }
           hyp_state = HYP_OFF;
         }
       }
     }
  }
  else
  {
     sprintf(emsg,"Using COVERPOS mnuemonic for DCE");
     xxx_LogStatus("XXXLIB",__FILE__,__LINE__,emsg);

    curr_start = 
        pcd->HYP_Group->ycoverpos.time[0];
    index = 0;
    while( curr_start>=pcd->GPS_Group->acsutctime.time[index] &&
           index<pcd->GPS_Group->acsutctime.count )
    {
          index++;
    }
    if( index>=0 && index<pcd->GPS_Group->acsutctime.count )
        printf("Cover position time duration %f ",pcd->GPS_Group->acsutctime.value.d[index]);
    curr_start =   
        pcd->HYP_Group->ycoverpos.time[pcd->HYP_Group->ycoverpos.count-1];
    index = 0;
    while( curr_start>=pcd->GPS_Group->acsutctime.time[index] && 
           index<pcd->GPS_Group->acsutctime.count )
    {
          index++;
    }
    if( index>=0 && index<pcd->GPS_Group->acsutctime.count )
        printf("%f\n",pcd->GPS_Group->acsutctime.value.d[index]);

    /* Go through HDF file checking to see when image_model is greater
        than 1.  This will indicate that the Hyperion instrument is
        turned on. */
     for ( index=0; index<pcd->HYP_Group->ycoverpos.count; index++ )
     {
       /* Cover is open when YCOVERPOS ~2678.  This check may need to change
        * according to this being an approximate value.
        */
       if( pcd->HYP_Group->ycoverpos.value.d[index] < 2700 )
       {
         if( hyp_state == HYP_OFF )
         {
           hyp_state = HYP_ON;
           start_index = index;
         }
       }
       else
       {
         if( hyp_state == HYP_ON )
         {
           /* Check to see if start and stop indexes are far enough apart
              to indicate imagery. */
           if( (index-start_index) > 20 )
           {
             printf("Hyperion imagery from %d %d (%d)\n",start_index,index,(index-start_index));
             /* pcd->DCE_List[pcd->num_DCE].gps_start_index = start_index; */
             /* pcd->DCE_List[pcd->num_DCE].gps_stop_index = index-1;      */
             pcd->DCE_List[pcd->num_DCE].hyp_start_index = start_index;
             pcd->DCE_List[pcd->num_DCE].hyp_stop_index = index-1;
             start_index = index;
             pcd->num_DCE++;
           }
           hyp_state = HYP_OFF;
         }
       }
     }
  }
   
  /* Loop through DCEs that were found above, find corresponding GPS 
     information. */
  for ( pcd->active_DCE=0; pcd->active_DCE < pcd->num_DCE; pcd->active_DCE++ )
  {
    if( pcd->HYP_Group->yimage_mode.count != 0 )
       curr_start = 
           pcd->HYP_Group->yimage_mode.time[pcd->DCE_List[pcd->active_DCE].hyp_start_index];
    else
       curr_start = 
           pcd->HYP_Group->ycoverpos.time[pcd->DCE_List[pcd->active_DCE].hyp_start_index];
    index = 0;
    while( curr_start>=pcd->GPS_Group->acsutctime.time[index] && 
           index<pcd->GPS_Group->acsutctime.count )
    {
          index++;
    }
    if( index>0 && index<pcd->GPS_Group->acsutctime.count )
       pcd->DCE_List[pcd->active_DCE].start_time = pcd->GPS_Group->acsutctime.value.d[index];
    else
    {
      sprintf(emsg,"Error finding start time for DCE");
      xxx_LogStatus("XXXLIB",__FILE__,__LINE__,emsg);
      return(ERROR);
    }
    pcd->DCE_List[pcd->active_DCE].gps_start_index = index; 
    if( pcd->HYP_Group->yimage_mode.count != 0 )
       curr_start 
           = pcd->HYP_Group->yimage_mode.time[pcd->DCE_List[pcd->active_DCE].hyp_stop_index];
    else
       curr_start 
           = pcd->HYP_Group->ycoverpos.time[pcd->DCE_List[pcd->active_DCE].hyp_stop_index];
    index = 0;
    while( curr_start>=pcd->GPS_Group->acsutctime.time[index] && 
           index<pcd->GPS_Group->acsutctime.count )
    {
          index++;
    }
    if( index>0 && index<(pcd->GPS_Group->acsutctime.count-1) )
      pcd->DCE_List[pcd->active_DCE].stop_time = pcd->GPS_Group->acsutctime.value.d[index];
    else
    {
      sprintf(emsg,"Error finding stop time for DCE");
      xxx_LogStatus("XXXLIB",__FILE__,__LINE__,emsg);
      return(ERROR);
    }
    pcd->DCE_List[pcd->active_DCE].gps_stop_index = index-1;
  }

  return(SUCCEED);
}
