#include "mfhdf.h"
#include "xxx_Telemetry.h"

int xxx_get_telem_field( int32 sd_id, telem_field *field )
{
  int32	sds_id;				/* ID number of selected SDS */
  int32	sds_index;			/* HDF file index of desired SDS */
  char  sds_name[VSNAMELENMAX+1]; 	/* SDS name from HDF file */
  int32 rank;				/* SDS rank (number of dimensions) */
  int32 data_type;			/* SDS data type */
  int32 dim_sizes[MAX_VAR_DIMS]; 	/* SDS dimensions */
  int32 n_attrs;			/* Number of SDS atrributes */
  int32	start[MAX_VAR_DIMS];		/* SDS read offset */
  int32	stride[MAX_VAR_DIMS];		/* SDS read interval */
  int32	edges[MAX_VAR_DIMS];		/* SDS elements to read */

  /* Get file index for telemetry time code SDS */
  if ( (sds_index = SDnametoindex( sd_id, field->time_mnemonic )) == FAIL )
  {
    printf("\nUnable to locate field mnemonic %s\n\n", field->time_mnemonic);
    return FAIL;
  }

  /* Connect to time code SDS */
  if ( (sds_id = SDselect( sd_id, sds_index )) == FAIL )
  {
    printf("\nError connecting to field mnemonic %s\n\n", field->time_mnemonic);
    return FAIL;
  }

  /* Get size information from time code SDS */
  if ( SDgetinfo(sds_id, sds_name, &rank, dim_sizes, &data_type, &n_attrs) == FAIL )
  {
    printf("\nError retrieving SDS info from %s\n\n", field->time_mnemonic);
    (void)SDendaccess(sds_id);
    return FAIL;
  }

  /* Check SDS rank */
  if ( rank != 1 )
  {
    printf("\nInvalid rank for SDS %s\n\n", field->time_mnemonic);
    (void)SDendaccess(sds_id);
    return FAIL;
  }

  /* Set telemetry field element count */
  field->count = dim_sizes[0];

  /* Check to see if there's any data here */
  if ( field->count > 0 )
  {

    /* Allocate telemetry field time code data memory */
    if ( (field->time = (double *)malloc( field->count*sizeof(double) )) == NULL )
    {
      printf("\nError allocating field memory for %s\n\n", field->time_mnemonic);
      (void)SDendaccess(sds_id);
      return FAIL;
    }
 
    /* Set up SDS read parameters */
    start[0] = 0;
    stride[0] = 1;
    edges[0] = field->count;

    /* Read SDS */
    if ( SDreaddata( sds_id, start, stride, edges, field->time ) == FAIL )
    {
      printf("\nError reading SDS %s\n\n", field->time_mnemonic);
      (void)SDendaccess(sds_id);
      return FAIL;
    }

  }
  else
  {
    printf("\nNo data present for SDS %s\n\n", field->time_mnemonic);
  }

  /* End SDS Access */
  (void)SDendaccess(sds_id);

  /* Get file index for telemetry data value SDS */
  if ( (sds_index = SDnametoindex( sd_id, field->value_mnemonic )) == FAIL )
  {
    printf("\nUnable to locate field mnemonic %s\n\n", field->value_mnemonic);
    return FAIL;
  }

  /* Connect to data value SDS */
  if ( (sds_id = SDselect( sd_id, sds_index )) == FAIL )
  {
    printf("\nError connecting to field mnemonic %s\n\n", field->value_mnemonic);
    return FAIL;
  }

  /* Get size information from data value SDS */
  if ( SDgetinfo(sds_id, sds_name, &rank, dim_sizes, &data_type, &n_attrs) == FAIL )
  {
    printf("\nError retrieving SDS info from %s\n\n", field->value_mnemonic);
    (void)SDendaccess(sds_id);
    return FAIL;
  }

  /* Check SDS rank */
  if ( rank != 1 )
  {
    printf("\nInvalid rank for SDS %s\n\n", field->value_mnemonic);
    (void)SDendaccess(sds_id);
    return FAIL;
  }

  /* Check telemetry field element count */
  if ( field->count != dim_sizes[0] )
  {
    printf("\nMismatch in time code and data value SDS sizes for %s\n\n", field->value_mnemonic);
    (void)SDendaccess(sds_id);
    return FAIL;
  }

  /* Check for available data */
  if ( field->count > 0 )
  {

    /* Set up SDS read parameters */
    start[0] = 0;
    stride[0] = 1;
    edges[0] = field->count;

    /* Allocate telemetry field data value memory */
    switch ( data_type )
    {
      case DFNT_FLOAT64:
      {
        if ( (field->value.d = (double *)malloc( field->count*sizeof(double) )) == NULL )
        {
          printf("\nError allocating field memory for %s\n\n", field->value_mnemonic);
          (void)SDendaccess(sds_id);
          return FAIL;
        }
 
        /* Read SDS */
        if ( SDreaddata( sds_id, start, stride, edges, field->value.d ) == FAIL )
        {
          printf("\nError reading SDS %s\n\n", field->value_mnemonic);
          (void)SDendaccess(sds_id);
          return FAIL;
        }
        break;
      }
      case DFNT_UINT16:
      {
        if ( (field->value.us = (uint16 *)malloc( field->count*sizeof(uint16) )) == NULL )
        {
          printf("\nError allocating field memory for %s\n\n", field->value_mnemonic);
          (void)SDendaccess(sds_id);
          return FAIL;
        }
 
        /* Read SDS */
        if ( SDreaddata( sds_id, start, stride, edges, field->value.us ) == FAIL )
        {
          printf("\nError reading SDS %s\n\n", field->value_mnemonic);
          (void)SDendaccess(sds_id);
          return FAIL;
        }
        break;
      }
      case DFNT_UINT8:
      {
        if ( (field->value.u = (uint8 *)malloc( field->count*sizeof(uint8) )) == NULL )
        {
          printf("\nError allocating field memory for %s\n\n", field->value_mnemonic);
          (void)SDendaccess(sds_id);
          return FAIL;
        }
 
        /* Read SDS */
        if ( SDreaddata( sds_id, start, stride, edges, field->value.u ) == FAIL )
        {
          printf("\nError reading SDS %s\n\n", field->value_mnemonic);
          (void)SDendaccess(sds_id);
          return FAIL;
        }
        break;
      }
      default:
      {
        printf("\nInvalid SDS data type %d\n\n", data_type);
        (void)SDendaccess(sds_id);
        return FAIL;
      }
    }
  }
  else
  {
    printf("\nNo data to read for field %s\n\n", field->value_mnemonic);
  }

  /* End SDS Access */
  (void)SDendaccess(sds_id);

  return SUCCEED;
}

