#include "mfhdf.h"
#include "xxx_ALIHK.h"

int xxx_read_ALI( int32 sd_id, ALI **ALI_Ptr )
{

  /* A slight change is made here.  Instead of the focal plane temp. label
     reading "A@IHSKP_FPE_TEMP(...", it currently (as of 8/31/2001) should
     read "A@IHSKP_T19(...".  If this changes in the future, it will have to
     be changed again. The label "P@IHSKP_FPE_TEMP has also been changed
     to "P@IHSKP_T19"... */

  ALI *ALI_Group;		/* Local version of ALI pointer */
  ALI ALI_Data={
    {"A@IHSKP_DATAGATE(TIME_MEASURED)","IHSKP_DATAGATE",0,NULL,NULL},
    {"A@IHSKP_T19(TIME_MEASURED)","P@IHSKP_T19",0,NULL,NULL},
    {"A@IHSKP_FPE_PAN_INT(TIME_MEASURED)","P@IHSKP_FPE_PAN_INT",0,NULL,NULL},
    {"A@IHSKP_FPE_LRATE(TIME_MEASURED)","P@IHSKP_FPE_LRATE",0,NULL,NULL},
    {"A@IHSKP_LAMPS(TIME_MEASURED)","P@IHSKP_LAMPS",0,NULL,NULL},
    {"A@IHSKP_DG_SET_MSW(TIME_MEASURED)","P@IHSKP_DG_SET_MSW",0,NULL,NULL},
    {"A@IHSKP_DG_SET_LSW(TIME_MEASURED)","P@IHSKP_DG_SET_LSW",0,NULL,NULL},
    {"A@IHSKP_DG_SET_SUB(TIME_MEASURED)","P@IHSKP_DG_SET_SUB",0,NULL,NULL},
    {"A@IHSKP_DG_CLR_MSW(TIME_MEASURED)","P@IHSKP_DG_CLR_MSW",0,NULL,NULL},
    {"A@IHSKP_DG_CLR_LSW(TIME_MEASURED)","P@IHSKP_DG_CLR_LSW",0,NULL,NULL},
    {"A@IHSKP_DG_CLR_SUB(TIME_MEASURED)","P@IHSKP_DG_CLR_SUB",0,NULL,NULL},
    {"A@IHSKP_CALSTAT(TIME_MEASURED)","IHSKP_CALSTAT",0,NULL,NULL},
    {"A@IHSKP_CALSTATUS(TIME_MEASURED)","P@IHSKP_CALSTATUS",0,NULL,NULL},
    {"A@IHSKP_INTTIME(TIME_MEASURED)","P@IHSKP_INTTIME",0,NULL,NULL},
    {"A@IHSKP_FPE_MS_INT(TIME_MEASURED)","P@IHSKP_FPE_MS_INT",0,NULL,NULL},
    {"A@IHSKP_TAC_STAT(TIME_MEASURED)","P@IHSKP_TAC_STAT",0,NULL,NULL},
    {"A@IHSKP_CALDIFF_STAT(TIME_MEASURED)","P@IHSKP_CALDIFF_STAT",0,NULL,NULL}
  };			/* ALI data structure template */

  /* Allocate ALI data structure */
  if ( (ALI_Group = (ALI *)malloc( sizeof( ALI ) )) == NULL )
  {
    printf("\nError allocating memory for ALI telemetry group\n\n");
    return FAIL;
  }

  /* Copy ALI template into new structure */
  memcpy( (void *)ALI_Group, (void *)&ALI_Data, sizeof( ALI ) );

  /* Read the ALI telemetry fields from their respective SDS's */
  if ( xxx_get_telem_field( sd_id, &ALI_Group->datamode ) == FAIL )
  {
    printf("\nError reading ALI datamode telemetry field\n\n");
    xxx_free_ALI( ALI_Group );
    return FAIL;
  }

  /* This section is needed to trend focal plane temperatures over time
     in the RPS database. */
  if ( xxx_get_telem_field( sd_id, &ALI_Group->fp_temp) == FAIL )
    {
      printf("\nError reading ALI fp_temp telemetry field\n\n");
      xxx_free_ALI( ALI_Group );
      return FAIL;
    }
  

  if ( xxx_get_telem_field( sd_id, &ALI_Group->pan_int_time) == FAIL )
  {
    printf("\nError reading ALI pan_int_time telemetry field\n\n");
    xxx_free_ALI( ALI_Group );
    return FAIL;
  }

  if ( xxx_get_telem_field( sd_id, &ALI_Group->frame_rate) == FAIL )
  {
    printf("\nError reading ALI frame_rate telemetry field\n\n");
    xxx_free_ALI( ALI_Group );
    return FAIL;
  }

  if ( xxx_get_telem_field( sd_id, &ALI_Group->dataset_type) == FAIL )
  {
    printf("\nError reading ALI dataset_type telemetry field\n\n");
    xxx_free_ALI( ALI_Group );
    return FAIL;
  }

  if ( xxx_get_telem_field( sd_id, &ALI_Group->starttime_msw) == FAIL )
  {
    printf("\nError reading ALI starttime_msw telemetry field\n\n");
    xxx_free_ALI( ALI_Group );
    return FAIL;
  }

  if ( xxx_get_telem_field( sd_id, &ALI_Group->starttime_lsw) == FAIL )
  {
    printf("\nError reading ALI starttime_lsw telemetry field\n\n");
    xxx_free_ALI( ALI_Group );
    return FAIL;
  }

  if ( xxx_get_telem_field( sd_id, &ALI_Group->starttime_sub) == FAIL )
  {
    printf("\nError reading ALI starttime_sub telemetry field\n\n");
    xxx_free_ALI( ALI_Group );
    return FAIL;
  }

  if ( xxx_get_telem_field( sd_id, &ALI_Group->stoptime_msw) == FAIL )
  {
    printf("\nError reading ALI stoptime_msw telemetry field\n\n");
    xxx_free_ALI( ALI_Group );
    return FAIL;
  }

  if ( xxx_get_telem_field( sd_id, &ALI_Group->stoptime_lsw) == FAIL )
  {
    printf("\nError reading ALI stoptime_lsw telemetry field\n\n");
    xxx_free_ALI( ALI_Group );
    return FAIL;
  }

  if ( xxx_get_telem_field( sd_id, &ALI_Group->stoptime_sub) == FAIL )
  {
    printf("\nError reading ALI stoptime_sub telemetry field\n\n");
    xxx_free_ALI( ALI_Group );
    return FAIL;
  }

  if ( xxx_get_telem_field( sd_id, &ALI_Group->suncal_pos) == FAIL )
  {
    printf("\nError reading ALI suncal_pos telemetry field\n\n");
    xxx_free_ALI( ALI_Group );
    return FAIL;
  }

  if ( xxx_get_telem_field( sd_id, &ALI_Group->cal_status) == FAIL )
  {
    printf("\nError reading ALI cal_status telemetry field\n\n");
    xxx_free_ALI( ALI_Group );
    return FAIL;
  }

  if ( xxx_get_telem_field( sd_id, &ALI_Group->int_time) == FAIL )
  {
    printf("\nError reading ALI int_time telemetry field\n\n");
    xxx_free_ALI( ALI_Group );
    return FAIL;
  }

  if ( xxx_get_telem_field( sd_id, &ALI_Group->ms_int_time) == FAIL )
  {
    printf("\nError reading ALI ms_int_time telemetry field\n\n");
    xxx_free_ALI( ALI_Group );
    return FAIL;
  }

  if ( xxx_get_telem_field( sd_id, &ALI_Group->cover_pos) == FAIL )
  {
    printf("\nError reading ALI cover_pos telemetry field\n\n");
    xxx_free_ALI( ALI_Group );
    return FAIL;
  }

  *ALI_Ptr = ALI_Group;
  return SUCCEED;
}

