#include "mfhdf.h"
#include "xxx_EFFHK.h"

int xxx_read_EFF( int32 sd_id, EFF **EFF_Ptr )
{
  EFF *EFF_Group;		/* Local version of EFF pointer */
  EFF EFF_Data={
    {"A@EFFSMODE(TIME_MEASURED)","EFFSMODE",0,NULL,NULL},
    {"A@P05AEFFUTCTIME(TIME_MEASURED)","P@P05AEFFUTCTIME",0,NULL,NULL},
    {"A@EFFLSXPA11(TIME_MEASURED)","P@EFFLSXPA11",0,NULL,NULL},
    {"A@EFFLSYPA11(TIME_MEASURED)","P@EFFLSYPA11",0,NULL,NULL},
    {"A@EFFLSZPA11(TIME_MEASURED)","P@EFFLSZPA11",0,NULL,NULL},
    {"A@EFFCXPA11(TIME_MEASURED)","P@EFFCXPA11",0,NULL,NULL},
    {"A@EFFCYPA11(TIME_MEASURED)","P@EFFCYPA11",0,NULL,NULL},
    {"A@EFFCZPA11(TIME_MEASURED)","P@EFFCZPA11",0,NULL,NULL},
    {"A@EFFRSVXPA11(TIME_MEASURED)","P@EFFRSVXPA11",0,NULL,NULL},
    {"A@EFFRSVYPA11(TIME_MEASURED)","P@EFFRSVYPA11",0,NULL,NULL},
    {"A@EFFRSVZPA11(TIME_MEASURED)","P@EFFRSVZPA11",0,NULL,NULL}
  };			/* EFF data structure template */

  /* Allocate EFF data structure */
  if ( (EFF_Group = (EFF *)malloc( sizeof( EFF ) )) == NULL )
  {
    printf("\nError allocating memory for EFF telemetry group\n\n");
    return FAIL;
  }

  /* Copy EFF template into new structure */
  memcpy( (void *)EFF_Group, (void *)&EFF_Data, sizeof( EFF ) );

  /* Read the EFF telemetry fields from their respective SDS's */
  if ( xxx_get_telem_field( sd_id, &EFF_Group->smode ) == FAIL )
  {
    printf("\nError reading EFF smode telemetry field\n\n");
    xxx_free_EFF( EFF_Group );
    return FAIL;
  }

  if ( xxx_get_telem_field( sd_id, &EFF_Group->utctime ) == FAIL )
  {
    printf("\nError reading EFF utctime telemetry field\n\n");
    xxx_free_EFF( EFF_Group );
    return FAIL;
  }

  if ( xxx_get_telem_field( sd_id, &EFF_Group->l7_xpos ) == FAIL )
  {
    printf("\nError reading EFF l7_xpos telemetry field\n\n");
    xxx_free_EFF( EFF_Group );
    return FAIL;
  }

  if ( xxx_get_telem_field( sd_id, &EFF_Group->l7_ypos ) == FAIL )
  {
    printf("\nError reading EFF l7_ypos telemetry field\n\n");
    xxx_free_EFF( EFF_Group );
    return FAIL;
  }

  if ( xxx_get_telem_field( sd_id, &EFF_Group->l7_zpos ) == FAIL )
  {
    printf("\nError reading EFF l7_zpos telemetry field\n\n");
    xxx_free_EFF( EFF_Group );
    return FAIL;
  }

  if ( xxx_get_telem_field( sd_id, &EFF_Group->eo1_xpos ) == FAIL )
  {
    printf("\nError reading EFF eo1_xpos telemetry field\n\n");
    xxx_free_EFF( EFF_Group );
    return FAIL;
  }

  if ( xxx_get_telem_field( sd_id, &EFF_Group->eo1_ypos ) == FAIL )
  {
    printf("\nError reading EFF eo1_ypos telemetry field\n\n");
    xxx_free_EFF( EFF_Group );
    return FAIL;
  }

  if ( xxx_get_telem_field( sd_id, &EFF_Group->eo1_zpos ) == FAIL )
  {
    printf("\nError reading EFF eo1_zpos telemetry field\n\n");
    xxx_free_EFF( EFF_Group );
    return FAIL;
  }

  if ( xxx_get_telem_field( sd_id, &EFF_Group->eo1_l7_relx ) == FAIL )
  {
    printf("\nError reading EFF eo1_l7_relx telemetry field\n\n");
    xxx_free_EFF( EFF_Group );
    return FAIL;
  }

  if ( xxx_get_telem_field( sd_id, &EFF_Group->eo1_l7_rely ) == FAIL )
  {
    printf("\nError reading EFF eo1_l7_rely telemetry field\n\n");
    xxx_free_EFF( EFF_Group );
    return FAIL;
  }

  if ( xxx_get_telem_field( sd_id, &EFF_Group->eo1_l7_relz ) == FAIL )
  {
    printf("\nError reading EFF eo1_l7_relz telemetry field\n\n");
    xxx_free_EFF( EFF_Group );
    return FAIL;
  }

  *EFF_Ptr = EFF_Group;
  return SUCCEED;
}

