/*************************************************************************
UNIT NAME:      xxx_read_ali_l0r_header.c 

PURPOSE: Reads a Level 0R ALI (from EO-1 instrument) header.  All data is 
         put in the structure, L0R_HEADER, defined in xxx_level0r.h.  It
         is up to the caller of this routine to access what data is
         needed by their routine and then to free up the memory
         associated with the structure xxx_level0r.h.
              
*************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <limits.h>
#include <unistd.h>
#include <sys/types.h>
#include <stddef.h>
#include "xxx_Types.h"
#include "xxx_LogStatus.h"
#include "xxx_ali_level0r.h"

int xxx_read_ali_l0r_header(
int32 sd_id,		/* I: SD file ID */
L0R_HEADER *p_header)	/* O: Structure containing L0R header */
{
   char emsg[STRLEN];		/* Error message buffer */
   char progname[STRLEN];	/* Program name */
   int32 file_attr_index;       /* Attribute index */

   sprintf(progname,"READ_L0R_HEADER");

   /* Initialize the header memory area.
   ------------------------------------*/
   memset( p_header, 0, sizeof(L0R_HEADER) );

   /* Read file attributes of SD header.
   ------------------------------------*/
   file_attr_index=0;
   if( SDreadattr(sd_id, file_attr_index, &p_header->filename )
       == FAIL )
   {
      sprintf(emsg,"Could not read attribute.");
      xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
      return(ERROR);
   }

   file_attr_index++;
   if( SDreadattr(sd_id, file_attr_index,
                  &p_header->data_product_level )
       == FAIL )
   {
      sprintf(emsg,"Could not read attribute.");
      xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
      return(ERROR);
   }

   file_attr_index++;
   if( SDreadattr(sd_id, file_attr_index,
                  p_header->software_version )
       == FAIL )
   {  
      sprintf(emsg,"Could not read attribute.");
      xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
      return(ERROR);
   }

   file_attr_index++;
   if( SDreadattr(sd_id, file_attr_index,
                  &p_header->big_little_endian )
       == FAIL )
   {  
      sprintf(emsg,"Could not read attribute.");
      xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
      return(ERROR);
   }

   file_attr_index++;
   if( SDreadattr(sd_id, file_attr_index,
                  p_header->time_of_generation )
       == FAIL )
   {  
      sprintf(emsg,"Could not read attribute.");
      xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
      return(ERROR);
   }

   file_attr_index++;
   if( SDreadattr(sd_id, file_attr_index,
                  &p_header->number_of_related_files )
       == FAIL )
   {  
      sprintf(emsg,"Could not read attribute.");
      xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
      return(ERROR);
   }

   file_attr_index++;
   if( SDreadattr(sd_id, file_attr_index,
                  p_header->related_file1 )
       == FAIL )
   {  
      sprintf(emsg,"Could not read attribute.");
      xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
      return(ERROR);
   }

   file_attr_index++;
   if( SDreadattr(sd_id, file_attr_index,
                  p_header->related_file2 )
       == FAIL )
   {  
      sprintf(emsg,"Could not read attribute.");
      xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
      return(ERROR);
   }
   file_attr_index++;

   if( SDreadattr(sd_id, file_attr_index,
                  p_header->related_file3 )
       == FAIL )
   {  
      sprintf(emsg,"Could not read attribute.");
      xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
      return(ERROR);
   }
   file_attr_index++;

   if( SDreadattr(sd_id, file_attr_index,
                  p_header->related_file4 )
       == FAIL )
   {  
      sprintf(emsg,"Could not read attribute.");
      xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
      return(ERROR);
   }
   file_attr_index++;

   if( SDreadattr(sd_id, file_attr_index,
                  p_header->processing_notes )
       == FAIL )
   { 
      sprintf(emsg,"Could not read attribute.");
      xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
      return(ERROR);
   }
   file_attr_index++;

   if( SDreadattr(sd_id, file_attr_index,
                  p_header->scene_id )
       == FAIL )
   { 
      sprintf(emsg,"No scene_id attribute.");
      xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
      (void)strcpy( (char *)p_header->scene_id, "None" );
   }
   file_attr_index++;

   if( SDreadattr(sd_id, file_attr_index,
                  p_header->target_description )
       == FAIL )
   { 
      sprintf(emsg,"No target_description attribute.");
      xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
      (void)strcpy( (char *)p_header->target_description, "None" );
   }

   return(SUCCESS);
}
