/*************************************************************************
UNIT NAME:      xxx_read_ali_l1r_header.c 

PURPOSE: Reads a Level 0R ALI (from EO-1 instrument) header.  All data is 
         put in the structure, L1R_HEADER, defined in xxx_level1r.h.  It
         is up to the caller of this routine to access what data is
         needed by their routine and then to free up the memory
         associated with the structure xxx_level1r.h.
              
*************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <limits.h>
#include <unistd.h>
#include <sys/types.h>
#include <stddef.h>
#include "xxx_Types.h"
#include "xxx_LogStatus.h"
#include "xxx_ali_level1r.h"

int xxx_read_ali_l1r_header(
int32 sd_id,		/* I: SD file ID */
L1R_HEADER *p_header)	/* O: Structure containing L0R header */
{
   char emsg[STRLEN];		/* Error message buffer */
   char progname[STRLEN];	/* Program name */
   int32 file_attr_index;       /* Attribute index */

   sprintf(progname,"READ_L1R_HEADER");

   /* Initialize header memory area.
   --------------------------------*/
   memset( p_header, 0, sizeof(L1R_HEADER) );

   /* Read file attributes of SD header.
   ------------------------------------*/
   if( ((file_attr_index = SDfindattr(sd_id, "Filename")) == FAIL) ||
       (SDreadattr(sd_id, file_attr_index, p_header->filename) == FAIL) )
   {
      sprintf(emsg,"Could not read attribute.");
      xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
      return(ERROR);
   }

   if( ((file_attr_index = SDfindattr(sd_id, "Data Product Level")) == FAIL) ||
       (SDreadattr(sd_id, file_attr_index, &p_header->data_product_level) == FAIL) )
   {
      sprintf(emsg,"Could not read attribute.");
      xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
      return(ERROR);
   }

   if( ((file_attr_index = SDfindattr(sd_id, "LZP Software Version")) == FAIL) ||
       (SDreadattr(sd_id, file_attr_index, p_header->software_version) == FAIL) )
   {  
      sprintf(emsg,"Could not read attribute.");
      xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
      return(ERROR);
   }

   if( ((file_attr_index = SDfindattr(sd_id, "Big (1) or Little (0) Endian")) == FAIL) ||
       (SDreadattr(sd_id, file_attr_index, &p_header->big_little_endian) == FAIL) )
   {  
      sprintf(emsg,"Could not read attribute.");
      xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
      return(ERROR);
   }

   if( ((file_attr_index = SDfindattr(sd_id, "Time of File Generation")) == FAIL) ||
       (SDreadattr(sd_id, file_attr_index, p_header->time_of_generation) == FAIL) )
   {  
      sprintf(emsg,"Could not read attribute.");
      xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
      return(ERROR);
   }

   if( ((file_attr_index = SDfindattr(sd_id, "Type of DCE")) == FAIL) ||
       (SDreadattr(sd_id, file_attr_index, p_header->type_of_DCE) == FAIL) )
   {  
      sprintf(emsg,"Could not read attribute.");
      xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
      return(ERROR);
   }

   if( ((file_attr_index = SDfindattr(sd_id, "Number Related Files")) == FAIL) ) 
   {
      p_header->number_of_related_files = 0;
   }
   else if( (SDreadattr(sd_id, file_attr_index, &p_header->number_of_related_files) == FAIL) )
   {  
      sprintf(emsg,"Could not read Number Related Files attribute.");
      xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
      return(ERROR);
   }

   if( ((file_attr_index = SDfindattr(sd_id, "Related File 1")) == FAIL) )
   {  
      (void)strcpy( (char *)p_header->related_file1, "None" );
   }
   else if((SDreadattr(sd_id, file_attr_index, p_header->related_file1) == FAIL) )
   {  
      sprintf(emsg,"Could not read Related File 1 attribute.");
      xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
      return(ERROR);
   }

   if( ((file_attr_index = SDfindattr(sd_id, "Related File 2")) == FAIL) )
   {  
      (void)strcpy( (char *)p_header->related_file2, "None" );
   }
   else if((SDreadattr(sd_id, file_attr_index, p_header->related_file2) == FAIL) )
   {  
      sprintf(emsg,"Could not read Related File 2 attribute.");
      xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
      return(ERROR);
   }

   if( ((file_attr_index = SDfindattr(sd_id, "Related File 3")) == FAIL) )
   {  
      (void)strcpy( (char *)p_header->related_file3, "None" );
   }
   else if((SDreadattr(sd_id, file_attr_index, p_header->related_file3) == FAIL) )
   {  
      sprintf(emsg,"Could not read Related File 3 attribute.");
      xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
      return(ERROR);
   }

   if( ((file_attr_index = SDfindattr(sd_id, "Related File 4")) == FAIL) )
   {  
      (void)strcpy( (char *)p_header->related_file4, "None" );
   }
   else if((SDreadattr(sd_id, file_attr_index, p_header->related_file4) == FAIL) )
   {  
      sprintf(emsg,"Could not read Related File 4 attribute.");
      xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
      return(ERROR);
   }

   if( ((file_attr_index = SDfindattr(sd_id, "Level 0 Processing Notes")) == FAIL) ||
       (SDreadattr(sd_id, file_attr_index, p_header->processing_notes) == FAIL) )
   { 
      sprintf(emsg,"Could not read attribute.");
      xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
      return(ERROR);
   }

   if( ((file_attr_index = SDfindattr(sd_id, "Cal Pipe Software Version")) == FAIL) ||
       (SDreadattr(sd_id, file_attr_index, p_header->cal_pipe_software_version) == FAIL) )
   { 
      sprintf(emsg,"Could not read attribute.");
      xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
      return(ERROR);
   }

   if( ((file_attr_index = SDfindattr(sd_id, "Cal Pipe Processing Notes")) == FAIL) ||
       (SDreadattr(sd_id, file_attr_index, p_header->cal_pipe_processing_notes) == FAIL) )
   { 
      sprintf(emsg,"Could not read attribute.");
      xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
      return(ERROR);
   }

   if( (file_attr_index = SDfindattr(sd_id, "Scene_Id")) == FAIL )
   {
      (void)strcpy( (char *)p_header->scene_id, "None" );
   }
   else if( SDreadattr(sd_id, file_attr_index, p_header->scene_id) == FAIL )
   { 
      sprintf(emsg,"Could not read attribute.");
      xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
      return(ERROR);
   }

   if( (file_attr_index = SDfindattr(sd_id, "Target_Description")) == FAIL )
   {
      (void)strcpy( (char *)p_header->target_description, "None" );
   }
   else if( SDreadattr(sd_id, file_attr_index, p_header->target_description) == FAIL )
   { 
      sprintf(emsg,"Could not read Target_Description attribute.");
      xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
      return(ERROR);
   }

   return(SUCCESS);
}
