/******************************************************************************
UNIT NAME:      write_level0r.c 

PURPOSE: Write a Level 0R ALI (from EO-1 instrument) data set.  
               
*******************************************************************************/
#include <stdio.h>
#include "xxx_Types.h"
#include "xxx_LogStatus.h"
#include "xxx_const.h"
#include "xxx_ali_level0r.h"

#define PAN_BAND 9       /* 0-referenced index to pan band */

int xxx_write_ali_level0r(
char *l0r_name,		/* I: Name of Level 0R file		*/
LEVEL0R *level0r)	/* I: Structure containing data		*/
{
   char emsg[STRLEN];		/* Error message buffer			*/
   char progname[STRLEN];	/* Progname				*/
   int32 sd_id;                 /* SD file ID                           */
   int32 sds_id;                /* SDS file ID                          */
   int32 start[3];		/* Starting dims of SDS "slab"		*/
   int32 edges[3];		/* Edge dims of SDS "slab"		*/
   int32 stride[3];		/* Increment dims of SDS "slab"		*/

   strcpy(progname,"WRITE_L0R");

   /* Get SD file ID
   ----------------*/
   if ((sd_id = SDstart(l0r_name, DFACC_CREATE)) == FAIL)
   {
      strcpy(emsg,"Could not open SDS file\n");
      xxx_LogStatus(progname,__FILE__,__LINE__, emsg);
      return(ERROR);
   }

   /* Set SD attributes.
   -------------------*/
   if( SDsetattr(sd_id, "Filename", DFNT_CHAR8, 32, level0r->header.filename ) == FAIL )
   {
      strcpy(emsg,"Could not write attribute.\n");
      xxx_LogStatus(progname,__FILE__,__LINE__, emsg);
      return(ERROR);
   }
   if( SDsetattr(sd_id, "Data Product Level", DFNT_INT16, 1, 
                 &level0r->header.data_product_level ) == FAIL )
   {
      strcpy(emsg,"Could not write attribute.\n");
      xxx_LogStatus(progname,__FILE__,__LINE__, emsg);
      return(ERROR);
   }
   if( SDsetattr(sd_id, "LZP Software Version", DFNT_INT16, 3, 
                 level0r->header.software_version ) == FAIL )
   {  
      strcpy(emsg,"Could not write attribute.\n");
      xxx_LogStatus(progname,__FILE__,__LINE__, emsg);
      return(ERROR);
   }
   if( SDsetattr(sd_id, "Big (1) or Little (0) Endian", DFNT_INT16, 1, 
                 &level0r->header.big_little_endian ) == FAIL )
   {  
      strcpy(emsg,"Could not write attribute.\n");
      xxx_LogStatus(progname,__FILE__,__LINE__, emsg);
      return(ERROR);
   }
   if( SDsetattr(sd_id, "Time of File Generation", DFNT_CHAR8, 14, 
                 level0r->header.time_of_generation ) == FAIL )
   {  
      strcpy(emsg,"Could not write attribute.\n");
      xxx_LogStatus(progname,__FILE__,__LINE__, emsg);
      return(ERROR);
   }
   if( SDsetattr(sd_id, "Number Related Files", DFNT_INT16, 1, 
                 &level0r->header.number_of_related_files ) == FAIL )
   {  
      strcpy(emsg,"Could not write attribute.\n");
      xxx_LogStatus(progname,__FILE__,__LINE__, emsg);
      return(ERROR);
   }
   if( SDsetattr(sd_id, "Related File 1", DFNT_CHAR8, 32, level0r->header.related_file1 ) == FAIL )
   {  
      strcpy(emsg,"Could not write attribute.\n");
      xxx_LogStatus(progname,__FILE__,__LINE__, emsg);
      return(ERROR);
   }
   if( SDsetattr(sd_id, "Related File 2", DFNT_CHAR8, 32, level0r->header.related_file2 ) == FAIL )
   {  
      strcpy(emsg,"Could not write attribute.\n");
      xxx_LogStatus(progname,__FILE__,__LINE__, emsg);
      return(ERROR);
   }
   if( SDsetattr(sd_id, "Related File 3", DFNT_CHAR8, 32, level0r->header.related_file3 ) == FAIL )
   {  
      strcpy(emsg,"Could not write attribute.\n");
      xxx_LogStatus(progname,__FILE__,__LINE__, emsg);
      return(ERROR);
   }
   if( SDsetattr(sd_id, "Related File 4", DFNT_CHAR8, 32, level0r->header.related_file4 ) == FAIL )
   {  
      strcpy(emsg,"Could not write attribute.\n");
      xxx_LogStatus(progname,__FILE__,__LINE__, emsg);
      return(ERROR);
   }
   if( SDsetattr(sd_id, "Level 0 Processing Notes", DFNT_CHAR8, 22,
                 level0r->header.processing_notes ) == FAIL )
   { 
      strcpy(emsg,"Could not write attribute.\n");
      xxx_LogStatus(progname,__FILE__,__LINE__, emsg);
      return(ERROR);
   }
   if( SDsetattr(sd_id, "Scene_Id", DFNT_CHAR8, 22,
                 level0r->header.scene_id ) == FAIL )
   { 
      strcpy(emsg,"Could not write attribute.\n");
      xxx_LogStatus(progname,__FILE__,__LINE__, emsg);
      return(ERROR);
   }
   if( SDsetattr(sd_id, "Target_Description", DFNT_CHAR8, 22,
                 level0r->header.target_description ) == FAIL )
   { 
      strcpy(emsg,"Could not write attribute.\n");
      xxx_LogStatus(progname,__FILE__,__LINE__, emsg);
      return(ERROR);
   }

   /***********************************
    * Write precal data multispectral * 
    ***********************************/
   if ( xxx_write_ali_l0r_cal( sd_id, level0r->precal[0] ) != SUCCESS )
   {
      sprintf(emsg,"Could not write multispectral precal SDS.");
      xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
      return(ERROR);
   }

   /**********************************
    * Write precal data panchromatic * 
    **********************************/
   if ( xxx_write_ali_l0r_cal( sd_id, level0r->precal[PAN_BAND] ) != SUCCESS )
   {
      sprintf(emsg,"Could not write panchromatic precal SDS.");
      xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
      return(ERROR);
   }

   /*****************************
    * Write image multispectral * 
    *****************************/
   if ( xxx_write_ali_l0r_image( sd_id, level0r->image[0] ) != SUCCESS )
   {
      sprintf(emsg,"Could not write multispectral image SDS.");
      xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
      return(ERROR);
   }

   /****************************
    * Write image panchromatic * 
    ****************************/
   if ( xxx_write_ali_l0r_image( sd_id, level0r->image[PAN_BAND] ) != SUCCESS )
   {
      sprintf(emsg,"Could not write panchromatic image SDS.");
      xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
      return(ERROR);
   }

   /*********************************
    * Write dark data multispectral * 
    *********************************/
   if ( xxx_write_ali_l0r_cal( sd_id, level0r->dark[0] ) != SUCCESS )
   {
      sprintf(emsg,"Could not write multispectral dark SDS.");
      xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
      return(ERROR);
   }

   /********************************
    * Write dark data panchromatic * 
    ********************************/
   if ( xxx_write_ali_l0r_cal( sd_id, level0r->dark[PAN_BAND] ) != SUCCESS )
   {
      sprintf(emsg,"Could not write panchromatic dark SDS.");
      xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
      return(ERROR);
   }

   /*********************************
    * Write lamp data multispectral * 
    *********************************/
   if ( xxx_write_ali_l0r_cal( sd_id, level0r->lamp[0] ) != SUCCESS )
   {
      sprintf(emsg,"Could not write multispectral lamp SDS.");
      xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
      return(ERROR);
   }

   /********************************
    * Write lamp data panchromatic * 
    ********************************/
   if ( xxx_write_ali_l0r_cal( sd_id, level0r->lamp[PAN_BAND] ) != SUCCESS )
   {
      sprintf(emsg,"Could not write panchromatic lamp SDS.");
      xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
      return(ERROR);
   }


   /* Close the SDS file
   --------------------*/
   if ( SDend( sd_id ) == FAIL )
   {
      strcpy(emsg,"Closing SD file ID failed!\n ");
      xxx_LogStatus(progname,__FILE__,__LINE__, emsg);
      return(ERROR);
   }

   return(SUCCESS);
}
