/******************************************************************************
UNIT NAME:      write_level1r.c 

PURPOSE: Write a Level 1R ALI (from EO-1 instrument) data set.  
               
*******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <limits.h>
#include <unistd.h>
#include <sys/types.h>
#include <stddef.h>
#include "xxx_Types.h"
#include "xxx_LogStatus.h"
#include "xxx_const.h"
#include "xxx_ali_level1r.h"

int xxx_write_ali_level1r(
char *l1r_name,		/* I: Name of Level 1R file		*/
LEVEL1R level1r)	/* I: Structure containint data		*/
{
   char emsg[STRLEN];		/* Error message buffer			*/
   char progname[STRLEN];	/* Progname				*/
   int32 sd_id;                 /* SD file ID                           */
   int32 sds_id;                /* SDS file ID                          */
   int32 start[3];		/* Starting dims of SDS "slab"		*/
   int32 edges[3];		/* Edge dims of SDS "slab"		*/
   int32 stride[3];		/* Increment dims of SDS "slab"		*/

   strcpy(progname,"WRITE_LEVEL1R");

   /* Get SD file ID
   ----------------*/
   if ((sd_id = SDstart(l1r_name, DFACC_CREATE)) == FAIL)
   {
      strcpy(emsg,"Could not open SDS file\n");
      xxx_LogStatus(progname,__FILE__,__LINE__, emsg);
      return(ERROR);
   }

   /* Set SD attributes.
   -------------------*/
   if( SDsetattr(sd_id, "Filename", DFNT_CHAR8, 32, level1r.header.filename ) == FAIL )
   {
      strcpy(emsg,"Could not write attribute.\n");
      xxx_LogStatus(progname,__FILE__,__LINE__, emsg);
      return(ERROR);
   }
   if( SDsetattr(sd_id, "Data Product Level", DFNT_INT16, 1, 
                 &level1r.header.data_product_level ) == FAIL )
   {
      strcpy(emsg,"Could not write attribute.\n");
      xxx_LogStatus(progname,__FILE__,__LINE__, emsg);
      return(ERROR);
   }
   if( SDsetattr(sd_id, "LZP Software Version", DFNT_INT16, 3, 
                 level1r.header.software_version ) == FAIL )
   {  
      strcpy(emsg,"Could not write attribute.\n");
      xxx_LogStatus(progname,__FILE__,__LINE__, emsg);
      return(ERROR);
   }
   if( SDsetattr(sd_id, "Big (1) or Little (0) Endian", DFNT_INT16, 1, 
                 &level1r.header.big_little_endian ) == FAIL )
   {  
      strcpy(emsg,"Could not write attribute.\n");
      xxx_LogStatus(progname,__FILE__,__LINE__, emsg);
      return(ERROR);
   }
   if( SDsetattr(sd_id, "Time of File Generation", DFNT_CHAR8, 14, 
                 level1r.header.time_of_generation ) == FAIL )
   {  
      strcpy(emsg,"Could not write attribute.\n");
      xxx_LogStatus(progname,__FILE__,__LINE__, emsg);
      return(ERROR);
   }
   if( SDsetattr(sd_id, "Type of DCE", DFNT_CHAR8, 4, level1r.header.type_of_DCE ) == FAIL )
   {  
      strcpy(emsg,"Could not write attribute.\n");
      xxx_LogStatus(progname,__FILE__,__LINE__, emsg);
      return(ERROR);
   }
   if( SDsetattr(sd_id, "Number Related Files", DFNT_INT16, 1, 
                 &level1r.header.number_of_related_files ) == FAIL )
   {  
      strcpy(emsg,"Could not write attribute.\n");
      xxx_LogStatus(progname,__FILE__,__LINE__, emsg);
      return(ERROR);
   }
   if( SDsetattr(sd_id, "Related File 1", DFNT_CHAR8, 32, level1r.header.related_file1 ) == FAIL )
   {  
      strcpy(emsg,"Could not write attribute.\n");
      xxx_LogStatus(progname,__FILE__,__LINE__, emsg);
      return(ERROR);
   }
   if( SDsetattr(sd_id, "Related File 2", DFNT_CHAR8, 32, level1r.header.related_file2 ) == FAIL )
   {  
      strcpy(emsg,"Could not write attribute.\n");
      xxx_LogStatus(progname,__FILE__,__LINE__, emsg);
      return(ERROR);
   }
   if( SDsetattr(sd_id, "Related File 3", DFNT_CHAR8, 32, level1r.header.related_file3 ) == FAIL )
   {  
      strcpy(emsg,"Could not write attribute.\n");
      xxx_LogStatus(progname,__FILE__,__LINE__, emsg);
      return(ERROR);
   }
   if( SDsetattr(sd_id, "Related File 4", DFNT_CHAR8, 32, level1r.header.related_file4 ) == FAIL )
   {  
      strcpy(emsg,"Could not write attribute.\n");
      xxx_LogStatus(progname,__FILE__,__LINE__, emsg);
      return(ERROR);
   }
   if( SDsetattr(sd_id, "Level 0 Processing Notes", DFNT_CHAR8, 21,
                 level1r.header.processing_notes ) == FAIL )
   { 
      strcpy(emsg,"Could not write attribute.\n");
      xxx_LogStatus(progname,__FILE__,__LINE__, emsg);
      return(ERROR);
   }
   if( SDsetattr(sd_id, "Cal Pipe Software Version", DFNT_INT16, 3, 
                 &level1r.header.cal_pipe_software_version ) == FAIL )
   { 
      strcpy(emsg,"Could not write attribute.\n");
      xxx_LogStatus(progname,__FILE__,__LINE__, emsg);
      return(ERROR);
   }
   if( SDsetattr(sd_id, "Cal Pipe Processing Notes", DFNT_CHAR8, 72, 
                 level1r.header.cal_pipe_processing_notes ) == FAIL )
   {
      strcpy(emsg,"Could not write attribute.\n");
      xxx_LogStatus(progname,__FILE__,__LINE__, emsg);
      return(ERROR);
   }
   if( SDsetattr(sd_id, "Scene_Id", DFNT_CHAR8, 22, 
                 level1r.header.scene_id ) == FAIL )
   {
      strcpy(emsg,"Could not write attribute.\n");
      xxx_LogStatus(progname,__FILE__,__LINE__, emsg);
      return(ERROR);
   }
   if( SDsetattr(sd_id, "Target_Description", DFNT_CHAR8, 22, 
                 level1r.header.target_description ) == FAIL )
   {
      strcpy(emsg,"Could not write attribute.\n");
      xxx_LogStatus(progname,__FILE__,__LINE__, emsg);
      return(ERROR);
   }

   /**********************************
    * Write image data multispectral * 
    **********************************/
   if ( xxx_write_ali_l1r_image( sd_id, level1r.image[0] ) != SUCCESS )
   {
      strcpy(emsg,"Could not write multispectral image SDS\n");
      xxx_LogStatus(progname,__FILE__,__LINE__, emsg);
      return(ERROR);
   }

   /******************************
    * Write Offset multispectral * 
    ******************************/
   if ( xxx_write_ali_l1r_cal( sd_id, level1r.offset[0] ) != SUCCESS )
   {
      strcpy(emsg,"Could not write multispectral offset SDS\n");
      xxx_LogStatus(progname,__FILE__,__LINE__, emsg);
      return(ERROR);
   }

   /********************************
    * Write response multispectral * 
    ********************************/
   if ( xxx_write_ali_l1r_cal( sd_id, level1r.response[0] ) != SUCCESS )
   {
      strcpy(emsg,"Could not write multispectral response SDS\n");
      xxx_LogStatus(progname,__FILE__,__LINE__, emsg);
      return(ERROR);
   }

   /************************
    * Write image data pan * 
    ************************/
   if ( xxx_write_ali_l1r_image( sd_id, level1r.image[PAN_BAND] ) != SUCCESS )
   {
      strcpy(emsg,"Could not write panchromatic image SDS\n");
      xxx_LogStatus(progname,__FILE__,__LINE__, emsg);
      return(ERROR);
   }

   /********************
    * Write Offset pan * 
    ********************/
   if ( xxx_write_ali_l1r_cal( sd_id, level1r.offset[PAN_BAND] ) != SUCCESS )
   {
      strcpy(emsg,"Could not write panchromatic offset SDS\n");
      xxx_LogStatus(progname,__FILE__,__LINE__, emsg);
      return(ERROR);
   }

   /**********************
    * Write response pan * 
    **********************/
   if ( xxx_write_ali_l1r_cal( sd_id, level1r.response[PAN_BAND] ) != SUCCESS )
   {
      strcpy(emsg,"Could not write panchromatic response SDS\n");
      xxx_LogStatus(progname,__FILE__,__LINE__, emsg);
      return(ERROR);
   }

   /* Close the SDS file
   --------------------*/
   if ( SDend( sd_id ) == FAIL )
   {
      strcpy(emsg,"Closing SD file ID failed!\n ");
      xxx_LogStatus(progname,__FILE__,__LINE__, emsg);
      return(ERROR);
   }

   return(SUCCESS);
}
