/******************************************************************************
                Landsat 7 Image Assessment System (IAS) Software
                        Property of the U.S. Government
                            NASA / GSFC / Code 510
*******************************************************************************

UNIT NAME:  xdb_Defines.h

PURPOSE:  Global Database related definitions used throughout the IAS 
 	  workstation software.

INVOCATION METHOD:
	#include <xdb_Defines.h>

EXTERNAL FILE/RECORD/DBTABLE REFERENCES:
Name				Use	Description
-------------------------	---	-------------------------------------------------
none

EXTERNAL VARIABLES:
Source		Name			Type		Use	Description
---------------	----------------	---------------	---	-----------------------
none

DEVELOPMENT HISTORY:
Author			Change-ID	Release	Date		Description of Change
-----------------	-----------	-------	--------	------------------------------
J. kerich		N/A		R1.0	03/20/97	Original Implementation
J. kerich		N/A		R2.0	09/10/97	Add XDB_TAR
D. Slafkosky            N/A             R2.0    10/30/97        Add L0R product deletion states
A. Williard             IAS970019       R2.0    10/31/97        Increase wo state size to 2, wo size to 7, and l0r size to 9
A. Williard             N/A             R2.0    12/01/97        Added macro for test script dir
D. Slafkosky            N/A             R2.0    12/16/97        Added XDB_DAAC        
J. kerich		IAS970051      	R3.0	02/06/98	Add XDB_L1RG
A. Williard             IASI980208      R3.0    3/6/98          Change size of XXX_SCRIPT_EXIT_STATUS_SIZE 
A. Williard             IASI980354      R3.0    5/26/98         Add l0r deletion
state for "User deleted l0r"

NOTES:
none

******************************************************************************/
#ifndef XDB_DEFINES_H
#define XDB_DEFINES_H

/***** Database specific parameters					   	*/

#define XDB_ORAERRMSGLEN	512	/* Max len allowed for DBMS error string 	*/
#define XDB_NOTCONNECTED        -1012   /* not connected to database                    */
#define XDB_NOTFOUND		100	/* DBMS-defined value for data not found 	*/
#define XDB_SUCCESS	 	0	/* DBMS-defined value for successful		*/
                                        /*    dbms transaction 			   	*/
#define XDB_NON_ERROR		99	/* Indicates db unit error not caused		*/
                                        /*   by dbms exception (e.g. out of memory)	*/
                                        /*   DB unit will set xxx_errno to appropriate	*/
                                        /*   error classification - see xxx_Errno.h  	*/
#define XDB_WARNING		99999	/* DB unit return error val if db warning found	*/
#define	XDB_FETCH_SIZE		100	/* Number of records per array fetch		*/
#define	XDB_INSERT_SIZE		100	/* Number of records per array insert		*/
#define XDB_PARSE_STMT_SIZE	2048	/* Size dynamic sql parse string		*/
#define	XDB_NULL		-1	/* Value returned if a retrieved db field = NULL*/
#define XDB_NULLSTR		' ' 	/* Value returned if a retrieved db field = NULL*/
#define XDB_NULL_ZERO		0	/* a retrieved db field = NULL			*/
#define XDB_NULL_ONE		1	/* Value returned if a retrieved db field = NULL*/
#define XDB_ALL_ROWS		99	/* Indicates retrieve all rows			*/

/***** Miscellaneous defines						   	*/

#define	XDB_WARN_MSG        	"Fetched value was truncated." /* db warning message	*/
#define XDB_NOTFOUND_MSG        "ORA-01403: no data found"    /* no data found message	*/

/***** Macros									*/

#define	XDB_IF_WARNING	( (sqlca.sqlwarn[1] == ' ') ? SUCCESS : ERROR)

#define XXX_EPHEMERIS_FILE_SIZE             19
#define XXX_EPHEMERIS_TYPE_SIZE             1
#define XXX_FILE_TYPE_SIZE                  4
#define XXX_HOST_ADDRESS_SIZE               15
#define XXX_HOST_NAME_SIZE                  15
#define XXX_INVOICE_SIZE                    50   /* DAAC_PRODUCT_ID */
#define XXX_L0R_HDFNAME_SIZE                50
#define XXX_MAXPATH_SIZE                    256
#define XXX_MESSAGE_SIZE                    256
#define XXX_PASSWORD_SIZE                   12
#define XXX_PRODUCT_ID_SIZE                 9
#define XXX_PROGRAM_ID_SIZE                 25
#define XXX_SCRIPT_ID_SIZE                  25
#define XXX_SCRIPT_EXIT_STATUS_SIZE         15
#define XXX_SOURCE_FILE_SIZE                80
#define XXX_TARGET_PATH_SIZE                80
#define XXX_WORKORDER_ID_SIZE               7
#define XXX_WORKORDER_STATE_SIZE            2
#define XXX_TRANS_TYPE_SIZE                 3
#define XXX_USERID_SIZE                     15
#define XXX_WOPARMNAME_SIZE     	    25          
#define XXX_WOPARMVALUE_SIZE                256

/* IAS Configuration file types */
#define XDB_DAN           "DAN"    /* DDM path */
#define XDB_DAAC          "DAAC"   /* DAAC path */
#define XDB_CPF           "CPF"    /* CPF path */
#define XDB_EMOC          "EMOC"   /* error MOCC path */
#define XDB_EPHM          "EPHM"   /* Ephemeris  Path */
#define XDB_L0R           "L0R"    /* L0R product path */
#define XDB_L1RG          "L1RG"   /* L1R/L1G product path */
#define XDB_MISC          "MISC"   /* misc. path */
#define XDB_RPT           "RPT"    /* reports path */
#define XDB_SCRP          "SCRP"   /* wo script path */
#define XDB_SCRTP         "SCRT"   /* wo script test path */
#define XDB_TAR           "TAR"    /* temp tar path */
#define XDB_TMOC          "TMOC"   /* temp MOCC path */
#define XDB_WO            "WO"     /* work order path */

/* L0R Product deletion states */
#define XDB_DONT_DELETE    0
#define XDB_OK_TO_DELETE   1
#define XDB_USER_DELETED   2
#define XDB_DELETED        3

#endif

