/******************************************************************************
                Landsat 7 Image Assessment System (IAS) Software
                        Property of the U.S. Government
                            NASA / GSFC / Code 510
*******************************************************************************

UNIT NAME:  xxx_EDCOdl.h

PURPOSE:  Defines ODL structures, constants, and prototype.

INVOCATION METHOD:
	#include <xxx_EDCOdl.h>

EXTERNAL FILE/RECORD/DBTABLE REFERENCES:
Name				Use	Description
-------------------------	---	-------------------------------------------------
none

EXTERNAL VARIABLES:
Source		Name			Type		Use	Description
---------------	----------------	---------------	---	-----------------------
hlimits.h	MAX_ORDER    		int		I	Defined max order size	
hlimits.h      	VSNAMELENMAX		int		I	Defined max vdata size	

DEVELOPMENT HISTORY:
Author			Change-ID	Release	Date		Description of Change
-----------------	-----------	-------	--------	------------------------------
J. kerich		N/A		R1.0	04/25/97	Original Implementation
J. kerich		IASI990538     	R3.2	11/23/98	Add xxx_ReadEDCOdl2 prototype

NOTES:
none

******************************************************************************/

#ifndef XXX_EDCODL_H
#define XXX_EDCODL_H

#include <stdlib.h>
#include <vg.h>
#include <hlimits.h>
#include <hdfi.h>

#define XXX_ODL_FIELD_SIZE		50	/* Maximum strlen on meta/CPF fields was 138 */

typedef struct xxx_ODL_TYPE {
	char8	        RawOdlLine[MAX_ORDER];
} xxx_ODL_TYPE;

typedef struct xxx_ODLControl_TYPE {
	int32		Count;		/* Number of rows in ODL format */
	xxx_ODL_TYPE	*p_ODL;	        /* Format of meta data and CPF  */

	/* vdata information about the file */
	char		vdataName[VSNAMELENMAX+1];
	char		vdataClass[VSNAMELENMAX+1];
	int32		Interlace;
	int32		vdataSize;
} xxx_ODLControl_TYPE;

xxx_ODLControl_TYPE *xxx_ReadEDCOdl(char *, char *, char *);
xxx_ODLControl_TYPE *xxx_ReadEDCOdl2(char *, char *, int, char *);

#define XXX_FREE_ODL(ODLControl) \
{ \
	if (ODLControl) \
	{ \
		free(ODLControl->p_ODL); \
		free(ODLControl); \
	} \
}

#endif
