#ifndef XXX_HYPPCD
#define XXX_HYPPCD

#include "mfhdf.h"	      /* HDF definitions */

#include "xxx_ACSHK.h"        /* Describes ACS telemetry structure */
#include "xxx_HYPHK.h"        /* Describes HYP telemetry structure */
#include "xxx_GPSHK.h"        /* Describes GPS telemetry structure */

#define MAX_DCE 256           /* Max number of DCEs in a housekeeping file (arbitrary) */

typedef struct HYPDCE_Desc
{
  double  start_time;         /* DCE start time in seconds from GPS epoch */
  double  stop_time;          /* DCE stop time in seconds from GPS epoch */
  double  acs_start_tc;       /* Time code of first ACS telemetry record in DCE */
  double  acs_stop_tc;        /* Time code of last ACS telemetry record in DCE */
  int     hyp_start_index;    /* Index of first Hyperion telemetry record in DCE */
  int     hyp_stop_index;     /* Index of last Hyperion telemetry record in DCE */
  int     gps_start_index;    /* Index of first GPS telemetry record in DCE */
  int     gps_stop_index;     /* Index of last GPS telemetry record in DCE */
  int     utcf_index;         /* Index of GPS UTCF time offset record for DCE */
} HYPDCE_Desc;

/* Define Hyperion PCD data structure */
typedef struct HYPPCD
{
  int           active_DCE;           /* Index of currently selected DCE */
  int           num_DCE;              /* Number of DCEs in this housekeeping interval */
  HYPDCE_Desc   DCE_List[MAX_DCE];    /* Array of DCE descriptors */
  HYP		*HYP_Group;           /* Pointer to ACS housekeeping data */
  ACS		*ACS_Group;           /* Pointer to ACS housekeeping data */
  GPS		*GPS_Group;           /* Pointer to GPS housekeeping data */
} HYPPCD;

/* Routine to load Hyperion PCD data structure */
int xxx_read_HYPPCD( HYPPCD *pcd, char *ddir, char *froot );

/* Utility to construct path name (fname) from directory, file root, and extension */
void xxx_hkfname( char *fname, char *ddir, char *froot, const char *ext );

/* Routine to free the dynamically allocated memory in the HYPPCD structure */
void xxx_free_HYPPCD( HYPPCD *pcd );

/* Routine to try to find breaks in DCEs */
int xxx_find_DCEs_hyp( HYPPCD *pcd );

#endif

