/******************************************************************************
                Landsat 7 Image Assessment System (IAS) Software
                        Property of the U.S. Government
                            NASA / GSFC / Code 510
*******************************************************************************

FILE NAME:  xxx_Imagery.h

PURPOSE:  Include file for time formats.

INVOCATION METHOD:
	#include <xxx_Imagery.h>

EXTERNAL FILE/RECORD/DBTABLE REFERENCES:
Name                    Use 	Description
----------------------- --- 	-------------------------------------------------
none

CHANGE HISTORY:
Author		Change ID	Build/Release	Date		Description of Change
------		---------	-------------	---------	---------------------
J. Kerich	n/a		R1.0		02/11/97	Original Implementation
J. Kerich	IASI970058      R2.0		09/30/97	Add DirectoryPath to structure
J. kerich	IASI990538      R3.2            11/23/98	Change xxx_ChkForSDS & xxx_ChkForSDSInHDF
								prototypes

NOTES:
none

******************************************************************************/

#ifndef XXX_IMAGERY_H
#define XXX_IMAGERY_H

#include <sys/param.h>
#include <limits.h>
#include <hdf.h>
#include <xxx_Types.h>

#define XXX_MAX_RANK  2

typedef struct xxx_SubDevideInfo_TYPE {
        int32       StartOffset;
        int32       RowsToRead;
        int32       ByteOffset;
        int32       BytesToRead;
        int32       CurrectRow;
} xxx_SubDevideInfo_TYPE;

typedef struct xxx_ImageryInfo_TYPE {
	boolean     EDC_DAAC_HDF;               /* opened a HDF directory file */
        int32       FileId;                     /* for vgroup checks */
        int32       SDS_Id;
	int32       SDId;
	char        SDSName[MAX_NC_NAME];
	int32       DataType;
	int32       Rank;
	int32       DimensionSizes[MAX_VAR_DIMS];
	char        DirectoryPath[MAXPATHLEN];
        xxx_SubDevideInfo_TYPE SD;
} xxx_ImageryInfo_TYPE;

int  xxx_ChkForSDS(xxx_ImageryInfo_TYPE *, int, char *);
int  xxx_ChkForSDSInHDF(xxx_ImageryInfo_TYPE *, int, char *);
long xxx_GetDataTypeSize(int32);
void xxx_FindDirectory(char *, char *);
char *xxx_FindFileName(char *);
int  xxx_OpenImagery(char *, int32, char *, int32, char *, xxx_ImageryInfo_TYPE *, char *);
int  xxx_CloseImagery(xxx_ImageryInfo_TYPE *, char *);
int  xxx_ReadImagery(xxx_ImageryInfo_TYPE *,  void *, int32, int32 *, int32, int32, char *);
int  xxx_GetSubdevidedImagery(xxx_ImageryInfo_TYPE *,  void **, char *);
int  xxx_WriteImagery(xxx_ImageryInfo_TYPE *,  void *, int32, int32, char *);

#endif

