/******************************************************************************
                Landsat 7 Image Assessment System (IAS) Software
                        Property of the U.S. Government
                            NASA / GSFC / Code 510
*******************************************************************************

UNIT NAME:
	xxx_ODL.h

PURPOSE:
	Defines ODL and CPF structures, constants, and prototype. 

INVOCATION METHOD:
	#include <xxx_ODL.h>

EXTERNAL FILE/RECORD/DBTABLE REFERENCES:
Name				Use	Description
-------------------------	---	-------------------------------------------------
None

EXTERNAL VARIABLES:
Source		Name			Type		Use	Description
---------------	----------------	---------------	---	-----------------------
None

DEVELOPMENT HISTORY:
Author			Change-ID	Release	Date		Description of Change
-----------------	-----------	-------	--------	------------------------------
T. Raheja	                        R1.0       03/97        Original Implementation
J. kerich		N/A		R1.0	05/01/97	Add EDC Odl logic
T. Raheja	                        R2.0    10/06/97        Add CPF structure and prototype
J. kerich		IAS970011      	R2.0	11/12/97	Add XXX_Long for 64 bit compile
A. Williard             N/A             R2.0    12/10/97        Update xxx_BuildWOParmODL prototype
J. kerich		IASI980365     	R3.1	06/24/98	Change xxx_PrintDiff prototype
A. Williard		IASI980499     	R3.1b	01/15/98	Add L1R_IMAGE macro
J. kerich		IASI990538     	R3.2	11/23/98	Add xxx_LPGSMeta file option

NOTES:
	None

******************************************************************************/

#ifndef XXX_ODL_H
#define XXX_ODL_H

#define NOT_ENOUGH_BUFFER_SIZE    -1

#include <lablib3.h>
#include <xxx_Types.h>
#include <xdb_Defines.h>

/* Warning status codes returned from xxx_GetODLField() */
#define XXX_E_NOT_ENOUGH_MEMORY_SUPPLIED        2 
#define XXX_E_ODL_NOT_FOUND                     3
#define XXX_E_ODL_INVALID_DATA_TYPE             4 

#define XXX_ADDED_KEYWORD                      'A'
#define XXX_DELETED_KEYWORD                    'K'
#define XXX_MODIFIED_KEYWORD                   'M'

#define XXX_NEW_GROUP                          'N'
#define XXX_DELETED_GROUP                      'G'

/* Constants for global work order param names */
#define XXX_PARM_WO_DIR     	"WO_DIRECTORY"
#define XXX_PARM_CPF        	"CAL_PARM_FILE"
#define XXX_PARM_EPHEM_FILE 	"FDF_NAME"
#define XXX_PARM_L0R_HDFNAME 	"L0R_HDFNAME"
#define XXX_PARM_L1R_IMAGE      "L1R_IMAGE"
#define XXX_PARM_SCRIPT_NAME 	"SCRIPT_NAME"
#define XXX_PARM_L0R_ID     	"L0R_ID"

/* Flag passed to xxx_OpenODL() indicating the type of ODL file */
typedef enum  {
	xxx_NoEDCMeta,          /* regular odl file */
	xxx_EDCMeta,            /* L0R's EDC Metadata file */
	xxx_LPGSMeta,           /* L1R's LPGS Metadata file */
	xxx_LPSMeta1,           /* L0R's LPS Metadata file */
	xxx_LPSMeta2,           /* L0R's LPS Metadata file */
	xxx_CPFMeta             /* L0R's Cal Parm File */
}
xxx_EDCOdl_TYPE;

/* Flag passed to xxx_GetODLField() indicating the ODL keyword's expected data type */
typedef enum  {
	XXX_Double, 
	XXX_ArrayOfString, 
	XXX_String,
	XXX_Int, 
	XXX_Long, 
	XXX_Float,
	XXX_Sci_Not
}
xxx_ValType_TYPE;

typedef struct {
	char      Object_Name[XXX_MAXPATH_SIZE+1];
	char      *p_GroupFileName;	/* Group file name (DCM task only) */
	char      *p_DerivedFrom;	/* Calibration parameter file the group was extracted from (DCM task only) */

	char      Kwd_Name[XXX_MAXPATH_SIZE+1];
	char      Pre_Kwd_Name[XXX_MAXPATH_SIZE+1];
	char      *Old_Value;
	char      *New_Value;
	char      Pre_Obj[XXX_MAXPATH_SIZE+1];
	int       Pre_Level;
	char      Flag;
	int       Level;
	char      Message[XDB_ORAERRMSGLEN+1];  /* Problem found when adding, deleting, or modifying group or keyword */
} xxx_Diff_TYPE;

OBJDESC* xxx_OpenODL(char *, xxx_EDCOdl_TYPE, char *);
int      xxx_CloseODL(OBJDESC*, char *);
int      xxx_GetODLField( void *, int, xxx_ValType_TYPE, OBJDESC *, char *, char *, char *,int * );
int      xxx_ConvertString(void *, xxx_ValType_TYPE, char *, char *);

int      xxx_BuildWOParmODL(char *, char *, char *, char *, char *, char *, char *, char *, char *);

int 	 xxx_PrintCPFGroup(OBJDESC *, char * , char *, char *, char *);
int 	 xxx_DiffCPF(OBJDESC *, OBJDESC *, xxx_Diff_TYPE **, int *, char *);
int 	 xxx_PrintDiff(char *, char *, int, xxx_Diff_TYPE *, boolean, char *);
void 	 xxx_FreeDiffCPF(int *, xxx_Diff_TYPE **, boolean);
	
#endif

