/******************************************************************************
  GPS correlation include file
   this file includes:
     function prototypes for the GPS correlation library routines
*******************************************************************************/
#ifndef _XXX_CORRELATION_H_
#define _XXX_CORRELATION_H_

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "xxx_Types.h"
#include "xxx_LogStatus.h"
#include "xxx_const.h"
#include "xxx_structures.h"

#define MAX_DIM  6	/* dimension for matrices used in fitting correlation 
			   offsets */
#define MINSIGMA 0.01	/* minimum sigma when calculating image subimage stats*/
#define NPEAKS 32	/* number of peak correlation values to track	      */
#define SWAP(a,b) tempr=(a);(a)=(b);(b)=tempr

/* GPS CORRELATION */

int xxx_grey_corr(float *images, float *imager, int  *srch_size, 
    int  *ref_size, float min_corr, CorrFitMethod fit_method, float max_disp,   
    float *nom_off, float *strength, float *fit_offset, float *est_err,       
    float *diag_disp, int  *mult_peak_flag, int  *edge_flag, 
    int  *low_peak_flag, int  *max_disp_flag, int abs_corr_coeff);

int xxx_grey_cross(float *images, float *imager, int  *srch_size,
    int  *ref_size, int  ncol, int  nrow, float *unormc);

int xxx_grey_cross_ss(const float *images, const float *imager, 
    const int *win_size, const int max_off, float *unormc);

void xxx_grey_eval(int  ncol, int  nrow, float *ccnorm, float *pkval,
    int  *ipkcol, int  *ipkrow, float *sums, float min_corr, float *strength,
    float *cpval, int  *mult_peak_flag, int  *edge_flag, int  *low_peak_flag);

int xxx_grey_norm(float *imager, float *images, int  *ref_size, int  *srch_size,
    int  ncol, int  nrow, float *unormc, float *ccnorm, float *pkval,
    int  *ipkcol, int  *ipkrow, float *sums, int abs_corr_coeff);

int xxx_grey_norm_ss(const int *surf_size, 
  const float *unormc, float *ccnorm,
  float *pkval, int *ipkcol, int *ipkrow, float sums[2], int abs_corr_coeff);

int xxx_fitreg(float *cpval, CorrFitMethod fit_method, float *pkoffs, 
    float *est_err);

void sums(float *cpval, CorrFitMethod fit_method, float *z, float *wghts, 
    float b[6][6], float *vector);
int invert(float (*matrix)[MAX_DIM], float (*outA)[MAX_DIM], int  n);
int lu_fact(int  n, float (*matrix)[MAX_DIM], int  *ipvt);
void back_solve(int  n, float (*matrix)[MAX_DIM], int  *ipvt, float *b);
int init_pivot(int  n, float (*matrix)[MAX_DIM], float *s);
int find_pivot(int  n, float (*matrix)[MAX_DIM], int  k, float *s);
void esterr(float *z, float *wghts, float bnvrs[MAX_DIM][MAX_DIM], double *coeffs,
        float *pkoffs, float *est_err);

void xxx_fft2d(double *data, int  nrows, int  ncols, int  isign);
void fftnd(double data[], int  nn[], int ndim, int isign);

#endif
