#include <string.h>
#include "gxx_utility.h"

/******************************************************************************
NAME:           gxx_find_underscore

PURPOSE:
Split the specified filename into two parts, the part before the
last underscore ('_') and the part after the last underscore.

DEVELOPMENT HISTORY:
$Header: /llite/cvseo1/eo1/src/global/xxxLib/xxxIntIO/gxx_find_underscore.c,v 1.1.1.1 2000/10/19 14:37:35 morfitt Exp $
AUTHOR        Change-ID    DATE     Description of Change
------        ---------  --------   ---------------------
Gail Schmidt     1.0     06/23/97   Original development.

RETURN VALUE:   Type = int
Value    Description
-----    -----------
none

NOTES:
If no underscore is present, look for an extension (i.e. '.xxx').  Strip
off the extension into the after filename.  Strip the filename up to the
extension into the before filename.

ALGORITHM REFERENCES:

******************************************************************************
                Landsat 7 Image Assessment System (IAS) Software
                        Property of the U.S. Government
                                NASA/GSFC/EDC
******************************************************************************/

void gxx_find_underscore (
  const char *filename,       /* I: Name of file to split              */
  char *before,               /* O: Filename before the underscore     */
  char *after)                /* O: Filename after the underscore      */
{
  char file_name[STRLEN],     /* Local copy of filename                */
       *ptr;                  /* String pointer                        */

  /* Make a local copy of filename so it is not destroyed
   */
  strcpy(file_name, filename);

  /* Check for an underscore
   */
  ptr = (char *) strrchr(file_name, '_');
  if (ptr != NULL)
  {
    *(ptr++) = '\0';
    strcpy (before, file_name);
    strcpy (after, ptr);
  }
  else
  {
    /* Check for an extension if no underscore was found.
     */
    ptr = (char *) strrchr(file_name, '.');
    if (ptr != NULL)
    {
      *(ptr++) = '\0';
      strcpy (before, file_name);
      strcpy (after, ptr);
    }
    else
    {
      strcpy (before, file_name);
      strcpy (after, "");
    }
  }

  return;
}
