/******************************************************************************
NAME:		gxx_open_gcp_file

PURPOSE:	Open the GCP data file with the specified access.

DEVELOPMENT HISTORY:
$Log: gxx_open_gcp_file.c,v $
Revision 1.1.1.1  2000/10/19 14:37:35  morfitt
ram:  Inital EO1 processing system.

Revision 1.1.1.1  1999/12/01 19:01:20  mbs
Import of IAS 3.22

Revision 1.1  1997/05/20 20:00:32  rockvam
New gxxUtil functions

Revision 1.2  1997/05/13 14:04:43  rockvam
change gps to gxx, use IAS library functions & defines, use typedefs

Revision 1.1  1997/05/02 18:34:17  rockvam
Initial revision


RETURN VALUE: 
Type = int
Value           Description
-----           -----------
SUCCESS     	Successfully opened file
ERROR     	Error occurred; file not opened

NOTES:		The GCP data file is an ASCII file.
		It may be opened for read, write, or update mode.  Opening
		for write mode will overwrite the file if it exists.  Opening
		for update is basically append mode (it will place the file 
		pointer at the end of the file); no updating of existing data 
		in the file is supported.

ALGORITHM REFERENCES:
*******************************************************************************
		Landsat 7 Image Assessment System (IAS) Software
		        Property of the U.S. Government
                                NASA/GSFC/EDC
******************************************************************************/
#include <string.h>
#include "xxx_Types.h"
#include "xxx_LogStatus.h"
#include "gxx_utility.h"

int gxx_open_gcp_file (
	char *gcp_data_filename,	/* I-Name of GCP data file	*/
	long access_mode,       	/* I-Requested access mode of file*/
	FILE **gcp_fptr)		/* O-GCP data file pointer	*/
{
char	msg[STRLEN];			/* error message		*/
char	type[3];			/* file access type for fopen	*/

if (access_mode == READ)
    strcpy(type,"r");
else if (access_mode == WRITE)
    strcpy(type,"w");
else if (access_mode == UPDATE)
    strcpy(type,"a");
else
    {
    xxx_LogStatus("GPSLIB",__FILE__,__LINE__,"Invalid file access mode");
    return(ERROR);
    }

*gcp_fptr = fopen(gcp_data_filename,type);
if (*gcp_fptr == NULL)
    {
    sprintf(msg,"Error opening %s\n",gcp_data_filename);
    xxx_LogStatus("GPSLIB",__FILE__,__LINE__,msg);
    return(ERROR);
    }

return(SUCCESS);
}
