/******************************************************************************
FUNCTION:	xxx_check_year_month_day

PURPOSE:
See if the start or end date is valid

RETURN VALUE:
Type = int
Value    Description
-----    -----------
ERROR    Error processing the model data
SUCCESS  Model data was updated  

NOTES:

ALGORITHM REFERENCES:
*******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "xxx_Types.h"
#include "xxx_LogStatus.h"
#include "xxx_const.h"
#include "xxx_math.h"

int xxx_check_year_month_day
(
  int year, 	/* I: year (YYYY) */
  int month, 	/* I: month (MM) */
  int day 		/* I: day (DD) */
)

{
double jdate;	/* the julian date */
int leap[12] = {31,29,31,30,31,30,31,31,30,31,30,31}; /* number of days in months
							 for a leap year */
int notleap[12] = {31,28,31,30,31,30,31,31,30,31,30,31}; /* number of days in
							    months, non leap yr */
int min_day;	/* smallest day # in month */
int max_day;	/* largest day # in month */


/* ********************************************************
   seeing if the entries are zeros
******************************************************** */
if (year == 0 || month == 0 || day == 0)
    {
    xxx_LogStatus("XXXLIB", __FILE__, __LINE__, "Warning: Zeros in date");
    return(SUCCESS);
    }

/* ********************************************************
    validating the date
******************************************************** */
if (year < MIN_YEAR || year > MAX_YEAR)
    {
    xxx_LogStatus("XXXLIB", __FILE__, __LINE__, "Invalid year");
    return(ERROR);
    }

if (month < MIN_MONTH || month > MAX_MONTH)
    {
    xxx_LogStatus("XXXLIB", __FILE__, __LINE__, "Invalid month");
    return(ERROR);
    }

if (xxx_isleap(year) == TRUE)
	max_day = leap[month - 1];
else
	max_day = notleap[month - 1];

min_day = 1.0;
if (day < min_day || day > max_day)
    {
    xxx_LogStatus("XXXLIB", __FILE__, __LINE__, "Invalid day");
    return(ERROR);
    }

return(SUCCESS); 
}
