/*******************************************************************************
NAME:		xxx_close_1g_image

PURPOSE:	Close the specified 1G image.

RETURN VALUE: 
Type = int
Value		Description
-----		-----------
SUCCESS		Successfully closed image
ERROR		Error; image not closed

NOTES:		Need to deattach all the data before closing the file

ALGORITHM REFERENCES:
*******************************************************************************/
#include <stdio.h>
#include "xxx_Types.h"
#include "xxx_LogStatus.h"
#include "xxx_hdf_io.h"

int xxx_close_1g_image (
	HDFIO *hdfid)		/* I-HDF IO structure id	*/
{
intn	hdf_status;		/* HDF function return status	*/
BandNumber band;		/* band loop counter		*/
char	msg[STRLEN];		/* Error message string		*/

if (hdfid == NULL)
    {
    return(SUCCESS);
    }

/*
 * Info about the group
 */
for (band = pan; band <= b7; band++)
    {
    if (hdfid->sds_id[(int)band] != -1)
	{
       /*
	* Done with this SDS
	*/
	hdf_status = SDendaccess(hdfid->sds_id[(int)band]);
	if (hdf_status == FAIL)
	    {
    	    free(hdfid);
    	    hdfid = NULL;
            sprintf(msg,"Error releasing SDS %s", hdfid->sds_name[(int)band]);
	    xxx_LogStatus("XXXLIB",__FILE__,__LINE__,msg);
            return(ERROR);
            }
	}
    }

/*
 * Close down
 */
hdf_status = SDend(hdfid->sd_id);
if (hdf_status == FAIL)
    {
    free(hdfid);
    hdfid = NULL;
    xxx_LogStatus("XXXLIB",__FILE__,__LINE__,"Error ending SDS interface");
    return(ERROR);
    }

Vdetach(hdfid->group_id);
Vend(hdfid->file_id);

hdf_status = Hclose(hdfid->file_id);
if (hdf_status == FAIL)
    {
    free(hdfid);
    hdfid = NULL;
    xxx_LogStatus("XXXLIB",__FILE__,__LINE__, "Error closing HDF file, check attachments");
    return(ERROR);
    }

free(hdfid);

return(SUCCESS);
}
