/******************************************************************************
NAME:           xxx_create_DDR_name

PURPOSE:
axx_create_DDR_name forms a DDR name given an associated HDF file name.  The
"_HDF" at the end of the root HDF file name is replaced with "_DDR".  If no 
"_HDF" is at the end of the file name, the "_DDR" is appended to the end of the 
root HDF file name.

NOTES:
none

ALGORITHM REFERENCES:
none

******************************************************************************/
#include <string.h>    /* string library prototypes */
#include "xxx_utility.h" /* createname and splitname prototypes */

/*
**
** MACROS
**
*/

#define DDR_SUBSTRING "_DDR"
/* defines the string to use to make the DDR file name */


void xxx_create_DDR_name
(
    char *DDR_name,        /* O: pointer to formed DDR name */
    const char *HDF_name   /* I: HDF name to form DDR name from */
)
{
    char directory[STRLEN]; /* directory name */
    char root[STRLEN];      /* root file name */
    char extension[STRLEN]; /* file extension */
    char *temp_char_ptr;    /* temporary character pointer */
    
    /* form the DDR file name from the DEM name */
    xxx_splitname(HDF_name, directory, root, extension);

    /* find the "_HDF" part of the file name if it exists */
    temp_char_ptr = strrchr(root,'_');
    if (temp_char_ptr == NULL)
    {
        /* no "_HDF" so just append the DDR_SUBSTRING */
        strcat(root, DDR_SUBSTRING);
    }
    else
    {
        if (strcmp(temp_char_ptr, "_HDF") == 0)
        {
            /* replace the "_HDF" with the DDR substring */
            strcpy(temp_char_ptr, DDR_SUBSTRING);
        }
        else
        {
            /* doesn't end with "_HDF" so just append the DDR substring */
            strcat(root, DDR_SUBSTRING);
        }
    }

    /* create DDR name */
    xxx_createname(directory, root, extension, DDR_name);
}
