/******************************************************************************
NAME:		xxx_extract_chip

PURPOSE:	Read the specified image chip

RETURN VALUE: 
Type = int
Value           Description
-----           -----------
SUCCESS     	Successfully extracted chip
ERROR     	Error; chip not extracted

NOTES:		The image chip is always assumed to be a flat binary file 
		containing chip_size[0] x chip_size[1] BYTE pixels

******************************************************************************/
#include <stdio.h>
#include <unistd.h>
#include "xxx_Types.h"
#include "xxx_LogStatus.h"
#include "xxx_hdf_io.h"

int xxx_extract_chip (
	const char *chip_name,	/* I-name of the image chip file*/
	const int  chip_size[2],/* I-expected size of image chip*/
	void *chip_buf)		/* O-buffer of chip image data	*/
{
FILE	*chip_fp;		/* chip file pointer		*/
long 	file_size;		/* size of the chip file	*/
char	msg[STRLEN];		/* error message string		*/

/*
 * Open the chip image
 */
chip_fp = fopen(chip_name,"r");
if (chip_fp == NULL)
    {
    sprintf(msg,"Error accessing image chip %s",chip_name);
    xxx_LogStatus("XXXLIB",__FILE__,__LINE__,msg);
    return(ERROR);
    }

/*
 * Check to make sure the chip size is the same as reported by the chip
 */
if (fseek(chip_fp, 0, SEEK_END) != 0)
    {
    fclose(chip_fp);
    sprintf(msg,"Error reading image chip %s",chip_name);
    xxx_LogStatus("XXXLIB",__FILE__,__LINE__,msg);
    return(ERROR);
    }
file_size = ftell(chip_fp);
if (file_size != ((long)chip_size[0] * chip_size[1]))
    {
    fclose(chip_fp);
    sprintf(msg,"Chip was %ld bytes, expected %ld bytes", file_size, chip_size[0] * chip_size[1]);
    xxx_LogStatus("XXXLIB",__FILE__,__LINE__,msg);
    return(ERROR);
    }
if (fseek(chip_fp, 0, SEEK_SET) != 0)
    {
    fclose(chip_fp);
    sprintf(msg,"Error reading image chip %s",chip_name);
    xxx_LogStatus("XXXLIB",__FILE__,__LINE__,msg);
    return(ERROR);
    }

/*
 * Read the full chip into the buffer
 */
if (fread(chip_buf, (chip_size[0] * chip_size[1]), 1, chip_fp) != 1)
    {
    fclose(chip_fp);
    sprintf(msg,"Error reading image chip %s",chip_name);
    xxx_LogStatus("XXXLIB",__FILE__,__LINE__,msg);
    return(ERROR);
    }

/*
 * Close the chip
 */
if (fclose(chip_fp) == EOF)
    {
    sprintf(msg,"Error closing image chip %s",chip_name);
    xxx_LogStatus("XXXLIB",__FILE__,__LINE__,msg);
    return(ERROR);
    }


return(SUCCESS);
}
