/******************************************************************************
NAME:           xxx_free_geom_grid

PURPOSE:
Frees the memory allocated to the geometric grid structure.

RETURN VALUE:  
Type = int
Value    Description
-----    -----------
SUCCESS  Successful completion
ERROR    Operation failed

NOTES:
This routine will loop through the specified bands, freeing the memory 
allocated to the grid.  If no bands were specified, then all the bands in the 
grid will be freed.  In this case, the bands array will be filled in with the
bands in the grid and nbands will be set to the correct value.

ALGORITHM REFERENCES:
See xxx_geom_grid.h for a description of the geometric grid structure.

******************************************************************************/

#include <stdlib.h>          /* free prototype */
#include <string.h>          /* memcpy prototype */
#include "xxx_geom_grid.h"   /* geometric grid definition */


void xxx_free_geom_grid 
(
  int *bands,                /* I/O: Array of bands to free from the grid */
  int *nbands,               /* I/O: Number of bands in the bands array   */
  xxx_GEOM_GRID_TYPE *grid   /* I/O: Grid structure to be freed           */
)
{
    int band;                    /* Loop variable */
    int band_index;              /* Current band index */

    /* If no bands are specified then free all the bands in the grid.
    */
    if (*nbands == 0)
    {
        *nbands = grid->nbands;
        memcpy (bands, grid->bandlist, *nbands * sizeof (int));
    }

    /* Loop through the bands freeing the correct reference bands.
    */
    for (band = 0; band < *nbands; band++)
    {
        /* Get the reference value of the current band */
        band_index = bands[band];
        
        free (grid->gridbands[band_index].in_lines);
        free (grid->gridbands[band_index].in_samps);
        free (grid->gridbands[band_index].out_lines);
        free (grid->gridbands[band_index].out_samps);
        free (grid->gridbands[band_index].poly_lines);
        free (grid->gridbands[band_index].poly_samps);
        free (grid->gridbands[band_index].forward_coeffs);
        free (grid->gridbands[band_index].reverse_coeffs);
        grid->gridbands[band_index].in_lines = NULL;
        grid->gridbands[band_index].in_samps = NULL;
        grid->gridbands[band_index].out_lines = NULL;
        grid->gridbands[band_index].out_samps = NULL;
        grid->gridbands[band_index].poly_lines = NULL;
        grid->gridbands[band_index].poly_samps = NULL;
        grid->gridbands[band_index].forward_coeffs = NULL;
        grid->gridbands[band_index].reverse_coeffs = NULL;
    }
}
