/*******************************************************************************
NAME:	xxx_get_system_table

PURPOSE:
	Read the IAS GPS system table into a structure.  The application
	is responsible for selecting the appropriate elements of the structure.
	The location of the system table is determined from the IAS_SYSTBL
	environment variable.

RETURN VALUE:
Type=int
Value           Description
-----           -----------
SUCCESS		system table data correctly read into structure
ERROR		error locating or reading system table

NOTES:

*******************************************************************************/
#include "xxx_systbl.h"
#include "xxx_utility.h"

int xxx_get_system_table(
	GPS_SYSTBL *systbl)	/* O-system table structure	*/
{
int 	status;			/* function return status		*/
char	*table_dir;		/* directory location of system table	*/
char	*table_name;		/* name of system table			*/
char	msg[STRLEN];		/* error message string			*/
OBJDESC *object;		/* parsed ODL object			*/

/*
 * Set up the system table name
 */
table_dir = getenv("EO1SYS");
if (table_dir == NULL)
    {
    xxx_LogStatus("XXXLIB",__FILE__,__LINE__,
	"EO1SYS environment variable not set, trying current directory");
    table_dir = ".";
    }

table_name = malloc(strlen(table_dir)+strlen(SYSTEM_TABLE_NAME)+2);
if (table_name == NULL)
    {
    xxx_LogStatus("XXXLIB",__FILE__,__LINE__,
	"Error allocating memory for system table name");
    return(ERROR);
    }

if (sprintf(table_name,"%s/%s",table_dir,SYSTEM_TABLE_NAME) < 0)
    {
    xxx_LogStatus("XXXLIB",__FILE__,__LINE__,
        "Error constructing system table name");
    return(ERROR);
    }

/*
 * Open the system table ODL file
 */
object = xxx_OpenODL(table_name, (xxx_EDCOdl_TYPE)xxx_NoEDCMeta, msg);
free(table_name);
if (object == NULL)
    {
    xxx_LogStatus("XXXLIB",__FILE__,__LINE__,msg);
    return(ERROR);
    }

/*
 * Extract the system table fields from the ojbect
 */
status = xxx_read_system_table(object, systbl);
if (status == ERROR)
    {
    xxx_LogStatus("XXXLIB",__FILE__,__LINE__,"Error reading system table");
    return(ERROR);
    }

/*
 * Close the file
 */
status = xxx_CloseODL(object, msg);
if (status == ERROR)
    {
    xxx_LogStatus("XXXLIB",__FILE__,__LINE__,msg);
    return(ERROR);
    }

return(SUCCESS);
}
