/******************************************************************************
NAME:		xxx_put_gcp

PURPOSE:	Write all GCP records to the specified file

RETURN VALUE: 
Type = int
Value           Description
-----           -----------
SUCCESS     	Successfully wrote out GCP data
ERROR     	Error occurred; GCP data not added to file

NOTES:		This function writes out a set of GCP data records.
		If the file exists it will be overwritten.

******************************************************************************/
#include "xxx_Types.h"
#include "xxx_LogStatus.h"
#include "xxx_utility.h"

int xxx_put_gcp (
	const char *gcp_data_filename,	/* I-Name of GCP data file to add to */
	const GCPDATA gcp_data[],	/* I-GCP data structures	     */
	int  num_gcp)			/* I-Number of GCP's		     */
{
FILE    *gcp_fptr;              /* GCP data file pointer                */
int	status;			/* function return status		*/
int 	curpt;			/* current GCP				*/
char	errmsg[STRLEN];		/* error message			*/

/*
 * Open the file
 */
gcp_fptr = fopen(gcp_data_filename, "w");
if (gcp_fptr == NULL)
    {
    sprintf(errmsg, "Error opening gcp data file %s", gcp_data_filename);
    xxx_LogStatus("XXXLIB", __FILE__, __LINE__, errmsg);
    return(ERROR);
    }

/*
 * Write all records
 */
for (curpt = 0; curpt < num_gcp; curpt++)
    {
    status = xxx_write_gcp(gcp_fptr, &gcp_data[curpt]);
    if (status == ERROR)
    	{
    	sprintf(errmsg, "Error writing to gcp data file %s", gcp_data_filename);
    	xxx_LogStatus("XXXLIB", __FILE__, __LINE__, errmsg);
    	return(ERROR);
    	}
    }

/*
 * Close the file
 */
status = fclose(gcp_fptr);
if (status != 0)
    {
    sprintf(errmsg, "Error closing gcp data file %s", gcp_data_filename);
    xxx_LogStatus("XXXLIB", __FILE__, __LINE__, errmsg);
    return(ERROR);
    }

return(SUCCESS);
}
