/*******************************************************************************
NAME:	xxx_read_system_table

DESCRIPTION:
	Read the IAS GPS system table into a structure.  The application
	is responsible for selecting the appropriate elements of the structure.

RETURN VALUE:
Type = int
Value           Description
-----           -----------
SUCCESS		system table data correctly read into structure
ERROR		error reading system table

NOTES:  If some of the expected entries are missing from the system table, this
	function returns ERROR.
*******************************************************************************/
#include "xxx_systbl.h"
#include "xxx_utility.h"

int xxx_read_system_table(
	const OBJDESC *odl_tree,	/* I-parsed ODL tree		*/
	GPS_SYSTBL *systbl)		/* O-system table entries	*/
{
int 	status;		/* return status 			*/
int	count;		/* count of entries for a table entry 	*/
int	fit;		/* correlation fit method integer value,*/
			/* to be converted to enumerated type	*/
char	msg[STRLEN];	/* error message string			*/

/*
 * Get the structure elements by name from the file
 */
status = xxx_GetODLField(&systbl->add_process_noise, 
	 sizeof(systbl->add_process_noise), XXX_Int, (OBJDESC *)odl_tree, 
	 "GPS_System_Table", "add_process_noise", msg, &count);
if (status != SUCCESS)
    {
    xxx_LogStatus("XXXLIB",__FILE__,__LINE__,msg);
    xxx_LogStatus("XXXLIB",__FILE__,__LINE__,
	"Error locating add_process_noise in system table");
    return(ERROR);
    }

status = xxx_GetODLField(systbl->variance_process_noise,
	 sizeof(systbl->variance_process_noise), XXX_Float, (OBJDESC *)odl_tree, 
	 "GPS_System_Table", "variance_process_noise", msg, &count);
if ((status != SUCCESS) || (count < 12))
    {
    xxx_LogStatus("XXXLIB",__FILE__,__LINE__,msg);
    xxx_LogStatus("XXXLIB",__FILE__,__LINE__,
	"Error locating variance_process_noise in system table");
    return(ERROR);
    }

status = xxx_GetODLField(&systbl->add_observation_noise,
	 sizeof(systbl->add_observation_noise), XXX_Int, (OBJDESC *)odl_tree, 
	 "GPS_System_Table", "add_observation_noise", msg, &count);
if (status != SUCCESS)
    {
    xxx_LogStatus("XXXLIB",__FILE__,__LINE__,msg);
    xxx_LogStatus("XXXLIB",__FILE__,__LINE__,
	"Error locating add_observation_noise in system table");
    return(ERROR);
    }

status = xxx_GetODLField(systbl->variance_observation_noise,
	 sizeof(systbl->variance_observation_noise), XXX_Float, (OBJDESC *)odl_tree, 
	 "GPS_System_Table", "variance_observation_noise", msg, &count);
if ((status != SUCCESS) || (count < 6))
    {
    xxx_LogStatus("XXXLIB",__FILE__,__LINE__,msg);
    xxx_LogStatus("XXXLIB",__FILE__,__LINE__,
	"Error locating variance_observation_noise in system table");
    return(ERROR);
    }

status = xxx_GetODLField(systbl->line_weights,
	 sizeof(systbl->line_weights), XXX_Float, (OBJDESC *)odl_tree, 
	 "GPS_System_Table", "line_weights", msg, &count);
if ((status != SUCCESS) || (count < 9))
    {
    xxx_LogStatus("XXXLIB",__FILE__,__LINE__,msg);
    xxx_LogStatus("XXXLIB",__FILE__,__LINE__,
	"Error locating line_weights in system table");
    return(ERROR);
    }

status = xxx_GetODLField(systbl->samp_weights,
	 sizeof(systbl->samp_weights), XXX_Float, (OBJDESC *)odl_tree, 
	 "GPS_System_Table", "samp_weights", msg, &count);
if ((status != SUCCESS) || (count < 9))
    {
    xxx_LogStatus("XXXLIB",__FILE__,__LINE__,msg);
    xxx_LogStatus("XXXLIB",__FILE__,__LINE__,
	"Error locating samp_weights in system table");
    return(ERROR);
    }

status = xxx_GetODLField(systbl->md_err_file_name,
	 sizeof(systbl->md_err_file_name), XXX_String, (OBJDESC *)odl_tree, 
	 "GPS_System_Table", "md_err_file_name", msg, &count);
if (status != SUCCESS)
    {
    xxx_LogStatus("XXXLIB",__FILE__,__LINE__,msg);
    xxx_LogStatus("XXXLIB",__FILE__,__LINE__,
	"Error locating md_err_file_name in system table");
    return(ERROR);
    }

status = xxx_GetODLField(systbl->md_tolerances,
	 sizeof(systbl->md_tolerances), XXX_Float, (OBJDESC *)odl_tree, 
	 "GPS_System_Table", "md_tolerances", msg, &count);
if ((status != SUCCESS) || (count < 4))
    {
    xxx_LogStatus("XXXLIB",__FILE__,__LINE__,msg);
    xxx_LogStatus("XXXLIB",__FILE__,__LINE__,
	"Error locating md_tolerances in system table");
    return(ERROR);
    }

status = xxx_GetODLField(&systbl->spheroid,
	 sizeof(systbl->spheroid), XXX_Int, (OBJDESC *)odl_tree, 
	 "GPS_System_Table", "spheroid", msg, &count);
if (status != SUCCESS)
    {
    xxx_LogStatus("XXXLIB",__FILE__,__LINE__,msg);
    xxx_LogStatus("XXXLIB",__FILE__,__LINE__,
	"Error locating spheroid in system table");
    return(ERROR);
    }

status = xxx_GetODLField(systbl->datum,
	 sizeof(systbl->datum), XXX_String, (OBJDESC *)odl_tree, 
	 "GPS_System_Table", "datum", msg, &count);
if (status != SUCCESS)
    {
    xxx_LogStatus("XXXLIB",__FILE__,__LINE__,msg);
    xxx_LogStatus("XXXLIB",__FILE__,__LINE__,
	"Error locating datum in system table");
    return(ERROR);
    }

status = xxx_GetODLField(&systbl->min_corr_focal_plane,
	 sizeof(systbl->min_corr_focal_plane), XXX_Float, (OBJDESC *)odl_tree, 
	 "GPS_System_Table", "min_corr_focal_plane", msg, &count);
if (status != SUCCESS)
    {
    xxx_LogStatus("XXXLIB",__FILE__,__LINE__,msg);
    xxx_LogStatus("XXXLIB",__FILE__,__LINE__,
	"Error locating min_corr_focal_plane in system table");
    return(ERROR);
    }

status = xxx_GetODLField(&systbl->min_corr_gcp_corr,
	 sizeof(systbl->min_corr_gcp_corr), XXX_Float, (OBJDESC *)odl_tree, 
	 "GPS_System_Table", "min_corr_gcp_corr", msg, &count);
if (status != SUCCESS)
    {
    xxx_LogStatus("XXXLIB",__FILE__,__LINE__,msg);
    xxx_LogStatus("XXXLIB",__FILE__,__LINE__,
	"Error locating min_corr_gcp_corr in system table");
    return(ERROR);
    }

status = xxx_GetODLField(&systbl->min_corr_i2i_char,
	 sizeof(systbl->min_corr_i2i_char), XXX_Float, (OBJDESC *)odl_tree, 
	 "GPS_System_Table", "min_corr_i2i_char", msg, &count);
if (status != SUCCESS)
    {
    xxx_LogStatus("XXXLIB",__FILE__,__LINE__,msg);
    xxx_LogStatus("XXXLIB",__FILE__,__LINE__,
	"Error locating min_corr_i2i_char in system table");
    return(ERROR);
    }

status = xxx_GetODLField(&systbl->min_corr_b2b_char,
	 sizeof(systbl->min_corr_b2b_char), XXX_Float, (OBJDESC *)odl_tree, 
	 "GPS_System_Table", "min_corr_b2b_char", msg, &count);
if (status != SUCCESS)
    {
    xxx_LogStatus("XXXLIB",__FILE__,__LINE__,msg);
    xxx_LogStatus("XXXLIB",__FILE__,__LINE__,
	"Error locating min_corr_b2b_char in system table");
    return(ERROR);
    }


status = xxx_GetODLField(&systbl->max_disp_focal_plane,
	 sizeof(systbl->max_disp_focal_plane), XXX_Float, (OBJDESC *)odl_tree, 
	 "GPS_System_Table", "max_disp_focal_plane", msg, &count);
if (status != SUCCESS)
    {
    xxx_LogStatus("XXXLIB",__FILE__,__LINE__,msg);
    xxx_LogStatus("XXXLIB",__FILE__,__LINE__,
	"Error locating max_disp_focal_plane in system table");
    return(ERROR);
    }

status = xxx_GetODLField(&systbl->max_disp_gcp_corr,
	 sizeof(systbl->max_disp_gcp_corr), XXX_Float, (OBJDESC *)odl_tree, 
	 "GPS_System_Table", "max_disp_gcp_corr", msg, &count);
if (status != SUCCESS)
    {
    xxx_LogStatus("XXXLIB",__FILE__,__LINE__,msg);
    xxx_LogStatus("XXXLIB",__FILE__,__LINE__,
	"Error locating max_disp_gcp_corr in system table");
    return(ERROR);
    }

status = xxx_GetODLField(&systbl->max_disp_i2i_char,
	 sizeof(systbl->max_disp_i2i_char), XXX_Float, (OBJDESC *)odl_tree, 
	 "GPS_System_Table", "max_disp_i2i_char", msg, &count);
if (status != SUCCESS)
    {
    xxx_LogStatus("XXXLIB",__FILE__,__LINE__,msg);
    xxx_LogStatus("XXXLIB",__FILE__,__LINE__,
	"Error locating max_disp_i2i_char in system table");
    return(ERROR);
    }

status = xxx_GetODLField(&systbl->max_disp_b2b_char,
	 sizeof(systbl->max_disp_b2b_char), XXX_Float, (OBJDESC *)odl_tree, 
	 "GPS_System_Table", "max_disp_b2b_char", msg, &count);
if (status != SUCCESS)
    {
    xxx_LogStatus("XXXLIB",__FILE__,__LINE__,msg);
    xxx_LogStatus("XXXLIB",__FILE__,__LINE__,
	"Error locating max_disp_b2b_char in system table");
    return(ERROR);
    }

status = xxx_GetODLField(systbl->nom_off,
	 sizeof(systbl->nom_off), XXX_Float, (OBJDESC *)odl_tree, 
	 "GPS_System_Table", "nom_off", msg, &count);
if ((status != SUCCESS) || (count < 2))
    {
    xxx_LogStatus("XXXLIB",__FILE__,__LINE__,msg);
    xxx_LogStatus("XXXLIB",__FILE__,__LINE__,
	"Error locating nom_off in system table");
    return(ERROR);
    }

status = xxx_GetODLField(&fit,
	 sizeof(fit), XXX_Int, (OBJDESC *)odl_tree, 
	 "GPS_System_Table", "fit_method", msg, &count);
if (status != SUCCESS)
    {
    xxx_LogStatus("XXXLIB",__FILE__,__LINE__,msg);
    xxx_LogStatus("XXXLIB",__FILE__,__LINE__,
	"Error locating fit_method in system table");
    return(ERROR);
    }
if (fit ==          (int)FIT_ELLIP_PARA)
    systbl->fit_method = FIT_ELLIP_PARA;
else if (fit ==     (int)FIT_ELLIP_GAUSS)
    systbl->fit_method = FIT_ELLIP_GAUSS;
else if (fit ==     (int)FIT_RECIP_PARA)
    systbl->fit_method = FIT_RECIP_PARA;
else if (fit ==     (int)FIT_ROUND)
    systbl->fit_method = FIT_ROUND;
else 
    systbl->fit_method = FIT_ERROR;

status = xxx_GetODLField(&systbl->max_processors_to_use,
	 sizeof(systbl->max_processors_to_use), XXX_Int, (OBJDESC *)odl_tree, 
	 "GPS_System_Table", "max_processors_to_use", msg, &count);
if (status != SUCCESS)
    {
    xxx_LogStatus("XXXLIB",__FILE__,__LINE__,msg);
    xxx_LogStatus("XXXLIB",__FILE__,__LINE__,
	"Error locating max_processors_to_use in system table");
    return(ERROR);
    }

status = xxx_GetODLField(&systbl->input_image_scans_to_buffer,
	 sizeof(systbl->input_image_scans_to_buffer), XXX_Int, 
         (OBJDESC *)odl_tree, "GPS_System_Table", 
         "input_image_scans_to_buffer", msg, &count);
if (status != SUCCESS)
    {
    xxx_LogStatus("XXXLIB",__FILE__,__LINE__,msg);
    xxx_LogStatus("XXXLIB",__FILE__,__LINE__,
	"Error locating input_image_scans_to_buffer in system table");
    return(ERROR);
    }

status = xxx_GetODLField(&systbl->max_output_image_buffer,
	 sizeof(systbl->max_output_image_buffer), XXX_Int, 
         (OBJDESC *)odl_tree, "GPS_System_Table", 
         "max_output_image_buffer", msg, &count);
if (status != SUCCESS)
    {
    xxx_LogStatus("XXXLIB",__FILE__,__LINE__,msg);
    xxx_LogStatus("XXXLIB",__FILE__,__LINE__,
	"Error locating max_output_image_buffer in system table");
    return(ERROR);
    }

status = xxx_GetODLField(&systbl->downsamp_wght,
	 sizeof(systbl->downsamp_wght), XXX_Float, (OBJDESC *)odl_tree, 
	 "GPS_System_Table", "downsamp_wght", msg, &count);
if (status != SUCCESS)
    {
    xxx_LogStatus("XXXLIB",__FILE__,__LINE__,msg);
    xxx_LogStatus("XXXLIB",__FILE__,__LINE__,
	"Error locating downsamp_wght in system table");
    return(ERROR);
    }

status = xxx_GetODLField(&systbl->focal_plane_point_weight,
	 sizeof(systbl->focal_plane_point_weight), XXX_Float, 
         (OBJDESC *)odl_tree, "GPS_System_Table", "focal_plane_pt_wght", msg, &count);
if (status != SUCCESS)
    {
    xxx_LogStatus("XXXLIB",__FILE__,__LINE__,msg);
    xxx_LogStatus("XXXLIB",__FILE__,__LINE__,
	"Error locating focal_plane_pt_wght in system table");
    return(ERROR);
    }
    
return(SUCCESS);
}
