/******************************************************************************
NAME:		xxx_write_gcp

PURPOSE:	Write one record to the GCP data file

DEVELOPMENT HISTORY:
$Log: xxx_write_gcp.c,v $
Revision 1.2  2001/07/09 12:40:34  choate
Removed BandNumber check

Revision 1.1  2000/12/07 23:05:54  storey
Original version.


RETURN VALUE: 
Type = int
Value           Description
-----           -----------
SUCCESS     	Successfully wrote GCP data record
ERROR     	Error occurred; GCP data not written

NOTES:		The file pointer is left at the end of the current record
		so sequential calls of xxx_write_gcp will sequentially write all
		the records in the file.

		The GCP data file is assumed to be an ASCII file containing
		three lines per GCP data record, with lines separated by 
		newlines.  The first line contains:
	pt_id ref_line ref_samp lat long elev pred_srch_line pred_srch_samp  $
	delta_y(line) delta_x(sample) accept/reject_flag
		The second line contains the reference image name
		The third line contains the search image name

ALGORITHM REFERENCES:
*******************************************************************************
		        Property of the U.S. Government
                            USGS EROS Data Center
******************************************************************************/
#include "xxx_Types.h"
#include "xxx_LogStatus.h"
#include "xxx_utility.h"

int xxx_write_gcp (
	FILE *gcp_fptr,			/* I: GCP data file pointer	*/
	const GCPDATA *gcp_data)	/* I: One GCP data record	*/
{
int	status;				/* function return status	*/
int 	srch_tmp;			/* temporary search band 	*/

srch_tmp = gcp_data->srch_band;

/*
 * Write the first line's information 
 */
status = fprintf(gcp_fptr,
    "%s %lf %lf %.12lf %.12lf %lf %lf %lf %lf %lf %ld %ld %ld\n",
     gcp_data->pt_id,		/* point id		*/
     gcp_data->ref_line,	/* reference line 	*/
     gcp_data->ref_samp,	/* reference sample	*/
     gcp_data->lat,		/* true latitude of pt	*/
     gcp_data->lon,		/* true longitude of pt	*/
     gcp_data->elev,		/* elevation of pt	*/
     gcp_data->pred_srch_line,	/* predicted search line*/
     gcp_data->pred_srch_samp,	/* predicted search samp*/
     gcp_data->fit_offset[0],	/* delta y (line)	*/
     gcp_data->fit_offset[1],	/* delta x (sample)	*/
     gcp_data->accept_flg,	/* accept(0)/reject(1) flag */
     gcp_data->ref_band,	/* band number in ref img*/
     srch_tmp);			/* band number in srch img*/

/*
 * Write the reference image name on the 2nd line
 */
status = fprintf(gcp_fptr,"%s\n",gcp_data->ref_name);
if (status < 0)
    {
    xxx_LogStatus("XXXLIB", __FILE__, __LINE__,
    	"Error writing reference image name");
    return(ERROR);
    }

/*
 * Write the search image name on the 3rd line
 */
status = fprintf(gcp_fptr,"%s\n",gcp_data->srch_name);
if (status < 0)
    {
    xxx_LogStatus("XXXLIB", __FILE__, __LINE__,
    	"Error writing search image name");
    return(ERROR);
    }

return(SUCCESS);
}
