/******************************************************************************
                Landsat 7 Image Assessment System (IAS) Software
                        Property of the U.S. Government
                            NASA / GSFC / Code 510
*******************************************************************************

UNIT NAME:  xxx_Errno.c

PURPOSE:  Definition of error return values and strings.

INVOCATION METHOD:
none

RETURN VALUE:
Type = void

Value          	Description
-------------- 	----------------------------------------------------------------
none

ARGUMENT LIST:
Name            Type            	Use 	Description
--------------- --------------- 	--- 	---------------------------------------
none

EXTERNAL FILE/RECORD/DBTABLE REFERENCES:
Name                    Use 	Description
----------------------- --- 	-------------------------------------------------
none

EXTERNAL VARIABLES:
Source          	Name                	Type	Use 	Description
--------------- 	------------------- 	-------	--- 	-----------------------
xxx_Errno.c		xxx_errno		long	I	Error number 
xxx_Errno.c		xxx_errlist		char **	I	List of IAS system errors
xxx_Errno.h		xxx_nerr		long	I	Size of xxx_errlist
xxx_Types.h		NULL        		int     I    	Defines NULL pointer


EXTERNAL UNIT/MACRO/PROCEDURE/FUNCTION REFERENCES:
Name                          	Description
----------------------------- 	-------------------------------------------------
XXX_ARRAY_SIZE	                Macro to return size of array

ABNORMAL TERMINATION CONDITIONS, ERROR MESSAGES:
none

ASSUMPTIONS, CONSTRAINTS, RESTRICTIONS:
none

DEVELOPMENT HISTORY:
Author            Change-ID   Release Date       Description of Change
----------------- ----------- ------- ---------- ------------------------------
J. kerich		      R1.0    02/14/97   Original Implementation
T. Raheja		      R1.0    05/20/97   Added error codes
J. kerich	  IASI980424  R3.1    08/11/98   Added XXX_E_FILE_NOT_FOUND & XXX_E_ODL_SYNTAX

NOTES:
See xxx_Errno.h.

Unit xxx_Errno.c PDL:

1	Initialize variables

End Unit xxx_Errno.c
******************************************************************************/

/* System headers */
#include <stdio.h>

/* Local headers */
#include <xxx_Errno.h>
#include <xxx_Types.h>


char *xxx_errlist[] = {
	/* XXX_E_NO_MEMORY */				"no memory available",
	/* XXX_E_NULL_INPUT */				"NULL input to function",
	/* XXX_E_NO_MORE_OBJECTS */			"no more objects",
	/* XXX_E_OBJECT_NOT_FOUND */			"object not found",
	/* XXX_E_OUT_OF_RANGE */			"input out of range",
	/* XXX_E_INVALID_DATA */			"invalid data",
	/* XXX_E_TIMEOUT */				"timeout",
	/* XXX_E_FAILURE */				"unknown failure",
	/* XXX_E_INSUFFICIENT_INPUT */			"insufficient input",
	/* XXX_E_FILE_NOT_FOUND */			"File not found",
	/* XXX_E_ODL_SYNTAX */			       	"ODL syntax",
	NULL
};

long xxx_nerr = XXX_ARRAY_SIZE(xxx_errlist) - 1;

long xxx_errno;
