/******************************************************************************
                Landsat 7 Image Assessment System (IAS) Software
                        Property of the U.S. Government
                            NASA / GSFC / Code 510
*******************************************************************************

UNIT NAME:  xxx_GetDataTypeSize.c

PURPOSE:  Return the data types size in bytes.

INVOCATION METHOD:
	Size = xxx_GetDataTypeSize (DataType)

RETURN VALUE:
Type = long

Value		Description
--------------	---------------------------------------------------------------
positive       	Data types size in bytes
0	        Datatype unknown

ARGUMENT LIST:
Name		Type		Use	Description
---------------	---------------	---	---------------------------------------
DataType        int32           I       HDF data type

EXTERNAL FILE/RECORD/DBTABLE REFERENCES:
Name		       	Use	Description
----------------------	---	-------------------------------------------------
hdf.h                   I       Define vdata structure, constants, and prototypes
hntdefs.h               I       Define constants
xxx_Imagery.h           I       Defines imagery structures, constants, and prototype


EXTERNAL VARIABLES:
Source		Name			Type		Use	Description
---------------	----------------	---------------	---	-----------------------
hntdefs.h       DFNT_FLOAT32            int             I       Defined data type
hntdefs.h       DFNT_FLOAT              int             I       Defined data type
hntdefs.h       DFNT_FLOAT64            int             I       Defined data type
hntdefs.h       DFNT_DOUBLE             int             I       Defined data type
hntdefs.h       DFNT_INT8               int             I       Defined data type
hntdefs.h       DFNT_UINT8              int             I       Defined data type
hntdefs.h       DFNT_INT16              int             I       Defined data type
hntdefs.h       DFNT_UINT16             int             I       Defined data type
hntdefs.h       DFNT_INT32              int             I       Defined data type
hntdefs.h       DFNT_UINT32             int             I       Defined data type
hntdefs.h       DFNT_INT64              int             I       Defined data type
hntdefs.h       DFNT_UINT64             int             I       Defined data type
hntdefs.h       DFNT_UCHAR8             int             I       Defined data type
hntdefs.h       DFNT_CHAR8              int             I       Defined data type
hntdefs.h       SIZE_FLOAT32            int             I       Defined data type size
hntdefs.h       SIZE_FLOAT64            int             I       Defined data type size
hntdefs.h       SIZE_INT8               int             I       Defined data type size
hntdefs.h       SIZE_UINT8              int             I       Defined data type size
hntdefs.h       SIZE_INT16              int             I       Defined data type size
hntdefs.h       SIZE_UINT16             int             I       Defined data type size
hntdefs.h       SIZE_INT32              int             I       Defined data type size
hntdefs.h       SIZE_UINT32             int             I       Defined data type size
hntdefs.h       SIZE_INT64              int             I       Defined data type size
hntdefs.h       SIZE_UINT64             int             I       Defined data type size
hntdefs.h       SIZE_UCHAR8             int             I       Defined data type size
hntdefs.h       SIZE_UCHAR              int             I       Defined data type size

EXTERNAL UNIT/MACRO/PROCEDURE/FUNCTION REFERENCES:
Name				Description
-----------------------------	-------------------------------------------------
none

ABNORMAL TERMINATION CONDITIONS, ERROR MESSAGES:
none

ASSUMPTIONS, CONSTRAINTS, RESTRICTIONS:
none

DEVELOPMENT HISTORY:
Author			Change-ID	Release	Date		Description of Change
-----------------	-----------	-------	--------	------------------------------
J. kerich		N/A		R1.0	03/25/97	Original Implementation

NOTES:

Unit xxx_GetDataTypeSize PDL:

	Return the size of the DataType

End Unit xxx_GetDataTypeSize
******************************************************************************/

/* Local headers */
#include <hdf.h>
#include <hntdefs.h>
#include <xxx_Imagery.h>

long xxx_GetDataTypeSize(int32 DataType)
{

	switch (DataType)
	{
	case DFNT_FLOAT32:         /* same DFNT_FLOAT */
		return(SIZE_FLOAT32);
	case DFNT_FLOAT64:         /* same DFNT_DOUBLE */
		return (SIZE_FLOAT64);
	case DFNT_INT8:
		return (SIZE_INT8);
	case DFNT_UINT8:
		return (SIZE_UINT8);
	case DFNT_INT16:
		return (SIZE_INT16);
	case DFNT_UINT16:
		return (SIZE_UINT16);
	case DFNT_INT32:
		return (SIZE_INT32);
	case  DFNT_UINT32:
		return (SIZE_UINT32);
	case DFNT_INT64:
		return (SIZE_INT64);
	case DFNT_UINT64:
		return (SIZE_UINT64);
	case DFNT_UCHAR8:          /* same DFNT_UCHAR */
		return (SIZE_UCHAR8);
	case DFNT_CHAR8:	   /* same  DFNT_CHAR */
		return(SIZE_UCHAR);
	}
	return (0);
}
