/*******************************************************************************
NAME:		GXX_CVTIME

PURPOSE:	Time conversion functions

DEVELOPMENT HISTORY:
$Log: gxx_cvtime.c,v $
Revision 1.1.1.1  2000/10/19 14:37:35  morfitt
ram:  Inital EO1 processing system.

Revision 1.1.1.1  1999/12/01 19:01:20  mbs
Import of IAS 3.22

 * 
 *    Rev 1.6   Fri Mar 06 07:59:40 1998   tbeckman
 * Changed longs to ints to remove warnings from the -64 compile.
 * 
 *    Rev 1.5   Fri Aug 22 10:12:26 1997   vanderv
 * added function to convert year, day of year,
 * hours, minutes, and seconds to seconds
 * 
 *    Rev 1.4   Fri Aug 08 10:16:54 1997   ias
 * Added prolog comments, removed the SUCCESS
 * retuns, changed gxx_cvtime to return the
 * total seconds instead of an argument, and
 * removed fsec from argument list of gxx_cvtsec.
gxx_cvtime.c,v $
Revision 1.3  1997/05/22 19:57:07  rockvam
Changes to build gxxMath using gxxInclude files.

********************************************************************************
               Landsat 7 Image Assessment System (IAS) Software
                       Property of the U.S. Government
                               NASA/GSFC/EDC
*******************************************************************************/
#include "gxx_geo_math.h"

/******************************************************************************
NAME:		gxx_cvtime

PURPOSE:	Convert the input hours, minutes, and seconds to total seconds

Return Value:
Type = int   (Total seconds)
******************************************************************************/
int  gxx_cvtime(int  hour,	/* I Hours		*/
	        int  min,	/* I Minutes		*/
	        int  sec)	/* I Seconds		*/
{
return (hour * 3600 + min * 60 + sec);
}

/******************************************************************************
NAME:		gxx_cvt2sec

PURPOSE:	Convert the input year, day, hours, minutes, and 
		seconds to total seconds

Return Value:
Type = double  (Total seconds)
******************************************************************************/
double gxx_cvt2sec(
		int  year,	/* I Year (4digits)	*/
		int  day,	/* I Day		*/
		int  hour,	/* I Hours		*/
	        int  min,	/* I Minutes		*/
	        double sec)	/* I Seconds		*/
{
double num_days;
if (gxx_isleap(year) == TRUE)
    num_days = 366.0;
else
    num_days = 365.0;

return ((double)SEC_PER_DAY*num_days*(double)year + 
	(double)day*(double)SEC_PER_DAY+
	(double)hour*3600 + (double)min*60 + sec);
}


/*******************************************************************************
NAME:		gxx_cvtsec

PURPOSE:	Convert total seconds to hours, minutes, and seconds.
*******************************************************************************/
void gxx_cvtsec(int  tsec,	/* I Total seconds	*/
    	        int  *hour,	/* O Hours		*/
	        int  *min,	/* O Minutes		*/
	        int  *sec)	/* O Seconds		*/
{

*hour = tsec / 3600;
*min =  (tsec % 3600) / 60;
*sec = tsec % 60;
}


/*******************************************************************************
NAME:		gxx_cvtmil

PURPOSE:	Convert total milleseconds to hours, minutes, seconds, and
		fractional seconds
*******************************************************************************/

void gxx_cvtmil(int  milsec,	/* I Total milleseconds	*/
	        int  *hour,	/* O Hours		*/
	        int  *min,	/* O Minutes		*/
	        int  *sec,	/* O Seconds		*/
	        int  *fsec)	/* O Fractional seconds	*/
{

*hour = milsec / MIL_PER_HOUR;
*min = (milsec % MIL_PER_HOUR) / MIL_PER_MIN;
*sec = (milsec % MIL_PER_MIN)  / MIL_PER_SEC;
*fsec = milsec % MIL_PER_SEC;
}
