/******************************************************************************
NAME:		latlondiffs

PURPOSE:        calculate the difference between our coords and
		the metadata's coords for one point.

DEVELOPMENT HISTORY:
AUTHOR	      Change-ID    DATE	    Description of Change
------	      ---------  --------   ---------------------
A.VanderVorste	1.0 	  04/97     Release 1.0 development.
$Log: gxx_latlondiffs.c,v $
Revision 1.1.1.1  2000/10/19 14:37:35  morfitt
ram:  Inital EO1 processing system.

Revision 1.1.1.1  1999/12/01 19:01:20  mbs
Import of IAS 3.22

 * 
 *    Rev 1.1   Wed Aug 20 14:03:26 1997   tgardner
 * changed function return to none in prologue,
 * fixed comments, and converted if tests to test
 * against 0.0.

RETURN VALUE:
None

NOTES:
1) LS7 will not cross the poles because: 1. the inclination angle is not
	large/high enough   2. the field of view is not wide enough
	to view the poles. 
2) It is assumed that the differences between points won't be larger than
   half the globe apart. (So we can tell the difference between crossing 
   the 0 degree meridian and crossing the 180 degree meridian.

ALGORITHM REFERENCES:

******************************************************************************
		Landsat 7 Image Assessment System (IAS) Software
		        Property of the U.S. Government
                                NASA/GSFC/EDC
******************************************************************************/
#include <stdio.h>
#include <math.h>
#include "gxx_const.h"


void gxx_latlondiffs(
    double blat,	/* I: beginning latitude in degrees		*/
    double blon,	/* I: beginning longitude in degrees		*/
    double elat,	/* I: ending latitude in degrees		*/
    double elon,	/* I: ending longitude in degrees		*/
    double *diff_lat,	/* O: lat difference in degrees			*/
    double *diff_lon)	/* O: lon difference in degrees			*/ 

{
double	neg,		/* the negative of the two numbers crossing 180 */
	new_360,	/* the negative coord coverted to b/t 0 and 360 */
	pos;		/* the positive of the two numbers crossing 180 */


/*
 Calculating the lat diffs (simple since we won't cross the poles! :)
*/
*diff_lat = MAX(elat,blat) - MIN(elat,blat);

/*
 Calculating the lon diffs
*/
/* if they're both in the eastern or both in the western hemisphere
   or crossing 0 degrees longitude				   */
if ((blon >= 0.0 && elon >= 0.0) || (blon <= 0.0 && elon <= 0.0) || 
    (fabs(blon) < 90.0 && fabs(elon) < 90.0))
    {
    *diff_lon = MAX(elon,blon) - MIN(elon,blon);
    }
else /* we crossed the intl. date line (180 degrees longitude) */ 
    {
    /* There is a negative and a positive number.
       Convert the negative number to a number between 0 and 360 */
    neg = MIN(blon,elon);
    pos = MAX(blon,elon);
    new_360 = neg + 360.0;
    *diff_lon = new_360 - pos;
    }

}

