/****************************************************************************
NAME: NEAREST_POWER2

PURPOSE:
Calculate the next power of two

DEVELOPMENT HISTORY:
VERSION  AUTHOR        DATE     REASON
-------  ------        -----    -----------------------------
  1.0	 D. Steinwand  04/92	Original development
  1.0    D. Hollaren   04/96    Incorporated into LAS
  1.0	 A.VanderVorste06/97	Incorporated into IAS

$Log: gxx_nearest_power2.c,v $
Revision 1.1.1.1  2000/10/19 14:37:35  morfitt
ram:  Inital EO1 processing system.

Revision 1.1.1.1  1999/12/01 19:01:20  mbs
Import of IAS 3.22

 * 
 *    Rev 1.3   Fri Mar 06 08:19:50 1998   tbeckman
 * Changed longs to ints to remove warnings in the -64 compile.
 * 
 *    Rev 1.2   Fri Aug 22 11:24:16 1997   tgardner
 * fixed prologue, added check of power of two
 * to make sure an endless loop does not occur
 * if input parameter is too large.

RETURN VALUE:
Type = int
Value    Description
-----    -----------
SUCCESS  Successful completion
ERROR    Operation failed

NOTES:

ALGORITHM REFERENCES:

*****************************************************************************/
#include <math.h>
#include "gxx_debug.h"
#include "gxx_utility.h"
#include "gxx_structures.h"

/* Given NS, find and return the next greatest power of two
  --------------------------------------------------------*/
int  gxx_nearest_power2(int  ns)
{
int  pow2;

/* there is a slight possibility that the next greatest power
   of two could be greater than the largest int value, if so
   pow2 will become negative so this case should be checked */

for (pow2 = 1;; pow2 = pow2 << 1)
   if ((pow2 >= ns) || (pow2 < 0.0)) break;

GXX_ASSERT(pow2 > 0.0);

return(pow2);
}
