/****************************************************************************
NAME:           POLY

PURPOSE:        Calculate the value of an n-th degree polynomial function.    

DEVELOPMENT HISTORY:
$Log: gxx_poly.c,v $
Revision 1.1.1.1  2000/10/19 14:37:35  morfitt
ram:  Inital EO1 processing system.

Revision 1.1.1.1  1999/12/01 19:01:20  mbs
Import of IAS 3.22

 * 
 *    Rev 1.6   Fri Mar 06 07:59:54 1998   tbeckman
 * Changed longs to ints to remove warnings from the -64 compile.
 * 
 *    Rev 1.5   Fri Aug 08 09:51:56 1997   gschmidt
 * Added the Property of clause. -gs
Revision 1.3  1997/06/02 19:03:06  gschmidt
Added the description of the type of polynomial evaluation, since we have
several different polynomial evaluation routines. -gs

 * Revision 1.2  1997/05/22  19:57:18  rockvam
 * Changes to build gxxMath using gxxInclude files.
 *

RETURN VALUE:
Type = double

NOTES:
This routine is from the EL1PS ephemeris poly.c routine.  It is the evaluation
of a 1-dimensional polynomial (x term only) of a given degree.

ALGORITHM REFERENCES:
        "Numerical Recipes, the Art of Scientific Computing"
        by Press, Flannery, Teukolsky, and Vetterling
        Cambridge University Press, Second Edition
	Section 5.3; Polynomials and Rational Functions

******************************************************************************
                Landsat 7 Image Assessment System (IAS) Software
                        Property of the U.S. Government
                                NASA/GSFC/EDC
******************************************************************************/

#include "gxx_geo_math.h"

double  gxx_poly(
  const double t,	/* I: Location where function is to be evaluated  */
  const int ncoe,       /* I: Order of polynomial (number of coeffs)      */
  const double coe[])   /* I: Array of polynomial coefficients	          */

{
int	i;	/* Looping variable					  */
double	p;	/* Value of the polynomial at t				  */

/*
  Evaluate the polynomial
  p = coe[0] + t*(coe[1] + t*(coe[2] + t*(coe[3] + ... t*(coe[n-1])...)))
*/
p = coe[ncoe-1];
for (i = ncoe-2; i >= 0; i--)
    p = p * t + coe[i];

return(p);
}
