/******************************************************************************
                        Property of the U.S. Government
                            NASA / GSFC / Code 510
*******************************************************************************

PURPOSE: xxx_Lagrange calculates the interpolation of the requested time

INVOCATION METHOD:
     sum = xxx_Lagrange( p_YY, p_XX, n_pts, in_time)

RETURN VALUE:
    Type = double

Value             Description
--------------    -------------------------------------------------------------
sum               the interpolated value

ARGUMENT LIST: 
Name             Type               Use Description
---------------- ------------------ --- ---------------------------------------
p_YY             double *           I   Pointer to the array of input values
p_XX             double *           I   Pointer to the array of times closest to the 
					  requested time
n_pts            int                I   No of points for the interpolation 
in_time          double             I   Requested time for interpolation

EXTERNAL FILE/RECORD/DBTABLE REFERENCES: 
Name                      Use Description
------------------------- --- -------------------------------------------------
None

EXTERNAL VARIABLES: 
Source          Name                 Type        Use Description 
--------------- -------------------  ----------- --- --------------------------
none

EXTERNAL UNIT/MACRO/PROCEDURE/FUNCTION REFERENCES:
Name                                Description
-----------------------------       -------------------------------------------
none

ABNORMAL TERMINATION CONDITIONS, ERROR MESSAGES:
none

ASSUMPTIONS, CONSTRAINTS, RESTRICTIONS:
None

DEVELOPMENT HISTORY:
Author            Change-ID   Release Date       Description of Change
----------------- ----------- ------- ---------- ------------------------------ 
T. Raheja                     R2.0    7/30/97    Original implementation

NOTES:
None

Unit xxx_Lagrange PDL:

1     DOFOR all the elements in the p_YY
  	  Copy the value of p_YY to local variable
2         DOFOR all the elements in the p_XX
             Calculate the interpolation for the requested time 
3         ENDDO
          Add the calculation to the sum
4     ENDDO

      RETURN sum

End xxx_Lagrange PDL:
******************************************************************************/
double xxx_Lagrange( double *p_YY, double *p_XX, int n_pts, double in_time)
{
	double term;
	double sum;
	int i,j;

	sum = 0.0;
	for (i=0; i< n_pts; i++)
	{
		term = p_YY[i];
		for(j=0; j< n_pts; j++)
		{
			if (j != i)
				term = term* ( in_time - p_XX[j]) / ( p_XX[i] - p_XX[j] );
		}
		sum = sum + term;
	}
	return sum;
}
