/*****************************************************************************
NAME: 		xxx_RMSE	

PURPOSE:
	Calculates the root mean square error.


DEVELOPMENT HISTORY:
$Log: xxx_RMSE.c,v $
Revision 1.1  2000/12/07 18:11:44  storey
Original version


RETURN VALUE:
Type = int
Value    Description
-----    -----------
SUCCESS  RMSE was calculated.
ERROR    Sample size was zero.


ALGORITHM REFERENCES:
******************************************************************************
                        Property of the U.S. Government
                           USGS EROS Data Center
*******************************************************************************/
#include <math.h>		/* sqrt prototype */
#include "xxx_LogStatus.h"	/* xxx_LogStatus prototype */
#include "xxx_Types.h"		/* ERROR/SUCCESS defines */
#include "xxx_math.h" 		/* xxx_RMSE prototype */


int xxx_RMSE
(
    const double *sample,   /* I: pointer to the data */
    int  nsamps,            /* I: number of samples */
    double *rms_error    	/* O: root mean square error of the data */
)
{
    int  i;	       			/* loop variable */
    double sum_sqr=0.0;		/* sum of the squared samples */

/*---------------------------------------------------------------------
    RMSE calculation.  If number of samples is invalid return ERROR. 
---------------------------------------------------------------------*/
    if (nsamps <= 0)
    {
        xxx_LogStatus("XXXLIB", __FILE__, __LINE__, "Sample size is invalid.");
		*rms_error = 0.0;
        return(ERROR);
    }

    for (i = 0; i < nsamps; i++)
        sum_sqr += sample[i] * sample[i];

    *rms_error = sqrt(sum_sqr / nsamps);

    return(SUCCESS);
}
