
/******************************************************************************
NAME:		xxx_check_fill

PURPOSE:  Check the input window for fill data over the specified percentage.


RETURN VALUE:   Type = int
Value    Description
-----    -----------
  1      Fill data was over the specified amount.
  0      Fill data was not over the specified amount.

NOTES:
This routine is useful to determine if there is too much fill data in a
window.  If too much fill data exists, then the window might not be good
for correlating.  Fill data has a value of 0.0.

ALGORITHM REFERENCES:

******************************************************************************/
#include <stdlib.h>

int xxx_check_fill (
  const float *window,          /* I: Image window                         */
  int  size,                    /* I: Size of the window (nlines * nsamps) */
  float fill_thresh)            /* I: Threshold for fill data              */
{
  int i,                /* Loop variable                          */
      count = 0;        /* Count of fill pixels                   */
  float fill;           /* Fraction of fill data                  */

  /* The size is <= 0, then don't process.
   */
  if (size <= 0)
    return (0);

  /* Count the number of fill pixels in the window.
   */
  for (i = 0; i < size; i++)
    if (window[i] == 0.0)
      count++;

  /* Determine the amount of fill.
   */
  fill = (float) count / (float) size;

  /* If the fill is > the specified fill_thresh then return 1.
   */
  if (fill > fill_thresh)
    return (1);

  /* Otherwise return 0.
   */
  return (0);
}
