/*****************************************************************************
NAME: 		xxx_corr_coeff

PURPOSE:
	Calculates the correlation coefficient between two data groups.


DEVELOPMENT HISTORY:
$Log: xxx_corr_coeff.c,v $
Revision 1.1  2001/02/05 14:07:28  storey
Original Version.


RETURN VALUE:
Type = int
Value    Description
-----    -----------
SUCCESS  correlation coefficient was calculated.
ERROR    Sample size or standard deviation was zero.


ALGORITHM REFERENCES:
******************************************************************************
                        Property of the U.S. Government
                            USGS EROS Data Center
*******************************************************************************/
#include "xxx_LogStatus.h"	/* xxx_LogStatus prototype */
#include "xxx_Types.h"		/* ERROR/SUCCESS defines */
#include "xxx_math.h"		/* xxx_corr_coeff prototype */


int xxx_corr_coeff
(
    const double *x_sample,     /* I: pointer to the x data */
    const double *y_sample,     /* I: pointer to the y data */
    int  nsamps,               	/* I: number of samples in x and in y */
    double x_mean,             	/* I: mean of the x data */
    double y_mean,             	/* I: mean of the y data */
    double x_std_dev,          	/* I: standard deviation of the x data */
    double y_std_dev,          	/* I: standard deviation of the y data */
    double *corr_coeff 	   		/* O: correlation coefficient between 
                                      the x and y data */
)
{
    int  i;	       			/* loop variable */
    double xy_dev_sum=0.0;	/* product of x and y deviation from means */
    double covariance;		/* covariance between x and y */

/*---------------------------------------------------------------------
    Calculate the covaraince and then the correlation coefficient 
    between the two data groups.  Return ERROR if calculations will
    result in divide by zero or value is invalid. 
---------------------------------------------------------------------*/
    if (nsamps <= 0 || x_std_dev <= 0.0 || y_std_dev <= 0.0)
    {
        xxx_LogStatus("XXXLIB", __FILE__, __LINE__, 
                      "Sample size and/or std. deviation is invalid.");
		*corr_coeff = 0.0;
        return(ERROR);
    }

    for (i = 0; i < nsamps; i++)
        xy_dev_sum += (x_sample[i] - x_mean) * (y_sample[i] - y_mean);

    covariance = xy_dev_sum / (nsamps-1);

    *corr_coeff = covariance / (x_std_dev * y_std_dev);

    return(SUCCESS);
}
