#include <stdlib.h>
#include <stdio.h>
#include "xxx_Types.h"
#include "xxx_const.h"

/****************************************************************************
NAME:				xxx_eval

PURPOSE:  Evaluates a polynomial at a given point

DEVELOPMENT HISTORY:
AUTHOR        Change-ID    DATE     Description of Change
------        ---------  --------   ---------------------
Gail Schmidt     1.0     05/30/97   Brought over from LAS.

RETURN VALUE:   Type = int
Value    Description
-----    -----------
SUCCESS  Successful completion
ERROR    Operation failed

NOTES:
This is the LAS c_eval.c function.  This is an evaluation of a bidimensional
polynomial (x and y terms) of a given degree, where the fit was done not using
the x*y terms.  (The gxx_eval_poly routine uses the x*y terms.)

ALGORITHM DESCRIPTION:
	Determine degree of polynomial
	Return evaluation

ALGORITHM REFERENCES:

******************************************************************************/

void xxx_eval (
  int degree,	   /* I: Degree of polynomial                      */
  const double *a, /* I: Array of polynomial coefficients          */
  double x,        /* I: X coordinates to be evaluated             */
  double y,	   /* I: Y coordinates to be evaluated             */
  double *val)     /* O: Value of the polynomial at the point      */
{
  if (degree == 4)
    *val = a[0] + x*(a[1]+x*(a[3]+x*(a[6]+x*a[10]+y*a[11])+y*a[7])
           +y*a[4]) + y*(a[2]+y*(a[5]+y*(a[9]+y*a[14]+x*a[13])
           +x*a[8]+x*x*a[12]));
  else if (degree == 3)
    *val = a[0] + x*(a[1]+x*(a[3]+a[6]*x+a[7]*y)+a[4]*y)
           + y*(a[2]+y*(a[5]+a[9]*y+a[8]*x));
  else if (degree == 2)
    *val = a[0] + x*(a[1]+a[3]*x+a[4]*y) + y*(a[2]+a[5]*y);
  else
    *val = a[0] + a[1]*x + a[2]*y;

  return;
}
