/****************************************************************************
NAME:				xxx_eval_poly	

PURPOSE:  Evaluates a polynomial at a given point

DEVELOPMENT HISTORY:
$Log: xxx_eval_poly.c,v $
Revision 1.2  2000/11/28 13:22:36  storey
Cleaned up include files.


RETURN VALUE:
Type = double

Value = Value of the polynomial at a given point.


NOTES:
     Takes two points in an x/y coordinate system and returns a new values
     acoording to the polynomail defined by the coefficients a.
     This routine is taken from the EL1PS inverse library eval_poly.c.  It
     is the evaluation of a bidimensional polynomial (x and y terms) of a
     given degree where the fit was done using x*y terms.  (The xxx_eval
     routine does not use the x*y terms.)

*****************************************************************************/
#include <math.h>
#include "xxx_math.h"

double xxx_eval_poly
(
   int degree,	/* I: Degree of polynomial */
   double *a,	/* I: Array of polynomial coefficients */
   double x,	/* I: X coordinates to be evaluated */
   double y	/* I: Y coordinates to be evaluated */
)
{
   double val=0.0;
   int i,m; 		/* loop variables */ 
   double xp,yp;	/* y coordinate raised to (degree) power */
   double x2,x3,y2,y3;	/* x and y values squared and cubed */
   double *coeff;	/* pointer to polynomial coefficients */

   if( degree==1 )
      val = a[0] + a[1] * x + a[2] * y + a[3] * x * y;
   else if( degree==2 )
   {
      x2 = x * x;
      y2 = y * y;
      val = a[0] + a[1] * x + a[2] * x2 + a[3] * y + a[4] * x * y +
            a[5] * x2 * y + a[6] * y2 + a[7] * x * y2 + a[8] * x2 * y2;
   }
   else if( degree==3 )
   {
      x2 = x * x;
      y2 = y * y;
      x3 = x2 * x;
      y3 = y2 * y;
      val = a[0] + a[1] * x + a[2] * x2 + a[3] * x3 + a[4] * y + 
            a[5] * x * y + a[6] * x2 * y + a[7] * x3 * y + a[8] * y2 +
            a[9] * x * y2 + a[10] * x2 * y2 + a[11] * x3 * y2 + 
            a[12] * y3 + a[13] * x * y3 + a[14] * x2 * y3 + a[15] * x3 * y3;
   }
   else
   {
      coeff = a;
      for( i=0 ; i<=degree ; i++ )
      {
         yp = pow(y,(double)i);
         for( m=0 ; m<=degree ; m++, coeff++ )
         {
            xp = pow(x,(double)m);
            val += xp*yp*(*coeff);
         }
      }
   }

   return (val);
}
