/*******************************************************************************
NAME:                           xxx_fulljd.c

PURPOSE: Find date from full julian date.

DEVELOPMENT HISTORY:
Ron Morfitt original development.

RETURN VALUE:
Type = integer


REFERENCES: "Practical Astronomy With Your Calculator" by Peter Duffett-Smith 
         Peter Duffet-Smith
         Third edition         Page 8

*******************************************************************************/

#include <math.h>
#include "xxx_const.h"
#include "xxx_math.h"

int xxx_fjd2date 
(
   double jd,		/* I: full julian day */
   int  *y,		/* O: year */ 
   int  *m,		/* O: month */ 
   double *dday		/* O: decimal day */
)
{

   int  a,		/* a-g, i correspond to intermediate */
        b,		/* variables used in the book cited in the REFERENCES */
        c,		/* section of the prologue */
        d,
	e,
	g,
	i;
   double f;            /* ditto, intermediate variable */


   jd += 0.5;
   i = (int)jd;
   f = jd - (double)i;
   a = (int)(((double)i-1867216.25)/36524.25);
   b = i+1+a-(int)((double)a/4.0);
   c = b+1524;
   d = (int)(((double)c-122.1)/365.25);
   e = (int)(365.25*(double)d);
   g = (int)(((double)c-(double)e)/30.6001);

   *dday = c-e+f-(int)((double)g*30.6001);
   *m = g-13; if(g < 13.5) *m = g-1;
   *y = d-4716; if(*m < 2.5) *y = d-4715;

    return(SUCCESS);
}
