/*******************************************************************************
NAME:                           xxx_fulljd.c

PURPOSE: Find the full julian date.

DEVELOPMENT HISTORY:
Chuck Wivell original development.
$Log: xxx_fulljd.c,v $
Revision 1.1  2000/11/22 21:26:09  storey
Original version - conversion from gxx_fulljd.c


RETURN VALUE:
Type = double


REFERENCES: "Practical Astronomy With Your Calculator" by Peter Duffett-Smith 
         Peter Duffet-Smith
         Second edition         Page 21

********************************************************************************
                       Property of the U.S. Government
                          USGS EROS Data Center

*******************************************************************************/

#include <math.h>
#include "xxx_const.h"
#include "xxx_math.h"

double xxx_fulljd 
(
   int  yr,		/* I: year */ 
   int  mo,		/* I: month */ 
   int  day,		/* I: day */
   double seconds	/* I: seconds of day */
)
{

   double fjd;		/* julian date in the Julian or Gregorian calander */
   int  a,		/* a, b, c, and d correspond to intermediate */
        b,		/* variables used in the book cited in the REFERENCES */
        c,		/* section of the prologue */
        d;

   double dday;		/* day of month plus fraction of day */

   dday = (double)(day) + seconds / (double)SEC_PER_DAY;

   if (mo == 1 || mo == 2) 
      {
      yr--;
      mo += 12;
      }

    a = (int)((double)(yr) / 100.0);
    b = 2 - a + (int)((double)(a) / 4.0);
    c = (int)(365.25 * yr);
    d = (int)(30.6001 * (mo + 1));
    fjd = b + c + d + dday + 1720994.5;
    return(fjd);
}
