/******************************************************************************
  EO1 geometric include file
   this file includes:
     function prototypes for the XXX Math library
*******************************************************************************/
#ifndef _XXX_MATH_H_
#define _XXX_MATH_H_

#include <math.h>
#include <stdlib.h>
#include "xxx_Types.h"
#include "xxx_const.h"
#include "xxx_structures.h"

#define	GPS_YEAR	1980	/* GPS epoch is 24:00:00 05-JAN-1980 */
#define	GPS_DAY		6

/* XXX GEO MATH */
int xxx_calc_transform_poly 
(
  const double source_proj_x[4],
  const double source_proj_y[4],
  const double target_proj_x[4],
  const double target_proj_y[4],
  int include_xy_term,
  double poly_x[4],
  double poly_y[4]);

int xxx_check_fill( float *window, int  size, float fill_thresh);
int xxx_QR(double *A, int m, int n, double *v, int flag);
int xxx_QRsolve(double *A, int m, int n, double *v, double *b, int iflag);
void xxx_anglebetwn(const VECTOR *v1, const VECTOR *v2, double *angle);
void xxx_cross(const VECTOR *vec1, const VECTOR *vec2, VECTOR *vec3);
double xxx_dot(const VECTOR *vec1,const VECTOR *vec2);
int xxx_earth2orbit(VECTOR satpos, VECTOR satvel, double transf_matrix[3][3]);
double xxx_eval_poly(int degree, double *a, double x, double y);
int xxx_fft(COMPLEX *, int, int);
double xxx_fulljd ( int yr, int mo, int day, double seconds);
int xxx_invmatrix(const double inmatrix[3][3], double outmatrix[3][3]);
int xxx_isleap (int year);
int xxx_linear_solution( double **A, int n, double b[]);
void xxx_multmatrix(const double[3][3], const double[3][3], double[3][3]);
double xxx_norm(const VECTOR *v);
double xxx_poly(const double t, const int ncoe, const double coe[]);
void xxx_transpose(double A[3][3]);
void xxx_transvec(VECTOR Xold, double Trans[3][3], VECTOR *Xnew);
int xxx_unit(const VECTOR *vec, VECTOR *uni);
int xxx_mean(const double *sample, int nsamps, double *mean);
int xxx_RMSE(const double *sample, int nsamps, double *rms_error);
int xxx_std_dev(const double *sample, int nsamps, double mean, 
                double *std_dev);
double xxx_t_conf(double threshold, int dof);
int xxx_corr_coeff(const double *x_sample, const double *y_sample, int nsamps,
                   double x_mean, double y_mean, double x_std_dev,
                   double y_std_dev, double *corr_coeff); 
void xxx_matrix_transpose( 
	const double *a, 	/* I: matrix to be transposed */
	double *b, 		/* O: transpose of matrix */
	const int row, 	/* I: number of rows in matrix */
	const int col );	/* I: number of columns in matrix */

int xxx_matrix_multiply( 
	const double *a,	/* I: ptr to matrix */
	const double *b, 	/* I: ptr to matrix */
	double *m, 		/* O: ptr to output matrix */
	const int arow, 	/* I: number of rows in a */
	const int acol, 	/* I: number of cols in a */
	const int brow, 	/* I: number of rows in b */
	const int bcol);	/* I: number of cols in b */

int xxx_matrix_inverse(
    const double *a,	/* I: input array */
    double *y,  	/* O: ouput array */
    const int n 	/* I: dimension of a (n y n) */
);

void xxx_matrix_add( 
    const double *a,	/* I: input matrix 1 */
    const double *b,  	/* I: input matrix 2 */
    double *m,  	/* O: output matrix */
    const int row,  	/* I: number of rows in matricies */
    const int col	/* I: number of cols in matricies */
);

void xxx_matrix_subtract(
    const double *a, 	/* I: input matrix 1 */
    const double *b, 	/* I: input matrix 2 */
    double *m, 		/* O: output matrix */
    const int row,  	/* I: number of rows in matrix */
    const int col   	/* I: number of cols in matrix */
);

int xxx_ludcmp(
    double *a, 		/* IO: input matrix, output LU decomp matrix */
    const int n,    	/* I: size of matrix */
    int *indx,		/* O: index */
    double *d  		/* O: flag */
);

void xxx_lubksb(
    double *a, 		/* IO: lu decomp matrix */
    const int n,    	/* I: size of matrix */
    const int *indx,	/* I: flag */
    double b[] 		/* O: solution vector */
);

/* xxx_cvtime.c functions */
int xxx_cvtime(int hour, int min, int sec);
void xxx_cvtsec(int tsec, int *hour, int *min, int *sec);
void xxx_cvtmil(int milsec, int *hour, int *min, int *sec, int *fsec);
double xxx_cvt2sec(int year, int day, int hour, int min, double sec);

/* xxx_rotate.c functions */
void xxx_rotatex(VECTOR, double, VECTOR *);
void xxx_rotatey(VECTOR, double, VECTOR *);
void xxx_rotatez(VECTOR, double, VECTOR *);

/* xxx_complex.c functions */
void xxx_abs_complex(const COMPLEX *cpx_in, double *value);
void xxx_add_complex(const COMPLEX *cpx_in1, const COMPLEX *cpx_in2, COMPLEX *cpx_out);
void xxx_conj_complex(const COMPLEX *cpx_in, COMPLEX *cpx_out);
int xxx_divide_complex(const COMPLEX *cpx_in1, const COMPLEX *cpx_in2, COMPLEX *cpx_out);
void xxx_multiply_complex(const COMPLEX *cpx_in1, const COMPLEX *cpx_in2, COMPLEX *cpx_out);
void xxx_scalar_complex(double scalar, const COMPLEX *cpx_in, COMPLEX *cpx_out);
int xxx_sqrt_complex(const COMPLEX *cpx_in, COMPLEX *cpx_out);
void xxx_sub_complex(const COMPLEX *cpx_in1, const COMPLEX *cpx_in2, COMPLEX *cpx_out);
/* ************************
   Kalman filter functions
   and definitions
  ************************* */

int xxx_predict_state(
    double *S,
    double *Xk,
    double *Xk1,
    int m
);

int xxx_kalman_gain(
    double *Pn,
    double *H,
    double *R,
    double *K,
    int m,
    int n
);

int xxx_predict_error_covar(
double *S,
double *Pn,
double *Pn1,
double *Q,
int m
);

int xxx_filter_state(
    double *Xk,
    double *Xk1,
    double *K,
    double *z,
    double *H,
    int m,
    int n
);

int xxx_filter_error_covar(
    double *K,
    double *H,
    double *Pn,
    double *Pn1,
    int m,
    int n
);

int xxx_smooth_gain(
double *P,
double *Pn,
double *S,
double *A,
int m
);

int xxx_smooth_state(
    double *X,
    double *Xk,
    double *XN,
    double *A,
    double *XN1,
    int m
);

double xxx_Lagrange(
    double *Y,
    double *X,
    int count,
    double time
);

#endif
