/*****************************************************************************
NAME: 		xxx_mean	

PURPOSE:
	Calculates the mean of the data.


DEVELOPMENT HISTORY:
$Log: xxx_mean.c,v $
Revision 1.1  2000/12/07 18:11:59  storey
Original version


RETURN VALUE:
Type = int
Value    Description
-----    -----------
SUCCESS  Mean was calculated.
ERROR    Sample size was zero.


ALGORITHM REFERENCES:
******************************************************************************
                        Property of the U.S. Government
                            USGS EROS Data Center
*******************************************************************************/
#include "xxx_LogStatus.h"	/* xxx_LogStatus prototype */
#include "xxx_Types.h"		/* ERROR/SUCCESS defines */
#include "xxx_math.h"		/* xxx_mean prototype */


int xxx_mean
(
    const double *sample,       /* I: pointer to the data */
    int nsamps,               	/* I: number of samples */
    double *mean             	/* O: mean of the data */
)
{
    int i;	       		/* loop variable */
    double sum=0.0;		/* sum of the samples */

/*---------------------------------------------------------------------
    Mean calculation.  If number of samples is invalid return ERROR.
---------------------------------------------------------------------*/
    if (nsamps <= 0)
    {
        xxx_LogStatus("XXXLIB", __FILE__, __LINE__, "Sample size is invalid.");
		*mean = 0.0;
        return(ERROR);
    }

    for (i = 0; i < nsamps; i++)
        sum += sample[i];

    *mean = sum / nsamps;

    return(SUCCESS);
}
