/*******************************************************************************
NAME: 		xxx_rotatex (r_old, angle, r_new)

PURPOSE:        
Transformation of coorindates through rotation around the x-axis and
specified angle.

DEVELOPMENT HISTORY:
$Log: xxx_rotate.c,v $
Revision 1.1  2000/11/22 21:26:17  storey
Original version - conversion from gxx_rotate.c



RETURN VALUE:
None

********************************************************************************
                       Property of the U.S. Government
                           USGS EROS Data Center
*******************************************************************************/
#include "xxx_math.h"

void  xxx_rotatex(VECTOR r_old, /* I: coordinates (x, y, z) in the old
                                          system                             */
                  double angle, /* I: angle to be rotated, in radian, positive
                                  	  anticlockwise                      */
                  VECTOR *r_new) /* O: coordinates in the new system */

{
  double ca, sa;		/* cosine and sine of angle		*/

  /* Compute the sine and cosine of the angle.
   */
  ca = cos(angle);
  sa = sin(angle);

  /* Calculate the new coords using the following:
   * new x coord = old x coord
   * new y coord = cos * old y coord + sine * old z coord
   * new z coord = cos * old z coord - sine * old y coord
   */
  r_new->x = r_old.x;
  r_new->y = ca * r_old.y + sa * r_old.z;
  r_new->z = ca * r_old.z - sa * r_old.y;
}




/******************************************************************************
NAME: 		xxx_rotatey(r_old, angle, r_new)

PURPOSE:        Transformation of coorindates through rotation around the
                y-axis and specified angle.

DEVELOPMENT HISTORY:
$Log: xxx_rotate.c,v $
Revision 1.1  2000/11/22 21:26:17  storey
Original version - conversion from gxx_rotate.c



RETURN VALUE:
None

******************************************************************************/
void  xxx_rotatey(VECTOR r_old, /* I: coordinates (x, y, z) in the old
                                          system                             */
                  double angle, /* I: angle to be rotated, in radian, positive
                                  anticlockwise                              */
                  VECTOR *r_new) /* O: coordinates in the new system */
{
  double ca, sa;		/* cosine and sine of angle		*/

  /* Compute the sine and cosine of the angle.
   */
  ca = cos(angle); sa = sin(angle);
  /* Calculate the new coords using the following:
   * new x coord = cosine * old x coord - sine * old z coord
   * new y coord = old y coord
   * new z coord = cosine * old z coord + sine * old x coord
   */
  r_new->x = ca * r_old.x - sa * r_old.z;
  r_new->y = r_old.y;
  r_new->z = ca * r_old.z + sa * r_old.x;
}




/******************************************************************************
NAME:           xxx_rotatez(r_old, angle, r_new)

PURPOSE:        Transformation of coorindates through rotation around the
                Z-axis and specified angle.

DEVELOPMENT HISTORY:
$Log: xxx_rotate.c,v $
Revision 1.1  2000/11/22 21:26:17  storey
Original version - conversion from gxx_rotate.c



RETURN VALUE:
None

******************************************************************************/
void  xxx_rotatez(VECTOR r_old, /* I: coordinates (x, y, z) in the old
                                          system                             */
                  double angle, /* I: angle to be rotated, in radian, positive
                                  anticlockwise                              */
                  VECTOR *r_new) /* O: coordinates in the new system */
{
  double ca, sa;		/* cosine and sine of angle		*/

  /* Compute the sine and cosine of the angle.
   */
  ca = cos(angle);
  sa = sin(angle);

  /* Calculate the new coords using the following:
   * new x coord = cosine * old x coord + sine * old y coord
   * new y coord = cosine * old y coord - sine * old x coord
   * new z coord = old z coord
   */
  r_new->x = ca * r_old.x + sa * r_old.y;
  r_new->y = ca * r_old.y - sa * r_old.x;
  r_new->z = r_old.z;
}
