/******************************************************************************
PURPOSE:
to smooth data after the kalman filter

ROUTINES:
xxx_smooth_gain
xxx_smooth_state

DEVELOPMENT HISTORY:
$Log: xxx_smooth.c,v $
Revision 1.1  2000/11/22 21:26:18  storey
Original version - conversion from gxx_smooth.c


NOTES:

ALGORITHM REFERENCES:

******************************************************************************
		        Property of the U.S. Government
                            USGS EROS Data Center
******************************************************************************/
#include <stdio.h>
#include <math.h>
#include <malloc.h>
#include "xxx_LogStatus.h"
#include "xxx_math.h"

/******************************************************************************
NAME: SMOOTH_GAIN

PURPOSE:
    smooth the gain matrix

RETURN VALUE:
Type = int
Value    Description
-----    -----------
SUCCESS  Successful completion
ERROR    Operation failed

NOTES:
******************************************************************************/
int xxx_smooth_gain(
double *P,	/* I: Filtered covariance matrix */
double *Pn,	/* I: predicted error covariance matrix at time K+1 */
double *S,	/* I: state transition matrix */
double *A,	/* O: smoothing gain matrix */
int m		/* I: size in m direction */
)
{
   double *t1, *St, *inv;
   int status;
   
   t1  = malloc( m * m * sizeof(double) );
   if (t1 == NULL)
 	{
	xxx_LogStatus("GPSLIB", __FILE__, __LINE__, 
            "Error allocating memory");
        return(ERROR);
	}
   St  = malloc( m * m * sizeof(double) );
   if (St == NULL)
 	{
	xxx_LogStatus("GPSLIB", __FILE__, __LINE__, 
            "Error allocating memory");
	free(t1);
        return(ERROR);
	}
   inv = malloc( m * m * sizeof(double) );
   if (inv == NULL)
 	{
	xxx_LogStatus("GPSLIB", __FILE__, __LINE__, 
            "Error allocating memory");
	free(t1);
	free(St);
        return(ERROR);
	}

   xxx_matrix_transpose( S, St, m, m );
   status = xxx_matrix_inverse( Pn, inv, m );
   if (status != SUCCESS)
    {
    xxx_LogStatus("GPSLIB", __FILE__, __LINE__, 
        "Error returned from xxx_matrix_multiply");
	free(t1);
	free(St);
	free(inv);
    return(ERROR);
    }

   status = xxx_matrix_multiply( P, St, t1, m, m, m, m );
   if (status != SUCCESS)
    {
    xxx_LogStatus("GPSLIB", __FILE__, __LINE__, 
        "Error returned from xxx_matrix_multiply");
	free(t1);
	free(St);
	free(inv);
    return(ERROR);
    }

   status = xxx_matrix_multiply( t1, inv, A, m, m, m, m );
   if (status != SUCCESS)
    {
    xxx_LogStatus("GPSLIB", __FILE__, __LINE__, 
        "Error returned from xxx_matrix_multiply");
	free(t1);
	free(St);
	free(inv);
    return(ERROR);
    }

   free(t1);
   free(St);
   free(inv);
   return(SUCCESS);
}

/******************************************************************************
NAME: SMOOTH_STATE

PURPOSE:
    smooth the state matrix

RETURN VALUE:
Type = int
Value    Description
-----    -----------
SUCCESS  Successful completion
ERROR    Operation failed

NOTES:
******************************************************************************/
int xxx_smooth_state(
    double *X,	/* I: state matrix */
    double *Xk,	/* I: state matrix at time K */
    double *XN,	/* I: estimate of state [X] up to N */
    double *A,	/* I: smoothing gain matrix */
    double *XN1,/* O: predicted error covariance at time K+1*/
    int m	/* I: size in m direction */
)
{
   double *t1, *t2;
   int status;
   
   t1  = malloc( m * m * sizeof(double) );
   if (t1 == NULL)
 	{
	xxx_LogStatus("GPSLIB", __FILE__, __LINE__, 
            "Error allocating memory");
        return(ERROR);
	}
   t2  = malloc( m * m * sizeof(double) );
   if (t2 == NULL)
 	{
	xxx_LogStatus("GPSLIB", __FILE__, __LINE__, 
            "Error allocating memory");
	free(t1);
        return(ERROR);
	}
 
   xxx_matrix_subtract( XN, Xk, t1, m, 1 );
   status = xxx_matrix_multiply( A, t1, t2, m, m, m, 1 );
   if (status != SUCCESS)
    {
    xxx_LogStatus("GPSLIB", __FILE__, __LINE__, 
        "Error returned from xxx_matrix_multiply");
	free(t1);
	free(t2);
    return(ERROR);
    }

   xxx_matrix_add( X, t2, XN1, m, 1 );
 
   free(t1);
   free(t2);
   return(SUCCESS);
}
