/*****************************************************************************
NAME: 		xxx_std_dev

PURPOSE:
	Calculates the sample standard deviation.


DEVELOPMENT HISTORY:
$Log: xxx_std_dev.c,v $
Revision 1.1  2000/12/07 18:12:14  storey
Original version


RETURN VALUE:
Type = int
Value    Description
-----    -----------
SUCCESS  Standard deviation was calculated.
ERROR    Sample size was zero.


ALGORITHM REFERENCES:
******************************************************************************
                        Property of the U.S. Government
                           USGS EROS Data Center
*******************************************************************************/
#include <math.h>		/* sqrt prototype */
#include "xxx_LogStatus.h"	/* xxx_LogStatus prototype */
#include "xxx_Types.h"		/* ERROR/SUCCESS defines */
#include "xxx_math.h"		/* xxx_std_dev prototype */


int xxx_std_dev
(
    const double *sample,   /* I: pointer to the data */
    int nsamps,             /* I: number of samples */
    double mean,            /* I: mean of the samples */
    double *std_dev 	   	/* O: standard deviation of the data */
)
{
    int i;	       			/* loop variable */
    double sum_sqr=0.0;		/* sum of the squared difference from mean */

/*---------------------------------------------------------------------
    Standard deviation calculation.  If number of samples is invalid
    return ERROR.
---------------------------------------------------------------------*/
    if (nsamps <= 1)
    {
        xxx_LogStatus("XXXLIB", __FILE__, __LINE__, "Sample size is invalid.");
	*std_dev = 0.0;
        return(ERROR);
    }
    
    for (i = 0; i < nsamps; i++)
	sum_sqr += sample[i] * sample[i];  /* sum of squares */
	
    sum_sqr -= nsamps * (mean * mean);	/* subtract mean squared for each
                                           sample */
    *std_dev = sqrt(sum_sqr / (nsamps-1));

    return(SUCCESS);
}
