#ifndef _EO1GRID_H_
#define _EO1GRID_H_

#include <stdlib.h>
#include <stdio.h>
#include <math.h>

#include "xxx_Types.h"
#include "xxx_LogStatus.h"
#include "xxx_Imagery.h"
#include "xxx_ODL.h"
#include "xxx_const.h"
#include "xxx_structures.h"
#include "axx_grid.h"
#include "axx_alimodel.h"
#include "exx_cpf.h"
#include "xxx_proj.h"
#include "axx_hdf_io.h"
#include "xxx_math.h"
#include "exx_geometric.h"
#include "xxx_geometric.h"
#include "xxx_utility.h"
#include "xxx_systbl.h"
#include "lablib3.h"

#define 	WRS_SCENE_SIZE_LINES 	5965       /* Number of lines in WRS scene */
#define 	WRS_SCENE_SIZE_SAMPS 	6967       /* Number of samps in WRS scene */
#define		MINIMUM_ELEVATION	-20.0
#define		MAXIMUM_ELEVATION	+80.0

/* Defines for the get_parms.c routine.
 */
#define GRID_CELL_SIZE_LINE_MULTI 50
#define GRID_CELL_SIZE_SAMP_MULTI 40 
#define GRID_CELL_SIZE_LINE_PAN 100
#define GRID_CELL_SIZE_SAMP_PAN 80 
#define GB2 2147483648
  /* This is 2 GB defined as 1 GB = 2^30.  Thus 2 GB = 2^31. */

/* Degree of the rough mapping polynomial.
 */
#define ROUGH_DEGREE 1

/* Value for padding the corner points.
 */
#define PADVAL 10.0

/* Prototype definitions for the ETM+ grid routines.
 */
int angle_to_map (
    axx_GRID_TYPE *grid,    /* I/O: Grid structure to examine/modify    */
    int  units);            /* I: Integral units for GCTP               */

int calc_center_and_rotation_angle (
    int path,               /* I: WRS path number                       */
    double row,             /* I: WRS row number (may be fractional)    */
    CPF *cpf,               /* I: Calibration parameters                */
    axx_GRID_TYPE *grid);   /* I/O: Grid structure                      */

int calc_det_offsets (
  ALIMODEL *alimodel,      /* I: ALI model structure                        */
  CPF *cpf,                /* I: CPF constants                              */
  double poly_x[3][4],     /* I: Array of x polynomial coefficients
                                 (15m, 30m, 60m) */
  double poly_y[3][4],     /* I: Array of y polynomial coefficients
                                 (15m, 30m, 60m) */
  axx_GRID_TYPE *grid);    /* I/O: Grid structure                           */

int calc_map_coefs (
  axx_GRID_TYPE *grid);      /* I/O: Grid structure                      */

int calc_path_oriented_frame (
    axx_GRID_TYPE *grid,     /* I/O: Grid structure                      */
    int  units,              /* I: Projection units                      */
    double poly_x[3][4],     /* O: Sample-to-projection coefficients
                                   (15m, 30m, 60m)                       */
    double poly_y[3][4]);    /* O: Line-to-projection coefficients
                                   (15m, 30m, 60m)                       */

int calc_rough_map_coefs (
  axx_GRID_TYPE *grid);      /* I/O: Grid structure                      */

int det_grid_ls (
  int  num_0r_lines[NBANDS], /* I: Number of lines in the 0R image    */
  int  num_0r_samps[NBANDS], /* I: Number of samples in the 0R image  */
  axx_GRID_TYPE *grid);         /* I/O: Grid structure to be freed       */

int det_num_grid_ls (
  int  num_0r_lines[NBANDS], /* I: Number of lines in the 0R image    */
  int  num_0r_samps[NBANDS], /* I: Number of samples in the 0R image  */
  axx_GRID_TYPE *grid);         /* I/O: Grid structure to be freed       */

int det_opt_band (
  const BandNumber *bands,
                        /* I: Array of bands to search for the optimal band */
  int nbands,           /* I: Number of bands in the list                   */
  BandNumber *opt_band);/* O: Optimal band in the list of bands             */

int get_parms (
  char param_filename[STRLEN],  /* I: Parameter file name                   */
  char scriptname[STRLEN],      /* O: Script name                           */
  char file_name_0R[STRLEN],    /* O: ETM+ file name for the 0R image       */
  char file_name_1R[STRLEN],    /* O: ETM+ file name for the 1R image       */
  char cal_parm_file[STRLEN],   /* O: Calibration parameter filename.       */
  char model_file_name[STRLEN], /* O: ETM+ model file name                  */
  struct DBL_XY frame_coords[2],/* O: User-input frame coordinates to define
                                      the frame of the output area          */
  int  *coord_code,             /* O: Code for coordinate units             */
  struct DBL_LS *lscoords,      /* O: Image coordinates of the first coord
                                      pair entered in frame_coords when proj
                                      params are being specified directly   */
  int  *path,                   /* O: WRS path number of the image          */
  double *row,                  /* O: WRS (fractional) row number           */
  float *zincrement,		/* O: Number of elevation planes	    */
  float *zmax,			/* O: Maximum elevation			    */
  float *zmin,			/* O: Maximum elevation			    */
  axx_GRID_TYPE *grid);         /* O: Grid structure to be filled           */

int grid_ols2ils (
  const axx_GRID_BAND_TYPE *grid_band_ptr, /* I: pointer to grid band */
  int  *row_ptr,     		/* O: pointer to input space grid row */
  int  *col_ptr,     		/* O: pointer to input space grid column */
  long input_nl,
  long input_ns,
  double oline,      		/* I: output space line to map to input space */
  double osamp,      		/* I: output space sample to map to input space */
  double *iline_ptr, 		/* O: pointer to input space line */
  double *isamp_ptr, 		/* O: pointer to input space sample */
  int  iplane);      		/* I: elevation plane */

int make_grid (
  ALIMODEL *alimodel,      /* I: ALI model structure                        */
  CPF *cpf,                /* I: CPF constants                              */
  double poly_x[3][4],     /* I: Array of x polynomial coefficients (15m, 30m, 60m) */
  double poly_y[3][4],     /* I: Array of y polynomial coefficients (15m, 30m, 60m) */
  axx_GRID_TYPE *grid);    /* I/O: Grid structure                           */

int minbox (
  ALIMODEL *alimodel,      /* I: ALI model structure                   */
  BandNumber opt_band,     /* I: "Optimal" band to work on             */
  int  num_0r_lines,       /* I: Number of lines in the 0R image for the optimal band */
  int  num_0r_samps,       /* I: Number of samps in the 0R image for the optimal band */
  CPF *cpf,                /* I: CPF constants                         */
  axx_GRID_TYPE *grid);    /* I/O: Grid structure                      */

int maxbox (
  ALIMODEL *alimodel,        /* I: ETM+ model structure                  */
  int  *num_0r_lines,        /* I: Number of lines in the 0R image for each band */
  int  *num_0r_samps,        /* I: Number of samps in the 0R image for each band */
  CPF *cpf,                  /* I: CPF constants                         */
  axx_GRID_TYPE *grid);      /* I/O: Grid structure                      */

int north_up (
  axx_GRID_TYPE *grid,          /* I/O: Grid structure                      */
  BandNumber opt_band,          /* I: "Optimal" band to work on             */
  struct DBL_XY frame_coords[2],/* I: User-input frame coordinates to define
                                      the frame of the output area          */
  struct DBL_LS lscoords,       /* I: Image coordinates of the first coord
                                      pair entered in frame_coords when proj
                                      params are being specified directly   */
  int  coord_code);             /* I: Coordinate units code                 */

int pad_corners (
  double pixsize,          /* I: Pixel size for the coordinates             */
  double *pxmax,           /* I/O: Maximum X projection coordinate          */
  double *pxmin,           /* I/O: Minimum X projection coordinate          */
  double *pymax,           /* I/O: Maximum Y projection coordinate          */
  double *pymin);          /* I/O: Minimum Y projection coordinate          */

int path_oriented (
  axx_GRID_TYPE *grid,     /* I/O: Grid structure                           */
  CPF *cpf,                /* I: CPF constants                              */
  int path,                /* I: Path of the image                          */
  double row,              /* I: Fractional row of the image                */
  double poly_x[3][4],     /* O: X polynomial coefficients (15m, 30m, 60m)  */
  double poly_y[3][4]);    /* O: Y polynomial coefficients (15m, 30m, 60m)  */

int read_num_ls (
  char *HDF_name,  /* I: HDF file name of image to open                    */
  const BandNumber *band, /* I: Bands of image to open                     */
  int  nbands,     /* I: Number of bands to process                        */
  int  nsca,       /* I: Number of SCAs                                    */
  int  nlines[NBANDS], /* O: Number of lines in image in band-referenced order */
  int  nsamps[NBANDS]); /* O: Number of samples in image in band-reference order*/

int validate_utm_zone (
  int  nlines,         /* I: Number of lines in image                    */
  int  nsamps,         /* I: Number of samples in image                  */
  BandNumber band,     /* I: Band number for the current band            */
  int sca,             /* I: SCA number                                  */
  ALIMODEL *alimodel,  /* I: ALI model structure                         */
  CPF *cpf,            /* I: CPF constants                               */
  int  zone,           /* I: Zone value to validate                      */
  int  *computed_zone, /* O: Computed zone value for the scene           */
  int *valid);         /* O: Flag to specify if the UTM zone was valid   */

int path_maxmin_box (
  ALIMODEL *amodel,         /* I: ALI model structure */
  BandNumber opt_band,      /* I: Optimal band number, used with MINBOX */
  int *num_0r_lines, /* I: Number of lines in the 0R image for the optimal band */
  int *num_0r_samps, /* I: Number of samps in the 0R image for the optimal band */
  CPF *cpf,                 /* I: CPF to determine WRS lat/long and angle */
  axx_GRID_TYPE *grid,      /* I/O: Grid structure to use/modify */
  int path,                 /* I: WRS path of product to provide */
  double row,               /* I: WRS row of product (may be fractional) */
  double poly_x[2][4],      /* O: Coefficients which map output lines/samples */
  double poly_y[2][4]);     /*    samples to projection coordinates in the
                                  order of 10m, 30m) */

int calc_path_oriented_maxbox_frame (
  ALIMODEL *amodel,            /* I: ALI model structure                 */
  int  *num_0r_lines, /* I: Number of lines in the 0R image for each band */
  int  *num_0r_samps, /* I: Number of samps in the 0R image for each band */
  CPF *cpf,                  /* I: CPF constants                        */
  axx_GRID_TYPE *grid,       /* I/O: Grid structure                     */
  int  units,                /* I: Projection units                     */
  double poly_x[2][4],       /* O: Coefficients which map output lines  */
  double poly_y[2][4]);      /*    & samples to projection coordinates  */
                             /*    in the order of 10m,30m */


int calc_path_oriented_minbox_frame (
  ALIMODEL *amodel,          /* I: ALI model structure                  */
  BandNumber opt_band,       /* I: "Optimal" band to work on            */
  int  num_0r_lines, /* I: Number of lines in the 0R image for the optimal band */
  int  num_0r_samps, /* I: Number of samps in the 0R image for the optimal band */
  CPF *cpf,                  /* I: CPF constants                        */
  axx_GRID_TYPE *grid,       /* I/O: Grid structure                     */
  int  units,                /* I: Projection units                     */
  double poly_x[2][4],       /* O: Coefficients which map output lines  */
  double poly_y[2][4]);      /*    & samples to projection coordinates  */
                             /*    in the order of 15m,30m              */

#endif
