#include "aligrid.h"

/******************************************************************************
NAME:           calc_rough_map_coefs

PURPOSE:
Calculate the rough mapping coefficients for the grid.


RETURN VALUE:   Type = int
Value    Description
-----    -----------
SUCCESS  Successful completion
ERROR    Operation failed

NOTES:
This routine will find the rough mapping coefficients for the grid.  The child
routine, grid_cell_poly, will be called for each band.  The rough mapping
coefficients will be stored in the grid structure for the appropriate band.

ALGORITHM REFERENCES:
See axx_grid.h for a description of the grid structure.

******************************************************************************/

int calc_rough_map_coefs (
  axx_GRID_TYPE *grid)  /* I/O: Grid structure                              */
{
  int  ref,             /* corresponding reference position for the band numbers */
       sca,		/* SCA number					    */
       stat,            /* Return status                                    */
       i;               /* Loop variable                                    */
  char ErrMsg[STRLEN];  /* Error message                                    */

  /* Loop through the bands in the grid.
   */
  for (i = 0; i < grid->nbands; i++)
  {
    ref = grid->bands[i];
    for(sca = 0; sca<grid->nscas; sca++ )
    {

      /* Calculate the rough mapping coefficients for the current band.
       */
      stat = axx_grid_cell_poly (&grid->gridbands[sca][ref],grid->gridbands[sca][ref].nzplanes);
      if (stat == ERROR)
      {
        strcpy (ErrMsg, "Error in the grid_cell_poly routine.");
        xxx_LogStatus ("CALC_ROUGH_POLY", __FILE__, __LINE__, ErrMsg);
        return (ERROR);
      }
    }
  }

  return (SUCCESS);
}
