#include <stdio.h>
#include <unistd.h>
#include <sys/types.h>
#include <stddef.h>
#include "xxx_const.h"
#include "axx_grid.h"

/******************************************************************************
NAME:           det_grid_ls

PURPOSE:
Determine the grid lines and samples in the image.

RETURN VALUE:   Type = int
Value    Description
-----    -----------
SUCCESS  Successful completion
ERROR    Operation failed

NOTES:
This routine will determine the in_lines and in_samps for the grid, based on
the specified band numbers in the grid.  The grid_cell_size.line and
grid_cell_size.samp in the grid specify the size of the grid cells and are
used to calculate the positioning for the grid lines and samples.  The
num_0r_lines and num_0r_samps arrays must be in band-referenced order
(regardless if there is information for that DDR or not).

ALGORITHM REFERENCES:
See axx_grid.h for a description of the grid structure.

******************************************************************************/

int det_grid_ls (
  int  num_0r_lines[NBANDS], /* I: Number of lines in the 0R image    */
  int  num_0r_samps[NBANDS], /* I: Number of samples in the 0R image  */
  axx_GRID_TYPE *grid)          /* I/O: Grid structure to be freed       */
{
  int  i, j, k;              /* loop counter                             */
  int  count;                /* counter for the number of lines/samps    */
  int  reference;            /* corresponding reference position for the
                                band numbers                             */

  /* Loop through the bands in the grid.  Note line/sample are zero-relative
   */
  for (i = 0; i < grid->nbands; i++)
  {
     reference = grid->bands[i];

     /* Fill the grid lines.
      */
     for( k=0 ; k<NSCA ; k++ )
     {
        grid->gridbands[k][reference].in_lines[0] = 0;
        for (j = grid->gridbands[k][reference].grid_cell_size.line, count = 1;
             j < num_0r_lines[reference];
             j += grid->gridbands[k][reference].grid_cell_size.line)
        {
           grid->gridbands[k][reference].in_lines[count++] = j - 1;
        }
        grid->gridbands[k][reference].in_lines[count] = num_0r_lines[reference] - 1;

        /* Fill the grid samples.
         */
        grid->gridbands[k][reference].in_samps[0] = 0;
        for (j = grid->gridbands[k][reference].grid_cell_size.samp, count = 1;
             j < num_0r_samps[reference];
             j += grid->gridbands[k][reference].grid_cell_size.samp)
        {
           grid->gridbands[k][reference].in_samps[count++] = j - 1;
        }
        grid->gridbands[k][reference].in_samps[count] = num_0r_samps[reference] - 1;
     }
  }

  return (SUCCESS);
}
