#include "aligrid.h"

/******************************************************************************
NAME:           det_num_grid_ls

PURPOSE:
Determine the number of grid lines and samples in the image.

RETURN VALUE:   Type = int
Value    Description
-----    -----------
SUCCESS  Successful completion
ERROR    Operation failed

ALGORITHM REFERENCES:
See axx_grid.h for a description of the grid structure.

******************************************************************************/

int det_num_grid_ls (
  int  num_0r_lines[NBANDS], /* I: Number of lines in the 0R image    */
  int  num_0r_samps[NBANDS], /* I: Number of samples in the 0R image  */
  axx_GRID_TYPE *grid)          /* I/O: Grid structure to be freed       */
{
  int  i;                    /* loop counter                             */
  int  sca;                  /* Loop counter for SCAs                    */
  int  reference;            /* corresponding reference position for the band numbers */

  /* Loop through the bands in the grid and calculate the number of lines
   * and samples in the grid.
   */
  for (i = 0; i < grid->nbands; i++)
  {
    reference = grid->bands[i];
    for(sca = 0; sca < grid->nscas; sca++)
    {
      grid->gridbands[sca][reference].ngrid_lines = num_0r_lines[reference] /
                           grid->gridbands[sca][reference].grid_cell_size.line + 1;
      if( ( num_0r_lines[reference] % grid->gridbands[sca][reference].grid_cell_size.line ) != 0 )
         grid->gridbands[sca][reference].ngrid_lines++;

      grid->gridbands[sca][reference].ngrid_samps = num_0r_samps[reference] /
                           grid->gridbands[sca][reference].grid_cell_size.samp + 1;
      if( ( num_0r_samps[reference] % grid->gridbands[sca][reference].grid_cell_size.samp) != 0 )
         grid->gridbands[sca][reference].ngrid_samps++;
#ifdef DEBUG
      printf("%d %d %d %d\n",i,sca,grid->gridbands[sca][reference].ngrid_lines,
             grid->gridbands[sca][reference].ngrid_samps);
#endif
    }
  }

  return (SUCCESS);
}
