/*****************************************************************************
NAME: GET_DATE_TIME

PURPOSE:
Decode the Level 1R image start time from text format: YYYYDDDHHMMSS
or YYYYDDDHHMMSS.SSS to a double array containing year, day of year,
and seconds of day.

DEVELOPMENT HISTORY:
  1.0    J. Storey      11/00   Original development
$Log: get_date_time.c,v $
Revision 1.2  2000/12/15 20:58:33  storey
Modified to work with either image start time (to nearest second) or
data gate open time (to nearest millisecond).

Revision 1.1  2000/11/28 20:42:41  storey
Original Version

RETURN VALUE:
None

Value    Description
-----    -----------

NOTES: 

ALGORITHM REFERENCES:
******************************************************************************
                        Property of the U.S. Government
                            USGS EROS Data Center
*******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "alinit.h"

void get_date_time(
    char *text_date,	/* I: Text-formatted YYYYDDDHHMMSS	*/
    double *time)	/* O: Double array of year, doy, sod	*/
{
/* Vars */
char	cyear[5],	/* Year substring from text string */
	cday[4],	/* Day of year substring from text string */
	chour[3],	/* Hour substring from text string */
	cminute[3],	/* Minute substring from text string */
	csecond[3],	/* Second substring from text string */
        cmsec[4];	/* Millisecond substring from text string */

int	year,		/* Year field from text string */
	day,		/* Day of year field from text string */
	hour,		/* Hour field from text string */
	minute,		/* Minute field from text string */
	second,		/* Second field from text string */
	msec;		/* Millisecond field from text string */

/* Decompose the text string into substrings */
strncpy( cyear,   &text_date[0], 4 );
cyear[4] = '\0';
strncpy( cday,    &text_date[4], 3 );
cday[3] = '\0';
strncpy( chour,   &text_date[7], 2 );
chour[2] = '\0';
strncpy( cminute, &text_date[9], 2 );
cminute[2] = '\0';
strncpy( csecond, &text_date[11], 2 );
csecond[2] = '\0';

/* Test length of input string to see if we have any milliseconds */
if ( strlen( text_date ) >= 17 )
  strncpy( cmsec, &text_date[14], 3 );
else
  strncpy( cmsec, "000", 3 );
cmsec[3] = '\0';

/* Convert the substrings to integers */
year = atoi( cyear );
day = atoi( cday );
hour = atoi( chour );
minute = atoi( cminute );
second = atoi( csecond );
msec = atoi( cmsec );

/* Load the output double array */
time[0] = (double)year;
time[1] = (double)day;
time[2] = (double)(hour*3600 + minute*60 + second) + (double)msec/1000.0;

}
