#ifndef _GET_RESAMPLE_PARAMETERS_H_
#define _GET_RESAMPLE_PARAMETERS_H_

/*
**
** INCLUDE FILES 
**
*/

#include "xxx_const.h"    /* BandNumber definition */

/*
**
** TYPE DEFINITIONS
**
*/

typedef enum
{
   CC_RESAMPLING,
   MTF_RESAMPLING,
   NN_RESAMPLING,
   BI_RESAMPLING,
   NUMBER_RESAMPLING_TYPES
} RESAMPLE_TYPE;

typedef enum
{
    BYTE_OUTPUT,
    INT16_OUTPUT,
    INT32_OUTPUT,
    FLOAT_OUTPUT,
    NUMBER_OUTPUT_TYPES
} OUTPUT_DATA_TYPE;

typedef struct
{
    int start_line;
    int start_sample;
    int end_line;
    int end_sample;
    int num_lines;
    int num_samples;
} WINDOW_TYPE;
/* The WINDOW_TYPE structure holds the parameters that define an image
   window.  Both the end line/sample and the number of lines/samples are
   included to allow the user to determine which method of defining the
   window is more convenient. */

/*
** 
** FUNCTION PROTOTYPES
**
*/

int read_resample_parameters
(
    const char *ODL_filename   /* INPUT : ODL filename that contains resample 
                                          parameters */
);

char *get_input_image_filename(void);
const BandNumber *get_band_list(void);
char *get_output_image_filename(void);
char *get_grid_filename(void);
int do_terrain_correction(void);
char *get_DEM_filename(void);
OUTPUT_DATA_TYPE get_output_data_type(void);
int get_output_data_size(void); /* returns bytes for current OUTPUT_DATA_TYPE */
int do_output_window(void);
const WINDOW_TYPE *get_output_window
(
    BandNumber band /* INPUT : band number for the desired the output window */
);
void set_output_window
(
    BandNumber band, /* INPUT : band number for the desired output window */
    int start_line,  /* INPUT : starting line for the output window */
    int start_sample,/* INPUT : starting sample for the output window */
    int num_lines,   /* INPUT : number of lines in the output window */
    int num_samples  /* INPUT : number of samples in the output window */
);
void set_scaled_output_window
(
    BandNumber band,     /* INPUT : band number for the desired output window */
    float line_scaling,  /* INPUT : scaling factor for the window line size */
    float sample_scaling /* INPUT : scaling factor for the window sample size */
);

RESAMPLE_TYPE get_resampling_type(void);

float get_output_range_minimum(void);
float get_output_range_maximum(void);
float get_cubic_convolution_alpha(void);
float get_fill_pixel(void);
char *get_script_name(void);
char *get_1R_filename(void);
char *get_1G_filename(void);
char *get_grid_filename(void);
int do_terrain_correction(void);
char *get_DEM_filename(void);
int do_delay_correction(void);
float get_gain_threshold(void);
OUTPUT_DATA_TYPE get_output_data_type(void);
int get_output_data_size(void);
RESAMPLE_TYPE get_resampling_type(void);
char *get_0R_filename(void);
char *get_CPF_filename(void);

#endif /* _GET_RESAMPLE_PARAMETERS_H_ */
