#include "combine_sca.h"
#include "generate_ddr.h"

main (int argc, char *argv[])
{
  int band;			 /* band counter */
  int sca;			 /* SCAband counter */
  int nbands;			 /* Number of bands present */
  int bands[NBANDS];
  int image_lines[NSCA][NBANDS]; /* lines in each band of output image */
  int image_samps[NSCA][NBANDS]; /* samples in each band of output image */
  int output_image_lines[NBANDS]; /* lines in each band of output image */
  int output_image_samps[NBANDS]; /* samples in each band of output image */
  DataType dtype[NBANDS]; 	 /* data type of each band of output */
  char output_image_name[STRLEN];  /* Output image name */
  char input_image_name[STRLEN];   /* Intput image name */
  char scriptname[STRLEN];       /* Script name for ODL */
  char param_filename[STRLEN];   /* Parameter file name */
  char msg[STRLEN];   		 /* Error message */
  BandNumber bandno[NBANDS];	 /* Bands present in imagery */
  HDFIO *input_hdf_io_ptr;       /* input HDF I/O file id */
  HDFIO *output_hdf_io_ptr;      /* output HDF I/O file id */
  float fill;			 /* Fill value */

  /* Print the starting message.
   */
  xxx_LogStatus (PROGRAM_NAME, __FILE__, __LINE__, "Initiate ...");

  /* Check the arguments and get the parameter file name from the arguments.
   */
  if (argc != 2)
  {
    strcpy (msg, "Usage: combine_sca ODLfile");
    xxx_LogStatus (PROGRAM_NAME, __FILE__, __LINE__, msg);
    exit (EXIT_FAILURE);
  }
  else
    strcpy (param_filename, argv[1]);

  for( band=0 ; band<NBANDS ; band++ ) bandno[band] = berror;

  /* Get user parameters from ODL file */
  if (get_parms (param_filename, scriptname, input_image_name, output_image_name, 
                 bands, &nbands, &fill) != SUCCESS )
  {
    strcpy (msg, "Error reading the parameters from the ODL file.");
    xxx_LogStatus (PROGRAM_NAME, __FILE__, __LINE__, msg);
    exit (EXIT_FAILURE);
  }

  /* Find enumerator band type for input bands */
  for( band=0 ; band<nbands ; band++ )
  {
    bandno[band] = axx_parse_user_band(bands[band]);
    if( bandno[band] == berror )
    {
      sprintf(msg, "Error in band number");
      xxx_LogStatus(PROGRAM_NAME,__FILE__,__LINE__, msg);
      exit(EXIT_FAILURE);
    }
  }

  /* Open input image */
  input_hdf_io_ptr = axx_open_1g_image(input_image_name, READ, &nbands, NSCA, bandno, dtype, 
                                       image_lines, image_samps);
  if (input_hdf_io_ptr == NULL)
  {
    sprintf(msg, "Error input output image: %s", input_image_name);
    xxx_LogStatus(PROGRAM_NAME,__FILE__,__LINE__, msg);
    exit(EXIT_FAILURE);
  }

  for( band=0 ; band<nbands ; band++)
    for( sca=1 ; sca<NSCA ; sca++ )
    {
      if( image_lines[0][band]!=image_lines[sca][band] )
      {
        sprintf(msg, "SCAs do not have same number of lines");
        xxx_LogStatus(PROGRAM_NAME,__FILE__,__LINE__, msg);
        exit(EXIT_FAILURE);
      }
      if( image_samps[0][band]!=image_samps[sca][band] )
      {
        sprintf(msg, "SCAs do not have same number of samps");
        xxx_LogStatus(PROGRAM_NAME,__FILE__,__LINE__, msg);
        exit(EXIT_FAILURE);
      }
    }
  for( band=0 ; band<nbands ; band++)
  {
      output_image_lines[band] = image_lines[0][band];
      output_image_samps[band] = image_samps[0][band];
  }

  /* Open input image */
  output_hdf_io_ptr = (HDFIO *)xxx_open_1g_image(output_image_name, WRITE, &nbands, bandno, dtype, 
                                        output_image_lines, output_image_samps);
  if (output_hdf_io_ptr == NULL)
  {
    sprintf(msg, "Error opening output image: %s", output_image_name);
    xxx_LogStatus(PROGRAM_NAME,__FILE__,__LINE__, msg);
    exit(EXIT_FAILURE);
  }

  /* Combine SCAs */
  if( combine(input_hdf_io_ptr, output_hdf_io_ptr, dtype[0], image_lines, image_samps, 
              nbands, bandno, fill) != SUCCESS )
  {
    sprintf(msg, "Error combining SCAs\n");
    xxx_LogStatus(PROGRAM_NAME,__FILE__,__LINE__, msg);
    exit(EXIT_FAILURE);
  }

  /* Create new DDR for output imagery */
  if( generate_ddr(input_image_name, output_image_name, nbands, bandno) != SUCCESS )
  {
    sprintf(msg, "Error writing combined SCA DDR\n");
    xxx_LogStatus(PROGRAM_NAME,__FILE__,__LINE__, msg);
    exit(EXIT_FAILURE);
  }

   /* generate an ENVI header file for each band (UTM only)
      continue processing on an error creating header files.
      -------------------------------------------------------*/
  if (write_envi_hdr(output_image_name) == ERROR)
  {
     strcpy(msg,"Error generating ENVI header files");
     xxx_LogStatus(PROGRAM_NAME,__FILE__,__LINE__, msg);
  }

  /* close the output 1g image */
  if (axx_close_1g_image(input_hdf_io_ptr) == ERROR)
  {
    strcpy(msg,"Error closing output image");
    xxx_LogStatus(PROGRAM_NAME,__FILE__,__LINE__, msg);
    exit(EXIT_FAILURE);
  }

  /* close the input 1g image */
  if (xxx_close_1g_image(output_hdf_io_ptr) == ERROR)
  {
    strcpy(msg,"Error closing input image");
    xxx_LogStatus(PROGRAM_NAME,__FILE__,__LINE__, msg);
    exit(EXIT_FAILURE);
  }

  /* Print the completion message.  */
  xxx_LogStatus (PROGRAM_NAME, __FILE__, __LINE__, "Successful completion.");

  return (EXIT_SUCCESS);
}
