/******************************************************************************
NAME:           generate_ddr

PURPOSE:

ROUTINES:
    generate_ddr

NOTES:
none

ALGORITHM REFERENCES:
none

******************************************************************************/
#include "combine_sca.h"

int generate_ddr
(
char *input_image_filename,
char *output_image_filename,
int nbands,
BandNumber *bandno
)
{
    xxx_DDRControl_TYPE *source_DDR_control; /* source image DDR control structure */
    xxx_DDRControl_TYPE target_DDR_control; /* target image DDR control structure */
    char error_msg[STRLEN]; /* error message for xxx libraries */
    char target_DDR_name[STRLEN]; /* target image DDR name */
    char source_DDR_name[STRLEN]; /* source image DDR name */
    int count;              /* loop counter */
    int index;              /* temporary index */
    xxx_DDR_TYPE *target_DDR_ptr;   /* pointer to current band's DDR */
    xxx_DDR_TYPE *source_DDR_ptr;   /* pointer to current band's DDR */
    int band;

    /* create the source image DDR name */
    xxx_create_DDR_name(source_DDR_name, input_image_filename);
   
    /* create the target image DDR name */
    xxx_create_DDR_name(target_DDR_name, output_image_filename);

    /* read the source image DDR */
    source_DDR_control = xxx_ReadDDR(source_DDR_name, error_msg);
    if (source_DDR_control == NULL)
    {
        /* error reading required item, log an error */
        xxx_LogStatus(PROGRAM_NAME,__FILE__,__LINE__, error_msg);
        return (ERROR);
    }

    /* copy the source DDR control structure to the target structure */
    memcpy(&target_DDR_control, source_DDR_control, sizeof(target_DDR_control));

    /* allocate memory for the DDR information of the bands generated */
    target_DDR_control.p_DDR = malloc(sizeof(xxx_DDR_TYPE) * nbands);
    if (target_DDR_control.p_DDR == NULL)
    {
        /* error in the memory allocation */
	xxx_LogStatus(PROGRAM_NAME,__FILE__, __LINE__, "Error allocating memory for 1g DDR.");
	/* free the source DDR before returning */
	XXX_FREE_DDR(source_DDR_control);
	return (ERROR);
    }
    target_DDR_control.Count = nbands;

    /* now that the source DDR information has been copied, customize the 
       data for the output image */
    for (count = 0; count < nbands; count++)
    {
        band = axx_get_user_band(bandno[count]) - 1;
        target_DDR_ptr = &target_DDR_control.p_DDR[count];

        target_DDR_ptr->BandNumber = source_DDR_control->p_DDR[band].BandNumber;

        target_DDR_ptr->NumberScansInScene = source_DDR_control->p_DDR[band].NumberScansInScene;
        target_DDR_ptr->NumberdetectorsPerScan = source_DDR_control->p_DDR[band].NumberdetectorsPerScan;
        target_DDR_ptr->NumberLinesInScene = source_DDR_control->p_DDR[band].NumberLinesInScene;
        target_DDR_ptr->NumberSamplesInScene = source_DDR_control->p_DDR[band].NumberSamplesInScene;

        target_DDR_ptr->DataType = source_DDR_control->p_DDR[band].DataType;

        target_DDR_ptr->MasterLine = source_DDR_control->p_DDR[band].MasterLine;
        target_DDR_ptr->MasterSample = source_DDR_control->p_DDR[band].MasterSample;

        /* set the dynamic range of the output pixels */
        target_DDR_ptr->MaxPixelValue = source_DDR_control->p_DDR[band].MaxPixelValue;
        target_DDR_ptr->MinPixelValue = source_DDR_control->p_DDR[band].MinPixelValue;
        target_DDR_ptr->MaxRadianceValue = source_DDR_control->p_DDR[band].MaxRadianceValue;
        target_DDR_ptr->MinRadianceValue = source_DDR_control->p_DDR[band].MinRadianceValue;

        /* copy other information from the grid */
        target_DDR_ptr->ProjectCode = source_DDR_control->p_DDR[band].ProjectCode;
        target_DDR_ptr->ZoneCode = source_DDR_control->p_DDR[band].ZoneCode;
        strcpy((char*)target_DDR_ptr->DatumCode, (char*)source_DDR_control->p_DDR[band].DatumCode);
        target_DDR_ptr->SpheroidCode = source_DDR_control->p_DDR[band].SpheroidCode;
        strcpy((char*)target_DDR_ptr->ProjectionUnits, 
               (char*)source_DDR_control->p_DDR[band].ProjectionUnits);
        for (index = 0; index < XXX_PROJECTION_COEF_SIZE; index++)
            target_DDR_ptr->ProjectionCoef[index] = source_DDR_control->p_DDR[band].ProjectionCoef[index];
        target_DDR_ptr->UpperLeft[1] = source_DDR_control->p_DDR[band].UpperLeft[1];
        target_DDR_ptr->UpperLeft[0] = source_DDR_control->p_DDR[band].UpperLeft[0];
        target_DDR_ptr->LowerLeft[1] = source_DDR_control->p_DDR[band].LowerLeft[1];
        target_DDR_ptr->LowerLeft[0] = source_DDR_control->p_DDR[band].LowerLeft[0];
        target_DDR_ptr->UpperRight[1] = source_DDR_control->p_DDR[band].UpperRight[1];
        target_DDR_ptr->UpperRight[0] = source_DDR_control->p_DDR[band].UpperRight[0];
        target_DDR_ptr->LowerRight[1] = source_DDR_control->p_DDR[band].LowerRight[1];
        target_DDR_ptr->LowerRight[0] = source_DDR_control->p_DDR[band].LowerRight[0];
        target_DDR_ptr->ProjectionDistY = source_DDR_control->p_DDR[band].ProjectionDistY;
        target_DDR_ptr->ProjectionDistX = source_DDR_control->p_DDR[band].ProjectionDistX;

        target_DDR_ptr->LineSubsampleInc = source_DDR_control->p_DDR[band].LineSubsampleInc;
        target_DDR_ptr->SampleSubsampleInc = source_DDR_control->p_DDR[band].SampleSubsampleInc;

        strcpy((char*)target_DDR_ptr->SpaceCraftSource, 
               (char*)source_DDR_control->p_DDR[band].SpaceCraftSource);
        strcpy((char*)target_DDR_ptr->InstrumentSource, 
               (char*)source_DDR_control->p_DDR[band].InstrumentSource);

        target_DDR_ptr->WRSPath = source_DDR_control->p_DDR[band].WRSPath;
        target_DDR_ptr->WRSRows = source_DDR_control->p_DDR[band].WRSRows;

        strcpy((char*)target_DDR_ptr->CaptureDirection, 
               (char*)source_DDR_control->p_DDR[band].CaptureDirection);
        strcpy((char*)target_DDR_ptr->CaptureDate, (char*)source_DDR_control->p_DDR[band].CaptureDate);
        strcpy((char*)target_DDR_ptr->CaptureTime, (char*)source_DDR_control->p_DDR[band].CaptureTime);

        for (index = 0; index < XXX_VALID_FLAGS; index++)
            target_DDR_ptr->FieldValidityFlags[index] = 
                   source_DDR_control->p_DDR[band].FieldValidityFlags[index];

        target_DDR_ptr->CorrectionType = source_DDR_control->p_DDR[band].CorrectionType;
    }

    /* write the target DDR to the 1g image */
    if (xxx_WriteDDR(target_DDR_name, &target_DDR_control, error_msg) == ERROR)
    {
        xxx_LogStatus(PROGRAM_NAME,__FILE__, __LINE__, error_msg);
	free(target_DDR_control.p_DDR);
	free(source_DDR_control->p_DDR);
	return (ERROR);
    }
    
    /* free the memory - note that the target DDR does not 
       use XXX_FREE_DDR since it is allocated on the stack */
    free(target_DDR_control.p_DDR);
    free(source_DDR_control->p_DDR);

    /* return that the DDR was successfully written */
    return (SUCCESS);
}
