#include "combine_sca.h"

/******************************************************************************
NAME:           get_parms

PURPOSE:
Get the grid parameters from the input parameter file and the system table.

RETURN VALUE:   Type = int
Value    Description
-----    -----------
SUCCESS  Successful completion
ERROR    Operation failed

******************************************************************************/

int get_parms (
  char param_filename[STRLEN],  /* I: Parameter file name                   */
  char scriptname[STRLEN], 	/* O: Script name                           */
  char input_image[STRLEN],     /* O: ALI file name for the 0R image        */
  char output_image[STRLEN],    /* O: ALI file name for the 1R image        */
  int *bands,
  int *nbands,
  float *fill)
{
  OBJDESC *in_parms;   /* Pointer to the structure containing all the ODL parameters. */
  char ErrorMessage[STRLEN];
  int i,               /* Loop variable                                     */
      count,           /* Number of values returned from GetODLField        */
      sca;             /* SCA counter                                       */

  /* Open the ODL parameter file.  (The openODL call should NOT determine
   * the input ODL filename.)
   */
  in_parms = xxx_OpenODL (param_filename, (xxx_EDCOdl_TYPE)xxx_NoEDCMeta, ErrorMessage);
  if (in_parms == NULL)
  {
    xxx_LogStatus (PROGRAM_NAME, __FILE__, __LINE__, ErrorMessage);
    return (ERROR);
  }

  /* Get the scriptname.
   */
  if (xxx_GetODLField (scriptname, STRLEN * sizeof (char), XXX_String, in_parms, NULL, 
                       "SCRIPT_NAME", ErrorMessage, &count) != SUCCESS)
  {
    xxx_LogStatus (PROGRAM_NAME, __FILE__, __LINE__, ErrorMessage);
    strcpy (ErrorMessage, "Error in the scriptname in the parameter file.");
    xxx_LogStatus (PROGRAM_NAME, __FILE__, __LINE__, ErrorMessage);
    xxx_CloseODL (in_parms, ErrorMessage);
    return (ERROR);
  }

  /* Get the input filename 
   */
  if (xxx_GetODLField (input_image, STRLEN * sizeof (char), XXX_String, in_parms, NULL, 
                       "INPUT_IMAGE", ErrorMessage, &count) != SUCCESS)
  {
    xxx_LogStatus (PROGRAM_NAME, __FILE__, __LINE__, ErrorMessage);
    strcpy (ErrorMessage, "Error in the input filename in the parameter file.");
    xxx_LogStatus (PROGRAM_NAME, __FILE__, __LINE__, ErrorMessage);
    xxx_CloseODL (in_parms, ErrorMessage);
    return (ERROR);
  }

  /* Get the output filename
   */
  if (xxx_GetODLField (output_image, STRLEN * sizeof (char), XXX_String, in_parms, NULL,
                       "OUTPUT_IMAGE", ErrorMessage, &count) != SUCCESS)
  {
    xxx_LogStatus (PROGRAM_NAME, __FILE__, __LINE__, ErrorMessage);
    strcpy (ErrorMessage, "Error in the output filename in the parameter file.");
    xxx_LogStatus (PROGRAM_NAME, __FILE__, __LINE__, ErrorMessage);
    xxx_CloseODL (in_parms, ErrorMessage);
    return (ERROR);
  }

  /* Get the band numbers to process.  
   */
  if (xxx_GetODLField (bands, NBANDS * sizeof (int), XXX_Int, in_parms, scriptname,
                      "BAND_LIST", ErrorMessage, nbands) != SUCCESS)
  {
    xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrorMessage);
    strcpy (ErrorMessage, "Error in the band numbers in the parameter file.");
    xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrorMessage);
    xxx_CloseODL (in_parms, ErrorMessage);
    return (ERROR);
  }

  /* Get the output filename
   */
  if (xxx_GetODLField (fill, sizeof (float), XXX_Float, in_parms, NULL,
                       "FILL", ErrorMessage, &count) != SUCCESS)
  {
    xxx_LogStatus (PROGRAM_NAME, __FILE__, __LINE__, ErrorMessage);
    strcpy (ErrorMessage, "Error in the reading fill value.");
    xxx_LogStatus (PROGRAM_NAME, __FILE__, __LINE__, ErrorMessage);
    xxx_CloseODL (in_parms, ErrorMessage);
    return (ERROR);
  }

  return (SUCCESS);
}
