#include <stdlib.h>        /* exit prototype */
#include <stdio.h>         /* sprintf prototype */
#include "xxx_Types.h"     /* SUCCESS/ERROR definitions */
#include "xxx_LogStatus.h" /* xxx_LogStatus prototype */
#include "xxx_const.h"     /* ALI constants */
#include "axx_grid.h"
#include "exx_cpf.h"

/******************************************************************************
NAME:           get_band_min_max_radiance

PURPOSE:
This routine returns the minimum and maximum radiance values for the 
indicated band.

RETURN VALUE:
Type = int
Value           Description
-----           -----------
SUCCESS         Radiance values successfully returned
ERROR           Radiance values not returned

NOTES:
none

******************************************************************************/

int get_band_min_max_radiance
(
    BandNumber band,         /* I: enumerated band number */
    float *min_radiance_ptr, /* O: pointer to minimum radiance */
    float *max_radiance_ptr  /* O: pointer to maximum radiance */
)
{
    float Lmaxout; /* maximum radiance of output pixel */
    float Lminout; /* minimum radiance of output pixel */
    CPF *cpf_ptr = (CPF *)get_cpf();  /* pointer to CPF data which holds the Lminout 
                   and Lmaxout for all the bands for both low and high gain */
                      
    /* determine the index for the current band into the gain scaling part
       of the CPF.  This is necessary since band 6 isn't split for the CPF */
    switch (band)
    {
        case pan:
           Lminout = cpf_ptr->Scaling_Parameters.PAN_Lmin_Lmax[0];
           Lmaxout = cpf_ptr->Scaling_Parameters.PAN_Lmin_Lmax[1];
           break;
        case b1p:
           Lminout = cpf_ptr->Scaling_Parameters.MS1P_Lmin_Lmax[0];
           Lmaxout = cpf_ptr->Scaling_Parameters.MS1P_Lmin_Lmax[1];
           break;
        case b1:
           Lminout = cpf_ptr->Scaling_Parameters.MS1_Lmin_Lmax[0];
           Lmaxout = cpf_ptr->Scaling_Parameters.MS1_Lmin_Lmax[1];
           break;
        case b2:
           Lminout = cpf_ptr->Scaling_Parameters.MS2_Lmin_Lmax[0];
           Lmaxout = cpf_ptr->Scaling_Parameters.MS2_Lmin_Lmax[1];
           break;
        case b3:
           Lminout = cpf_ptr->Scaling_Parameters.MS3_Lmin_Lmax[0];
           Lmaxout = cpf_ptr->Scaling_Parameters.MS3_Lmin_Lmax[1];
           break;
        case b4:
           Lminout = cpf_ptr->Scaling_Parameters.MS4_Lmin_Lmax[0];
           Lmaxout = cpf_ptr->Scaling_Parameters.MS4_Lmin_Lmax[1];
           break;
        case b4p:
           Lminout = cpf_ptr->Scaling_Parameters.MS4P_Lmin_Lmax[0];
           Lmaxout = cpf_ptr->Scaling_Parameters.MS4P_Lmin_Lmax[1];
           break;
        case b5p:
           Lminout = cpf_ptr->Scaling_Parameters.MS5P_Lmin_Lmax[0];
           Lmaxout = cpf_ptr->Scaling_Parameters.MS5P_Lmin_Lmax[1];
           break;
        case b5:
           Lminout = cpf_ptr->Scaling_Parameters.MS5_Lmin_Lmax[0];
           Lmaxout = cpf_ptr->Scaling_Parameters.MS5_Lmin_Lmax[1];
           break;
        case b7:
           Lminout = cpf_ptr->Scaling_Parameters.MS7_Lmin_Lmax[0];
           Lmaxout = cpf_ptr->Scaling_Parameters.MS7_Lmin_Lmax[1];
           break;
        default:
           xxx_LogStatus("GET_BAND_MIN_MAX_RADIANCE",__FILE__,__LINE__, "Illegal band value");
           return (ERROR);
           break;
    }    

    /* verify the Lminout and Lmaxout are legal */
    if (Lmaxout <= Lminout)
    {
        xxx_LogStatus("GET_BAND_MIN_MAX_RADIANCE",__FILE__,__LINE__,"Illegal radiance values in CPF");
        return (ERROR);
    }

    *min_radiance_ptr = Lminout;
    *max_radiance_ptr = Lmaxout;

    return (SUCCESS);
} 
