/****************************************************************************
NAME:		dump_alimodel

PURPOSE:  Dump a binary HDF ALI model file to ASCII output.

DEVELOPMENT HISTORY:
$Log: dump_alimodel.c,v $
Revision 1.1  2001/01/03 18:01:54  storey
Original version



RETURN VALUE:
Type=int
Value		Description
-----		-----------
EXIT_SUCCESS	dump_alimodel completed successfully
EXIT_FAILURE	Error occurred while running dump_alimodel

*******************************************************************************
                        Property of the U.S. Government
                             USGS EROS Data Center
*****************************************************************************/
#include "dump_alimodel.h"

main(int argc, char *argv[])
{
char 		amodel_filename[STRLEN];	/* ALI model file name */
ALIMODEL	amodel;				/* ALI model structure */

/*
  Set messages to standard output
*/
setvbuf(stdout, NULL, _IOLBF, 0);
/*
  Make sure an input file exists
*/
if (argc != 2)
{
    xxx_LogStatus("DUMP_ALIMODEL", __FILE__, __LINE__,
	    	  "Usage: dump_alimodel modelfile");
    exit(EXIT_FAILURE);
}

/*
  Initial message
*/
xxx_LogStatus("DUMP_ALIMODEL",__FILE__, __LINE__, "Initiated");

/*
  Get the input parameters
*/
strcpy( amodel_filename, argv[1] );

/*
  Read the model file
*/
if ( axx_get_alimodel ( amodel_filename, &amodel ) != SUCCESS )
{
    xxx_LogStatus("DUMP_ALIMODEL",__FILE__, __LINE__,
                  "Error reading ALI model file");
    exit(EXIT_FAILURE);
}

/*
  Dump the model file
*/
(void)print_amodel( amodel );

/*
  Write completion message
*/
xxx_LogStatus("DUMP_ALIMODEL",__FILE__, __LINE__, "Successful completion");

return(EXIT_SUCCESS);
}
