/****************************************************************************
NAME:		dump_hypmodel

PURPOSE:  Dump a binary HDF Hyperion model file to ASCII output.

DEVELOPMENT HISTORY:

RETURN VALUE:
Type=int
Value		Description
-----		-----------
EXIT_SUCCESS	dump_hypmodel completed successfully
EXIT_FAILURE	Error occurred while running dump_hypmodel

*******************************************************************************
                        Property of the U.S. Government
                             USGS EROS Data Center
*****************************************************************************/
#include "dump_hypmodel.h"

main(int argc, char *argv[])
{
char 		hmodel_filename[STRLEN];	/* Hyperion model file name */
HYPMODEL	hmodel;				/* Hyperion model structure */

/*
  Set messages to standard output
*/
setvbuf(stdout, NULL, _IOLBF, 0);
/*
  Make sure an input file exists
*/
if (argc != 2)
{
    xxx_LogStatus("DUMP_HYPMODEL", __FILE__, __LINE__, "Usage: dump_hypmodel modelfile");
    exit(EXIT_FAILURE);
}

/*
  Initial message
*/
xxx_LogStatus("DUMP_HYPMODEL",__FILE__, __LINE__, "Initiated");

/*
  Get the input parameters
*/
strcpy( hmodel_filename, argv[1] );

/*
  Read the model file
*/
if ( hxx_get_hypmodel ( hmodel_filename, &hmodel ) != SUCCESS )
{
    xxx_LogStatus("DUMP_HYPMODEL",__FILE__, __LINE__, "Error reading Hyperion model file");
    exit(EXIT_FAILURE);
}

/*
  Dump the model file
*/
(void)print_hmodel( hmodel );

/*
  Write completion message
*/
xxx_LogStatus("DUMP_HYPMODEL",__FILE__, __LINE__, "Successful completion");

return(EXIT_SUCCESS);
}
