#include "foot_print.h"	

#define		COVER_POS		0

int get_pan_samples(
   CPF *cpf,
   int sca,
   int band)
{
   return(cpf->Focal_Plane_Parameters.SCA_Parameters[sca].Number_Detectors[band]);
}

int axx_collects_pcd(
    ALIPCD *pcd,		/* I: Payload Correction Data from the satellite */
    EphemType ephem_type,       /* I: ephem type to use GPSPTS or ACSPTS */
    CPF **cpf)
{
   int 	ii, jj, kk, qq;     	/* Loop variables                            */
   double  dtime;		/* time offset				     */
   double  gpstime;		/* GPS reference time                        */
   double  timecode;		/* Time code corresponding to gpstime        */
   double  timecode_rate;	/* Spacecraft clock rate (seconds per second)*/
   int hour, minute;		/* Start time hour and minute                */
   double second;		/* Start time seconds			     */
   char    Message[STRLEN];	/* Output message text		    	     */
   FILE	*fptr;			/* Output DCE corner point file pointer      */

   struct EPHEM ephem[NPCD];    /* ephemeris data from the PCD */
   double xp;                   /* x shift pole wander in arc sec. */
   double yp;                   /* y shift pole wander in arc sec. */
   double ut1_utc;              /* UT1 - UTC time difference in seconds */
   double mjd;                  /* modified Julian date */
   int num_valid;

   int sca;
   BandNumber bandno = pan;
   int band;
   double lat;
   double lon;
   double ul_latlon[2],
          ur_latlon[2],
          ll_latlon[2],
          lr_latlon[2];
   double max_lat;
   double max_lon;
   double min_lat;
   double min_lon;
   int number_of_lines;
   int number_of_samps;
   DetType dettype = NOMINAL;
   double elev = 0.0;
   double in_lines[2];
   double in_samps[2];

   ALIMODEL  amodel;               /* ALI model structure                    */

   axx_zero_amodel(&amodel);

   amodel.ephem_type = ephem_type;
   amodel.correction_type = SYSTEMATIC;

   band = axx_get_user_band(bandno);

   fptr = fopen( "alicorners.txt", "w" );

   pcd->active_DCE = 0;
   for (ii=0; ii<pcd->num_DCE; ii++)
   /* for (ii=1; ii<pcd->num_DCE; ii++) */
   {
      pcd->active_DCE = ii;
   
      sprintf(Message,"Selected DCE %d",pcd->active_DCE);
      xxx_LogStatus("AXXLIB", __FILE__, __LINE__,Message);
      
      /* Convert the actual DCE time to year, doy, sod */
      exx_gpstoyds( pcd->DCE_List[pcd->active_DCE].start_time, amodel.dce_time );
   
      sprintf(Message,"DCE Duration:  %lf",
              pcd->DCE_List[pcd->active_DCE].stop_time-pcd->DCE_List[pcd->active_DCE].start_time);
      xxx_LogStatus("AXXLIB", __FILE__, __LINE__,Message);

      /*******************************************************************
        Get the frame rate and integration time parameters from the ALI
        housekeeping data. First pick a representative telemetry record
        index in the middle of the DCE (these parameters shouldn't change
        within a DCE).
      ********************************************************************/
      jj = (pcd->DCE_List[pcd->active_DCE].ali_start_index + 
            pcd->DCE_List[pcd->active_DCE].ali_stop_index) / 2;
      
      sprintf(Message,"Cover position %f", pcd->ALI_Group->cover_pos.value.d[jj]);
      xxx_LogStatus("AXXLIB", __FILE__, __LINE__,Message);
      amodel.ms_frame_rate = pcd->ALI_Group->frame_rate.value.d[jj];
      amodel.pan_frame_rate = 3 * amodel.ms_frame_rate;
      amodel.ms_int_time = pcd->ALI_Group->ms_int_time.value.d[jj];
      amodel.pan_int_time = pcd->ALI_Group->pan_int_time.value.d[jj];

      number_of_lines = 
         3.0 * (pcd->DCE_List[pcd->active_DCE].stop_time-pcd->DCE_List[pcd->active_DCE].start_time) *
         pcd->ALI_Group->frame_rate.value.d[jj];
      number_of_samps = get_pan_samples(*cpf,0,band-1) - 1;
      
      in_lines[0] = 1.0;
      in_lines[1] = number_of_lines;
      in_samps[0] = 1.0;
      in_samps[1] = number_of_samps;

      /* in_samps[1] = cpf->Focal_Plane_Parameters.SCA_Parameters[0].Number_Detectors[band-1]; */
      sprintf(Message,"Number of PAN band lines %d samples %d",number_of_lines,number_of_samps);
      xxx_LogStatus("AXXLIB", __FILE__, __LINE__,Message);

      /* find the ACS time */
      /* The ACS time reference is in the GPS telemetry. It gives a sequence
         of GPS times (time relative to the GPS epoch) that are time stamped
         with the spacecraft time codes. This provides a basis for building
         the relationship between the spacecraft clock (time codes) and
         GPS time. What we're going to do here is figure out what this
         (presumably linear) relationship is for this DCE and initialize
         the exx_init_timecode routine so that we can subsequently use the
         exx_timecodetogps and exx_gpstotimecode routines to convert back
         and forth (e.g., when mucking with the ephemeris and attitude data). */
      kk = pcd->DCE_List[pcd->active_DCE].gps_start_index;
      qq = pcd->DCE_List[pcd->active_DCE].gps_stop_index;
      if ( qq <= kk )
      {
         qq = kk + 1;
         if ( qq >= pcd->GPS_Group->acsutctime.count )
         {
            kk--;
            qq--;
         }
      }
      gpstime = pcd->GPS_Group->acsutctime.value.d[kk];
      timecode = pcd->GPS_Group->acsutctime.time[kk];
      
      /* IMPROVEMENT_OPPORTUNITY
         We'll do the rate computation from the two end points for now.
         Could also do a regression with all the points. */
      
      timecode_rate = (pcd->GPS_Group->acsutctime.value.d[qq] - gpstime)
                    / (pcd->GPS_Group->acsutctime.time[qq] - timecode);
      
      exx_init_timecode( gpstime, timecode, timecode_rate );
      
      /* Now we'll use this timing information to find the start time for the
         quaternion data */
      
      timecode = pcd->DCE_List[pcd->active_DCE].acs_start_tc;
      kk = xxx_find_index( timecode, pcd->ACS_Group->quat1 );
      timecode = pcd->ACS_Group->quat1.time[kk];
      gpstime = exx_timecodetogps( timecode );
      
      /* Finally, we'll load the acs_time array */
      exx_gpstoyds( gpstime, amodel.acs_time );

      /* Make sure our data is in the effective time period for the CPF */
      if (exx_check_date(amodel.dce_time, *cpf) != SUCCESS)
      {
         xxx_LogStatus ("AXXLIB", __FILE__, __LINE__,
                        "Error returned from exx_check_date.");
         return(ERROR);
      }

      /* Select the valid ephemeris points. */
      if (exx_valid_ephem (pcd, amodel.ephem_type, *cpf, &amodel.nephem, ephem) == ERROR)
      {
         xxx_LogStatus ("AXXLIB", __FILE__, __LINE__,
                        "Error obtaining valid ephemeris points exx_valid_ephem");
         /* return (ERROR); */
      }
      else
      {

         /* Get x and y shift pole wander and UT1-UTC time difference */
         if (exx_getmjdcoords (ephem->time, &mjd, &xp, &yp, &ut1_utc, *cpf) == ERROR)
         {
            xxx_LogStatus ("AXXLIB", __FILE__, __LINE__,
                           "Error returned from exx_getmjdcoords");
            return (ERROR);
         }
   
         /* Generate updated model information. */
         if (axx_process_ali_model (ephem, xp, yp, ut1_utc, *cpf, pcd, &amodel) == ERROR)
         {
            xxx_LogStatus ("AXXLIB", __FILE__, __LINE__, "Error updating the ALI model");
            return (ERROR);
         }
         for( qq=0 ; qq<2 ; qq++ )
         {
            for( kk=0 ; kk<2 ; kk++ )
            {
               if( axx_forward_ali (in_lines[qq], in_samps[kk], bandno, 0, elev, *cpf, &amodel, 
                                    dettype, &lat, &lon) == ERROR )
               {
                  xxx_LogStatus ("AXXLIB", __FILE__, __LINE__, "Error returned from ALI model");
                  xxx_LogStatus ("AXXLIB", __FILE__, __LINE__, "Possible extraterrestrial observation");
                  lat = 0.0;
                  lon = 0.0;
               }
               if( qq==0 && kk==0 )
               {
                  max_lat = lat;
                  max_lon = lon;
                  min_lat = lat;
                  min_lon = lon;
                  ur_latlon[0] = lat*R2D;
                  ur_latlon[1] = lon*R2D;
               }
               else
               {
                  if( lat > max_lat ) max_lat = lat;
                  if( lat < min_lat ) min_lat = lat;
                  if( lon > max_lon ) max_lon = lon;
                  if( lon < min_lon ) min_lon = lon;
               }
               if( qq==0 && kk==0 )
               {
                  ur_latlon[0] = lat*R2D;
                  ur_latlon[1] = lon*R2D;
               }
               if ( qq==1 && kk==0 )
               {
                  lr_latlon[0] = lat*R2D;
                  lr_latlon[1] = lon*R2D;
               }
            }
         }
         for( qq=0 ; qq<2 ; qq++ )
         {
            for( kk=0 ; kk<2 ; kk++ )
            {
               if( axx_forward_ali (in_lines[qq], in_samps[kk], bandno, 3, elev, *cpf, &amodel,
                                    dettype, &lat, &lon) == ERROR )
               {
                  xxx_LogStatus ("AXXLIB", __FILE__, __LINE__, "Error returned from ALI model");
                  return (ERROR);
               }
               if( lat > max_lat ) max_lat = lat;
               if( lat < min_lat ) min_lat = lat;
               if( lon > max_lon ) max_lon = lon;
               if( lon < min_lon ) min_lon = lon;
               if( qq==0 && kk==1 )
               {
                  ul_latlon[0] = lat*R2D;
                  ul_latlon[1] = lon*R2D;
               }
               if ( qq==1 && kk==1 )
               {
                  ll_latlon[0] = lat*R2D;
                  ll_latlon[1] = lon*R2D;
               }
            }
         }
         sprintf(Message, "Bounding area %f %f %f %f",(R2D*min_lat),(R2D*min_lon),(R2D*max_lat),(R2D*max_lon));
         xxx_LogStatus("AXXLIB", __FILE__, __LINE__,Message);
         if ( pcd->ALI_Group->cover_pos.value.d[jj] > 0.0 )
         {
           second = amodel.dce_time[2];
           hour = (int)floor(second/3600.0);
           second -= (double)hour*3600.0;
           minute = (int)floor(second/60.0);
           second -= (double)minute*60.0;
           fprintf(fptr,"%04d:%03d:%02d:%02d:%06.3lf\n",
                   (int)floor(amodel.dce_time[0]+0.5), (int)floor(amodel.dce_time[1]+0.5), hour, minute, second );
           fprintf(fptr,"UL:  Lat %10.6lf  Lon %11.6lf\n", ul_latlon[0], ul_latlon[1] );
           fprintf(fptr,"UR:  Lat %10.6lf  Lon %11.6lf\n", ur_latlon[0], ur_latlon[1] );
           fprintf(fptr,"LL:  Lat %10.6lf  Lon %11.6lf\n", ll_latlon[0], ll_latlon[1] );
           fprintf(fptr,"LR:  Lat %10.6lf  Lon %11.6lf\n", lr_latlon[0], lr_latlon[1] );
         }
         else
         {
           second = amodel.dce_time[2];
           hour = (int)floor(second/3600.0);
           second -= (double)hour*3600.0;
           minute = (int)floor(second/60.0);
           second -= (double)minute*60.0;
           if( COVER_POS )
           {
             printf("Cover position failed\n");
             printf("%04d:%03d:%02d:%02d:%06.3lf\n",
                 (int)floor(amodel.dce_time[0]+0.5), (int)floor(amodel.dce_time[1]+0.5), hour, minute, second );
             printf("UL:  Lat %10.6lf  Lon %11.6lf\n", ul_latlon[0], ul_latlon[1] );
             printf("UR:  Lat %10.6lf  Lon %11.6lf\n", ur_latlon[0], ur_latlon[1] );
             printf("LL:  Lat %10.6lf  Lon %11.6lf\n", ll_latlon[0], ll_latlon[1] );
             printf("LR:  Lat %10.6lf  Lon %11.6lf\n", lr_latlon[0], lr_latlon[1] );
           }
         }
      }
   }
   fclose(fptr);
      
   return(SUCCESS);
}
