#include "foot_print.h"

int axx_run_foot_print
(
   char *hk_dir,			/* I: Housekeeping data directory */
   char *hk_root,			/* I: Housekeeping data file root name */
   EphemType ephem_type,                /* I: ephem type to use GPSPTS or ACSPTS */
   char *cpf_name,			/* I: Path of the cpf (optional) */
   CPF **cpf				/* O: CPF structure */
)
{
   ALIPCD pcd;				/* ALI PCD data structure */
   struct EPHEM ephem[2];		/* ephemeris data from the PCD */
   double xp;				/* x shift pole wander in arc sec. */ 
   double yp;				/* y shift pole wander in arc sec. */ 
   double ut1_utc;			/* UT1 - UTC time difference in seconds */
   double mjd;				/* modified Julian date */
   char	ErrMsg[STRLEN];			/* error message used by xxx_LogError */

/*----------------------------------------------------------------------
  Read information needed to process model data. First get the PCD 
  information.
----------------------------------------------------------------------*/
   if ((xxx_read_ALIPCD (&pcd, hk_dir, hk_root )) == FAIL)
   {
      sprintf(ErrMsg,"Error loading ALI PCD from housekeeping file");
      xxx_LogStatus ("AXXLIB", __FILE__, __LINE__, ErrMsg);
      return (ERROR);
   }

/*----------------------------------------------------------------------
   Get the CPF information.
----------------------------------------------------------------------*/
   *cpf = exx_read_cpf (cpf_name);
   if (*cpf == NULL)
   {
      xxx_LogStatus ("AXXLIB", __FILE__, __LINE__,
                     "Error reading CPF in exx_read_cpf");
      xxx_free_ALIPCD( &pcd );
      return (ERROR);
   }

/*----------------------------------------------------------------------
   Get initial timing data from the ALIPCD structure.
   This includes using the amodel->dce_time values initially populated
   from the image start time to determine which DCE we're in, and
   updating the dce_time array with more accurate data.
----------------------------------------------------------------------*/
   if (axx_collects_pcd( &pcd, ephem_type, cpf ) != SUCCESS) 
   {
      xxx_LogStatus("AXXLIB", __FILE__, __LINE__, "Error returned from axx_collects_pcd");
      xxx_free_ALIPCD( &pcd );
      return(ERROR);
   }

/* ---------------------
   free the pcd
   --------------------- */
   xxx_free_ALIPCD( &pcd );

   return(SUCCESS); 
}
