#include "foot_print.h"

int main ( int argc, char *argv[])
{
/* vars */
int  status;		    /* status of return from function	 	*/
CPF *cpf;		    /* CPF structure				*/
char err_msg[STRLEN];	    /* message containing the error		*/

/* getpar's vars */
char parm_file[STRLEN];		/* the name of the input parameter file   */
char cpf_name[STRLEN];		/* name of the cpf file                   */
char hk_data_dir[STRLEN];	/* Housekeeping data file directory name  */
char hk_file_root[STRLEN];	/* Housekeeping root file name (no ext)	  */
char ephemtype[4];		/* Ephemeris type selection "ACS" or "GPS"*/
EphemType ephem_type;

/**********************************************************************/
/* make the std out line buffered */
setvbuf(stdout, NULL, _IOLBF, 0);

/* checking the number of arguments */
if (argc != 2)
{
    strcpy(err_msg,"Usage: alinit Parameter_File");
    xxx_LogStatus("FOOT_PRINT", __FILE__, __LINE__, err_msg);
    exit (EXIT_FAILURE);
}

/* telling the user we've started the prog */
xxx_LogStatus("FOOT_PRINT", __FILE__, __LINE__, "Initiated.");

/* getting the Parameter File */
strcpy(parm_file, (const char *)argv[1]);

/* Get user's parameters */
status = getpar(parm_file, cpf_name, hk_data_dir, hk_file_root, ephemtype);
if (status != SUCCESS)
{
    sprintf(err_msg,"Error reading parameters from %s", parm_file);
    xxx_LogStatus("FOOT_PRINT", __FILE__, __LINE__, err_msg);
    exit (EXIT_FAILURE);
}

/*
  Set the ephemeris selection type
*/
if ( strncmp( ephemtype, "GPS", 3 ) == 0 )
  ephem_type = GPSPTS;
else
  ephem_type = ACSPTS;

if (axx_run_foot_print(hk_data_dir, hk_file_root, ephem_type, cpf_name, &cpf) != SUCCESS)
{
    strcpy(err_msg,"Error returned from axx_run_foot_print.");
    xxx_LogStatus("FOOT_PRINT", __FILE__, __LINE__, err_msg);
    exit (EXIT_FAILURE);
}

if (hxx_run_foot_print(hk_data_dir, hk_file_root, ephem_type, cpf_name, &cpf) != SUCCESS)
{
    strcpy(err_msg,"Error returned from hxx_run_foot_print.");
    xxx_LogStatus("FOOT_PRINT", __FILE__, __LINE__, err_msg);
    exit (EXIT_FAILURE);
}

/* free the cpf */
exx_free_cpf(cpf);

/* Telling the User we're done with tminit */
xxx_LogStatus("FOOT_PRINT", __FILE__, __LINE__, "Successful Completion.");

return (EXIT_SUCCESS);
}
