#include <stddef.h>
#include <stdlib.h>
#include <stdio.h>
#include <sys/types.h>
#include <strings.h>
#include <ctype.h>
#include "xxx_ODL.h"
#include "xxx_LogStatus.h"      /* prototype for xxx_LogStatus.c */
#include "xxx_Types.h"          /* defines the ERROR variables */
#include "xxx_math.h"   	/* prototype definitions */
#include "xxx_structures.h"
#include "xxx_const.h"
#include "exx_cpf.h"
#include "xxx_ALIPCD.h"         /* defines the PCD structure */
#include "lablib3.h"
#include "axx_alimodel.h"
#include "hxx_hypmodel.h"
#include "axx_init_model.h"
#include "exx_init_model.h"

/* ********** */
/* structures */
/* ********** */

/*************************/
/* Prototypes            */
/*************************/

int getpar(
    char parm_file[STRLEN],	/* I: the name of the input Parm File 	*/
    char cpf_name[STRLEN],	/* O: name of the cpf files		*/
    char hk_data_dir[STRLEN],	/* O: housekeeping data directory 	*/
    char hk_file_root[STRLEN],	/* O: housekeeping file name root 	*/
    char ephemtype[4]);		/* O: "ACS" or "GPS" ephemeris option	*/

int axx_collects_pcd(
    ALIPCD *pcd,                /* I: Payload Correction Data from the satellite */
    EphemType ephem_type,       /* I: ephem type to use GPSPTS or ACSPTS */
    CPF **cpf);

int hxx_collects_pcd(
    HYPPCD *pcd,                /* I: Payload Correction Data from the satellite */
    EphemType ephem_type,       /* I: ephem type to use GPSPTS or ACSPTS */
    CPF **cpf);

int axx_run_foot_print(
   char *hk_dir,                        /* I: Housekeeping data directory */
   char *hk_root,                       /* I: Housekeeping data file root name */
   EphemType ephem_type,                /* I: ephem type to use GPSPTS or ACSPTS */
   char *cpf_name,                      /* I: Path of the cpf (optional) */
   CPF **cpf);                          /* O: CPF structure */

int hxx_run_foot_print(
   char *hk_dir,                        /* I: Housekeeping data directory */
   char *hk_root,                       /* I: Housekeeping data file root name */
   EphemType ephem_type,                /* I: ephem type to use GPSPTS or ACSPTS */
   char *cpf_name,                      /* I: Path of the cpf (optional) */
   CPF **cpf);                          /* O: CPF structure */

int exx_foot_print_ephem(
    const ALIPCD *pcd,          /* I: pcd structure */
    EphemType ephem_type,       /* I: ephem type to use GPSPTS or ACSPTS */
    const CPF *cpf,             /* I: Calibration Parameter File structure */
    int *num_valid,             /* O: the number of valid ephemeris points */
    struct EPHEM *ephem);       /* O: ephemeris struct with valid ephem data */

int axx_forward_ali(double line, double sample, BandNumber band, int sca,
        double tarelev, const CPF *cpf, ALIMODEL *alimodel, DetType dettype,
        double *tarlatd, double *tarlong);

int hxx_forward_hyp(double line, double sample, HBandNumber band, 
        double tarelev, const CPF *cpf, HYPMODEL *hypmodel, 
        double *tarlatd, double *tarlong);

