#include "foot_print.h"

int getpar(
    char parm_file[STRLEN],	/* I: the name of the Parm File 	*/
    char cpf_name[STRLEN],	/* O: name of the cpf files		*/
    char hk_data_dir[STRLEN],	/* O: housekeeping data directory 	*/
    char hk_file_root[STRLEN],	/* O: housekeeping data file name root 	*/
    char ephemtype[4])		/* O: "GPS" or "ACS" ephemeris switch	*/
{
/* Vars */
char err_msg[STRLEN];		/* error message */
int array_count;		/* number of elements in array */
int status;			/* status of return from function */
char temp_char_ptr[12];    /* temporary character pointer */
int len;

/* GSFC Vars */
xxx_ValType_TYPE parm_type;	/* type parameter read from Parm File */

/* ODL vars */
OBJDESC *in_parms;
xxx_EDCOdl_TYPE odl_type;

/* ******************************/
/* **** OPEN the Parm File **** */
/* ******************************/
odl_type = xxx_NoEDCMeta;	/* don't determine input ODL file name */
in_parms = xxx_OpenODL(parm_file,odl_type,err_msg);
if (in_parms == NULL)
{
    /* print xxx_ funct's errmsg */
    xxx_LogStatus("FOOT_PRINT",__FILE__,__LINE__,err_msg);

    sprintf(err_msg, "Error opening ODL file: %s",parm_file);
    xxx_LogStatus("FOOT_PRINT",__FILE__,__LINE__,err_msg);
    xxx_CloseODL(in_parms, err_msg);
    return(ERROR);
}

/* ******************************/
/* **** READ the Parm File **** */
/* ******************************/
/* name of the Calibration parameter file */
parm_type = XXX_String;
status = xxx_GetODLField(cpf_name,STRLEN, parm_type, in_parms, "FOOT_PRINT", "CAL_PARM_FILE", 
                         err_msg, &array_count);
if (status != SUCCESS || strcmp(cpf_name,"") == 0) 
{
    xxx_LogStatus("FOOT_PRINT", __FILE__, __LINE__,
    "Could not retrieve input CPF filename");
    xxx_CloseODL(in_parms, err_msg);
    return(ERROR);
}

/* name of the Housekeeping data directory */
parm_type = XXX_String;
status = xxx_GetODLField(hk_data_dir,STRLEN, parm_type, in_parms, "FOOT_PRINT", "HK_DATA_DIR", 
                         err_msg, &array_count);
if (status != SUCCESS || strcmp(hk_data_dir,"") == 0) 
{
    xxx_LogStatus("FOOT_PRINT", __FILE__, __LINE__,
    "Could not retrieve input housekeeping data directory name");
    xxx_CloseODL(in_parms, err_msg);
    return(ERROR);
}

/* name of the Housekeeping data file root name */
parm_type = XXX_String;
status = xxx_GetODLField( hk_file_root, STRLEN, parm_type, in_parms, "FOOT_PRINT", "HK_FILE_ROOT", 
                         err_msg, &array_count);
if (status != SUCCESS || strcmp(hk_file_root,"") == 0) 
{
    xxx_LogStatus("FOOT_PRINT", __FILE__, __LINE__,
    "Could not retrieve input housekeeping file root name");
    xxx_CloseODL(in_parms, err_msg);
    return(ERROR);
}

/* Ephemeris type selection switch */
parm_type = XXX_String;
status = xxx_GetODLField(ephemtype, STRLEN, parm_type, in_parms, "FOOT_PRINT", "EPHEM_TYPE", 
                         err_msg, &array_count);
if (status != SUCCESS || strcmp(ephemtype,"") == 0) 
{
    xxx_LogStatus("FOOT_PRINT", __FILE__, __LINE__,
    "Could not retrieve ephemeris type, will use ACS");
    strcpy(ephemtype,"ACS");
}

/* ****************************************** */
/* **** CLOSE the IAS Geometry Parm File **** */
/* ****************************************** */
status = xxx_CloseODL(in_parms, err_msg);
if (status != SUCCESS) 
{
    /* print xxx_ funct's errmsg */
    xxx_LogStatus("FOOT_PRINT",__FILE__,__LINE__,err_msg);

    xxx_LogStatus("FOOT_PRINT", __FILE__, __LINE__, 
	"Error closing ODL file");
    return(ERROR);
}

return(SUCCESS);
}
