#include <stdio.h>
#include <string.h>
#include "xxx_Types.h"  	/* defines the ERROR variables */
#include "xxx_HYPPCD.h"		/* defines the PCD structure */
#include "xxx_LogStatus.h"	/* prototype for xxx_LogStatus.c */
#include "xxx_const.h"  	/* defines several constants used */
#include "hxx_hypmodel.h"	/* model structure definition */
#include "hxx_init_model.h"
#include "xxx_math.h"	/* prototype definitions */

#define         EPS             2.0

int get_hyp_samples(CPF *cpf)
{
   return(cpf->Hyp_Focal_Plane_Parameters.Number_Detectors);
}

double get_hyp_frame_rate(CPF *cpf)
{
   return(cpf->Hyp_Scan_Parameters.Frame_Rate);
}

int hxx_collects_pcd(
    HYPPCD *pcd,		/* I: Payload Correction Data from the satellite */
    EphemType ephem_type,       /* I: ephem type to use GPSPTS or ACSPTS */
    CPF **cpf)
{
int 	ii, jj, kk, qq;     	/* Loop variables                            */
double  dtime;			/* time offset				     */
double  gpstime;		/* GPS reference time                        */
double  timecode;		/* Time code corresponding to gpstime        */
double  timecode_rate;		/* Spacecraft clock rate (seconds per second)*/
int hour, minute;            	/* Start time hour and minute                */
double second;               	/* Start time seconds                        */
char    Message[STRLEN];	/* Output message text		    	     */
int step_buffer=25;		/* Amount to step before/after imagery time  */
double curr_start;
int curr_index;
double swir_time_code;		/* SWIR Time code in GPS		     */
double vnir_time_code;		/* VNIR Time code in GPS		     */

FILE *fptr;			/* Output DCE corner point file pointer      */

struct EPHEM ephem[NPCD];    /* ephemeris data from the PCD */
double xp;                   /* x shift pole wander in arc sec. */
double yp;                   /* y shift pole wander in arc sec. */
double ut1_utc;              /* UT1 - UTC time difference in seconds */
double mjd;                  /* modified Julian date */
int num_valid;

double start_cover_pos;
double end_cover_pos;
double mid_cover_pos;
double lat;
double lon;
double ul_latlon[2],
       ur_latlon[2],
       ll_latlon[2],
       lr_latlon[2];
double max_lat;
double max_lon;
double min_lat;
double min_lon;
int number_of_lines;
int number_of_samps;
double in_lines[2];
double in_samps[2];
double elev=0.0;
HBandNumber bandno = h1;

HYPMODEL hmodel;

double stop_time[3];

hxx_zero_hmodel(&hmodel);

/* 
  Setting the satellite number
*/
hmodel.satnum = 1;
/*
  Setting the correction type in the model to systematic
*/
hmodel.correction_type = SYSTEMATIC;

/*
   Set frame rate
*/
hmodel.frame_rate = get_hyp_frame_rate(*cpf);

/*
  Set the ephemeris selection type
*/
hmodel.ephem_type = ephem_type;

if( hxx_los_coeffs(&hmodel, *cpf) != SUCCESS )
{
    xxx_LogStatus("HXXLIB", __FILE__, __LINE__, "Error returned from hxx_los_coeffs");
    return(ERROR);
}

fptr = fopen( "hypcorners.txt", "w" );

pcd->active_DCE = -1;
/* for (ii=0; ii<pcd->num_DCE; ii++) */
for (ii=0; ii<pcd->num_DCE; ii++)
{
   pcd->active_DCE = ii;

   sprintf(Message,"Selected DCE %d",pcd->active_DCE);
   xxx_LogStatus("HXXLIB", __FILE__, __LINE__,Message);

   /* Convert the actual DCE time to year, doy, sod */
   exx_gpstoyds( pcd->DCE_List[pcd->active_DCE].start_time, hmodel.dce_time );

   kk = 0;
   jj = pcd->DCE_List[pcd->active_DCE].gps_start_index - kk;
   while( jj>1 && kk<step_buffer )
   {
      if( fabs(pcd->GPS_Group->acsutctime.value.d[jj]-pcd->GPS_Group->acsutctime.value.d[jj-1]) > EPS )
         break;
      kk++;
      jj = pcd->DCE_List[pcd->active_DCE].gps_start_index - kk;
   }
   pcd->DCE_List[pcd->active_DCE].gps_start_index = jj;
   pcd->DCE_List[pcd->active_DCE].acs_start_tc = pcd->GPS_Group->acsutctime.time[jj];
   kk = 0;
   jj = pcd->DCE_List[pcd->active_DCE].gps_stop_index + kk;
   while( jj<(pcd->GPS_Group->acsutctime.count-1) && kk<step_buffer )
   {
      if( fabs(pcd->GPS_Group->acsutctime.value.d[jj]-pcd->GPS_Group->acsutctime.value.d[jj-1]) > EPS )
         break;
      kk++;
      jj = pcd->DCE_List[pcd->active_DCE].gps_stop_index + kk;
   }
   pcd->DCE_List[pcd->active_DCE].gps_stop_index = jj;
   pcd->DCE_List[pcd->active_DCE].acs_stop_tc = pcd->GPS_Group->acsutctime.time[jj];

   /* Find the GPS UTCF record that immediately follows this time code */
   curr_index = 0;
   curr_start = pcd->DCE_List[pcd->active_DCE].acs_start_tc;
   while ( pcd->GPS_Group->utcftime.time[curr_index] < curr_start &&
               curr_index < pcd->GPS_Group->utcftime.count ) curr_index++;
   pcd->DCE_List[pcd->active_DCE].utcf_index = curr_index;

   sprintf(Message,"Starting index for imagery %d",pcd->DCE_List[pcd->active_DCE].gps_start_index);
   xxx_LogStatus("HXXLIB", __FILE__, __LINE__,Message);
   sprintf(Message,"DCE LIST Start time %f",pcd->DCE_List[pcd->active_DCE].start_time);
   xxx_LogStatus("HXXLIB", __FILE__, __LINE__,Message);
   sprintf(Message,"ACS Start %f",pcd->DCE_List[pcd->active_DCE].acs_start_tc);
   xxx_LogStatus("HXXLIB", __FILE__, __LINE__,Message);

   sprintf(Message,"Stop index for imagery %d",pcd->DCE_List[pcd->active_DCE].gps_stop_index);
   xxx_LogStatus("HXXLIB", __FILE__, __LINE__,Message);
   sprintf(Message,"DCE LIST Stop time %f",pcd->DCE_List[pcd->active_DCE].stop_time);
   xxx_LogStatus("HXXLIB", __FILE__, __LINE__,Message);
   sprintf(Message,"ACS Stop %f",pcd->DCE_List[pcd->active_DCE].acs_stop_tc);
   xxx_LogStatus("HXXLIB", __FILE__, __LINE__,Message);
   
   number_of_lines =
       (pcd->DCE_List[pcd->active_DCE].stop_time-pcd->DCE_List[pcd->active_DCE].start_time) *
       hmodel.frame_rate;
   number_of_samps = get_hyp_samples(*cpf);
   hmodel.num_det = number_of_samps;
   sprintf(Message,"Lines: %d   Samples: %d", number_of_lines, number_of_samps);
   xxx_LogStatus("HXXLIB",__FILE__,__LINE__,Message);

   in_lines[0] = 1.0;
   in_lines[1] = number_of_lines;
   in_samps[0] = 1.0;
   in_samps[1] = number_of_samps;

   /* Convert SWIR/VNIR time codes from MET to GPS epoch */
   /**************************************************************************************
   vnir_time_code = hmodel.vnir_time_code + pcd->GPS_Group->utcftime.value.d[curr_index];
   swir_time_code = hmodel.swir_time_code + pcd->GPS_Group->utcftime.value.d[curr_index];
   printf("Delta VNIR/SWIR time codes (GPS) %f\n",vnir_time_code-swir_time_code);
   **************************************************************************************/
   
   /* Convert the actual DCE time to year, doy, sod */
   hmodel.vnir_time_code = exx_gpstime( hmodel.dce_time );
   hmodel.swir_time_code = exx_gpstime( hmodel.dce_time );
   
   sprintf(Message,"Refined DCE Time:  %04d %03d %lf",
           (int)hmodel.dce_time[0], (int)hmodel.dce_time[1], hmodel.dce_time[2]);
   xxx_LogStatus("HXXLIB", __FILE__, __LINE__,Message);
   
   sprintf(Message,"DCE Duration:  %lf",
           pcd->DCE_List[pcd->active_DCE].stop_time-pcd->DCE_List[pcd->active_DCE].start_time);
   xxx_LogStatus("HXXLIB", __FILE__, __LINE__,Message);
   
   /* find the ACS time */
   /* The ACS time reference is in the GPS telemetry. It gives a sequence
      of GPS times (time relative to the GPS epoch) that are time stamped
      with the spacecraft time codes. This provides a basis for building
      the relationship between the spacecraft clock (time codes) and
      GPS time. What we're going to do here is figure out what this
      (presumably linear) relationship is for this DCE and initialize
      the exx_init_timecode routine so that we can subsequently use the
      exx_timecodetogps and exx_gpstotimecode routines to convert back
      and forth (e.g., when mucking with the ephemeris and attitude data). */
   
   kk = pcd->DCE_List[pcd->active_DCE].gps_start_index;
   qq = pcd->DCE_List[pcd->active_DCE].gps_stop_index;
   if ( qq <= kk )
   {
      qq = kk + 1;
      if ( qq >= pcd->GPS_Group->acsutctime.count )
      {
         kk--;
         qq--;
      }
   }
   gpstime = pcd->GPS_Group->acsutctime.value.d[kk];

   timecode = pcd->GPS_Group->acsutctime.time[kk];
   
   /* IMPROVEMENT_OPPORTUNITY
      We'll do the rate computation from the two end points for now.
      Could also do a regression with all the points. */
   
   timecode_rate = (pcd->GPS_Group->acsutctime.value.d[qq] - gpstime)
                 / (pcd->GPS_Group->acsutctime.time[qq] - timecode);
   
   exx_init_timecode( gpstime, timecode, timecode_rate );
   
   /* Now we'll use this timing information to find the stop time for the
      quaternion data */

   timecode = pcd->DCE_List[pcd->active_DCE].acs_stop_tc;
   qq = xxx_find_index( timecode, pcd->ACS_Group->quat1 );
   timecode = pcd->ACS_Group->quat1.time[qq];
   gpstime = exx_timecodetogps( timecode );
   exx_gpstoyds( gpstime, stop_time );

   /* Now we'll use this timing information to find the start time for the
      quaternion data */
   
   timecode = pcd->DCE_List[pcd->active_DCE].acs_start_tc;
   kk = xxx_find_index( timecode, pcd->ACS_Group->quat1 );
   timecode = pcd->ACS_Group->quat1.time[kk];
   gpstime = exx_timecodetogps( timecode );
   
   /* Finally, we'll load the acs_time array */
   exx_gpstoyds( gpstime, hmodel.acs_time );
   sprintf(Message,"ACS Start Time:  %04d %03d %lf",
           (int)hmodel.acs_time[0], (int)hmodel.acs_time[1], hmodel.acs_time[2]);
   xxx_LogStatus("HXXLIB", __FILE__, __LINE__,Message);
   sprintf(Message,"ACS Stop Time:  %04d %03d %lf",
           (int)stop_time[0], (int)stop_time[1], stop_time[2]);
   xxx_LogStatus("HXXLIB", __FILE__, __LINE__,Message);
   
   /* Make sure our data is in the effective time period for the CPF */
   if (exx_check_date(hmodel.dce_time, *cpf) != SUCCESS)
   {
      strcpy(Message,"Error returned from exx_check_date.");
      xxx_LogStatus ("HXXLIB", __FILE__, __LINE__,Message);
      return(ERROR);
   }

   /* Select the valid ephemeris points. */
   if (hxx_valid_ephem (pcd, hmodel.ephem_type, *cpf, &hmodel.nephem, ephem) == ERROR)
   {
      strcpy(Message,"Error obtaining valid ephemeris points hxx_valid_ephem");
      xxx_LogStatus ("HXXLIB", __FILE__, __LINE__,Message);
      /* return (ERROR); */
   }
   else
   {
      /* Get x and y shift pole wander and UT1-UTC time difference */
      if (exx_getmjdcoords (ephem->time, &mjd, &xp, &yp, &ut1_utc, *cpf) == ERROR)
      {
         strcpy(Message,"Error returned from exx_getmjdcoords");
         xxx_LogStatus ("HXXLIB", __FILE__, __LINE__,Message);
         return (ERROR);
      }
   
      /* Generate updated model information. */
      if (hxx_process_hyp_model (ephem, xp, yp, ut1_utc, *cpf, pcd, &hmodel) == ERROR)
      {
         strcpy(Message,"Error updating the Hyperion model");
         xxx_LogStatus ("HXXLIB", __FILE__, __LINE__, Message);
         return (ERROR);
      }
      for( qq=0 ; qq<2 ; qq++ )
      {
         for( kk=0 ; kk<2 ; kk++ )
         {
            if( hxx_forward_hyp (in_lines[qq], in_samps[kk], bandno, elev, *cpf, &hmodel,
                                 &lat, &lon) == ERROR )
            {
               xxx_LogStatus ("HXXLIB", __FILE__, __LINE__, "Error returned from Hyperion model");
               xxx_LogStatus ("HXXLIB", __FILE__, __LINE__, "Possible extraterrestrial observation");
               lat = 0.0;
               lon = 0.0;
            }
            if( qq==0 && kk==0 )
            {
               max_lat = lat;
               max_lon = lon;
               min_lat = lat;
               min_lon = lon;
            }
            else
            {
               if( lat > max_lat ) max_lat = lat;
               if( lat < min_lat ) min_lat = lat;
               if( lon > max_lon ) max_lon = lon;
               if( lon < min_lon ) min_lon = lon;
            }
            if( qq==0 && kk==0 )
            {
               ul_latlon[0] = lat*R2D;
               ul_latlon[1] = lon*R2D;
            }
            if ( qq==1 && kk==0 )
            {
               ll_latlon[0] = lat*R2D;
               ll_latlon[1] = lon*R2D;
            }
            if( qq==0 && kk==1 )
            {
               ur_latlon[0] = lat*R2D;
               ur_latlon[1] = lon*R2D;
            }
            if ( qq==1 && kk==1 )
            {
               lr_latlon[0] = lat*R2D;
               lr_latlon[1] = lon*R2D;
            }
         }
      }
      sprintf(Message, "Bounding area %f %f %f %f",(R2D*min_lat),(R2D*min_lon),(R2D*max_lat),(R2D*max_lon));
      xxx_LogStatus("HXXLIB", __FILE__, __LINE__,Message);
      second = hmodel.dce_time[2];
      hour = (int)floor(second/3600.0);
      second -= (double)hour*3600.0;
      minute = (int)floor(second/60.0);
      second -= (double)minute*60.0;
      fprintf(fptr,"%04d:%03d:%02d:%02d:%06.3lf\n",
              (int)floor(hmodel.dce_time[0]+0.5), (int)floor(hmodel.dce_time[1]+0.5), hour, minute, second );
      fprintf(fptr,"UL:  Lat %10.6lf  Lon %11.6lf\n", ul_latlon[0], ul_latlon[1] );
      fprintf(fptr,"UR:  Lat %10.6lf  Lon %11.6lf\n", ur_latlon[0], ur_latlon[1] );
      fprintf(fptr,"LL:  Lat %10.6lf  Lon %11.6lf\n", ll_latlon[0], ll_latlon[1] );
      fprintf(fptr,"LR:  Lat %10.6lf  Lon %11.6lf\n", lr_latlon[0], lr_latlon[1] );
   }
}
fclose(fptr);

return(SUCCESS);
}
