#include "xxx_LogStatus.h"
#include "xxx_HYPPCD.h"
#include "exx_geometric.h"
#include "hxx_hypmodel.h"

int hxx_run_foot_print
(
   char *hk_dir,			/* I: Housekeeping data directory */
   char *hk_root,			/* I: Housekeeping data file root name */
   EphemType ephem_type,                /* I: ephem type to use GPSPTS or ACSPTS */
   char *cpf_name,			/* I: Path of the cpf (optional) */
   CPF **cpf				/* O: CPF structure */
)
{
   HYPPCD pcd;				/* ALI PCD data structure */
   struct EPHEM ephem[NPCD];		/* ephemeris data from the PCD */
   double xp;				/* x shift pole wander in arc sec. */ 
   double yp;				/* y shift pole wander in arc sec. */ 
   double ut1_utc;			/* UT1 - UTC time difference in seconds */
   double mjd;				/* modified Julian date */
   char	ErrMsg[STRLEN];			/* error message used by xxx_LogError */

/*----------------------------------------------------------------------
  Read information needed to process model data. First get the PCD 
  information.
----------------------------------------------------------------------*/

   if ((xxx_read_HYPPCD (&pcd, hk_dir, hk_root )) == FAIL)
   {
      sprintf(ErrMsg,"Error loading Hyperion PCD from housekeeping file");
      xxx_LogStatus ("HXXLIB", __FILE__, __LINE__, ErrMsg);
      return (ERROR);
   }

   if (xxx_find_DCEs_hyp(&pcd) == FAIL)
   {
      sprintf(ErrMsg,"Error finding Hyperion PCD from housekeeping file");
      xxx_LogStatus ("HXXLIB", __FILE__, __LINE__, ErrMsg);
      return (ERROR);
   }

/*----------------------------------------------------------------------
   Get initial timing data from the HYPPCD structure.
   This includes using the hmodel->dce_time values initially populated
   from the image start time to determine which DCE we're in, and
   updating the dce_time array with more accurate data.
----------------------------------------------------------------------*/
   if (hxx_collects_pcd( &pcd, ephem_type, cpf ) != SUCCESS) 
   {
      xxx_LogStatus("HXXLIB", __FILE__, __LINE__,
                    "Error returned from axx_out_of_pcd");
      xxx_free_HYPPCD( &pcd );
      return(ERROR);
   }

/*---------------------
   free the pcd
   ---------------------*/
   xxx_free_HYPPCD( &pcd );

   return(SUCCESS); 
}
