/*
 *  File:  cp_consts.h
 *
 *  Description:  File with constants for cal pipeline
 *
 *  Author:       Jenifer Brinker Evans
 *
 *  Date        Author          Action
 * -----------------------------------------------------------------------
 * 02/19/98     JBE             Version 1.1 complete
 *
 * 04/06/98	MDG		changed file name from cal_pipe.h.h to 
 *					cp_consts.h
 *				added MAX_SENSORS & MAX_DS_TYPES
 * 04/21/99     JBE             Added more coefficient types for database
 */


#ifndef _CP_CONSTS_
#define _CP_CONSTS_


#define MAX_DATASETS 9

#define MAX_DS_TYPES 25
#define MAX_IN_TYPES 9
enum ds_types {LEVEL0, CAL1, CAL2, CAL3, CAL4, PRECAL, DARK, LAMP, MAP, LEVEL1R, RESPONSE, OFFSET, CAL1R, CAL2R, CAL3R, CAL4R, CAL1_RESP, CAL2_RESP, CAL3_RESP, CAL4_RESP, CAL1_OFF, CAL2_OFF, CAL3_OFF, CAL4_OFF,MAPOUT};

#define MAX_SENSORS 2
enum sensor_types {MS, PAN};

#define MAX_COEFF_TYPES 5 
enum coeff_types {RESP_COEFF, OFF_COEFF, INTCAL, SOLCAL, LEAKYPIX};

/* Max bands only used for prototyping and variable definitions. */
#define MAX_BANDS 9

/* Constants used in leaky pixel correction */
/*  K3B3 is for SCA3, Band 3.  K4B2 is for SCA4, Band 2 */
#define K3B3 0.015
#define K4B2 0.032

#define TimeVarying 0

#define SCALE_FACTOR 300

#define PIXELH 0.04

/* MACROS */

#ifndef ABS        /* absolute value */
  #define ABS(a) (((a)<(0.0))?((-1)*(a)):(a))
#endif

#ifndef MIN        /* minimum value */
  #define MIN(a,b) (((a)<(b))?(a):(b))
#endif

#ifndef MAX        /* maximum value */
  #define MAX(a,b) (((a)>(b))?(a):(b))
#endif

#ifndef SQUARE     /* squared value */
  #define SQUARE(a) ((a)*(a))
#endif

#ifndef ROUND      /* rounded value */
  #define ROUND(a) ( (a > 0)? ((int)((a) + 0.5)): ((int)((a) -0.5)) )
#endif

#ifndef ROUND_UP      /* round to higher value, 5.3 to 6, -5.3 to -5 */
  #define ROUND_UP(a) ( (a > 0)? ((int)((a) + 0.99)): ((int)((a)-0.01 )) )
#endif

#ifndef TRUE
  #define TRUE	1
#endif

#ifndef FALSE
  #define FALSE	0
#endif

#ifndef HAVE_BOOLEAN
  typedef int boolean;
#endif

#endif	/* ifndef _CP_CONSTS_ */



