/*
 *  File:         
 *
 *  SCCS id:      " @(#)cp_leaky_pixel.c	3.2 8/30/1 " 
 *
 *  Description:  
 *
 *  Subroutines:  
 *
 *  Author:	  Jenifer Brinker Evans
 *
 *  Date	Author		Action
 * -----------------------------------------------------------------------
 * 12/00	JBE		Version 1.1 complete
 * 04/05/01     JBE             Flip leaky pixel gains for new data.
 * 04/18/01     JBE             Change correction from linfit to svdfit
 *                              Also included bug correction associated with
 *                                using FLIP flag.
 *  6/27/01     JBE             Correction for building svd arrays for SCA4.
 *                               One pixel not getting correct value.
 *
 */

#include "math.h"
#include "hdf.h"
#include "mfhdf.h"
#include "cp_includes.h"
#include "cp_svdconsts.h"


extern DATA_SET Dataset[MAX_SENSORS][MAX_DS_TYPES];
extern GAIN_DB DatabaseCoeff[MAX_SENSORS][MAX_COEFF_TYPES];
extern int32 FLIP;
extern FILE *logfile;
extern char *SensorName[2];


void linfit(float x[],float y[],int ndata, float *a,float *b,
	    float *siga,float *sigb,float *chi2);
void AdjustLeakyPixels(short *cal_data, int32 start);



/*
  
  printf("a:  %9.5f     b:  %9.5f    siga:  %7.5f   sigb:  %7.5f\n",
	 a,b,siga,sigb);

	 */

/***************************************************************************/
/***************************************************************************/
void CalibrateLeakyPixels(enum sensor_types Sensor,
			  enum ds_types DataType,
		   short *fpdata, 
		   int32 dim[2], 
		   int32 dim_table[2],
		   int32 along, 
		   float *lp_gain_data,
		   float *offset_data, 
		   float *response_data, 
		   short *cal_data,
		   float integ_time,
		   int32 sca_num,
		   int32 band)


{

  int32 start, n,k, cross;
  int32 cal_value_scaled;
  int32 flip_const;  
  float modified_DN,cal_value;
  float k_val;
  static int loop=0;
  

  ++loop;
  
  if ((sca_num == 3) && (band == 3))
    {      
      /* if command line arg -f, use old way pre 2/15/01 */
      if (FLIP)
	start = 0;
      else /* else new way with left-to-right flip */
	start = 1;
      k_val = K3B3;

      /* To handle flipping of leaky pixel gains, and difference between
	 even and odd leakers, set flip_const.*/
      flip_const = 2;
      
    }  
  else 
    {
      /* if command line arg -f, use old way pre 2/15/01 */
      if (FLIP)
	start = 1;
      else /* else new way with left-to-right flip */
	start = 0;
      k_val = K4B2;

      /* To handle flipping of leaky pixel gains, and difference between
	 even and odd leakers, set flip_const.*/
      flip_const = 1;
    }
  
  

  /*  Do offset correction, gain correction, and scaling on 
      every other pixel -- the pixels affected by the leakers
      */
  for(cross = start; cross < Dataset[Sensor][DataType].CROSS; cross += 2)
    {			
      n = index2(along,cross,dim);	  
      

      /* 
       *  if data is fill data, set calibrated data and response data 
       *   equal to matching fill data, and offset data is already 
       *   equal to 0
       */
      if ((fpdata[n] & 0xf000) != 0)
	{
	  cal_data[cross] = fpdata[n];
	  response_data[cross] = fpdata[n];
	  /* if (loop == 500) printf("using fill data:  n= %d\n",n); */
	  
	}
      
      else
	{	
	  modified_DN = (short)(fpdata[n] - offset_data[cross]);


	  if (modified_DN < -10)
	    modified_DN = -10;
	
	  /* If the data has been flipped, the data is opposite than the
	     leaky pixel gains.  There are gain values for every other pixel,
	     hence the need to divide by two.
	     */
	  if (FLIP)
	    k = (int)(cross/2);
	  else
	    k = ((Dataset[Sensor][DataType].CROSS - cross + flip_const)/2) - 1;	  

	  
	  cal_value = k_val*modified_DN/lp_gain_data[k];	  
	  
	  /* convert from float to scaled int */
	  cal_value_scaled = ROUND((cal_value)*SCALE_FACTOR);

	  /* Check for case of hot pixels or anomoly that would overflow
	     short int */
	  if (cal_value_scaled >= 32767) cal_value_scaled = 32767;	  

	  cal_data[cross] = (short)cal_value_scaled;
	  	  
	  
	}
      
      


      
    }
  
  AdjustLeakyPixels(cal_data, start);
  
  return;
}



/***************************************************************************/
/***************************************************************************/
/*
 *  Name:  ReadLeakyGains
 * 
 *  Description:  Reads gain values from appropriate database dataset
 *
 *  Inputs:
 *        Sensor        enum sensor_type  Which sensor, MS or PAN
 *
 *  Outputs:
 *        lp_gain_data  float             the gain values read
 *
 *  Returns:  nothing
 *
 */

void ReadLeakyGains(enum sensor_types Sensor, float *lp_gain_data)
{
  static int32 database_start[3], database_edge[3];
  int32 status;

    
  database_start[0] = 0;
  database_start[1] = 0;
  database_start[2] = 0;
  database_edge[0] = 1;
  database_edge[1] = DatabaseCoeff[Sensor][LEAKYPIX].CROSS;
  database_edge[2] = DatabaseCoeff[Sensor][LEAKYPIX].LENGTH;
  
  /* read response data for appropriate band */
  status = SDreaddata(DatabaseCoeff[Sensor][LEAKYPIX].SDS_ID, database_start,
		      NULL, database_edge,(VOIDP)lp_gain_data);

  if (status == FAIL)
    {
      printf("\nERROR:  SDreaddata failed!  %d \n", (int)status);
      fprintf(logfile,"\nERROR:  SDreaddata failed!  %d \n", (int)status);
      fprintf(logfile,"sds_id: %ld  start: %ld %ld %ld    edge: %ld %ld %ld /n",
	      DatabaseCoeff[Sensor][LEAKYPIX].SDS_ID, database_start[0], 
	      database_start[1], database_start[2],
	      database_edge[0], database_edge[1], database_edge[2]);	  
      fprintf(logfile,"   Unable to read leaky pixel coefficients.  Exiting.\n"
	      );
      printf("   Unable to read leaky pixel coefficients.  Exiting.\n");
      GracefulExit();
      
      return;	  
    }      
  
  

  return;
}



/***************************************************************************/
/***************************************************************************/
/*
 *  Name:  AdjustLeakyPixels
 * 
 *  Description:  Takes one frame of data at a time and performs
 *                leaky pixel adjustment.  Current algorithm takes
 *                difference between a leaked into pixel and the
 *                adjacent pixel as an x-value.  And the value of 
 *                that adjacent pixel as the y-value.  A linear fit
 *                across the 160 pairs are done to get y = a+bx.  
 *                The a and b are used to add a correction value to
 *                each of the leaked in to pixels.  
 *
 *  Inputs:
 *        start             int           indicates whether the leaked into
 *                                          values are the evens or the odds
 *        cal_data          short         the data being used and corrected
 *
 *  Outputs:
 *        cal_data          short         the corrected values
 *
 *  Returns:  nothing
 *
 */

void AdjustLeakyPixels(short *cal_data, int32 start)

{
  float avg_even[161], avg_odd[161], diff[161], new[160];
  int32 i, num=160;
  float coef[order+1], temp_scale;
  float u[161][order+1],w[order+1],v[order+1][order+1];
  void fpoly(float, float *, int);  
  static int poly_order = order;
  static int loop = 0;
  
  
  
  /*  Break cal_data into two arrays.  One that has been leaked into,
      and one that is the average of the two neighbors that have not 
      been leaked into.

      Special note:  the svd routines begin incrementing on 1.  All
      other stuff in cal pipe uses traditional C method of 0.
      */

  temp_scale = (float)(SCALE_FACTOR);
  
  ++loop;
  

  if (start == 0)
    {

      /* since the first pixel only has one neighbor, use it alone */
      avg_odd[1] = (cal_data[1]/temp_scale);
      diff[1] = (cal_data[0]/temp_scale)-avg_odd[1];
      
      for (i=1; i < (num); ++i)
	{
	  avg_odd[i+1] = (cal_data[(2*i)-1] + cal_data[(2*i)+1])
	    /(2.0*temp_scale);
	  diff[i+1] = (cal_data[(2*i)]/temp_scale) - avg_odd[i+1];
	  
	}

      svdfit(avg_odd,diff,num,coef,poly_order,u,v,w,fpoly);
	  

      for (i=0; i< num; ++i)
	{
	  new[i] = (cal_data[(2*i)]/temp_scale) 
	    - (coef[1] + coef[2]*avg_odd[(i+1)] +
	       coef[3]*avg_odd[(i+1)]*avg_odd[(i+1)]);
	}

    }
  
  else  /*if start == 1 */
    {

      /* since the last pixel only has one neighbor, use it alone */
      avg_even[num] = cal_data[2*(num-1)]/temp_scale;
      diff[num] = (cal_data[(2*(num-1))+1]/temp_scale)-avg_even[num];
      
      for (i=0; i < (num-1); ++i)
	{       
	  avg_even[i+1] = (cal_data[(2*i)] + cal_data[(2*i)+2])/(2.0*temp_scale);
	  diff[i+1] = (cal_data[(2*i)+1]/temp_scale) - avg_even[i+1];
		    
	}

      svdfit(avg_even,diff,num,coef,poly_order,u,v,w,fpoly);


      for (i=0; i< num; ++i)
	{
	  new[i] = (cal_data[(2*i)+1]/temp_scale) -
	    (coef[1] + coef[2]*avg_even[i+1] +
	     coef[3]*avg_even[i+1]*avg_even[i+1]);
	  
	}

    }
      
  
  /* take new computed values and convert to 16 bit int and
     put in proper spots in cal_data
     */
  for (i = start; i < 2*num; i+=2)
    {
      cal_data[i] = (short)ROUND(new[(int32)(i/2)]*SCALE_FACTOR);
    }

  
  return;

}



    

/***************************************************************************/
/***************************************************************************/
/*
 *  Name:  linfit
 * 
 *  Description:  Copied from Numerical Recipes in C.  Given a set of points
 *  x[1..ndata], y[1..ndata] and standard deviations assumed to be one, fit
 *  them to a straight line y = a+bx by minimizing chi**2.  Returned are 
 *  a,b and their respective probable uncertainties siga and sigb, the
 *  chi-square chi2.
 *
 *  Inputs: 
 *      x            float        array of x data points
 *      y            float        array of y data points
 *      ndata        int          number of elements in x and y
 *
 *  Outputs:
 *      a            float        a from y = a + bx
 *      b            float        b from y = a + bx
 *      siga         float        error on a
 *      sigb         float        error on b
 *      chi2         float        the chi square
 *
 *  Returns:         nothing
 *
 */



void linfit(float x[],float y[],int ndata, float *a,float *b,
	    float *siga,float *sigb,float *chi2)

{
  int i;
  float t, sxoss, sx=0.0, sy = 0.0, st2 = 0.0, ss, sigdat;
  float gammq();
  
  *b = 0.0;
  
  for (i = 0; i < ndata; ++i)
    {
      sx += x[i];
      sy += y[i];
    }

  ss = ndata;
  
  sxoss = sx/ss;
  
  for (i=0; i < ndata; ++i)
    {
      t = x[i] - sxoss;
      st2 += t*t;
      *b += t*y[i];
    }

  *b /= st2;
  *a = (sy - sx*(*b))/ss;
  *siga = sqrt((1.0 + sx*sx/(ss*st2))/ss);
  *sigb = sqrt(1.0/st2);
  *chi2 = 0.0;
  for (i = 0; i <= ndata; i++)
    *chi2 += SQUARE(y[i] - (*a) - (*b)*x[i]);
  sigdat = sqrt((*chi2)/(ndata-2));
  *siga *= sigdat;
  *sigb *= sigdat;
  

  return;
  
}

   















