/*
 *  File:  cp_pixel_map.c
 *
 *  Description:  This file has the routine for copying the MS SWIR pixel
 *                map from the Level 0 HDF file to the Level 1R HDF file.
 *
 *  Subroutines:  CopyPixelMap
 *
 *  Author:	  Jenifer Brinker Evans
 *
 *  Date	Author		Action
 * -----------------------------------------------------------------------
 * 10/02/98	JBE		Version 1.1 complete
 */


#include "hdf.h"
#include "mfhdf.h"
#include "cp_includes.h"


extern DATA_SET Dataset[MAX_SENSORS][MAX_DS_TYPES];
extern FILE *logfile;





/***************************************************************************/
/***************************************************************************/
/*
 *  Name:  CopyPixelMap
 * 
 *  Description:  Function checks for the existence of a pixel map
 *                in the Level 0 data.  If there, it copies the 
 *                pixel map to the Level 1R data.
 *
 *  Inputs:   Nothing
 *
 *  Outputs:  Modifies Level 1R MAPOUT dataset
 *
 *  Returns:  Nothing
 *
 */

void CopyPixelMap()
{  
short *map_data;
int32 status;
int32 start[2], edge[2];
int32 num_pixels;
enum sensor_types Sensor;

Sensor = MS;

start[0] = 0;
start[1] = 0;
edge[0] = 1;
edge[1] = Dataset[Sensor][MAP].CROSS;


if (Dataset[Sensor][MAP].EXISTS)
  {

    /* Malloc space for reading single band of data. */
    num_pixels = Dataset[Sensor][MAP].CROSS;
    map_data = (short *)malloc((unsigned) num_pixels*sizeof(short));
    
    status = SDreaddata(Dataset[Sensor][MAP].SDS_ID, start, NULL, edge, 
			(VOIDP)map_data);
    
    if (status == FAIL)
      {
	fprintf(logfile,"\nERROR:  SDreaddata for Pixel Map failed! %d \n",status);
	fprintf(logfile,"sds_id: %d /n", Dataset[Sensor][MAP].SDS_ID);	  
      }
    else
      {
	status = SDwritedata(Dataset[Sensor][MAPOUT].SDS_ID, start, NULL, 
			     edge, (VOIDP)map_data);
	
	if (status == FAIL)
	  {
	    fprintf(logfile,"\nERROR:  SDwritedata Pixel Map failed! %d\n",status);
	    fprintf(logfile,"sds_id: %d /n", Dataset[Sensor][MAPOUT].SDS_ID); 
	  }
      }

    free((short *)map_data);
  }

}

