;Routines for Hyperion Radiometric cals.
; Lawrence Ong
; Release: 0.0b March 22, 2002
; Revisions:
;         April 09, 2002 :  Added Image Start Time attr to output
;         May   02, 2002 :  Change SWIR output prefix label 
;                           from *.cal to *_swir.cal
;         May   08, 2002 :  Added checks for INT over/under flows
;                           when rescaling: multiply by 80.
;         May   14, 2002 :  Added keyword switch for edc/gsfc cal
;                           files directories
;
function HypSwirCal,filename,predarkfile,postdarkfile,GainFile=GainFile,trend0=trend0,index=index,gsfc=gsfc

    nswir=172
    offsettoswir=71

;get smear corrected predark
    
    prefile=predarkfile+".echo"
    print,"reading: ",prefile
    OPEN_HDF,prefile,$
      arrsize,datasets,sds_id,sd_id,dsname,attrname,sds_name
    nfov=long(arrsize(0))
    nspec=long(arrsize(1))
    nframes=long(arrsize(2))
    hdf_sd_attrinfo,SDS_ID,5,name=time_UTC,data=pre_t
    HDF_SD_GETDATA, sds_id(0),predark0,$
      COUNT=[nfov,nspec,nframes],$
      START=[0,0,0]
    HDF_SD_ENDACCESS, sds_id
    HDF_SD_END, sd_id
    help,predark0

;get smear corrected postdark

    postfile=postdarkfile+".echo"
    print,"reading: ",postfile
    OPEN_HDF,postfile,$
      arrsize,datasets,sds_id,sd_id,dsname,attrname,sds_name
    nfov=long(arrsize(0))
    nspec=long(arrsize(1))
    nframes=long(arrsize(2))
    hdf_sd_attrinfo,SDS_ID,5,name=time_UTC,data=post_t
    HDF_SD_GETDATA, sds_id(0),postdark0,$
      COUNT=[nfov,nspec,nframes],$
      START=[0,0,0]
    HDF_SD_ENDACCESS, sds_id
    HDF_SD_END, sd_id
    help,postdark0

;Get Swir data
    imagefile=filename+".echo"
    if keyword_set(index) then $
      imagefile=strcompress(imagefile+"."+string(index,format='(i2.2)'),/remove_all)
    print,"reading: ",imagefile
    OPEN_HDF,imagefile,$
      arrsize,datasets,sds_id,sd_id,dsname,attrname,sds_name
    nfov=long(arrsize(0))
    nspec=long(arrsize(1))
    nframes=long(arrsize(2))
    hdf_sd_attrinfo,SDS_ID,0,name=Level0file,data=L0file
    hdf_sd_attrinfo,SDS_ID,1,name=Smearfile,data=Sfile
    hdf_sd_attrinfo,SDS_ID,2,name=ratiofile,data=rfile
    hdf_sd_attrinfo,SDS_ID,3,name=TotalFrames,data=tframes
    hdf_sd_attrinfo,SDS_ID,4,name=frameoffset,data=offset
    hdf_sd_attrinfo,SDS_ID,5,name=time_UTC,data=img_t
    HDF_SD_GETDATA, sds_id(0),swir0,$
      COUNT=[nfov,nspec,nframes],$
      START=[0,0,0]
    HDF_SD_ENDACCESS, sds_id
    HDF_SD_END, sd_id
    help,swir0

;Average and trend darkfiles
    print,"Averaging darks"
    predark=fltarr(nfov,nspec)
    postdark=fltarr(nfov,nspec)
    predark(*,*)=0.0
    postdark(*,*)=0.0
    for i=0,255 do for j=0,nswir-1 do $
      predark(i,j)=mean(predark0(i,j,*))
    for i=0,255 do for j=0,nswir-1 do $
      postdark(i,j)=mean(postdark0(i,j,*))
    
;Trend dark values
    print,"Trending dark"
    dark=fltarr(nfov,nswir,nframes)
    dark_diff=postdark-predark

    if keyword_set(trend0) then begin
        for i=0,nframes-1 do $
          dark(*,*,i) = predark(*,*) + float(dark_diff(*,*)*(i+offset(0)))/$
          float((tframes(0)))
    endif else begin
        dce_frames=countframes(pre_t,post_t)
        img_offset=countframes(pre_t,img_t)
        print,pre_t," : ",img_t," : ",post_t
        print,dce_frames,img_offset
        if dce_frames eq 0 then $
          for i=0,nframes-1 do dark(*,*,i) = predark(*,*) else $
          for i=0,nframes-1 do $
          dark(*,*,i) = predark(*,*) + $
          float(dark_diff(*,*)*(i+offset(0)+img_offset))/dce_frames
    endelse
    print,offset(0),tframes(0)

;darksubtract
      swir0=swir0-dark
    
;Apply gains
    print,"Applying gains"
    if not keyword_set(gainfile) then $
      if not keyword_set(gsfc) then begin
        gainfile = 'HypGain_revC.dat.svf'
      endif else begin
        gainfile = '/u03/EO-1/data/Cals/Hyp/HypGain_revC.dat.svf'
      endelse

    if (findfile(gainfile) eq "") then begin
        print,"gainfile not found"
        return,0
    endif
   
    g=fltarr(256,242)
    openr,1,gainfile      
    readu,1,g
    close,1
    for i=0,nframes-1 do $
      swir0(*,*,i)=swir0(*,*,i)*g(*,70:*)

    swir0=round(swir0*80) 

;   Added May 8, 2002
    w=where(swir0 gt 32767,icount)
    if icount gt 0 then begin
        print,'*** INT overflow detected',icount        
        swir0(w)= 32767
    endif
    w=where(swir0 lt -32768,jcount)
    if jcount gt 0 then begin
        print,'*** INT underflow detected',jcount
        swir0(w)= -32768
    endif

    swir0=fix(swir0)
;write out results
    label=filename+"_swir.cal"
    if keyword_set(index) then $
      label=strcompress(label+"."+string(index,format='(i2.2)'),/remove_all)
    print,"output to: ",label
    sdout_id=hdf_sd_start(label,/create)
    sds_id=hdf_sd_create(sdout_id,label,[nfov,nspec,nframes],/INT)
    HDF_SD_ATTRSET,sds_ID,"Level0file",L0file
    HDF_SD_ATTRSET,sds_ID,"SmearCorrectedfile",sfile
    HDF_SD_ATTRSET,sds_ID,"Inputfile",imagefile
    HDF_SD_ATTRSET,sds_ID,"EchoRatiofile",rfile
    HDF_SD_ATTRSET,sds_ID,"HypGainfile",gainfile
    HDF_SD_ATTRSET,sds_ID,"L0_nFrames",tframes
    HDF_SD_ATTRSET,sds_ID,"frameoffset",offset
    HDF_SD_ATTRSET,sds_ID,time_UTC,img_t
    HDF_SD_ATTRSET,sds_ID,"INT overflow",icount
    HDF_SD_ATTRSET,sds_ID,"INT underflow",icount
    HDF_SD_ADDDATA,sds_id,swir0
    hdf_SD_ENDACCESS,sds_ID
    HDF_SD_END,sdout_id
    return,1
end






