;Routines for Hyperion Echo Correction
; Lawrence Ong
; Release: 0.0b March 22, 2002
; Revisions:
;       May 14, 2002: Added keyword switch for edc/gsfc cal
;                           files directories. Change return code.
;

forward_function RM_Echo

function FixEcho,filename,ratiofile=ratiofile,index=index,gsfc=gsfc
    nswir=172
    
    inputfile=filename+".smear"
    if keyword_set(index) then $
      inputfile=strcompress(inputfile+"."+string(index,format='(i2.2)'),/remove_all)
    print,"Reading: ",inputfile
    OPEN_HDF,inputfile,$
      arrsize,datasets,sds_id,sd_id,dsname,attrname,sds_name
    hdf_sd_attrinfo,SDS_ID,0,name=Level0file,data=L0file
    hdf_sd_attrinfo,SDS_ID,1,name=L0_nFrames,data=tframes
    hdf_sd_attrinfo,SDS_ID,2,name=frameoffset,data=offset
    hdf_sd_attrinfo,SDS_ID,3,name=time_UTC,data=t
    nfov=long(arrsize(0))
    nspec=long(arrsize(1))
    nframes=long(arrsize(2))
    if (nspec ne nswir) then begin
        print,"Incorrect number of channels"
        return,1
    endif	
;    if not keyword_set(nframes) then nframes=tframes
;    if not keyword_set(offset) then offset=0
    HDF_SD_GETDATA, sds_id(0),dat0,$
      COUNT=[nfov,nspec,nframes],$
      START=[0,0,0]
    HDF_SD_ENDACCESS, sds_id
    HDF_SD_END, sd_id  
    help,dat0

    print,"Fixing echo"

    if not keyword_set(ratiofile) then $
      if not keyword_set(gsfc) then begin
        ratiofile = '/sg1/sab1/pscar/Data/ratio_revB.dat'
      endif else begin
        ratiofile = '/u03/EO-1/data/Cals/Hyp/ratio_revB.dat'
      endelse

    if (findfile(ratiofile) eq "") then begin
        print,"echo ratio file not found"
        return,0
    endif
 
    dat0=rm_echo(dat0,ratiofile)

    label=filename+".echo"
    if keyword_set(index) then $
      label=strcompress(label+"."+string(index,format='(i2.2)'),/remove_all)
    print,"output to: ",label
    sdout_id=hdf_sd_start(label,/create)
    sds_id=hdf_sd_create(sdout_id,label,[nfov,nspec,nframes],/FLOAT)
    HDF_SD_ATTRSET,sds_ID,"Level0file",L0file
    HDF_SD_ATTRSET,sds_ID,"Inputfile",inputfile
    HDF_SD_ATTRSET,sds_ID,"Ratiofile",ratiofile
    HDF_SD_ATTRSET,sds_ID,"L0_nFrames",tframes
    HDF_SD_ATTRSET,sds_ID,"frameoffset",offset
    HDF_SD_ATTRSET,sds_ID,"time_UTC",t
    HDF_SD_ADDDATA,sds_id,dat0
    hdf_SD_ENDACCESS,sds_ID
    HDF_SD_END,sdout_id
    return,1
end
    

function rm_echo,img0,ratiofile
    a=size(img0)
    nfov=a(1)
    nspec=a(2)
    nframes=a(3)
    nhidden=4l
    echoOffset=11l
    img=fltarr(nfov+nhidden,nframes)

    if ((nspec ne 172) or (nfov ne 256))then begin
        print,"Incorrect Swir dimensions"
        print, "nspec =",nspec
        print, "nfov =",nfov
        return,0
    endif

    r=fltarr(nspec,nfov) 
    openr,1,ratiofile
    readu,1,r
    close,1

    istart=0
    iend=nspec-1

    nIfov=nfov+nhidden
    echoJ=intarr(nIfov,nframes)
    echoI=intarr(nIfov,nframes)
    index=0l
    echoindex=0l
    for j=nframes-1l,1l,-1l do for i=nIfov-1l,0l,-1l do begin
        index=j*nIfov+i
        echoIndex=index-(nIfov-echoOffset)
        if (echoIndex ge 0) then begin
            echoJ(i,j)=echoIndex/nIfov
            echoI(i,j)=echoIndex mod nIfov
        endif else begin
            stop
            echoJ(i,j)=-1
            echoI(i,j)=-1
        endelse        
    endfor
    echoj(*,0)=echoj(*,1)
    echoi(*,0)=echoi(*,1)
    echoAmount=fltarr(nspec)
    f=fltarr(nspec,4)
    for ispec=0,nspec-1 do $
      f(ispec,*)=[1.-0.959,1.-0.938,1.-0.971,1.-0.959]    
    r=[[r],[f]]
    img=fltarr(nspec,4,nframes)
    img0=transpose(img0,[1,0,2])
    img(*,*,*)=img0(*,245:248,*)
    img0=transpose([[img0],[img]],[1,0,2])
    

    for j=nframes-1l,0l,-1l do  for i=nIfov-1l,0l,-1l do begin
        echoAmount(*)=$
          (img0(echoI(i,j),*,echoJ(i,j)) * $
           r(*,echoi(i,j)) )
        img0(i,*,j)=img0(i,*,j)-echoAmount(*)
    endfor
    
    if (keyword_set(DEBUG)) then begin
        openw,1,"rm_echo_out.dat"
        writeu,1,img0
        close,1
    endif

    return,img0(0:255,*,*)
end

