;Routines for Hyperion Smear Correction
; Lawrence Ong
; Release: 0.0b March 22, 2002
; Revisions:
;
;  May  30, 2002 Lawrence Ong   Fixed getting image time from L0 attributes
;  June 11, 2002 Lawrence Ong   Re-fix getting image time from L0
;                               attributes, look in both global and
;                               dataset attributes



 
forward_function SWIRLeak

function FixSmear,filename,offset=offset,nframes=nframes,all=all,bitesize=bitesize

    nswir=172
    offset2swir=70
    print,"Reading: ",filename
    OPEN_HDF,filename,arrsize,datasets,sds_id,sd_id,dsname,attrname,sds_name

    attrindex = hdf_sd_attrfind(sd_id, 'time_UTC')
    if attrindex eq -1 then begin
        print,"!!Can't find image time in global attributes"
        t=0
    endif else $
      hdf_sd_attrinfo,SD_ID,attrindex,name=imgTime,data=t

    
    if attrindex eq -1 then begin
        attrindex = hdf_sd_attrfind(sds_id, 'time_UTC')
        if attrindex eq -1 then begin
            print,"!!Can't find image time in dataset attributes"
            t=0
        endif else $
          hdf_sd_attrinfo,SDS_ID,attrindex,name=imgTime,data=t
    endif

    nfov=long(arrsize(0))
    nspec=long(arrsize(1))
    tframes=long(arrsize(2))
    if keyword_set(all) then begin
        if not keyword_set(bitesize) then bitesize=1600
        offset=0
        nframes=bitesize
    endif else begin
        if not keyword_set(offset) then offset=0
        if not keyword_set(nframes) then nframes=tframes-offset
    endelse

    icount=0
    done=0

    repeat begin
        print,icount,offset,nframes
        HDF_SD_GETDATA, sds_id(0),dat0,$
          COUNT=[nfov,nswir,nframes],$
          START=[0,offset2swir,offset]
        HDF_SD_ENDACCESS, sds_id
        HDF_SD_END, sd_id
        help,dat0
        dat0=float(dat0)
        
        leak=SWIRLeak(dat0)
        
        icount=icount+1
        if keyword_set(all) then $
        label=strcompress(filename+".smear."+$
                          string(icount,format='(i2.2)'),/remove_all) else $
        label=strcompress(filename+".smear",/remove_all)
          
        print,"output to: ",label
        sdout_id=hdf_sd_start(label,/create)
        sds_id=hdf_sd_create(sdout_id,label,[nfov,nswir,nframes],/FLOAT)
        HDF_SD_ATTRSET,sds_ID,"Inputfile",filename
        HDF_SD_ATTRSET,sds_ID,"L0_nFrames",tframes
        HDF_SD_ATTRSET,sds_ID,"frameoffset",offset
        HDF_SD_ATTRSET,sds_ID,"time_UTC",t
        HDF_SD_ADDDATA,sds_id,(dat0-leak)
        hdf_SD_ENDACCESS,sds_ID
        HDF_SD_END,sdout_id
        if keyword_set(all) then begin
            offset=offset+nframes-1 < tframes
            nframes=tframes-offset < bitesize
            if nframes le 1 then done = 1 else $
              OPEN_HDF,filename,arrsize,datasets,$
              sds_id,sd_id,dsname,attrname,sds_name
        endif else done = 1
            
    endrep until done
    return,1
end

function SWIRLeak,dark,invert=invert
    print,"Calculating Spectral Leakage"
    a=size(dark)
    nsamples=a(1)
    nchannels=a(2)
    n_frames=a(3)

    coeff=dblarr(3,4)
    coeff(*,0)=[2.7426732e-9, -2.5982560e-6, 6.0993214e-2]
    coeff(*,1)=[2.5400892e-9, -3.0107557e-6, 5.8708724e-2]
    coeff(*,2)=[3.1079314e-9, -3.4234294e-6, 6.8951602e-2]
    coeff(*,3)=[3.2542773e-9, -4.7728721e-6, 6.7096899e-2]
    
;start smear correction
    leakage=fltarr(nsamples,nchannels,n_frames)
    if keyword_set(invert) then begin
        jstart=2
        jend=nchannels-1
        j_incr=1
        xz=-2        
    endif else begin
        jstart=nchannels-1-2
        jend=0
        j_incr=-1
        xz=2        
    endelse
    
    for j=jstart,jend,j_incr do $
      for k=0,nsamples-1 do begin
;choose odd or even spectral & fov pixels
        if (j/2 eq j/2.) and (k/2 eq k/2.) then os = 2 ; was 0
        if (j/2 eq j/2.) and (k/2 ne k/2.) then os = 3 ; was 2
        if (j/2 ne j/2.) and (k/2 ne k/2.) then os = 1 ; was 3
        if (j/2 ne j/2.) and (k/2 eq k/2.) then os = 0 ; was 1
	
        leakage(k,j,*)=coeff(2,os)*(dark(k,j+xz,*)) + $
          coeff(1,os)*dark(k,j+xz,*)^2 + $
          coeff(0,os)*dark(k,j+xz,*)^3
    endfor
    
    b=where(leakage lt 0,count)
    if count gt 0 then begin
        leakage(b)=0.0
        print,"Negative leakage detected"
    endif
    
    return,leakage
end


