;Routines for Hyperion Radiometric cals.
; Lawrence Ong
; Release: 0.0b March 22, 2002
; Revisions:
;         May   08, 2002 :  Added checks for INT over/under flows
;                           when rescaling: multiply by 80.
;                           Fixed error in L0_nFrames.
;         May   14, 2002 :  Added keyword switch for edc/gsfc cal
;                           files directories
;

function HypVnirCal,filename,predarkfile,postdarkfile,GainFile=GainFile,offset=offset,nframes=nframes,usepredark=usepredark,gsfc=gsfc
;get predark
    
    nvnir=70

;Get VNIR data
    print,"reading: ",filename
    OPEN_HDF,filename,$
      arrsize,datasets,sds_id,sd_id,dsname,attrname,sds_name
    nfov=long(arrsize(0))
    nspec=long(arrsize(1))
    tframes=long(arrsize(2))
    L0_nFrames=tframes
    if not keyword_set(offset) then offset=0
    if not keyword_set(nframes) then nframes=tframes-offset 
    HDF_SD_GETDATA, sds_id(0),vnir0,$
      COUNT=[nfov,nvnir,nframes],$
      START=[0,0,offset]
    HDF_SD_ENDACCESS, sds_id
    HDF_SD_END, sd_id
    vnir0=float(vnir0)
    help,vnir0

;Get Darkfile
    if keyword_set(usepredark) then begin
        print,"using Predark"
        darkfile=predarkfile
    endif else begin
        print,"using Postdark"
        darkfile=postdarkfile
    endelse

    print,"reading: ",darkfile
    OPEN_HDF,darkfile,$
      arrsize,datasets,sds_id,sd_id,dsname,attrname,sds_name
    nfov=long(arrsize(0))
    nspec=long(arrsize(1))
    tframes=long(arrsize(2))
    HDF_SD_GETDATA, sds_id(0),dark0,$
      COUNT=[nfov,nvnir,tframes],$
      START=[0,0,0]
    HDF_SD_ENDACCESS, sds_id
    HDF_SD_END, sd_id
    help,dark0

;Average and trend darkfiles
    print,"Averaging darks"
    dark=fltarr(nfov,nvnir)
    dark(*,*)=0.0
    for i=0,255 do for j=0,nvnir-1 do $
      dark(i,j)=mean(dark0(i,j,*))
    for i=0,nframes-1 do $
      vnir0(*,*,i)=vnir0(*,*,i)-dark(*,*)

;Apply gains
    print,"Applying gains"
    if not keyword_set(gainfile) then $
      if not keyword_set(gsfc) then begin
        gainfile = 'HypGain_revC.dat.svf'
      endif else begin
        gainfile = '/u03/EO-1/data/Cals/Hyp/HypGain_revC.dat.svf'
      endelse

    if (findfile(gainfile) eq "") then begin
        print,"gain file not found"
        return,0
    endif

    g=fltarr(256,242)
    openr,1,gainfile      
    readu,1,g
    close,1
    for i=0,nframes-1 do $
      vnir0(*,*,i)=vnir0(*,*,i)*g(*,0:nvnir-1)

    vnir0=round(vnir0*40)

;   Added May 8, 2002
    w=where(vnir0 gt 32767,icount)
    if icount gt 0 then begin
        print,'*** INT overflow detected',icount
        vnir0(w)= 32767
    endif
    w=where(vnir0 lt -32768,jcount)
    if jcount gt 0 then begin
        print,'*** INT underflow detected',jcount
        vnir0(w)= -32768
    endif

    vnir0=fix(vnir0)
;write out results
    label=filename+"_vnir.cal"
    print,"output to: ",label
    sdout_id=hdf_sd_start(label,/create)
    sds_id=hdf_sd_create(sd_id,label,[nfov,nvnir,nframes],/INT)
    HDF_SD_ATTRSET,sds_ID,"Inputfile",filename
    HDF_SD_ATTRSET,sds_ID,"Darkfile",darkfile
    HDF_SD_ATTRSET,sds_ID,"HypGainfile",gainfile
    HDF_SD_ATTRSET,sds_ID,"L0_nFrames",L0_nFrames
    HDF_SD_ATTRSET,sds_ID,"frameoffset",offset
    HDF_SD_ATTRSET,sds_ID,"INT overflow",icount
    HDF_SD_ATTRSET,sds_ID,"INT underflow",icount
    HDF_SD_ADDDATA,sds_id,vnir0
    hdf_SD_ENDACCESS,sds_ID
    HDF_SD_END,sdout_id
    return,1
end
