;Routine to combine calibrated Hyperion vnir and swir parts
; Lawrence Ong
; Release: 0.0b April 09, 2002
;
; Revisions:
;
;  May 8, 2002  Pat Scaramuzza  Reworked output attributes and added comments.
;  May 9, 2002  Lawrence Ong    Fixed vnir/swir frame count mismatch trap.
;                               Get new frame offset attribute.
;  May 23, 2002 Pat Scaramuzza  Added Flag Mask -- read in *_mask file and include mask in L1.
;  June 12, 2002 Pat Scaramuzza Final changes for production --
;                               Changed 'L1B1' suffix to 'L1R'
;                               Changed L1 file name and SD label to 'EO1H...'
;                               Changed SD attribute #2 from HIP to HYP, and deleted attribute #10
;                               Added ENVI header file creation
;                               Added keywords for spectralfile and bandwidthfile
;  June 13, 2002 Pat Scaramuzza Small bug fixes.
;

function hypWrap,prefix,pre,post,scene_ID=scene_ID,imgTime=imgTime,uframes=uframes,gsfc=gsfc,satlevel=satlevel,gainfile=gainfile,spectralfile=spectralfile,bandwidthfile=bandwidthfile

    ; First check to see if all the associated files are found.

    IF not keyword_set(gainfile) THEN BEGIN
       IF not keyword_set(gsfc) THEN BEGIN
          gainfile = 'HypGain_revC.dat.svf'
       ENDIF ELSE BEGIN
          gainfile = '/u03/EO-1/data/Cals/Hyp/HypGain_revC.dat'
       ENDELSE
    ENDIF

    IF (findfile(gainfile) eq "") THEN BEGIN
        print,"Gain file ("+gainfile+") not found"
        return,0
    ENDIF

    IF not keyword_set(spectralfile) THEN BEGIN
       IF not keyword_set(gsfc) THEN BEGIN
          spectralfile = 'SpectralL0_revA.1.dat
       ENDIF ELSE BEGIN
          spectralfile = '/u03/EO-1/data/Cals/Hyp/SpectralL0_revA.1.dat'
       ENDELSE
    ENDIF

    IF (findfile(spectralfile) eq "") THEN BEGIN
        print,"Spectral file ("+spectralfile+") not found"
        return,0
    ENDIF

    IF not keyword_set(bandwidthfile) THEN BEGIN
       IF not keyword_set(gsfc) THEN BEGIN
          bandwidthfile = 'BandwidthL0_revA.1.dat'
       ENDIF ELSE BEGIN
          bandwidthfile = '/u03/EO-1/data/Cals/Hyp/BandwidthL0_revA.1.dat'
       ENDELSE
    ENDIF

    IF (findfile(bandwidthfile) eq "") THEN BEGIN
        print,"Bandwidth file ("+bandwidthfile+") not found"
        return,0
    ENDIF

    ; Get attributes from the L0 file.
   
    OPEN_HDF,prefix,$
      arrsize,datasets,sds_id,sd_id,dsname,attrname,sds_name

    IF (keyword_set(scene_ID) eq 0) THEN BEGIN

       ; Only look for Scene_ID if it wasn't passed to this function.
       attrindex = hdf_sd_attrfind(sd_id, 'Scene_Id')
       IF (attrindex ne -1) THEN BEGIN
          hdf_sd_attrinfo, sd_id, attrindex, data=scene_ID 
       ENDIF ELSE BEGIN

          ; It's not in the L0.  Ask the user.
          print,'Scene ID not found in L0 attributes. Found: ', LBL_SceneID
          sceneID=""
          print,'Enter Scene_ID: '
          read,scene_ID
       ENDELSE
    ENDIF
    print, "SceneID : ", scene_ID

    attrindex = hdf_sd_attrfind(sd_id, 'File Type')
    ; Previous versions of the DPS did not produce the File Type attribute.
    IF (attrindex ne -1) THEN BEGIN
       hdf_sd_attrinfo, sd_id, attrindex, data=l0_filetype 
    ENDIF ELSE BEGIN
       l0_filetype = "Ground image"
    ENDELSE
    print, "File Type : ", l0_filetype
    attrindex = hdf_sd_attrfind(sd_id, 'File Byte Order')
    hdf_sd_attrinfo, sd_id, attrindex, data=l0_filebyteorder 
    print, "File Byte Order : ", l0_filebyteorder
    attrindex = hdf_sd_attrfind(sd_id, 'Interleave Format')
    hdf_sd_attrinfo, sd_id, attrindex, data=l0_interleave 
    print, "Interleave Format : ", l0_interleave
    attrindex = hdf_sd_attrfind(sd_id, 'Frame Number Range')
    hdf_sd_attrinfo, sd_id, attrindex, data=l0_framerange 
    print, "Frame Number Range : ", l0_framerange
    attrindex = hdf_sd_attrfind(sd_id, 'Number of Pixels with Missing Data Flag')
    hdf_sd_attrinfo, sd_id, attrindex, data=l0_misspixflags 
    print, "No. Pixels with Missing Data Flags : ", l0_misspixflags

    ; Close the L0 file.

    HDF_SD_ENDACCESS, sds_id
    HDF_SD_END, sd_id

    ; Get the Flag Mask.

    OPEN_HDF,prefix+'_mask',$
      satsize,datasets,sds_id,sd_id,dsname,attrname,sds_name
    HDF_SD_GETDATA, sds_id, satdata
    HDF_SD_ENDACCESS, sds_id
    HDF_SD_END, sd_id

    ; Get the data from the VNIR file.
    
    vnirfile=prefix+"_vnir.cal"
    OPEN_HDF,vnirfile,$
      arrsize,datasets,sds_id,sd_id,dsname,attrname,sds_name
    nfov=long(arrsize(0))
    nspec=long(arrsize(1))
    vnir_nframes=long(arrsize(2))
    HDF_SD_GETDATA, sds_id(0),vnir
    HDF_SD_ENDACCESS, sds_id
    HDF_SD_END, sd_id

    help,vnir
    help, vnir_nframes

    ; Go through the SWIR files (*_swir.cal.*).

    error=0
    swirfile=findfile(prefix+"_swir.cal*",count=nswirfiles)
    offset=0
    for ifile=0,nswirfiles-1 do begin
    
        OPEN_HDF,swirfile(ifile),$
          arrsize,datasets,sds_id,sd_id,dsname,attrname,sds_name
        nfov=long(arrsize(0))
        nspec=long(arrsize(1))
        tframes=long(arrsize(2))

        ; Grab attributes.

        attrindex = hdf_sd_attrfind(sds_id, 'Level0file')
        hdf_sd_attrinfo, sds_id, attrindex, data=L0file
        print,'   Level0file : ',L0file      

        attrindex = hdf_sd_attrfind(sds_id, 'EchoRatiofile')
        hdf_sd_attrinfo, sds_id, attrindex, data=rfile
        print,'   EchoRatiofile : ',rfile    

        attrindex = hdf_sd_attrfind(sds_id, 'HypGainfile')
        hdf_sd_attrinfo, sds_id, attrindex, data=l0gainfile
        print,'   HypGainfile : ',l0gainfile   

        attrindex = hdf_sd_attrfind(sds_id, 'L0_nFrames')
        hdf_sd_attrinfo, sds_id, attrindex, data=L0_nframes
        print,'   L0_nFrames : ',L0_nframes 

        ; New attribute 05/09/02
        attrindex = hdf_sd_attrfind(sds_id, 'frameoffset')
        hdf_sd_attrinfo, sds_id, attrindex, data=SwirFrameOffset
        print,'   SwirFrameOffset : ',SwirFrameOffset

        attrindex = hdf_sd_attrfind(sds_id, 'time_UTC')
        hdf_sd_attrinfo, sds_id, attrindex, data=img_t         
        if keyword_set(imgtime) then img_t = imgtime
        print,'   time_UTC : ',img_t         
        print,'  size(time_UTC) : ', size(img_t)

        ; Perform some sanity checks on the number of frames.
 
        if (L0_nFrames eq -1) then begin
            print,"Attr: L0_nFrames not found"
            error=error+1
        endif

        ; Use the number of frames passed to this function, if any.
        if keyword_set(uframes) then L0_nFrames=uframes
        
        if error gt 0 then begin            
            ; There was a problem -- do not create the L1 product.
            HDF_SD_ENDACCESS, sds_id
            HDF_SD_END, sd_id
            return,0
        endif

        ; Get the SWIR data.
        HDF_SD_GETDATA, sds_id(0),dat

        ; Close the SWIR file.
        HDF_SD_ENDACCESS, sds_id
        HDF_SD_END, sd_id

        help,dat

        IF (ifile gt 0) THEN BEGIN
            dat=dat(*,*,1:*)
            tframes=tframes-1
            tswirframes=tswirframes+tframes
            print, 'Tframes : ', tframes, '   TSWIRFrames : ', tswirframes
        ENDIF ELSE BEGIN            
            tswirframes=tframes

            ; Create the L1 product label.
            label=strcompress(strmid(scene_ID,0,21)+'.L1R',/remove_all)
            label = strmid(label,0,3)+'H'+strmid(label,3)
            print, 'Level 1 label:  ', label

            ; Create the L1 product file.
            print,"Writing to: ",label
            sdout_id=hdf_sd_start(label,/create)

            ; Set the Global Attributes.
            HDF_SD_ATTRSET,sdout_id,"File Type",l0_filetype, /STRING
            HDF_SD_ATTRSET,sdout_id,"L1 File Generated By",'HYP version 1.1', /STRING
            HDF_SD_ATTRSET,sdout_id,"Time of L1 File Generation", strmid(systime(/utc), 4), /STRING
            HDF_SD_ATTRSET,sdout_id,"EchoRatiofile",rfile, /STRING
            HDF_SD_ATTRSET,sdout_id,"Calibration Parameter File",l0gainfile, /STRING
            HDF_SD_ATTRSET,sdout_id,"ImageStartTime",img_t, /STRING
            HDF_SD_ATTRSET,sdout_id,"File Byte Order",l0_filebyteorder, /STRING
            HDF_SD_ATTRSET,sdout_id,"Interleave Format",l0_interleave, /STRING
            HDF_SD_ATTRSET,sdout_id,"Frame Number Range",l0_framerange, /LONG
            ; HDF_SD_ATTRSET,sdout_id,"Number of Pixels with Missing Data Flag (L0)",l0_misspixflags, /LONG
            HDF_SD_ATTRSET,sdout_id,"Scene_ID",scene_ID, /STRING
            HDF_SD_ATTRSET,sdout_id,"Level0 image filename",L0file, /STRING
            HDF_SD_ATTRSET,sdout_id,"Level0 Pre-Dark filename",pre, /STRING
            HDF_SD_ATTRSET,sdout_id,"Level0 Post-Dark filename",post, /STRING
            HDF_SD_ATTRSET,sdout_id,"Dark Offset method [Trended | Predark | Postdark]",'Trended', /STRING

            ; Put the VNIR Data first in the file.

            print,"Adding: ",vnirfile
            sdsout_id=hdf_sd_create(sdout_id,label,$
                                    [nfov,242,vnir_nframes+1],/INT)
            HDF_SD_ADDDATA,sdsout_id,vnir,count=[256,70,vnir_nframes]

            ; Set the local Attributes for this SDS.

            HDF_SD_ATTRSET,sdsout_id,"Number of Cross Track Pixels", 256, /LONG
            HDF_SD_ATTRSET,sdsout_id,"Number of Along Track Pixels", L0_nframes+1, /LONG
            HDF_SD_ATTRSET,sdsout_id,"Number of Bands", 242, /LONG
            HDF_SD_ATTRSET,sdsout_id,"VNIR Scaling Factor","w/m^2/sr/micron * 40", /STRING
            HDF_SD_ATTRSET,sdsout_id,"SWIR Scaling Factor","w/m^2/sr/micron * 80", /STRING
            HDF_SD_ATTRSET,sdsout_id,"Data units","w/m^2/sr/micron", /STRING

        ENDELSE

        ; Put in the SWIR data.
        print,"Adding: ",swirfile(ifile)            
        HDF_SD_ADDDATA,sdsout_id,dat(1:128,*,*),$
          count=[128,172,tframes],start=[0,70,offset]
        HDF_SD_ADDDATA,sdsout_id,dat(129:255,*,*),$
          count=[127,172,tframes],start=[128,70,offset+1]
        offset=offset+tframes

    ENDFOR

    zeros=intarr(1,172,tswirframes+1)
    zeros(*,*,*)=0
    HDF_SD_ADDDATA,sdsout_id,zeros,$
      count=[1,172,tswirframes],start=[255,70,0]


    if vnir_nframes ne tswirframes then begin
        print,"WARNING: Vnir and Swir nframes do not match"
        print,vnir_nframes,tframes
        stop
    endif

    HDF_SD_ENDACCESS,sdsout_id

    ; Create the band description datasets.

    ; Spectral Center Wavelengths.
    openr, lun, spectralfile, /get_lun
    util = replicate(' ', 256)
    readf, lun, util
    free_lun, lun
    dat = fltarr(256, 242)
    FOR i=0, 255 DO dat[i, 0:*] = FLOAT(STRSPLIT(util[i], ' ', /extract))

    specmeans = fltarr(242)
    FOR i=0,241 DO specmeans[i] = MEAN(dat[*,i])

    sdsout_id = HDF_SD_CREATE(sdout_id, 'Spectral Center Wavelengths', [ 256, 242 ], /FLOAT)
    HDF_SD_ADDDATA, sdsout_id, dat
    HDF_SD_ATTRSET, sdsout_id, "Data Units","nanometers", /STRING
    HDF_SD_ENDACCESS, sdsout_id

    ; Spectral Bandwidths.
    openr, lun, bandwidthfile, /get_lun
    util = replicate(' ', 256)
    readf, lun, util
    free_lun, lun
    dat = fltarr(256, 242)
    FOR i=0, 255 DO dat[i, 0:*] = FLOAT(STRSPLIT(util[i], ' ', /extract))

    widthmeans = fltarr(242)
    FOR i=0,241 DO widthmeans[i] = MEAN(dat[*,i])

    sdsout_id = HDF_SD_CREATE(sdout_id, 'Spectral Bandwidths', [ 256, 242 ], /FLOAT)
    HDF_SD_ADDDATA, sdsout_id, dat
    HDF_SD_ATTRSET, sdsout_id, "Data Units","nanometers", /STRING
    HDF_SD_ENDACCESS, sdsout_id

    ; Gain Coefficients.
    openr, lun, gainfile, /get_lun
    dat = fltarr(256, 242)
    readu, lun, dat
    free_lun, lun

    sdsout_id = HDF_SD_CREATE(sdout_id, 'Gain Coefficients', [ 256, 242 ], /FLOAT)
    HDF_SD_ADDDATA, sdsout_id, dat
    HDF_SD_ATTRSET, sdsout_id, "Data Units","w/m^2/sr/micron", /STRING
    HDF_SD_ENDACCESS, sdsout_id

    ; Flag Mask.

    sdsout_id = HDF_SD_CREATE(sdout_id, 'Flag Mask', satsize, /BYTE)
    HDF_SD_ADDDATA, sdsout_id, satdata
    HDF_SD_ENDACCESS, sdsout_id

    HDF_SD_END,sdout_id

    ; Create an ENVI header file.
    print, 'Creating ENVI header'
    STRPUT, label, 'hdr', STRLEN(label)-3
    openw, lun, label, /get_lun
    printf, lun, 'ENVI'
    printf, lun, 'description = { Hyperion L1 Data Product ['+systime(/utc)+']}'
    printf, lun, 'samples = ', 256
    printf, lun, 'lines = ', L0_nframes+1
    printf, lun, 'bands = ', 242
    printf, lun, 'header offset = 0'
    printf, lun, 'file type = HDF Scientific Data'
    printf, lun, 'data type = 2'
    printf, lun, 'interleave = bil'
    printf, lun, 'sensor type = Unknown'
    printf, lun, 'byte order = 1'
    printf, lun, 'read procedures = { hdf_read_spatial, hdf_read_spectral}'
    printf, lun, 'subset procedure = hdf_read_scroll'

    printf, lun, 'band names = {'
    util = '(256x242x'+STRTRIM(L0_nframes+1,2)+')' 
    FOR i=1,241 DO printf, lun, util+':Hyperion L1: Band '+STRTRIM(i,2)+','
    printf, lun, util+':Hyperion L1: Band 242}'

    printf, lun, 'wavelength = {'
    printf, lun, format='(8(f7.2, ", "))', specmeans[0:239]
    printf, lun, format='(f7.2, ", ", f7.2, "}")', specmeans[240], specmeans[241]

    printf, lun, 'fwhm = {'
    printf, lun, format='(8(f7.4, ", "))', widthmeans[0:239]
    printf, lun, format='(f7.4, ", ", f7.4, "}")', widthmeans[240], widthmeans[241]

    free_lun, lun

    return,1
end


