; Searches current directory for hyperion L0 files
; and executes routines to process to L1.
;
; Lawrence Ong
; Release: 0.0b May 14, 2002
;
; Revisions:
;
; Requires: IDL v5.5
;           open_hdf.pro
;           HypCal.pro
;           HypEchoCorrect.pro
;           HypSmearCorrect.pro
;           HypVnirCal.pro
;           HypWrap.pro
;           countFrames.pro
;
pro go,gsfc=gsfc,bitesize=bitesize

    ; check for input files
    file=findfile("*.L0",COUNT=nfiles)
    post1=0
    post2=0
    image=0
    predark=0

    if keyword_set(bitesize) then framesPerBite=float(bitesize) $
      else framesPerBite=1600.

    for ifile=0,nfiles-1 do begin
        OPEN_HDF,file(ifile), $
          arrsize,datasets,sds_id,sd_id,dsname,attrname,sds_name
        hdf_sd_attrinfo,SD_ID,16,name=Lbl,data=datatype
        if lbl eq 'Data Stream' then begin
            case datatype of
                'Pre-dark': begin
                    prefile=file(ifile)
                    print,'Found Pre-dark: ',file(ifile)
                    predark=1
                end
                'Image': begin
                    L0=file(ifile)
                    print,'Found Image: ',file(ifile)
                    hdf_sd_attrinfo,SD_ID,5,name=lbl,data=nframes
                    image=1
                end
                'Post-dark 1': begin
                    Postfile1=file(ifile)
                    print,'Found Post-dark 1: ',file(ifile)
                    post1=1
                end
                'Post-dark 2': begin
                    Postfile2=file(ifile)
                    print,'Found Post-dark 2: ',file(ifile)
                    post2=1
                end
                else : $
                    print,'Found ',lbl,': ',datatype
            endcase
        ENDIF
        HDF_SD_ENDACCESS, sds_id
        HDF_SD_END, sd_id
    endfor

    if not image then begin
        print,'No image found'
        return
    endif

    nindex=ceil(nframes(0)/framesPerBite)
    post=""

    if post1 then post = postfile1 $
    else begin
        if post2 then post=postfile2    
    endelse

    pre=post
    if predark then begin
        pre=prefile
    endif else begin
        if post1 or post2 then $
          pre=post else begin
            print,'No Darks found'
            return
        endelse
    endelse    

    print,'   Image:    ',L0
    print,'   Predark:  ',pre
    print,'   Postdark: ',post
    
    ; Start process
    print,'---Vnir Cal---'
    if not HypVnirCal(L0,pre,post,gsfc=gsfc) then stop
    print,'---fixsmear predark---'
    if not fixsmear(pre) then stop
    print,'---fixecho predark---'
    if not fixecho(pre,gsfc=gsfc) then stop
    print,'---fixsmear postdark---'
    if not fixsmear(post) then stop
    print,'---fixecho predark---'
    if not fixecho(post,gsfc=gsfc) then stop
    print,'---fixsmear image---'
    if not fixsmear(L0,/all,bitesize=framesPerBite) then stop
    for index=1,nindex do $
        if not fixecho(L0,index=index,gsfc=gsfc) then stop
    print,'---fixecho image---'
    for index=1,nindex do $
        if not HypSwirCal(L0,pre,post,index=index,gsfc=gsfc) then stop
    print,'---HypWrap---'
    if not HypWrap(L0,pre,post,gsfc=gsfc) then stop
    print,'DONE'
    return
end




