#!/bin/bash
#
# daily_checks.sh
#
# This script checks several things and emails the results.
#

#
# DEFINE SHELL VARIABLES AND REMOVE THE OLD MESSAGE FILE.
#

RUN_DIR=/home/eo1/bin/daily
LOG_DIR=/home/eo1
MSG=$RUN_DIR/msg.txt
MAIL_TO="kim.rinehart.ctr@usgs.gov,michael.neiers.ctr@usgs.gov,katherine.goodale.ctr@usgs.gov"
rm $MSG > /dev/null 2>&1


#
# SEE IF EO1 HAS ANY MAIL THAT NEEDS TO BE READ
#

if [ -s /var/mail/eo1 ]; then
  echo -e "*************************             *************************" >> $MSG
  echo -e "************************* MAIL EXISTS *************************" >> $MSG
  echo -e "*************************             *************************" >> $MSG
else
  echo -e "*************************         *************************" >> $MSG
  echo -e "************************* NO MAIL *************************" >> $MSG
  echo -e "*************************         *************************" >> $MSG
fi


#
# SEND THE LOGS THEN MOVE THEM TO THE ARCHIVE
#
#  WE ONLY WANT TO DO THIS IF THE LEVEL 0 PROCESSING IS **NOT** RUNNING
#  WE CHECK TO SEE IF THE *LOCK* FILE EXISTS AND JUST WAIT UNTIL IT'S GONE
#

while [ -e /home/eo1/ingest/lock ]
do
  sleep 2
done

echo -e "\n*************************               *************************" >> $MSG
echo -e "************************* EO1INGEST.LOG *************************" >> $MSG
echo -e "*************************               *************************\n" >> $MSG
cat $LOG_DIR/eo1ingest.log  >> $MSG
cat $LOG_DIR/eo1ingest.log >> $LOG_DIR/old_logs/eo1ingest.log
cat /dev/null > $LOG_DIR/eo1ingest.log

echo -e "\n*************************            *************************" >> $MSG
echo -e "************************* INGEST.OUT *************************" >> $MSG
echo -e "*************************            *************************\n" >> $MSG
cat $LOG_DIR/ingest/ingest.out | egrep -v '[AEH]XXLIB|FOOT_PRINT|Hyperion imagery from' >> $MSG
cat $LOG_DIR/ingest/ingest.out >> $LOG_DIR/old_logs/ingest.out
cat /dev/null > $LOG_DIR/ingest/ingest.out

echo -e "\n*************************            *************************" >> $MSG
echo -e "************************* INGEST.ERR *************************" >> $MSG
echo -e "*************************            *************************\n" >> $MSG
cat $LOG_DIR/ingest/ingest.err | /usr/xpg4/bin/grep -v -f $RUN_DIR/grepfile >> $MSG
cat $LOG_DIR/ingest/ingest.err | /usr/xpg4/bin/grep -v -f $RUN_DIR/grepfile >> $LOG_DIR/old_logs/ingest.err
cat /dev/null > $LOG_DIR/ingest/ingest.err

# First of the month, rename the old logs
if test `date +%d` -eq 1
then
	suffix=`date +%Y%m -d yesterday`
	for log in $LOG_DIR/old_logs/eo1ingest.log $LOG_DIR/old_logs/ingest.out $LOG_DIR/old_logs/ingest.err
	do
		mv -i $log $log.$suffix
	done
fi


#
# LOOK FOR DATABASE RECORDS THAT NEED TO BE REGENERATED BECAUSE OF MISSING COORDINATES
# OR LOOK ANGLES
#

echo -e "\n****                                                  ****" >> $MSG
echo -e "**** MISSING ALI OR HYP DB COORDINATES OR LOOK ANGLES ****" >> $MSG
echo -e "****               IN THE LAST 45 DAYS                ****" >> $MSG
echo -e "****                                                  ****\n" >> $MSG
$RUN_DIR/regen.pl >> $MSG


#
# LOOK FOR MISSING DATA
#

echo -e "\n***************                                     ***************" >> $MSG
echo -e "*************** MISSING L0 DATA PER META/DAILY FILE ***************" >> $MSG
echo -e "***************      (FOR FULL WEEK 2 WKS AGO)      ***************" >> $MSG
echo -e "***************                                     ***************\n" >> $MSG
$RUN_DIR/L0datacheck.pl -v >> $MSG 2>&1


#
# SEND THE MESSAGE FILE
#

/bin/mailx -s "eo1 daily" $MAIL_TO < $MSG > /dev/null 2>&1
