package eo1;
use strict;
use vars qw (@ISA @EXPORT);
use Exporter;
@ISA = qw(Exporter);
@EXPORT = qw(readindexfile relocate);

# Read an index file and build the list of files and sizes.
sub readindexfile
	{
	my ($toc_file) = @_;
	my @filelist;			# list of files on the tape, from the 00index?????? file
	my %filesize;			# sizes (in bytes) of the files in @filelist
	open(TOC, "$toc_file") || return undef;
	my $currentdir = "meta";
	while (<TOC>)
		{
		chomp;
		next if m/^d/;
		if (m/^[-rwx]+\s+\d+\s+\S+\s+\S+\s+(\d+) [A-Z][a-z][a-z] [ \d]\d \d\d:\d\d (.*)/)
			{
			my $file = "$currentdir/$2";
			my $size = $1;
			# The deflate_hyp file is in the hyp directory, but this is not
			# obvious from the index file (at least in the first two tapes).
			$file = "hyp/deflate_hyp" if ($file =~ m/deflate_hyp$/);
			$filesize{$file} = $size;
			push(@filelist, $file);
			}
		elsif (m/(.*):$/)
			{
			$currentdir = $1;
			}
		}
	close(TOC);
	return (\@filelist, \%filesize);
	}

# Give a tape file name, return it's disk file name
sub relocate
	{
	my ($old, $filelist) = @_;
	my $new;
	my $file = $old;
	my $filetype = 2;	# 1 for data, 2 for metadata
	$file =~ s".*/"";	# strip off the directory parts
	# Look for a scene ID:
	if ($old =~ m/(EO1\d\d\d\d\d\d\d\d\d\d\d\d\d[01][01][012][NPKLSDAIX][FOPQSZ0-9])/)
		{
		my $sceneid = $1;
		$new = substr($sceneid, 9, 4) . "/" .  substr($sceneid, 13, 3);
		if ($old =~ m"(ali|hyp|lac)/")
			{
			$new = "$new/$1/" . get_ee_sceneid($old, @$filelist) . "/$file";
			$filetype = 1;
			}
		else
			{
			$new = "$new/meta/$file";
			}
		}
	elsif ($old =~ m/0event_(\d\d\d\d)(\d\d\d)\.(rpt|txt)/)
		{
		$new = "$1/$2/$file";
		}
	# Look for something like year and day-of-year.
	# Note that this won't work after 2009, but this is supposed to be a one-year mission.
	elsif (($old =~ m/(20\d\d)([0-3]\d\d)/) || ($old =~ m/(20\d\d)_([0-3]\d\d)/))
		{
		$new = "$1/$2/meta/$file";
		}
	else
		{
		$new = "meta/$file";
		}
	return ($new, $filetype);
	}

# Given a data file name, return the EE entity ID by matching the scene
# ID with a 00meta*.DAT file.  For example, on tape EO1L23, file
# hyp/20033100911_PF1/EO11740732003310110PX/EO12003310_448F448E_r1_PF1_01.L0
# returns # EO1H1740732003310110PX_PF1_01 because there is a
# 00meta_EO11740732003310110PX_PF1_01.DAT file on the tape.
# because
sub get_ee_sceneid
	{
	my $file = shift;
	my @candidates;
	if ($file =~ m"(ali|hyp|lac)/[^/]*_?(...)(_\d\d)?/(EO1\d\d\d\d\d\d\d\d\d\d\d\d\d.....)/")
		{
		my ($inst, $recsta, $sceneid) = ($1, $2, $4);
		$inst =~ s/ali/A/ || $inst =~ s/hyp/H/  || $inst =~ s/lac/L/;
		foreach my $f (@_)
			{
			next unless $f =~ m/00meta_EO1(..................)_(...)_(\d\d).DAT/;
			if (("EO1$1" eq $sceneid) && ($2 eq $recsta))
				{
				push(@candidates, "EO1$inst$1_$2_$3");
				}
			}
		}
	print STDERR "Ambiguous entity ID for $file: @candidates\n" if @candidates > 1;
	print STDERR "No entity ID for $file\n" if @candidates < 1;
	return $candidates[0];
	}
1;
