/*
 * Read the HDF attributes from an EO-1 data file and output
 * metadata fields.
 */
#include "mfhdf.h"
#include <unistd.h>

char *getattr(char *attr_name, int32 sd_id);
int All = 0;

main(int argc, char **argv ) 
	{
	int option;
	int i;
	int verbose = 0;
	while ((option = getopt(argc, argv, "av")) != -1)
		{
		switch (option)
			{
			case 'a':
				All = 1;
				break;
			case 'v':
				verbose = 1;
				break;
			}
		}
	for (i = optind; i < argc; i++)
		eo1info(argv[i], verbose);
	exit(0);
	}
eo1info(char *argv, int verbose)
	{
	int32 sd_id, num_datasets, num_attr;
	int32 sds_index;
	int32 sds_id;
	char *buffer;

	if (verbose)
		printf("File: %s\n", argv);
	sd_id = SDstart(argv, DFACC_READ);
	if (sd_id == FAIL)
		{
		fprintf (stderr, "SDstart failed.\n");
		HEprint (stderr, 0);
		exit (1);
		}

	if (buffer = getattr("Scene_Id", sd_id))
		{
		if (verbose)
			printf("Scene_Id: %s\n", buffer);
		else
			printf("%s: %s", argv, buffer);
		}
	if (verbose)
		{
		if (buffer = getattr("Target_Description", sd_id))
			printf("Target_Description: %s\n", buffer);
		}
	if (buffer = getattr("Data Stream", sd_id))
		{
		if (verbose)
			printf("Data Stream: %s\n", buffer);
		else
			printf(" %s", buffer);
		}
	if (buffer = getattr("time_UTC", sd_id))
		{
		if (verbose)
			printf("time_UTC: %s\n", buffer);
		else
			printf(" %s", buffer);
		}
	if (SDfileinfo(sd_id, &num_datasets, &num_attr) == FAIL)
		{
		fprintf(stderr, "SDfileinfo failed\n");
		exit(1);
		}
	for (sds_index = 0; sds_index < num_datasets; sds_index++)
		{
		sds_id = SDselect(sd_id, sds_index);
		if (sds_id == FAIL)
			{
			fprintf(stderr, "SDselect(%d, %d) failed\n", sd_id, sds_index);
			exit(1);
			}
		if (buffer = getattr("Dataset Type", sds_id))
			{
			if (!All && strncmp(buffer, "LEVEL0", 6))
				continue;
			if (verbose)
				printf("Dataset Type: %s\n", buffer);
			}
		if (buffer = getattr("Data Gate Open Time", sds_id))
			if (verbose)
				printf("Data Gate Open Time: %s\n", buffer);
			else
				printf(" %s", buffer);
		if (buffer = getattr("Data Gate Close Time", sds_id))
			if (verbose)
				printf("Data Gate Close Time: %s\n", buffer);
			else
				printf(" %s", buffer);
		}
	if (SDend(sd_id) == FAIL)
		{
		fprintf (stderr, "SDend failed.\n");
		HEprint (stderr, 0);
		exit (1);
		}
	printf("\n");
	}


char *getattr(char *attr_name, int32 sd_id)
	{
	int32 attr_index, num_type, count, status; 
	char name[64];
	char *buffer;

	/* Find the file attribute */
	attr_index = SDfindattr(sd_id, attr_name);
	if (attr_index == FAIL)
		return (char *)0;

	/* Get information about the file attribute. */
	status = SDattrinfo(sd_id, attr_index, name, &num_type, &count);
	if (status == FAIL)
		{
		fprintf (stderr, "SDattrinfo failed.\n");
		HEprint (stderr, 0);
		return (char *)0;
		}
	buffer = (char *)malloc(count+1);
	if (!buffer)
		{
		perror("out of memory");
		exit(1);
		}

	/* Read the attribute data. */
	status = SDreadattr(sd_id, attr_index, buffer);
	if (status == FAIL)
		{
		fprintf (stderr, "SDreadattr failed.\n");
		HEprint (stderr, 0);
		return (char *)0;
		}
	else
		{
		return buffer;
		}
	}
