#!/usr/bin/perl -I/usr/local/lib/perl5/site_perl
use strict;
BEGIN {
	my $dir = $0;
	$dir =~ s"/[^/]*$"";
	push(@INC, $dir) if ($dir);
}
use vars qw($Usage $Version $opt_d $opt_l $opt_s $opt_T $opt_n $opt_c $opt_D $opt_a
	);
use Fcntl;
$Version = "1.25 (November 3, 2015)";
use Getopt::Std;
use Digest::MD5;
getopts("d:l:Ts:nc:a:") || die(<<END_USAGE);
Usage: $0 [-d dir] [-l file] [-T] [-s c] [-n] [-c contactid] [-D silodir] [-a archivedir]

Options:
	-d dir		Use "dir" as the working directory (default
			/home/eo1).
	-l file		Use "file" as the log file (default: eo1ingest.log)
	-T		test mode
	-s c		set the field separator character to "c", default is "|"
	-n		close STDIN, STDOUT, STDERR, fork and exit.  This
			allows the script to run in the background if started
			from a remote system.
	-c contactid	use "contactid" in generating the tape locations.
			An example would be 20042980313_LGS.  This is normally
			the name of the directory on the DPS corresponding
			to a single downlink.  There is no default.
	-D silodir	Use "silodir" as the root directory on the
			silo (default /hsm/lta4/eo1).
	-a archivedir	Write a list of files sent to the silo into
			a text file in this directory for the archive
			program to process (default /home/ops/EO1_ingest)
END_USAGE
$opt_d ||= "/home/eo1";
$opt_D ||= "/hsm/lta4/eo1";
$opt_l ||= "eo1ingest.log";
$opt_s ||= "|";
$opt_a ||= "/home/ops/EO1_ingest";
$ENV{PATH} = "$opt_d/bin:/usr/local/bin:$ENV{PATH}";
push(@INC, "$opt_d/bin");

use eo1;
use reportfiles;
require "gmt.pl";
my $ingestdir = "$opt_d/ingest";
if ($opt_n)
	{
	my $pid = fork();
	exit(0) if ($pid);
	if (!defined($pid))
		{
		print STDERR "fork: $!";
		exit(1);
		}
	close(STDIN);
	open(STDIN, "</dev/null");
	close(STDOUT);
	open(STDOUT, ">>$ingestdir/ingest.out");
	close(STDERR);
	open(STDERR, ">>$ingestdir/ingest.err");
	}
dirlock($ingestdir);
print STDOUT "Start Up -- ", &timestamp, "\n";
print STDERR "Start Up -- ", &timestamp, "\n";
umask(002);	# Make sure everything is group-writeable

execute("fixrecstation.pl");	# change USAK03 station ID to AK3 so ingest and level-1 processing will work

my $now = time();
&log("eo1ingest_auto Started pid $$");
opendir(INGEST, $ingestdir) || fatal_error("$ingestdir: $!");
my $file;
my %ali;	# arrays of files by scene ID
my %hyp;	# hash of structures
my @aliscenes;	# ALI scenes for which we need to generate browse & metadata.
my @hypscenes;	# Hyperion scenes for which we need to generate browse & metadata.
my %aliexclusions = readexclusions("$opt_d/inventory/exclusions.ali");
my %hypexclusions = readexclusions("$opt_d/inventory/exclusions.hyp");
my %subdirectories;	# subdirectories of the ingest directory that should be removed
my @indexfiles;	# 00indexEO1??? files, to be processed last
my @silo_archive;	# list of files written to silo, sent to archive request

my @ali_fieldnames = qw(entity_id orbit_path orbit_row acquisition_date
	cloud_cover quality scene_status rec_station special_request
	scene_start_time scene_center_time scene_stop_time sun_azimuth
	sun_elevation data_type scene_center_lat scene_center_long
	ul_corner_lat ul_corner_long ur_corner_lat ur_corner_long
	ll_corner_lat ll_corner_long lr_corner_lat lr_corner_long
	day_or_night browse_availability browse_image_file_path
	working_dlt_id working_dlt_location archive_dlt_id
	archive_dlt_location raw_dlt_id raw_dlt_location cal_file_used
	file_number directory_name restriction_comments look_angle inclination
	hsm_path file_size num_files);

my @hyp_fieldnames = qw(entity_id orbit_path orbit_row acquisition_date
	cloud_cover quality scene_status rec_station special_request
	scene_start_time scene_center_time scene_stop_time
	sun_azimuth sun_elevation data_type processing_level
	scene_center_lat scene_center_long ul_corner_lat
	ul_corner_long ur_corner_lat ur_corner_long ll_corner_lat
	ll_corner_long lr_corner_lat lr_corner_long day_or_night
	browse_availability browse_image_file_path working_dlt_id
	working_dlt_location archive_dlt_id archive_dlt_location
	working_level1_id working_level1_location archive_level1_id
	archive_level1_location raw_dlt_id raw_dlt_location cal_file_used
	file_number directory_name restriction_comments look_angle inclination
	hsm_path file_size num_files);

sub save_ali_data_file
	{
	my ($file, $station_contact) = @_;
	my $info = `eo1info $ingestdir/$file`;
	chomp $info;
	my ($fname, $sceneid, $ts1, $ts2) = split(/\s+/, $info);
	if ($sceneid =~ m/^EO1/)
		{
		$ali{$sceneid}->{sceneid} = $sceneid;
		push(@{$ali{$sceneid}->{files}}, $file);
		$ali{$sceneid}->{ts1} = $ts1;
		$ali{$sceneid}->{ts2} = $ts2;
		$ali{$sceneid}->{station_contact} = $station_contact;
		}
	else
		{
		&log("Bad eo1info $file: $info");
		}
	}
sub save_hyp_data_file
	{
	my ($file, $station_contact) = @_;
	my $info = `eo1info $ingestdir/$file`;
	chomp $info;
	my ($fname, $sceneid, $filetype, $ts) = split(/\s+/, $info);
	if ($sceneid =~ m/^EO1/)
		{
		$hyp{$sceneid}->{sceneid} = $sceneid;
		$hyp{$sceneid}->{station_contact} = $station_contact;
		push(@{$hyp{$sceneid}->{files}}, $file);
		if ($filetype eq "Image")
			{
			$hyp{$sceneid}->{imagefile} = $file;
			$hyp{$sceneid}->{ts} = $ts;
			}
		}
	else
		{
		&log("Bad eo1info $file: $info");
		}
	}

INGEST_DIR_READ:
while (defined($file = readdir(INGEST)))
	{
	next if $file =~ m/^(\.|lock)/;
	if (-d "$ingestdir/$file")
		{
		if ($file eq "meta")
			{
			ingestmeta("$ingestdir/$file");
			next;
			}
		next unless $file =~ m/^(\d\d\d\d\d\d\d\d\d\d\d_...)$/;
		$subdirectories{$file} = 1;
		my $station_contact = $1;
		unless (opendir(CONTACT, "$ingestdir/$file"))
			{
			&error("$ingestdir/$file: $!");
			next;
			}
		my $bof = telldir(CONTACT);
		my $f;
		while (defined($f = readdir(CONTACT)))
			{
			next if $f =~ m/^\./;
			my @s = stat("$ingestdir/$file/$f");
			if (@s && ($s[9] >= ($now - 600)))
				{
				# Files still being updated, skip it for now.
				delete $subdirectories{$file};
				closedir(CONTACT);
				next INGEST_DIR_READ;
				}
			}
		seekdir(CONTACT, $bof);
		while (defined($f = readdir(CONTACT)))
			{
			next if $f =~ m/^\./;
			if ($f =~ m/\.M[1234]Z$/)
				{
				save_ali_data_file("$file/$f", $station_contact);
				}
			elsif ($f =~ m/\.L0$/)
				{
				save_hyp_data_file("$file/$f", $station_contact);
				}
			else
				{
				&log("Skipped file $file/$f");
				}
			}
		closedir(CONTACT);
		}
	elsif ($file =~ m/00index(EO1...)/)
		{
		push(@indexfiles, "$ingestdir/$file");
		}
	elsif ($file =~ m/\.M[1234]Z$/)
		{
		save_ali_data_file($file, $opt_c);
		}
	elsif ($file =~ m/\.L0$/)
		{
		save_hyp_data_file($file, $opt_c);
		}
	else
		{
		# &log("Skipped file $file");
		}
	}
closedir(INGEST);

# Having found all the files in the ingest directory, figure out what we have
# and move the data files to their proper directories.
foreach my $sceneid (keys %ali)
	{
	# First, make sure there are at least four image files.
	if (@{$ali{$sceneid}->{files}} < 4)
		{
		&log("ALI $sceneid missing files");
		next;
		}
	my @files = sort  @{$ali{$sceneid}->{files}};
	my $entity_id = $sceneid;
	my $destdir;
	$entity_id =~ s/EO1/EO1A/;
	if ($files[0] =~ m/(_..._..).M[1234]Z/)
		{
		$entity_id .= $1;
		}
	unless ($sceneid =~ m/EO1\d\d\d\d\d\d(\d\d\d\d)(\d\d\d)/)
		{
		&log("Bad sceneid $sceneid @files");
		next;
		}
	my ($year, $doy) = ($1, $2);
	unless (checkhousekeeping($opt_D, $year, $doy))
		{
		&log("Missing housekeeping files for $sceneid");
		next;
		}
	$destdir = "$opt_D/$year/$doy/ali/$entity_id";
	$ali{$sceneid}->{hsm_path} = "$opt_D/$year/$doy/ali/$entity_id";
	$ali{$sceneid}->{file_size} = 0;
	$ali{$sceneid}->{num_files} = 0;
	foreach my $f (@files)
		{
		$ali{$sceneid}->{file_size} += -s "$ingestdir/$f";
		$ali{$sceneid}->{num_files}++;
		my $d = $f;
		$d =~ s".*/"";
		renameifnewer("$ingestdir/$f", "$destdir/$d");
		computechecksum("$destdir/$d");
		}
	push(@aliscenes, $entity_id);
	}

foreach my $sceneid (keys %hyp)
	{
	# Make sure we have image data.
	unless ($hyp{$sceneid}->{imagefile})
		{
		&log("Hyp $sceneid missing image data");
		next;
		}
	my @files = sort @{$hyp{$sceneid}->{files}};
	my $entity_id = $sceneid;
	my $destdir;
	$entity_id =~ s/EO1/EO1H/;
	if ($files[0] =~ m/(_..._..).L0/)
		{
		$entity_id .= $1;
		}
	unless ($sceneid =~ m/EO1\d\d\d\d\d\d(\d\d\d\d)(\d\d\d)/)
		{
		&log("Bad sceneid $sceneid @files");
		next;
		}
	my ($year, $doy) = ($1, $2);
	unless (checkhousekeeping($opt_D, $year, $doy))
		{
		&log("Missing housekeeping files for $sceneid");
		next;
		}
	$destdir = "$opt_D/$year/$doy/hyp/$entity_id";
	$hyp{$sceneid}->{hsm_path} = "$opt_D/$year/$doy/hyp/$entity_id";
	$hyp{$sceneid}->{file_size} = 0;
	$hyp{$sceneid}->{num_files} = 0;
	foreach my $f (@files)
		{
		$hyp{$sceneid}->{file_size} += -s "$ingestdir/$f";
		$hyp{$sceneid}->{num_files}++;
		my $d = $f;
		$d =~ s".*/"";
		renameifnewer("$ingestdir/$f", "$destdir/$d");
		computechecksum("$destdir/$d");
		}
	push(@hypscenes, $entity_id);
	}

# For any scenes that were processed, generate browse and metadata.
my @t = localtime($now);
my $unique = sprintf("%04d%02d%02d%02d%02d%02d", $t[5]+1900,
	$t[4]+1, $t[3], $t[2], $t[1], $t[0]);
if (@aliscenes)
	{
	open(ALIMETADATA, ">$ingestdir/tmp/EO1$unique.ali") || die("tmp/EO1$unique.ali: $!");
	foreach my $entity_id (@aliscenes)
		{
		inventory_metadata("ali", $entity_id, \*ALIMETADATA);
		}
	close(ALIMETADATA);
	}

if (@hypscenes)
	{
	open(HYPMETADATA, ">$ingestdir/tmp/EO1$unique.hyp") || die("tmp/EO1$unique.hyp: $!");
	foreach my $entity_id (@hypscenes)
		{
		inventory_metadata("hyp", $entity_id, \*HYPMETADATA);
		}
	close(HYPMETADATA);
	}

# Copy the browse to the browse server.
my $tobecopied = "";
foreach my $i (qw(ali hyp))
	{
	if (-d "$ingestdir/tmp/browsetmp/eo-1/$i")
		{
		$tobecopied .= " eo-1/$i";
		}
	}
if ($tobecopied)
	{
	execute("cd $ingestdir/tmp/browsetmp;tar cf - $tobecopied | ssh -q -lneiers -p22 edclxs154 'cd /data/inventory/browse;tar xpBf -'");
	execute("cd $ingestdir/tmp/browsetmp;rm -fr $tobecopied");
	}

# Copy the metadata to the inventory server
$tobecopied = "";
foreach my $i (qw(ali hyp))
	{
	if (-s "$ingestdir/tmp/EO1$unique.$i")
		{
		$tobecopied .= " EO1$unique.$i";
		}
	}
if ($tobecopied)
	{
	# execute("rcp $tobecopied edcsgs12:/inv/inv/eo1/data");
	execute("cd $ingestdir/tmp; tar cf - $tobecopied | ssh -q -lneiers -p22 edclxs152 'cd /inv/inv/eo1/data;tar xvBf -'");
	# Generate .meta files with additional metadata.
	execute("cd $ingestdir/tmp; $opt_d/bin/genmetadata.pl -d$opt_d $tobecopied");
	}


# Remove any subdirectories that should be empty now.
# Ignore errors, since the most likely cause is that the directory is
# not empty and we should leave the files for further investigation.
foreach my $subdir (keys %subdirectories)
	{
	rmdir "$ingestdir/$subdir";
	}

# Process any index files (indicating that a level-0 tape was generated)
ingestindexfiles(sort @indexfiles);

# Write out the list for the archive
if (@silo_archive)
	{
	if (open(ARCHIVE, ">$opt_a/EO1_archive_$unique"))
		{
		print ARCHIVE join("\n", @silo_archive);
		close(ARCHIVE);
		}
	else
		{
		&error("$opt_a/EO1_archive_$unique: $!");
		}
	}

# Done, remove the lock and exit.
&log("eo1ingest_auto Ended pid $$");
dirunlock($ingestdir);
# Run the script to move the raw data
execute("/home/eo1/bin/moveraw2silo.pl");
exit(0);

# Generate a line of metadata for a single scene
sub inventory_metadata
	{
	my ($instrument, $entity_id, $fh) = @_;
	my $sceneid = $entity_id;
	$sceneid =~ s/EO1[AH]/EO1/;
	$sceneid =~ s/_..._..$//;
	my $rec;
	$rec->{entity_id} = $entity_id;
	my $rpt_rec = eo1::get_rpt_rec($sceneid, $opt_D);

	# Orbit path/row come from the daily report file.
	$rec->{orbit_path} = $rpt_rec->{PATH};
	$rec->{orbit_row} = $rpt_rec->{'START ROW'};

	# Scene start and stop time:
	#   For ALI use the data gate open time from the HDF header.
	#   For Hyperion use the time range from the daily report file
	#     and run hypimagetimes to find the real times from the
	#     housekeeping files.
	# The fallback for both is to use the times from the daily
	# report file (which tend to be less accurate).
	$rec->{scene_start_time} = $rpt_rec->{starttime};
	if (($instrument eq "ali") && $ali{$sceneid}->{ts1})
		{
		($rec->{scene_start_time} = $ali{$sceneid}->{ts1}) =~
			s/(\d\d\d\d)(\d\d\d)(\d\d)(\d\d)(\d\d)/$1:$2:$3:$4:$5/;
		}

	$rec->{scene_stop_time} = $rpt_rec->{endtime};
	if (($instrument eq "ali") && $ali{$sceneid}->{ts2})
		{
		($rec->{scene_stop_time} = $ali{$sceneid}->{ts2}) =~
			s/(\d\d\d\d)(\d\d\d)(\d\d)(\d\d)(\d\d)/$1:$2:$3:$4:$5/;
		}
	if ($instrument eq "hyp")
		{
		if ($rec->{entity_id} =~ m/EO1[AH]\d\d\d\d\d\d(\d\d\d\d)(\d\d\d)/)
			{
			my ($year, $doy) = ($1, $2);
			my $hyphk = "$opt_D/$year/$doy/meta/${year}_${doy}_hyp.hdf";
			my $info;
			if (defined($rpt_rec->{starttime}) && defined($rpt_rec->{endtime}))
				{
				$info = `hypimagetimes -i "$rpt_rec->{starttime}-$rpt_rec->{endtime}" $hyphk`;
				}
			else
				{
				my $ts1 = $hyp{$sceneid}->{ts};
				$ts1 =~ s/(\d\d\d\d)(\d\d\d)(\d\d)(\d\d)(\d\d)/$1:$2:$3:$4:$5/;
				my $ts2 = gmtadd($ts1, 14);
				$info = `hypimagetimes -i "$ts1-$ts2" $hyphk`;
				}
			chomp $info;
			if ($info =~ m/(\S+) (\S+)/)
				{
				$rec->{scene_start_time} = $1;
				$rec->{scene_stop_time} = $2;
				}
			}
		}

	# Set the acquisition_date from the start time.  As a fallback,
	# if the start time is missing, use the date portion of the scene ID.
	if (defined($rec->{scene_start_time}))
		{
		$rec->{acquisition_date} = yydddtoyymmmdd(substr($rec->{scene_start_time}, 0, 4),
			substr($rec->{scene_start_time}, 5, 3));

		}
	else
		{
		$rec->{acquisition_date} = yydddtoyymmmdd(substr($sceneid, 9, 4), substr($sceneid, 13,3));
		}

	# The special request (which is really the target name) comes from
	# the daily report file, the fallback is to use the value from
	# the HDF header in one of the image files.
	if (defined($rpt_rec->{'SPECIAL REQUEST'}))
		{
		$rec->{special_request} = $rpt_rec->{'SPECIAL REQUEST'};
		}
	else
		{
		my $f;
		if ($instrument eq "ali")
			{
			$f = ${$ali{$sceneid}->{files}}[0];
			}
		else
			{
			$f = ${$hyp{$sceneid}->{files}}[0];
			}
		$f =~ s".*/"";
		my %ins = ("A" => "ali", "H" => "hyp");
		if ($entity_id =~ m/EO1([AH])\d\d\d\d\d\d(\d\d\d\d)(\d\d\d)/)
			{
			my $datadir = "$opt_D/$2/$3/$ins{$1}/$entity_id";
			my @info = `eo1info -v $datadir/$f`;
			foreach (@info)
				{
				chomp;
				if (m/Target_Description: (.*)/)
					{
					$rec->{special_request} = $1;
					$rec->{special_request} =~ s/\s+$//;
					last;
					}
				}
			}
		}

	# Get the receiving station from the tail of the entity ID.
	if ($entity_id =~ m/_(...)_..$/)
		{
		$rec->{rec_station} = $1;
		}
	# Get corner point lat/lons.  Note that this depends on the
	# scene start and stop times.
	getcorners($rec);

	# Generate the browse...
	execute("regen${instrument}browse.pl -d $opt_d -D $opt_D -t $ingestdir/tmp $entity_id\n");
	# then check to see if it worked.
	my $browsepath = generatebrowsepath($entity_id);
	if (-f "$ingestdir/tmp/browsetmp/eo-1/$instrument/$browsepath")
		{
		$rec->{browse_availability} = 'Y';
		$rec->{browse_image_file_path} = $browsepath;
		}

	$rec->{file_number} = ($instrument eq "ali" ? 2 : 3);
	my $station_contact;
	if ($instrument eq "ali")
		{
		$station_contact = $ali{$sceneid}->{station_contact};
		$rec->{hsm_path} = $ali{$sceneid}->{hsm_path};
		$rec->{file_size} = $ali{$sceneid}->{file_size};
		$rec->{num_files} = $ali{$sceneid}->{num_files};
		}
	else
		{
		$station_contact = $hyp{$sceneid}->{station_contact};
		$rec->{hsm_path} = $hyp{$sceneid}->{hsm_path};
		$rec->{file_size} = $hyp{$sceneid}->{file_size};
		$rec->{num_files} = $hyp{$sceneid}->{num_files};
		}
	$rec->{directory_name} = "$instrument/$station_contact/$sceneid";
	$rec->{restriction_comments} = get_restriction_comments($rec);
	$rec->{look_angle} = sprintf("%0.4f", compute_lookangle($rec));
	my $inclination = compute_inclination($rec);
	$rec->{inclination} = defined($inclination) ? sprintf("%0.2f", $inclination) : "";

	# Output a single inventory metadata update line:
	my @fieldnames = ($instrument eq "ali" ? @ali_fieldnames: @hyp_fieldnames);
	foreach (@fieldnames)
		{
		print $fh "$rec->{$_}$opt_s";
		}
	print $fh "\n";
	}

# Write an entry to the log file.  The entry is prepended with the current
# date and time.  We open and close the log file to minimize the chance of
# multiple programs overwriting each other.
sub log
	{
	open(LOGFILE, ">>$opt_d/$opt_l") || warn("$opt_d/$opt_l: $!");
	print LOGFILE &timestamp, @_, "\n";
	close(LOGFILE);
	}

# Return a string with the current time and date
sub timestamp
	{
	my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(time);
	return sprintf("%4d/%02d/%02d %02d:%02d:%02d ",
		$year + 1900, $mon + 1, $mday, $hour, $min, $sec);
	}

# Print an error to STDERR and log it.
sub error
	{
	&log("Error: @_");
	print STDERR "Error: @_\n";
	}

# Call error and exit
sub fatal_error
	{
	error(@_);
	exit(1);
	}

# Standard Unix lock file, with PID so detect leftover lock files.
# If an instance is already running, sliently exit.
sub dirlock
	{
	my $ingestdir = $_[0];
	my $tmplockfile = "$ingestdir/lock.$$";
	my $lockfile = "$ingestdir/lock";
	open(LOCK, ">$tmplockfile") || die("$tmplockfile");
	print LOCK "$$\n";
	close(LOCK);
tryagain:
	if (link($tmplockfile, $lockfile))
		{
		unlink($tmplockfile);
		return;
		}
	open(LOCK, "<$lockfile") || die("$lockfile");
	my $pid = <LOCK>;
	close(LOCK);
	chomp $pid;
	if (kill(0, $pid))
		{
		unlink($tmplockfile);
		exit(0)
		}
	&log("Removing broken lockfile for pid $pid");
	unlink($lockfile);
	sleep(1);
	goto tryagain;
	}
sub dirunlock
	{
	my $ingestdir = $_[0];
	unlink("$ingestdir/lock");
	}

# mkmdirsfor takes a filename argument and creates any directories necessary
sub mkdirsfor
	{
	my ($file) = @_;

	return 0 unless ($file =~ s?/[^/]*$??);
	return 1 if -d $file;
	&mkdirsfor($file);
	if ($opt_T)
		{
		print "mkdir $file\n";
		}
	else
		{
		mkdir($file, 0777);
		}
	}

# Compare the mtime of two files, return -1, 0, or 1 if the first is
# (respectively) older, the same, or newer than the second
sub compare_file_mtime
	{
	my ($file1, $file2) = @_;
	my @s1 = stat($file1);
	return -1 unless @s1;
	my @s2 = stat($file2);
	return 1 unless @s2;
	return $s1[9] <=> $s2[9];
	}
sub renameifnewer
	{
	my ($old, $new) = @_;
	if (-f $new)
		{
		if (compare_file_mtime($old, $new) < 0)
			{
			&log("file exists and is newer: $old");
			return;
			}
		else
			{
			&log("replacing file $new");
			}
		}
	mkdirsfor($new);
	rename($old, $new) || execute("mv $old $new");
	}
# Call "system" with the supplied parameters, unless the -T option is
# set, in which case just print the command.
sub execute
	{
	if ($opt_T)
		{
		print "@_\n";
		}
	else
		{
		system @_;
		}
	}

# Given an entity ID, return the browse path
sub generatebrowsepath
	{
	my $sceneid = $_[0];
	$sceneid =~ s/EO1[AH]/EO1/;
	return sprintf("%d/%d/%d/%s.jpeg", $1, $2, $3, $sceneid)
		if ($sceneid =~ m/EO1(\d\d\d)(\d\d\d)(\d\d\d\d)/);
	return undef;
	}

# Convert year and day of year to year, month, day of month.
sub yydddtoyymmmdd
	{
	my ($year, $day) = @_;
	my @days = (31,28,31,30,31,30,31,31,30,31,30,31);
	my @monthname = qw(Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec);
	# Leap year check.
	if ((($year % 4) == 0) && ((($year % 100) != 0) || ($year % 400 == 0)))
		{
		$days[1] = 29;
		}
	my $month;
	for ($month = 0; $day > $days[$month]; $month++)
		{
		$day -= $days[$month];
		}
	return sprintf("%02d-%s-%4d", $day, $monthname[$month], $year);
	}

# Fill in the corner point coordinates
my %alicorners;
my %hypcorners;
my %footprint_files_read;
sub getcorners
	{
	my $rec = $_[0];
	my $corners;		# pointer to either %alicorners or %hypcorners
	my $entity_id = $rec->{entity_id};
	my $instrument;		# "ali" or "hyp"

	if ($entity_id =~ m/EO1A/)
		{
		$corners = \%alicorners;
		$instrument = "ali";
		}
	else
		{
		$corners = \%hypcorners;
		$instrument = "hyp";
		}
	return unless ($entity_id =~ m/EO1[AH]\d\d\d\d\d\d(\d\d\d\d)(\d\d\d)/);
	my ($year, $doy) = ($1, $2);
	my $footprint_file = "$opt_d/inventory/footprint/${instrument}corners.$year$doy.txt";
	unless ($footprint_files_read{$footprint_file})
		{
		if (compare_file_mtime($footprint_file, "$opt_D/$year/$doy/meta/${year}_${doy}_acs.hdf") < 0)
			{
			execute("cd $opt_d/inventory/footprint;./gencorners.pl -d $opt_d -D $opt_D $year/$doy");
			}
		$footprint_files_read{$footprint_file} = 1;
		}
	readcorners("$opt_d/inventory/${instrument}corners.txt", $corners);

	# Find the corner point coordinates
	my $time;
	my $timediff = 99999999;	# difference between target time and corner time
	if ($rec->{scene_start_time})
		{
		foreach (keys %{$corners})
			{
			if ((gmtdiff($rec->{scene_start_time}, $_) >= 0) &&
			   (gmtdiff($_, $rec->{scene_stop_time}) >= 0))
				{
				$timediff = 0;
				$time = $_;
				last;
				}
			my $diff = abs(gmtdiff($_, $rec->{scene_start_time}));
			if ((!defined($timediff)) || ($diff < $timediff))
				{
				$timediff = $diff;
				$time = $_;
				}
			}
		}
	if ($timediff > 3)
		{
		# $missing_corners++;
		print STDERR "No corners for $entity_id $rec->{scene_start_time}, $rec->{scene_stop_time}, closest is $time\n";
		}
	else
		{
		if ($rec->{orbit_row} > 122)
			{
			# Ascending scene: the corner points are "flipped"
			$rec->{ul_corner_lat} = $corners->{$time}->{LRlat};
			$rec->{ul_corner_long} = $corners->{$time}->{LRlon};
			$rec->{ur_corner_lat} = $corners->{$time}->{LLlat};
			$rec->{ur_corner_long} = $corners->{$time}->{LLlon};
			$rec->{ll_corner_lat} = $corners->{$time}->{URlat};
			$rec->{ll_corner_long} = $corners->{$time}->{URlon};
			$rec->{lr_corner_lat} = $corners->{$time}->{ULlat};
			$rec->{lr_corner_long} = $corners->{$time}->{ULlon};
			}
		else
			{
			$rec->{ul_corner_lat} = $corners->{$time}->{ULlat};
			$rec->{ul_corner_long} = $corners->{$time}->{ULlon};
			$rec->{ur_corner_lat} = $corners->{$time}->{URlat};
			$rec->{ur_corner_long} = $corners->{$time}->{URlon};
			$rec->{ll_corner_lat} = $corners->{$time}->{LLlat};
			$rec->{ll_corner_long} = $corners->{$time}->{LLlon};
			$rec->{lr_corner_lat} = $corners->{$time}->{LRlat};
			$rec->{lr_corner_long} = $corners->{$time}->{LRlon};
			}
		# If we have coordinates, compute the sun angles at the center point
		my $ctr_lat = ($corners->{$time}->{ULlat} +
			$corners->{$time}->{URlat} +
			$corners->{$time}->{LLlat} +
			$corners->{$time}->{LRlat}) /4;
		my $ctr_lon = ($corners->{$time}->{ULlon} +
			$corners->{$time}->{URlon} +
			$corners->{$time}->{LLlon} +
			$corners->{$time}->{LRlon}) /4;
		($rec->{sun_elevation},
		 $rec->{sun_azimuth}) = split(/\s+/, 
		 	`sunangles $time $ctr_lat, $ctr_lon`);
		$rec->{day_or_night} = 
			($rec->{sun_elevation} < 0.0) ? "N" : "D";
		}
	}
# Read the corners.txt file and build a hash of corner points by time.
sub readcorners
	{
	my ($filename, $corners) = @_;
	my $time;
	open(CORNERS, "<$filename") || return;
	while (<CORNERS>)
		{
		chomp;
		if (m/^\d\d\d\d:\d\d\d:\d\d:\d\d:\d\d/)
			{
			$time = $_;
			}
		elsif (m/(UL|UR|LL|LR):\s*Lat\s*([-0-9\.]+)\s*Lon\s*([-0-9\.]+)/)
			{
			$corners->{$time}->{"${1}lat"} = $2;
			$corners->{$time}->{"${1}lon"} = $3;
			}
		}
	close(CORNERS);
	}

# Compare two dates, in the form "01-Apr-2002", return -1, 0, 1 if the
# first is (respectively) less than, equal to or greater than the second.
sub cmpdate
	{
	my ($a, $b) = @_;
	my ($a_year, $a_month, $a_day);
	my ($b_year, $b_month, $b_day);
	my %monthno = (qw(Jan 01 Feb 02 Mar 03 Apr 04 May 05 Jun 06 Jul 07 Aug 08 Sep 09 Oct 10 Nov 11 Dec 12));
	if ($a =~ m"(\d+)[-/]([A-Z][a-z][a-z])[/-](\d\d\d\d)")
		{
		$a_year = $3;
		$a_day = $1;
		$a_month = $monthno{$2};
		}
	if ($b =~ m"(\d+)[-/]([A-Z][a-z][a-z])[/-](\d\d\d\d)")
		{
		$b_year = $3;
		$b_day = $1;
		$b_month = $monthno{$2};
		}
	($a_year <=> $b_year) || ($a_month <=> $b_month) || ($a_day <=> $b_day);
	}
# Look for restrictions on this scene.
sub get_restriction_comments
	{
	my $rec = $_[0];
	my @restrictions;		# to be returned
	my %exclusions;
	my $sceneid = $rec->{entity_id};
	$sceneid =~ s/EO1[AH]/EO1/;
	$sceneid =~ s/_..._..$//;

	push(@restrictions, "Calibration") if (($sceneid =~ m/^EO1000000/) ||
		($rec->{special_request} =~ m/90Yaw/));
	push(@restrictions, "Instrument check-out")
		if (cmpdate($rec->{acquisition_date}, "15-Mar-2001") < 0);
	push(@restrictions, "DOD collect")
		if ($rec->{special_request} =~ m/Warfigher|Paules|W\.A\.A\./);
	push(@restrictions, "PPT test scene")
		if (($rec->{acquisition_date} eq "07-Jan-2002") ||
		    ($rec->{acquisition_date} eq "14-Mar-2002") ||
		    ($rec->{acquisition_date} eq "25-Apr-2002"));
	if ($rec->{entity_id} =~ m/EO1A/)
		{
		%exclusions = %aliexclusions;
		}
	if ($rec->{entity_id} =~ m/EO1H/)
		{
		%exclusions = %hypexclusions;
		push(@restrictions, "Israel")
			if ($rec->{entity_id} =~ m/^EO1H174039/);
		}
	foreach (keys %exclusions)
		{
		if ($sceneid =~ m/$_/)
			{
			push(@restrictions, $exclusions{$_});
			}
		}
	return(join(";", @restrictions));
	}


# Read a file with lines like this:
#	EO10920842001329112KP Frame rate anomaly
# extract the scene ID and point to the exclusion reason.  Returns a hash.
sub readexclusions
	{
	my %ex;
	unless (open(EX, "<$_[0]"))
		{
		warn("$_[0]: $!");
		return undef;
		}
	while (<EX>)
		{
		chomp;
		if (m/(\S+)\s+(.*)/)
			{
			$ex{$1} = $2;
			}
		}
	close(EX);
	return %ex;
	}

sub compute_lookangle
	{
	my $rec = $_[0];
	my $starttime = $rec->{scene_start_time};
	my $stoptime = $rec->{scene_stop_time};
	my $instrument = ($rec->{entity_id} =~ m/EO1A/ ? "ali" : "hyp");
	unless ($starttime =~ m/^(\d\d\d\d):?(\d\d\d)/)
		{
		print STDERR "$0: Starttime $starttime, no lookangles for $rec->{entity_id}\n";
		return undef;
		}
	my $acs_filename = "$opt_D/$1/$2/meta/$1_$2_acs.hdf";
	unless (-f $acs_filename)
		{
		print STDERR "$0: No file $acs_filename, no lookangles for $rec->{entity_id}\n";
		return undef;
		}
	$starttime =~ s/://g;
	$stoptime =~ s/://g;
	my $output = `ulimit -t 60; cd $opt_d/inventory/lookangle;./lookangleparams $acs_filename $starttime $stoptime | octave -q`;
	unless ($? == 0)
		{
		print STDERR "$0: No lookangles for $rec->{entity_id}\n";
		return undef;
		}
	my $ali;
	my $hyp;
	if ($output =~ m/ALI_lookangle\s*=\s*([-0-9\.e]+)/)
		{
		$ali = $1;
		}
	if ($output =~ m/HSI_lookangle\s*=\s*([-0-9\.e]+)/)
		{
		$hyp = $1;
		}
	if ($instrument eq "ali")
		{
		return $ali;
		}
	else
		{
		return $hyp;
		}
	}

sub compute_inclination
	{
	my $rec = $_[0];
	my $starttime = $rec->{scene_start_time};
	my $stoptime = $rec->{scene_stop_time};
	unless ($starttime =~ m/^(\d\d\d\d):?(\d\d\d)/)
		{
		print STDERR "$0: Starttime $starttime, no inclination for $rec->{entity_id}\n";
		return undef;
		}
	my $acs_filename = "$opt_D/$1/$2/meta/$1_$2_acs.hdf";
	unless (-f $acs_filename)
		{
		print STDERR "$0: No file $acs_filename, no inclination for $rec->{entity_id}\n";
		return undef;
		}
	$starttime =~ s/://g;
	$stoptime =~ s/://g;
	my $output = `cd $opt_d/inventory/lookangle;./inclination $acs_filename $starttime $stoptime`;
	unless ($? == 0)
		{
		print STDERR "$0: No lookangles for $rec->{entity_id}\n";
		return undef;
		}
	if ($output =~ m/([-0-9\.e]+)/)
		{
		return $1;
		}
	return undef;
	}

# Given a year and day-of-year, check to see if all the housekeeping
# files are in place.
sub checkhousekeeping
	{
	my ($dir, $year, $doy) = @_;
	foreach my $hk (qw(acs gps ali hyp))
		{
		unless (-f "$dir/$year/$doy/meta/${year}_${doy}_${hk}.hdf")
			{
			&log("Missing $dir/$year/$doy/meta/${year}_${doy}_${hk}.hdf");
			return 0;
			}
		}
	# We also need either the report file or the daily text file.
	my $two_digit_year = substr($year, 2, 2);
	unless (-f "$dir/$year/$doy/0event_${year}${doy}.rpt" ||
		-f "$dir/$year/$doy/meta/daily_${two_digit_year}${doy}.txt")
			{
			&log("Missing rpt file for $year/$doy $dir/$year/$doy/0event_${year}${doy}.rpt || $dir/$year/$doy/meta/daily_${two_digit_year}${doy}.txt");
			# Around 2016-01-08 the daily text files started showing up about three days late.
			# Since we can ingest without them, temporarily(?) disable this check (but still print the warning)
			# return 0;
			}
	return 1;
	}

# Ingest a "meta" subdirectory
sub ingestmeta
	{
	my $srcdir = $_[0];
	my $file;
	unless (opendir(META, $srcdir))
		{
		&log("$srcdir: $!");
		return;
		}
	while (defined($file = readdir(META)))
		{
		next if $file =~ m/^\./;
		if ($file =~ m"^(\d\d\d\d)_(\d\d\d)_(.*)\.hdf$")
			{
			# 2001_164_acs.hdf
			my ($year, $day, $which) = ($1, $2, $3);
			renameifnewer("$srcdir/$file", "$opt_D/$year/$day/meta/${year}_${day}_$which.hdf");
			computechecksum("$opt_D/$year/$day/meta/${year}_${day}_$which.hdf");
			}
		elsif ($file =~ m"^00meta_EO1\d\d\d\d\d\d(\d\d\d\d)(\d\d\d).*\.DAT$")
			{
			# 00meta_EO10000002001163111DS_SGS_01.DAT
			my ($year, $day) = ($1, $2);
			renameifnewer("$srcdir/$file", "$opt_D/$year/$day/meta/$file");
			computechecksum("$opt_D/$year/$day/meta/$file");
			}
		elsif ($file =~ m"^0event_(\d\d\d\d)(\d\d\d)\.(rpt|txt)$")
			{
			# 0event_2001157.rpt
			my ($year, $day) = ($1, $2);
			renameifnewer("$srcdir/$file", "$opt_D/$year/$day/$file");
			computechecksum("$opt_D/$year/$day/$file");
			}
		elsif ($file =~ m"^(\d\d\d\d)(\d\d\d).*(DQM|WFD|GSSF)$")
			{
			my ($year, $day) = ($1, $2);
			renameifnewer("$srcdir/$file", "$opt_D/$year/$day/meta/$file");
			computechecksum("$opt_D/$year/$day/meta/$file");
			}
		elsif ($file =~ m"^00index.*TCRF$")
			{
			renameifnewer("$srcdir/$file", "$opt_D/meta/$file");
			computechecksum("$opt_D/meta/$file");
			}
                elsif ($file =~ m"^daily_(\d\d)(\d\d\d)\.txt$")
                        {
                        # daily_06040.txt need to manually prefix the year with 20
			my ($year, $day) = ("20$1", $2);
                        renameifnewer("$srcdir/$file", "$opt_D/$year/$day/meta/$file");
			computechecksum("$opt_D/$year/$day/meta/$file");
                        }
                elsif ($file =~ m"^daily_(\d\d\d\d)_(\d\d\d)$")
                        {
			my ($year, $day) = ($1, $2);
                        renameifnewer("$srcdir/$file", "$opt_D/$year/$day/meta/$file");
			computechecksum("$opt_D/$year/$day/meta/$file");
                        }
		elsif ($file =~ m"(\d\d\d\d)(\d\d\d)\d\d\d\d_.*_(acs|ali|eff|gps|hyp|lac|mis|wrp|xbd).hdf")
			{
			# Per-downlink housekeeping files added 2008-07-24
			my ($year, $day) = ($1, $2);
                        renameifnewer("$srcdir/$file", "$opt_D/$year/$day/meta/$file");
			computechecksum("$opt_D/$year/$day/meta/$file");
			}
		else
			{
			&log("Skipped file $srcdir/$file");
			}
		}
	closedir(META);
	}

# Handle a tape index file:
# TBD: read the index file and make sure all the files are here.
# What happens if not?  Send email to somebody?
sub ingestindexfiles
	{
	my %read;
	foreach my $indexfile (sort @_)
		{
		next unless ($indexfile =~ m/00index(EO1...)/);
		my $tapeid = $1;
		unless (open(INDEX, "<$indexfile"))
			{
			&log("$indexfile: $!");
			next;
			}
		my @contents = <INDEX>;
		close(INDEX);
		if ($read{"@contents"})
			{
			# Duplicate, just move it to the tmp directory
			my $destfile = $indexfile;
			$destfile =~ s"00index"tmp/00index";
			rename($indexfile, $destfile) || execute("mv $indexfile $destfile");
			next;
			}
		$read{"@contents"} = $indexfile;
		my $destfile = "$opt_D/meta/00index$tapeid";
		&log("Tape $tapeid");
		rename($indexfile, $destfile) || execute("mv $indexfile $destfile");
		computechecksum($destfile);
		execute("cd $opt_d/inventory;./runpost_auto.pl $tapeid > $tapeid.log 2>&1");
		}
	}

# Given a file name, compute the MD5 checksum and append it to the
# "Checksums" file, along with the file mtime and the current time.
# Note that the file should have been moved to it's final location already.
sub computechecksum
	{
	my $full_pathname = $_[0];
	my $short_filename = $full_pathname;
	$short_filename =~ s"$opt_D/"";
	if ($opt_T)
		{
		print "md5 $full_pathname\n";
		next;
		}
	if (open(DATA, "<$full_pathname"))
		{
		my @s = stat($full_pathname);
		my $md5 = new Digest::MD5;
		$md5->addfile(\*DATA);
		if (open(CHECKSUMS, ">>$opt_d/inventory/Checksums"))
			{
			printf(CHECKSUMS "%s %s %d %d\n", $short_filename, $md5->hexdigest, $s[9], time());
			close(CHECKSUMS);
			}
		else
			{
			&log("$opt_d/inventory/Checksums: $!");
			}
		close(DATA);
		}
	}
