#!/usr/bin/perl
use strict;
use vars qw($opt_d $opt_T $opt_v);
use Getopt::Std;
getopts("d:Tv") || die(<<END_USAGE);
Usage:
	$0 [-d dir] [-T] [-v]
Where
	-d dir		Use "dir" as the working directory (default /home/eo1).
	-T		test mode
	-v		verbose
Look through the ingest directory looking for files with the USAK03 receiving
station in the name and change it to AK3 so that ingest will work.
END_USAGE
$opt_d ||= "/home/eo1";
my $ingestdir = "$opt_d/ingest";
opendir(INGEST, $ingestdir) || die("$ingestdir: $!");
my $file;
while (defined($file = readdir(INGEST))) {
	if ($file eq "meta") {
		# Read meta directory, looking for these: 00meta_EO11050702015160110KF_20151610407_USAK03_01.DAT
		# and copy it to 00meta_EO11050702015160110KF_AK3_01.DAT while changing the file names
		# for the level-0 data to match the way we rename them below.
		opendir(META, "$ingestdir/$file") || die("$ingestdir/$file: $!");
		my $dat;
		while (defined($dat = readdir(META))) {
			if ($dat =~ m/(00meta_.*_)\d\d\d\d\d\d\d\d\d\d\d_USAK03(_..\.DAT)/) {
				# renamefile("$ingestdir/$file/$dat", "$ingestdir/$file/${1}AK3$2");
				my $fixDATcmd = "sed -e 's/USAK03.M/AK3_01.M/' -e 's/USAK03/AK3/g' < $ingestdir/$file/$dat > $ingestdir/$file/${1}AK3$2";
				if ($opt_T) {
					print "$fixDATcmd\n";
				} else {
					system($fixDATcmd);
				}
			}
		}
		closedir(META);
	} elsif (-d "$ingestdir/$file" && ($file =~ m/^(\d\d\d\d\d\d\d\d\d\d\d)_USAK03$/)) {
		my @s = stat("$ingestdir/$file");
		if (@s && ($s[9] >= (time() - 600))) {
			# Files still being updated, skip it for now.
			print STDERR "Skip directory $file, mtime: ", scalar(localtime($s[9])), "\n" if $opt_v;
			next;
		}
		my $station = "${1}_AK3";
		opendir(DOWNLINK, "$ingestdir/$file") || die("$ingestdir/$file: $!");
		my $datafile;
		while (defined($datafile = readdir(DOWNLINK))) {
			if ($datafile =~ m/(.*)_USAK03.(M[1234]Z)/) {
				renamefile("$ingestdir/$file/$datafile", "$ingestdir/$file/${1}_AK3_01.$2");
			} elsif ($datafile =~ m/(.*)_USAK03_(..\.L0)/) {
				renamefile("$ingestdir/$file/$datafile", "$ingestdir/$file/${1}_AK3_$2");
			}
		}
		closedir(DOWNLINK);
		renamefile("$ingestdir/$file", "$ingestdir/$station");
	}
}
closedir(INGEST);

sub renamefile
{
	my $old = shift;
	my $new = shift;
	if (-f $new) {
		print "$new file exists\n";
		return;
	}
	if ($opt_T) {
		print "mv -i $old $new\n";
	} else {
		rename($old, $new) || warn("mv $old $new: $!");
	}
}
