#!/usr/local/bin/perl
# Generate metadata files to supplement the *.DAT files from the level-0 tape.
use strict;
use vars qw($opt_d $opt_s $opt_D);
my $Usage = <<END_USAGE;
$0 [-d dir] [-s chr] [-D dir] files...
where
	-d dir	use "dir" as the top level director, default is /edclxs37/eo1
	-s chr	use "chr" as the separator character for the input metadata, default is |
	-D dir	Use "dir" as the root directory on the silo (default /hsm/lta4/eo1).
	files	one or more metadata files generated by readmeta.pl
END_USAGE

use Getopt::Std;
getopts("d:s:D:");
$opt_d ||= "/edclxs37/eo1";
$opt_s ||= "\\|";
$opt_D ||= "/hsm/lta4/eo1";

# Read the CSV metadata file produced by readmeta.pl
while (<>)
	{
	chomp;
	my @fields = split(/$opt_s/, $_);
	if ($fields[0] =~ m/^EO1([AH])(\d\d\d)(\d\d\d)(\d\d\d\d)(\d\d\d)/)
		{
		my ($inst, $path, $row, $year, $dayofyear) = ($1, $2, $3, $4, $5);
		my $sceneid = $fields[0];
		$sceneid =~ s/_..._\d\d$//;
		my $destfile = "$opt_D/$year/$dayofyear/meta/$sceneid.meta";
		my @coords;
		if ($inst eq "A")
			{
			@coords = @fields[17..24];
			}
		elsif ($inst eq "H")
			{
			@coords = @fields[18..25];
			}
		else
			{
			print STDERR "Bad scene id skipped $fields[0]\n";
			next;
			}

		mkdirsfor($destfile);
		open(META, ">$destfile") || die("$destfile: $!");
		print META "PRODUCT_UL_CORNER_LAT = $coords[0]\n";
		print META "PRODUCT_UL_CORNER_LON = $coords[1]\n";
		print META "PRODUCT_UR_CORNER_LAT = $coords[2]\n";
		print META "PRODUCT_UR_CORNER_LON = $coords[3]\n";
		print META "PRODUCT_LL_CORNER_LAT = $coords[4]\n";
		print META "PRODUCT_LL_CORNER_LON = $coords[5]\n";
		print META "PRODUCT_LR_CORNER_LAT = $coords[6]\n";
		print META "PRODUCT_LR_CORNER_LON = $coords[7]\n";
		close(META);
		}
	else
		{
		print STDERR "Skipping $_\n";
		}
	}

# mkmdirsfor takes a filename argument and creates any directories necessary
sub mkdirsfor
	{
	my ($file) = @_;

	return 0 unless ($file =~ s?/[^/]*$??);
	return 1 if -d $file;
	&mkdirsfor($file);
	mkdir($file, 0777);
	}
