# return the difference between two GMT values, in seconds with one decimal place
sub gmtdiff
	{
	my $diff = 0;
	my ($begin, $end) = @_;
	my ($byear, $bday, $bhour, $bmin, $bsec, $btenth) = split(/[\/:.]/, $begin);
	my ($eyear, $eday, $ehour, $emin, $esec, $etenth) = split(/[\/:.]/, $end);

	while ($eyear > $byear)
		{
		$eyear--;
		$eday += days_per_year($eyear);
		}
	while ($eyear < $byear)
		{
		$eday -= days_per_year($eyear);
		$eyear++;
		}
	$ehour += ($eday - $bday) * 24 if ($eday != $bday);
	$emin += ($ehour - $bhour) * 60 if ($ehour != $bhour);
	$esec += ($emin - $bmin) * 60 if ($emin != $bmin);
	$diff = $esec - $bsec;
	$etenth = "0.$etenth";
	$btenth = "0.$btenth";
	if ($etenth < $btenth)
		{
		$diff--;
		$etenth += 1.0;
		}
	$diff += ($etenth - $btenth);
	return $diff;
	}

# add a time, in seconds, to a GMT
sub gmtadd
	{
	my ($gmt, $incr) = @_;
	if ($incr < 0)
		{
		return gmtsub($gmt, -$incr);
		}
	my ($year, $day, $hour, $min, $sec, $tenth) = split(/[\/:.]/, $gmt);
	my $days_this_year = days_per_year($year);

	$tenth = substr($tenth, 0, 1);	# round to tenths
	$tenth += int (10 * ($incr - int($incr) + 0.01));
	while ($tenth > 9)
		{
		$sec++;
		$tenth -= 10;
		}
	$incr = int($incr);

	$sec += $incr % 60;
	while ($sec > 59)
		{
		$min++;
		$sec -= 60;
		}
	$incr = int($incr / 60);

	$min += $incr % 60;
	while ($min > 59)
		{
		$hour++;
		$min -= 60;
		}
	$incr = int($incr / 60);

	$hour += $incr % 24;
	while ($hour > 23)
		{
		$day++;
		$hour -= 24;
		}
	$incr = int($incr/24);

	while (($day + $incr) > $days_this_year)
		{
		$incr -= $days_this_year;
		$year++;
		$days_this_year = days_per_year($year);
		}
	$day += $incr;

	return sprintf("%4d/%03d:%02d:%02d:%02d.%1d",
		$year, $day, $hour, $min, $sec, $tenth);
	}
# subtract a time, in seconds, to a GMT
sub gmtsub
	{
	my ($gmt, $decr) = @_;
	if ($decr < 0)
		{
		return gmtadd($gmt, -$decr);
		}
	my ($year, $day, $hour, $min, $sec, $tenth) = split(/[\/:.]/, $gmt);

	$tenth = substr($tenth, 0, 1);	# round to tenths
	$tenth -= int (10 * ($decr - int($decr) + 0.01));
	while ($tenth < 0)
		{
		$sec--;
		$tenth += 10;
		}
	$decr = int($decr);

	$sec -= $decr % 60;
	while ($sec < 0)
		{
		$min--;
		$sec += 60;
		}
	$decr = int($decr / 60);

	$min -= $decr % 60;
	while ($min < 0)
		{
		$hour--;
		$min += 60;
		}
	$decr = int($decr / 60);

	$hour -= $decr % 24;
	while ($hour < 0)
		{
		$day--;
		$hour += 24;
		}
	$decr = int($decr/24);

	while ($day <= $decr)
		{
		$year--;
		$decr -= days_per_year($year);
		}
	$day -= $decr;

	return sprintf("%4d/%03d:%02d:%02d:%02d.%1d",
		$year, $day, $hour, $min, $sec, $tenth);
	}
sub days_per_year
	{
	my ($year) = @_;
	return 366 if ((($year % 4) == 0) && ((($year % 100) != 0) || (($year % 400) == 0)));
	return 365;
	}
1;
