        function        a  =  qtoa( q )
%
%  QTOA  -  Converts array of quaternions to attitude matrices
%
%  Usage:   function    a  =  qtoa( q )
%
%  Input:   In the following "n" is the number of quaternions
%           q =  4-by-n array of quaternions, with each 
%                column corresponding to a different quaternion
%
%  Output:  a =  3-by-3n array of rotation matrices arranged side by side,
%                i.e., 1st matrix is in columns 1 to 3, 2nd matrix 
%                is in columns 4 to 6, etc.
%
%  External References: none
%
%  23 Sep 96  -  J. Sedlak - replace old QTOA with a routine that will
%                            handle array of multiple input quaternions

% Build it first as a 9xn matrix

   q1q1 = q(1,:).*q(1,:);
   q2q2 = q(2,:).*q(2,:);
   q3q3 = q(3,:).*q(3,:);
   q4q4 = q(4,:).*q(4,:);

   q1q2 = q(1,:).*q(2,:);
   q1q3 = q(1,:).*q(3,:);
   q1q4 = q(1,:).*q(4,:);

   q2q3 = q(2,:).*q(3,:);
   q2q4 = q(2,:).*q(4,:);
   q3q4 = q(3,:).*q(4,:);

   a9xn = [    q1q1 - q2q2 - q3q3 + q4q4;
            2*(q1q2 - q3q4);
            2*(q1q3 + q2q4);
            2*(q1q2 + q3q4);
              -q1q1 + q2q2 - q3q3 + q4q4;
            2*(q2q3 - q1q4);
            2*(q1q3 - q2q4);
            2*(q2q3 + q1q4);
              -q1q1 - q2q2 + q3q3 + q4q4];

% Reshape it into a 3x3n matrix

   n = size(q, 2);
   a = reshape( a9xn, 3, 3*n);
