#!/usr/bin/perl -I/usr/local/lib/perl5
#
# Seth Shulman copies files to us that used to be on the CCS system at GSFC.
# Files are copied to the ingest directory.  Naming conventions are:
#   Definitive Ephemeris - EO1_yyyydoy_def.e
#   WRS2DN               - EO1yyyydoyWRS2DN.L00
#   Predicted Ephemeris  - EO1yyyydoySTKEPH.[LS]00
#       where yyyy is four digit year and doy is 3 digit day of year
#             the rest are constants
#
# This program moves the definitive ephemeris files to the production
# directory and mails the WRS2DN files.
#
use MIME::Lite;

#
# INITIALIZE SOME VARIABLES
#
  $indir = "/home/dps03/incoming"; # Seth copies files here
  $dedir = "/home/eo1/defeph"; # Move Definitive Ephemeris files here
  $dehsm = "/hsm/lta4/eo1/defeph"; # Second copy of the definitive ephemeris files
  $pehsm = "/hsm/lta4/eo1/preeph"; # Second copy of the definitive ephemeris files
  $wrbkdir = "/home/eo1/wrs2dn"; # Org wrs2dn copied here
  $mailto = "michael.neiers.ctr\@usgs.gov";

#
# AND WE'RE OFF
#
  opendir(IDIR, $indir) || fatal_error("$indir: $!");

  while (defined($infile = readdir(IDIR))) {
    if ($infile =~ m/^EO1_(\d\d\d\d)\d\d\d_def.e$/) {
#
#     strip off year and use in destination directory naming convention
#     check for or make the year subdirectory
#     copy the file there and move the original file (dps03 user) to backup dir
#
      $year = $1;

      -d "$dehsm/$year" || mkdir("$dehsm/$year") || warn("mkdir $dehsm/$year: $!");
      -d "$dedir/$year" || mkdir("$dedir/$year") || warn("mkdir $dedir/$year: $!");

      system("/bin/cp $indir/$infile $dehsm/$year");
      system("/bin/mv $indir/$infile $dedir/$year");
      system("cd $dehsm/$year;/home/eo1/bin/DElinks.pl $infile");

    } elsif ($infile =~ m/^EO1\d\d\d\d\d\d\dWRS2DN.L00$/) {
#
#     mail the file (as attachment) to the list of users
#     move the original file (dps03 user) to backup dir
#
      open(INF, "$indir/$infile");

      while (read(INF, $buf, 0x100000)) {
        $data .= $buf;
      }

      close(INF);

      $email = new MIME::Lite
               To => $mailto,
               Subject => "WRS2DN Report",
               Type => "text/plain",
               Encoding => "base64",
               Filename => "$infile",
               Disposition => "attachment",
               Data => $data;
      $email->send;

      system("scp -q $indir/$infile earthe\@edclxs152:/home/earthe/production/data/indata/eo1/
");
      system("/bin/mv $indir/$infile $wrbkdir");
    } elsif ($infile =~ m/^EO1(\d\d\d\d)(\d\d\d)STKEPH.[LS]00/) {
      $year = $1;
      $doy = $2;
      -d "$pehsm/$year" || mkdir("$pehsm/$year") || warn("mkdir $pehsm/$year: $!");
      system("/bin/mv -i $indir/$infile $pehsm/$year");
      # system("ln -s $infile $pehsm/$year/EO1_${year}${doy}_def.e");
    }
  }

  closedir(IDIR);
