#!/usr/bin/perl
use strict;
use vars qw($opt_d $opt_i $opt_T);
use Getopt::Std;

getopts("d:i:T") || die(<<END_USAGE);
Usage:
	$0 [-d dir] [-i #] [-T]
Where:
	-d dir	top-level directory, default is /hsm/lta4/eo1_noarch/raw
	-i #	ingore subdirs with files less than # seconds old (default 3600, or 1 hour)
	-T	test mode: just print, don't do anything
Looks for download subdirectories and moves them to the appropriate year directory.
This is called by eo1ingest_auto and assumes the ingest directory is already locked.
END_USAGE
$opt_d ||= "/hsm/lta4/eo1_noarch/raw";
$opt_i = 3600 unless defined($opt_i);

my $srcdir = "$opt_d/ingest";
opendir(DIR, $srcdir) || die("$srcdir: $!");
my $subdir;
my $now = time();
while (defined($subdir = readdir(DIR))) {
	next unless $subdir =~ m/^(\d\d\d\d)(\d\d\d)\d\d\d\d_(...)/;
	my ($year, $doy, $station) = ($1, $2, $3);
	next unless checktimes("$srcdir/$subdir", $now - $opt_i);
	my $destdir = "$opt_d/$year";
	-d $destdir || mkdir($destdir) || warn("mkdir $destdir: $!");
	execute("mv -i $srcdir/$subdir $destdir/$subdir");
}
closedir(DIR);

# compare the mtime of every file in a directory to a given time and
# return 1 if they are all older.
sub checktimes
{
	my ($subdir, $cutofftime) = @_;
	opendir(SUBDIR, $subdir) || return 0;
	my $file;
	while (defined($file = readdir(SUBDIR))) {
		next if $file =~ m/^\./;
		my @s = stat("$subdir/$file");
		if ($s[9] >= $cutofftime) {
			closedir(SUBDIR);
			return 0;
		}
	}
	closedir(SUBDIR);
	return 1;
}

# Call "system" with the supplied parameters, unless the -T option is
# set, in which case just print the command.
sub execute
	{
	if ($opt_T)
		{
		print "@_\n";
		}
	else
		{
		system @_;
		}
	}

# mkmdirsfor takes a filename argument and creates any directories necessary
sub mkdirsfor
	{
	my ($file) = @_;

	return 0 unless ($file =~ s?/[^/]*$??);
	return 1 if -d $file;
	&mkdirsfor($file);
	if ($opt_T)
		{
		print "mkdir $file\n";
		}
	else
		{
		mkdir($file, 0777);
		}
	}
