#!/usr/local/bin/perl
# Run after eo1ingest.pl to generate browse and inventory updates.
# Expects a tape ID as an argument.
use strict;
use vars qw($opt_v $opt_d $opt_T $opt_t $opt_m $opt_b $opt_M $opt_h $opt_B $opt_s $opt_n);
my $Usage =<<END_USAGE;
Usage: $0 [-v] [-d dir] [-t dir] [-T] [-m] [-b] [-h] [-s sortkey] -n tape...
 or    $0 -M sceneids...
Where
	-v	verbose
	-d dir	Use "dir" as the source directory, default is /edclxs37/eo1
	-t dir	Use "dir" as the temporary directory for generating ALI browse.
	-T	Test mode, just print, don't do anything.
	-m	Generate metadata only.
	-h	Generate HTML metadata, default is plain text.
	-b	Generate browse only.
	tape..	One or more tape IDs, e.g. EO1G53.
	-M sceneids	Generate metadata for the list of sceneids
	-B sceneids	Generate browse for the list of sceneids (requires tape)
	-s sortkey	Sort by "sceneid" or "date", default is date.
	-n	Don't copy the browse to the EE server.
	
END_USAGE
use Getopt::Std;
getopts("vd:t:TmbM:hB:s:n") || die($Usage);
$opt_d ||= "/edclxs37/eo1";
$opt_t ||= "/edclxs37/eo1/tmp";
$ENV{PATH} .= ":/usr/local/bin:/usr/local/bin/pbm:$opt_d/bin";

my @filelist;			# list of files on the tape, from the 00index?????? file
my %filesize;			# sizes (in bytes) of the files in @filelist
my %aliscenes;
my %hypscenes;
my %sceneids;
my $browse_url = "http://edcsns17.cr.usgs.gov/browse/eo-1";
my $sortroutine = \&byacqdate;
if ($opt_s)
	{
	if ($opt_s eq "date")
		{
		$sortroutine = \&byacqdate;
		}
	elsif ($opt_s eq "sceneid")
		{
		$sortroutine = \&bysceneid;
		}
	else
		{
		die("bad sort option: $opt_s");
		}
	}

if ($opt_M)
	{
	foreach (split(/[,\s]/, $opt_M))
		{
		$sceneids{$_} = 1;
		if ($_ =~ m/EO1\d\d\d\d\d\d\d\d\d\d\d\d\d1/)
			{
			$hypscenes{$_} = 1;
			}
		if ($_ =~ m/EO1\d\d\d\d\d\d\d\d\d\d\d\d\d[01]1/)
			{
			$aliscenes{$_} = 1;
			}
		}
	dometadata(sort $sortroutine keys %sceneids);
	}

foreach my $tapeid (@ARGV)
	{
	readindexfile("$opt_d/meta", "00index$tapeid");
	foreach my $file (@filelist)
		{
		if ($file =~ m"^ali/([^/]+)/([^/]+)/([^/]+)(_..._\d\d)\.M[1234]Z$")
			{
			my $sceneid = "$2$4";
			push(@{$aliscenes{$sceneid}}, $file);
			$sceneids{$sceneid}++;
			}
		elsif ($file =~ m"^hyp/([^/]+)/([^/]+)/([^/]+)(_..._\d\d).L0$")
			{
			my $sceneid = "$2$4";
			push(@{$hypscenes{$sceneid}}, $file);
			$sceneids{$sceneid}++;
			}
		}
	if ($opt_B)	# Just generate browse for a list of scenes on this tape
		{
		foreach (split(/[,\s]/, $opt_B))
			{
			if ($aliscenes{$_})
				{
				print "Generate ALI browse for $_\n" if $opt_v;
				alibrowse($_, ${$aliscenes{$_}}[0]);
				}
			if ($hypscenes{$_})
				{
				my $file;
				my $max = 0;
				print "Generate Hyperion browse for $_\n" if $opt_v;
				# The largest file typically contains the image data.
				foreach (@{$hypscenes{$_}})
					{
					if ($max < $filesize{$_})
						{
						$max = $filesize{$_};
						$file = $_;
						}
					}
				hypbrowse($_, $file, ${$hypscenes{$_}}[0]);
				}
			}
		next;
		}
	unless ($opt_m)
		{
		foreach my $scene (sort keys %aliscenes)
			{
			print "Generate ALI browse for $scene\n" if $opt_v;
			alibrowse($scene, ${$aliscenes{$scene}}[0]);
			}
		foreach my $scene (sort keys %hypscenes)
			{
			my $file;
			my $max = 0;
			print "Generate Hyperion browse for $scene\n" if $opt_v;
			# The largest file typically contains the image data.
			foreach (@{$hypscenes{$scene}})
				{
				if ($max < $filesize{$_})
					{
					$max = $filesize{$_};
					$file = $_;
					}
				}
			hypbrowse($scene, $file, ${$hypscenes{$scene}}[0]);
			}
		# Copy the browse to edcsgs17
		unless ($opt_n)
			{
			execute("cd $opt_t/browsetmp;tar cf - ali hyp | ssh -x edcsns17 'cd /edcsns17/browse1/eo-1;tar xBf -'");
			execute("ssh -n -x edcsns17 find /edcsns17/browse1/eo-1 -type f -print > $opt_t/browse.files");
			}
		}
	unless ($opt_b)
		{
		print "Contents of EO-1 level 0 tape $tapeid (", &getstorageloc($tapeid), ")\n";
		dometadata(sort $sortroutine keys %sceneids);
		}
	}

sub dometadata
	{
	if ($opt_h)
		{
		dometadata_html(@_);
		}
	else
		{
		dometadata_plaintext(@_);
		}
	}

sub dometadata_plaintext
		{
		print "\n";
		foreach my $sceneid (@_)
			{
			next unless ($sceneid =~ m/EO1(\d\d\d)(\d\d\d)(\d\d\d\d)(\d\d\d)/);
			my ($path, $row, $year, $yday) = ($1, $2, $3, $4);
			my $short_sceneid = substr($sceneid,0,21);
			my %dat = readdat($sceneid);
			my %rpt_record = readreportfile("$opt_d/$year/$yday/0event_$year$yday.rpt");
			#First line
			printf("%s %s %s\n",
				$sceneid,
				$dat{groundstation},
				($dat{request} ||
					$rpt_record{$short_sceneid}->{'SPECIAL REQUEST'}));
			if ($aliscenes{$sceneid})
				{
				printf("ALI browse <%s/ali/%d/%d/%04d/$sceneid.jpeg>\n",
					$browse_url, $path, $row, $year);
				}
			if ($hypscenes{$sceneid})
				{
				printf("HYP browse <%s/hyp/%d/%d/%04d/$sceneid.jpeg>\n",
					$browse_url, $path, $row, $year);
				}
			print "\n";
			}
		}

sub dometadata_html
		{
		print qq'<table border="1">\n';
		print <<END_HTML;
<tr>
 <th>Scene ID</th>
 <th>Scene Request File</th>
 <th>Site Lat/Lon</th>
 <th>Station</th>
 <th>Duration</th>
 <th>Browse</th>
</tr>
END_HTML
		foreach my $sceneid (@_)
			{
			next unless ($sceneid =~ m/EO1(\d\d\d)(\d\d\d)(\d\d\d\d)(\d\d\d)/);
			my ($path, $row, $year, $yday) = ($1, $2, $3, $4);
			my $short_sceneid = substr($sceneid,0,21);
			my %dat = readdat($sceneid);
			my %rpt_record = readreportfile("$opt_d/$year/$yday/0event_$year$yday.rpt");
			print "<tr>\n";

			# Scene ID
			print " <td>$sceneid</td>\n";
			# Scene Request File
			print " <td>", ($dat{request} ||
				$rpt_record{$short_sceneid}->{'SPECIAL REQUEST'} ||
				"&nbsp;"), "</td>\n";
			# Site Latitude/Longitude
			print qq' <td align="right">',
				($dat{site_lat} || $rpt_record{$short_sceneid}->{'SITE LATITUDE'} || "&nbsp;"),
				"<br>",
				($dat{site_lon} || $rpt_record{$short_sceneid}->{'SITE LONGITUDE'} || "&nbsp;"),
				"</td>\n";
			# Receiving station
			print " <td>", ($dat{groundstation} || "&nbsp;"), "</td>\n";
			# Duration, one line each for ALI and Hyperion
			print " <td>ALI: $dat{duration_ali}<br>Hyp: $dat{duration_hyp}</td>\n";

			# Browse links:
			print "<td>";
			if ($aliscenes{$sceneid})
				{
				printf(qq'<a href="%s/ali/%d/%d/%04d/$sceneid.jpeg">ALI</a>',
					$browse_url, $path, $row, $year);
				}
			else
				{
				print "&nbsp;";
				}
			print "<br>";
			if ($hypscenes{$sceneid})
				{
				printf(qq'<a href="%s/hyp/%d/%d/%04d/$sceneid.jpeg">Hyperion</a>',
					$browse_url, $path, $row, $year);
				}
			else
				{
				print "&nbsp;";
				}
			print "</td>\n";

			print "</tr>\n";
			}
		print "</table>\n";
		}

sub alibrowse
	{
	my ($scene, $file) = @_;
	$file =~ s".*/"";
	if ($scene =~ m/EO1(\d\d\d)(\d\d\d)(\d\d\d\d)(\d\d\d)/)
		{
		my ($path, $row, $year, $day) = ($1, $2, $3, $4);
		my $ali_scene = $scene;
		$ali_scene =~ s/EO1(\d)/EO1A$1/;
		# We need to run cal_pipe to generate the L1 product first.
		# This is done in a temporary directory.
		mkdirsfor("$opt_t/$scene/x");
		execute("ln -s $opt_d/coefs/* $opt_t/$scene");
		$file =~ s/\.M[1234]Z$//;
		foreach my $sca (1..4)
			{
			symboliclink("$opt_d/$year/$day/ali/$ali_scene/$file.M${sca}Z", "$opt_t/$scene/$file.M${sca}Z") || die("$opt_t/$scene/$file.M${sca}Z: $!");
			execute("(cd $opt_t/$scene; /edclxs37/eo1/bin/cal_pipe $file.M${sca}Z LINEAR_Dec13_2001_SCA$sca.SCA$sca)");
			if ($?)
				{
				print STDERR "$scene: cal_pipe failed with $?\n";
				return;
				}
			}
		my $browse_dest = sprintf("$opt_t/browsetmp/%s/%d/%d/%d/%s.jpeg",
			"ali", $path, $row, $year, $scene);
		mkdirsfor($browse_dest);
		execute("alibrowse -b5,4,2 $opt_t/$scene | pnmscale 0.125 | cjpeg > $browse_dest");
		execute("rm -fr $opt_t/$scene\n");
		unless (-s $browse_dest)
			{
			print STDERR "zero length browse removed: $browse_dest\n";
			unlink $browse_dest;
			}
		}
	else
		{
		print STDERR "Bad sceneid passed to alibrowse: $scene\n";
		}
	}

sub hypbrowse
	{
	my ($scene, $file, $cal_file) = @_;
	$file =~ s".*/"";
	$cal_file =~ s".*/"";
	if ($scene =~ m/EO1(\d\d\d)(\d\d\d)(\d\d\d\d)(\d\d\d)/)
		{
		my ($path, $row, $year, $day) = ($1, $2, $3, $4);
		my $hyp_scene = $scene;
		$hyp_scene =~ s/EO1(\d)/EO1H$1/;
		my $pathname = "$opt_d/$year/$day/hyp/$hyp_scene/$file";
		my $cal_pathname = "$opt_d/$year/$day/hyp/$hyp_scene/$cal_file";
		my $browse_dest = sprintf("$opt_t/browsetmp/%s/%d/%d/%d/%s.jpeg",
			"hyp", $path, $row, $year, $scene);
		mkdirsfor($browse_dest);
		if (-f $pathname)
			{
			execute("hypbrowse -g$opt_d/bin/HypGain_revC.dat.svf -b40,31,13 -c$cal_pathname $pathname | pnmscale 0.25 | cjpeg > $browse_dest");
			unless (-s $browse_dest)
				{
				print STDERR "zero length browse removed: $browse_dest\n";
				unlink $browse_dest;
				}
			}
		else
			{
			print STDERR "Missing file $pathname, skipping $scene\n";
			}
		}
	else
		{
		print STDERR "Bad sceneid passed to hypbrowse: $scene\n";
		}
	}
# Read an index file and build the list of files and sizes.
sub readindexfile
	{
	my ($workingdir, $toc_file) = @_;
	open(TOC, "$workingdir/$toc_file") || die("$workingdir/$toc_file: $!");
	my $currentdir = "meta";
	while (<TOC>)
		{
		chomp;
		next if m/^d/;
		if (m/^[-rwx]+\s+\d+\s+\S+\s+\S+\s+(\d+) [A-Z][a-z][a-z] [ \d]\d \d\d:\d\d (.*)/)
			{
			my $file = "$currentdir/$2";
			my $size = $1;
			# The deflate_hyp file is in the hyp directory, but this is not
			# obvious from the index file (at least in the first two tapes).
			$file = "hyp/deflate_hyp" if ($file =~ m/deflate_hyp$/);
			$filesize{$file} = $size;
			push(@filelist, $file);
			}
		elsif (m/(.*):$/)
			{
			$currentdir = $1;
			}
		}
	close(TOC);
	}

# Call "system" with the supplied parameters, unless the -T option is
# set, in which case just print the command.
sub execute
	{
	if ($opt_T)
		{
		print "@_\n";
		}
	else
		{
		system @_;
		}
	}
# Wrapper around symlink that just prints if -T is set.
sub symboliclink
	{
	if ($opt_T)
		{
		print "ln -s @_\n";
		return 1;
		}
	symlink($_[0], $_[1]);
	}

# Find and read the 00meta_<sceneid>*.DAT file and return a hash with some values.
sub readdat
	{
	my ($sceneid) = @_;
	my %dat_file_info;
	unless ($sceneid =~ m/^EO1\d\d\d\d\d\d(\d\d\d\d)(\d\d\d)/)
		{
		print STDERR "Bad sceneid $sceneid passwd to finddat\n";
		return undef;
		}
	my ($year, $month) = ($1, $2);
	my $file;
	my $dat_file_used;
	if (!opendir(DATDIR, "$opt_d/$year/$month/meta"))
		{
		warn("$sceneid: $opt_d/$year/$month/meta: $!");
		print STDERR "No DAT file for $sceneid\n";
		return undef;
		}
	while (defined($file = readdir(DATDIR)))
		{
		if ($file =~ m/^00meta_$sceneid.*DAT$/)
			{
			my $longest_ali = 0;	# Longest duration file (which should be the image)
			my $longest_hyp = 0;	# Longest duration file (which should be the image)
			if (!open(DATFILE, "<$opt_d/$year/$month/meta/$file"))
				{
				warn("$opt_d/$year/$month/meta/$file: $!");
				next;
				}
			$dat_file_info{filename} = $file;
			while (<DATFILE>)
				{
				if ((m/^ALI\s+\S+\s+([0-9\.]+)\s+([0-9\.]+)\s+([0-9\.]+)/) && ($3 > $longest_ali))
					{
					$dat_file_info{starttime_ali} = $1;
					$dat_file_info{stoptime_ali} = $2;
					$dat_file_info{duration_ali} = $3;
					$longest_ali = $3;
					}
				elsif (m/^ALI Start Time\s+(\d\d\d\d)-(\d\d\d) (\d\d):{0,1}(\d\d):{0,1}(\d\d)/)
					{
					$dat_file_info{starttime_ali} = "$1:$2:$3:$4:$5" unless $longest_ali
					}
				elsif (m/^ALI Stop Time\s+(\d\d\d\d)-(\d\d\d) (\d\d):{0,1}(\d\d):{0,1}(\d\d)/)
					{
					$dat_file_info{stoptime_ali} = "$1:$2:$3:$4:$5" unless $longest_ali
					}
				elsif ((m/^HYP\s+\S+\s+([0-9\.]+)\s+([0-9\.]+)\s+([0-9\.]+)/) && ($3 > $longest_hyp))
					{
					$dat_file_info{starttime_hyp} = $1;
					$dat_file_info{stoptime_hyp} = $2;
					$dat_file_info{duration_hyp} = $3;
					$longest_hyp = $3;
					}
				elsif (m/^HYP Start Time\s+(\d\d\d\d)-(\d\d\d) (\d\d):{0,1}(\d\d):{0,1}(\d\d)/)
					{
					$dat_file_info{starttime_hyp} = "$1:$2:$3:$4:$5" unless $longest_hyp
					}
				elsif (m/^HYP Stop Time\s+(\d\d\d\d)-(\d\d\d) (\d\d):{0,1}(\d\d):{0,1}(\d\d)/)
					{
					$dat_file_info{stoptime_hyp} = "$1:$2:$3:$4:$5" unless $longest_hyp
					}
				elsif (m/^Ground Site ID\s*(\S\S\S)/)
					{
					$dat_file_info{groundstation} = $1;
					}
				elsif (m/^Instrument Mode\s+(\d+)/)
					{
					$dat_file_info{imode} = $1;
					}
				elsif (m/Scene Request File\s*(.*)/)
					{
					$dat_file_info{request} = $1;
					}
				elsif (m/Site Latitude\s*(.*)/)
					{
					($dat_file_info{site_lat} = $1) =~ s/\s*$//;
					}
				elsif (m/Site Longitude\s*(.*)/)
					{
					($dat_file_info{site_lon} = $1) =~ s/\s*$//;
					}
				}
			close(DATFILE);
			}
		}
	closedir(DATDIR);
	unless (%dat_file_info)
		{
		if ($sceneid =~ m/(.*)_\d\d$/)
			{
			%dat_file_info = readdat($1);
			if (%dat_file_info)
				{
				print STDERR "Using $dat_file_info{filename} for $sceneid\n";
				}
			else
				{
				print STDERR "No DAT file for $sceneid\n"
				}
			}
		}
	return %dat_file_info;
	}
# Open the eo1ingest.log file and find the tape storage location.
sub getstorageloc
	{
	my ($tapeid) = @_;
	my $storageloc = "";
	open(LOG, "<$opt_d/eo1ingest.log") || return undef;
	while (<LOG>)
		{
		if (m/$tapeid (\S+) Started/)
			{
			$storageloc = $1;
			}
		}
	close(LOG);
	return $storageloc;
	}
# Read a *.rpt file and return a hash (by sceneid) of "records"
sub readreportfile
	{
	my ($reportfile) = @_;
	my $rec;
	my %rpt_records = undef;
	open(RPT, "<$reportfile") || warn("$reportfile: $!");
	while (<RPT>)
		{
		if (m/^ADD (\d\d\d\d:\d\d\d:\d\d:\d\d:\d\d).*(\d\d\d\d:\d\d\d:\d\d:\d\d:\d\d).*'([^']*)'/)
			{
			$rec = {};
			$rec->{starttime} = $1;
			$rec->{endtime} = $2;
			$rec->{recordtype} = $3;
			$rec->{reportfile} = $reportfile;
			next;
			}
		next unless defined $rec;
		if (m/^;$/)
			{
			if (defined($rec->{'SCENE ID'}))
				{
				if (defined($rpt_records{$rec->{'SCENE ID'}}))
					{
					print STDERR "Duplicate scene ID: ",
						$rec->{'SCENE ID'}, " in\n\t",
						$rpt_records{$rec->{'SCENE ID'}}->{reportfile},
						"\n\t$reportfile\n" if $opt_v;
					# Merge the old and new records
					my $oldrec = $rpt_records{$rec->{'SCENE ID'}};
					foreach my $key (%$rec)
						{
						$oldrec->{$key} = $rec->{$key};
						}
					}
				else
					{
					$rpt_records{$rec->{'SCENE ID'}} = $rec;
					}
				}
			undef $rec;
			}
		elsif (m/'([^']*)'\s+=\s+'([^']*)'/)
			{
			$rec->{$1} = $2;
			}
		}
	close(RPT);
	return %rpt_records;
	}
# mkmdirsfor takes a filename argument and creates any directories necessary
sub mkdirsfor
	{
	my ($file) = @_;

	return 0 unless ($file =~ s?/[^/]*$??);
	return 1 if -d $file;
	&mkdirsfor($file);
	if ($opt_T)
		{
		print "mkdir $file\n";
		}
	else
		{
		mkdir($file, 0777);
		}
	}

# Sort by acquisition date
sub byacqdate
	{
	return ((substr($a, 9, 7) cmp substr($b, 9, 7)) || ($a cmp $b));
	}
# Sort by scene ID
sub bysceneid
	{
	$a cmp $b;
	}
