#!/usr/local/bin/perl
# Read a list of ALI scene IDs and regenerate the browse.
use strict;
use vars qw($opt_v $opt_d $opt_D $opt_T $opt_t $opt_f $opt_F $opt_r $opt_L);
my $Usage =<<END_USAGE;
Usage: $0 [-v] [-d dir] [-D dir] [-t dir] [-T] [-f file] [-F] [-r] [-L] [sceneids]
Where
	-v	verbose
	-d dir	Use "dir" as the source directory, default is /home/eo1
	-D dir	Use "dir" as the data directory, default is /hsm/lta4/eo1
	-t dir	Use "dir" as the temporary directory for generating ALI browse.
	-T	Test mode, just print, don't do anything.
	-f file	read a list of scene IDs from file
	-F	Force: overwrite existing browse images (default is to skip)
	-r	remote: run find on edcsns17 to look for old browse
	-L	flipped data: run cal_pipe and alibrowse with -f
END_USAGE
use Getopt::Std;
getopts("vd:D:t:Tf:FrL") || die($Usage);
$opt_d ||= "/home/eo1";
$opt_D ||= "/hsm/lta4/eo1";
$opt_t ||= "tmp";
$ENV{PATH} .= ":/usr/local/bin:/usr/local/bin/pbm:$opt_d/bin";

if ($opt_f && open(F, "<$opt_f"))
	{
	while (<F>)
		{
		chomp;
		alibrowse($_);
		}
	close(F);
	}
if ($opt_r)
	{
	doremote("edcsns17", "/edcsns17/browse1/eo-1/ali");
	}
foreach (@ARGV)
	{
	alibrowse($_);
	}

sub alibrowse
	{
	my ($scene) = @_;
	if ($scene =~ m/EO1A?(\d\d\d)(\d\d\d)(\d\d\d\d)(\d\d\d)/)
		{
		my ($path, $row, $year, $day) = ($1, $2, $3, $4);
		my $ali_scene = $scene;
		my $scene_woA = $scene;
		$ali_scene =~ s/EO1(\d)/EO1A$1/;
		$scene_woA =~ s/EO1A/EO1/;
		my $dirname = "$opt_D/$year/$day/ali/$ali_scene";
		unless (-d $dirname)
			{
			print STDERR "No directory for $scene\n" if $opt_v;
			return;
			}
		my $browse_dest = sprintf("$opt_t/browsetmp/eo-1/%s/%d/%d/%d/%s.jpeg",
			"ali", $path, $row, $year, $scene_woA);
		if (-s $browse_dest && (!$opt_F))
			{
			print STDERR "$scene already done\n" if $opt_v;
			return
			}
		# We need to run cal_pipe to generate the L1 product first.
		# This is done in a temporary directory.
		my $flipped = "";
		my $pnmflip = "";
		if ($opt_L)
			{
			$flipped = "-f";
			$pnmflip = "| pnmflip -lr";
			}
		mkdirsfor("$opt_t/$scene/x");
		execute("ln -s $opt_d/coefs/* $opt_t/$scene");
		execute("ln -s $opt_D/$year/$day/ali/$ali_scene/*.M?Z $opt_t/$scene");
		foreach my $sca (1..4)
			{
			execute("(cd $opt_t/$scene; $opt_d/bin/cal_pipe $flipped *.M${sca}Z LINEAR_Dec13_2001_SCA$sca.SCA$sca) > /dev/null 2>&1");
			if ($?)
				{
				print STDERR "$scene: cal_pipe failed with $?\n";
				execute("rm -fr $opt_t/$scene\n");
				return;
				}
			}
		mkdirsfor($browse_dest);
		execute("alibrowse -b5,4,2 $flipped $opt_t/$scene | pnmscale -quiet 0.125 $pnmflip | cjpeg > $browse_dest");
		execute("rm -fr $opt_t/$scene\n");
		unless (-s $browse_dest)
			{
			print STDERR "zero length browse removed: $browse_dest\n";
			unlink $browse_dest;
			}
		}
	else
		{
		print STDERR "Bad sceneid passed to alibrowse: $scene\n";
		}
	}

# mkmdirsfor takes a filename argument and creates any directories necessary
sub mkdirsfor
	{
	my ($file) = @_;

	return 0 unless ($file =~ s?/[^/]*$??);
	return 1 if -d $file;
	&mkdirsfor($file);
	if ($opt_T)
		{
		print "mkdir $file\n";
		}
	else
		{
		mkdir($file, 0777);
		}
	}

# Call "system" with the supplied parameters, unless the -T option is
# set, in which case just print the command.
sub execute
	{
	if ($opt_T)
		{
		print "@_\n";
		}
	else
		{
		system @_;
		}
	}

sub doremote
	{
	use Time::Local;
	my ($remhost, $remdir) = @_;
	# Compute days since March 20, 2002
	my $days = int((time() - timelocal(0,0,0,21,3-1,2002-1900))/(24 * 3600));
	my $day2 = $days + 2;

	open(SSH, "ssh -x -q $remhost find $remdir -type f -mtime +$days -mtime -$day2 -print |") || die("fork: $!");while (<SSH>)
		{
		chomp;
		next unless s/\.jpeg$//;
		next if m/EO1000000/;	# Skip calibration data, for now.
		s".*/"";
		alibrowse($_);
		}
	close(SSH) || warn("ssh returned $?");
	}
