#!/usr/local/bin/perl
# Read a list of Hyperion scene IDs and regenerate the browse.
use strict;
use vars qw($opt_v $opt_d $opt_D $opt_T $opt_t $opt_f $opt_F $opt_r);
my $Usage =<<END_USAGE;
Usage: $0 [-v] [-d dir] [-D dir] [-t dir] [-T] [-f file] [-F] [-r] [sceneids]
Where
	-v	verbose
	-d dir	Use "dir" as the source directory, default is /home/eo1
	-D dir	Use "dir" as the data directory, default is /hsm/lta4/eo1
	-t dir	Use "dir" as the temporary directory for generating ALI browse.
	-T	Test mode, just print, don't do anything.
	-f file	read a list of scene IDs from file
	-F	Force: overwrite existing browse images (default is to skip)
	-r	remote: run find on edcsns17 to look for old browse
END_USAGE
use Getopt::Std;
getopts("vd:D:t:Tf:Fr") || die($Usage);
$opt_d ||= "/home/eo1";
$opt_D ||= "/hsm/lta4/eo1";
$opt_t ||= "tmp";
$ENV{PATH} .= ":/usr/local/bin:/usr/local/bin/pbm:$opt_d/bin";

if ($opt_f && open(F, "<$opt_f"))
	{
	while (<F>)
		{
		chomp;
		hypbrowse($_);
		}
	close(F);
	}
if ($opt_r)
	{
	doremote("edcsns17", "/edcsns17/browse1/eo-1/hyp");
	}
foreach (@ARGV)
	{
	hypbrowse($_);
	}

sub hypbrowse
	{
	my ($scene) = @_;
	if ($scene =~ m/EO1H?(\d\d\d)(\d\d\d)(\d\d\d\d)(\d\d\d)/)
		{
		my ($path, $row, $year, $day) = ($1, $2, $3, $4);
		my $hyp_scene = $scene;
		my $scene_woH = $scene;
		my @files;
		$hyp_scene =~ s/EO1(\d)/EO1H$1/;
		$scene_woH =~ s/EO1H/EO1/;
		my $dirname = "$opt_D/$year/$day/hyp/$hyp_scene";
		unless (opendir(D, $dirname))
			{
			print STDERR "No directory for $scene\n" if $opt_v;
			return;
			}
		my $file;
		while (defined($file = readdir(D)))
			{
			next unless $file =~ m/\.L0$/;
			push(@files, $file);
			}
		closedir(D);
		@files = sort(@files);

		my $pathname = "$dirname/$files[1]";
		my $cal_pathname = "$dirname/$files[0]";
		my $browse_dest = sprintf("$opt_t/browsetmp/eo-1/%s/%d/%d/%d/%s.jpeg",
			"hyp", $path, $row, $year, $scene_woH);
		if (-s $browse_dest && (!$opt_F))
			{
			print STDERR "$scene already done\n" if $opt_v;
			return
			}
		mkdirsfor($browse_dest);
		if (-f $pathname)
			{
			execute("hypbrowse -b40,31,13 -g$opt_d/bin/HypGain_revC.dat.svf -c$cal_pathname $pathname | pnmscale -quiet 0.25 | cjpeg > $browse_dest");
			unless (-s $browse_dest)
				{
				print STDERR "zero length browse removed: $browse_dest\n";
				unlink $browse_dest;
				}
			}
		else
			{
			print STDERR "Missing file $pathname, skipping $scene\n";
			}
		}
	else
		{
		print STDERR "Bad sceneid passed to hypbrowse: $scene\n";
		}
	}

# mkmdirsfor takes a filename argument and creates any directories necessary
sub mkdirsfor
	{
	my ($file) = @_;

	return 0 unless ($file =~ s?/[^/]*$??);
	return 1 if -d $file;
	&mkdirsfor($file);
	if ($opt_T)
		{
		print "mkdir $file\n";
		}
	else
		{
		mkdir($file, 0777);
		}
	}

# Call "system" with the supplied parameters, unless the -T option is
# set, in which case just print the command.
sub execute
	{
	if ($opt_T)
		{
		print "@_\n";
		}
	else
		{
		system @_;
		}
	}

sub doremote
	{
	use Time::Local;
	my ($remhost, $remdir) = @_;
	# Compute days since May 26, 2004
	my $days = int((time() - timelocal(0,0,0,26,4,104))/(24 * 3600));

	open(SSH, "ssh -x -q $remhost find $remdir -type f -mtime +$days  -print |") || die("fork: $!");while (<SSH>)
		{
		chomp;
		next unless s/\.jpeg$//;
		next if m/EO1000000/;	# Skip calibration data, for now.
		s".*/"";
		hypbrowse($_);
		}
	close(SSH) || warn("ssh returned $?");
	}
