#!/usr/bin/perl -I/usr/local/lib/perl5/site_perl
use strict;
BEGIN {
	my $dir = $0;
	$dir =~ s"/[^/]*$"";
	push(@INC, $dir) if ($dir);
}
use vars qw($Usage $opt_d $opt_D $opt_s $opt_T $opt_X $opt_z
	);
use Fcntl;
use Getopt::Std;
getopts("d:D:Ts:Xz:") || die(<<END_USAGE);
Given one or more scene IDs, regenerate the inventory metadata.
Much of this was take from eo1ingest_auto.pl
Usage: $0 [-d dir] -[-D dir] [-T] [-s c] [-X] sceneids...

Options:
	-d dir		Use "dir" as the working directory (default /home/eo1).
	-D dir		Use "dir" as the data directory (default is /hsm/lta4/eo1)
	-T		test mode
	-s c		set the field separator character to "c", default is "|"
	-X		debug
	-z #		Corner points must be withing # seconds (default is 3).  Use with caution
END_USAGE
$opt_d ||= "/home/eo1";
$opt_D ||= "/hsm/lta4/eo1";
$opt_s ||= "|";
$opt_z = 3 unless defined($opt_z);
$ENV{PATH} = "$opt_d/bin:$ENV{PATH}";
push(@INC, "$opt_d/bin");

use eo1;
use reportfiles;
require "gmt.pl";

my $file;
my %ali;	# arrays of files by scene ID
my %hyp;	# hash of structures
my @aliscenes;	# ALI scenes for which we need to generate browse & metadata.
my @hypscenes;	# Hyperion scenes for which we need to generate browse & metadata.
my %aliexclusions = readexclusions("$opt_d/inventory/exclusions.ali");
my %hypexclusions = readexclusions("$opt_d/inventory/exclusions.hyp");
my %subdirectories;	# subdirectories of the ingest directory that should be removed
my @indexfiles;	# 00indexEO1??? files, to be processed last

my @ali_fieldnames = qw(entity_id orbit_path orbit_row acquisition_date
	cloud_cover quality scene_status rec_station special_request
	scene_start_time scene_center_time scene_stop_time sun_azimuth
	sun_elevation data_type scene_center_lat scene_center_long
	ul_corner_lat ul_corner_long ur_corner_lat ur_corner_long
	ll_corner_lat ll_corner_long lr_corner_lat lr_corner_long
	day_or_night browse_availability browse_image_file_path
	working_dlt_id working_dlt_location archive_dlt_id
	archive_dlt_location raw_dlt_id raw_dlt_location cal_file_used
	file_number directory_name restriction_comments look_angle inclination
	hsm_path file_size num_files);

my @hyp_fieldnames = qw(entity_id orbit_path orbit_row acquisition_date
	cloud_cover quality scene_status rec_station special_request
	scene_start_time scene_center_time scene_stop_time
	sun_azimuth sun_elevation data_type processing_level
	scene_center_lat scene_center_long ul_corner_lat
	ul_corner_long ur_corner_lat ur_corner_long ll_corner_lat
	ll_corner_long lr_corner_lat lr_corner_long day_or_night
	browse_availability browse_image_file_path working_dlt_id
	working_dlt_location archive_dlt_id archive_dlt_location
	working_level1_id working_level1_location archive_level1_id
	archive_level1_location raw_dlt_id raw_dlt_location cal_file_used
	file_number directory_name restriction_comments look_angle inclination
	hsm_path file_size num_files);

foreach my $entity_id (@ARGV)
	{
	if ($entity_id =~ m/EO1A\d\d\d\d\d\d(\d\d\d\d)(\d\d\d)/)
		{
		my $datadir = finddatadir("$opt_D/$1/$2/ali", \$entity_id);
		unless ($datadir)
			{
			&error("No data directory for $entity_id");
			next;
			}
		unless (opendir(DATA, $datadir))
			{
			&error("$datadir: $!");
			next;
			}
		my $file;
		while (defined($file = readdir(DATA)))
			{
			next if $file =~ m/^\./;
			my $info = `eo1info $datadir/$file`;
			if ($? != 0)
				{
				&error("eo1info $datadir/$file");
				}
			chomp $info;
			my ($fname, $sceneid, $ts1, $ts2) = split(/\s+/, $info);
			$ali{$sceneid}->{sceneid} = $sceneid;
			push(@{$ali{$sceneid}->{files}}, $file);
			$ali{$sceneid}->{ts1} = $ts1;
			$ali{$sceneid}->{ts2} = $ts2;
			$ali{$sceneid}->{station_contact} = "unknown";
			$ali{$sceneid}->{file_size} += -s "$datadir/$file";
			$ali{$sceneid}->{num_files}++;
			$ali{$sceneid}->{hsm_path} = $datadir;
			$ali{$sceneid}->{hsm_path} =~ s"edclxs37"hsm/lta4";
			}
		closedir(DATA);
		push(@aliscenes, $entity_id);
		}
	elsif ($entity_id =~ m/EO1H\d\d\d\d\d\d(\d\d\d\d)(\d\d\d)/)
		{
		my $datadir = finddatadir("$opt_D/$1/$2/hyp", \$entity_id);
		unless ($datadir)
			{
			&error("No data directory for $entity_id");
			next;
			}
		unless (opendir(DATA, $datadir))
			{
			&error("$datadir: $!");
			next;
			}
		my $file;
		while (defined($file = readdir(DATA)))
			{
			next if $file =~ m/^\./;
			my $info = `eo1info $datadir/$file`;
			if ($? != 0)
				{
				&error("eo1info $datadir/$file");
				}
			chomp $info;
			my ($fname, $sceneid, $filetype, $ts) = split(/\s+/, $info);
			$hyp{$sceneid}->{sceneid} = $sceneid;
			$hyp{$sceneid}->{station_contact} = "unknown";
			$hyp{$sceneid}->{file_size} += -s "$datadir/$file";
			$hyp{$sceneid}->{num_files}++;
			$hyp{$sceneid}->{hsm_path} = $datadir;
			$hyp{$sceneid}->{hsm_path} =~ s"edclxs37"hsm/lta4";
			push(@{$hyp{$sceneid}->{files}}, $file);
			if ($filetype eq "Image")
				{
				$hyp{$sceneid}->{imagefile} = $file;
				$hyp{$sceneid}->{ts} = $ts;
				}
			}
		closedir(DATA);
		push(@hypscenes, $entity_id);
		}
	else
		{
		&error("Skipped $entity_id");
		}
	}

if (@aliscenes)
	{
	foreach my $entity_id (@aliscenes)
		{
		inventory_metadata("ali", $entity_id, \*STDOUT);
		}
	}

if (@hypscenes)
	{
	foreach my $entity_id (@hypscenes)
		{
		inventory_metadata("hyp", $entity_id, \*STDOUT);
		}
	}

exit(0);

# Generate a line of metadata for a single scene
sub inventory_metadata
	{
	my ($instrument, $entity_id, $fh) = @_;
	my $sceneid = $entity_id;
	$sceneid =~ s/EO1[AH]/EO1/;
	$sceneid =~ s/_..._..$//;
	my $rec;
	$rec->{entity_id} = $entity_id;
	my $rpt_rec = eo1::get_rpt_rec($sceneid, $opt_D);
	if ($opt_X)
		{
		print "rpt_rec for $sceneid:\n";
		foreach my $f (sort keys(%$rpt_rec))
			{
			print "\t$f\t$rpt_rec->{$f}\n";
			}
		}
	# Orbit path/row come from the daily report file.
	$rec->{orbit_path} = $rpt_rec->{PATH};
	$rec->{orbit_row} = $rpt_rec->{'START ROW'};

	# Scene start and stop time:
	#   For ALI use the data gate open time from the HDF header.
	#   For Hyperion use the time range from the daily report file
	#     and run hypimagetimes to find the real times from the
	#     housekeeping files.
	# The fallback for both is to use the times from the daily
	# report file (which tend to be less accurate).
	$rec->{scene_start_time} = $rpt_rec->{starttime};
	if (($instrument eq "ali") && $ali{$sceneid}->{ts1})
		{
		($rec->{scene_start_time} = $ali{$sceneid}->{ts1}) =~
			s/(\d\d\d\d)(\d\d\d)(\d\d)(\d\d)(\d\d)/$1:$2:$3:$4:$5/;
		}

	$rec->{scene_stop_time} = $rpt_rec->{endtime};
	if (($instrument eq "ali") && $ali{$sceneid}->{ts2})
		{
		($rec->{scene_stop_time} = $ali{$sceneid}->{ts2}) =~
			s/(\d\d\d\d)(\d\d\d)(\d\d)(\d\d)(\d\d)/$1:$2:$3:$4:$5/;
		}
	if ($instrument eq "hyp")
		{
		if ($rec->{entity_id} =~ m/EO1[AH]\d\d\d\d\d\d(\d\d\d\d)(\d\d\d)/)
			{
			my ($year, $doy) = ($1, $2);
			my $hyphk = "$opt_D/$year/$doy/meta/${year}_${doy}_hyp.hdf";
			my $info;
			if (defined($rpt_rec->{starttime}) && defined($rpt_rec->{endtime}))
				{
				$info = `hypimagetimes -i "$rpt_rec->{starttime}-$rpt_rec->{endtime}" $hyphk`;
				}
			else
				{
				my $ts1 = $hyp{$sceneid}->{ts};
				$ts1 =~ s/(\d\d\d\d)(\d\d\d)(\d\d)(\d\d)(\d\d)/$1:$2:$3:$4:$5/;
				my $ts2 = gmtadd($ts1, 14);
				$info = `hypimagetimes -i "$ts1-$ts2" $hyphk`;
				}
			chomp $info;
			if ($info =~ m/(\S+) (\S+)/)
				{
				$rec->{scene_start_time} = $1;
				$rec->{scene_stop_time} = $2;
				}
			}
		}

	# Set the acquisition_date from the start time.  As a fallback,
	# if the start time is missing, use the date portion of the scene ID.
	if (defined($rec->{scene_start_time}))
		{
		$rec->{acquisition_date} = yydddtoyymmmdd(substr($rec->{scene_start_time}, 0, 4),
			substr($rec->{scene_start_time}, 5, 3));

		}
	else
		{
		$rec->{acquisition_date} = yydddtoyymmmdd(substr($sceneid, 9, 4), substr($sceneid, 13,3));
		}

	# The special request (which is really the target name) comes from
	# the daily report file, the fallback is to use the value from
	# the HDF header in one of the image files.
	if (defined($rpt_rec->{'SPECIAL REQUEST'}))
		{
		$rec->{special_request} = $rpt_rec->{'SPECIAL REQUEST'};
		}
	else
		{
		my $f;
		if ($instrument eq "ali")
			{
			$f = ${$ali{$sceneid}->{files}}[0];
			}
		else
			{
			$f = ${$hyp{$sceneid}->{files}}[0];
			}
		my %ins = ("A" => "ali", "H" => "hyp");
		if ($entity_id =~ m/EO1([AH])\d\d\d\d\d\d(\d\d\d\d)(\d\d\d)/)
			{
			my $datadir = "$opt_D/$2/$3/$ins{$1}/$entity_id";
			my @info = `eo1info -v $datadir/$f`;
			foreach (@info)
				{
				chomp;
				if (m/Target_Description: (.*)/)
					{
					$rec->{special_request} = $1;
					$rec->{special_request} =~ s/\s+$//;
					last;
					}
				}
			}
		}

	# Get the receiving station from the tail of the entity ID.
	if ($entity_id =~ m/_(...)_..$/)
		{
		$rec->{rec_station} = $1;
		}
	# Get corner point lat/lons.  Note that this depends on the
	# scene start and stop times.
	getcorners($rec);

	# Assume there is browse...
	$rec->{browse_availability} = 'Y';
	$rec->{browse_image_file_path} = generatebrowsepath($entity_id);

	$rec->{file_number} = ($instrument eq "ali" ? 2 : 3);
	my $station_contact;
	if ($instrument eq "ali")
		{
		$station_contact = $ali{$sceneid}->{station_contact};
		$rec->{hsm_path} = $ali{$sceneid}->{hsm_path};
		$rec->{file_size} = $ali{$sceneid}->{file_size};
		$rec->{num_files} = $ali{$sceneid}->{num_files};
		}
	else
		{
		$station_contact = $hyp{$sceneid}->{station_contact};
		$rec->{hsm_path} = $hyp{$sceneid}->{hsm_path};
		$rec->{file_size} = $hyp{$sceneid}->{file_size};
		$rec->{num_files} = $hyp{$sceneid}->{num_files};
		}
	$rec->{directory_name} = "$instrument/$station_contact/$sceneid";
	$rec->{restriction_comments} = get_restriction_comments($rec);
	$rec->{look_angle} = sprintf("%0.4f", compute_lookangle($rec));
	my $inclination = compute_inclination($rec);
	$rec->{inclination} = defined($inclination) ? sprintf("%0.2f", $inclination) : "";

	# Output a single inventory metadata update line:
	my @fieldnames = ($instrument eq "ali" ? @ali_fieldnames: @hyp_fieldnames);
	foreach (@fieldnames)
		{
		print $fh "$rec->{$_}$opt_s";
		}
	print $fh "\n";
	}


sub log
	{
	print STDERR @_,"\n";
	}

# Print an error to STDERR and log it.
sub error
	{
	&log("Error: @_");
	}

# Call error and exit
sub fatal_error
	{
	error(@_);
	exit(1);
	}


# Call "system" with the supplied parameters, unless the -T option is
# set, in which case just print the command.
sub execute
	{
	if ($opt_T)
		{
		print "@_\n";
		}
	else
		{
		system @_;
		}
	}

# Given an entity ID, return the browse path
sub generatebrowsepath
	{
	my $sceneid = $_[0];
	$sceneid =~ s/EO1[AH]/EO1/;
	return sprintf("%d/%d/%d/%s.jpeg", $1, $2, $3, $sceneid)
		if ($sceneid =~ m/EO1(\d\d\d)(\d\d\d)(\d\d\d\d)/);
	return undef;
	}

# Convert year and day of year to year, month, day of month.
sub yydddtoyymmmdd
	{
	my ($year, $day) = @_;
	my @days = (31,28,31,30,31,30,31,31,30,31,30,31);
	my @monthname = qw(Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec);
	# Leap year check.
	if ((($year % 4) == 0) && ((($year % 100) != 0) || ($year % 400 == 0)))
		{
		$days[1] = 29;
		}
	my $month;
	for ($month = 0; $day > $days[$month]; $month++)
		{
		$day -= $days[$month];
		}
	return sprintf("%02d-%s-%4d", $day, $monthname[$month], $year);
	}

# Fill in the corner point coordinates
my %alicorners;
my %hypcorners;
my %footprint_files_read;
sub getcorners
	{
	my $rec = $_[0];
	my $corners;		# pointer to either %alicorners or %hypcorners
	my $entity_id = $rec->{entity_id};
	my $instrument;		# "ali" or "hyp"

	if ($entity_id =~ m/EO1A/)
		{
		$corners = \%alicorners;
		$instrument = "ali";
		}
	else
		{
		$corners = \%hypcorners;
		$instrument = "hyp";
		}
	return unless ($entity_id =~ m/EO1[AH]\d\d\d\d\d\d(\d\d\d\d)(\d\d\d)/);
	my ($year, $doy) = ($1, $2);
	my $footprint_file = "$opt_d/inventory/footprint/${instrument}corners.$year$doy.txt";
	unless ($footprint_files_read{$footprint_file})
		{
		unless (-f $footprint_file)
			{
			execute("cd $opt_d/inventory/footprint;./gencorners.pl -d $opt_d $year/$doy");
			}
		$footprint_files_read{$footprint_file} = 1;
		}
	readcorners("$opt_d/inventory/${instrument}corners.txt", $corners);

	# Find the corner point coordinates
	my $time;
	my $timediff = 99999999;	# difference between target time and corner time
	if ($rec->{scene_start_time})
		{
		foreach (keys %{$corners})
			{
			if ((gmtdiff($rec->{scene_start_time}, $_) >= 0) &&
			   (gmtdiff($_, $rec->{scene_stop_time}) >= 0))
				{
				$timediff = 0;
				$time = $_;
				last;
				}
			my $diff = abs(gmtdiff($_, $rec->{scene_start_time}));
			if ((!defined($timediff)) || ($diff < $timediff))
				{
				$timediff = $diff;
				$time = $_;
				}
			}
		}
	if ($timediff > $opt_z)
		{
		# $missing_corners++;
		print STDERR "No corners for $entity_id $rec->{scene_start_time}, $rec->{scene_stop_time}, closest is $time\n";
		}
	else
		{
		if ($rec->{orbit_row} > 122)
			{
			# Ascending scene: the corner points are "flipped"
			$rec->{ul_corner_lat} = $corners->{$time}->{LRlat};
			$rec->{ul_corner_long} = $corners->{$time}->{LRlon};
			$rec->{ur_corner_lat} = $corners->{$time}->{LLlat};
			$rec->{ur_corner_long} = $corners->{$time}->{LLlon};
			$rec->{ll_corner_lat} = $corners->{$time}->{URlat};
			$rec->{ll_corner_long} = $corners->{$time}->{URlon};
			$rec->{lr_corner_lat} = $corners->{$time}->{ULlat};
			$rec->{lr_corner_long} = $corners->{$time}->{ULlon};
			}
		else
			{
			$rec->{ul_corner_lat} = $corners->{$time}->{ULlat};
			$rec->{ul_corner_long} = $corners->{$time}->{ULlon};
			$rec->{ur_corner_lat} = $corners->{$time}->{URlat};
			$rec->{ur_corner_long} = $corners->{$time}->{URlon};
			$rec->{ll_corner_lat} = $corners->{$time}->{LLlat};
			$rec->{ll_corner_long} = $corners->{$time}->{LLlon};
			$rec->{lr_corner_lat} = $corners->{$time}->{LRlat};
			$rec->{lr_corner_long} = $corners->{$time}->{LRlon};
			}
		# If we have coordinates, compute the sun angles at the center point
		my $ctr_lat = ($corners->{$time}->{ULlat} +
			$corners->{$time}->{URlat} +
			$corners->{$time}->{LLlat} +
			$corners->{$time}->{LRlat}) /4;
		my $ctr_lon = ($corners->{$time}->{ULlon} +
			$corners->{$time}->{URlon} +
			$corners->{$time}->{LLlon} +
			$corners->{$time}->{LRlon}) /4;
		($rec->{sun_elevation},
		 $rec->{sun_azimuth}) = split(/\s+/, 
		 	`sunangles $time $ctr_lat, $ctr_lon`);
		$rec->{day_or_night} = 
			($rec->{sun_elevation} < 0.0) ? "N" : "D";
		}
	}
# Read the corners.txt file and build a hash of corner points by time.
sub readcorners
	{
	my ($filename, $corners) = @_;
	my $time;
	open(CORNERS, "<$filename") || return;
	while (<CORNERS>)
		{
		chomp;
		if (m/^\d\d\d\d:\d\d\d:\d\d:\d\d:\d\d/)
			{
			$time = $_;
			}
		elsif (m/(UL|UR|LL|LR):\s*Lat\s*([-0-9\.]+)\s*Lon\s*([-0-9\.]+)/)
			{
			$corners->{$time}->{"${1}lat"} = $2;
			$corners->{$time}->{"${1}lon"} = $3;
			}
		}
	close(CORNERS);
	}

# Compare two dates, in the form "01-Apr-2002", return -1, 0, 1 if the
# first is (respectively) less than, equal to or greater than the second.
sub cmpdate
	{
	my ($a, $b) = @_;
	my ($a_year, $a_month, $a_day);
	my ($b_year, $b_month, $b_day);
	my %monthno = (qw(Jan 01 Feb 02 Mar 03 Apr 04 May 05 Jun 06 Jul 07 Aug 08 Sep 09 Oct 10 Nov 11 Dec 12));
	if ($a =~ m"(\d+)[-/]([A-Z][a-z][a-z])[/-](\d\d\d\d)")
		{
		$a_year = $3;
		$a_day = $1;
		$a_month = $monthno{$2};
		}
	if ($b =~ m"(\d+)[-/]([A-Z][a-z][a-z])[/-](\d\d\d\d)")
		{
		$b_year = $3;
		$b_day = $1;
		$b_month = $monthno{$2};
		}
	($a_year <=> $b_year) || ($a_month <=> $b_month) || ($a_day <=> $b_day);
	}
# Look for restrictions on this scene.
sub get_restriction_comments
	{
	my $rec = $_[0];
	my @restrictions;		# to be returned
	my %exclusions;
	my $sceneid = $rec->{entity_id};
	$sceneid =~ s/EO1[AH]/EO1/;
	$sceneid =~ s/_..._..$//;

	push(@restrictions, "Calibration") if (($sceneid =~ m/^EO1000000/) ||
		($rec->{special_request} =~ m/90Yaw/));
	push(@restrictions, "Instrument check-out")
		if (cmpdate($rec->{acquisition_date}, "15-Mar-2001") < 0);
	push(@restrictions, "DOD collect")
		if ($rec->{special_request} =~ m/Warfigher|Paules|W\.A\.A\./);
	push(@restrictions, "PPT test scene")
		if (($rec->{acquisition_date} eq "07-Jan-2002") ||
		    ($rec->{acquisition_date} eq "14-Mar-2002") ||
		    ($rec->{acquisition_date} eq "25-Apr-2002"));
	if ($rec->{entity_id} =~ m/EO1A/)
		{
		%exclusions = %aliexclusions;
		}
	if ($rec->{entity_id} =~ m/EO1H/)
		{
		%exclusions = %hypexclusions;
		push(@restrictions, "Israel")
			if ($rec->{entity_id} =~ m/^EO1H174039/);
		}
	foreach (keys %exclusions)
		{
		if ($sceneid =~ m/$_/)
			{
			push(@restrictions, $exclusions{$_});
			}
		}
	return(join(";", @restrictions));
	}


# Read a file with lines like this:
#	EO10920842001329112KP Frame rate anomaly
# extract the scene ID and point to the exclusion reason.  Returns a hash.
sub readexclusions
	{
	my %ex;
	unless (open(EX, "<$_[0]"))
		{
		warn("$_[0]: $!");
		return undef;
		}
	while (<EX>)
		{
		chomp;
		if (m/(\S+)\s+(.*)/)
			{
			$ex{$1} = $2;
			}
		}
	close(EX);
	return %ex;
	}

sub compute_lookangle
	{
	my $rec = $_[0];
	my $starttime = $rec->{scene_start_time};
	my $stoptime = $rec->{scene_stop_time};
	my $instrument = ($rec->{entity_id} =~ m/EO1A/ ? "ali" : "hyp");
	unless ($starttime =~ m/^(\d\d\d\d):?(\d\d\d)/)
		{
		print STDERR "$0: Starttime $starttime, no lookangles for $rec->{entity_id}\n";
		return undef;
		}
	my $year = $1;
	my $doy = $2;
	$starttime =~ s/://g;
	$stoptime =~ s/://g;
	foreach my $acs_filename (glob("$opt_D/$year/$doy/meta/*_acs.hdf"))
		{
		print "cd $opt_d/inventory/lookangle;./lookangleparams $acs_filename $starttime $stoptime | octave -q\n" if $opt_X;
		system "cd $opt_d/inventory/lookangle;./lookangleparams $acs_filename $starttime $stoptime" if $opt_X;
		my $output = `cd $opt_d/inventory/lookangle;./lookangleparams $acs_filename $starttime $stoptime | octave -q`;
		if (($? == 0) && $output =~ m/ALI_lookangle\s*=\s*([-0-9\.e]+).*\n.*HSI_lookangle\s*=\s*([-0-9\.e]+)/)
			{
			return ($instrument eq "ali" ? $1 : $2);
			}
		}
	print STDERR "$0: No lookangles for $rec->{entity_id}\n";
	return undef;
	}

sub compute_inclination
	{
	my $rec = $_[0];
	my $starttime = $rec->{scene_start_time};
	my $stoptime = $rec->{scene_stop_time};
	unless ($starttime =~ m/^(\d\d\d\d):?(\d\d\d)/)
		{
		print STDERR "$0: Starttime $starttime, no inclination for $rec->{entity_id}\n";
		return undef;
		}
	my $year = $1;
	my $doy = $2;
	$starttime =~ s/://g;
	$stoptime =~ s/://g;
	foreach my $acs_filename (glob("$opt_D/$year/$doy/meta/*_acs.hdf"))
		{
		my $output = `cd $opt_d/inventory/lookangle;./inclination $acs_filename $starttime $stoptime`;
		if (($? == 0) && ($output =~ m/([-0-9\.e]+)/))
			{
			return $1;
			}
		}
	print STDERR "$0: No inclination for $rec->{entity_id}\n";
	return undef;
	}

# Given a year and day-of-year, check to see if all the housekeeping
# files are in place.
sub checkhousekeeping
	{
	my ($dir, $year, $doy) = @_;
	foreach my $hk (qw(acs gps ali hyp))
		{
		unless (-f "$opt_D/$year/$doy/meta/${year}_${doy}_${hk}.hdf")
			{
			&log("Missing $opt_D/$year/$doy/meta/${year}_${doy}_${hk}.hdf");
			return 0;
			}
		}
	# We also need either the report file or the daily text file.
	my $two_digit_year = substr($year, 2, 2);
	unless (-f "$opt_D/$year/$doy/0event_${year}${doy}.rpt" ||
		-f "$opt_D/$year/$doy/meta/daily_${two_digit_year}${doy}.txt")
			{
			&log("Missing rpt file for $year/$doy $opt_D/$year/$doy/0event_${year}${doy}.rpt || $opt_D/$year/$doy/meta/daily_${two_digit_year}${doy}.txt");
			return 0;
			}
	return 1;
	}

# Given a parent directory and a (possibly partial) scene ID, find
# the a data directory.
sub finddatadir
	{
	my ($parentdir, $sceneidref) = @_;
	return "$parentdir/$$sceneidref" if (-d "$parentdir/$$sceneidref");
	unless (opendir(PARENT, $parentdir))
		{
		&error("$parentdir: $!");
		return undef;
		}
	my $file;
	while (defined($file = readdir(PARENT)))
		{
		next if $file =~ m/^\./;
		if (($file =~ m/^$$sceneidref/) && -d "$parentdir/$file")
			{
			closedir(PARENT);
			$$sceneidref = $file;
			return "$parentdir/$file";
			}
		}
	closedir(PARENT);
	return undef;
	}
