package eo1;
use strict;
use vars qw (@ISA @EXPORT);
use Exporter;
@ISA = qw(Exporter);
@EXPORT = qw(get_rpt_rec);

my %rpt_records;	# Records from the report files, by sceneid
my %rpt_files_read;	# Previously processed report files, by name

sub get_rpt_rec
	{
	my ($sceneid, $dir) = @_;
	return $rpt_records{$sceneid} if ($rpt_records{$sceneid});

	return undef unless $sceneid =~ m/EO1\d\d\d\d\d\d(\d\d\d\d)(\d\d\d)/;
	my ($year, $doy) = ($1, $2);
	foreach my $rpt_file (find_rpt_files($year, $doy, $dir))
		{
		if ($rpt_file && !$rpt_files_read{$rpt_file})
			{
			readreportfile($rpt_file, \%rpt_records);
			$rpt_files_read{$rpt_file} = 1;
			}
		}
	unless ($rpt_records{$sceneid})
		{
		# Try the day before.
		my $y = $year;
		my $d = $doy - 1;
		if ($d < 1)
			{
			$year--;
			if ((($year % 4) == 0) && ((($year % 100) != 0) || ($year % 400 == 0)))
				{
				$d = 366;
				}
			else
				{
				$d = 365;
				}
			}
		foreach my $rpt_file (find_rpt_files($y, $d, $dir))
			{
			if ($rpt_file && !$rpt_files_read{$rpt_file})
				{
				readreportfile($rpt_file, \%rpt_records);
				$rpt_files_read{$rpt_file} = 1;
				}
			}
		}
	
	unless ($rpt_records{$sceneid})
		{
		# Try the day after.
		my $y = $year;
		my $d = $doy + 1;
		my $maxday = 365;
		if ((($year % 4) == 0) && ((($year % 100) != 0) || ($year % 400 == 0)))
			{
			$maxday = 366;
			}
		if ($d > $maxday)
			{
			$d = 1;
			$year++;
			}
		foreach my $rpt_file (find_rpt_files($y, $d, $dir))
			{
			if ($rpt_file && !$rpt_files_read{$rpt_file})
				{
				readreportfile($rpt_file, \%rpt_records);
				$rpt_files_read{$rpt_file} = 1;
				}
			}
		}
	if ($rpt_records{$sceneid})
		{
		unless ($rpt_records{$sceneid}->{'SPECIAL REQUEST'})
			{
			# Calibration data
			$rpt_records{$sceneid}->{'SPECIAL REQUEST'} = $rpt_records{$sceneid}->{'INTCAL_TYPE'};
			}
		return $rpt_records{$sceneid}
		}

	# Not found
	return undef;
	}
sub find_rpt_files
	{
	my ($year, $doy, $dir) = @_;
	my $rpt_file = undef;
	my @rpt_files = ();
	$doy = sprintf("%03d", $doy);

	$rpt_file = "$dir/$year/$doy/0event_${year}${doy}.rpt";
	push(@rpt_files, $rpt_file) if -f $rpt_file;

	$rpt_file = "$dir/$year/$doy/0event_${year}${doy}.txt";
	push(@rpt_files, $rpt_file) if -f $rpt_file;

	my $two_digit_year = substr($year, 2, 2);
	$rpt_file = "$dir/$year/$doy/meta/daily_${two_digit_year}${doy}.txt";
	push(@rpt_files, $rpt_file) if -f $rpt_file;

	return @rpt_files;
	}
# Read a *.rpt file and return a hash (by sceneid) of "records".
# Optionally, if the third parameter is a reference to an array,
# pushd each record (not just ScheduledScene records) on to the array.
sub readreportfile
	{
	my ($reportfile, $rpt_records, $rpt_array) = @_;
	my $rec;
	open(RPT, "<$reportfile") || warn("$reportfile: $!");
	while (<RPT>)
		{
		if (m/^ADD (\d\d\d\d:\d\d\d:\d\d:\d\d:\d\d).*(\d\d\d\d:\d\d\d:\d\d:\d\d:\d\d).*'([^']*)'/)
			{
			saverec($rec, $rpt_records, $rpt_array);
			$rec = {};
			$rec->{starttime} = $1;
			$rec->{endtime} = $2;
			$rec->{recordtype} = $3;
			$rec->{reportfile} = $reportfile;
			next;
			}
		next unless defined $rec;
		if (m/^;$/)
			{
			saverec($rec, $rpt_records, $rpt_array);
			undef $rec;
			}
		elsif (m/'([^']*)'\s+=\s+'(.*)'/)
			{
			my ($name, $value) = ($1, $2);
			$value =~ s/''/'/g;
			$rec->{$name} = $value;
			}
		}
	saverec($rec, $rpt_records, $rpt_array);
	close(RPT);
	}
sub saverec
	{
	my ($rec, $rpt_records, $rpt_array) = @_;
	return unless defined($rec);
	push(@{$rpt_array}, $rec) if (defined($rpt_array));
	return unless defined($rpt_records);
	return unless (defined($rec->{'SCENE ID'}));
	if (defined($rpt_records->{$rec->{'SCENE ID'}}))
		{
		# Merge the records
		my $prev_rec = $rpt_records->{$rec->{'SCENE ID'}};
		foreach my $f (keys %$rec)
			{
			$prev_rec->{$f} = $rec->{$f};
			}
		}
	else
		{
		$rpt_records->{$rec->{'SCENE ID'}} = $rec;
		}
	}
1;
