#!/usr/local/bin/perl
use strict;
use vars qw($opt_d);
use Getopt::Std;
getopts("d:") || die("quit");
$opt_d ||= "/edclxs37/eo1";


my $tapeid;
my $tapeseq;
foreach my $tapeid (@ARGV)
	{
	if (-f "$tapeid.html")
		{
		print STDERR "$tapeid.html already exists, skipping\n";
		next;
		}
	if (-f "$opt_d/meta/00index$tapeid")
		{
		$tapeseq = getnexttapeseq();
		}
	else
		{
		die("No file: $opt_d/meta/$tapeid");
		}
	# update the "eo1tapes" file with the tape storage location
	update_tape_location($tapeid);

	# Generate new cornerpoints
	system("cd footprint;./gencorners.pl;make");

	# Generate browse
	system("$opt_d/bin/posteo1ingest.pl -b $tapeid");
	system("cp $opt_d/tmp/browse.files .");
	system("rm -fr $opt_d/tmp/browsetmp/???");

	# Generate email messages to be sent to various people
	system("$opt_d/bin/posteo1ingest.pl -m -h $tapeid > $tapeid.html");
	edit("$tapeid.html", $tapeid, $tapeseq);
	system("$opt_d/bin/posteo1ingest.pl -m -h -s sceneid $tapeid > $tapeid.sceneid.html");
	edit("$tapeid.sceneid.html", $tapeid, $tapeseq);
	system("$opt_d/bin/posteo1ingest.pl -m $tapeid > $tapeseq.txt");
	edit("$tapeseq.txt", $tapeid, $tapeseq);
	# system("rcp $tapeid.html sg1:public_html/eo1/tape$tapeseq.html");
	system("sh send $tapeid");
	system("sh send2 $tapeseq");

	# Generate metadate updates for the inventory database
	system("sh generatemetadata $tapeid");
	}

sub getnexttapeseq
	{
	my $file;
	my $maxseq = 0;
	opendir(CWD, ".") || die(".: $!");
	while (defined($file = readdir(CWD)))
		{
		next unless $file =~ m/^(\d+)\.txt$/;
		$maxseq = $1 if $1 > $maxseq;
		}
	closedir(CWD);
	return($maxseq + 1);
	}

sub edit
	{
	my ($filename, $tapeid, $tapeseq) = @_;
	open(FILE, "<$filename") || die("$filename: $!");
	my @contents = <FILE>;
	close(FILE);
	$contents[0] =~ s/$tapeid.*$/$tapeseq ($tapeid)/;
	open(FILE, ">$filename") || die("$filename: $!");
	print FILE @contents;
	close(FILE);
	}

sub update_tape_location
	{
	my ($tapeid) = @_;

	unless (open(INGESTLOG, "<$opt_d/eo1ingest.log"))
		{
		print STDERR "$opt_d/eo1ingest.log: $!\n";
		print STDERR "Skipping the tape location update\n";
		return;
		}
	my $tape_location = undef;
	while (<INGESTLOG>)
		{
		next unless m/$tapeid (.+) Started/;
		$tape_location = $1;
		# Keep going so we get the _last_ one, just in case it was restarted.
		}
	close(INGESTLOG);
	if ($tape_location)
		{
		unless (open(TAPELOCS, ">>eo1tapes"))
			{
			print STDERR "eo1tapes:$!\n";
			print STDERR "Skipping the tape location update\n";
			return;
			}
		print TAPELOCS "$tapeid $tape_location\n";
		close(TAPELOCS);
		print STDERR "updated eo1tapes with $tapeid $tape_location\n";
		}
	}
