#!/bin/csh -f

# Change Log:
#  12/18/08  clf  Updated code for move from lta1 to lta4 -- cr 17680

# Do any necessary set up and change to the working directory
set opsbin      = "/home/ops/bin"
set completedir = "/home/ops/EO1_completed"
set procdir     = "/home/ops/EO1_proc"
set workdir     = "/home/ops/EO1_work"
set arc_dir     = "/home/ops/EO1_archive"
set errordir    = "/home/ops/EO1_error"
set logfile     = "/home/ops/EO1_log/archive.log"
set tarlocation = "archive/holding_area"
set tarsystem   = "edcsns7"
set exitstatus = 0

# First see if there is a file to process
set current_tar = `ls -1 $arc_dir | grep cur | head -1`
if ( "$current_tar" == "" ) then
    exit 0
endif
# Extract the tarfile name
set tarfile = `echo $current_tar | awk -F'.' '{print $1}'`
if ( ! -e $arc_dir/$tarfile ) then
    echo -n `date +'%D'`"   "`date +'%T'` >> $logfile
    echo "  Error $arc_dir/$tarfile does not exist" >> $logfile
    mv $arc_dir/$current_tar $errordir/.
    exit 1
endif
set timestamp = `echo $tarfile | awk -F'_' '{print $2}'`
set metafile = "metadatalist_$timestamp"

# Don't want the ingest_e01data process messing with the regular cron process
# so do it in a subdirectory.
set datestr = `date +%Y%m%d%H%M%S`
set workdir = $workdir/Temp_work_$datestr
mkdir -p $workdir
cd $workdir
mv $arc_dir/$current_tar $arc_dir/$tarfile $arc_dir/$metafile .

# Now comes a tricky part.  It could be weeks from the time the tarlist was
# begun, maybe several weeks.  In the interim metadata and houskeeping files
# may have been added/updated.  So what to do?  First impulse is to use the
# metadata file for the this tarlist and go through the entire process again:
# locating all the files associated with the entity; rebuilding the tarlist;
# and recomputing the size of the archive.  Hopefully not too many large files
# have been added putting the size over the 200G limit.  Of course that size
# does not take into consideration hardware compression so it might be okay.
# Part of the first impulse is to call this process again maybe using a
# command line parameter to indicate the use of an entirely new set of
# current_tar and tarlisting.  Don't have a second idea yet.
# Not sure where this data file will be placed but it will contain the tape ID
# followed by the original tarlist.  The values in the file will be used by
# this script later to determine the metadata file and storage location.

# Calling it this way will recreate the tarlist completely
$opsbin/ingest_e01data.csh --tarfile $tarfile --skip-assoc-checks
if ( $status ) then
    echo -n `date +'%D'`"   "`date +'%T'` >> $logfile
    echo "  Error recreating the tar" >> $logfile
    set exitstatus = 1
else
    # Send a work order file to the archive system.  An attempt to give some
    # priority to tar lists that have been in the works for some time, give
    # the work order the same time stamp as the tar list.
    set tstamp = `echo $tarfile | awk -F'_' '{print $2}'`
    set workorder = "workorder_$tstamp"
    echo "$HOST|$procdir|eo1_offsite|/hsm/lta4/eo1|$tarfile" > $workorder
    scp -P 22 $workorder ${tarsystem}:${tarlocation}/.
    if ( $status ) then
        echo -n `date +'%D'`"   "`date +'%T'` >> $logfile
	echo "  Error copying $workorder to ${tarsystem}:$tarlocation" \
	    >> $logfile
	set exitstatus = 1
    else
	# Copy the tarlist to the tape archive system
	scp -P 22 $tarfile ${tarsystem}:${tarlocation}/.
	if ( $status ) then
	    echo -n `date +'%D'`"   "`date +'%T'` >> $logfile
	    echo "  Error copying $tarfile to archive system" >> $logfile
	    set exitstatus = 1
        endif
    endif
endif

if ( $exitstatus ) then
    echo -n `date +'%D'`"   "`date +'%T'` >> $logfile
    echo "  Files are in $workdir" >> $logfile
    echo " EO1 error: Files are in $workdir" > mailfile
    echo " $workorder $tarfile" >> mailfile
    #Mail -s "EO1 error" dlloyd@usgs.gov < mailfile
else
    echo -n `date +'%D'`"   "`date +'%T'` >> $logfile
    echo "  archive_eo1data successful completion" >> $logfile
    mv $metafile $procdir/.
    mv $tarfile $current_tar $completedir/Old_tarfiles/.
    echo " Created EO1 workorder" > mailfile
    echo " $workorder $tarfile" >> mailfile
    #Mail -s "EO1 Archive Information" dlloyd@usgs.gov < mailfile
    cd
    rm -r $workdir
endif

exit $exitstatus
