#!/bin/csh -f

#
# Change Log:
#    07/30/08  clf  Updated code to use /hsm instead of /dmf -- cr 17116
#    12/18/08  clf  Updated code to use lta4 instead of lta1 -- cr 17680
#    05/07/09  clf  Updated finddir from eo1/raw to eo1_noarch/raw -- cr 18295
#

set workdir = "/home/ops/EO1_work/raw_work"
set wofile = "/home/ops/EO1_work/raw_work/raw_workorder"
set logfile = "/home/ops/EO1_log/archive_raw.log"
set bindir  = "/home/ops/bin"
set finddir = "/hsm/lta4/eo1_noarch/raw"
set archsys = "edcsns7"
set archdir = "/hsm/home/lta/ops/archive/archive"
set exit_status = 0
set lockfil = "/edclxs37/data11/raw/cplock"
set progman = "@usgs.gov"

# Need an argument to give us the proper working directory
#---------------------------------------------------------
if ( ${#argv} == 1 ) then
    if ( "$1" == "sband" ) then
        set workdir = "/home/ops/EO1_work/sband_work"
        set wofile = "/home/ops/EO1_work/sband_work/sband_workorder"
        set finddir = "/hsm/lta4/eo1/Sbandraw"
    endif
endif

# Once the trigger point to make a tape has been reached, the NoRun file is
# placed in the working directory to prevent new workorders from being created
# until the silo has been cleaned.  Until the silo has been cleaned every
# time this process runs it will produce a new work order.
#-----------------------------------------------------------------------------
cd $workdir
if ( -e NoRun ) then
    echo "Will not execute due to NoRun file" >> $logfile
    exit 0
endif
rm MailFile

# Don't do the check if files are being copied.  There was a problem
# with a tape where one file didn't make the cut.  We believe this 
# happened during a file copy and the trigger limit was reached before
# the last file began copying.  Rich Lee says the lock file will exist
# during transfers.
# This loop has a good chance of being infinite so give up after an hour
#-----------------------------------------------------------------------
set num_sleeps = 60
set wait_flag = 1
while ( $wait_flag )
    if ( -e $lockfil ) then
        @ $num_sleeps--
        sleep 60
    else
        set wait_flag = 0
    endif
    if ( $num_sleeps == 0 ) then
        #Mail -s "$0 Giving up on wait" $progman
        exit 1
    endif
end

# Sorting the file into groups is necessary because the find command does
# not always give the list in groups.  The files must be grouped in order
# to keep them on the same tape.  The files are in groupls of four for
# regular raw -- still don't know about sband.
#
# eo1raw.list is a list of files currently on the silo NOT in proper order
#-------------------------------------------------------------------------
ssh edcsns7 "find $finddir -type f -exec ls -l {} \;" > eo1raw.list

# Delimit the listing so there is a pipe around file size and before name
#-------------------------------------------------------------------------
sed -f sedcoms eo1raw.list > delimited_list

# Sort the list on file name so the files are grouped.  For example:
#
# /hsm/lta4/eo1/raw/20060642007_SGS/2006064210855.dqm
# /hsm/lta4/eo1/raw/20060642007_SGS/2006064212019.dqm
# /hsm/lta4/eo1/raw/20060642007_SGS/e2006064210855.f3
# /hsm/lta4/eo1/raw/20060642007_SGS/e2006064212019.f3
#
# or
# /hsm/lta4/eo1/Sbandraw/2005_240_000_I00
# /hsm/lta4/eo1/Sbandraw/2005_240_000_Q01
# /hsm/lta4/eo1/Sbandraw/2005_240_000_Q02
# /hsm/lta4/eo1/Sbandraw/2005_240_000_Q04
#
# is a proper group of files.  Note that the sorted list should be used in
# place of eo1raw.list from here on.
#-------------------------------------------------------------------------
sort -t '|' --key=4 delimited_list > sorted_list

# Now use the sorted_list to extract the file sizes
#-------------------------------------------------------------------------
awk -F'|' '{printf("%s\n",$2)}' sorted_list > file_sizes

# And calculate the total size of the list
#-----------------------------------------
$bindir/sum_list < file_sizes > file_total

# See if the trigger point has been reached.
#------------------------------------------
set lim = `grep OVERLIMIT file_total`
if ( "$lim" != "" ) then
echo "**** Automatically running: run_check_eo1raw ****" > mailfile
#Mail -s "$0 reached limit exiting" $progman < mailfile
rm mailfile
$bindir/run_check_eo1raw
exit 1

    echo -n `date +'%D'`"   "`date +'%T'` >> $logfile
    echo "  Time to make a tape" >> $logfile
    cat file_total >> $logfile
    set tstamp = `date +'%Y%m%d%H%M%S'`

    # Prevent further runs of this process until silo cleaned
    #--------------------------------------------------------
    touch NoRun

    # Get the number of files to archive and shorten the sorted list to
    # this number
    #------------------------------------------------------------------
    set nfiles = `grep ARCHIVE file_total | awk -F':' '{print $2}'`
    head -n $nfiles sorted_list > sorted_list_$tstamp

    # Grab the IDs from the file (uniqe_IDs) and get the count of files for
    # each ID (ID_count).
    #---------------------------
    awk -F'/' '{print $6}' sorted_list_$tstamp | sort -u > uniqe_IDs
    echo "Checking for file counts of four (4)" >> MailFile
    set bail = "NO"
    set i = 0
    while ( $i < $nfiles )
        @ i++
	set id = `head -$i uniqe_IDs | tail -1`
	set num_IDs = `grep $id sorted_list_$tstamp| wc | \
	    awk -F' ' '{print $1}'`
	if ( $num_IDs != 4 ) then
	    set bail = "YES"
	    echo "$id $num_IDs" >> MailFile
	endif
    end
    if ( "$bail" == "YES" ) then
        #Mail -s "Problem with EO-1 raw" $progman < MailFile
	exit 1
    endif

    # As an additional check, there should be four files in each group
    # if there isn't a multiple of four let me know and exit
    #-----------------------------------------------------------------
    set num_files = `grep ARCHIVE file_total | awk -F':' '{print $2}'`
    if ( $num_files % 4 != 0 ) then
        #Mail -s "$0 odd count check it out" $progman
        exit 1
    endif

    # Here's another check
    #---------------------
    if ( "$1" == "sband" ) then
        set n1 = `grep _I00 sorted_list_$tstamp | wc | awk -F' ' '{print $1}'`
        set n2 = `grep _Q01 sorted_list_$tstamp | wc | awk -F' ' '{print $1}'`
        set n3 = `grep _Q02 sorted_list_$tstamp | wc | awk -F' ' '{print $1}'`
        set n4 = `grep _Q04 sorted_list_$tstamp | wc | awk -F' ' '{print $1}'`
        if ( $n1 != $n2 ) then
            #Mail -s "EO-1 raw Sband bad count in file" $progman
            exit 1
        endif
        if ( $n1 != $n3 ) then
            #Mail -s "EO-1 raw Sband bad count in file" $progman
            exit 1
        endif
        if ( $n1 != $n4 ) then
            #Mail -s "EO-1 raw Sband bad count in file" $progman
            exit 1
        endif
    else
        set n1 = `grep dqm sorted_list_$tstamp | wc | awk -F' ' '{print $1}'`
        set n2 = `grep f3 sorted_list_$tstamp | wc | awk -F' ' '{print $1}'`
        if ( $n1 != $n2 ) then
            #Mail -s "EO-1 raw bad count in file" $progman
            exit 1
        endif
    endif
    
    # Create the tarlisting from the shortened list and the workorder
    #----------------------------------------------------------------
    sed 's/\/hsm\/lta4\//|/' sorted_list_$tstamp | awk -F'|' '{print $5}' \
        > tarfile_$tstamp
    sed "s/XXXYYY/tarfile_$tstamp/" $wofile > workorder_$tstamp
    
    # Send the workorder to the archive system
    #-----------------------------------------
    scp tarfile_$tstamp ${archsys}:$archdir/.
    if ( $status ) then
        echo "Error copying tarfile_$tstamp to ${archsys}" >> $logfile
	set exit_status = 1
    else
        scp workorder_$tstamp ${archsys}:$archdir/.
	if ( $status ) then
	    echo "Error copying workorder_$tstamp to ${archsys}" >> $logfile
	    set exit_status = 1
	endif
    endif

    if ( $exit_status ) then
        echo "Error copying workorder to $archsys" >> MAilFile
    else
        echo "Workorder copied successfully" >> MAilFile
	mv file_total file_total_$tstamp
	mv tarfile_$tstamp workorder_$tstamp sorted_list_$tstamp old/.
        mv file_total_$tstamp old/.
    endif

    #Mail -s "EO-1 raw process" $progman < MAilFile
    rm MAilFile
endif

exit $exit_status

