/* The input file, which is a sorted ls -l, looks like this:

(In the following I have replaced '-rw-rw-r-- 1 eo1 eo1' with 'JUNK' for
formatting issues)

JUNK|3009|2007-02-07 01:58|/dmf/eo1/raw/20070370459_PF1/2007037050050.dqm
JUNK|3193|2007-02-07 01:58|/dmf/eo1/raw/20070370459_PF1/2007037050100.dqm
JUNK|2313281792|2007-02-07 02:01|/dmf/eo1/raw/20070370459_PF1/e2007037050050.f3
JUNK|2332757504|2007-02-07 02:05|/dmf/eo1/raw/20070370459_PF1/e2007037050100.f3

The pipes have been added to delimit the fields of real importance:

field 1 - no importance for this function
field 2 - file size
field 3 - no importance for this function
field 4 - file

There are supposed to be 4 files as listed above; however, there can be 6
or 8 according to the man.  I suppose any multiple of 2.  The real problem
is we don't keep these data on the silo and there is no way of knowing
that the downlink has been see before.  Let me give an example.  On
January 18 we create a tape containing 4 file with ID 20070180052_AGS.
The following day four more files with the same ID show up.  When the time
comes to create a new tape several weeks later we don't know about the
previous 4 and we create a tape containing the new four.  Problem is we
have 8 files on two tapes and *one* storage location in the database.
A similar problem occurs when say an additional two files show up
subsequent to a tape being made.  Either way this is something that can't
be handled here.

What can be handled is: Keep groups together on the current tape.  Once
the limit has been reached it is quite possible that the limit will split
up files.  We will insure that all files are on the tape adjusting the
list, removing rearranging whatever, until we have a good list.
------------------------------------------------------------------------*/
#include <stdio.h>
#include <unistd.h>
#include <string.h>

#define LIMIT (360*1024*1024)

int main( int argc, char *argv[] )
{
    FILE *inf, *ouf, *Mail;
    int wc_infile, wc_IDs;
    int i, id;
    char **input_list, **file_names;
    char **unique_IDs;
    unsigned int *file_sizes;
    int *id_count;
    //unsigned int kb = 0, b = 0;
    //unsigned int sum = 0;
    char buf[513];
    //int over_lim_idx;
    //char limit_ID[100];
    //int  limit_ID_idx;
    int opt;
    char input_file_name[100];
    char uID_file_name[100];
    char output_file_name[100];
    int arguments = 0;


    /* Process command line arguments */
    while (( opt = getopt( argc, argv, "i:s:d:n:o:" )) != -1 )
    {
        switch ( opt )
	{
	    case 'i' :
	        strcpy( input_file_name, optarg );
		arguments++;
		break;
	    
	    case 's' :
	        wc_infile = atoi( optarg );
		arguments++;
		break;

	    case 'd' :
	        strcpy( uID_file_name, optarg );
		arguments++;
		break;
	    
	    case 'n' :
	        wc_IDs = atoi( optarg );
		arguments++;
		break;
	    
	    case 'o' :
	        strcpy( output_file_name, optarg );
		arguments++;
		break;
	    
	    default  :
		arguments = 0;
		break;
	}  /* END switch opt */
    }  /* END while getopt */

    /* All five arguments must be specified */
    if ( arguments < 5 )
    {
       fprintf( stderr,
           "\nUsage: %s -i <input file> -s <wc input file>\n", argv[0] );
       fprintf( stderr,
           "       -d <unique IDs file> -n <wc unique IDs file>\n" );
       fprintf( stderr,
           "       -o <output file>\n\n" );
       return 1;
    }

    /* Open the input and output files */
    inf = fopen( input_file_name, "r" );
    if ( !inf )
    {
        fprintf( stderr, "Uable to open input file: %s\n", input_file_name );
	return 1;
    }

    ouf = fopen( output_file_name, "w" );
    if ( !ouf )
    {
        fprintf( stderr, "Uable to open output file: %s\n", output_file_name );
	return 1;
    }

    Mail = fopen( "MailFile", "a" );
    if ( !Mail )
    {
        fprintf( stderr, "Uable to open mail file: MailFile\n" );
	return 1;
    }

    /* Allocate arrays */
    input_list = (char **)malloc( wc_infile * sizeof(char *));
    file_names = (char **)malloc( wc_infile * sizeof(char *));
    unique_IDs = (char **)malloc( wc_IDs * sizeof(char *));
    file_sizes = (unsigned int *)malloc( wc_infile * sizeof(unsigned int));
    id_count   = (int *)malloc( wc_IDs * sizeof(int));
    if ( !input_list || !file_names || !file_sizes || !unique_IDs ||
         !id_count )
    {
        fprintf( stderr, "Error allocating memory\n" );
	return 1;
    }
    for ( i = 0; i < wc_infile; i++ )
    {
        input_list[i] = (char *)malloc( 512 );
        file_names[i] = (char *)malloc( 512 );
	if ( !input_list[i] || !file_names[i] )
	{
	    fprintf( stderr, "Error allocating memory\n" );
	    return 1;
	}
    }
    for ( i = 0; i < wc_infile; i++ )
    {
        unique_IDs[i] = (char *)malloc( 100 );
	if ( !unique_IDs[i] )
	{
	    fprintf( stderr, "Error allocating memory\n" );
	    return 1;
	}
    }


    /* Read the input file */
    for ( i = 0; i < wc_infile; i++ )
    {
        char *s;
        char *cp;
	unsigned int   n;

	/* Read a line into a temporary buffer */
	s = fgets( buf, 511, inf );
	if ( !s )
	{
	    fprintf( stderr, "Error reading from infile\n" );
	    return 1;
	}
	/* Make a copy of the buffer for later use */
	strcpy( input_list[i], buf );

	/* Get a pointer to one character after the first delimiter */
        cp = strchr( buf, '|' );
	cp++;
	/* Convert the ASCII value to an unsigned integer and save it */
	sscanf( cp, "%u", &n );
	file_sizes[i] = n;

	/* Get a pointer to one character after the next delimiter */
	s = cp;
	cp = strchr( s, '|' );
	cp++;
	/* and one after the last delimiter */
	s = cp;
	cp = strchr( s, '|' );
	cp++;

	/* Find the new line character and replace it with NULL */
	s = strchr( cp, '\n' );
	if ( s ) *s = '\0';

	/* Finally save a copy of the file name */
	strcpy( file_names[i], cp );

    }  /* END for i */

    /* Close the input file and open the file containing the unique ID list */
    fclose ( inf );
    inf = fopen( uID_file_name, "r" );
    if ( !inf )
    {
        fprintf( stderr, "Unable to open file: %s\n", uID_file_name );
	return 1;
    }

    /* Read the IDs */
    for ( id = 0; id < wc_IDs; id++ )
    {
        char *s;
        char *cp;

	cp = fgets( buf, 99, inf );
	if ( !cp )
	{
	    fprintf( stderr, "Error reading from infile\n" );
	    return 1;
	}

	/* Replace the new line character with a null and make a copy */
	s = strchr( cp, '\n' );
	if ( s ) *s = '\0';
	strcpy( unique_IDs[id], cp );
    }  /* END for id */
    fclose( inf );


#if 0
/***** I don't think I need this ************************************/
    /* Find out at what point in the list we have reached the limit */
    for ( i = 0; i < wc_infile; i++ )
    {
        kb = file_sizes[i] / 1024;
	b += file_sizes[i] % 1024;

	sum += kb;
	if ( b >= 1024 )
	{
	    sum++;
	    b -= 1024;
	}

	if ( sum >= LIMIT )
	{
	    over_lim_idx = i;
	    printf( "Limit %u reached at index %d\n", sum, i );
	    break;
	}
    }
    if ( (long)sum - (long)LIMIT < 0 )
        printf( "Under limit by %ld\n", (long)sum - (long)LIMIT );
    else
	printf( "Overlimit by: %u\n", sum - LIMIT );

    /* What is the ID at this location */
    {
    char *s;
    strcpy( buf, file_names[over_lim_idx] );
    s = strrchr( buf, '/' );
    *s = '\0';
    s = strrchr( buf, '/' );
    s++;
    strcpy( limit_ID, s );
    }
printf( "ID: %s, %s\n", limit_ID, file_names[over_lim_idx] );
/***** To here ******************************************************/
#endif
    


    /* Count the number of files belonging to each unique ID.  This is not a
     * very elegant way to do this but the list should be small roughly 100
     * unique IDs and 400 files. */
    //limit_ID_idx = -999;
    for ( id = 0; id < wc_IDs; id++ )
    {
	id_count[id] = 0;
        for ( i = 0; i < wc_infile; i++ )
	{
	    if ( strstr( file_names[i], unique_IDs[id] ) != NULL )
	        id_count[id]++;
	}

    }
#if 0
printf( "%d: %s %d\n", id, unique_IDs[id], id_count[id] );
/**** If I don't calulate the limit above this will have to go as well *****/
	if ( strcmp( limit_ID, unique_IDs[id] ) == 0 )
	    limit_ID_idx = id;
the closing brace above would go here
printf( "ID index is: %d\n", limit_ID_idx );
#endif


    /* Go through the list of unique IDs and build a new listing excluding all
     * IDs that don't have 4, 6 or 8 files. Go through the entire list and
     * only add file that either keep the total Kbytes below the limit, or
     * only push it over by one Gig. */
    {
    unsigned int sum = 0;
    int id, fidx;
    int i;
    unsigned int ksum;

    /* Give the mail file a header block */
    fprintf( Mail, "\nBeginning process %s\n", argv[0] );

    /* Loop through all unique IDs */
    for ( id = 0, fidx = 0; id < wc_IDs; id++ )
    {
        switch ( id_count[id] )
	{
	    /* These are the three cases supported: */
	    case 4 :
	    case 6 :
	    case 8 :
		/* Compute the size of the group */
		for ( i = 0, ksum = 0; i < id_count[id]; i++ )
		    ksum += file_sizes[fidx+i] / 1024;

		/* check that it doesn't put the sum overlimit */
		if ( sum + ksum > LIMIT )
		{
		    /* Compute how much over the limit */
		    int n;
		    n = (int)(((long)sum+(long)ksum - LIMIT)/1024);

		    /* If it's greater than a Gig forget it but report */
		    if ( n > 1000 )
		    {
			fprintf( Mail, "Excluding ID: %s put over by %dMb\n",
			    unique_IDs[id], n );
			break;
		    }
		    /* Otherwise report that it's been kept */
		    else
		        fprintf( Mail, " Allowing ID: %s\n", unique_IDs[id] );
		}
		/* Add the group sum to the total */
		sum += ksum;

		/* And put the files in the new list */
		for ( i = 0; i < id_count[id]; i++ )
		    fprintf( ouf, "%s", input_list[fidx+i] );

	        break;

	    /* All others are ignored but reported */
	    default:
		fprintf( Mail, "*** Error ID: %s only has %d files ***\n",
		    unique_IDs[id],
		    id_count[id] );
	        break;
	}  /* END switch */

	/* Increment the file index by the number of files in the group */
	fidx += id_count[id];

    }  /* END for i */
    }  /* END block */

    /* Give the mail file a closing block */
    fprintf( Mail, "Completed process %s\n\n", argv[0] );

    fclose( ouf );
    fclose( Mail );
    return 0;
}
