#!/bin/csh -f
# clean_silo.csh
#
# To remove files from the /hsm file system.  At present this is for EO-1 raw
# data.
#
# Usage: clean_silo.csh prefix file_list
#
# Where: prefix -- leading part of file path
#        file_list -- list of files to remove
#
# In general the file_list does not contain a full path, so the combination
# of prefix and each line of the file_list must be a full path name.
#-----------------------------------------------------------------------------

# Must have two arguments
#------------------------
if ( ${#argv} != 2 ) then
    echo "Usage: prefix file_list"
    exit 1
endif

# Check that file list exists
#----------------------------
if ( ! -e $2 ) then
    echo "No such file: $2"
    exit 1
endif

# Pull individual entries from the list and remove the file
#----------------------------------------------------------
set n = `wc $2 | awk -F' ' '{print $1}'`
# This is set to one (1) to ignore the first file in the list
set i = 1
while ( "$i" < "$n" )
    @ i++
    set part2 = `tail +$i $2 | head -1`
    set file = "$1/$part2"

    echo "Removing: $file"
    rm -f $file
end

# Now remove empty subdirs.  The last entry in file $2 looks list this:
# $part2 is eo1/raw/20060441823_PF1/e2006044194012.f3
# We want the next directory up the path.in other words:
# eo1/raw
#----------------------------------------------------------------------
set part2 = `echo $part2 | awk -F'/' '{printf("%s/%s",$1,$2)}'`
set dirpath = "$1/$part2"
echo "Removing directories:"
find $dirpath -empty -depth -type d -print -exec rmdir {} \;
