#!/bin/csh -f

# Do any necessary set up and change to the working directory
set procdir   = "/home/ops/EO1_proc"
set errordir  = "/home/ops/EO1_error"
set workdir   = "/home/ops/EO1_work"
set completedir   = "/home/ops/EO1_completed"
set logfile   = "/home/ops/EO1_log/create_meta.log"
set invserv   = "edclxs53"
set recipiants = "goodale@usgs.gov"
cd $workdir

# See if there are any files to be processed
set n = `ls -1 $procdir | grep proc | wc -l`
if ( $n == 0 ) exit 0

# Check for any processed files in $procdir.  If there are add the storage
# location to the end of each line and ship it off to the inventory database.
# There shouldn't be many files here so the wildcard won't be a problem.
foreach file ( $procdir/proc* )
    set tapeID = `head -1 $file`
    # The off-site label is numeric and begins with the dateset_id
    # Get the dateset_id
    set dateset_id = `grep DATESET_ID $file | awk -F'|' '{print $2}'`
    set first3 = `expr $tapeID : '\(...\)'`
    if ( "$first3" != "$dateset_id" ) then
        echo -n `date +'%D'`"   "`date +'%T'` >> $logfile
	echo "  ERROR $file dataset ID mismatch $first3 vs. $dateset_id" \
	    >> $logfile
	mv $file $errordir/.

	# Error notification
        echo -n `date +'%D'`"   "`date +'%T'` > MailFile
	echo "  ERROR $file dataset ID mismatch $first3 vs. $dateset_id" \
	    >> MailFile
	echo "File moved to $errordir" >> MailFile
	Mail -s "Error in EO1 metadata" $recipiants < MailFile
	rm MailFile

	continue
    endif
    # Check that the remainder of the characters are numeric
    expr "$tapeID" : "...[0-9][0-9][0-9][0-9][0-9][0-9]" > /dev/null
    if ( $status ) then
        # It's an error condition
	mv $file $errordir/.
	echo -n `date +'%D'`"   "`date +'%T'` >> $logfile
	echo "  ERROR $file contains incorrect tape ID: $tapeID" >> $logfile
	mv $file $errordir/.

	# Error notification
        echo -n `date +'%D'`"   "`date +'%T'` > MailFile
	echo "  ERROR $file contains incorrect tape ID: $tapeID" >> MailFile
	echo "File moved to $errordir" >> MailFile
	Mail -s "Error in EO1 metadata" $recipiants < MailFile
	rm MailFile

	continue
    endif
    # Now make sure the tape ID is nine, and only nine, characters long
    set tapeidlen = `expr $tapeID : '.*'`
    if ( $tapeidlen != 9 ) then
        # It's an error condition
	mv $file $errordir/.
	echo -n `date +'%D'`"   "`date +'%T'` >> $logfile
	echo \
	    "  ERROR $file contains tape ID of length $tapeidlen: $tapeID" \
	    >> $logfile
	mv $file $errordir/.

	# Error notification
        echo -n `date +'%D'`"   "`date +'%T'` > MailFile
	echo \
	    "  ERROR $file contains tape ID of length $tapeidlen: $tapeID" \
	    >> MailFile
	echo "File moved to $errordir" >> MailFile
	Mail -s "Error in EO1 metadata" $recipiants < MailFile
	rm MailFile

	continue
    endif

    # Find the metadata file that this proc file goes with.  Best way is to
    # grep for one of the entity ID's
    set ent = `grep \/ali\/ $file | head -1 | awk -F'/' '{print $4}'`
    if ( "$ent" == "" ) then
	set ent = `grep \/hyp\/ $file | head -1 | awk -F'/' '{print $4}'`
    endif
    if ( "$ent" == "" ) then
	echo -n `date +'%D'`"   "`date +'%T'` >> $logfile
	echo "  ERROR could not find an entity ID in $file" >> $logfile
	mv $file $errordir/.
	continue
    endif
    set metafile = `grep -l $ent $procdir/metadatalist_*`
    if ( "$metafile" == "" ) then
	echo -n `date +'%D'`"   "`date +'%T'` >> $logfile
	echo "  ERROR could not find metadata file for $ent" >> $logfile
	mv $file $errordir/.
	continue
    endif

    # Sanity check ... Make sure the entities in the metadata file are the
    # same as the entities in the processfile list.  If they are not notify me
    # and skip to the next file
    set metanum = `wc -l $metafile | awk '{print $1}'`
    grep \/ali\/ $file | awk -F'/' '{print $4}' | sort -u >  ent_ids
    grep \/hyp\/ $file | awk -F'/' '{print $4}' | sort -u >> ent_ids
    set procnum = `wc -l ent_ids | awk '{print $1}'`
    if ( $metanum != $procnum ) then
        echo -n `date +'%D'`"   "`date +'%T'` >> $logfile
	echo "  ERROR in entity count $metanum != $procnum" >> $logfile
	mv $file $metafile $errordir/.
	echo -n `date +'%D'`"   "`date +'%T'` >> Mailfile
	echo "  ERROR in entity count $metanum != $procnum" >> Mailfile
	#Mail -s "EO1 process error" @usgs.gov < Mailfile
	rm Mailfile
	continue
    endif
    
    # The metadata file contains only entity ID's we need it to be like
    # ENTITY_ID|MEDIA_ID|MEDIA_TYPE|DATESET_ID|MEDIA_DESCRIPTION|1PROJECT|
    # 2PROJECT_DESC_CODE|3PROJECT_DESCRIPTION
    # Each of these values are in the processed file
    #--------------------------------------------------------------------
    set codes = "MEDIA_TYPE DATESET_ID MEDIA_DESCRIPTION 1PROJECT 2PROJECT_DESC_CODE 3PROJECT_DESCRIPTION"
    set metastring = "$tapeID"
    foreach code ( $codes )
        set item = `grep $code $file | awk -F'|' '{print $2}'`
	set metastring = "${metastring}|${item}"
    end
    awk /EO1/'{\
        printf "%s|%s\n", $1, META \
	}' META="$metastring" $metafile > ${metafile}.tmp
    mv -f ${metafile}.tmp $metafile

    # Inventory folks want the data split between ali and hyperion so...
    set datetime = `echo $metafile | awk -F'_' '{print $3}'`
    grep EO1A $metafile > archloc_${datetime}.ali
    grep EO1H $metafile > archloc_${datetime}.hyp

    # Now copy them to the inventory database server.  On copy errors remove
    # the interim files and continue on to the next file leaving the processed
    # file and metadata files in their current locations.  Next time the
    # process runs it will try them again.  If the first copy is successful
    # and the second is not, remove the first file from the inventory server.
    if ( `wc archloc_${datetime}.ali | awk -F' ' '{print $1}'` > 0 ) then
	scp archloc_${datetime}.ali ops@${invserv}:/inv/inv/eo1/data/.
	if ( $status ) then
	    echo -n `date +'%D'`"   "`date +'%T'` >> $logfile
	    echo "  ERROR copying archloc_${datetime}.ali to $invserv" >> \
		$logfile
	    rm archloc_${datetime}.ali
	    rm archloc_${datetime}.hyp
	    continue
	endif
	ssh ops@$invserv chmod 666 /inv/inv/eo1/data/archloc_${datetime}.ali
	if ( $status ) then
            echo -n `date +'%D'`"   "`date +'%T'` >> $logfile
	    echo "  ERROR could not chmod archloc_${datetime}.ali" >> $logfile
	endif
    endif
    if ( `wc archloc_${datetime}.hyp | awk -F' ' '{print $1}'` > 0 ) then
	scp archloc_${datetime}.hyp ops@${invserv}:/inv/inv/eo1/data/.
	if ( $status ) then
	    echo -n `date +'%D'`"   "`date +'%T'` >> $logfile
	    echo "  ERROR copying archloc_${datetime}.hyp to $invserv" >> \
		$logfile
	    ssh ops@$invserv rm /inv/inv/eo1/data/archloc_${datetime}.ali
	    rm archloc_${datetime}.ali
	    rm archloc_${datetime}.hyp
	    continue
	ssh ops@$invserv chmod 666 /inv/inv/eo1/data/archloc_${datetime}.hyp
	if ( $status ) then
            echo -n `date +'%D'`"   "`date +'%T'` >> $logfile
	    echo "  ERROR could not chmod archloc_${datetime}.hyp" >> $logfile
	endif
    endif

    # Move the processed and metatdata files to Old directories and remove
    # interim files
    mv $file $completedir/.
    mv $metafile $completedir/.
    rm archloc_${datetime}.ali
    rm archloc_${datetime}.hyp

    # Processing message
    echo -n `date +'%D'`"   "`date +'%T'` >> $logfile
    echo "  Processed: $file" >> $logfile
    echo -n `date +'%D'`"   "`date +'%T'` >> $logfile
    echo "        and: $metafile" >> $logfile

end

exit 0
