#!/bin/csh -f

# Change Log:
#   08/19/08  clf  Updated code to show the proper path for cleanup -- cr 17116
#   12/18/08  clf  Updated code for data move from lta1 to lta4 -- cr 17680
#   05/12/09  clf  Updated code for new tape ids -- cr 18305

# Do any necessary set up and change to the working directory
set procdir   = "/home/ops/EO1_raw"
set errordir  = "/home/ops/EO1_error"
set workdir   = "/home/ops/EO1_work"
set completedir   = "/home/ops/EO1_completed"
set logfile   = "/home/ops/EO1_log/create_meta.log"
set invserv   = "edclxs53"
set bandflag = "N"
cd $workdir

if ( ${#argv} == 1 ) then
    if ( "$1" == "sband" ) then
        set procdir   = "/home/ops/EO1_raw/Sband"
        set bandflag = "Y"
    endif
endif


# See if there are any files to be processed
set n = `ls -1 $procdir | grep proc | wc -l`
if ( $n == 0 ) exit 0

# Check for any processed files in $procdir.  If there are add the storage
# location to the end of each line and ship it off to the inventory database.
# There shouldn't be many files here so the wildcard won't be a problem.
foreach file ( $procdir/proc* )
    set datetime = `echo $file | awk -F'_' '{print $3}'`
    set tapeID = `head -1 $file`
    # At the current time tapeids begin with an alpha character and are 
    # followed by 5 alpha numeric values.  Test for this condition.
    expr "$tapeID" : "[a-z][a-z0-9][a-z0-9][a-z0-9][a-z0-9][a-z0-9]" > /dev/null
    if ( $status ) then
        # It's an error condition
	mv $file $errordir/.
	echo -n `date +'%D'`"   "`date +'%T'` >> $logfile
	echo "  ERROR $file contains incorrect tape ID: $tapeID" >> $logfile
	continue
    endif
    # Now make sure the tape ID is six, and only six, characters long
    set tapeidlen = `expr $tapeID : '.*'`
    if ( $tapeidlen != 6 ) then
        # It's an error condition
	mv $file $errordir/.
	echo -n `date +'%D'`"   "`date +'%T'` >> $logfile
	echo \
	    "  ERROR $file contains tape ID of length $tapeidlen: $tapeID" \
	    >> $logfile
	continue
    endif

    # We need a list of the ID's
    grep raw $file | awk -F'/' '{print $3}' | sort -u > metafile
    # The metadata file contains only entity ID's we need it to be like
    # ENTITY_ID|TAPE_ID|FLAG so make it so
    awk '{\
        printf "%s|%s|%s\n", $1, TAPE, FLAG \
	}' TAPE=$tapeID FLAG=$bandflag metafile > archloc_${datetime}.raw
    rm metafile 

    # Now copy it to the inventory database server.  On copy errors remove
    # the interim files and continue on to the next file leaving the processed
    # file in its current locations.  Next time the process runs it will try 
    # again.
	scp archloc_${datetime}.raw ops@${invserv}:/inv/inv/eo1/data/.
	if ( $status ) then
	    echo -n `date +'%D'`"   "`date +'%T'` >> $logfile
	    echo "  ERROR copying archloc_${datetime}.raw to $invserv" >> \
		$logfile
	    rm archloc_${datetime}.raw
	    continue
	endif
	ssh ops@$invserv chmod 666 /inv/inv/eo1/data/archloc_${datetime}.raw
	if ( $status ) then
            echo -n `date +'%D'`"   "`date +'%T'` >> $logfile
	    echo "  ERROR could not chmod archloc_${datetime}.raw" >> $logfile
	endif

    # Move the processed and metatdata files to Old directories and remove
    # interim files
    mv $file $completedir/.
    rm archloc_${datetime}.raw

    # Processing message
    echo -n `date +'%D'`"   "`date +'%T'` >> $logfile
    echo "  Processed: $file" >> $logfile

    # Make a notification
    set tfile = $file:t
    echo -n `date +'%D'`"   "`date +'%T'` >> MailFile
    echo "  Processed: $file" >> MailFile
    echo "" >> MailFile
    echo "Check that the tape is good before running cleanup" >> MailFile
    echo "On edclxs37 run:" >> Mailfile
    echo "/home/ops/bin/clean_silo.csh /hsm/lta4 $completedir/$tfile" >> MailFile
    echo "" >> MailFile
    echo "After cleanup remove lockfile:" >> MailFile
    if ( "$bandflag" == "Y" ) then
        echo "rm /home/ops/EO1_work/sband_work/NoRun" >> MailFile
    else
        echo "rm /home/ops/EO1_work/raw_work/NoRun" >> MailFile
    endif
    #Mail -s "EO-1 raw" @usgs.gov < MailFile
    Mail -s "EO-1 raw" ogren@usgs.gov burkman@usgs.gov < MailFile
    rm MailFile


end

exit 0
