#!/bin/csh -f

# Change Log:
#  12/18/08  clf  Updated to move data from lta1 to lta4 -- cr 17680
#
#
# input files are of the type
# 
# /hsm/lta4/eo1/YEAR/DAY/SENSOR/ENTITY_ID file_name
# 
# where SENSOR is either ali or hyp or could be meta or blank
# YEAR is 4-digit year
# DAY is 3-digit julian day of year
# ENTITY_ID looks like EO1A1030632005090110PF_WPS_01 for ali
# or                   EO1H1030632005090110PF_WPS_01 for hyperion
# 
# 
# to get the metadata and .DAT file:
# for .meta strip off the _WPS_01
# for .DAT remove the A or H in 4th character

# I'm thinking for those instances where some of the metadata, houskeeping,
# or hdf files are missing and when we process a tarlisting, a command line
# argument will be passed in indicating the the process should complete for an
# entity regardless of whether all associated files are present.

# Do any necessary set up and change to the working directory
set opsbin    = "/home/ops/bin"
set ingestdir = "/home/ops/EO1_ingest"
set procdir   = "/hove/ops/EO1_proc"
set workdir   = "/home/ops/EO1_work"
set logfile   = "/home/ops/EO1_log/archive.log"
set arc_dir   = "/home/ops/EO1_archive"
set current_tar = "current_tar"
set exitstatus = 0

# Check for command line arguments and set flags appropriately
set skip_flag = "0"
set tar_flag = "0"
while ( ${#argv} )
    switch ( $argv[1] )
        case "--skip-assoc-checks" :
	    set skip_flag = "1"
	    breaksw
        case "--tarfile" :
	    shift
	    set infile = "$argv[1]"
	    if ( ! -e $infile ) then
	        echo -n `date +'%D'`"   "`date +'%T'` >> $logfile
		echo "  $infile does not exist" >> $logfile
		exit 1
	    endif
	    set tar_flag = "1"
	    set current_tar = "CURRENT_tar"
            set workdir = `pwd`
	    if ( -e $current_tar ) rm -f $current_tar
	    breaksw
	default :
	    echo "$argv[1] " >> $logfile
	    set text = "$argv[1]"
	    goto USAGE
	    breaksw
    endsw
    shift
end
cd $workdir

# If the input is from a pre-existing tar list use that list as input
if ( $tar_flag ) then
    grep \/ali\/ $infile >  temp1
    grep \/hyp\/ $infile >> temp1
    awk -F'/' '{printf("|%s/%s/%s/%s %s\n",$1,$2,$3,$4,$5)}' temp1 > temp2
    sed 's/|/\/hsm\/lta4\/eo1\//' temp2 > working_file
    rm temp1 temp2

# Otherwise use the  ingest directory
else
    # See if there are any files to be processed.  If not exit.
    set n = `ls -1 $ingestdir | grep EO1 | wc -l`
    if ( $n == 0 ) exit 0

    # Get any new files from ingest directory and concatenate them into one
    # working file.  Move the input files to an old ingest files directory
    if ( -e working_file ) rm -f working_file
    foreach file ( $ingestdir/EO1_archive_* )
	cat $file >> working_file
	echo "" >> working_file
	mv $file Old_ingest_files/.

	echo -n `date +'%D'`"   "`date +'%T'` >> $logfile
	echo "  Processing $file" >> $logfile
    end
endif

# Pull the ENTITY_IDs from the working file.  the entity ID's are in the ali
# and hyp directories so first grep out the entries from there.  Next grab the
# entity ID's which are subdirectories after ali and hyp.  There happens to be
# a file after the ID separated by a space.  Finally replace the working file
# with the temporary file -- the working file now contains only the scene data
cat working_file | grep \/ali\/  > tempor
cat working_file | grep \/hyp\/ >> tempor
# Add the in files that are waiting for other data files if the future file
# exists
if ( -e future_file ) then
    cat future_file >> tempor
    rm future_file
endif
cat tempor | awk -F'/' '{print $8}' | awk -F' ' '{print $1}' | sort -u > entlist
# So now entlist contains entnum entity ID's.  These entity ID's are of the
# form: EO1A1030632005090110PF_WPS_01 for ali or with an H in character 3,
# after the EO1 part, for hyperion
mv -f tempor working_file
set entnum = `wc -l entlist | awk -F' ' '{print $1}'`

# Process the list.  Note that cshell has problems with how many characters
# can fit in a variable so process the list in a while loop using the counter
# to get at each entry in turn.  Also set up two flags.  The tflag is passed
# to tarlist.csh and as long as it's not null a new tarlisting will not be
# created.  It will be held in this state until the last file for the current
# entity is processed.  At that time it is set to null to allow a new
# tarlisting if needed.  The newarchive flag will be set to the tarlisting
# when tarlist.csh returns with status of "I created a new tarlist" meaning
# it's time to create an archive tape.  So:
# for each ENTITY_ID 
set i = 1
set newarchive = ""
set newtarlist = ""
while ( "$i" <= "$entnum" )
    # Get the next entity ID from the entlist
    set ent = `tail +$i entlist | head -1`

    # Get its data path.  The data path that is returned starts at /hsm and
    # continues until the julian day.  eg /hsm/lta4/eo1/2005/090
    set data_path = `grep -m 1 $ent working_file | awk -F' ' '{print $1}'`
    set data_path = \
        `echo $data_path | awk -F'/' '{printf("/hsm/lta4/eo1/%s/%s",$5,$6)}'`

    # Create a list of the ents files.  Grab the entries that contain the
    # entity ID and replace the space with a / so that the output file contains
    # a complete path to each data file.  Entries look like:
    # /hsm/lta4/eo1/2005/092/ali/EO1A2290822005092110PF_SGS_01/
    # EO12005092135908_SGS_01.M1Z
    grep $ent working_file | sed 's/ /\//' > ent_files

    # Check for the .meta file.  If the file does not exist don't process the
    # entity at this time.  Put the entity ID into the future file and
    # continue with the next entity.  If the file exists put the full path in
    # the metadata file list.  Entries look like:
    # /hsm/lta4/eo1/2005/092/meta/EO1A2290822005092110PF.meta
    # If the skip flag is set don't send the entity to the future file
    set meta_file = `echo $ent | awk -F'_' '{print $1}'`
    set meta_file = "$data_path/meta/$meta_file.meta"
    if ( ! -e $meta_file && ! $skip_flag ) then
	echo -n `date +'%D'`"   "`date +'%T'` >> $logfile
	echo -n "   Informational   " >> $logfile
	echo "  $ent sent to future for missing .meta file" >> $logfile
        # Put this entity in a future file
	grep $ent working_file >> future_file
	# and continue to next entity
	@ i++
	continue
    else
	echo $meta_file > metafiles
    endif

    # Now check for the .DAT file.  The same comment about its presense or
    # absense applies.  Entries look like:
    # /hsm/lta4/eo1/2005/092/meta/00meta_EO12290822005092110PF_SGS_01.DAT
    # If the skip flag is set don't send the entity to the future file
    set dat_file = `echo $ent | sed 's/^EO1[AH]/EO1/'`
    set dat_file = `ls -1 $data_path/meta | grep $dat_file`
    if ( "$dat_file" != "" ) then
        set dat_file = "$data_path/meta/$dat_file"
        echo $dat_file >> metafiles
    endif
    if ( "$dat_file" == "" && ! $skip_flag ) then
	echo -n `date +'%D'`"   "`date +'%T'` >> $logfile
	echo -n "   Informational   " >> $logfile
	echo "  $ent sent to future for missing .DAT file" >> $logfile
        # Put this entity in a future file
	grep $ent working_file >> future_file
	# and continue to next entity
	@ i++
	continue
    endif

    # get a list of the names of the YYYY_DDD*.hdf files.  From what I've seen
    # of the data there should be files with an acs, ali, eff, gps, hyp, lac,
    # mis, wrp and xbd in the file name.  There may be also be a number of
    # nxt_ files.  For now if there isn't the at least nine files store the
    # entity ID and continue.  Entries look like:
    # /hsm/lta4/eo1/2005/092/meta/2005_092_00_12_acs.hdf
    # If the skip flag is set don't send the entity to the future file
    ls -1 $data_path/meta/*.hdf > hdf_files
    set hdf_num = `wc -l hdf_files | awk -F' ' '{print $1}'`
    # I guess just make sure there are nine
    if ( "$hdf_num" < "9" && ! $skip_flag ) then
	echo -n `date +'%D'`"   "`date +'%T'` >> $logfile
	echo -n "   Informational   " >> $logfile
	echo "  $ent sent to future because of $hdf_num hdf files " >> $logfile
        # Put this entity in a future file
	grep $ent working_file >> future_file
	# and continue to next entity
	@ i++
	continue
    endif

    # There are more files of interest.  A report file in the
    # /hsm/lta4/eo1/YYYY/DDD directory and any txt files in the meta directory.
    # Examples include:
    # /hsm/lta4/eo1/2005/097/0event_2005097.rpt
    # /hsm/lta4/eo1/2005/097/meta/daily_05097.txt
    ls -1 $data_path/*.rpt > misc_files
    ls -1 $data_path/meta/*.txt >> misc_files

    # There are now four files that contain the paths to the image data,
    # metadata and houskeeping files.  Concatenate the files and count the
    # number of entries.  We put the entity files at the end of the list to
    # gurantee the last file to be processed will be added to the tar list,
    # which is the only reason for keeping the files separate in the first
    # place.
    cat metafiles hdf_files misc_files ent_files > all_files
    set all_num = `wc -l all_files | awk -F' ' '{print $1}'`

    # A little set up before processing the files.  Get the tarlist from the
    # current_tar file, if present, so that entries can be checked.  Note that
    # if there is no current_tar file, as is the case when this is run the
    # first time, there is nothing to check against.  So the flag is set
    # indicating whether there is a current_tar file or not.
    if ( -e $current_tar ) then
	set tarlist = `cat $current_tar | awk -F'-' '{print $1}'`
        set have_tarlis = 1
    else
        set have_tarlis = 0
    endif
    set arch_root = "/hsm/lta4/eo1"

    # Now it's time to get to work.  For each of the entries in the all_files
    # file get the size of the file and call tarlist.csh to add it to the list
    # and update the total Kbyte count for the next archive tape.
    set n = 1
    set newtarlist = ""
    while ( "$n" <= "$all_num" )
	# Get the next file name: this is a complete path.
	set file = `tail +$n all_files | head -1`
	# Remove the /hsm/lta4/eo1/ part of the file name.  This is so the tarlist
	# can be checked and it is what will be passed to tarlist.csh.
	set tarlistname = `echo $file | sed 's/\/hsm\/lta4\/eo1\///'`

	# Check the tarlist to see if the file is already there.  If it is
	# there is no need to add it again -- in fact it is undesireable to
	# have files more than once.
        # may have to put in this additional check
        # if ( $have_tarlis && "$n" < "$all_num" && "$newtarlist" == "" ) then
	if ( $have_tarlis ) then
	    set entry = `grep $tarlistname $tarlist`
	    if ( "$entry" != "" ) then
		@ n++
		continue
	    endif
	endif

	# If this is the last time through the loop null the tflag to allow
	# this file to trigger a new tarlist.  If it does trigger a new
	# tarlist this file will still remain in the original list.  This bit
	# of magic keeps all of the entities files on one tape.
	if ( "$n" == "$all_num" && "$newtarlist" == "tartime" ) then
	    set tflag = ""
	else
	    set tflag = "-nonew"
	endif
	# However if operating on a tar listing don't ever allow
	if ( $tar_flag ) then
	    set tflag = "-nonew"
	endif
	    
	# Get the file size and add the file to the tarlist
	set file_size = `ls -l $file | awk -F' ' '{print $5}'`
	$opsbin/tarlist.csh $tarlistname $file_size $current_tar $logfile $tflag
	# There now has to be some check of the return status in case the
	# trigger point has been reached and an LTO needs making.  If one
	# needs creating the newarchive flag will be set with the tarlist
	switch ( $status )
	    case 2  : # usage error
		echo -n `date +'%D'`"   "`date +'%T'` >> $logfile
		echo -n "   ERROR           " >> $logfile
		echo "  $file $file_size tarlist.csh usage error" >> $logfile
		set exitstatus = 1
		breaksw
	    # When this value is returned the current tar listing has sufficient
	    # size to create an LTO.  For EO1 this is ignored since the tflag
	    # will be set on the last file allowing one to be created a bit
	    # later.
	    case 20 : # says it's time to create new archive
		echo -n `date +'%D'`"   "`date +'%T'` >> $logfile
		echo -n "   Informational   " >> $logfile
		echo "  tarlist.csh says it's time to create a new archive" \
		    >> $logfile
                set newtarlist = "tartime"
		breaksw
	    # This value indicates that a new tarlisting has been created and
	    # a tape must be made.  Don't ignore this.  Set the flag but also
	    # get the name of the new tarlist in case there are other entities
	    # to process.
	    case 30 : # a new tarlisting has been created
		echo -n `date +'%D'`"   "`date +'%T'` >> $logfile
		echo -n "   Informational   " >> $logfile
		echo "  tarlist.csh has created a new archive" >> $logfile
		set newarchive = "$tarlist"
		set tarlist = `cat $current_tar | awk -F'-' '{print $1}'`
		set have_tarlis = 1
		breaksw
	    case 0  : # success
		breaksw
	    default : # unknown error
		echo -n `date +'%D'`"   "`date +'%T'` >> $logfile
		echo -n "   ERROR           " >> $logfile
		echo "  $file $file_size tarlist.csh error" >> $logfile
		set exitstatus = 1
		breaksw
	endsw

	# goto the next file
        @ n++
    end

    # Add ENTITY_ID to metadatalist and move the metadatalist away if a new
    # archive tape is to be made.
    echo $ent >> metadatalist

    # A new archive is to be made so move the tar list and metadata to the
    # archive directory.
    if ( "$newarchive" != "" ) then
	set trailer = `echo $newarchive | awk -F'_' '{print $2}'`
	mv metadatalist $arc_dir/metadatalist_$trailer
	mv $newarchive ${newarchive}.cur $arc_dir/.

	# Once done reset newarchive
	set newarchive = ""
    endif

    # goto the next entity ID
    @ i++
end
# Clean up all the temporary files
rm -f all_files ent_files entlist hdf_files metafiles misc_files working_file

# have to move the new tarfile to the original
if ( $tar_flag ) then
    mv $infile ${infile}.old
    mv ${infile}.cur ${infile}.cur.old
    mv $tarlist $infile
    set tarsize = `cat $current_tar | awk -F'-' '{printf("-%s-%s",$2,$3)}'`
    echo "$infile$tarsize" > ${infile}.cur
    rm $current_tar
endif

exit $exitstatus

USAGE:
echo " Usage: $argv[0] [--tarfile file] [--skip-assoc-checks]" >> $logfile
echo -n `date +'%D'`"   "`date +'%T'` >> $logfile
echo -n "   Usage Error     " >> $logfile
echo "  $text" >> $logfile
exit 1

